@@ -30,7 +30,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *	$Id$
+ * $FreeBSD: stable/2.1/bin/csh/dir.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #ifndef lint
@@ -84,7 +84,7 @@ dinit(hp)
     /* Don't believe the login shell home, because it may be a symlink */
     tcp = getwd(path);		/* see ngetwd.c for System V version */
     if (tcp == NULL || *tcp == '\0') {
-	(void) fprintf(csherr, "csh: %s\n", path);
+	(void) fprintf(csherr, "csh: %s: %s\n", path, strerror(errno));
 	if (hp && *hp) {
 	    tcp = short2str(hp);
 	    if (chdir(tcp) == -1)
@@ -221,7 +221,7 @@ printdirs()
 	}
 	if (!(dirflag & DIR_LONG) && hp != NULL && !eq(hp, STRslash) &&
 	    (len = Strlen(hp), Strncmp(hp, dp->di_name, len) == 0) &&
-	    (dp->di_name[len] == '\0' || dp->di_name[len] == '/')) 
+	    (dp->di_name[len] == '\0' || dp->di_name[len] == '/'))
 	    len = Strlen(s = (dp->di_name + len)) + 2;
 	else
 	    len = Strlen(s = dp->di_name) + 1;
