@@ -30,7 +30,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *	$Id$
+ * $FreeBSD: stable/2.1/bin/csh/exec.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #ifndef lint
@@ -50,7 +50,7 @@ static char sccsid[] = "@(#)exec.c	8.1 (Berkeley) 5/31/93";
 # include <stdarg.h>
 #else
 # include <varargs.h>
-#endif 
+#endif
 
 #include "csh.h"
 #include "extern.h"
@@ -656,7 +656,7 @@ tellmewhat(lex)
     register struct biltins *bptr;
     register struct wordent *sp = lex->next;
     bool    aliased = 0;
-    Char   *s0, *s1, *s2;
+    Char   *s0, *s1, *s2, *cmd;
     Char    qc;
 
     if (adrof1(sp->word, &aliases)) {
@@ -696,13 +696,15 @@ tellmewhat(lex)
 	if (eq(sp->word, str2short(bptr->bname))) {
 	    if (aliased)
 		prlex(cshout, lex);
-	    (void) fprintf(cshout, "%s: shell built-in command.\n", 
+	    (void) fprintf(cshout, "%s: shell built-in command.\n",
 			   vis_str(sp->word));
 	    sp->word = s0;	/* we save and then restore this */
 	    return;
 	}
     }
 
+    sp->word = cmd = globone(sp->word, G_IGNORE);
+
     if ((i = iscommand(strip(sp->word))) != 0) {
 	register Char **pv;
 	register struct varent *v;
@@ -717,10 +719,14 @@ tellmewhat(lex)
 	while (--i)
 	    pv++;
 	if (pv[0][0] == 0 || eq(pv[0], STRdot)) {
-	    sp->word = Strspl(STRdotsl, sp->word);
-	    prlex(cshout, lex);
-	    xfree((ptr_t) sp->word);
+	    if (!slash) {
+		sp->word = Strspl(STRdotsl, sp->word);
+		prlex(cshout, lex);
+		xfree((ptr_t) sp->word);
+	    } else
+		prlex(cshout, lex);
 	    sp->word = s0;	/* we save and then restore this */
+	    xfree((ptr_t) cmd);
 	    return;
 	}
 	s1 = Strspl(*pv, STRslash);
@@ -734,5 +740,6 @@ tellmewhat(lex)
 	    prlex(cshout, lex);
 	(void) fprintf(csherr, "%s: Command not found.\n", vis_str(sp->word));
     }
+    xfree((ptr_t) cmd);
     sp->word = s0;		/* we save and then restore this */
 }
