@@ -34,7 +34,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *	$Id: pat_rep.c,v 1.2 1994/09/24 02:56:24 davidg Exp $
+ * $FreeBSD: stable/2.1/bin/pax/pat_rep.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #ifndef lint
@@ -88,7 +88,7 @@ static int resub __P((regex_t *, regmatch_t *, char *, char *, char *));
  *	parses the -s replacement string; compiles the regular expression
  *	and stores the compiled value and it's replacement string together in
  *	replacement string list. Input to this function is of the form:
- *		/old/new/pg 
+ *		/old/new/pg
  *	The first char in the string specifies the delimiter used by this
  *	replacement string. "Old" is a regular expression in "ed" format which
  *	is compiled by regcomp() and is applied to filenames. "new" is the
@@ -348,7 +348,7 @@ pat_sel(arcn)
 
 	/*
 	 * we reach this point only when we allow a single selected match per
-	 * pattern, if the pattern matches a directory and we do not have -d 
+	 * pattern, if the pattern matches a directory and we do not have -d
 	 * (dflag) we are done with this pattern. We may also be handed a file
 	 * in the subtree of a directory. in that case when we are operating
 	 * with -d, this pattern was already selected and we are done
@@ -372,7 +372,7 @@ pat_sel(arcn)
 		 */
 		if (pt->pend != NULL)
 			*pt->pend = '\0';
-			
+
 		if ((pt->pstr = strdup(arcn->name)) == NULL) {
 			warn(1, "Pattern select out of memory");
 			if (pt->pend != NULL)
@@ -397,7 +397,7 @@ pat_sel(arcn)
 		if (*(pt->pstr + len) == '/') {
 			*(pt->pstr + len) = '\0';
 			pt->plen = len;
-		} 
+		}
 		pt->flgs = DIR_MTCH | MTCH;
 		arcn->pat = pt;
 		return(0);
@@ -438,7 +438,7 @@ pat_sel(arcn)
  *	this archive member is "selected" we process and mark the pattern as
  *	one which matched a selected archive member (see pat_sel())
  * Return:
- *	0 if this archive member should be processed, 1 if it should be 
+ *	0 if this archive member should be processed, 1 if it should be
  *	skipped and -1 if we are done with all patterns (and pax should quit
  *	looking for more members)
  */
@@ -515,7 +515,7 @@ pat_match(arcn)
 /*
  * fn_match()
  * Return:
- *	0 if this archive member should be processed, 1 if it should be 
+ *	0 if this archive member should be processed, 1 if it should be
  *	skipped and -1 if we are done with all patterns (and pax should quit
  *	looking for more members)
  *	Note: *pend may be changed to show where the prefix ends.
@@ -564,7 +564,7 @@ fn_match(pattern, string, pend)
 		case '*':
 			c = *pattern;
 			/*
-			 * Collapse multiple *'s. 
+			 * Collapse multiple *'s.
 			 */
 			while (c == '*')
 				c = *++pattern;
@@ -649,7 +649,7 @@ range_match(pattern, test)
  *	if we spot any file links to the old file name in the future, we will
  *	know exactly how to fix the file link.
  * Return:
- *	0 continue to  process file, 1 skip this file, -1 pax is finished 
+ *	0 continue to  process file, 1 skip this file, -1 pax is finished
  */
 
 #if __STDC__
@@ -862,7 +862,7 @@ fix_path(or_name, or_len, dir_name, dir_len)
 	*or_len = len;
 
 	/*
-	 * enough space, shift 
+	 * enough space, shift
 	 */
 	while (src >= start)
 		*dest-- = *src--;
@@ -1014,7 +1014,7 @@ rep_name(name, nlen, prnt)
 			 */
 		} while (pt->flgs & GLOB);
 
-		if (found) 
+		if (found)
 			break;
 
 		/*
@@ -1037,7 +1037,7 @@ rep_name(name, nlen, prnt)
 				warn(1,"Replacement name too long %s >> %s",
 				    name, nname);
 			return(1);
-		} 
+		}
 
 		/*
 		 * inform the user of the result if wanted
@@ -1046,7 +1046,7 @@ rep_name(name, nlen, prnt)
 			if (*nname == '\0')
 				(void)fprintf(stderr,"%s >> <empty string>\n",
 				    name);
-			else 
+			else
 				(void)fprintf(stderr,"%s >> %s\n", name, nname);
 		}
 
@@ -1054,7 +1054,7 @@ rep_name(name, nlen, prnt)
 		 * if empty inform the caller this file is to be skipped
 		 * otherwise copy the new name over the orig name and return
 		 */
-		if (*nname == '\0') 
+		if (*nname == '\0')
 			return(1);
 		*nlen = l_strncpy(name, nname, PAXPATHLEN + 1);
 	}
