@@ -33,7 +33,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *	$Id: parser.c,v 1.4 1995/01/09 20:00:54 paul Exp $
+ * $FreeBSD: stable/2.1/bin/sh/parser.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #ifndef lint
@@ -150,29 +150,12 @@ list(nlflag) {
 	for (;;) {
 		switch (readtoken()) {
 		case TBACKGND:
-			if (n1->type == NCMD || n1->type == NPIPE) {
-				n1->ncmd.backgnd = 1;
-			} else if (n1->type == NREDIR) {
-				n1->type = NBACKGND;
-			} else {
-				n3 = (union node *)stalloc(sizeof (struct nredir));
-				n3->type = NBACKGND;
-				n3->nredir.n = n1;
-				n3->nredir.redirect = NULL;
-				n1 = n3;
-			}
-			goto tsemi;
 		case TNL:
-			tokpushback++;
+			parseheredoc();
+			if (nlflag)
+				return n1;
 			/* fall through */
-tsemi:	    case TSEMI:
-			if (readtoken() == TNL) {
-				parseheredoc();
-				if (nlflag)
-					return n1;
-			} else {
-				tokpushback++;
-			}
+		case TSEMI:
 			checkkwd = 2;
 			if (tokendlist[peektoken()])
 				return n1;
@@ -212,6 +195,19 @@ andor() {
 		} else if (t == TOR) {
 			t = NOR;
 		} else {
+			if (t == TBACKGND) {
+				if (n1->type == NCMD || n1->type == NPIPE) {
+					n1->ncmd.backgnd = 1;
+				} else if (n1->type == NREDIR) {
+					n1->type = NBACKGND;
+				} else {
+					n3 = (union node *)stalloc(sizeof (struct nredir));
+					n3->type = NBACKGND;
+					n3->nredir.n = n1;
+					n3->nredir.redirect = NULL;
+					n1 = n3;
+				}
+			}
 			tokpushback++;
 			return n1;
 		}
@@ -442,7 +438,12 @@ TRACE(("expecting DO got %s %s\n", tokname[got], got == TWORD ? wordtext : ""));
 		checkkwd = 1;
 		break;
 	/* Handle an empty command like other simple commands.  */
+	case TAND:
+	case TOR:
 	case TNL:
+	case TSEMI:
+	/* Handle EOF as an empty command, too */
+	case TEOF:
 	case TWORD:
 		tokpushback++;
 		return simplecmd(rpp, redir);
@@ -472,7 +473,7 @@ TRACE(("expecting DO got %s %s\n", tokname[got], got == TWORD ? wordtext : ""));
 
 
 STATIC union node *
-simplecmd(rpp, redir) 
+simplecmd(rpp, redir)
 	union node **rpp, *redir;
 	{
 	union node *args, **app;
@@ -486,10 +487,10 @@ simplecmd(rpp, redir)
 
 	args = NULL;
 	app = &args;
-	/* 
+	/*
 	 * We save the incoming value, because we need this for shell
 	 * functions.  There can not be a redirect or an argument between
-	 * the function name and the open parenthesis.  
+	 * the function name and the open parenthesis.
 	 */
 	orig_rpp = rpp;
 
@@ -630,7 +631,7 @@ readtoken() {
 #ifdef DEBUG
 	int alreadyseen = tokpushback;
 #endif
-	
+
 	top:
 	t = xxreadtoken();
 
@@ -926,7 +927,7 @@ readtoken1(firstc, syntax, eofmark, striptabs)
 						} else
 							USTPUTC(')', out);
 					} else {
-						/* 
+						/*
 						 * unbalanced parens
 						 *  (don't 2nd guess - no error)
 						 */
@@ -1176,9 +1177,13 @@ parsebackq: {
                 register c;
                 int savelen;
                 char *str;
- 
+
                 STARTSTACKSTR(out);
                 while ((c = pgetc ()) != '`') {
+                       if (c == PEOF) {
+                                startlinno = plinno;
+                                synerror("EOF in backquote substitution");
+                       }
                        if (c == '\\') {
                                 c = pgetc ();
                                 if ( c != '\\' && c != '`' && c != '$'
