@@ -3,7 +3,7 @@
  * Author:	Marc van Kempen
  * Desc:	Implementation of UI-objects:
  *		- String input fields
- *		- List selection 
+ *		- List selection
  *		- Buttons
  *
  * Copyright (c) 1995, Marc van Kempen
@@ -17,7 +17,7 @@
  * responsible for the proper functioning of this software, nor does
  * the author assume any responsibility for damages incurred with
  * its use.
- * 
+ *
  */
 
 #include <stdlib.h>
@@ -29,7 +29,6 @@
 
 #define ESC 27
 
-
 /***********************************************************************
  *
  * Obj routines
@@ -79,14 +78,14 @@ FreeObj(ComposeObj *Obj)
  */
 {
     ComposeObj	*o = Obj;
-    
+
     o = Obj;
     while (o) {
 	o = Obj->next;
 	free(Obj);
 	Obj = o;
     }
-	
+
     return;
 } /* FreeObj() */
 
@@ -94,7 +93,7 @@ FreeObj(ComposeObj *Obj)
 int
 ReadObj(ComposeObj *Obj)
 /*
- * Desc: navigate through the different objects calling their 
+ * Desc: navigate through the different objects calling their
  *	 respective navigation routines as necessary
  * Pre:  Obj != NULL
  */
@@ -122,6 +121,7 @@ ReadObj(ComposeObj *Obj)
 	    break;
 	}
 	switch(ret) {
+	case KEY_DOWN:
 	case SEL_CR:
 	case SEL_TAB:	/* move to the next object in the list */
 	    if (o->next != NULL) {
@@ -130,6 +130,8 @@ ReadObj(ComposeObj *Obj)
 		o = Obj;	/* beginning of the list */
 	    }
 	    break;
+
+	case KEY_UP:
 	case SEL_BACKTAB: /* move to the previous object in the list */
 	    if (o->prev != NULL) {
 		o = o->prev;	/* previous object */
@@ -137,6 +139,7 @@ ReadObj(ComposeObj *Obj)
 		o = last;	/* end of the list */
 	    }
 	    break;
+
 	case KEY_F(1): /* display help_file */
 	case '?':
 	    display_helpfile();
@@ -145,7 +148,7 @@ ReadObj(ComposeObj *Obj)
     }
 
     return(ret);
-    
+
 } /* ReadObj() */
 
 
@@ -177,6 +180,7 @@ PollObj(ComposeObj **Obj)
 	break;
     }
     switch(ret) {
+    case KEY_DOWN:
     case SEL_CR:
     case SEL_TAB:		     /* move to the next object in the list */
 	if ((*Obj)->next != NULL) {
@@ -185,6 +189,8 @@ PollObj(ComposeObj **Obj)
 	    *Obj = first;	     /* beginning of the list */
 	}
 	break;
+
+    case KEY_UP:
     case SEL_BACKTAB: 		     /* move to the previous object in the list */
 	if ((*Obj)->prev != NULL) {
 	    *Obj = (*Obj)->prev;     /* previous object */
@@ -225,13 +231,29 @@ DelObj(ComposeObj *Obj)
 
     FreeObj(o);
 } /* DelObj() */
-    
+
 /***********************************************************************
  *
  * StringObj routines
  *
  ***********************************************************************/
 
+static void
+outstr(WINDOW *win, char *str, int attrs)
+{
+    if (attrs & DITEM_NO_ECHO) {
+	char *cpy;
+	int n = strlen(str);
+
+	cpy = alloca(n + 1);
+	memset(cpy, '*', n);
+	cpy[n] = '\0';
+	waddstr(win, cpy);
+    }
+    else
+	waddstr(win, str);
+}
+
 void
 RefreshStringObj(StringObj *so)
 /*
@@ -249,9 +271,9 @@ RefreshStringObj(StringObj *so)
     wmove(so->win, so->y+2, so->x+1);
     if (strlen(so->s) > so->w-2) {
 	strncpy(tmp, (char *) so->s + strlen(so->s) - so->w + 2, so->w - 1);
-	waddstr(so->win, tmp);
+	outstr(so->win, tmp, so->attr_mask);
     } else {
-	waddstr(so->win, so->s);
+	outstr(so->win, so->s, so->attr_mask);
     }
 
     return;
@@ -265,7 +287,7 @@ NewStringObj(WINDOW *win, char *title, char *s, int y, int x, int w, int len)
  */
 {
     StringObj	*so;
-    
+
     /* Initialize a new object */
     so = (StringObj *) malloc( sizeof(StringObj) );
     if (!so) {
@@ -285,6 +307,7 @@ NewStringObj(WINDOW *win, char *title, char *s, int y, int x, int w, int len)
     so->w = w;
     so->len = len;
     so->win = win;
+    so->attr_mask = DialogInputAttrs;	/* Grossly use a global to avoid changing API */
 
     /* Draw it on the screen */
     RefreshStringObj(so);
@@ -302,8 +325,8 @@ SelectStringObj(StringObj *so)
     char	tmp[so->len+1];
 
     strcpy(tmp, so->s);
-    key = line_edit(so->win, so->y+2, so->x+1, 
-		    so->len, so->w-2, inputbox_attr, TRUE, tmp);
+    key = line_edit(so->win, so->y+2, so->x+1,
+		    so->len, so->w-2, inputbox_attr, TRUE, tmp, so->attr_mask);
     if ((key == '\n') || (key == '\r') || (key == '\t') || key == (KEY_BTAB) ) {
 	strcpy(so->s, tmp);
     }
@@ -335,7 +358,7 @@ DelStringObj(StringObj *so)
 
    return;
 }
-  
+
 /***********************************************************************
  *
  * ListObj routines
@@ -377,7 +400,7 @@ DrawNames(ListObj *lo)
 	i++;
     }
 
-    return;  
+    return;
 } /* DrawNames() */
 
 void
@@ -401,12 +424,12 @@ RefreshListObj(ListObj *lo)
     sprintf(perc, "(%3d%%)", MIN(100, (int) (100 * (lo->sel+lo->h-2) / MAX(1, lo->n))));
     wmove(lo->win, lo->y + lo->h, lo->x + lo->w - 8);
     wattrset(lo->win, dialog_attr);
-    waddstr(lo->win, perc); 
-    
-   
+    waddstr(lo->win, perc);
+
+
     return;
 } /* RefreshListObj() */
-  
+
 ListObj *
 NewListObj(WINDOW *win, char *title, char **list, char *listelt, int y, int x,
 	   int h, int w, int n)
@@ -485,7 +508,7 @@ UpdateListObj(ListObj *lo, char **list, int n)
 	}
     } else {
         lo->seld = NULL;
-    }    
+    }
     lo->n = n;
     lo->scroll = 0;
     lo->sel = 0;
@@ -521,7 +544,7 @@ SelectListObj(ListObj *lo)
 
     key = wgetch(lo->win);
     quit = FALSE;
-    while ((key != '\t') && (key != '\n') && (key != '\r') 
+    while ((key != '\t') && (key != '\n') && (key != '\r')
 	   && (key != ESC) && (key != KEY_F(1)) && (key != '?') && !quit) {
 	/* first draw current item in normal video */
 	wmove(lo->win, sel_y, sel_x);
@@ -620,11 +643,11 @@ SelectListObj(ListObj *lo)
 	    break;
 	}
 	/* Draw % indication */
-	sprintf(perc, "(%3d%%)", MIN(100, (int) 
+	sprintf(perc, "(%3d%%)", MIN(100, (int)
 				     (100 * (lo->sel+lo->h - 2) / MAX(1, lo->n))));
 	wmove(lo->win, lo->y + lo->h, lo->x + lo->w - 8);
 	wattrset(lo->win, dialog_attr);
-	waddstr(lo->win, perc); 
+	waddstr(lo->win, perc);
 
 	/* draw current item in inverse */
 	wmove(lo->win, sel_y, sel_x);
@@ -632,8 +655,8 @@ SelectListObj(ListObj *lo)
 	if (strlen(lo->name[lo->sel]) > lo->w - 2) {
 	    /* when printing in inverse video show the last characters in the */
 	    /* name that will fit in the window */
-	    strncpy(tmp, 
-		    lo->name[lo->sel] + strlen(lo->name[lo->sel]) - (lo->w - 2), 
+	    strncpy(tmp,
+		    lo->name[lo->sel] + strlen(lo->name[lo->sel]) - (lo->w - 2),
 		    lo->w - 2);
 	    tmp[lo->w - 2] = 0;
 	    waddstr(lo->win, tmp);
@@ -642,7 +665,7 @@ SelectListObj(ListObj *lo)
 	}
 	if (!quit) key = wgetch(lo->win);
     }
-		
+
     if (key == ESC) {
 	return(SEL_ESC);
     }
@@ -674,8 +697,8 @@ DelListObj(ListObj *lo)
 
 void
 MarkCurrentListObj(ListObj *lo)
-/* 
- * Desc: mark the current item for the selection list 
+/*
+ * Desc: mark the current item for the selection list
  */
 {
     lo->seld[lo->sel] = !(lo->seld[lo->sel]);
@@ -707,7 +730,7 @@ UnMarkAllListObj(ListObj *lo)
  */
 {
     int i;
-    
+
     for (i=0; i<lo->n; i++) {
         lo->seld[i] = FALSE;
     }
@@ -745,7 +768,7 @@ NewButtonObj(WINDOW *win, char *title, int *pushed, int y, int x)
     ButtonObj	*bo;
 
     bo = (ButtonObj *) malloc( sizeof(ButtonObj) );
-    
+
     bo->win = win;
     bo->title = (char *) malloc( strlen(title) + 1);
     strcpy(bo->title, title);
@@ -771,7 +794,7 @@ SelectButtonObj(ButtonObj *bo)
     print_button(bo->win, bo->title, bo->y+1, bo->x+2, TRUE);
     wmove(bo->win, bo->y+1, bo->x+(bo->w/2)-1);
     key = wgetch(bo->win);
-    print_button(bo->win, bo->title, bo->y+1, bo->x+2, FALSE);   
+    print_button(bo->win, bo->title, bo->y+1, bo->x+2, FALSE);
     switch(key) {
     case '\t':
 	return(SEL_TAB);
