@@ -216,12 +216,12 @@ int _rl_horizontal_scroll_mode = 0;
 
 /* Non-zero means to display an asterisk at the starts of history lines
    which have been modified. */
-int _rl_mark_modified_lines = 0;  
+int _rl_mark_modified_lines = 0;
 
 /* The style of `bell' notification preferred.  This can be set to NO_BELL,
    AUDIBLE_BELL, or VISIBLE_BELL. */
 int _rl_bell_preference = AUDIBLE_BELL;
-     
+
 /* Line buffer and maintenence. */
 char *rl_line_buffer = (char *)NULL;
 int rl_line_buffer_len = 0;
@@ -1003,7 +1003,7 @@ readline_initialize_everything ()
       _rl_convert_meta_chars_to_ascii = 0;
       _rl_output_meta_chars = 1;
     }
-      
+
   /* Read in the init file. */
   rl_read_init_file ((char *)NULL);
 
@@ -2110,7 +2110,7 @@ rl_quoted_insert (count, key)
   int c;
 
   c = rl_read_key ();
-  return (rl_insert (count, c));  
+  return (rl_insert (count, c));
 }
 
 /* Insert a tab character. */
@@ -2224,7 +2224,7 @@ rl_delete (count, invoking_key)
     }
   else
     return (rl_delete_text (rl_point, rl_point + 1));
-  
+
 }
 
 /* Delete all spaces and tabs around point. */
@@ -3122,7 +3122,7 @@ rl_kill_line (direction, ignore)
     return (rl_backward_kill_line (1));
   else
     {
-      rl_end_of_line ();
+      rl_end_of_line (1, ignore);
       if (orig_point != rl_point)
 	rl_kill_text (orig_point, rl_point);
       rl_point = orig_point;
@@ -3145,7 +3145,7 @@ rl_backward_kill_line (direction, ignore)
 	ding ();
       else
 	{
-	  rl_beg_of_line ();
+	  rl_beg_of_line (1, ignore);
 	  rl_kill_text (orig_point, rl_point);
 	}
     }
@@ -3169,7 +3169,7 @@ rl_yank (count, ignore)
 {
   if (!rl_kill_ring)
     {
-      rl_abort ();
+      rl_abort (count, ignore);
       return -1;
     }
 
@@ -3190,7 +3190,7 @@ rl_yank_pop (count, key)
   if (((rl_last_func != rl_yank_pop) && (rl_last_func != rl_yank)) ||
       !rl_kill_ring)
     {
-      rl_abort ();
+      rl_abort (1, key);
       return -1;
     }
 
@@ -3209,7 +3209,7 @@ rl_yank_pop (count, key)
     }
   else
     {
-      rl_abort ();
+      rl_abort (1, key);
       return -1;
     }
 }
