@@ -179,8 +179,8 @@
 #include "cvs.h"
 
 #ifndef lint
-static char rcsid[] = "$CVSid: @(#)history.c 1.33 94/09/21 $";
-USE(rcsid)
+static const char rcsid[] = "$CVSid: @(#)history.c 1.33 94/09/21 $";
+USE(rcsid);
 #endif
 
 static struct hrec
@@ -201,7 +201,7 @@ static struct hrec
 static char *fill_hrec PROTO((char *line, struct hrec * hr));
 static int accept_hrec PROTO((struct hrec * hr, struct hrec * lr));
 static int select_hrec PROTO((struct hrec * hr));
-static int sort_order PROTO((CONST PTR l, CONST PTR r));
+static int sort_order PROTO((const PTR l, const PTR r));
 static int within PROTO((char *find, char *string));
 static time_t date_and_time PROTO((char *date_str));
 static void expand_modules PROTO((void));
@@ -268,11 +268,9 @@ static char **mod_list;		/* Ptr to array of ptrs to module names */
 static int mod_max;		/* Number of elements allocated */
 static int mod_count;		/* Number of elements used */
 
-static int histsize;
-static char *histdata;
 static char *histfile;		/* Ptr to the history file name */
 
-static char *history_usg[] =
+static const char *const history_usg[] =
 {
     "Usage: %s %s [-report] [-flags] [-options args] [files...]\n\n",
     "   Reports:\n",
@@ -307,12 +305,12 @@ static char *history_usg[] =
 */
 static int
 sort_order (l, r)
-    CONST PTR l;
-    CONST PTR r;
+    const PTR l;
+    const PTR r;
 {
     int i;
-    CONST struct hrec *left = (CONST struct hrec *) l;
-    CONST struct hrec *right = (CONST struct hrec *) r;
+    const struct hrec *left = (const struct hrec *) l;
+    const struct hrec *right = (const struct hrec *) r;
 
     if (user_sort)	/* If Sort by username, compare users */
     {
@@ -531,6 +529,62 @@ history (argc, argv)
     else if (report_count > 1)
 	error (1, 0, "Only one report type allowed from: \"-Tcomx\".");
 
+#ifdef CLIENT_SUPPORT
+    if (client_active)
+    {
+	struct file_list_str *f1;
+	char **mod;
+
+	/* We're the client side.  Fire up the remote server.  */
+	start_server ();
+	
+	ign_setup ();
+
+	if (tag_report)
+	    send_arg("-T");
+	if (all_users)
+	    send_arg("-a");
+	if (modified)
+	    send_arg("-c");
+	if (last_entry)
+	    send_arg("-l");
+	if (v_checkout)
+	    send_arg("-o");
+	if (working)
+	    send_arg("-w");
+	if (histfile)
+	    send_arg("-X");
+	if (since_date)
+	    option_with_arg ("-D", asctime (gmtime (&since_date)));
+	if (backto[0] != '\0')
+	    option_with_arg ("-b", backto);
+	for (f1 = file_list; f1 < &file_list[file_count]; ++f1)
+	{
+	    if (f1->l_file[0] == '*')
+		option_with_arg ("-p", f1->l_file + 1);
+	    else
+		option_with_arg ("-f", f1->l_file);
+	}
+	if (module_report)
+	    send_arg("-m");
+	for (mod = mod_list; mod < &mod_list[mod_count]; ++mod)
+	    option_with_arg ("-n", *mod);
+	if (since_rev != NULL)
+	    option_with_arg ("-r", since_rev);
+	if (since_tag != NULL)
+	    option_with_arg ("-t", since_tag);
+	for (mod = user_list; mod < &user_list[user_count]; ++mod)
+	    option_with_arg ("-u", *mod);
+	if (extract)
+	    option_with_arg ("-x", rec_types);
+	option_with_arg ("-z", tz_name);
+
+	if (fprintf (to_server, "history\n") < 0)
+	    error (1, errno, "writing to server");
+        return get_responses_and_close ();
+    }
+#endif
+
     if (all_users)
 	save_user ("");
 
@@ -621,7 +675,8 @@ history_write (type, update_dir, revs, name, repository)
 {
     char fname[PATH_MAX], workdir[PATH_MAX], homedir[PATH_MAX];
     static char username[20];		/* !!! Should be global */
-    FILE *fp;
+    int fd;
+    char *line;
     char *slash = "", *cp, *cp2, *repos;
     int i;
     static char *tilde = "";
@@ -638,8 +693,17 @@ history_write (type, update_dir, revs, name, repository)
 	return;
     }
 
-    if (!(fp = Fopen (fname, "a")))	/* Some directory not there! */
+    if (trace)
+#ifdef SERVER_SUPPORT
+	fprintf (stderr, "%c-> fopen(%s,a)\n",
+		 (server_active) ? 'S' : ' ', fname);
+#else
+	fprintf (stderr, "-> fopen(%s,a)\n", fname);
+#endif
+    if (noexec)
 	return;
+    if ((fd = open (fname, O_WRONLY | O_APPEND | O_CREAT, 0666)) < 0)
+	error (1, errno, "cannot open history file: %s", fname);
 
     repos = Short_Repository (repository);
 
@@ -747,10 +811,23 @@ history_write (type, update_dir, revs, name, repository)
 	(void) sprintf ((cp + 1), "*%x", i);
     }
 
-    if (fprintf (fp, "%c%08x|%s|%s|%s|%s|%s\n", type, time ((time_t *) NULL),
-		 username, workdir, repos, revs ? revs : "", name) == EOF)
+    if (!revs)
+	revs = "";
+    line = xmalloc (strlen (username) + strlen (workdir) + strlen (repos)
+		    + strlen (revs) + strlen (name) + 100);
+    sprintf (line, "%c%08lx|%s|%s|%s|%s|%s\n",
+	     type, (long) time ((time_t *) NULL),
+	     username, workdir, repos, revs, name);
+
+    /* Lessen some race conditions on non-Posix-compliant hosts.  */
+    if (lseek (fd, (off_t) 0, SEEK_END) == -1)
+	error (1, errno, "cannot seek to end of history file: %s", fname);
+
+    if (write (fd, line, strlen (line)) < 0)
 	error (1, errno, "cannot write to history file: %s", fname);
-    (void) fclose (fp);
+    free (line);
+    if (close (fd) != 0)
+	error (1, errno, "cannot close history file: %s", fname);
 }
 
 /*
@@ -931,8 +1008,7 @@ read_hrecs (fname)
     /* Exactly enough space for lines data */
     if (!(i = st_buf.st_size))
 	error (1, 0, "history file is empty");
-    histdata = cp = xmalloc (i + 2);
-    histsize = i;
+    cp = xmalloc (i + 2);
 
     if (read (fd, cp, i) != i)
 	error (1, errno, "cannot read log file");
