@@ -21,6 +21,7 @@
  *		commit		Checks files into the repository
  *		diff		Runs diffs between revisions
  *		log		Prints "rlog" information for files
+ *		login		Record user, host, repos, password
  *		add		Adds an entry to the repository
  *		remove		Removes an entry from the repository
  *		status		Status info on the revisions
@@ -35,16 +36,40 @@
 #include "cvs.h"
 #include "patchlevel.h"
 
-#ifndef lint
-char rcsid[] = "$CVSid: @(#)main.c 1.78 94/10/07 $\n";
-USE(rcsid)
+#if HAVE_KERBEROS
+#include <sys/socket.h>
+#include <netinet/in.h>
+#include <krb.h>
+#ifndef HAVE_KRB_GET_ERR_TEXT
+#define krb_get_err_text(status) krb_err_txt[status]
+#endif
 #endif
 
-extern char *getenv ();
+#ifndef lint
+static const char rcsid[] = "$CVSid: @(#)main.c 1.78 94/10/07 $\n";
+USE(rcsid);
+#endif
 
 char *program_name;
-char *command_name = "";
+char *program_path;
+/*
+ * Initialize comamnd_name to "cvs" so that the first call to
+ * read_cvsrc tries to find global cvs options.
+ */
+char *command_name = "cvs";
 
+/*
+ * Since some systems don't define this...
+ */
+#ifndef MAXHOSTNAMELEN
+#define MAXHOSTNAMELEN  256
+#endif
+
+char hostname[MAXHOSTNAMELEN];
+
+#ifdef AUTH_CLIENT_SUPPORT
+int use_authenticating_server = FALSE;
+#endif /* AUTH_CLIENT_SUPPORT */
 int use_editor = TRUE;
 int use_cvsrc = TRUE;
 int cvswrite = !CVSREAD_DFLT;
@@ -53,6 +78,7 @@ int quiet = FALSE;
 int trace = FALSE;
 int noexec = FALSE;
 int logoff = FALSE;
+mode_t cvsumask = UMASK_DFLT;
 
 char *CurDir;
 
@@ -77,6 +103,9 @@ int diff PROTO((int argc, char **argv));
 int history PROTO((int argc, char **argv));
 int import PROTO((int argc, char **argv));
 int cvslog PROTO((int argc, char **argv));
+#ifdef AUTH_CLIENT_SUPPORT
+int login PROTO((int argc, char **argv));
+#endif /* AUTH_CLIENT_SUPPORT */
 int patch PROTO((int argc, char **argv));
 int release PROTO((int argc, char **argv));
 int cvsremove PROTO((int argc, char **argv));
@@ -85,41 +114,69 @@ int status PROTO((int argc, char **argv));
 int tag PROTO((int argc, char **argv));
 int update PROTO((int argc, char **argv));
 
-struct cmd
+const struct cmd
 {
     char *fullname;		/* Full name of the function (e.g. "commit") */
     char *nick1;		/* alternate name (e.g. "ci") */
     char *nick2;		/* another alternate names (e.g. "ci") */
     int (*func) ();		/* Function takes (argc, argv) arguments. */
+#ifdef CLIENT_SUPPORT
+    int (*client_func) ();	/* Function to do it via the protocol.  */
+#endif
 } cmds[] =
 
 {
-    { "add", "ad", "new", add },
-    { "admin", "adm", "rcs", admin },
-    { "checkout", "co", "get", checkout },
-    { "commit", "ci", "com", commit },
-    { "diff", "di", "dif", diff },
-    { "export", "exp", "ex", checkout },
-    { "history", "hi", "his", history },
-    { "import", "im", "imp", import },
-    { "log", "lo", "rlog", cvslog },
-    { "rdiff", "patch", "pa", patch },
-    { "release", "re", "rel", release },
-    { "remove", "rm", "delete", cvsremove },
-    { "status", "st", "stat", status },
-    { "rtag", "rt", "rfreeze", rtag },
-    { "tag", "ta", "freeze", tag },
-    { "update", "up", "upd", update },
-    { NULL, NULL, NULL, NULL },
+#ifdef CLIENT_SUPPORT
+#define CMD_ENTRY(n1, n2, n3, f1, f2) { n1, n2, n3, f1, f2 }
+#else
+#define CMD_ENTRY(n1, n2, n3, f1, f2) { n1, n2, n3, f1 }
+#endif
+
+    CMD_ENTRY("add",      "ad",    "new",     add,       client_add),
+#ifndef CVS_NOADMIN
+    CMD_ENTRY("admin",    "adm",   "rcs",     admin,     client_admin),
+#endif
+    CMD_ENTRY("checkout", "co",    "get",     checkout,  client_checkout),
+    CMD_ENTRY("commit",   "ci",    "com",     commit,    client_commit),
+    CMD_ENTRY("diff",     "di",    "dif",     diff,      client_diff),
+    CMD_ENTRY("export",   "exp",   "ex",      checkout,  client_export),
+    CMD_ENTRY("history",  "hi",    "his",     history,   client_history),
+    CMD_ENTRY("import",   "im",    "imp",     import,    client_import),
+    CMD_ENTRY("log",      "lo",    "rlog",    cvslog,    client_log),
+#ifdef AUTH_CLIENT_SUPPORT
+    CMD_ENTRY("login",    "logon", "lgn",     login,     login),
+#endif /* AUTH_CLIENT_SUPPORT */
+    CMD_ENTRY("rdiff",    "patch", "pa",      patch,     client_rdiff),
+    CMD_ENTRY("release",  "re",    "rel",     release,   client_release),
+    CMD_ENTRY("remove",   "rm",    "delete",  cvsremove, client_remove),
+    CMD_ENTRY("status",   "st",    "stat",    status,    client_status),
+    CMD_ENTRY("rtag",     "rt",    "rfreeze", rtag,      client_rtag),
+    CMD_ENTRY("tag",      "ta",    "freeze",  tag,       client_tag),
+    CMD_ENTRY("update",   "up",    "upd",     update,    client_update),
+
+#ifdef SERVER_SUPPORT
+    /*
+     * The client_func is also server because we might have picked up a
+     * CVSROOT environment variable containing a colon.  The client will send
+     * the real root later.
+     */
+    CMD_ENTRY("server",   "server", "server", server,    server),
+#endif
+    CMD_ENTRY(NULL, NULL, NULL, NULL, NULL),
+
+#undef CMD_ENTRY
 };
 
-static char *usg[] =
+static const char *const usg[] =
 {
     "Usage: %s [cvs-options] command [command-options] [files...]\n",
     "    Where 'cvs-options' are:\n",
     "        -H           Displays Usage information for command\n",
     "        -Q           Cause CVS to be really quiet.\n",
     "        -q           Cause CVS to be somewhat quiet.\n",
+#ifdef AUTH_CLIENT_SUPPORT
+    "        -a           Use the authenticating server, not rsh.\n",
+#endif /* AUTH_CLIENT_SUPPORT */
     "        -r           Make checked-out files read-only\n",
     "        -w           Make checked-out files read-write (default)\n",
     "        -l           Turn History logging off\n",
@@ -130,6 +187,9 @@ static char *usg[] =
     "        -e editor    Use 'editor' for editing log information\n",
     "        -d CVS_root  Overrides $CVSROOT as the root of the CVS tree\n",
     "        -f           Do not use the ~/.cvsrc file\n",
+#ifdef CLIENT_SUPPORT
+    "        -z #         Use 'gzip -#' for net traffic if possible.\n",
+#endif
     "\n",
     "    and where 'command' is:\n",
     "        add          Adds a new file/directory to the repository\n",
@@ -141,6 +201,9 @@ static char *usg[] =
     "        import       Import sources into CVS, using vendor branches\n",
     "        export       Export sources from CVS, similar to checkout\n",
     "        log          Prints out 'rlog' information for files\n",
+#ifdef AUTH_CLIENT_SUPPORT
+    "        login        Prompt for password for authenticating server.\n",
+#endif /* AUTH_CLIENT_SUPPORT */
     "        rdiff        'patch' format diffs between releases\n",
     "        release      Indicate that a Module is no longer in use\n",
     "        remove       Removes an entry from the repository\n",
@@ -157,29 +220,61 @@ main_cleanup ()
     exit (1);
 }
 
+static void
+error_cleanup ()
+{
+    Lock_Cleanup();
+#ifdef SERVER_SUPPORT
+    if (server_active)
+	server_cleanup (0);
+#endif
+}
+
 int
 main (argc, argv)
     int argc;
-    char *argv[];
+    char **argv;
 {
     extern char *version_string;
-    char *cp;
-    struct cmd *cm;
-    int c, help = FALSE, err = 0;
+    extern char *config_string;
+    char *cp, *end;
+    const struct cmd *cm;
+    int c, err = 0;
+    static int help = FALSE, version_flag = FALSE;
     int rcsbin_update_env, cvs_update_env = 0;
     char tmp[PATH_MAX];
+    static struct option long_options[] =
+      {
+        {"help", 0, &help, TRUE},
+        {"version", 0, &version_flag, TRUE},
+        {0, 0, 0, 0}
+      };
+    /* `getopt_long' stores the option index here, but right now we
+        don't use it. */
+    int option_index = 0;
+
+    error_set_cleanup (error_cleanup);
+
+/* The IBM TCP/IP library under OS/2 needs to be initialized: */
+#ifdef NEED_CALL_SOCKINIT
+	if (SockInit () != TRUE)
+	{
+          fprintf (stderr, "SockInit() failed!\n");
+          exit (1);
+	}
+#endif /* NEED_CALL_SOCKINIT */
 
     /*
      * Just save the last component of the path for error messages
      */
-    if ((program_name = strrchr (argv[0], '/')) == NULL)
-	program_name = argv[0];
-    else
-	program_name++;
+    program_path = xstrdup (argv[0]);
+    program_name = last_component (argv[0]);
 
     CurDir = xmalloc (PATH_MAX);
+#ifndef SERVER_SUPPORT
     if (!getwd (CurDir))
 	error (1, 0, "cannot get working directory: %s", CurDir);
+#endif
 
     /*
      * Query the environment variables up-front, so that
@@ -196,6 +291,8 @@ main (argc, argv)
  	Editor = cp;
     else if ((cp = getenv (EDITOR2_ENV)) != NULL)
 	Editor = cp;
+    else if ((cp = getenv (EDITOR3_ENV)) != NULL)
+	Editor = cp;
     if ((cp = getenv (CVSROOT_ENV)) != NULL)
     {
 	CVSroot = cp;
@@ -203,12 +300,36 @@ main (argc, argv)
     }
     if (getenv (CVSREAD_ENV) != NULL)
 	cvswrite = FALSE;
-
-    optind = 1;
-    while ((c = getopt (argc, argv, "Qqrwtnlvb:e:d:Hf")) != -1)
+    if ((cp = getenv (CVSUMASK_ENV)) != NULL)
     {
+	/* FIXME: Should be accepting symbolic as well as numeric mask.  */
+	cvsumask = strtol (cp, &end, 8) & 0777;
+	if (*end != '\0')
+	    error (1, errno, "invalid umask value in %s (%s)",
+		CVSUMASK_ENV, cp);
+    }
+
+    /*
+     * Scan cvsrc file for global options.
+     */
+    read_cvsrc(&argc, &argv);
+
+    /* This has the effect of setting getopt's ordering to REQUIRE_ORDER,
+       which is what we need to distinguish between global options and
+       command options.  FIXME: It would appear to be possible to do this
+       much less kludgily by passing "+" as the first character to the
+       option string we pass to getopt_long.  */
+    optind = 1;
+
+    while ((c = getopt_long
+            (argc, argv, "Qqrawtnlvb:e:d:Hfz:", long_options, &option_index))
+           != EOF)
+      {
 	switch (c)
-	{
+          {
+            case 0:
+                /* getopt_long took care of setting the flag. */ 
+                break;
 	    case 'Q':
 		really_quiet = TRUE;
 		/* FALL THROUGH */
@@ -218,6 +339,11 @@ main (argc, argv)
 	    case 'r':
 		cvswrite = FALSE;
 		break;
+#ifdef AUTH_CLIENT_SUPPORT
+	    case 'a':
+		use_authenticating_server = TRUE;
+		break;
+#endif /* AUTH_CLIENT_SUPPORT */
 	    case 'w':
 		cvswrite = TRUE;
 		break;
@@ -230,11 +356,7 @@ main (argc, argv)
 		logoff = TRUE;
 		break;
 	    case 'v':
-		(void) fputs (version_string, stdout);
-		(void) sprintf (tmp, "Patch Level: %d\n", PATCHLEVEL);
-		(void) fputs (tmp, stdout);
-		(void) fputs ("\nCopyright (c) 1993-1994 Brian Berliner\nCopyright (c) 1993-1994 david d `zoo' zuhn\nCopyright (c) 1992, Brian Berliner and Jeff Polk\nCopyright (c) 1989-1992, Brian Berliner\n\nCVS may be copied only under the terms of the GNU General Public License,\na copy of which can be found with the CVS distribution kit.\n", stdout);
-		exit (0);
+                version_flag = TRUE;
 		break;
 	    case 'b':
 		Rcsbin = optarg;
@@ -254,55 +376,181 @@ main (argc, argv)
             case 'f':
 		use_cvsrc = FALSE;
 		break;
+#ifdef CLIENT_SUPPORT
+	    case 'z':
+		gzip_level = atoi (optarg);
+		if (gzip_level <= 0 || gzip_level > 9)
+		  error (1, 0,
+			 "gzip compression level must be between 1 and 9");
+		break;
+#endif
 	    case '?':
 	    default:
-		usage (usg);
+                usage (usg);
 	}
     }
+
+    if (version_flag == TRUE)
+      {
+        (void) fputs (version_string, stdout);
+        (void) fputs (config_string, stdout);
+        (void) sprintf (tmp, "Patch Level: %d\n", PATCHLEVEL);
+        (void) fputs (tmp, stdout);
+        (void) fputs ("\n", stdout);
+        (void) fputs ("Copyright (c) 1993-1994 Brian Berliner\n", stdout);
+        (void) fputs ("Copyright (c) 1993-1994 david d `zoo' zuhn\n", stdout);
+        (void) fputs ("Copyright (c) 1992, Brian Berliner and Jeff Polk\n", stdout);
+        (void) fputs ("Copyright (c) 1989-1992, Brian Berliner\n", stdout);
+        (void) fputs ("\n", stdout);
+        (void) fputs ("CVS may be copied only under the terms of the GNU General Public License,\n", stdout);
+        (void) fputs ("a copy of which can be found with the CVS distribution kit.\n", stdout);
+        exit (0);
+      }
+
     argc -= optind;
     argv += optind;
     if (argc < 1)
 	usage (usg);
 
+#ifdef HAVE_KERBEROS
+    /* If we are invoked with a single argument "kserver", then we are
+       running as Kerberos server as root.  Do the authentication as
+       the very first thing, to minimize the amount of time we are
+       running as root.  */
+    if (strcmp (argv[0], "kserver") == 0)
+    {
+	int status;
+	char instance[INST_SZ];
+	struct sockaddr_in peer;
+	struct sockaddr_in laddr;
+	int len;
+	KTEXT_ST ticket;
+	AUTH_DAT auth;
+	char version[KRB_SENDAUTH_VLEN];
+	Key_schedule sched;
+	char user[ANAME_SZ];
+	struct passwd *pw;
+
+	strcpy (instance, "*");
+	len = sizeof peer;
+	if (getpeername (STDIN_FILENO, (struct sockaddr *) &peer, &len) < 0
+	    || getsockname (STDIN_FILENO, (struct sockaddr *) &laddr,
+			    &len) < 0)
+	{
+	    printf ("E Fatal error, aborting.\n\
+error %s getpeername or getsockname failed\n", strerror (errno));
+	    exit (1);
+	}
+
+	status = krb_recvauth (KOPT_DO_MUTUAL, STDIN_FILENO, &ticket, "rcmd",
+			       instance, &peer, &laddr, &auth, "", sched,
+			       version);
+	if (status != KSUCCESS)
+	{
+	    printf ("E Fatal error, aborting.\n\
+error 0 kerberos: %s\n", krb_get_err_text(status));
+	    exit (1);
+	}
+
+	/* Get the local name.  */
+	status = krb_kntoln (&auth, user);
+	if (status != KSUCCESS)
+	{
+	    printf ("E Fatal error, aborting.\n\
+error 0 kerberos: can't get local name: %s\n", krb_get_err_text(status));
+	    exit (1);
+	}
+
+	pw = getpwnam (user);
+	if (pw == NULL)
+	{
+	    printf ("E Fatal error, aborting.\n\
+error 0 %s: no such user\n", user);
+	    exit (1);
+	}
+
+	initgroups (pw->pw_name, pw->pw_gid);
+	setgid (pw->pw_gid);
+	setuid (pw->pw_uid);
+	/* Inhibit access by randoms.  Don't want people randomly
+	   changing our temporary tree before we check things in.  */
+	umask (077);
+
+#if HAVE_PUTENV
+	/* Set LOGNAME and USER in the environment, in case they are
+           already set to something else.  */
+	{
+	    char *env;
+
+	    env = xmalloc (sizeof "LOGNAME=" + strlen (user));
+	    (void) sprintf (env, "LOGNAME=%s", user);
+	    (void) putenv (env);
+
+	    env = xmalloc (sizeof "USER=" + strlen (user));
+	    (void) sprintf (env, "USER=%s", user);
+	    (void) putenv (env);
+	}
+#endif
+
+	/* Pretend we were invoked as a plain server.  */
+	argv[0] = "server";
+    }
+#endif /* HAVE_KERBEROS */
+
+
+#if defined(AUTH_SERVER_SUPPORT) && defined(SERVER_SUPPORT)
+    if (strcmp (argv[0], "pserver") == 0)
+    {
+      /* Gets username and password from client, authenticates, then
+         switches to run as that user and sends an ACK back to the
+         client. */
+      authenticate_connection ();
+      
+      /* Pretend we were invoked as a plain server.  */
+      argv[0] = "server";
+    }
+#endif /* AUTH_SERVER_SUPPORT && SERVER_SUPPORT */
+
+
 #ifdef CVSADM_ROOT
     /*
      * See if we are able to find a 'better' value for CVSroot in the
      * CVSADM_ROOT directory.
      */
+#ifdef SERVER_SUPPORT
+    if (strcmp (argv[0], "server") == 0 && CVSroot == NULL)
+        CVSADM_Root = NULL;
+    else
+        CVSADM_Root = Name_Root((char *) NULL, (char *) NULL);
+#else /* No SERVER_SUPPORT */
     CVSADM_Root = Name_Root((char *) NULL, (char *) NULL);
+#endif /* No SERVER_SUPPORT */
     if (CVSADM_Root != NULL)
     {
-        if (CVSroot == NULL)
+        if (CVSroot == NULL || !cvs_update_env)
         {
 	    CVSroot = CVSADM_Root;
 	    cvs_update_env = 1;	/* need to update environment */
         }
+#ifdef CLIENT_SUPPORT
+        else if (!getenv ("CVS_IGNORE_REMOTE_ROOT"))
+#else
         else
+#endif
         {
             /*
 	     * Now for the hard part, compare the two directories. If they
 	     * are not identical, then abort this command.
 	     */
-            if ((strcmp (CVSroot, CVSADM_Root) != 0) &&
+            if ((fncmp (CVSroot, CVSADM_Root) != 0) &&
 		!same_directories(CVSroot, CVSADM_Root))
 	    {
-	        error (0, 0, "%s value for CVS Root found in %s",
-		       CVSADM_Root, CVSADM_ROOT);
-		if (cvs_update_env)
-		{
-		    error (0, 0, "does not match command line -d %s setting",
-			   CVSroot);
-		    error (1, 0,
-			   "you may wish to try the cvs command again without the -d option ");
-                }
-		else
-		{
-		    error (0, 0,
-			   "does not match CVSROOT environment value of %s",
-			   CVSroot);
-		    error (1, 0,
-			   "you may wish to unsetenv CVSROOT and try again");
-                }
+              error (0, 0, "%s value for CVS Root found in %s",
+                     CVSADM_Root, CVSADM_ROOT);
+              error (0, 0, "does not match command line -d %s setting",
+                     CVSroot);
+              error (1, 0,
+                      "you may wish to try the cvs command again without the -d option ");
 	    }
         }
     }
@@ -322,6 +570,9 @@ main (argc, argv)
 	 * we assume that we can't work in the repository.
 	 * BUT, only if the history file exists.
 	 */
+#ifdef SERVER_SUPPORT
+        if (strcmp (command_name, "server") != 0 || CVSroot != NULL)
+#endif
 	{
 	    char path[PATH_MAX];
 	    int save_errno;
@@ -330,16 +581,23 @@ main (argc, argv)
 		error (1, 0, "You don't have a %s environment variable",
 		       CVSROOT_ENV);
 	    (void) sprintf (path, "%s/%s", CVSroot, CVSROOTADM);
-	    if (access (path, R_OK | X_OK))
+	    if (!isaccessible (path, R_OK | X_OK))
 	    {
 		save_errno = errno;
+#ifdef CLIENT_SUPPORT
+		if (strchr (CVSroot, ':') == NULL)
+		{
+#endif
 		error (0, 0,
 		    "Sorry, you don't have sufficient access to %s", CVSroot);
 		error (1, save_errno, "%s", path);
+#ifdef CLIENT_SUPPORT
+		}
+#endif
 	    }
 	    (void) strcat (path, "/");
 	    (void) strcat (path, CVSROOTADM_HISTORY);
-	    if (isfile (path) && access (path, R_OK | W_OK))
+	    if (isfile (path) && !isaccessible (path, R_OK | W_OK))
 	    {
 		save_errno = errno;
 		error (0, 0,
@@ -349,6 +607,16 @@ main (argc, argv)
 	}
     }
 
+#ifdef SERVER_SUPPORT
+    if (strcmp (command_name, "server") == 0)
+	/* This is only used for writing into the history file.  Might
+	   be nice to have hostname and/or remote path, on the other hand
+	   I'm not sure whether it is worth the trouble.  */
+	strcpy (CurDir, "<remote>");
+    else if (!getwd (CurDir))
+	error (1, 0, "cannot get working directory: %s", CurDir);
+#endif
+
 #ifdef HAVE_PUTENV
     /* Now, see if we should update the environment with the Rcsbin value */
     if (cvs_update_env)
@@ -404,11 +672,30 @@ main (argc, argv)
     else
     {
 	command_name = cm->fullname;	/* Global pointer for later use */
+
+	/* make sure we clean up on error */
+#ifdef SIGHUP
 	(void) SIG_register (SIGHUP, main_cleanup);
+	(void) SIG_register (SIGHUP, Lock_Cleanup);
+#endif
+#ifdef SIGINT
 	(void) SIG_register (SIGINT, main_cleanup);
+	(void) SIG_register (SIGINT, Lock_Cleanup);
+#endif
+#ifdef SIGQUIT
 	(void) SIG_register (SIGQUIT, main_cleanup);
+	(void) SIG_register (SIGQUIT, Lock_Cleanup);
+#endif
+#ifdef SIGPIPE
 	(void) SIG_register (SIGPIPE, main_cleanup);
+	(void) SIG_register (SIGPIPE, Lock_Cleanup);
+#endif
+#ifdef SIGTERM
 	(void) SIG_register (SIGTERM, main_cleanup);
+	(void) SIG_register (SIGTERM, Lock_Cleanup);
+#endif
+
+	gethostname(hostname, sizeof (hostname));
 
 #ifdef HAVE_SETVBUF
 	/*
@@ -422,8 +709,19 @@ main (argc, argv)
 	if (use_cvsrc)
 	  read_cvsrc(&argc, &argv);
 
+#ifdef CLIENT_SUPPORT
+	/* If cvsroot contains a colon, try to do it via the protocol.  */
+        {
+	    char *p = CVSroot == NULL ? NULL : strchr (CVSroot, ':');
+	    if (p)
+		err = (*(cm->client_func)) (argc, argv);
+	    else
+		err = (*(cm->func)) (argc, argv);
+	}
+#else /* No CLIENT_SUPPORT */
 	err = (*(cm->func)) (argc, argv);
 
+#endif /* No CLIENT_SUPPORT */
     }
     /*
      * If the command's error count is modulo 256, we need to change it
@@ -462,7 +760,7 @@ Make_Date (rawdate)
 
 void
 usage (cpp)
-    register char **cpp;
+    register const char *const *cpp;
 {
     (void) fprintf (stderr, *cpp++, program_name, command_name);
     for (; *cpp; cpp++)
