@@ -11,12 +11,24 @@
 #define  ABSTRACT_DEFAULT 	NULL
 #define  VOLSET_ID_DEFAULT 	NULL
 #define  VOLUME_ID_DEFAULT 	"CDROM"
+
 #ifdef __FreeBSD__
 #define  SYSTEM_ID_DEFAULT 	"FreeBSD"
-#else
+#endif
+
 #ifdef __QNX__
 #define  SYSTEM_ID_DEFAULT 	"QNX"
-#else
+#endif
+
+#ifdef __linux__
 #define  SYSTEM_ID_DEFAULT 	"LINUX"
 #endif
+
+
+#ifdef __osf__
+#define  SYSTEM_ID_DEFAULT 	"OSF"
+#endif
+
+#ifndef SYSTEM_ID_DEFAULT
+#define  SYSTEM_ID_DEFAULT 	"UNKNOWNSYS"
 #endif
