@@ -1,6 +1,6 @@
 /*
  * 9-Dec-93 R.-D. Marzusch, marzusch@odiehh.hanse.de:
- * added 'exclude' option (-x) to specify pathnames NOT to be included in 
+ * added 'exclude' option (-x) to specify pathnames NOT to be included in
  * CD image.
  */
 
@@ -29,8 +29,7 @@ char * fn;
     return;
   }
 
- 
-  excl[i] = malloc(strlen(fn)+1);
+  excl[i] = (char *) malloc(strlen(fn)+1);
   if (! excl[i]) {
     fprintf(stderr,"Can't allocate memory for excluded filename\n");
     return;
