@@ -1,6 +1,25 @@
-/* $Header: /home/cvs/386BSD/src/gnu/usr.bin/patch/pch.c,v 1.3 1994/02/17 22:20:36 jkh Exp $
+/* $FreeBSD: stable/2.1/gnu/usr.bin/patch/pch.c 50952 1999-09-05 11:54:50Z peter $
  *
  * $Log: pch.c,v $
+ * Revision 1.5.4.2  1996/04/11  10:33:45  markm
+ * Merge with HEAD an improved Index: priority fix.
+ *
+ * Revision 1.5.4.1  1995/09/17 06:39:59  davidg
+ * Brought in changes from rev 1.6: Give 'Index:' priority over other paths.
+ *
+ * Revision 1.6  1995/09/14  04:33:35  gibbs
+ * Give "Index" specified filenames preference over other filenames specified
+ * in the diff.  This makes it so that diffs containing files in different
+ * subdirectories that have the same name not patch the same file.  For example
+ * a diff with patches to Makefile, des/Makefile, usr.bin/Makefile would attempt
+ * to patch Makefile three times.
+ *
+ * Revision 1.5  1995/05/30  05:02:35  rgrimes
+ * Remove trailing whitespace.
+ *
+ * Revision 1.4  1994/02/25  21:46:07  phk
+ * added the -C/-check again.
+ *
  * Revision 1.3  1994/02/17  22:20:36  jkh
  * Put this back - I was somehow under the erroneous impression that patch was in
  * ports, until I saw the the commit messages, that is! :-)  All changed backed out.
@@ -18,29 +37,29 @@
  * Revision 2.0.1.7  88/06/03  15:13:28  lwall
  * patch10: Can now find patches in shar scripts.
  * patch10: Hunks that swapped and then swapped back could core dump.
- * 
+ *
  * Revision 2.0.1.6  87/06/04  16:18:13  lwall
  * pch_swap didn't swap p_bfake and p_efake.
- * 
+ *
  * Revision 2.0.1.5  87/01/30  22:47:42  lwall
  * Improved responses to mangled patches.
- * 
+ *
  * Revision 2.0.1.4  87/01/05  16:59:53  lwall
  * New-style context diffs caused double call to free().
- * 
+ *
  * Revision 2.0.1.3  86/11/14  10:08:33  lwall
  * Fixed problem where a long pattern wouldn't grow the hunk.
  * Also restored p_input_line when backtracking so error messages are right.
- * 
+ *
  * Revision 2.0.1.2  86/11/03  17:49:52  lwall
  * New-style delete triggers spurious assertion error.
- * 
+ *
  * Revision 2.0.1.1  86/10/29  15:52:08  lwall
  * Could falsely report new-style context diff.
- * 
+ *
  * Revision 2.0  86/09/17  15:39:37  lwall
  * Baseline for netwide release.
- * 
+ *
  */
 
 #include "EXTERN.h"
@@ -132,7 +151,7 @@ void
 grow_hunkmax()
 {
     hunkmax *= 2;
-    /* 
+    /*
      * Note that on most systems, only the p_line array ever gets fresh memory
      * since p_len can move into p_line's old space, and p_Char can move into
      * p_len's old space.  Not on PDP-11's however.  But it doesn't matter.
@@ -269,10 +288,10 @@ intuit_diff_type()
 	    else
 		indent++;
 	}
-	for (t=s; isdigit(*t) || *t == ','; t++) ; 
+	for (t=s; isdigit(*t) || *t == ','; t++) ;
 	this_is_a_command = (isdigit(*s) &&
 	  (*t == 'd' || *t == 'c' || *t == 'a') );
-	if (first_command_line < 0L && this_is_a_command) { 
+	if (first_command_line < 0L && this_is_a_command) {
 	    first_command_line = this_line;
 	    fcl_line = p_input_line;
 	    p_indent = indent;		/* assume this for now */
@@ -328,7 +347,7 @@ intuit_diff_type()
 	    retval = (*(s-1) == '*' ? NEW_CONTEXT_DIFF : CONTEXT_DIFF);
 	    goto scan_exit;
 	}
-	if ((!diff_type || diff_type == NORMAL_DIFF) && 
+	if ((!diff_type || diff_type == NORMAL_DIFF) &&
 	  last_line_was_command &&
 	  (strnEQ(s, "< ", 2) || strnEQ(s, "> ", 2)) ) {
 	    p_start = previous_line;
@@ -346,18 +365,20 @@ intuit_diff_type()
 	    oldname = fetchname(oldtmp, strippath, ok_to_create_file);
 	if (newtmp != Nullch)
 	    newname = fetchname(newtmp, strippath, ok_to_create_file);
-	if (oldname && newname) {
+	if (indname)
+	    filearg[0] = savestr(indname);
+	else if (oldname && newname) {
 	    if (strlen(oldname) < strlen(newname))
 		filearg[0] = savestr(oldname);
 	    else
 		filearg[0] = savestr(newname);
 	}
+	else if (indname)
+	    filearg[0] = savestr(indname);
 	else if (oldname)
 	    filearg[0] = savestr(oldname);
 	else if (newname)
 	    filearg[0] = savestr(newname);
-	else if (indname)
-	    filearg[0] = savestr(indname);
     }
     if (bestguess) {
 	free(bestguess);
@@ -698,19 +719,19 @@ another_hunk()
 	    else
 		p_len[p_end] = 0;
 	}
-	
+
     hunk_done:
 	if (p_end >=0 && !repl_beginning)
 	    fatal2("no --- found in patch at line %ld\n", pch_hunk_beg());
 
 	if (repl_missing) {
-	    
+
 	    /* reset state back to just after --- */
 	    p_input_line = repl_patch_line;
 	    for (p_end--; p_end > repl_beginning; p_end--)
 		free(p_line[p_end]);
 	    Fseek(pfp, repl_backtrack_position, 0);
-	    
+
 	    /* redundant 'new' context lines were omitted - set */
 	    /* up to fill them in from the old file context */
 	    if (!p_context && p_repl_lines == 1) {
@@ -749,7 +770,7 @@ another_hunk()
 "the new style...)");
 	    diff_type = NEW_CONTEXT_DIFF;
 	}
-	
+
 	/* if there were omitted context lines, fill them in now */
 	if (fillcnt) {
 	    p_bfake = filldst;		/* remember where not to free() */
@@ -1081,7 +1102,7 @@ pch_swap()
     i = p_first;
     p_first = p_newfirst;
     p_newfirst = i;
-    
+
     /* make a scratch copy */
 
     tp_line = p_line;
