@@ -1,9 +1,7 @@
-/*
- *                     RCS revision number handling
- */
+/* Handle RCS revision numbers.  */
 
-/* Copyright (C) 1982, 1988, 1989 Walter Tichy
-   Copyright 1990, 1991 by Paul Eggert
+/* Copyright 1982, 1988, 1989 Walter Tichy
+   Copyright 1990, 1991, 1992, 1993, 1994, 1995 Paul Eggert
    Distributed under license by the Free Software Foundation, Inc.
 
 This file is part of RCS.
@@ -19,8 +17,9 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with RCS; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+along with RCS; see the file COPYING.
+If not, write to the Free Software Foundation,
+59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
 Report problems and direct all questions to:
 
@@ -28,10 +27,31 @@ Report problems and direct all questions to:
 
 */
 
-
-
-
-/* $Log: rcsrev.c,v $
+/*
+ * Revision 5.10  1995/06/16 06:19:24  eggert
+ * Update FSF address.
+ *
+ * Revision 5.9  1995/06/01 16:23:43  eggert
+ * (cmpdate, normalizeyear): New functions work around MKS RCS incompatibility.
+ * (cmpnum, compartial): s[d] -> *(s+d) to work around Cray compiler bug.
+ * (genrevs, genbranch): cmpnum -> cmpdate
+ *
+ * Revision 5.8  1994/03/17 14:05:48  eggert
+ * Remove lint.
+ *
+ * Revision 5.7  1993/11/09 17:40:15  eggert
+ * Fix format string typos.
+ *
+ * Revision 5.6  1993/11/03 17:42:27  eggert
+ * Revision number `.N' now stands for `D.N', where D is the default branch.
+ * Add -z.  Improve quality of diagnostics.  Add `namedrev' for Name support.
+ *
+ * Revision 5.5  1992/07/28  16:12:44  eggert
+ * Identifiers may now start with a digit.  Avoid `unsigned'.
+ *
+ * Revision 5.4  1992/01/06  02:42:34  eggert
+ * while (E) ; -> while (E) continue;
+ *
  * Revision 5.3  1991/08/19  03:13:55  eggert
  * Add `-r$', `-rB.'.  Remove botches like `<now>' from messages.  Tune.
  *
@@ -48,25 +68,25 @@ Report problems and direct all questions to:
  *
  * Revision 4.5  89/05/01  15:13:22  narten
  * changed copyright header to reflect current distribution rules
- * 
+ *
  * Revision 4.4  87/12/18  11:45:22  narten
- * more lint cleanups. Also, the NOTREACHED comment is no longer necessary, 
+ * more lint cleanups. Also, the NOTREACHED comment is no longer necessary,
  * since there's now a return value there with a value. (Guy Harris)
- * 
+ *
  * Revision 4.3  87/10/18  10:38:42  narten
- * Updating version numbers. Changes relative to version 1.1 actually 
+ * Updating version numbers. Changes relative to version 1.1 actually
  * relative to 4.1
- * 
+ *
  * Revision 1.3  87/09/24  14:00:37  narten
- * Sources now pass through lint (if you ignore printf/sprintf/fprintf 
+ * Sources now pass through lint (if you ignore printf/sprintf/fprintf
  * warnings)
- * 
+ *
  * Revision 1.2  87/03/27  14:22:37  jenkins
  * Port to suns
- * 
+ *
  * Revision 4.1  83/03/25  21:10:45  wft
  * Only changed $Header to $Id.
- * 
+ *
  * Revision 3.4  82/12/04  13:24:08  wft
  * Replaced getdelta() with gettree().
  *
@@ -83,25 +103,21 @@ Report problems and direct all questions to:
  * in that case.
  */
 
-
-
-/*
-#define REVTEST
-*/
-/* version REVTEST is for testing the routines that generate a sequence
- * of delta numbers needed to regenerate a given delta.
- */
-
 #include "rcsbase.h"
 
-libId(revId, "$Id: rcsrev.c,v 5.3 1991/08/19 03:13:55 eggert Exp $")
+libId(revId, "$FreeBSD: stable/2.1/gnu/usr.bin/rcs/lib/rcsrev.c 50952 1999-09-05 11:54:50Z peter $")
 
 static char const *branchtip P((char const*));
-static struct hshentry *genbranch P((struct hshentry const*,char const*,unsigned,char const*,char const*,char const*,struct hshentries**));
+static char const *lookupsym P((char const*));
+static char const *normalizeyear P((char const*,char[5]));
+static struct hshentry *genbranch P((struct hshentry const*,char const*,int,char const*,char const*,char const*,struct hshentries**));
+static void absent P((char const*,int));
+static void cantfindbranch P((char const*,char const[datesize],char const*,char const*));
+static void store1 P((struct hshentries***,struct hshentry*));
 
 
 
-	unsigned
+	int
 countnumflds(s)
 	char const *s;
 /* Given a pointer s to a dotted number (date or revision number),
@@ -109,9 +125,9 @@ countnumflds(s)
  */
 {
 	register char const *sp;
-	register unsigned count;
-        if ((sp=s)==nil) return(0);
-        if (*sp == '\0') return(0);
+	register int count;
+	if (!(sp=s) || !*sp)
+		return 0;
         count = 1;
 	do {
                 if (*sp++ == '.') count++;
@@ -123,12 +139,12 @@ countnumflds(s)
 getbranchno(revno,branchno)
 	char const *revno;
 	struct buf *branchno;
-/* Given a non-nil revision number revno, getbranchno copies the number of the branch
+/* Given a revision number revno, getbranchno copies the number of the branch
  * on which revno is into branchno. If revno itself is a branch number,
  * it is copied unchanged.
  */
 {
-	register unsigned numflds;
+	register int numflds;
 	register char *tp;
 
 	bufscpy(branchno, revno);
@@ -137,7 +153,7 @@ getbranchno(revno,branchno)
 		tp = branchno->string;
 		while (--numflds)
 			while (*tp++ != '.')
-				;
+				continue;
                 *(tp-1)='\0';
         }
 }
@@ -156,8 +172,8 @@ int cmpnum(num1, num2)
 	register size_t d1, d2;
 	register int r;
 
-        s1=num1==nil?"":num1;
-        s2=num2==nil?"":num2;
+	s1 = num1 ? num1 : "";
+	s2 = num2 ? num2 : "";
 
 	for (;;) {
 		/* Give precedence to shorter one.  */
@@ -167,8 +183,10 @@ int cmpnum(num1, num2)
 			return -1;
 
 		/* Strip leading zeros, then find number of digits.  */
-		while (*s1=='0') ++s1;  for (d1=0; isdigit(s1[d1]); d1++) ;
-		while (*s2=='0') ++s2;  for (d2=0; isdigit(s2[d2]); d2++) ;
+		while (*s1=='0') ++s1;
+		while (*s2=='0') ++s2;
+		for (d1=0; isdigit(*(s1+d1)); d1++) continue;
+		for (d2=0; isdigit(*(s2+d2)); d2++) continue;
 
 		/* Do not convert to integer; it might overflow!  */
 		if (d1 != d2)
@@ -188,7 +206,7 @@ int cmpnum(num1, num2)
 
 int cmpnumfld(num1, num2, fld)
 	char const *num1, *num2;
-	unsigned fld;
+	int fld;
 /* Compare the two dotted numbers at field fld.
  * num1 and num2 must have at least fld fields.
  * fld must be positive.
@@ -202,25 +220,63 @@ int cmpnumfld(num1, num2, fld)
         /* skip fld-1 fields */
 	while (--fld) {
 		while (*s1++ != '.')
-			;
+			continue;
 		while (*s2++ != '.')
-			;
+			continue;
 	}
         /* Now s1 and s2 point to the beginning of the respective fields */
-	while (*s1=='0') ++s1;  for (d1=0; isdigit(s1[d1]); d1++) ;
-	while (*s2=='0') ++s2;  for (d2=0; isdigit(s2[d2]); d2++) ;
+	while (*s1=='0') ++s1;  for (d1=0; isdigit(*(s1+d1)); d1++) continue;
+	while (*s2=='0') ++s2;  for (d2=0; isdigit(*(s2+d2)); d2++) continue;
 
 	return d1<d2 ? -1 : d1==d2 ? memcmp(s1,s2,d1) : 1;
 }
 
 
+	int
+cmpdate(d1, d2)
+	char const *d1, *d2;
+/*
+* Compare the two dates.  This is just like cmpnum,
+* except that for compatibility with old versions of RCS,
+* 1900 is added to dates with two-digit years.
+*/
+{
+	char year1[5], year2[5];
+	int r = cmpnumfld(normalizeyear(d1,year1), normalizeyear(d2,year2), 1);
+
+	if (r)
+		return r;
+	else {
+		while (isdigit(*d1)) d1++;  d1 += *d1=='.';
+		while (isdigit(*d2)) d2++;  d2 += *d2=='.';
+		return cmpnum(d1, d2);
+	}
+}
+
+	static char const *
+normalizeyear(date, year)
+	char const *date;
+	char year[5];
+{
+	if (isdigit(date[0]) && isdigit(date[1]) && !isdigit(date[2])) {
+		year[0] = '1';
+		year[1] = '9';
+		year[2] = date[0];
+		year[3] = date[1];
+		year[4] = 0;
+		return year;
+	} else
+		return date;
+}
+
+
 	static void
 cantfindbranch(revno, date, author, state)
 	char const *revno, date[datesize], *author, *state;
 {
-	char datebuf[datesize];
+	char datebuf[datesize + zonelenmax];
 
-	error("No revision on branch %s has%s%s%s%s%s%s.",
+	rcserror("No revision on branch %s has%s%s%s%s%s%s.",
 		revno,
 		date ? " a date before " : "",
 		date ? date2str(date,datebuf) : "",
@@ -234,11 +290,11 @@ cantfindbranch(revno, date, author, state)
 	static void
 absent(revno, field)
 	char const *revno;
-	unsigned field;
+	int field;
 {
 	struct buf t;
 	bufautobegin(&t);
-	error("%s %s absent", field&1?"revision":"branch",
+	rcserror("%s %s absent", field&1?"revision":"branch",
 		partialno(&t,revno,field)
 	);
 	bufautoend(&t);
@@ -248,7 +304,7 @@ absent(revno, field)
 	int
 compartial(num1, num2, length)
 	char const *num1, *num2;
-	unsigned length;
+	int length;
 
 /*   compare the first "length" fields of two dot numbers;
      the omitted field is considered to be larger than any number  */
@@ -267,20 +323,21 @@ compartial(num1, num2, length)
 	    if (!*s1) return 1;
 	    if (!*s2) return -1;
 
-	    while (*s1=='0') ++s1;  for (d1=0; isdigit(s1[d1]); d1++) ;
-	    while (*s2=='0') ++s2;  for (d2=0; isdigit(s2[d2]); d2++) ;
+	    while (*s1=='0') ++s1; for (d1=0; isdigit(*(s1+d1)); d1++) continue;
+	    while (*s2=='0') ++s2; for (d2=0; isdigit(*(s2+d2)); d2++) continue;
 
 	    if (d1 != d2)
 		    return d1<d2 ? -1 : 1;
 	    if ((r = memcmp(s1, s2, d1)))
 		    return r;
+	    if (!--length)
+		    return 0;
+
 	    s1 += d1;
 	    s2 += d1;
 
 	    if (*s1 == '.') s1++;
             if (*s2 == '.') s2++;
-
-	    if ( --length == 0 ) return 0;
 	}
 }
 
@@ -288,7 +345,7 @@ compartial(num1, num2, length)
 char * partialno(rev1,rev2,length)
 	struct buf *rev1;
 	char const *rev2;
-	register unsigned length;
+	register int length;
 /* Function: Copies length fields of revision number rev2 into rev1.
  * Return rev1's string.
  */
@@ -335,20 +392,20 @@ struct hshentry * genrevs(revno,date,author,state,store)
  * revision given by revno, date, author, and state, and stores pointers
  * to these deltas into a list whose starting address is given by store.
  * The last delta (target delta) is returned.
- * If the proper delta could not be found, nil is returned.
+ * If the proper delta could not be found, 0 is returned.
  */
 {
-	unsigned length;
+	int length;
         register struct hshentry * next;
         int result;
 	char const *branchnum;
 	struct buf t;
-	char datebuf[datesize];
+	char datebuf[datesize + zonelenmax];
 
 	bufautobegin(&t);
 
 	if (!(next = Head)) {
-		error("RCS file empty");
+		rcserror("RCS file empty");
 		goto norev;
         }
 
@@ -360,7 +417,7 @@ struct hshentry * genrevs(revno,date,author,state,store)
 			store1(&store, next);
                         next = next->next;
 			if (!next) {
-			    error("branch number %s too low", partialno(&t,revno,1));
+			    rcserror("branch number %s too low", partialno(&t,revno,1));
 			    goto norev;
 			}
                 }
@@ -373,19 +430,19 @@ struct hshentry * genrevs(revno,date,author,state,store)
         if (length<=1){
                 /* pick latest one on given branch */
                 branchnum = next->num; /* works even for empty revno*/
-                while ((next!=nil) &&
-                       (cmpnumfld(branchnum,next->num,1)==0) &&
-                       !(
-                        (date==nil?1:(cmpnum(date,next->date)>=0)) &&
-                        (author==nil?1:(strcmp(author,next->author)==0)) &&
-                        (state ==nil?1:(strcmp(state, next->state) ==0))
-                        )
-                       )
+		while (next &&
+		       cmpnumfld(branchnum,next->num,1) == 0 &&
+		       (
+			(date && cmpdate(date,next->date) < 0) ||
+			(author && strcmp(author,next->author) != 0) ||
+			(state && strcmp(state,next->state) != 0)
+		       )
+		      )
 		{
 			store1(&store, next);
                         next=next->next;
                 }
-                if ((next==nil) ||
+		if (!next ||
                     (cmpnumfld(branchnum,next->num,1)!=0))/*overshot*/ {
 			cantfindbranch(
 				length ? revno : partialno(&t,branchnum,1),
@@ -395,7 +452,7 @@ struct hshentry * genrevs(revno,date,author,state,store)
                 } else {
 			store1(&store, next);
                 }
-                *store = nil;
+		*store = 0;
                 return next;
         }
 
@@ -409,8 +466,8 @@ struct hshentry * genrevs(revno,date,author,state,store)
 			break;
         }
 
-        if ((next==nil) || (cmpnumfld(revno,next->num,1)!=0)) {
-		error("revision number %s too low", partialno(&t,revno,2));
+	if (!next || cmpnumfld(revno,next->num,1) != 0) {
+		rcserror("revision number %s too low", partialno(&t,revno,2));
 		goto norev;
         }
         if ((length>2) && (result!=0)) {
@@ -424,29 +481,33 @@ struct hshentry * genrevs(revno,date,author,state,store)
         if (length>2)
                 return genbranch(next,revno,length,date,author,state,store);
         else { /* length == 2*/
-                if ((date!=nil) && (cmpnum(date,next->date)<0)){
-			error("Revision %s has date %s.",
+		if (date && cmpdate(date,next->date)<0) {
+			rcserror("Revision %s has date %s.",
 				next->num,
 				date2str(next->date, datebuf)
 			);
-                        return nil;
+			return 0;
+		}
+		if (author && strcmp(author,next->author)!=0) {
+			rcserror("Revision %s has author %s.",
+				next->num, next->author
+			);
+			return 0;
                 }
-                if ((author!=nil)&&(strcmp(author,next->author)!=0)) {
-                        error("Revision %s has author %s.",next->num,next->author);
-                        return nil;
+		if (state && strcmp(state,next->state)!=0) {
+			rcserror("Revision %s has state %s.",
+				next->num,
+				next->state ? next->state : "<empty>"
+			);
+			return 0;
                 }
-                if ((state!=nil)&&(strcmp(state,next->state)!=0)) {
-                        error("Revision %s has state %s.",next->num,
-                               next->state==nil?"<empty>":next->state);
-                        return nil;
-                }
-                *store=nil;
+		*store = 0;
                 return next;
         }
 
     norev:
 	bufautoend(&t);
-	return nil;
+	return 0;
 }
 
 
@@ -456,7 +517,7 @@ struct hshentry * genrevs(revno,date,author,state,store)
 genbranch(bpoint, revno, length, date, author, state, store)
 	struct hshentry const *bpoint;
 	char const *revno;
-	unsigned length;
+	int length;
 	char const *date, *author, *state;
 	struct hshentries **store;
 /* Function: given a branchpoint, a revision number, date, author, and state,
@@ -464,15 +525,15 @@ genbranch(bpoint, revno, length, date, author, state, store)
  * from the branch point on.
  * Pointers to the found deltas are stored in a list beginning with store.
  * revno must be on a side branch.
- * return nil on error
+ * Return 0 on error.
  */
 {
-	unsigned field;
+	int field;
         register struct hshentry * next, * trail;
 	register struct branchhead const *bhead;
         int result;
 	struct buf t;
-	char datebuf[datesize];
+	char datebuf[datesize + zonelenmax];
 
 	field = 3;
         bhead = bpoint->branches;
@@ -480,9 +541,11 @@ genbranch(bpoint, revno, length, date, author, state, store)
 	do {
 		if (!bhead) {
 			bufautobegin(&t);
-			error("no side branches present for %s", partialno(&t,revno,field-1));
+			rcserror("no side branches present for %s",
+				partialno(&t,revno,field-1)
+			);
 			bufautoend(&t);
-			return nil;
+			return 0;
 		}
 
                 /*find branch head*/
@@ -491,31 +554,33 @@ genbranch(bpoint, revno, length, date, author, state, store)
                         bhead = bhead->nextbranch;
 			if (!bhead) {
 			    bufautobegin(&t);
-			    error("branch number %s too high",partialno(&t,revno,field));
+			    rcserror("branch number %s too high",
+				partialno(&t,revno,field)
+			    );
 			    bufautoend(&t);
-			    return nil;
+			    return 0;
 			}
                 }
 
 		if (result<0) {
 		    absent(revno, field);
-		    return nil;
+		    return 0;
 		}
 
                 next = bhead->hsh;
                 if (length==field) {
                         /* pick latest one on that branch */
-                        trail=nil;
-                        do { if ((date==nil?1:(cmpnum(date,next->date)>=0)) &&
-                                 (author==nil?1:(strcmp(author,next->author)==0)) &&
-                                 (state ==nil?1:(strcmp(state, next->state) ==0))
+			trail = 0;
+			do { if ((!date || cmpdate(date,next->date)>=0) &&
+				 (!author || strcmp(author,next->author)==0) &&
+				 (!state || strcmp(state,next->state)==0)
                              ) trail = next;
                              next=next->next;
-                        } while (next!=nil);
+			} while (next);
 
-                        if (trail==nil) {
+			if (!trail) {
 			     cantfindbranch(revno, date, author, state);
-                             return nil;
+			     return 0;
                         } else { /* print up to last one suitable */
                              next = bhead->hsh;
                              while (next!=trail) {
@@ -524,7 +589,7 @@ genbranch(bpoint, revno, length, date, author, state, store)
                              }
 			     store1(&store, next);
                         }
-			*store = nil;
+			*store = 0;
                         return next;
                 }
 
@@ -533,44 +598,49 @@ genbranch(bpoint, revno, length, date, author, state, store)
                 /* check low */
                 if (cmpnumfld(revno,next->num,field+1)<0) {
 			bufautobegin(&t);
-			error("revision number %s too low", partialno(&t,revno,field+1));
+			rcserror("revision number %s too low",
+				partialno(&t,revno,field+1)
+			);
 			bufautoend(&t);
-                        return(nil);
+			return 0;
                 }
 		do {
 			store1(&store, next);
                         trail = next;
                         next = next->next;
-                } while ((next!=nil) &&
-                       (cmpnumfld(revno,next->num,field+1) >=0));
+		} while (next && cmpnumfld(revno,next->num,field+1)>=0);
 
                 if ((length>field+1) &&  /*need exact hit */
                     (cmpnumfld(revno,trail->num,field+1) !=0)){
 			absent(revno, field+1);
-                        return(nil);
+			return 0;
                 }
                 if (length == field+1) {
-                        if ((date!=nil) && (cmpnum(date,trail->date)<0)){
-				error("Revision %s has date %s.",
+			if (date && cmpdate(date,trail->date)<0) {
+				rcserror("Revision %s has date %s.",
 					trail->num,
 					date2str(trail->date, datebuf)
 				);
-                                return nil;
+				return 0;
                         }
-                        if ((author!=nil)&&(strcmp(author,trail->author)!=0)) {
-                                error("Revision %s has author %s.",trail->num,trail->author);
-                                return nil;
+			if (author && strcmp(author,trail->author)!=0) {
+				rcserror("Revision %s has author %s.",
+					trail->num, trail->author
+				);
+				return 0;
                         }
-                        if ((state!=nil)&&(strcmp(state,trail->state)!=0)) {
-                                error("Revision %s has state %s.",trail->num,
-                                       trail->state==nil?"<empty>":trail->state);
-                                return nil;
+			if (state && strcmp(state,trail->state)!=0) {
+				rcserror("Revision %s has state %s.",
+					trail->num,
+					trail->state ? trail->state : "<empty>"
+				);
+				return 0;
                         }
                 }
                 bhead = trail->branches;
 
 	} while ((field+=2) <= length);
-        * store = nil;
+	*store = 0;
         return trail;
 }
 
@@ -580,17 +650,14 @@ lookupsym(id)
 	char const *id;
 /* Function: looks up id in the list of symbolic names starting
  * with pointer SYMBOLS, and returns a pointer to the corresponding
- * revision number. Returns nil if not present.
+ * revision number.  Return 0 if not present.
  */
 {
 	register struct assoc const *next;
-        next = Symbols;
-        while (next!=nil) {
+	for (next = Symbols;  next;  next = next->nextassoc)
                 if (strcmp(id, next->symbol)==0)
 			return next->num;
-                else    next=next->nextassoc;
-        }
-        return nil;
+	return 0;
 }
 
 int expandsym(source, target)
@@ -617,13 +684,12 @@ fexpandsym(source, target, fp)
 	register char const *sp, *bp;
 	register char *tp;
 	char const *tlim;
-        register enum tokens d;
-	unsigned dots;
+	int dots;
 
 	sp = source;
 	bufalloc(target, 1);
 	tp = target->string;
-	if (!sp || !*sp) { /*accept nil pointer as a legal value*/
+	if (!sp || !*sp) { /* Accept 0 pointer as a legal value.  */
                 *tp='\0';
                 return true;
         }
@@ -631,7 +697,7 @@ fexpandsym(source, target, fp)
 		if (!getoldkeys(fp))
 			return false;
 		if (!*prevrev.string) {
-			error("working file lacks revision number");
+			workerror("working file lacks revision number");
 			return false;
 		}
 		bufscpy(target, prevrev.string);
@@ -641,72 +707,122 @@ fexpandsym(source, target, fp)
 	dots = 0;
 
 	for (;;) {
-		switch (ctab[(unsigned char)*sp]) {
-		    case DIGIT:
-			while (*sp=='0' && isdigit(sp[1]))
-                                /* skip leading zeroes */
-                                sp++;
-			do {
-				if (tlim <= tp)
-					tp = bufenlarge(target, &tlim);
-			} while (isdigit(*tp++ = *sp++));
-			--sp;
-			tp[-1] = '\0';
-			break;
+		register char *p = tp;
+		size_t s = tp - target->string;
+		int id = false;
+		for (;;) {
+		    switch (ctab[(unsigned char)*sp]) {
+			case IDCHAR:
+			case LETTER:
+			case Letter:
+			    id = true;
+			    /* fall into */
+			case DIGIT:
+			    if (tlim <= p)
+				    p = bufenlarge(target, &tlim);
+			    *p++ = *sp++;
+			    continue;
 
-		    case LETTER:
-		    case Letter:
-			{
-			register char *p = tp;
-			register size_t s = tp - target->string;
-			do {
-				if (tlim <= p)
-					p = bufenlarge(target, &tlim);
-				*p++ = *sp++;
-			} while ((d=ctab[(unsigned char)*sp])==LETTER ||
-			      d==Letter || d==DIGIT ||
-                              (d==IDCHAR));
-			if (tlim <= p)
-				p = bufenlarge(target, &tlim);
-			*p = 0;
-			tp = target->string + s;
-			}
+			default:
+			    break;
+		    }
+		    break;
+		}
+		if (tlim <= p)
+			p = bufenlarge(target, &tlim);
+		*p = 0;
+		tp = target->string + s;
+
+		if (id) {
 			bp = lookupsym(tp);
-                        if (bp==nil) {
-				error("Symbolic number %s is undefined.", tp);
+			if (!bp) {
+				rcserror("Symbolic name `%s' is undefined.",tp);
                                 return false;
                         }
-			do {
-				if (tlim <= tp)
-					tp = bufenlarge(target, &tlim);
-			} while ((*tp++ = *bp++));
-			break;
+		} else {
+			/* skip leading zeros */
+			for (bp = tp;  *bp=='0' && isdigit(bp[1]);  bp++)
+				continue;
+
+			if (!*bp)
+			    if (s || *sp!='.')
+				break;
+			    else {
+				/* Insert default branch before initial `.'.  */
+				char const *b;
+				if (Dbranch)
+				    b = Dbranch;
+				else if (Head)
+				    b = Head->num;
+				else
+				    break;
+				getbranchno(b, target);
+				bp = tp = target->string;
+				tlim = tp + target->size;
+			    }
+		}
+
+		while ((*tp++ = *bp++))
+			if (tlim <= tp)
+				tp = bufenlarge(target, &tlim);
 
-		    default:
-			goto improper;
-                }
 		switch (*sp++) {
-		    case '\0': return true;
-		    case '.': break;
-		    default: goto improper;
-		}
-		if (!*sp) {
-			if (dots & 1)
-				goto improper;
-			if (!(bp = branchtip(target->string)))
-				return false;
-			bufscpy(target, bp);
+		    case '\0':
 			return true;
+
+		    case '.':
+			if (!*sp) {
+				if (dots & 1)
+					break;
+				if (!(bp = branchtip(target->string)))
+					return false;
+				bufscpy(target, bp);
+				return true;
+			}
+			++dots;
+			tp[-1] = '.';
+			continue;
 		}
-		++dots;
-		tp[-1] = '.';
+		break;
         }
 
-    improper:
-	error("improper revision number: %s", source);
+	rcserror("improper revision number: %s", source);
 	return false;
 }
 
+	char const *
+namedrev(name, delta)
+	char const *name;
+	struct hshentry *delta;
+/* Yield NAME if it names DELTA, 0 otherwise.  */
+{
+	if (name) {
+		char const *id = 0, *p, *val;
+		for (p = name;  ;  p++)
+			switch (ctab[(unsigned char)*p]) {
+				case IDCHAR:
+				case LETTER:
+				case Letter:
+					id = name;
+					break;
+
+				case DIGIT:
+					break;
+
+				case UNKN:
+					if (!*p && id &&
+						(val = lookupsym(id)) &&
+						strcmp(val, delta->num) == 0
+					)
+						return id;
+					/* fall into */
+				default:
+					return 0;
+			}
+	}
+	return 0;
+}
+
 	static char const *
 branchtip(branch)
 	char const *branch;
@@ -728,6 +844,11 @@ tiprev()
 
 #ifdef REVTEST
 
+/*
+* Test the routines that generate a sequence of delta numbers
+* needed to regenerate a given delta.
+*/
+
 char const cmdid[] = "revtest";
 
 	int
@@ -761,20 +882,20 @@ int argc; char * argv[];
                 /* all output goes to stderr, to have diagnostics and       */
                 /* errors in sequence.                                      */
 		aputs("\nEnter revision number or <return> or '.': ",stderr);
-		if (!gets(symrevno)) break;
+		if (!fgets(symrevno, 100, stdin)) break;
                 if (*symrevno == '.') break;
 		aprintf(stderr,"%s;\n",symrevno);
 		expandsym(symrevno,&numricrevno);
 		aprintf(stderr,"expanded number: %s; ",numricrevno.string);
 		aprintf(stderr,"Date: ");
-		gets(date); aprintf(stderr,"%s; ",date);
+		fgets(date, 20, stdin); aprintf(stderr,"%s; ",date);
 		aprintf(stderr,"Author: ");
-		gets(author); aprintf(stderr,"%s; ",author);
+		fgets(author, 20, stdin); aprintf(stderr,"%s; ",author);
 		aprintf(stderr,"State: ");
-		gets(state); aprintf(stderr, "%s;\n", state);
-		target = genrevs(numricrevno.string, *date?date:(char *)nil, *author?author:(char *)nil,
-				 *state?state:(char*)nil, &gendeltas);
-                if (target!=nil) {
+		fgets(state, 20, stdin); aprintf(stderr, "%s;\n", state);
+		target = genrevs(numricrevno.string, *date?date:(char *)0, *author?author:(char *)0,
+				 *state?state:(char*)0, &gendeltas);
+		if (target) {
 			while (gendeltas) {
 				aprintf(stderr,"%s\n",gendeltas->first->num);
 				gendeltas = gendeltas->next;
@@ -785,6 +906,6 @@ int argc; char * argv[];
 	exitmain(EXIT_SUCCESS);
 }
 
-exiting void exiterr() { _exit(EXIT_FAILURE); }
+void exiterr() { _exit(EXIT_FAILURE); }
 
 #endif
