@@ -14,17 +14,17 @@
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 ** General Public License for more details.
-** 
+**
 ** You should have received a copy of the GNU General Public
 ** License along with the NYS YP Server; see the file COPYING.  If
 ** not, write to the Free Software Foundation, Inc., 675 Mass Ave,
 ** Cambridge, MA 02139, USA.
 **
 ** Author: Peter Eriksson <pen@signum.se>
-** Ported to FreeBSD and hacked all to pieces 
+** Ported to FreeBSD and hacked all to pieces
 ** by Bill Paul <wpaul@ctr.columbia.edu>
 **
-**	$Id: server.c,v 1.4 1995/04/01 19:31:12 wpaul Exp $
+** $FreeBSD: stable/2.1/gnu/usr.sbin/ypserv/server.c 50952 1999-09-05 11:54:50Z peter $
 **
 */
 
@@ -86,7 +86,8 @@ extern int errno;
 
 int debug_flag = 0;
 int dns_flag   = 0;
-
+int children   = 0;
+int forked     = 0;
 
 void verr(fmt, ap)
     const char *fmt;
@@ -151,7 +152,7 @@ static int is_valid_host(struct sockaddr_in *sin)
 		       inet_ntoa(sin->sin_addr),
 		       "");
 
-    if (sin->sin_addr.s_addr != oldaddr || status != oldstatus ) {
+    if (!status && (sin->sin_addr.s_addr != oldaddr || status != oldstatus)) {
 	syslog(status?allow_severity:deny_severity,
 	       "%sconnect from %s\n",status?"":"refused ",
 	       h?h:inet_ntoa(sin->sin_addr));
@@ -170,7 +171,7 @@ void *ypproc_null_2_svc(void *dummy,
 {
     static int foo;
     struct sockaddr_in *rqhost;
-    
+
 
     rqhost = svc_getcaller(rqstp->rq_xprt);
     if (!is_valid_host(rqhost))
@@ -184,14 +185,14 @@ void *ypproc_null_2_svc(void *dummy,
     return (void *) &foo;
 }
 
-    
+
 /*
 ** Return 1 if the name is a valid domain name served by us, else 0.
 */
 static int is_valid_domain(const char *domain)
 {
     struct stat sbuf;
-    
+
 
     if (domain == NULL ||
 	strcmp(domain, "binding") == 0 ||
@@ -199,7 +200,7 @@ static int is_valid_domain(const char *domain)
 	strcmp(domain, ".") == 0 ||
 	strchr(domain, '/'))
 	return 0;
-    
+
     if (stat(domain, &sbuf) < 0 || !S_ISDIR(sbuf.st_mode))
 	return 0;
 
@@ -213,10 +214,10 @@ bool_t *ypproc_domain_2_svc(domainname *name,
 {
     static bool_t result;
     struct sockaddr_in *rqhost;
-    
-    
+
+
     rqhost = svc_getcaller(rqstp->rq_xprt);
-	
+
     if (debug_flag)
 	Perror("ypproc_domain(\"%s\") [From: %s:%d]\n",
 		*name,
@@ -227,19 +228,19 @@ bool_t *ypproc_domain_2_svc(domainname *name,
     {
 	if (debug_flag)
 	    Perror("\t-> Ignored (not a valid source host)\n");
-	
+
 	return NULL;
     }
-    
+
     if (is_valid_domain(*name))
 	result = TRUE;
     else
 	result = FALSE;
-    
+
     if (debug_flag)
 	Perror("\t-> %s.\n",
 		(result == TRUE ? "Ok" : "Not served by us"));
-    
+
     return &result;
 }
 
@@ -250,7 +251,7 @@ bool_t *ypproc_domain_nonack_2_svc(domainname *name,
     static bool_t result;
     struct sockaddr_in *rqhost;
 
-    
+
     rqhost = svc_getcaller(rqstp->rq_xprt);
 
     if (debug_flag)
@@ -263,7 +264,7 @@ bool_t *ypproc_domain_nonack_2_svc(domainname *name,
     {
 	if (debug_flag)
 	    Perror("\t-> Ignored (not a valid source host)\n");
-	
+
 	return NULL;
     }
 
@@ -275,10 +276,10 @@ bool_t *ypproc_domain_nonack_2_svc(domainname *name,
 	/* Bail out and don't return any RPC value */
 	return NULL;
     }
-    
+
     if (debug_flag)
 	Perror("\t-> OK.\n");
-    
+
     result = TRUE;
     return &result;
 }
@@ -297,21 +298,20 @@ static DB *open_database(const char *domain,
     if (map[0] == '.' || strchr(map, '/'))
 	return 0;
 
-    strcpy(buf, domain);
-    strcat(buf, "/");
-    strcat(buf, map);
-    
+    sprintf (buf, "%s/%s", domain, map);
+
     dbp = dbopen(buf,O_RDONLY|O_EXCL, PERM_SECURE, DB_HASH, &openinfo);
 
     if (debug_flag > 1 && dbp == NULL)
-	Perror("dbopen(): ",strerror(errno));
-    
+	Perror("dbopen(%s): %s", map, strerror(errno));
+
     return dbp;
 }
 
 
 #define F_ALL   0x01
 #define F_NEXT  0x02
+#define F_YPALL 0x08
 
 /*
 ** Get a record from a DB database.
@@ -326,8 +326,8 @@ int read_database(DB *dbp,
 {
     int first_flag = 0;
     DBT nkey, ckey, dummyval;
-    
-    
+
+
     if (ikey == NULL || ikey->data == NULL)
     {
 	(dbp->seq)(dbp,&ckey,&dummyval,R_FIRST);
@@ -340,18 +340,23 @@ int read_database(DB *dbp,
 	/*
 	** This crap would be unnecessary if R_CURSOR actually worked.
 	*/
-	    (dbp->seq)(dbp,&ckey,&dummyval,R_FIRST);
-	    while(strncmp((char *)ikey->data,ckey.data,(int)ikey->size) ||
-			ikey->size != ckey.size)
-		(dbp->seq)(dbp,&ckey,&dummyval,R_NEXT);
+	    if (flags < F_YPALL)
+	    {
+		(dbp->seq)(dbp,&ckey,&dummyval,R_FIRST);
+		while(strncmp((char *)ikey->data,ckey.data,(int)ikey->size) ||
+		    ikey->size != ckey.size)
+		    (dbp->seq)(dbp,&ckey,&dummyval,R_NEXT);
+	    }
 	    if ((dbp->seq)(dbp,&ckey,&dummyval,R_NEXT))
 		ckey.data = NULL;
+#ifdef GNU_YPSERV_ARTIFACT
 		free(dummyval.data);
+#endif
 	}
 	else
 	    ckey = *ikey;
     }
-    
+
     if (ckey.data == NULL)
     {
 	return (flags & F_NEXT) ? YP_NOMORE : YP_NOKEY;
@@ -361,10 +366,11 @@ int read_database(DB *dbp,
     {
 	if ((dbp->get)(dbp,&ckey,dval,0))
 	{
+#ifdef GNU_YPSERV_ARTIFACT
 	    /* Free key, unless it comes from the caller! */
 	    if (ikey == NULL || ckey.data != ikey->data)
 		free(ckey.data);
-
+#endif
 	    if (ikey && ikey->data != NULL)
 	    {
 		return YP_NOKEY;
@@ -375,28 +381,31 @@ int read_database(DB *dbp,
 		else
 		    return YP_FALSE;
 	}
-	
+
 	if ((flags & F_ALL) || strncmp(ckey.data, "YP_", 3) != 0)
 	{
 	    if (okey)
 		*okey = ckey;
+#ifdef GNU_YPSERV_ARTIFACT
 	    else if (ikey == NULL || ikey->data != ckey.data)
 		free(ckey.data);
-	    
+#endif
 	    return YP_TRUE;
 	}
 
 	/* Free old value */
+#ifdef GNU_YPSERV_ARTIFACT
 	free(dval->data);
-
+#endif
 	if ((dbp->seq)(dbp,&nkey,&dummyval,R_NEXT))
 		nkey.data = NULL;
+#ifdef GNU_YPSERV_ARTIFACT
 	free(dummyval.data);
 
 	/* Free old key, unless it comes from the caller! */
 	if (ikey == NULL || ckey.data != ikey->data)
 	    free(ckey.data);
-
+#endif
 	if (ckey.data == NULL || nkey.data == NULL)
 	    return YP_NOMORE;
 
@@ -414,11 +423,11 @@ static unsigned long get_dtm(const char *domain,
     struct stat sbuf;
     char buf[1025];
 
-    
+
     strcpy(buf, domain);
     strcat(buf, "/");
     strcat(buf, map);
-    
+
     if (stat(buf, &sbuf) < 0)
 	return 0;
     else
@@ -435,7 +444,7 @@ ypresp_val *ypproc_match_2_svc(ypreq_key *key,
     static ypresp_val result;
     struct sockaddr_in *rqhost;
 
-    
+
     rqhost = svc_getcaller(rqstp->rq_xprt);
 
     if (debug_flag)
@@ -443,7 +452,7 @@ ypresp_val *ypproc_match_2_svc(ypreq_key *key,
 	Perror("ypproc_match(): [From: %s:%d]\n",
 		inet_ntoa(rqhost->sin_addr),
 		ntohs(rqhost->sin_port));
-		
+
 	Perror("\t\tdomainname = \"%s\"\n",
 		key->domain);
 	Perror("\t\tmapname = \"%s\"\n",
@@ -457,7 +466,7 @@ ypresp_val *ypproc_match_2_svc(ypreq_key *key,
     {
 	if (debug_flag)
 	    Perror("\t-> Ignored (not a valid source host)\n");
-	
+
 	return NULL;
     }
 
@@ -478,10 +487,20 @@ ypresp_val *ypproc_match_2_svc(ypreq_key *key,
     result.val.valdat_len = 0;
     if (result.val.valdat_val)
     {
+#ifdef GNU_YPSERV_ARTIFACT
+	/*
+	 * In general, if you malloc() data in an RPC service
+	 * routine, you have to free() it the next time that
+	 * routine is called since the XDR routines won't free
+	 * it for you. However, in this case, we don't have to
+	 * do that because the DB routines do garbage collection
+	 * for us.
+	 */
 	free(result.val.valdat_val);
+#endif
 	result.val.valdat_val = NULL;
     }
-    
+
     if (key->domain[0] == '\0' || key->map[0] == '\0')
 	result.stat = YP_BADARGS;
     else if (!is_valid_domain(key->domain))
@@ -489,7 +508,7 @@ ypresp_val *ypproc_match_2_svc(ypreq_key *key,
     else
     {
 	DBT rdat, qdat;
-	
+
 	DB *dbp = open_database(key->domain, key->map);
 	if (dbp == NULL)
 	    result.stat = YP_NOMAP;
@@ -509,7 +528,7 @@ ypresp_val *ypproc_match_2_svc(ypreq_key *key,
 	    (void)(dbp->close)(dbp);
 	}
     }
-	
+
 	if (debug_flag)
 	{
 	if (result.stat == YP_TRUE)
@@ -525,26 +544,36 @@ ypresp_val *ypproc_match_2_svc(ypreq_key *key,
     ** Do the jive thing if we didn't find the host in the YP map
     ** and we have enabled the magic DNS lookup stuff.
     **
-    ** XXX Perhaps this should be done in a sub-process for performance
-    **     reasons. Later.
+    ** DNS lookups are handled in a subprocess so that the server
+    ** doesn't block while waiting for requests to complete.
     */
     if (result.stat != YP_TRUE && strstr(key->map, "hosts") && dns_flag)
     {
 	char *cp = NULL;
+	char			nbuf[YPMAXRECORD];
 
-	key->key.keydat_val[key->key.keydat_len] = '\0';
+	if (children < MAX_CHILDREN && fork())
+	{
+	    children++;
+	    return NULL;
+	}
+	else
+	    forked++;
+
+	bcopy(key->key.keydat_val, nbuf, key->key.keydat_len);
+	nbuf[key->key.keydat_len] = '\0';
 
 	if (debug_flag)
-	    Perror("Doing DNS lookup of %s\n", key->key.keydat_val);
-	
+	    Perror("Doing DNS lookup of %s\n", nbuf);
+
 	if (strcmp(key->map, "hosts.byname") == 0)
-	    cp = dnsname(key->key.keydat_val);
+	    cp = dnsname(nbuf);
 	else  if (strcmp(key->map, "hosts.byaddr") == 0)
-	    cp = dnsaddr(key->key.keydat_val);
-	    
+	    cp = dnsaddr(nbuf);
+
 	if (cp)
 	{
-	    
+
 	    if (debug_flag)
 		Perror("\t-> OK (%s)\n", cp);
 
@@ -563,7 +592,7 @@ ypresp_val *ypproc_match_2_svc(ypreq_key *key,
 	    result.stat = YP_NOKEY;
 	}
     }
-    
+
     return &result;
 }
 
@@ -574,7 +603,7 @@ ypresp_key_val *ypproc_first_2_svc(ypreq_nokey *key,
 {
     static ypresp_key_val result;
     struct sockaddr_in *rqhost;
-    
+
 
     rqhost = svc_getcaller(rqstp->rq_xprt);
 
@@ -583,7 +612,7 @@ ypresp_key_val *ypproc_first_2_svc(ypreq_nokey *key,
 	Perror("ypproc_first(): [From: %s:%d]\n",
 		inet_ntoa(rqhost->sin_addr),
 		ntohs(rqhost->sin_port));
-		
+
 	Perror("\tdomainname = \"%s\"\n", key->domain);
 	Perror("\tmapname = \"%s\"\n", key->map);
 #if 0
@@ -597,7 +626,7 @@ ypresp_key_val *ypproc_first_2_svc(ypreq_nokey *key,
     {
 	if (debug_flag)
 	    Perror("\t-> Ignored (not a valid source host)\n");
-	
+
 	return NULL;
     }
 
@@ -618,14 +647,18 @@ ypresp_key_val *ypproc_first_2_svc(ypreq_nokey *key,
     result.key.keydat_len = 0;
     if (result.key.keydat_val)
     {
+#ifdef GNU_YPSERV_ARTIFACT
 	free(result.key.keydat_val);
+#endif
 	result.key.keydat_val = NULL;
     }
-    
+
     result.val.valdat_len = 0;
     if (result.val.valdat_val)
     {
+#ifdef GNU_YPSERV_ARTIFACT
 	free(result.val.valdat_val);
+#endif
 	result.val.valdat_val = NULL;
     }
 
@@ -636,7 +669,7 @@ ypresp_key_val *ypproc_first_2_svc(ypreq_nokey *key,
     else
     {
 	DBT dkey, dval;
-	
+
 	DB *dbp = open_database(key->domain, key->map);
 	if (dbp == NULL)
 	    result.stat = YP_NOMAP;
@@ -648,15 +681,15 @@ ypresp_key_val *ypproc_first_2_svc(ypreq_nokey *key,
 	    {
 		result.key.keydat_len = dkey.size;
 		result.key.keydat_val = dkey.data;
-		
+
 		result.val.valdat_len = dval.size;
 		result.val.valdat_val = dval.data;
 	    }
-	    
+
 	    (void)(dbp->close)(dbp);
 	}
     }
-    
+
     if (debug_flag)
     {
 	if (result.stat == YP_TRUE)
@@ -665,11 +698,11 @@ ypresp_key_val *ypproc_first_2_svc(ypreq_nokey *key,
 		    result.key.keydat_val,
 		    (int) result.val.valdat_len,
 		    result.val.valdat_val);
-		    
+
 	else
 	    Perror("\t-> Error #%d\n", result.stat);
     }
-    
+
     return &result;
 }
 
@@ -679,8 +712,8 @@ ypresp_key_val *ypproc_next_2_svc(ypreq_key *key,
 {
     static ypresp_key_val result;
     struct sockaddr_in *rqhost;
-    
-    
+
+
     rqhost = svc_getcaller(rqstp->rq_xprt);
 
     if (debug_flag)
@@ -688,7 +721,7 @@ ypresp_key_val *ypproc_next_2_svc(ypreq_key *key,
 	Perror("ypproc_next(): [From: %s:%d]\n",
 		inet_ntoa(rqhost->sin_addr),
 		ntohs(rqhost->sin_port));
-		
+
 	Perror("\tdomainname = \"%s\"\n", key->domain);
 	Perror("\tmapname = \"%s\"\n", key->map);
 	Perror("\tkeydat = \"%.*s\"\n",
@@ -700,7 +733,7 @@ ypresp_key_val *ypproc_next_2_svc(ypreq_key *key,
     {
 	if (debug_flag)
 	    Perror("\t-> Ignored (not a valid source host)\n");
-	
+
 	return NULL;
     }
 
@@ -721,17 +754,21 @@ ypresp_key_val *ypproc_next_2_svc(ypreq_key *key,
     result.key.keydat_len = 0;
     if (result.key.keydat_val)
     {
+#ifdef GNU_YPSERV_ARTIFACT
 	free(result.key.keydat_val);
+#endif
 	result.key.keydat_val = NULL;
     }
-    
+
     result.val.valdat_len = 0;
     if (result.val.valdat_val)
     {
+#ifdef GNU_YPSERV_ARTIFACT
 	free(result.val.valdat_val);
+#endif
 	result.val.valdat_val = NULL;
     }
-    
+
     if (key->map[0] == '\0' || key->domain[0] == '\0')
 	result.stat = YP_BADARGS;
     else if (!is_valid_domain(key->domain))
@@ -755,14 +792,14 @@ ypresp_key_val *ypproc_next_2_svc(ypreq_key *key,
 	    {
 		result.key.keydat_len = okey.size;
 		result.key.keydat_val = okey.data;
-		
+
 		result.val.valdat_len = dval.size;
 		result.val.valdat_val = dval.data;
 	    }
 	    (void)(dbp->close)(dbp);
 	}
     }
-    
+
     if (debug_flag)
     {
 	if (result.stat == YP_TRUE)
@@ -774,7 +811,7 @@ ypresp_key_val *ypproc_next_2_svc(ypreq_key *key,
 	else
 	    Perror("\t-> Error #%d\n", result.stat);
     }
-    
+
     return &result;
 }
 
@@ -790,14 +827,15 @@ static void print_ypmap_parms(const struct ypmap_parms *pp)
 
 
 /*
-** Clean up after ypxfr child processes signal their termination.
+** Clean up after child processes signal their termination.
 */
 void reapchild(sig)
 int sig;
 {
     int st;
 
-    wait3(&st, WNOHANG, NULL);
+    while (wait3(&st, WNOHANG, NULL) > 0)
+	children--;
 }
 
 /*
@@ -809,6 +847,8 @@ ypresp_xfr *ypproc_xfr_2_svc(ypreq_xfr *xfr,
     static ypresp_xfr result;
     struct sockaddr_in *rqhost;
     char ypxfr_command[MAXPATHLEN];
+    ypresp_master *mres;
+    ypreq_nokey mreq;
 
     rqhost = svc_getcaller(rqstp->rq_xprt);
 
@@ -817,7 +857,7 @@ ypresp_xfr *ypproc_xfr_2_svc(ypreq_xfr *xfr,
 	Perror("ypproc_xfr_2_svc(): [From: %s:%d]\n\tmap_parms:\n",
 		inet_ntoa(rqhost->sin_addr),
 		ntohs(rqhost->sin_port));
-	
+
 	print_ypmap_parms(&xfr->map_parms);
 	Perror("\t\ttransid = %u\n", xfr->transid);
 	Perror("\t\tprog = %u\n", xfr->prog);
@@ -828,7 +868,7 @@ ypresp_xfr *ypproc_xfr_2_svc(ypreq_xfr *xfr,
     {
 	if (debug_flag)
 	    Perror("\t-> Ignored (not a valid source host)\n");
-	
+
 	return NULL;
     }
 
@@ -846,6 +886,20 @@ ypresp_xfr *ypproc_xfr_2_svc(ypreq_xfr *xfr,
 	return &result;
     }
 
+    mreq.domain = xfr->map_parms.domain;
+    mreq.map = xfr->map_parms.map;
+
+    mres = ypproc_master_2_svc(&mreq, rqstp);
+
+    if (mres->stat != YP_TRUE) {
+	Perror("couldn't find master for map %s@%s", xfr->map_parms.map,
+						xfr->map_parms.domain);
+	Perror("host at %s (%s) may be pulling my leg", xfr->map_parms.peer,
+						inet_ntoa(rqhost->sin_addr));
+	result.xfrstat = YPXFR_REFUSED;
+	return &result;
+    }
+
     switch(fork())
     {
 	case 0:
@@ -857,7 +911,7 @@ ypresp_xfr *ypproc_xfr_2_svc(ypreq_xfr *xfr,
 	    sprintf (g, "%u", xfr->prog);
 	    sprintf (p, "%u", xfr->port);
 	    execl(ypxfr_command, "ypxfr", "-d", xfr->map_parms.domain, "-h",
-		xfr->map_parms.peer, "-f", "-C", t, g, 
+		mres->peer, "-f", "-C", t, g,
 		inet_ntoa(rqhost->sin_addr), p, xfr->map_parms.map, NULL);
 	    Perror("ypxfr execl(): %s",strerror(errno));
 	    exit(0);
@@ -867,7 +921,6 @@ ypresp_xfr *ypproc_xfr_2_svc(ypreq_xfr *xfr,
 	    result.xfrstat = YPXFR_XFRERR;
 	default:
 	{
-	    signal(SIGCHLD, reapchild);
 	    result.xfrstat = YPXFR_SUCC;
 	    break;
 	}
@@ -883,7 +936,7 @@ void *ypproc_clear_2_svc(void *dummy,
 {
     static int foo;
     struct sockaddr_in *rqhost;
-    
+
 
     rqhost = svc_getcaller(rqstp->rq_xprt);
 
@@ -891,12 +944,12 @@ void *ypproc_clear_2_svc(void *dummy,
     	Perror("ypproc_clear_2_svc() [From: %s:%d]\n",
 		inet_ntoa(rqhost->sin_addr),
 		ntohs(rqhost->sin_port));
-		
+
     if (!is_valid_host(rqhost))
     {
 	if (debug_flag)
 	    Perror("\t-> Ignored (not a valid source host)\n");
-	
+
 	return NULL;
     }
 
@@ -927,8 +980,8 @@ static int ypall_encode(ypresp_key_val *val,
 
     dkey.data = val->key.keydat_val;
     dkey.size = val->key.keydat_len;
-    
-    val->stat = read_database((DB *) data, &dkey, &okey, &dval, F_NEXT);
+
+    val->stat = read_database((DB *) data, &dkey, &okey, &dval, F_NEXT | F_YPALL);
 
     if (val->stat == YP_TRUE)
     {
@@ -939,7 +992,7 @@ static int ypall_encode(ypresp_key_val *val,
 	val->val.valdat_len = dval.size;
     }
 
-    
+
     return val->stat;
 }
 
@@ -951,7 +1004,7 @@ ypresp_all *ypproc_all_2_svc(ypreq_nokey *nokey,
     extern __xdr_ypall_cb_t __xdr_ypall_cb;
     struct sockaddr_in *rqhost;
 
-    
+
     rqhost = svc_getcaller(rqstp->rq_xprt);
 
     if (debug_flag)
@@ -959,7 +1012,7 @@ ypresp_all *ypproc_all_2_svc(ypreq_nokey *nokey,
 	Perror("ypproc_all_2_svc(): [From: %s:%d]\n",
 		inet_ntoa(rqhost->sin_addr),
 		ntohs(rqhost->sin_port));
-		
+
 	Perror("\t\tdomain = \"%s\"\n", nokey->domain);
 	Perror("\t\tmap = \"%s\"\n", nokey->map);
     }
@@ -968,14 +1021,22 @@ ypresp_all *ypproc_all_2_svc(ypreq_nokey *nokey,
     {
 	if (debug_flag)
 	    Perror("\t-> Ignored (not a valid source host)\n");
-	
+
 	return NULL;
     }
 
+    if (children < MAX_CHILDREN && fork())
+    {
+	children++;
+	return NULL;
+    }
+    else
+	forked++;
+
     __xdr_ypall_cb.u.encode = NULL;
     __xdr_ypall_cb.u.close  = NULL;
     __xdr_ypall_cb.data = NULL;
-   
+
     result.more = TRUE;
 
     /*
@@ -991,7 +1052,7 @@ ypresp_all *ypproc_all_2_svc(ypreq_nokey *nokey,
 	result.ypresp_all_u.val.stat = YP_YPERR;
 	return &result;
     }
-    
+
     if (nokey->map[0] == '\0' || nokey->domain[0] == '\0')
 	result.ypresp_all_u.val.stat = YP_BADARGS;
     else if (!is_valid_domain(nokey->domain))
@@ -999,7 +1060,7 @@ ypresp_all *ypproc_all_2_svc(ypreq_nokey *nokey,
     else
     {
 	DBT dkey, dval;
-	
+
 	DB *dbp = open_database(nokey->domain, nokey->map);
 	if (dbp == NULL)
 	    result.ypresp_all_u.val.stat = YP_NOMAP;
@@ -1015,7 +1076,7 @@ ypresp_all *ypproc_all_2_svc(ypreq_nokey *nokey,
 	    {
 		result.ypresp_all_u.val.key.keydat_len = dkey.size;
 		result.ypresp_all_u.val.key.keydat_val = dkey.data;
-		
+
 		result.ypresp_all_u.val.val.valdat_len = dval.size;
 		result.ypresp_all_u.val.val.valdat_val = dval.data;
 
@@ -1029,7 +1090,7 @@ ypresp_all *ypproc_all_2_svc(ypreq_nokey *nokey,
 	    (void)(dbp->close)(dbp);
 	}
     }
-    
+
     return &result;
 }
 
@@ -1039,8 +1100,8 @@ ypresp_master *ypproc_master_2_svc(ypreq_nokey *nokey,
 {
     static ypresp_master result;
     struct sockaddr_in *rqhost;
-    
-    
+
+
     rqhost = svc_getcaller(rqstp->rq_xprt);
 
     if (debug_flag)
@@ -1048,7 +1109,7 @@ ypresp_master *ypproc_master_2_svc(ypreq_nokey *nokey,
 	Perror("ypproc_master_2_svc(): [From: %s:%d]\n",
 		inet_ntoa(rqhost->sin_addr),
 		ntohs(rqhost->sin_port));
-		
+
 	Perror("\t\tdomain = \"%s\"\n", nokey->domain);
 	Perror("\t\tmap = \"%s\"\n", nokey->map);
     }
@@ -1057,7 +1118,7 @@ ypresp_master *ypproc_master_2_svc(ypreq_nokey *nokey,
     {
 	if (debug_flag)
 	    Perror("\t-> Ignored (not a valid source host)\n");
-	
+
 	return NULL;
     }
 
@@ -1077,7 +1138,9 @@ ypresp_master *ypproc_master_2_svc(ypreq_nokey *nokey,
 
     if (result.peer)
     {
+#ifdef GNU_YPSERV_ARTIFACT
 	free(result.peer);
+#endif
 	result.peer = NULL;
     }
 
@@ -1096,11 +1159,8 @@ ypresp_master *ypproc_master_2_svc(ypreq_nokey *nokey,
 
 	    key.size = sizeof("YP_MASTER_NAME")-1;
 	    key.data = "YP_MASTER_NAME";
-	    
-	    if ((dbp->get)(dbp,&key,&val,0))
-		val.data = NULL;
 
-	    if (val.data == NULL)
+	    if ((dbp->get)(dbp,&key,&val,0))
 	    {
 		/* No YP_MASTER_NAME record in map? Assume we are Master */
 		static char hostbuf[1025];
@@ -1125,7 +1185,7 @@ ypresp_master *ypproc_master_2_svc(ypreq_nokey *nokey,
 
     if (debug_flag)
 	Perror("\t-> Peer = \"%s\"\n", result.peer);
-    
+
     return &result;
 }
 
@@ -1135,8 +1195,8 @@ ypresp_order *ypproc_order_2_svc(ypreq_nokey *nokey,
 {
     static ypresp_order result;
     struct sockaddr_in *rqhost;
-    
-    
+
+
     rqhost = svc_getcaller(rqstp->rq_xprt);
 
     if (debug_flag)
@@ -1144,7 +1204,7 @@ ypresp_order *ypproc_order_2_svc(ypreq_nokey *nokey,
 	Perror("ypproc_order_2_svc(): [From: %s:%d]\n",
 		inet_ntoa(rqhost->sin_addr),
 		ntohs(rqhost->sin_port));
-		
+
 	Perror("\t\tdomain = \"%s\"\n", nokey->domain);
 	Perror("\t\tmap = \"%s\"\n", nokey->map);
     }
@@ -1153,7 +1213,7 @@ ypresp_order *ypproc_order_2_svc(ypreq_nokey *nokey,
     {
 	if (debug_flag)
 	    Perror("\t-> Ignored (not a valid source host)\n");
-	
+
 	return NULL;
     }
 
@@ -1188,9 +1248,8 @@ ypresp_order *ypproc_order_2_svc(ypreq_nokey *nokey,
 
 	    key.size = sizeof("YP_LAST_MODIFIED")-1;
 	    key.data = "YP_LAST_MODIFIED";
-	    
-	    (dbp->get)(dbp,&key,&val,0);
-	    if (val.data == NULL)
+
+	    if ((dbp->get)(dbp,&key,&val,0))
 	    {
 		/* No YP_LAST_MODIFIED record in map? Use DTM timestamp.. */
 		result.ordernum = get_dtm(nokey->domain, nokey->map);
@@ -1198,7 +1257,9 @@ ypresp_order *ypproc_order_2_svc(ypreq_nokey *nokey,
 	    else
 	    {
 		result.ordernum = atoi(val.data);
+#ifdef GNU_YPSERV_ARTIFACT
 		free(val.data);
+#endif
 	    }
 
 	    result.stat = YP_TRUE;
@@ -1208,7 +1269,7 @@ ypresp_order *ypproc_order_2_svc(ypreq_nokey *nokey,
 
     if (debug_flag)
 	Perror("-> Order # %d\n", result.ordernum);
-    
+
     return &result;
 }
 
@@ -1257,7 +1318,7 @@ ypresp_maplist *ypproc_maplist_2_svc(domainname *name,
 {
     static ypresp_maplist result;
     struct sockaddr_in *rqhost;
-    
+
 
     rqhost = svc_getcaller(rqstp->rq_xprt);
 
@@ -1266,7 +1327,7 @@ ypresp_maplist *ypproc_maplist_2_svc(domainname *name,
 	Perror("ypproc_maplist_2_svc(): [From: %s:%d]\n",
 		inet_ntoa(rqhost->sin_addr),
 		ntohs(rqhost->sin_port));
-		
+
 	Perror("\t\tdomain = \"%s\"\n", *name);
     }
 
@@ -1274,13 +1335,13 @@ ypresp_maplist *ypproc_maplist_2_svc(domainname *name,
     {
 	if (debug_flag)
 	    Perror("\t-> Ignored (not a valid source host)\n");
-	
+
 	return NULL;
     }
 
     if (result.maps)
 	free_maplist(result.maps);
-    
+
     result.maps = NULL;
 
     if ((*name)[0] == '\0')
@@ -1300,13 +1361,13 @@ ypresp_maplist *ypproc_maplist_2_svc(domainname *name,
 	    {
 		Perror("%s: opendir: %s", progname,strerror(errno));
 	    }
-	    
+
 	    result.stat = YP_BADDB;
 	}
 	else
 	{
 	    struct dirent *dep;
-	    
+
 	    while ((dep = readdir(dp)) != NULL)
 		if (add_maplist(&result.maps, dep->d_name) < 0)
 		{
@@ -1317,7 +1378,7 @@ ypresp_maplist *ypproc_maplist_2_svc(domainname *name,
 	    result.stat = YP_TRUE;
 	}
     }
-    
+
     if (debug_flag)
     {
 	if (result.stat == YP_TRUE)
@@ -1336,6 +1397,6 @@ ypresp_maplist *ypproc_maplist_2_svc(domainname *name,
 	else
 	    Perror("\t-> Error #%d\n", result.stat);
     }
-    
+
     return &result;
 }
