@@ -28,7 +28,7 @@
  */
 
 #ifndef LINT
-static char *rcsid = "$Id: yplib.c,v 1.8 1995/04/26 19:03:05 wpaul Exp $";
+static char *rcsid = "$FreeBSD: stable/2.1/lib/libc/yp/yplib.c 50952 1999-09-05 11:54:50Z peter $";
 #endif
 
 #include <sys/param.h>
@@ -204,6 +204,10 @@ struct dom_binding **ypdb;
 	int new = 0, r;
 	int retries = 0;
 
+	/* Not allowed; bad doggie. Bad. */
+	if (strchr(dom, '/') != NULL)
+		return(YPERR_BADARGS);
+
 	gpid = getpid();
 	if( !(pid==-1 || pid==gpid) ) {
 		ysd = _ypbindlist;
@@ -291,12 +295,21 @@ skipit:
 		bzero((char *)&clnt_sin, sizeof clnt_sin);
 		clnt_sin.sin_family = AF_INET;
 		clnt_sin.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
-		
+
 		clnt_sock = RPC_ANYSOCK;
 		client = clnttcp_create(&clnt_sin, YPBINDPROG, YPBINDVERS, &clnt_sock,
 			0, 0);
 		if(client==NULL) {
-			clnt_pcreateerror("clnttcp_create");
+			/*
+			 * These conditions indicate ypbind just isn't
+			 * alive -- we probably don't want to shoot our
+			 * mouth off in this case; instead generate error
+			 * messages only for really exotic problems.
+			 */
+			if (rpc_createerr.cf_stat != RPC_PROGNOTREGISTERED &&
+			   (rpc_createerr.cf_stat != RPC_SYSTEMERROR &&
+			   rpc_createerr.cf_error.re_errno == ECONNREFUSED))
+				clnt_pcreateerror("clnttcp_create");
 			if(new)
 				free(ysd);
 			return (YPERR_YPBIND);
@@ -364,7 +377,8 @@ static void
 _yp_unbind(ypb)
 struct dom_binding *ypb;
 {
-	clnt_destroy(ypb->dom_client);
+	if (ypb->dom_client != NULL)
+		clnt_destroy(ypb->dom_client);
 	ypb->dom_client = NULL;
 	ypb->dom_socket = -1;
 }
@@ -423,10 +437,6 @@ int *outvallen;
 	    indomain == NULL || !strlen(indomain))
 		return YPERR_BADARGS;
 
-again:
-	if( _yp_dobind(indomain, &ysd) != 0)
-		return YPERR_DOMAIN;
-
 #ifdef YPMATCHCACHE
 	if( !strcmp(_yp_domain, indomain) && ypmatch_find(inmap, inkey,
 	    inkeylen, &yprv.valdat.dptr, &yprv.valdat.dsize)) {
@@ -438,6 +448,10 @@ again:
 	}
 #endif
 
+again:
+	if( _yp_dobind(indomain, &ysd) != 0)
+		return YPERR_DOMAIN;
+
 	tv.tv_sec = _yplib_timeout;
 	tv.tv_usec = 0;
 
@@ -683,6 +697,17 @@ again:
 
 	r = clnt_call(ysd->dom_client, YPPROC_ORDER,
 		xdr_ypreq_nokey, &yprnk, xdr_ypresp_order, &ypro, tv);
+
+	/*
+	 * NIS+ in YP compat mode doesn't support the YPPROC_ORDER
+	 * procedure.
+	 */
+
+	if (r == RPC_PROCUNAVAIL) {
+		return(YPERR_YPERR);
+		goto bail;
+	}
+
 	if(r != RPC_SUCCESS) {
 		clnt_perror(ysd->dom_client, "yp_order: clnt_call");
 		ysd->dom_vers = -1;
@@ -691,6 +716,8 @@ again:
 
 	*outorder = ypro.ordernum;
 	xdr_free(xdr_ypresp_order, (char *)&ypro);
+
+bail:
 	_yp_unbind(ysd);
 	return ypprot_err(ypro.status);
 }
@@ -835,7 +862,7 @@ unsigned int incode;
 	case YP_NOMAP:
 		return YPERR_MAP;
 	case YP_NODOM:
-		return YPERR_NODOM;
+		return YPERR_DOMAIN;
 	case YP_NOKEY:
 		return YPERR_KEY;
 	case YP_BADOP:
