@@ -1,5 +1,5 @@
 /* MD2.H - header file for MD2C.C
- * $FreeBSD: stable/2.0.5/lib/libmd/md2.h 4245 1994-11-07 20:48:35Z phk $
+ * $FreeBSD: stable/2.1/lib/libmd/md2.h 12981 1995-12-22 22:09:31Z peter $
  */
 
 /* Copyright (C) 1990-2, RSA Data Security, Inc. Created 1990. All
@@ -23,7 +23,7 @@
 #ifndef _MD2_H_
 #define _MD2_H_
 
-typedef struct {
+typedef struct MD2Context {
   unsigned char state[16];	/* state */
   unsigned char checksum[16];	/* checksum */
   unsigned int count;		/* number of bytes, modulo 16 */
@@ -33,8 +33,8 @@ typedef struct {
 void   MD2Init(MD2_CTX *);
 void   MD2Update(MD2_CTX *, const unsigned char *, unsigned int);
 void   MD2Final(unsigned char [16], MD2_CTX *);
-char * MD2End(MD2_CTX *);
-char * MD2File(char *);
-char * MD2Data(const unsigned char *, unsigned int);
+char * MD2End(MD2_CTX *, char *);
+char * MD2File(char *, char *);
+char * MD2Data(const unsigned char *, unsigned int, char *);
 
 #endif /* _MD2_H_ */
