@@ -8,16 +8,16 @@
  *
  * Developed at SunPro, a Sun Microsystems, Inc. business.
  * Permission to use, copy, modify, and distribute this
- * software is freely granted, provided that this notice 
+ * software is freely granted, provided that this notice
  * is preserved.
  * ====================================================
  */
 
 #ifndef lint
-static char rcsid[] = "$Id: w_atan2f.c,v 1.1 1994/08/10 20:33:41 jtc Exp $";
+static char rcsid[] = "$FreeBSD: stable/2.1/lib/msun/src/w_atan2f.c 50952 1999-09-05 11:54:50Z peter $";
 #endif
 
-/* 
+/*
  * wrapper atan2f(y,x)
  */
 
