@@ -5,16 +5,16 @@
  *
  * Developed at SunPro, a Sun Microsystems, Inc. business.
  * Permission to use, copy, modify, and distribute this
- * software is freely granted, provided that this notice 
+ * software is freely granted, provided that this notice
  * is preserved.
  * ====================================================
  */
 
 #ifndef lint
-static char rcsid[] = "$Id: w_sqrt.c,v 1.4 1994/08/10 20:35:48 jtc Exp $";
+static char rcsid[] = "$FreeBSD: stable/2.1/lib/msun/src/w_sqrt.c 50952 1999-09-05 11:54:50Z peter $";
 #endif
 
-/* 
+/*
  * wrapper sqrt(x)
  */
 
