@@ -30,7 +30,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *	$Id$
+ * $FreeBSD: stable/2.1/libexec/ftpd/ftpd.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #ifndef lint
@@ -113,6 +113,8 @@ int	debug;
 int	timeout = 900;    /* timeout after 15 minutes of inactivity */
 int	maxtimeout = 7200;/* don't allow idle time to be set beyond 2 hours */
 int	logging;
+int	restricted_data_ports = 1;
+int	paranoid = 1;
 int	guest;
 #ifdef STATS
 int	stats;
@@ -258,9 +260,9 @@ main(argc, argv, envp)
 
 
 #ifdef STATS
-	while ((ch = getopt(argc, argv, "dlSt:T:u:v")) != EOF) {
+	while ((ch = getopt(argc, argv, "RdlSUt:T:u:v")) != EOF) {
 #else
-	while ((ch = getopt(argc, argv, "dlt:T:u:v")) != EOF) {
+	while ((ch = getopt(argc, argv, "RdlUt:T:u:v")) != EOF) {
 #endif
 		switch (ch) {
 		case 'd':
@@ -271,6 +273,14 @@ main(argc, argv, envp)
 			logging++;	/* > 1 == extra logging */
 			break;
 
+		case 'R':
+			paranoid = 0;
+			break;
+
+		case 'U':
+			restricted_data_ports = 0;
+			break;
+
 		case 't':
 			timeout = atoi(optarg);
 			if (maxtimeout < timeout)
@@ -574,6 +584,7 @@ pass(passwd)
 #endif
 		/* The strcmp does not catch null passwords! */
 		if (pw == NULL || *pw->pw_passwd == '\0' ||
+		    (pw->pw_expire && time(NULL) >= pw->pw_expire) ||
 		    strcmp(xpasswd, pw->pw_passwd)) {
 			reply(530, "Login incorrect.");
 			if (logging)
@@ -1019,7 +1030,7 @@ send_data(instr, outstr, blksize, filesize, isreg)
 		 * are sending a regular file
 		 */
 		netfd = fileno(outstr);
-		filefd = fileno(instr);	
+		filefd = fileno(instr);
 
 		if (isreg && filesize < (off_t)16 * 1024 * 1024) {
 			buf = mmap(0, filesize, PROT_READ, MAP_SHARED, filefd,
@@ -1052,7 +1063,7 @@ oldway:
 			perror_reply(451, "Local resource failure: malloc");
 			return;
 		}
-				
+
 		while ((cnt = read(filefd, buf, (u_int)blksize)) > 0 &&
 		    write(netfd, buf, cnt) == cnt)
 			byte_count += cnt;
@@ -1518,21 +1529,49 @@ void
 passive()
 {
 	int len;
+	u_short port;
 	char *p, *a;
 
+	if (pw == NULL) {
+		reply(530, "Please login with USER and PASS");
+		return;
+	}
 	pdata = socket(AF_INET, SOCK_STREAM, 0);
 	if (pdata < 0) {
 		perror_reply(425, "Can't open passive connection");
 		return;
 	}
-	pasv_addr = ctrl_addr;
-	pasv_addr.sin_port = 0;
-	(void) seteuid((uid_t)0);
-	if (bind(pdata, (struct sockaddr *)&pasv_addr, sizeof(pasv_addr)) < 0) {
+
+	if (restricted_data_ports) {
+		for (port = FTP_DATA_BOTTOM; port <= FTP_DATA_TOP; port++) {
+			pasv_addr = ctrl_addr;
+			pasv_addr.sin_port = htons(port);
+			(void) seteuid((uid_t)0);
+			if (bind(pdata, (struct sockaddr *)&pasv_addr,
+				 sizeof(pasv_addr)) < 0) {
+				(void) seteuid((uid_t)pw->pw_uid);
+				if (errno == EADDRINUSE)
+					continue;
+				else
+					goto pasv_error;
+			}
+			(void) seteuid((uid_t)pw->pw_uid);
+			break;
+		}
+		if (port > FTP_DATA_TOP)
+			goto pasv_error;
+	} else {
+		pasv_addr = ctrl_addr;
+		pasv_addr.sin_port = 0;
+		(void) seteuid((uid_t)0);
+		if (bind(pdata, (struct sockaddr *)&pasv_addr,
+			 sizeof(pasv_addr)) < 0) {
+			(void) seteuid((uid_t)pw->pw_uid);
+			goto pasv_error;
+		}
 		(void) seteuid((uid_t)pw->pw_uid);
-		goto pasv_error;
 	}
-	(void) seteuid((uid_t)pw->pw_uid);
+
 	len = sizeof(pasv_addr);
 	if (getsockname(pdata, (struct sockaddr *) &pasv_addr, &len) < 0)
 		goto pasv_error;
