@@ -143,25 +143,6 @@ localstat()
 	newmap = tty_isnewmap();
 #endif	/* defined(CRAY2) && defined(UNICOS5) */
 
-	/*
-	 * Check for state of BINARY options.
-	 */
-	if (tty_isbinaryin()) {
-		if (his_want_state_is_wont(TELOPT_BINARY))
-			send_do(TELOPT_BINARY, 1);
-	} else {
-		if (his_want_state_is_will(TELOPT_BINARY))
-			send_dont(TELOPT_BINARY, 1);
-	}
-
-	if (tty_isbinaryout()) {
-		if (my_want_state_is_wont(TELOPT_BINARY))
-			send_will(TELOPT_BINARY, 1);
-	} else {
-		if (my_want_state_is_will(TELOPT_BINARY))
-			send_wont(TELOPT_BINARY, 1);
-	}
-
 	/*
 	 * Check for changes to flow control if client supports it.
 	 */
@@ -181,6 +162,34 @@ localstat()
 		tty_setlinemode(uselinemode);
 	}
 
+        if (uselinemode) { 
+
+            /*
+             * Check for state of BINARY options.
+             *
+             * We only need to do the binary dance if we are actually going
+             * to use linemode.  As this confuses some telnet clients that dont
+             * support linemode, and doesnt gain us anything, we dont do it 
+             * unless we're doing linemode.  -Crh (henrich@msu.edu)
+             */
+
+	    if (tty_isbinaryin()) {
+		    if (his_want_state_is_wont(TELOPT_BINARY))
+			    send_do(TELOPT_BINARY, 1);
+	    } else {
+		    if (his_want_state_is_will(TELOPT_BINARY))
+			    send_dont(TELOPT_BINARY, 1);
+	    }
+    
+	    if (tty_isbinaryout()) {
+		    if (my_want_state_is_wont(TELOPT_BINARY))
+			    send_will(TELOPT_BINARY, 1);
+	    } else {
+		    if (my_want_state_is_will(TELOPT_BINARY))
+			    send_wont(TELOPT_BINARY, 1);
+	    }
+
+        }
 
 	/*
 	 * Do echo mode handling as soon as we know what the
@@ -301,7 +310,7 @@ localstat()
 			nfrontp += 7;
 			editmode = useeditmode;
 		}
-							
+
 
 		/*
 		 * Check for changes to special characters in use.
@@ -403,7 +412,7 @@ clientstat(code, parm1, parm2)
 					uselinemode = 1;
 				}
 			}
-		
+
 			/*
 			 * Quit now if we can't do it.
 			 */
@@ -444,7 +453,7 @@ clientstat(code, parm1, parm2)
 				send_will(TELOPT_ECHO, 1);
 		}
 		break;
-	
+
 	case LM_MODE:
 	    {
 		register int ack, changed;
@@ -493,7 +502,7 @@ clientstat(code, parm1, parm2)
  					IAC, SE);
  				nfrontp += 7;
  			}
- 		
+
 			editmode = useeditmode;
 		}
 
@@ -527,9 +536,9 @@ clientstat(code, parm1, parm2)
 		(void) ioctl(pty, TIOCSWINSZ, (char *)&ws);
 	    }
 #endif	/* TIOCSWINSZ */
-		
+
 		break;
-	
+
 	case TELOPT_TSPEED:
 	    {
 		def_tspeed = parm1;
@@ -594,7 +603,7 @@ _termstat()
  *
  * Some things should not be done until after the login process has started
  * and all the pty modes are set to what they are supposed to be.  This
- * function is called when the pty state has been processed for the first time. 
+ * function is called when the pty state has been processed for the first time.
  * It calls other functions that do things that were deferred in each module.
  */
 	void
