@@ -31,6 +31,11 @@
  * SUCH DAMAGE.
  */
 
+/*
+ *  951109 - Andrew@pubnix.net - Changed to iterative buffer growing mechanism
+ *				 for ifconfig -a so all interfaces are queried.
+ *
+ */
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright (c) 1983, 1993\n\
@@ -68,8 +73,8 @@ static char sccsid[] = "@(#)ifconfig.c	8.2 (Berkeley) 2/16/94";
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#include <nlist.h> 
-#include <kvm.h>   
+#include <nlist.h>
+#include <kvm.h>
 #include <fcntl.h>
 
 struct	ifreq		ifr, ridreq;
@@ -208,22 +213,33 @@ main(argc, argv)
 		exit(1);
 	}
 	if (strstr(name, "-a")) {
+		char *buffer;
 		struct ifconf ifc;
-#define MAX_INTERFACES 50	/* Yeah right. */
-		char buffer[MAX_INTERFACES * sizeof(struct ifreq)];
 		struct ifreq *ifptr, *end;
 		int ifflags, selectflag = -1;
+		int oldbufsize, bufsize = sizeof(struct ifreq);	
 
 		if (strstr(name, "-au"))
 			selectflag = 1;
 		if (strstr(name, "-ad"))
 			selectflag = 0;
-		ifc.ifc_len = sizeof(buffer);
-		ifc.ifc_buf = buffer;
-		if (ioctl(s, SIOCGIFCONF, (char *) &ifc) < 0) {
-			perror("ifconfig (SIOCGIFCONF)");
-			exit (1);
-		}
+		buffer = malloc(bufsize);	/* allocate first buffer */
+		ifc.ifc_len = bufsize;		/* Initial setting */
+		/*
+		 * Itterate through here until we don't get any more data
+		 */
+		do {
+			oldbufsize = ifc.ifc_len;
+			bufsize += 1+sizeof(struct ifreq);
+			buffer = realloc((void *)buffer, bufsize);
+			ifc.ifc_len = bufsize;
+			ifc.ifc_buf = buffer;
+			if (ioctl(s, SIOCGIFCONF, (char *) &ifc) < 0) {
+				perror("ifconfig (SIOCGIFCONF)");
+				exit (1);
+			}
+		} while (ifc.ifc_len > oldbufsize);
+		
 		ifflags = ifc.ifc_req->ifr_flags;
 		end = (struct ifreq *) (ifc.ifc_buf + ifc.ifc_len);
 		ifptr = ifc.ifc_req;
@@ -235,7 +251,7 @@ main(argc, argv)
 				perror("ifconfig: socket");
 				exit(1);
 			}
-			if (ifptr->ifr_flags == ifflags)
+			if (ifptr->ifr_flags == ifflags) 
 				ifconfig(argc,argv,af,rafp,selectflag);
 			if(ifptr->ifr_addr.sa_len)	/* Dohw! */
 				ifptr = (struct ifreq *) ((caddr_t) ifptr +
@@ -243,6 +259,7 @@ main(argc, argv)
 				sizeof(struct sockaddr));
 			ifptr++;
 		}
+		free(buffer);
 	} else
 		ifconfig(argc,argv,af,rafp, -1);
 
@@ -287,7 +304,7 @@ ifconfig(argc,argv,af,rafp,flag)
 		perror("ioctl (SIOCGIFMTU)");
 	else
 		mtu = ifr.ifr_mtu;
-	if (argc == 0) {	
+	if (argc == 0) {
 		status();
 		return(0);
 	}
@@ -701,7 +718,7 @@ ether_status()
 						m += printf("%02x%c",
 							*cp++ & 0xff,
 							n > 0 ? ':' : ' ');
-					putchar('\n');	
+					putchar('\n');
 				}
 				break;
 			}
@@ -710,7 +727,7 @@ ether_status()
 	}
 	kvm_close(kvmd);
 }
-	
+
 Perror(cmd)
 	char *cmd;
 {
