@@ -43,7 +43,7 @@ static char copyright[] =
 #ifndef lint
 /*static char sccsid[] = "From: @(#)mountd.c	8.8 (Berkeley) 2/20/94";*/
 static const char rcsid[] =
-	"$Id: mountd.c,v 1.5 1995/03/26 23:36:38 wpaul Exp $";
+  "$FreeBSD: stable/2.1/sbin/mountd/mountd.c 50952 1999-09-05 11:54:50Z peter $";
 #endif /*not lint*/
 
 #include <sys/param.h>
@@ -205,7 +205,8 @@ struct ucred def_anon = {
 	1,
 	{ (gid_t) -2 }
 };
-int root_only = 1;
+int resvport_only = 1;
+int dir_only = 1;
 int opt_flags;
 /* Bits for above */
 #define	OP_MAPROOT	0x01
@@ -236,8 +237,9 @@ main(argc, argv)
 	int argc;
 	char **argv;
 {
-	SVCXPRT *transp;
+	SVCXPRT *udptransp, *tcptransp;
 	int c;
+#ifdef __FreeBSD__
 	struct vfsconf *vfc;
 
 	vfc = getvfsbyname("nfs");
@@ -250,17 +252,21 @@ main(argc, argv)
 	if(!vfc) {
 		errx(1, "NFS support is not available in the running kernel");
 	}
+#endif	/* __FreeBSD__ */
 
-	while ((c = getopt(argc, argv, "dn")) != EOF)
+	while ((c = getopt(argc, argv, "dnr")) != EOF)
 		switch (c) {
+		case 'n':
+			resvport_only = 0;
+			break;
+		case 'r':
+			dir_only = 0;
+			break;
 		case 'd':
 			debug = debug ? 0 : 1;
 			break;
-		case 'n':
-			root_only = 0;
-			break;
 		default:
-			fprintf(stderr, "Usage: mountd [-n] [export_file]\n");
+			fprintf(stderr, "Usage: mountd [-r] [-n] [export_file]\n");
 			exit(1);
 		};
 	argc -= optind;
@@ -295,13 +301,16 @@ main(argc, argv)
 		fclose(pidfile);
 	  }
 	}
-	if ((transp = svcudp_create(RPC_ANYSOCK)) == NULL) {
+	if ((udptransp = svcudp_create(RPC_ANYSOCK)) == NULL ||
+	    (tcptransp = svctcp_create(RPC_ANYSOCK, 0, 0)) == NULL) {
 		syslog(LOG_ERR, "Can't create socket");
 		exit(1);
 	}
 	pmap_unset(RPCPROG_MNT, RPCMNT_VER1);
-	if (!svc_register(transp, RPCPROG_MNT, RPCMNT_VER1, mntsrv,
-	    IPPROTO_UDP)) {
+	if (!svc_register(udptransp, RPCPROG_MNT, RPCMNT_VER1, mntsrv,
+	    IPPROTO_UDP) ||
+	    !svc_register(tcptransp, RPCPROG_MNT, RPCMNT_VER1, mntsrv,
+	    IPPROTO_TCP)) {
 		syslog(LOG_ERR, "Can't register mount");
 		exit(1);
 	}
@@ -326,6 +335,7 @@ mntsrv(rqstp, transp)
 	struct statfs fsb;
 	struct hostent *hp;
 	u_long saddr;
+	u_short sport;
 	char rpcpath[RPCMNT_PATHLEN+1], dirpath[MAXPATHLEN];
 	int bad = ENOENT, omask, defset;
 	uid_t uid = -2;
@@ -342,6 +352,7 @@ mntsrv(rqstp, transp)
 	}
 
 	saddr = transp->xp_raddr.sin_addr.s_addr;
+	sport = ntohs(transp->xp_raddr.sin_port);
 	hp = (struct hostent *)NULL;
 	switch (rqstp->rq_proc) {
 	case NULLPROC:
@@ -349,7 +360,7 @@ mntsrv(rqstp, transp)
 			syslog(LOG_ERR, "Can't send reply");
 		return;
 	case RPCMNT_MOUNT:
-		if ((uid != 0 && root_only) || uid == -2) {
+		if (sport >= IPPORT_RESERVED && resvport_only) {
 			svcerr_weakauth(transp);
 			return;
 		}
@@ -360,11 +371,13 @@ mntsrv(rqstp, transp)
 
 		/*
 		 * Get the real pathname and make sure it is a directory
-		 * that exists.
+		 * or a regular file if the -r option was specified
+		 * and it exists.
 		 */
 		if (realpath(rpcpath, dirpath) == 0 ||
 		    stat(dirpath, &stb) < 0 ||
-		    (stb.st_mode & S_IFMT) != S_IFDIR ||
+		    (!S_ISDIR(stb.st_mode) &&
+		     (dir_only || !S_ISREG(stb.st_mode))) ||
 		    statfs(dirpath, &fsb) < 0) {
 			chdir("/");	/* Just in case realpath doesn't */
 			if (debug)
@@ -418,7 +431,7 @@ mntsrv(rqstp, transp)
 			syslog(LOG_ERR, "Can't send reply");
 		return;
 	case RPCMNT_UMOUNT:
-		if ((uid != 0 && root_only) || uid == -2) {
+		if (sport >= IPPORT_RESERVED && resvport_only) {
 			svcerr_weakauth(transp);
 			return;
 		}
@@ -434,7 +447,7 @@ mntsrv(rqstp, transp)
 		del_mlist(inet_ntoa(transp->xp_raddr.sin_addr), dirpath);
 		return;
 	case RPCMNT_UMNTALL:
-		if ((uid != 0 && root_only) || uid == -2) {
+		if (sport >= IPPORT_RESERVED && resvport_only) {
 			svcerr_weakauth(transp);
 			return;
 		}
@@ -580,14 +593,14 @@ put_exlist(dp, xdrsp, adp, putdefp)
 					if (!xdr_bool(xdrsp, &true))
 						return (1);
 					strp = grp->gr_ptr.gt_hostent->h_name;
-					if (!xdr_string(xdrsp, &strp, 
+					if (!xdr_string(xdrsp, &strp,
 					    RPCMNT_NAMELEN))
 						return (1);
 				} else if (grp->gr_type == GT_NET) {
 					if (!xdr_bool(xdrsp, &true))
 						return (1);
 					strp = grp->gr_ptr.gt_net.nt_name;
-					if (!xdr_string(xdrsp, &strp, 
+					if (!xdr_string(xdrsp, &strp,
 					    RPCMNT_NAMELEN))
 						return (1);
 				}
@@ -829,7 +842,7 @@ get_exportlist()
 			hpe = (struct hostent *)malloc(sizeof(struct hostent));
 			if (hpe == (struct hostent *)NULL)
 				out_of_mem();
-			hpe->h_name = "Default";
+			hpe->h_name = strdup("Default");
 			hpe->h_addrtype = AF_INET;
 			hpe->h_length = sizeof (u_long);
 			hpe->h_addr_list = (char **)NULL;
