@@ -46,6 +46,7 @@ static char sccsid[] = "@(#)mkfs.c	8.3 (Berkeley) 2/3/94";
 #include <sys/disklabel.h>
 #include <sys/file.h>
 #include <sys/mman.h>
+#include <sys/ioctl.h>
 
 #ifndef STANDALONE
 #include <a.out.h>
@@ -84,8 +85,6 @@ extern int	sectorsize;	/* bytes/sector */
 extern int	rpm;		/* revolutions/minute of drive */
 extern int	interleave;	/* hardware sector interleave */
 extern int	trackskew;	/* sector 0 skew, per track */
-extern int	headswitch;	/* head switch time, usec */
-extern int	trackseek;	/* track-to-track seek, usec */
 extern int	fsize;		/* fragment size */
 extern int	bsize;		/* block size */
 extern int	cpg;		/* cylinders/cylinder group */
@@ -103,6 +102,10 @@ extern u_long	memleft;	/* virtual memory available */
 extern caddr_t	membase;	/* start address of memory based filesystem */
 extern caddr_t	malloc(), calloc();
 extern char *	filename;
+#ifdef FSIRAND
+extern long	random();
+extern void	srandom();
+#endif
 
 union {
 	struct fs fs;
@@ -121,6 +124,7 @@ struct dinode zino[MAXBSIZE / sizeof(struct dinode)];
 
 int	fsi, fso;
 daddr_t	alloc();
+static int charsperline();
 
 mkfs(pp, fsys, fi, fo)
 	struct partition *pp;
@@ -136,9 +140,14 @@ mkfs(pp, fsys, fi, fo)
 	time_t utime;
 	quad_t sizepb;
 	void started();
+	int width;
+	char tmpbuf[100];	/* XXX this will break in about 2,500 years */
 
 #ifndef STANDALONE
 	time(&utime);
+#endif
+#ifdef FSIRAND
+	srandom(utime ^ getpid());
 #endif
 	if (mfs) {
 		ppid = getpid();
@@ -393,7 +402,7 @@ mkfs(pp, fsys, fi, fo)
 			    fsize, sblock.fs_fsize);
 		exit(23);
 	}
-	/* 
+	/*
 	 * Calculate the number of cylinders per group
 	 */
 	sblock.fs_cpg = cpg;
@@ -586,8 +595,6 @@ next:
 	sblock.fs_rotdelay = rotdelay;
 	sblock.fs_minfree = minfree;
 	sblock.fs_maxcontig = maxcontig;
-	sblock.fs_headswitch = headswitch;
-	sblock.fs_trkseek = trackseek;
 	sblock.fs_maxbpg = maxbpg;
 	sblock.fs_rps = rpm / 60;
 	sblock.fs_optim = opt;
@@ -599,6 +606,11 @@ next:
 	sblock.fs_fmod = 0;
 	sblock.fs_ronly = 0;
 	sblock.fs_clean = 1;
+#ifdef FSIRAND
+	sblock.fs_id[0] = (long)utime;
+	sblock.fs_id[1] = random();
+#endif
+
 	/*
 	 * Dump out summary information about file system.
 	 */
@@ -619,14 +631,21 @@ next:
 	 * then print out indices of cylinder groups.
 	 */
 	if (!mfs)
-		printf("super-block backups (for fsck -b #) at:");
+		printf("super-block backups (for fsck -b #) at:\n");
+	i = 0;
+	width = charsperline();
 	for (cylno = 0; cylno < sblock.fs_ncg; cylno++) {
 		initcg(cylno, utime);
 		if (mfs)
 			continue;
-		if (cylno % 9 == 0)
+		j = sprintf(tmpbuf, " %d,",
+			fsbtodb(&sblock, cgsblock(&sblock, cylno)));
+		if (i+j >= width) {
 			printf("\n");
-		printf(" %d,", fsbtodb(&sblock, cgsblock(&sblock, cylno)));
+			i = 0;
+		}
+		i += j;
+		printf("%s", tmpbuf);
 		fflush(stdout);
 	}
 	if (!mfs)
@@ -645,7 +664,7 @@ next:
 			sblock.fs_cssize - i < sblock.fs_bsize ?
 			    sblock.fs_cssize - i : sblock.fs_bsize,
 			((char *)fscs) + i);
-	/* 
+	/*
 	 * Write out the duplicate super blocks
 	 */
 	for (cylno = 0; cylno < sblock.fs_ncg; cylno++)
@@ -712,7 +731,7 @@ initcg(cylno, utime)
 		acg.cg_nclusterblks = acg.cg_ndblk / sblock.fs_frag;
 	acg.cg_btotoff = &acg.cg_space[0] - (u_char *)(&acg.cg_link);
 	acg.cg_boff = acg.cg_btotoff + sblock.fs_cpg * sizeof(long);
-	acg.cg_iusedoff = acg.cg_boff + 
+	acg.cg_iusedoff = acg.cg_boff +
 		sblock.fs_cpg * sblock.fs_nrpos * sizeof(short);
 	acg.cg_freeoff = acg.cg_iusedoff + howmany(sblock.fs_ipg, NBBY);
 	if (sblock.fs_contigsumsize <= 0) {
@@ -739,9 +758,14 @@ initcg(cylno, utime)
 			setbit(cg_inosused(&acg), i);
 			acg.cg_cs.cs_nifree--;
 		}
-	for (i = 0; i < sblock.fs_ipg / INOPF(&sblock); i += sblock.fs_frag)
+	for (i = 0; i < sblock.fs_ipg / INOPF(&sblock); i += sblock.fs_frag) {
+#ifdef FSIRAND
+		for (j = 0; j < sblock.fs_bsize / sizeof(struct dinode); j++)
+			zino[j].di_gen = random();
+#endif
 		wtfs(fsbtodb(&sblock, cgimin(&sblock, cylno) + i),
 		    sblock.fs_bsize, (char *)zino);
+	}
 	if (cylno > 0) {
 		/*
 		 * In cylno 0, beginning space is reserved
@@ -1007,6 +1031,9 @@ iput(ip, ino)
 	daddr_t d;
 	int c;
 
+#ifdef FSIRAND
+	ip->di_gen = random();
+#endif
 	c = ino_to_cg(&sblock, ino);
 	rdfs(fsbtodb(&sblock, cgtod(&sblock, 0)), sblock.fs_cgsize,
 	    (char *)&acg);
@@ -1110,7 +1137,7 @@ calloc(size, numelm)
 free(ptr)
 	char *ptr;
 {
-	
+
 	/* do not worry about it for now */
 }
 
@@ -1263,3 +1290,26 @@ setblock(fs, cp, h)
 		return;
 	}
 }
+
+/*
+ * Determine the number of characters in a
+ * single line.
+ */
+
+static int
+charsperline()
+{
+	int columns;
+	char *cp;
+	struct winsize ws;
+	extern char *getenv();
+
+	columns = 0;
+	if (ioctl(0, TIOCGWINSZ, &ws) != -1)
+		columns = ws.ws_col;
+	if (columns == 0 && (cp = getenv("COLUMNS")))
+		columns = atoi(cp);
+	if (columns == 0)
+		columns = 80;	/* last resort */
+	return columns;
+}
