@@ -189,6 +189,7 @@ int	nrpos = NRPOS;		/* # of distinguished rotational positions */
 int	bbsize = BBSIZE;	/* boot block size */
 int	sbsize = SBSIZE;	/* superblock size */
 int	mntflags = MNT_ASYNC;	/* flags to be passed to mount */
+int	t_or_u_flag = 0;	/* user has specified -t or -u */
 u_long	memleft;		/* virtual memory available */
 caddr_t	membase;		/* start address of memory based filesystem */
 char	*filename;
@@ -326,10 +327,12 @@ main(argc, argv)
 				fatal("%s: bad file system size", optarg);
 			break;
 		case 't':
+			t_or_u_flag++;
 			if ((ntracks = atoi(optarg)) < 0)
 				fatal("%s: bad total tracks", optarg);
 			break;
 		case 'u':
+			t_or_u_flag++;
 			if ((nsectors = atoi(optarg)) < 0)
 				fatal("%s: bad sectors/track", optarg);
 			break;
@@ -474,7 +477,7 @@ main(argc, argv)
 	 * transfer size permitted by the controller or buffering.
 	 */
 	if (maxcontig == 0)
-		maxcontig = MAX(1, MIN(MAXPHYS, MAXBSIZE) / bsize - 1);
+		maxcontig = MAX(1, MAXPHYS / bsize - 1);
 	if (density == 0)
 		density = NFPI * fsize;
 	if (minfree < MINFREE && opt != FS_OPTSPACE) {
@@ -494,7 +497,12 @@ main(argc, argv)
 			cylspares = 0;
 	}
 	secpercyl = nsectors * ntracks - cylspares;
-	if (secpercyl != lp->d_secpercyl)
+	/*
+	 * Only complain if -t or -u have been specified; the default
+	 * case (4096 sectors per cylinder) is intented to disagree
+	 * with the disklabel.
+	 */
+	if (t_or_u_flag && secpercyl != lp->d_secpercyl)
 		fprintf(stderr, "%s (%d) %s (%lu)\n",
 			"Warning: calculated sectors per cylinder", secpercyl,
 			"disagrees with disk label", lp->d_secpercyl);
