@@ -1,8 +1,8 @@
 /*
- * Rdisc (this program) was developed by Sun Microsystems, Inc. and is 
- * provided for unrestricted use provided that this legend is included on 
- * all tape media and as a part of the software program in whole or part.  
- * Users may copy or modify Rdisc without charge, and they may freely 
+ * Rdisc (this program) was developed by Sun Microsystems, Inc. and is
+ * provided for unrestricted use provided that this legend is included on
+ * all tape media and as a part of the software program in whole or part.
+ * Users may copy or modify Rdisc without charge, and they may freely
  * distribute it.
  *
  * RDISC IS PROVIDED AS IS WITH NO WARRANTIES OF ANY KIND INCLUDING THE
@@ -43,50 +43,34 @@
 #include <netinet/ip.h>
 #include <netinet/ip_icmp.h>
 
-/*
- * The next include contains all defs and structures for multicast
- * that are not in SunOS 4.1.x. On a SunOS 4.1.x system none of this code
- * is ever used because it does not support multicast
- * Fraser Gardiner - Sun Microsystems Australia
- */
-
-#include "rdisc.h"
 #include <netdb.h>
 #include <arpa/inet.h>
 
 #include <string.h>
 #include <syslog.h>
+#include <stdlib.h>
+#include <unistd.h>
+#include <err.h>
+#include <sys/cdefs.h>
 
-/* 
+#if __STDC__
+#include <stdarg.h>
+#else
+#include <varargs.h>
+#endif
+
+/*
  * TBD
  *	Use 255.255.255.255 for broadcasts - not the interface broadcast
  *	address.
  */
 
-char *malloc();
-void exit();
-long gethostid();
-long random();
-time_t time();
-long strtol();
-
 #ifdef lint
 #define ALLIGN(ptr)	(ptr ? 0 : 0)
 #else
 #define ALLIGN(ptr)	(ptr)
 #endif
 
-#ifdef SYSV
-#define signal(s,f)	sigset(s,f)
-#define random()	rand()
-#endif
-
-void solicitor(), advertise(), pr_pack(), init();
-void bzero(), initlog(), logerr(), logtrace(), logdebug(), logperror();
-void age_table(), record_router(), add_route(), del_route(), rtioctl();
-int support_multicast(), sendbcast(), sendmcast(), sendbcastif(), sendmcastif();
-int ismulticast(), isbroadcast(), in_cksum(), is_directly_connected();
-static int join();
 
 #define ICMP_ROUTER_ADVERTISEMENT	9
 #define ICMP_ROUTER_SOLICITATION	10
@@ -96,21 +80,6 @@ static int join();
 
 #define MAXIFS 32
 
-/* For router advertisement */
-struct icmp_ra {
-	u_char	icmp_type;		/* type of message, see below */
-	u_char	icmp_code;		/* type sub code */
-	u_short	icmp_cksum;		/* ones complement cksum of struct */
-	u_char	icmp_num_addrs;
-	u_char	icmp_wpa;		/* Words per address */
-	short 	icmp_lifetime;
-};
-
-struct icmp_ra_addr {
-	u_long	addr;
-	u_long preference;
-};
-
 /* Router constants */
 #define	MAX_INITIAL_ADVERT_INTERVAL	16
 #define	MAX_INITIAL_ADVERTISEMENTS  	3
@@ -143,8 +112,6 @@ int interfaces_size;			/* Number of elements in interfaces */
 #define	MAXPACKET	4096	/* max packet size */
 u_char	packet[MAXPACKET];
 
-/* fraser */
-int debugfile;
 char usage[] =
 "Usage:	rdisc [-s] [-v] [-f] [-a] [send_address] [receive_address]\n\
         rdisc -r [-v] [-p <preference>] [-T <secs>] \n\
@@ -162,7 +129,7 @@ int solicit = 0;
 int responder;
 int ntransmitted = 0;
 int nreceived = 0;
-int forever = 0;	/* Never give up on host. If 0 defer fork until 
+int forever = 0;	/* Never give up on host. If 0 defer fork until
 			 * first response.
 			 */
 
@@ -177,15 +144,49 @@ int preference = 0;		/* Setable with -p option */
 /* Host variables */
 int max_solicitations = MAX_SOLICITATIONS;
 unsigned int solicitation_interval = SOLICITATION_INTERVAL;
-int best_preference = 1;  	/* Set to record only the router(s) with the 
+int best_preference = 1;  	/* Set to record only the router(s) with the
 				   best preference in the kernel. Not set
 				   puts all routes in the kernel. */
 
+/* Prototypes */
+void	do_fork		__P((void));
+int	main		__P((int, char **));
+void	timer		__P((int));
+void	solicitor	__P((struct sockaddr_in *));
+void	advertise	__P((struct sockaddr_in *));
+char *	pr_type		__P((int));
+char *	pr_name		__P((struct in_addr));
+void	pr_pack		__P((char *, int, struct sockaddr_in *));
+int	in_cksum	__P((u_short *, int));
+void	finish		__P((int));
+int	isbroadcast	__P((struct sockaddr_in *));
+int	ismulticast	__P((struct sockaddr_in *));
+int	sendbcast	__P((int, char *, int));
+int	sendbcastif	__P((int, char *, int, struct interface *));
+int	sendmcast	__P((int, char *, int, struct sockaddr_in *));
+int	sendmcastif	__P((int, char *, int, struct sockaddr_in *, struct interface *));
+void	init		__P((void));
+void	initifs		__P((int));
+int	support_multicast	__P((void));
+int	is_directly_connected	__P((struct in_addr));
+struct table * find_router	__P((struct in_addr));
+int	max_preference	__P((void));
+void	age_table	__P((int));
+void	record_router	__P((struct in_addr, long, int));
+void	add_route	__P((struct in_addr));
+void	del_route	__P((struct in_addr));
+void	rtioctl		__P((struct in_addr, int));
+void	initlog		__P((void));
+void	logerr		__P((char *fmt, ...));
+void	logtrace	__P((char *fmt, ...));
+void	logdebug	__P((char *fmt, ...));
+void	logperror	__P((char *str));
+void	prusage		__P((void));
+int	join		__P((int sock, struct sockaddr_in *sin));
 
-void finish(), catcher(), timer(), initifs();
-char *pr_name();
 
-static void prusage()
+void
+prusage()
 {
 	(void) fprintf(stderr, usage);
 	exit(1);
@@ -196,26 +197,19 @@ do_fork()
 {
 	int t;
 	long pid;
-	
+
 	if (trace)
 		return;
 
-	if (pid =fork())
+	if ((pid = fork()))
 		exit(0);
 
 	for (t = 0; t < 20; t++)
 		if (t != s)
 			(void) close(t);
 
-#ifndef SYSV
-	t = open("/dev/tty", 2);
-	if (t >= 0) {
-		(void) ioctl(t, TIOCNOTTY, (char *)0);
-		(void) close(t);
-	}
-#else
-	(void) setpgrp ();
-#endif
+	(void) setsid ();
+
 	initlog();
 }
 
@@ -224,24 +218,23 @@ do_fork()
  */
 char    *sendaddress, *recvaddress;
 
+int
 main(argc, argv)
-char *argv[];
+	int argc;
+	char *argv[];
 {
-#ifndef SYSV
-	struct sigvec sv;
-#endif
+	struct sigaction sa;
 	struct sockaddr_in from;
-	char **av = argv;
 	struct sockaddr_in *to = &whereto;
 	struct sockaddr_in joinaddr;
 	int val;
+	int c;
 
 	min_adv_int =( max_adv_int * 3 / 4);
 	lifetime = (3*max_adv_int);
 
-	argc--, av++;
-	while (argc > 0 && *av[0] == '-') {
-		while (*++av[0]) switch (*av[0]) {
+	while ((c = getopt(argc, argv, "dtvsrabfT:p:")) != EOF) {
+		switch(c) {
 		case 'd':
 			debug = 1;
 			break;
@@ -267,39 +260,28 @@ char *argv[];
 			forever = 1;
 			break;
 		case 'T':
-			argc--, av++;
-			if (argc != 0) {
-				val = strtol(av[0], (char **)NULL, 0);
-				if (val < 4 || val > 1800) {
-					(void) fprintf(stderr, 
-				          "Bad Max Advertizement Interval\n");
-					exit(1);
-				}
-				max_adv_int = val;
-				min_adv_int =( max_adv_int * 3 / 4);
-				lifetime = (3*max_adv_int);
-			} else {
-				prusage();
-				/* NOTREACHED*/
-			} 
-			goto next;
-		case 'p':
-			argc--, av++;
-			if (argc != 0) {
-				val = strtol(av[0], (char **)NULL, 0);
-				preference = val;
-			} else {
-				prusage();
-				/* NOTREACHED*/
+			val = strtol(optarg, (char **)NULL, 0);
+			if (val < 4 || val > 1800) {
+				(void) fprintf(stderr,
+			          "Bad Max Advertizement Interval\n");
+				exit(1);
 			}
-			goto next;
+			max_adv_int = val;
+			min_adv_int =( max_adv_int * 3 / 4);
+			lifetime = (3*max_adv_int);
+			break;
+		case 'p':
+			val = strtol(optarg, (char **)NULL, 0);
+			preference = val;
+			break;
 		default:
 			prusage();
 			/* NOTREACHED*/
 		}
-	next:
-		argc--, av++;
 	}
+	argc -= optind;
+	argv += optind;
+
 	if( argc < 1)  {
 		if (support_multicast()) {
 			if (responder)
@@ -309,8 +291,8 @@ char *argv[];
 		} else
 			sendaddress = "255.255.255.255";
 	} else {
-		sendaddress = av[0];
-		argc--;
+		sendaddress = argv[0];
+		argc--; argv++;
 	}
 
 	if (argc < 1) {
@@ -322,8 +304,8 @@ char *argv[];
 		} else
 			recvaddress = "255.255.255.255";
 	} else {
-		recvaddress = av[0];
-		argc--;
+		recvaddress = argv[0];
+		argc--; argv++;
 	}
 	if (argc != 0) {
 		(void) fprintf(stderr, "Extra paramaters\n");
@@ -354,12 +336,8 @@ char *argv[];
 	joinaddr.sin_addr.s_addr = inet_addr(recvaddress);
 
 	if (responder) {
-#ifdef SYSV
 		/* TBD fix this to be more random */
-		srand((u_int)time((time_t *)NULL));
-#else
-		srandom((int)gethostid());
-#endif
+		srandom((int)gethostid()+(int)time((time_t *)NULL));
 	}
 
 	if ((s = socket(AF_INET, SOCK_RAW, IPPROTO_ICMP)) < 0) {
@@ -367,11 +345,7 @@ char *argv[];
 		exit(5);
 	}
 
-#ifdef SYSV
 	setvbuf( stdout, NULL, _IOLBF, 0 );
-#else
-	setlinebuf( stdout );
-#endif
 
 	(void) signal( SIGINT, finish );
 	(void) signal( SIGTERM, finish );
@@ -384,26 +358,23 @@ char *argv[];
 	}
 
 
-#ifdef SYSV
-	(void) sigset(SIGALRM, timer);
-#else
 	/*
 	 * Make sure that this signal actually interrupts (rather than
 	 * restarts) the recvfrom call below.
 	 */
-	sv.sv_handler = timer;
-	sv.sv_mask = 0;
-	sv.sv_flags = SV_INTERRUPT;
-	(void) sigvec(SIGALRM, &sv, (struct sigvec *)NULL);
-#endif
-	timer();	/* start things going */
+	memset(&sa, 0, sizeof(sa));
+	sa.sa_handler = timer;
+	sigemptyset(&sa.sa_mask);
+	sa.sa_flags = 0;	/* note: no SA_RESTART */
+	(void) sigaction(SIGALRM, &sa, (struct sigaction *)NULL);
+	timer(0);	/* start things going */
 
 	for (;;) {
 		int len = sizeof (packet);
 		int fromlen = sizeof (from);
 		int cc;
 
-		if ( (cc=recvfrom(s, (char *)packet, len, 0, 
+		if ( (cc=recvfrom(s, (char *)packet, len, 0,
 				  (struct sockaddr *)&from, &fromlen)) < 0) {
 			if( errno == EINTR )
 				continue;
@@ -418,24 +389,21 @@ char *argv[];
 #define TIMER_INTERVAL 	3
 #define GETIFCONF_TIMER	30
 
-static left_until_advertise;
+int left_until_advertise;
 
 /* Called every TIMER_INTERVAL */
-void timer()
+void timer(sig)
+     int sig;
 {
-	static time;
-	static left_until_getifconf;
-	static left_until_solicit;
-
-
-	time += TIMER_INTERVAL;
+	static int left_until_getifconf;
+	static int left_until_solicit;
 
 	left_until_getifconf -= TIMER_INTERVAL;
 	left_until_advertise -= TIMER_INTERVAL;
 	left_until_solicit -= TIMER_INTERVAL;
 
 	if (left_until_getifconf < 0) {
-		initifs();
+		initifs(0);
 		left_until_getifconf = GETIFCONF_TIMER;
 	}
 	if (responder && left_until_advertise <= 0) {
@@ -444,8 +412,8 @@ void timer()
 		if (ntransmitted < initial_advertisements)
 			left_until_advertise = initial_advert_interval;
 		else
-			left_until_advertise = min_adv_int + 
-				((max_adv_int - min_adv_int) * 
+			left_until_advertise = min_adv_int +
+				((max_adv_int - min_adv_int) *
 				 (random() % 1000)/1000);
 	} else if (solicit && left_until_solicit <= 0) {
 		ntransmitted++;
@@ -464,9 +432,9 @@ void timer()
 
 /*
  * 			S O L I C I T O R
- * 
- * Compose and transmit an ICMP ROUTER SOLICITATION REQUEST packet.  
- * The IP packet will be added on by the kernel.  
+ *
+ * Compose and transmit an ICMP ROUTER SOLICITATION REQUEST packet.
+ * The IP packet will be added on by the kernel.
  */
 void
 solicitor(sin)
@@ -475,7 +443,7 @@ solicitor(sin)
 	static u_char outpack[MAXPACKET];
 	register struct icmp *icp = (struct icmp *) ALLIGN(outpack);
 	int packetlen, i;
-	
+
 	if (verbose) {
 		logtrace("Sending solicitation to %s\n",
 			 pr_name(sin->sin_addr));
@@ -489,7 +457,7 @@ solicitor(sin)
 	/* Compute ICMP checksum here */
 	icp->icmp_cksum = in_cksum( (u_short *)icp, packetlen );
 
-	if (isbroadcast(sin)) 
+	if (isbroadcast(sin))
 		i = sendbcast(s, (char *)outpack, packetlen);
 	else if (ismulticast(sin))
 		i = sendmcast( s, (char *)outpack, packetlen, sin);
@@ -508,41 +476,41 @@ solicitor(sin)
 
 /*
  * 			A V E R T I S E
- * 
- * Compose and transmit an ICMP ROUTER ADVERTISEMENT packet.  
- * The IP packet will be added on by the kernel.  
+ *
+ * Compose and transmit an ICMP ROUTER ADVERTISEMENT packet.
+ * The IP packet will be added on by the kernel.
  */
 void
 advertise(sin)
 	struct sockaddr_in *sin;
 {
 	static u_char outpack[MAXPACKET];
-	register struct icmp_ra *rap = (struct icmp_ra *) ALLIGN(outpack);
+	register struct icmp *rap = (struct icmp *) ALLIGN(outpack);
 	struct icmp_ra_addr *ap;
 	int packetlen, i, cc;
-	
+
 	if (verbose) {
 		logtrace("Sending advertisement to %s\n",
 			 pr_name(sin->sin_addr));
 	}
-	
+
 	for (i = 0; i < num_interfaces; i++) {
 		rap->icmp_type = ICMP_ROUTER_ADVERTISEMENT;
 		rap->icmp_code = 0;
 		rap->icmp_cksum = 0;
-		rap->icmp_num_addrs = 0; 
+		rap->icmp_num_addrs = 0;
 		rap->icmp_wpa = 2;
 		rap->icmp_lifetime = lifetime;
 		packetlen = 8;
 
-		/* 
-		 * TODO handle multiple logical interfaces per 
+		/*
+		 * TODO handle multiple logical interfaces per
 		 * physical interface. (increment with rap->icmp_wpa * 4 for
 		 * each address.)
 		 */
 		ap = (struct icmp_ra_addr *)ALLIGN(outpack + ICMP_MINLEN);
-		ap->addr = interfaces[i].localaddr.s_addr;
-		ap->preference = interfaces[i].preference;
+		ap->ira_addr = interfaces[i].localaddr.s_addr;
+		ap->ira_preference = interfaces[i].preference;
 		packetlen += rap->icmp_wpa * 4;
 		rap->icmp_num_addrs++;
 
@@ -550,15 +518,15 @@ advertise(sin)
 		/* Compute ICMP checksum here */
 		rap->icmp_cksum = in_cksum( (u_short *)rap, packetlen );
 
-		if (isbroadcast(sin)) 
-			cc = sendbcastif(s, (char *)outpack, packetlen, 
+		if (isbroadcast(sin))
+			cc = sendbcastif(s, (char *)outpack, packetlen,
 					&interfaces[i]);
 		else if (ismulticast(sin))
 			cc = sendmcastif( s, (char *)outpack, packetlen, sin,
 					&interfaces[i]);
 		else {
 			struct interface *ifp = &interfaces[i];
-			/* 
+			/*
 			 * Verify that the interface matches the destination
 			 * address.
 			 */
@@ -567,11 +535,11 @@ advertise(sin)
 				if (debug) {
 					logdebug("Unicast to %s ",
 						 pr_name(sin->sin_addr));
-					logdebug("on interface %s\n",	
+					logdebug("on interface %s\n",
 						 pr_name(ifp->address));
 				}
 				cc = sendto( s, (char *)outpack, packetlen, 0,
-					    (struct sockaddr *)sin, 
+					    (struct sockaddr *)sin,
 					    sizeof(struct sockaddr));
 			} else
 				cc = packetlen;
@@ -637,7 +605,7 @@ char *pr_name(addr)
 	static char buf[256];
 
 	phe = gethostbyaddr((char *)&addr.s_addr, 4, AF_INET);
-	if (phe == NULL) 
+	if (phe == NULL)
 		return( inet_ntoa(addr));
 	(void) sprintf(buf, "%s (%s)", phe->h_name, inet_ntoa(addr));
 	return(buf);
@@ -681,7 +649,7 @@ struct sockaddr_in *from;
 	}
 	switch (icp->icmp_type) {
 	case ICMP_ROUTER_ADVERTISEMENT: {
-		struct icmp_ra *rap = (struct icmp_ra *)ALLIGN(icp);
+		struct icmp *rap = (struct icmp *)ALLIGN(icp);
 		struct icmp_ra_addr *ap;
 
 		if (responder)
@@ -711,7 +679,7 @@ struct sockaddr_in *from;
 			return;
 		}
 		if (rap->icmp_num_addrs < 1) {
-			if (verbose) 
+			if (verbose)
 				logtrace("ICMP %s from %s: No addresses\n",
 					 pr_type((int)rap->icmp_type),
 					 pr_name(from->sin_addr));
@@ -725,17 +693,18 @@ struct sockaddr_in *from;
 					 rap->icmp_wpa);
 			return;
 		}
-		if ((unsigned)cc < 
+		if ((unsigned)cc <
 		    ICMP_MINLEN + rap->icmp_num_addrs * rap->icmp_wpa * 4) {
 			if (verbose)
 				logtrace("ICMP %s from %s: Too short %d, %d\n",
 					      pr_type((int)rap->icmp_type),
 					      pr_name(from->sin_addr),
-					      cc, 
-					      ICMP_MINLEN + 
+					      cc,
+					      ICMP_MINLEN +
 					      rap->icmp_num_addrs * rap->icmp_wpa * 4);
 			return;
 		}
+		rap->icmp_lifetime = ntohs(rap->icmp_lifetime);
 		if (rap->icmp_lifetime < 4) {
 			if (verbose)
 				logtrace("ICMP %s from %s: Lifetime = %d\n",
@@ -756,17 +725,17 @@ struct sockaddr_in *from;
 		for (i = 0; (unsigned)i < rap->icmp_num_addrs; i++) {
 			struct in_addr ina;
 			ap = (struct icmp_ra_addr *)
-				ALLIGN(buf + hlen + ICMP_MINLEN + 
+				ALLIGN(buf + hlen + ICMP_MINLEN +
 				       i * rap->icmp_wpa * 4);
-			ina.s_addr = ntohl(ap->addr);
+			ina.s_addr = ap->ira_addr;
 			if (verbose)
-				logtrace("\taddress %s, preference 0x%x\n", 
+				logtrace("\taddress %s, preference 0x%x\n",
 					      pr_name(ina),
-					      ntohl(ap->preference));
+					      ntohl(ap->ira_preference));
 			if (!responder) {
 				if (is_directly_connected(ina))
-					record_router(ina, 
-						      (long)ntohl(ap->preference),
+					record_router(ina,
+						      (long)ntohl(ap->ira_preference),
 						      rap->icmp_lifetime);
 			}
 		}
@@ -782,7 +751,7 @@ struct sockaddr_in *from;
 		}
 		break;
 	}
-		
+
 	case ICMP_ROUTER_SOLICITATION: {
 		struct sockaddr_in sin;
 
@@ -818,7 +787,7 @@ struct sockaddr_in *from;
 				logtrace("ICMP %s from %s: Too short %d, %d\n",
 					      pr_type((int)icp->icmp_type),
 					      pr_name(from->sin_addr),
-					      cc, 
+					      cc,
 					      ICMP_MINLEN);
 			return;
 		}
@@ -827,8 +796,8 @@ struct sockaddr_in *from;
 			logtrace("ICMP %s from %s\n",
 				      pr_type((int)icp->icmp_type),
 				      pr_name(from->sin_addr));
-		
- 		if (!responder) 
+
+ 		if (!responder)
 			break;
 
 		/* Check that ip_src is either a neighboor
@@ -844,8 +813,8 @@ struct sockaddr_in *from;
 			else
 				sin.sin_addr.s_addr = INADDR_BROADCAST;
 			/* Restart the timer when we broadcast */
-			left_until_advertise = min_adv_int + 
-				((max_adv_int - min_adv_int) 
+			left_until_advertise = min_adv_int +
+				((max_adv_int - min_adv_int)
 				 * (random() % 1000)/1000);
 		}
 		else {
@@ -863,7 +832,7 @@ struct sockaddr_in *from;
 		advertise(&sin);
 		break;
 	}
-	}
+	} /* end switch */
 }
 
 
@@ -873,6 +842,7 @@ struct sockaddr_in *from;
  * Checksum routine for Internet Protocol family headers (C Version)
  *
  */
+int
 in_cksum(addr, len)
 u_short *addr;
 int len;
@@ -919,7 +889,8 @@ int len;
  * the statistics output from becomming intermingled.
  */
 void
-finish()
+finish(sig)
+	int sig;
 {
 	if (responder) {
 		int i;
@@ -932,12 +903,13 @@ finish()
 			interfaces[i].preference = IGNORE_PREFERENCE;
 		ntransmitted++;
 		advertise(&whereto);
-	}		
-	logtrace("\n----%s rdisc Statistics----\n", sendaddress );
-	logtrace("%d packets transmitted, ", ntransmitted );
-	logtrace("%d packets received, ", nreceived );
-	logtrace("\n");
-	(void) fflush(stdout);
+	}
+	if (verbose) {
+	    logtrace("\n----%s rdisc Statistics----\n", sendaddress );
+	    logtrace("%d packets transmitted, ", ntransmitted );
+	    logtrace("%d packets received, ", nreceived );
+	    (void) fflush(stdout);
+	}
 	exit(0);
 }
 
@@ -955,40 +927,42 @@ unsigned char *data;
 	while (len) {
 		register int i;
 		char charstring[17];
-			
+
 		(void)strcpy(charstring,"                "); /* 16 spaces */
 		for (i = 0; i < 16; i++) {
-			/* output the bytes one at a time, 
-			 * not going pas "len" bytes 
+			/* output the bytes one at a time,
+			 * not going pas "len" bytes
 			 */
 			if (len) {
 				char ch = *data & 0x7f; /* strip parity */
-				if (!isprint((u_char)ch)) 
+				if (!isprint((u_char)ch))
 					ch = ' '; /* ensure printable */
 				charstring[i] = ch;
 				(void) fprintf(out,"%02x ",*data++);
 				len--;
-			} else 
+			} else
 				(void) fprintf(out,"   ");
-			if (i==7) 
+			if (i==7)
 				(void) fprintf(out,"   ");
 		}
-		
+
 		(void) fprintf(out,"    *%s*\n",charstring);
 	}
 }
 #endif notdef
 
+int
 isbroadcast(sin)
 	struct sockaddr_in *sin;
 {
 	return (sin->sin_addr.s_addr == INADDR_BROADCAST);
 }
 
+int
 ismulticast(sin)
 	struct sockaddr_in *sin;
 {
-	return (IN_CLASSD(sin->sin_addr.s_addr));
+	return (IN_CLASSD(ntohl(sin->sin_addr.s_addr)));
 }
 
 /* From libc/rpc/pmap_rmt.c */
@@ -1000,7 +974,7 @@ sendbcast(s, packet, packetlen)
 	int packetlen;
 {
 	int i, cc;
-	
+
 	for (i = 0; i < num_interfaces; i++) {
 		if ((interfaces[i].flags & IFF_BROADCAST) == 0)
 			continue;
@@ -1023,7 +997,7 @@ sendbcastif(s, packet, packetlen, ifp)
 	struct sockaddr_in baddr;
 
 	baddr.sin_family = AF_INET;
-	
+
 	if ((ifp->flags & IFF_BROADCAST) == 0)
 		return (packetlen);
 
@@ -1049,7 +1023,7 @@ sendmcast(s, packet, packetlen, sin)
 	struct sockaddr_in *sin;
 {
 	int i, cc;
-	
+
 	for (i = 0; i < num_interfaces; i++) {
 		if ((interfaces[i].flags & IFF_MULTICAST) == 0)
 			continue;
@@ -1059,7 +1033,7 @@ sendmcast(s, packet, packetlen, sin)
 		}
 	}
 	return (packetlen);
-}	
+}
 
 int
 sendmcastif(s, packet, packetlen, sin, ifp)
@@ -1073,7 +1047,7 @@ sendmcastif(s, packet, packetlen, sin, ifp)
 	struct sockaddr_in ifaddr;
 
 	ifaddr.sin_family = AF_INET;
-	
+
 	if ((ifp->flags & IFF_MULTICAST) == 0)
 		return (packetlen);
 
@@ -1081,7 +1055,7 @@ sendmcastif(s, packet, packetlen, sin, ifp)
 	if (debug)
 		logdebug("Multicast to interface %s\n",
 			 pr_name(ifaddr.sin_addr));
-	if (setsockopt(s, IPPROTO_IP, IP_MULTICAST_IF, 
+	if (setsockopt(s, IPPROTO_IP, IP_MULTICAST_IF,
 		       (char *)&ifaddr.sin_addr,
 		       sizeof(ifaddr.sin_addr)) < 0) {
 		logperror("setsockopt (IP_MULTICAST_IF)");
@@ -1104,17 +1078,18 @@ init()
 {
 	int i;
 
-	initifs();
+	initifs(0);
 	for (i = 0; i < interfaces_size; i++)
 		interfaces[i].preference = preference;
 }
 
 void
-initifs()
+initifs(sig)
+	int sig;
 {
 	int	sock;
 	struct ifconf ifc;
-	struct ifreq ifreq, *ifr;
+	struct ifreq ifreq, *ifrp, *ifend;
 	struct sockaddr_in *sin;
 	int n, i;
 	char *buf;
@@ -1140,7 +1115,8 @@ initifs()
 		(void) close(sock);
 		return;
 	}
-	if (interfaces) 
+	bzero(buf, bufsize);
+	if (interfaces)
 		interfaces = (struct interface *)ALLIGN(realloc((char *)interfaces,
 					 numifs * sizeof(struct interface)));
 	else
@@ -1153,7 +1129,7 @@ initifs()
 		return;
 	}
 	interfaces_size = numifs;
-						
+
 	ifc.ifc_len = bufsize;
 	ifc.ifc_buf = buf;
 	if (ioctl(sock, SIOCGIFCONF, (char *)&ifc) < 0) {
@@ -1162,14 +1138,19 @@ initifs()
 		(void) free(buf);
 		return;
 	}
-	ifr = ifc.ifc_req;
-	for (i = 0, n = ifc.ifc_len/sizeof (struct ifreq); n > 0; n--, ifr++) {
-		ifreq = *ifr;
+	ifrp = (struct ifreq *)buf;
+	ifend = (struct ifreq *)(buf + ifc.ifc_len);
+	for (i = 0; ifrp < ifend; ifrp = (struct ifreq *)((char *)ifrp + n)) {
+		ifreq = *ifrp;
 		if (ioctl(sock, SIOCGIFFLAGS, (char *)&ifreq) < 0) {
 			logperror("initifs: ioctl (get interface flags)");
+			n = 0;
 			continue;
 		}
-		if (ifr->ifr_addr.sa_family != AF_INET) 
+		n = ifrp->ifr_addr.sa_len + sizeof(ifrp->ifr_name);
+		if (n < sizeof(*ifrp))
+		    n = sizeof(*ifrp);
+		if (ifrp->ifr_addr.sa_family != AF_INET)
 			continue;
 		if ((ifreq.ifr_flags & IFF_UP) == 0)
 			continue;
@@ -1177,7 +1158,7 @@ initifs()
 			continue;
 		if ((ifreq.ifr_flags & (IFF_MULTICAST | IFF_BROADCAST)) == 0)
 			continue;
-		sin = (struct sockaddr_in *)ALLIGN(&ifr->ifr_addr);
+		sin = (struct sockaddr_in *)ALLIGN(&ifrp->ifr_addr);
 		interfaces[i].localaddr = sin->sin_addr;
 		interfaces[i].flags = ifreq.ifr_flags;
 		interfaces[i].netmask.s_addr = (unsigned long)0xffffffff;
@@ -1229,7 +1210,7 @@ initifs()
 	(void) free(buf);
 }
 
-static int 
+int
 join(sock, sin)
 	int sock;
 	struct sockaddr_in *sin;
@@ -1264,7 +1245,7 @@ int support_multicast()
 		return (0);
 	}
 
-	if (setsockopt(sock, IPPROTO_IP, IP_MULTICAST_TTL, 
+	if (setsockopt(sock, IPPROTO_IP, IP_MULTICAST_TTL,
 		       (char *)&ttl, sizeof(ttl)) < 0) {
 		(void) close(sock);
 		return (0);
@@ -1282,7 +1263,7 @@ is_directly_connected(in)
 	for (i = 0; i < num_interfaces; i++) {
 		/* Check that the subnetwork numbers match */
 
-		if ((in.s_addr & interfaces[i].netmask.s_addr ) == 
+		if ((in.s_addr & interfaces[i].netmask.s_addr ) ==
 		    (interfaces[i].remoteaddr.s_addr & interfaces[i].netmask.s_addr))
 			return (1);
 	}
@@ -1342,15 +1323,15 @@ age_table(time)
 	int recalculate_max = 0;
 	int max = max_preference();
 
-	tpp = &table; 
+	tpp = &table;
 	while (*tpp != NULL) {
 		tp = *tpp;
 		tp->remaining_time -= time;
 		if (tp->remaining_time <= 0) {
 			*tpp = tp->next;
-			if (tp->in_kernel) 
+			if (tp->in_kernel)
 				del_route(tp->router);
-			if (best_preference && 
+			if (best_preference &&
 			    tp->preference == max)
 				recalculate_max++;
 			free((char *)tp);
@@ -1384,9 +1365,9 @@ record_router(router, preference, ttl)
 	int old_max = max_preference();
 	int changed_up = 0;	/* max preference could have increased */
 	int changed_down = 0;	/* max preference could have decreased */
-	
+
 	if (debug)
-		logdebug("Recording %s, preference 0x%x\n", 
+		logdebug("Recording %s, preference 0x%x\n",
 			      pr_name(router),
 			      preference);
 	tp = find_router(router);
@@ -1413,7 +1394,7 @@ record_router(router, preference, ttl)
 		tp->next = table;
 		table = tp;
 	}
-	if (!tp->in_kernel && 
+	if (!tp->in_kernel &&
 	    (!best_preference || tp->preference == max_preference()) &&
 	    tp->preference != IGNORE_PREFERENCE) {
 		add_route(tp->router);
@@ -1429,7 +1410,7 @@ record_router(router, preference, ttl)
 		if (new_max != IGNORE_PREFERENCE) {
 			tp = table;
 			while (tp) {
-				if (tp->preference == new_max && 
+				if (tp->preference == new_max &&
 				    !tp->in_kernel) {
 					add_route(tp->router);
 					tp->in_kernel++;
@@ -1452,7 +1433,7 @@ record_router(router, preference, ttl)
 	}
 }
 
-	
+
 #include <net/route.h>
 
 void
@@ -1461,7 +1442,7 @@ add_route(addr)
 {
 	if (debug)
 		logdebug("Add default route to %s\n", pr_name(addr));
-	rtioctl(addr, SIOCADDRT);
+	rtioctl(addr, RTM_ADD);
 }
 
 void
@@ -1470,7 +1451,7 @@ del_route(addr)
 {
 	if (debug)
 		logdebug("Delete default route to %s\n", pr_name(addr));
-	rtioctl(addr, SIOCDELRT);
+	rtioctl(addr, RTM_DELETE);
 }
 
 void
@@ -1479,98 +1460,153 @@ rtioctl(addr, op)
 	int	op;
 {
 	int sock;
-	struct rtentry rt;
-	struct sockaddr_in *sin;
-	bzero((char *)&rt, sizeof(struct rtentry));
-	rt.rt_dst.sa_family = AF_INET;
-	rt.rt_gateway.sa_family = AF_INET;
-	sin = (struct sockaddr_in *)ALLIGN(&rt.rt_gateway);
-	sin->sin_addr = addr;
-	rt.rt_flags = RTF_UP | RTF_GATEWAY;
-	
-	sock = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
+	struct {
+		struct rt_msghdr m_rtm;
+		struct sockaddr_in m_dst;
+		struct sockaddr_in m_gateway;
+		struct sockaddr_in m_netmask;
+	} m_rtmsg;
+	static int seq = 0;
+
+	bzero(&m_rtmsg, sizeof(m_rtmsg));
+
+	m_rtmsg.m_rtm.rtm_type = op;
+	m_rtmsg.m_rtm.rtm_flags = RTF_GATEWAY | RTF_UP;		/* XXX more? */
+	m_rtmsg.m_rtm.rtm_version = RTM_VERSION;
+	m_rtmsg.m_rtm.rtm_seq = ++seq;
+	m_rtmsg.m_rtm.rtm_addrs = RTA_DST | RTA_GATEWAY | RTA_NETMASK;
+
+	/* initialise route metrics to zero.. tcp may fill these in */
+	bzero(&m_rtmsg.m_rtm.rtm_rmx, sizeof(m_rtmsg.m_rtm.rtm_rmx));
+
+	m_rtmsg.m_rtm.rtm_inits = 0;
+	m_rtmsg.m_rtm.rtm_msglen = sizeof(m_rtmsg);
+
+	m_rtmsg.m_dst.sin_len = sizeof(struct sockaddr_in);
+	m_rtmsg.m_dst.sin_family = AF_INET;
+	m_rtmsg.m_dst.sin_addr.s_addr = 0;		/* default */
+
+	/* XXX: is setting a zero length right? */
+	m_rtmsg.m_netmask.sin_len = 0;
+	m_rtmsg.m_netmask.sin_family = AF_INET;
+	m_rtmsg.m_netmask.sin_addr.s_addr = 0;		/* default */
+
+	m_rtmsg.m_gateway.sin_len = sizeof(struct sockaddr_in);
+	m_rtmsg.m_gateway.sin_family = AF_INET;
+	m_rtmsg.m_gateway.sin_addr = addr;		/* gateway */
+
+	sock = socket(PF_ROUTE, SOCK_RAW, 0);
 	if (sock < 0) {
 		logperror("rtioctl: socket");
 		return;
 	}
-	if (ioctl(sock, op, (char *)&rt) < 0) {
-		if (!(op == SIOCADDRT && errno == EEXIST))
-			logperror("ioctl (add/delete route)");
+	if (write(sock, &m_rtmsg, sizeof(m_rtmsg)) != sizeof(m_rtmsg)) {
+		if (!(op == RTM_ADD && errno == EEXIST))
+			logperror("rtioctl: write");
 	}
-	(void) close(sock);
+	close(sock);
 }
 
 
-	
+
 /*
  * LOGGER
  */
 
 
-static logging = 0;
+static int logging = 0;
 
 void
 initlog()
 {
 	logging++;
-	openlog("in.rdiscd", LOG_PID | LOG_CONS, LOG_DAEMON);
+	openlog("rdisc", LOG_PID | LOG_CONS, LOG_DAEMON);
 }
 
 /* VARARGS1 */
 void
-logerr(fmt, a,b,c,d,e,f,g,h)
+#if __STDC__
+logerr(char *fmt, ...)
+#else
+logerr(fmt, va_alist)
 	char *fmt;
+	va_dcl
+#endif
 {
+	va_list ap;
+
+#if __STDC__
+	va_start(ap, fmt);
+#else
+	va_start(ap);
+#endif
+
 	if (logging)
-		syslog(LOG_ERR, fmt, a,b,c,d,e,f,g,h);
+		vsyslog(LOG_ERR, fmt, ap);
 	else
-		(void) fprintf(stderr, fmt, a,b,c,d,e,f,g,h);
+		(void) vfprintf(stderr, fmt, ap);
+
+	va_end(ap);
 }
 
 /* VARARGS1 */
 void
-logtrace(fmt, a,b,c,d,e,f,g,h)
+#if __STDC__
+logtrace(char *fmt, ...)
+#else
+logtrace(fmt, va_alist)
 	char *fmt;
+	va_dcl
+#endif
 {
+	va_list ap;
+
+#if __STDC__
+	va_start(ap, fmt);
+#else
+	va_start(ap);
+#endif
+
 	if (logging)
-		syslog(LOG_INFO, fmt, a,b,c,d,e,f,g,h);
+		vsyslog(LOG_INFO, fmt, ap);
 	else
-		(void) fprintf(stdout, fmt, a,b,c,d,e,f,g,h);
+		(void) vfprintf(stdout, fmt, ap);
+
+	va_end(ap);
 }
 
 /* VARARGS1 */
 void
-logdebug(fmt, a,b,c,d,e,f,g,h)
+#if __STDC__
+logdebug(char *fmt, ...)
+#else
+logdebug(fmt, va_alist)
 	char *fmt;
+	va_dcl
+#endif
 {
-	if (logging)
-		syslog(LOG_DEBUG, fmt, a,b,c,d,e,f,g,h);
-	else
-		(void) fprintf(stdout, fmt, a,b,c,d,e,f,g,h);
-}
+	va_list ap;
 
-extern char *sys_errlist[];
-extern int errno;
+#if __STDC__
+	va_start(ap, fmt);
+#else
+	va_start(ap);
+#endif
+
+	if (logging)
+		vsyslog(LOG_DEBUG, fmt, ap);
+	else
+		(void) vfprintf(stdout, fmt, ap);
+
+	va_end(ap);
+}
 
 void
 logperror(str)
 	char *str;
 {
 	if (logging)
-		syslog(LOG_ERR, "%s: %s\n", str, sys_errlist[errno]);
+		syslog(LOG_ERR, "%s: %m", str);
 	else
-		(void) fprintf(stderr, "%s: %s\n", str, sys_errlist[errno]);
+		(void) warn("%s", str);
 }
-
-
-#ifdef SYSV
-void
-bzero(cp, len)
-	char *cp;
-	int len;
-{
-	while (--len>=0)
-		*cp++ = 0;
-}
-#endif
-
