@@ -2,11 +2,11 @@
  * Written By Julian ELischer
  * Copyright julian Elischer 1993.
  * Permission is granted to use or redistribute this file in any way as long
- * as this notice remains. Julian Elischer does not guarantee that this file 
- * is totally correct for any given task and users of this file must 
+ * as this notice remains. Julian Elischer does not guarantee that this file
+ * is totally correct for any given task and users of this file must
  * accept responsibility for any damage that occurs from the application of this
  * file.
- * 
+ *
  * (julian@tfs.com julian@dialix.oz.au)
  *
  * User SCSI hooks added by Peter Dufault:
@@ -14,7 +14,7 @@
  * Copyright (c) 1994 HD Associates
  * (contact: dufault@hda.com)
  * All rights reserved.
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -26,7 +26,7 @@
  * 3. The name of HD Associates
  *    may not be used to endorse or promote products derived from this software
  *    without specific prior written permission.
- * 
+ *
  * THIS SOFTWARE IS PROVIDED BY HD ASSOCIATES ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -39,7 +39,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *	$Id: scsi.c,v 1.6 1995/05/01 12:54:32 dufault Exp $
+ * $FreeBSD: stable/2.1/sbin/scsi/scsi.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #include <stdio.h>
@@ -51,6 +51,7 @@
 #include <sys/file.h>
 #include <scsi.h>
 #include <ctype.h>
+#include <signal.h>
 
 int	fd;
 int	debuglevel;
@@ -280,8 +281,8 @@ do_cmd(int fd, char *fmt, int argc, char **argv)
 	struct get_hook h;
 	scsireq_t *scsireq = scsireq_new();
 	enum data_phase data_phase;
-	int count;
-	char *data_fmt;
+	int count, amount;
+	char *data_fmt, *bp;
 
 	h.argc = argc;
 	h.argv = argv;
@@ -334,11 +335,26 @@ do_cmd(int fd, char *fmt, int argc, char **argv)
 			{
 				if (strcmp(data_fmt, "-") == 0)	/* Read data from stdin */
 				{
-					if (read(0, scsireq->databuf, count) != count)
+					bp = (char *)scsireq->databuf;
+					while (count > 0 && (amount = read(0, bp, count)) > 0)
+					{
+						count -= amount;
+						bp += amount;
+					}
+					if (amount == -1)
 					{
 						perror("read");
 						exit(errno);
 					}
+					else if (amount == 0)
+					{
+						/* early EOF */
+						fprintf(stderr,
+							"Warning: only read %lu bytes out of %lu.\n",
+							scsireq->datalen - (u_long)count,
+							scsireq->datalen);
+						scsireq->datalen -= (u_long)count;
+					}
 				}
 				else
 				{
@@ -365,11 +381,22 @@ do_cmd(int fd, char *fmt, int argc, char **argv)
 	{
 		if (strcmp(data_fmt, "-") == 0)	/* stdout */
 		{
-			if (write(1, scsireq->databuf, count) != count)
+			bp = (char *)scsireq->databuf;
+			while (count > 0 && (amount = write(1, bp, count)) > 0)
+			{
+				count -= amount;
+				bp += amount;
+			}
+			if (amount < 0)
 			{
 				perror("write");
 				exit(errno);
 			}
+			else if (amount == 0)
+				fprintf(stderr, "Warning: wrote only %d bytes out of %d.\n",
+					scsireq->datalen - count,
+					scsireq->datalen);
+			
 		}
 		else
 		{
@@ -540,7 +567,7 @@ static char *mode_lookup(int page)
 			if (c == EOF) {
 				fprintf(stderr, "Expected %c.\n", END_ENTRY);
 			}
-				
+
 			if (c == START_ENTRY) {
 				match++;
 			}
@@ -573,6 +600,7 @@ struct editinfo
 } editinfo[64];	/* XXX Bogus fixed size */
 
 static int editind;
+volatile int edit_opened;
 static FILE *edit_file;
 static char edit_name[L_tmpnam];
 
@@ -582,6 +610,30 @@ edit_rewind(void)
 	editind = 0;
 }
 
+static void
+edit_done(void)
+{
+	int opened;
+
+	sigset_t all, prev;
+	sigfillset(&all);
+
+	(void)sigprocmask(SIG_SETMASK, &all, &prev);
+
+	opened = (int)edit_opened;
+	edit_opened = 0;
+
+	(void)sigprocmask(SIG_SETMASK, &prev, 0);
+
+	if (opened)
+	{
+		if (fclose(edit_file))
+			perror(edit_name);
+		if (unlink(edit_name))
+			perror(edit_name);
+	}
+}
+
 static void
 edit_init(void)
 {
@@ -594,6 +646,9 @@ edit_init(void)
 		perror(edit_name);
 		exit(errno);
 	}
+	edit_opened = 1;
+
+	atexit(edit_done);
 }
 
 static void
@@ -757,7 +812,7 @@ mode_edit(int fd, int page, int edit, int argc, char *argv[])
 		edit_edit();
 
 		edit_rewind();
-		scsireq_buff_encode_visit(mode_pars, mh->mdl, 
+		scsireq_buff_encode_visit(mode_pars, mh->mdl,
 		fmt, edit_get, 0);
 
 		/* Eliminate block descriptors:
@@ -778,6 +833,8 @@ mode_edit(int fd, int page, int edit, int argc, char *argv[])
 		mh->mdl, fmt, arg_put, 0);
 #endif
 
+		edit_done();
+
 		/* Make it permanent if pageselect is three.
 		 */
 
@@ -884,7 +941,7 @@ void main(int argc, char **argv)
 	} else if(reprobe) {
 		scaddr.scbus = bus;
 		scaddr.target = targ;
-		scaddr.lun = lun;	
+		scaddr.lun = lun;
 
 		if (ioctl(fd,SCIOCREPROBE,&scaddr) == -1)
 			perror("ioctl");
@@ -913,4 +970,5 @@ void main(int argc, char **argv)
 	} else if (modeflag) {
 		mode_edit(fd, modepage, editflag, argc, argv);
 	}
+	exit(0);
 }
