@@ -32,7 +32,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *	$Id: crypt.c,v 1.4 1994/09/07 07:47:08 pst Exp $
+ * $FreeBSD: stable/2.1/secure/lib/libcrypt/crypt.c 50952 1999-09-05 11:54:50Z peter $
  *
  * This is an original implementation of the DES and the crypt(3) interfaces
  * by David Burren <davidb@werj.com.au>.
@@ -58,10 +58,9 @@
 #include <sys/types.h>
 #include <sys/param.h>
 #include <pwd.h>
+#include <string.h>
 
-#ifdef DEBUG
-# include <stdio.h>
-#endif
+char *crypt_md5(const char *pw, const char *salt);
 
 /* We can't always assume gcc */
 #ifdef __GNUC__
@@ -324,7 +323,7 @@ des_init()
 
 	/*
 	 * Invert the P-box permutation, and convert into OR-masks for
-	 * handling the output of the S-box arrays setup above. 
+	 * handling the output of the S-box arrays setup above.
 	 */
 	for (i = 0; i < 32; i++)
 		un_pbox[pbox[i] - 1] = i;
@@ -366,7 +365,7 @@ static int
 des_setkey(const char *key)
 {
 	u_long	k0, k1, rawkey0, rawkey1;
-	int	shifts, i, b, round;
+	int	shifts, round;
 
 	if (!des_initialised)
 		des_init();
@@ -413,14 +412,13 @@ des_setkey(const char *key)
 	shifts = 0;
 	for (round = 0; round < 16; round++) {
 		u_long	t0, t1;
-		int	bit;
 
 		shifts += key_shifts[round];
 
 		t0 = (k0 << shifts) | (k0 >> (28 - shifts));
 		t1 = (k1 << shifts) | (k1 >> (28 - shifts));
 
-		de_keysl[15 - round] = 
+		de_keysl[15 - round] =
 		en_keysl[round] = comp_maskl[0][(t0 >> 21) & 0x7f]
 				| comp_maskl[1][(t0 >> 14) & 0x7f]
 				| comp_maskl[2][(t0 >> 7) & 0x7f]
@@ -449,9 +447,9 @@ do_des(	u_long l_in, u_long r_in, u_long *l_out, u_long *r_out, int count)
 	/*
 	 *	l_in, r_in, l_out, and r_out are in pseudo-"big-endian" format.
 	 */
-	u_long	mask, rawl, rawr, l, r, *kl, *kr, *kl1, *kr1;
+	u_long	l, r, *kl, *kr, *kl1, *kr1;
 	u_long	f, r48l, r48r;
-	int	i, j, b, round;
+	int	round;
 
 	if (count == 0) {
 		return(1);
@@ -580,7 +578,6 @@ des_cipher(const char *in, char *out, long salt, int count)
 	return(retval);
 }
 
-
 char *
 crypt(char *key, char *setting)
 {
@@ -589,6 +586,9 @@ crypt(char *key, char *setting)
 	u_char		*p, *q;
 	static u_char	output[21];
 
+	if (!strncmp(setting, "$1$", 3))
+		return crypt_md5(key, setting);
+
 	if (!des_initialised)
 		des_init();
 
@@ -599,7 +599,7 @@ crypt(char *key, char *setting)
 	 */
 	q = (u_char *) keybuf;
 	while (q - (u_char *) keybuf - 8) {
-		if (*q++ = *key << 1)
+		if ((*q++ = *key << 1))
 			key++;
 	}
 	if (des_setkey((u_char *) keybuf))
