@@ -1,7 +1,7 @@
 /*
- *	$Source: /home/ncvs/src/lib/libtelnet/kerberos5.c,v $
- *	$Author: rgrimes $
- *	$Id: kerberos5.c,v 1.1.1.1 1994/05/27 05:00:20 rgrimes Exp $
+ *	$Source: /home/ncvs/src/secure/lib/libtelnet/kerberos5.c,v $
+ *	$Author: pst $
+ * $FreeBSD: stable/2.1/secure/lib/libtelnet/kerberos5.c 50952 1999-09-05 11:54:50Z peter $
  */
 
 #if !defined(lint) && !defined(SABER)
@@ -9,7 +9,7 @@ static
 #ifdef __STDC__
 const
 #endif
-char rcsid_kerberos5_c[] = "$Id: kerberos5.c,v 1.1.1.1 1994/05/27 05:00:20 rgrimes Exp $";
+char rcsid_kerberos5_c[] = "$FreeBSD: stable/2.1/secure/lib/libtelnet/kerberos5.c 50952 1999-09-05 11:54:50Z peter $";
 #endif /* lint */
 
 /*-
@@ -46,7 +46,7 @@ char rcsid_kerberos5_c[] = "$Id: kerberos5.c,v 1.1.1.1 1994/05/27 05:00:20 rgrim
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)kerberos5.c	8.2 (Berkeley) 12/15/93";
+static char sccsid[] = "@(#)kerberos5.c	8.3 (Berkeley) 5/30/95";
 #endif /* not lint */
 
 /*
@@ -84,7 +84,7 @@ static char sccsid[] = "@(#)kerberos5.c	8.2 (Berkeley) 12/15/93";
 
 /* kerberos 5 include files (ext-proto.h) will get an appropriate stdlib.h
    and string.h/strings.h */
- 
+
 #include "encrypt.h"
 #include "auth.h"
 #include "misc.h"
@@ -203,8 +203,8 @@ kerberos5_send(ap)
 	ksum.checksum_type = CKSUMTYPE_CRC32;
 	ksum.contents = sum;
 	ksum.length = sizeof(sum);
-	bzero((Voidptr )sum, sizeof(sum));
-	
+	memset((Voidptr )sum, 0, sizeof(sum));
+
         if (!UserNameRequested) {
                 if (auth_debug_mode) {
                         printf("Kerberos V5: no user name supplied\r\n");
@@ -254,9 +254,9 @@ kerberos5_send(ap)
 		krb5_free_host_realm(realms);
 		return(0);
 	}
-					 
 
-	bzero((char *)&creds, sizeof(creds));
+
+	memset((char *)&creds, 0, sizeof(creds));
 	creds.server = server;
 
 	if (r = krb5_cc_get_principal(ccache, &creds.client)) {
@@ -284,7 +284,7 @@ kerberos5_send(ap)
 	    ap_opts = AP_OPTS_MUTUAL_REQUIRED;
 	else
 	    ap_opts = 0;
-	    
+
 	r = krb5_mk_req_extended(ap_opts, &ksum, krb5_kdc_default_options, 0,
 #ifdef	ENCRYPTION
 				 &newkey,
@@ -305,12 +305,12 @@ kerberos5_send(ap)
 	    if (newkey->keytype != KEYTYPE_DES) {
 		if (creds.keyblock.keytype == KEYTYPE_DES)
 		    /* use the session key in credentials instead */
-		    memcpy((char *)session_key,
+		    memmove((char *)session_key,
 			   (char *)creds.keyblock.contents, sizeof(Block));
 		else
 		    /* XXX ? */;
 	    } else {
-		memcpy((char *)session_key, (char *)newkey->contents,
+		memmove((char *)session_key, (char *)newkey->contents,
 		       sizeof(Block));
 	    }
 	    krb5_free_keyblock(newkey);
@@ -441,7 +441,7 @@ kerberos5_is(ap, data, cnt)
 			goto errout;
 		    }
 		    Data(ap, KRB_RESPONSE, outbuf.data, outbuf.length);
-		} 
+		}
 		if (krb5_unparse_name(authdat->ticket->enc_part2 ->client,
 				      					&name))
 			name = 0;
@@ -451,19 +451,21 @@ kerberos5_is(ap, data, cnt)
 							name ? name : "");
 		}
                 auth_finished(ap, AUTH_USER);
-		
+
 		free(name);
 	    	if (authdat->authenticator->subkey &&
 		    authdat->authenticator->subkey->keytype == KEYTYPE_DES) {
-		    bcopy((Voidptr )authdat->authenticator->subkey->contents,
-			  (Voidptr )session_key, sizeof(Block));
+		    memmove((Voidptr )session_key,
+			   (Voidptr )authdat->authenticator->subkey->contents,
+			   sizeof(Block));
 		} else if (authdat->ticket->enc_part2->session->keytype ==
 			   KEYTYPE_DES) {
-		    bcopy((Voidptr )authdat->ticket->enc_part2->session->contents,
-			  (Voidptr )session_key, sizeof(Block));
+		    memmove((Voidptr )session_key,
+			(Voidptr )authdat->ticket->enc_part2->session->contents,
+			sizeof(Block));
 		} else
 		    break;
-		
+
 #ifdef ENCRYPTION
 		skey.type = SK_DES;
 		skey.length = 8;
@@ -475,17 +477,17 @@ kerberos5_is(ap, data, cnt)
 	case KRB_FORWARD:
 		inbuf.data = (char *)data;
 		inbuf.length = cnt;
-		if (r = rd_and_store_for_creds(&inbuf, authdat->ticket, 
+		if (r = rd_and_store_for_creds(&inbuf, authdat->ticket,
 					       UserNameRequested)) {
 		    char errbuf[128];
-		    
+
 		    (void) strcpy(errbuf, "Read forwarded creds failed: ");
 		    (void) strcat(errbuf, error_message(r));
 		    Data(ap, KRB_FORWARD_REJECT, errbuf, -1);
 		    if (auth_debug_mode)
 		      printf("Could not read forwarded credentials\r\n");
 		}
-		else 
+		else
 		  Data(ap, KRB_FORWARD_ACCEPT, 0, 0);
 		  if (auth_debug_mode)
 		    printf("Forwarded credentials obtained\r\n");
@@ -690,16 +692,16 @@ kerberos5_forward(ap)
     krb5_ccache ccache;
     int i;
 
-    if (!(local_creds = (krb5_creds *) 
+    if (!(local_creds = (krb5_creds *)
 	  calloc(1, sizeof(*local_creds)))) {
-	if (auth_debug_mode) 
+	if (auth_debug_mode)
 	  printf("Kerberos V5: could not allocate memory for credentials\r\n");
 	return;
     }
 
     if (r = krb5_sname_to_principal(RemoteHostName, "host", 1,
 				    &local_creds->server)) {
-	if (auth_debug_mode) 
+	if (auth_debug_mode)
 	  printf("Kerberos V5: could not build server name - %s\r\n",
 		 error_message(r));
 	krb5_free_creds(local_creds);
@@ -707,7 +709,7 @@ kerberos5_forward(ap)
     }
 
     if (r = krb5_cc_default(&ccache)) {
-	if (auth_debug_mode) 
+	if (auth_debug_mode)
 	  printf("Kerberos V5: could not get default ccache - %s\r\n",
 		 error_message(r));
 	krb5_free_creds(local_creds);
@@ -715,7 +717,7 @@ kerberos5_forward(ap)
     }
 
     if (r = krb5_cc_get_principal(ccache, &local_creds->client)) {
-	if (auth_debug_mode) 
+	if (auth_debug_mode)
 	  printf("Kerberos V5: could not get default principal - %s\r\n",
 		 error_message(r));
 	krb5_free_creds(local_creds);
@@ -724,7 +726,7 @@ kerberos5_forward(ap)
 
     /* Get ticket from credentials cache */
     if (r = krb5_get_credentials(KRB5_GC_CACHED, ccache, local_creds)) {
-	if (auth_debug_mode) 
+	if (auth_debug_mode)
 	  printf("Kerberos V5: could not obtain credentials - %s\r\n",
 		 error_message(r));
 	krb5_free_creds(local_creds);
@@ -738,13 +740,13 @@ kerberos5_forward(ap)
 			  &local_creds->keyblock,
 			  forward_flags & OPTS_FORWARDABLE_CREDS,
 			  &forw_creds)) {
-	if (auth_debug_mode) 
+	if (auth_debug_mode)
 	  printf("Kerberos V5: error getting forwarded creds - %s\r\n",
 		 error_message(r));
 	krb5_free_creds(local_creds);
 	return;
     }
-    
+
     /* Send forwarded credentials */
     if (!Data(ap, KRB_FORWARD, forw_creds.data, forw_creds.length)) {
 	if (auth_debug_mode)
