@@ -32,9 +32,12 @@
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)utility.c	8.2 (Berkeley) 12/15/93";
+static char sccsid[] = "@(#)utility.c	8.4 (Berkeley) 5/30/95";
 #endif /* not lint */
 
+#ifdef __FreeBSD__
+#include <locale.h>
+#endif
 #define PRINTOPTIONS
 #include "telnetd.h"
 
@@ -93,6 +96,7 @@ stilloob(s)
     do {
 	FD_ZERO(&excepts);
 	FD_SET(s, &excepts);
+	memset((char *)&timeout, 0, sizeof timeout);
 	value = select(s+1, (fd_set *)0, (fd_set *)0, &excepts, &timeout);
     } while ((value == -1) && (errno == EINTR));
 
@@ -220,7 +224,7 @@ netclear()
 		next = nextitem(next);
 	    } while (wewant(next) && (nfrontp > next));
 	    length = next-thisitem;
-	    bcopy(thisitem, good, length);
+	    memmove(good, thisitem, length);
 	    good += length;
 	    thisitem = next;
 	} else {
@@ -327,7 +331,7 @@ writenet(ptr, len)
 		netflush();
 	}
 
-	bcopy(ptr, nfrontp, len);
+	memmove(nfrontp, ptr, len);
 	nfrontp += len;
 
 }  /* end of writenet */
@@ -368,7 +372,7 @@ fatalperror(f, msg)
 {
 	char buf[BUFSIZ], *strerror();
 
-	(void) sprintf(buf, "%s: %s\r\n", msg, strerror(errno));
+	(void) sprintf(buf, "%s: %s", msg, strerror(errno));
 	fatal(f, buf);
 }
 
@@ -433,12 +437,16 @@ putchr(cc)
 	*putlocation++ = cc;
 }
 
+#ifdef __FreeBSD__
+static char fmtstr[] = { "%+" };
+#else
 /*
  * This is split on two lines so that SCCS will not see the M
  * between two % signs and expand it...
  */
 static char fmtstr[] = { "%l:%M\
 %P on %A, %d %B %Y" };
+#endif
 
 	void
 putf(cp, where)
@@ -449,9 +457,9 @@ putf(cp, where)
 	time_t t;
 	char db[100];
 #ifdef	STREAMSPTY
-	extern char *index();
+	extern char *strchr();
 #else
-	extern char *rindex();
+	extern char *strrchr();
 #endif
 
 	putlocation = where;
@@ -466,9 +474,9 @@ putf(cp, where)
 		case 't':
 #ifdef	STREAMSPTY
 			/* names are like /dev/pts/2 -- we want pts/2 */
-			slash = index(line+1, '/');
+			slash = strchr(line+1, '/');
 #else
-			slash = rindex(line, '/');
+			slash = strrchr(line, '/');
 #endif
 			if (slash == (char *) 0)
 				putstr(line);
@@ -481,6 +489,9 @@ putf(cp, where)
 			break;
 
 		case 'd':
+#ifdef __FreeBSD__
+			setlocale(LC_TIME, "");
+#endif
 			(void)time(&t);
 			(void)strftime(db, sizeof(db), fmtstr, localtime(&t));
 			putstr(db);
@@ -713,7 +724,7 @@ printsub(direction, pointer, length)
 		    break;
 		}
 		break;
-		
+
 	    case LM_SLC:
 		sprintf(nfrontp, "SLC");
 		nfrontp += strlen(nfrontp);
@@ -863,7 +874,7 @@ printsub(direction, pointer, length)
 			nfrontp += strlen(nfrontp);
 
 			break;
-				
+
 		    default:
 			sprintf(nfrontp, " %d", pointer[i]);
 			nfrontp += strlen(nfrontp);
@@ -966,7 +977,7 @@ printsub(direction, pointer, length)
 	case TELOPT_AUTHENTICATION:
 	    sprintf(nfrontp, "AUTHENTICATION");
 	    nfrontp += strlen(nfrontp);
-	
+
 	    if (length < 2) {
 		sprintf(nfrontp, " (empty suboption??\?)");
 		nfrontp += strlen(nfrontp);
@@ -1171,13 +1182,13 @@ printdata(tag, ptr, cnt)
 		nfrontp += strlen(nfrontp);
 		for (i = 0; i < 20 && cnt; i++) {
 			sprintf(nfrontp, "%02x", *ptr);
-			nfrontp += strlen(nfrontp); 
+			nfrontp += strlen(nfrontp);
 			if (isprint(*ptr)) {
 				xbuf[i] = *ptr;
 			} else {
 				xbuf[i] = '.';
 			}
-			if (i % 2) { 
+			if (i % 2) {
 				*nfrontp = ' ';
 				nfrontp++;
 			}
@@ -1187,6 +1198,6 @@ printdata(tag, ptr, cnt)
 		xbuf[i] = '\0';
 		sprintf(nfrontp, " %s\r\n", xbuf );
 		nfrontp += strlen(nfrontp);
-	} 
+	}
 }
 #endif /* DIAGNOSTICS */
