@@ -1,6 +1,6 @@
-static char     nic38_id[] = "@(#)$Id: nic3008.c,v 1.4 1995/03/28 07:54:31 bde Exp $";
+static char     nic38_id[] = "@(#)$FreeBSD: stable/2.1/sys/gnu/i386/isa/nic3008.c 50950 1999-09-05 08:39:22Z peter $";
 /*******************************************************************************
- *  II - Version 0.1 $Revision: 1.4 $   $State: Exp $
+ *  II - Version 0.1 $Revision: 1.7 $   $State: Exp $
  *
  * Copyright 1994 Dietmar Friede
  *******************************************************************************
@@ -10,6 +10,16 @@ static char     nic38_id[] = "@(#)$Id: nic3008.c,v 1.4 1995/03/28 07:54:31 bde E
  *
  *******************************************************************************
  * $Log: nic3008.c,v $
+ * Revision 1.7  1995/09/08  11:06:46  bde
+ * Fix benign type mismatches in devsw functions.  82 out of 299 devsw
+ * functions were wrong.
+ *
+ * Revision 1.6  1995/05/30  07:57:57  rgrimes
+ * Remove trailing whitespace.
+ *
+ * Revision 1.5  1995/05/11  19:25:55  rgrimes
+ * Fix -Wformat warnings from LINT kernel.
+ *
  * Revision 1.4  1995/03/28  07:54:31  bde
  * Add and move declarations to fix all of the warnings from `gcc -Wimplicit'
  * (except in netccitt, netiso and netns) that I didn't notice when I fixed
@@ -86,7 +96,7 @@ int             nic_connect(), nic_listen(), nic_disconnect(), nic_accept();
 int             nic_output();
 extern void isdn_start_out();
 
-static void     s_intr(), reset_req(), reset_card(); 
+static void     s_intr(), reset_req(), reset_card();
 static int      cstrcmp(), discon_req(), reset_plci(), sel_b2_prot_req();
 
 static short    bsintr;
@@ -545,7 +555,7 @@ reset_card(struct nic_softc * sc)
  * We forbid all but first open
  */
 int
-nicopen(dev_t dev, int flag)
+nicopen(dev_t dev, int flags, int fmt, struct proc *p)
 {
 	struct nic_softc *sc;
 	u_char          unit;
@@ -576,7 +586,7 @@ nicopen(dev_t dev, int flag)
  * nicclose() Close device
  */
 int
-nicclose(dev_t dev, int flag)
+nicclose(dev_t dev, int flags, int fmt, struct proc *p)
 {
 	struct nic_softc *sc = &nic_sc[minor(dev)];
 
@@ -585,7 +595,7 @@ nicclose(dev_t dev, int flag)
 }
 
 int
-nicioctl(dev_t dev, int cmd, caddr_t data, int flag)
+nicioctl(dev_t dev, int cmd, caddr_t data, int flags, struct proc *p)
 {
 	int             error;
 	u_char          unit;
