@@ -3,31 +3,32 @@
  *
  * Copyright (c) 1994 UKAI, Fumitoshi.
  * Copyright (c) 1994-1995 by HOSOKAWA, Tatsumi <hosokawa@mt.cs.keio.ac.jp>
+ * Copyright (c) 1996 Nate Williams <nate@FreeBSD.org>
  *
  * This software may be used, modified, copied, and distributed, in
  * both source and binary form provided that the above copyright and
- * these terms are retained. Under no circumstances is the author 
- * responsible for the proper functioning of this software, nor does 
- * the author assume any responsibility for damages incurred with its 
+ * these terms are retained. Under no circumstances is the author
+ * responsible for the proper functioning of this software, nor does
+ * the author assume any responsibility for damages incurred with its
  * use.
  *
  * Sep, 1994	Implemented on FreeBSD 1.1.5.1R (Toshiba AVS001WD)
  *
- *	$Id: apm.c,v 1.10 1994/12/16 07:31:47 phk Exp $
+ * $FreeBSD: stable/2.1/sys/i386/apm/apm.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include "apm.h"
 
-#if NAPM > 0
+#if NAPM > 1
+#error only one APM device may be configured
+#endif
 
-#ifdef __FreeBSD__
 #include <sys/param.h>
-#include "conf.h"
+#include <sys/conf.h>
 #include <sys/kernel.h>
 #include <sys/systm.h>
 #include <sys/malloc.h>
 #include <sys/ioctl.h>
-#include <sys/tty.h>
 #include <sys/file.h>
 #include <sys/proc.h>
 #include <sys/vnode.h>
@@ -37,75 +38,67 @@
 #include <machine/segments.h>
 #include <machine/clock.h>
 #include <vm/vm.h>
+#include <vm/vm_param.h>
+#include <vm/pmap.h>
 #include <sys/syslog.h>
+#include <sys/devconf.h>
 #include "apm_setup.h"
-#endif /* __FreeBSD__ */
 
-#ifdef MACH_KERNEL
-#include <mach/std_types.h>
-#include <sys/types.h>
-#include <sys/time.h>
-#include <device/conf.h>
-#include <device/errno.h>
-#include <device/tty.h>
-#include <device/io_req.h>
-#include <kern/time_out.h>
-
-#include <i386/ipl.h>
-#include <i386/pio.h>
-#include <i386/seg.h>
-#include <i386/machspl.h>
-#include <chips/busses.h>
-#include <i386at/apm_bios.h>
-#include <i386at/apm_setup.h>
-#include <i386at/apm_segments.h>
-#endif /* MACH_KERNEL */
+static int apm_display_off __P((void));
+static int apm_int __P((u_long *eax, u_long *ebx, u_long *ecx));
+static void apm_resume __P((void));
 
 /* static data */
 struct apm_softc {
-	int	initialized, active, halt_cpu;
+	int	initialized, active;
+	int	always_halt_cpu, slow_idle_cpu;
+	int	disabled, disengaged;
 	u_int	minorversion, majorversion;
 	u_int	cs32_base, cs16_base, ds_base;
 	u_int	cs_limit, ds_limit;
 	u_int	cs_entry;
 	u_int	intversion;
-	int	idle_cpu, disabled, disengaged;
 	struct apmhook sc_suspend;
 	struct apmhook sc_resume;
-}; 
+};
 
-static struct apm_softc apm_softc[NAPM];
-static struct apm_softc *master_softc = NULL; 	/* XXX */
-struct apmhook	*hook[NAPM_HOOK];		/* XXX */
-#ifdef APM_SLOWSTART
-int		apm_slowstart = 0;
-int		apm_ss_cnt = 0;
-static int	apm_slowstart_p = 0;
-int		apm_slowstart_stat = 0;
-#endif /* APM_SLOWSTART */
+static struct kern_devconf kdc_apm = {
+	0, 0, 0,		/* filled in by dev_attach */
+	"apm", 0, { MDDT_ISA, 0 },
+	isa_generic_externalize, 0, 0, ISA_EXTERNALLEN,
+	&kdc_isa0,		/* parent */
+	0,			/* parentdata */
+	DC_UNCONFIGURED,	/* state */
+	"Advanced Power Management BIOS",
+	DC_CLS_MISC		/* class */
+};
 
-#ifdef MACH_KERNEL
-extern struct fake_descriptor     gdt[GDTSZ];
-extern void fix_desc(struct fake_descriptor *, int);
-#endif /* MACH_KERNEL */
+
+static struct apm_softc apm_softc;
+static struct apmhook	*hook[NAPM_HOOK];		/* XXX */
 
 #define is_enabled(foo) ((foo) ? "enabled" : "disabled")
 
 /* Map version number to integer (keeps ordering of version numbers) */
 #define INTVERSION(major, minor)	((major)*100 + (minor))
 
-#ifdef __FreeBSD__
 static timeout_t apm_timeout;
-#endif /* __FreeBSD__ */
-#ifdef MACH_KERNEL
-static void apm_timeout(void *);
-#endif /* MACH_KERNEL */
+
+static void
+apm_registerdev(struct isa_device *id)
+{
+	if (kdc_apm.kdc_isa)
+		return;
+	kdc_apm.kdc_state = DC_UNCONFIGURED;
+	kdc_apm.kdc_unit = 0;
+	kdc_apm.kdc_isa = id;
+	dev_attach(&kdc_apm);
+}
 
 /* setup APM GDT discriptors */
-static void 
-setup_apm_gdt(u_int code32_base, u_int code16_base, u_int data_base, u_int code_limit, u_int data_limit) 
+static void
+setup_apm_gdt(u_int code32_base, u_int code16_base, u_int data_base, u_int code_limit, u_int data_limit)
 {
-#ifdef __FreeBSD__
 	/* setup 32bit code segment */
 	gdt_segs[GAPMCODE32_SEL].ssd_base  = code32_base;
 	gdt_segs[GAPMCODE32_SEL].ssd_limit = code_limit;
@@ -122,68 +115,42 @@ setup_apm_gdt(u_int code32_base, u_int code16_base, u_int data_base, u_int code_
 	ssdtosd(gdt_segs + GAPMCODE32_SEL, &gdt[GAPMCODE32_SEL].sd);
 	ssdtosd(gdt_segs + GAPMCODE16_SEL, &gdt[GAPMCODE16_SEL].sd);
 	ssdtosd(gdt_segs + GAPMDATA_SEL  , &gdt[GAPMDATA_SEL  ].sd);
-#endif /* __FreeBSD__ */
-#ifdef MACH_KERNEL
-	/* setup 32bit code segment */
-	gdt[sel_idx(GAPMCODE32_SEL)].offset       = code32_base;
-	gdt[sel_idx(GAPMCODE32_SEL)].lim_or_seg   = code_limit;
-	gdt[sel_idx(GAPMCODE32_SEL)].size_or_wdct = SZ_32;
-	gdt[sel_idx(GAPMCODE32_SEL)].access       = ACC_P | ACC_PL_K | ACC_CODE_R;
-
-	/* setup 16bit code segment */
-	gdt[sel_idx(GAPMCODE16_SEL)].offset       = code16_base;
-	gdt[sel_idx(GAPMCODE16_SEL)].lim_or_seg   = code_limit;
-	gdt[sel_idx(GAPMCODE16_SEL)].size_or_wdct = 0;
-	gdt[sel_idx(GAPMCODE16_SEL)].access       = ACC_P | ACC_PL_K | ACC_CODE_R;
-
-	/* setup data segment */
-	gdt[sel_idx(GAPMDATA_SEL  )].offset       = data_base;
-	gdt[sel_idx(GAPMDATA_SEL  )].lim_or_seg   = data_limit;
-	gdt[sel_idx(GAPMDATA_SEL  )].size_or_wdct = 0;
-	gdt[sel_idx(GAPMDATA_SEL  )].access       = ACC_P | ACC_PL_K | ACC_DATA_W;
-
-	/* reflect these changes on physical GDT */
-	fix_desc(gdt + sel_idx(GAPMCODE32_SEL), 1);
-	fix_desc(gdt + sel_idx(GAPMCODE16_SEL), 1);
-	fix_desc(gdt + sel_idx(GAPMDATA_SEL)  , 1);
-#endif /* MACH_KERNEL */
 }
 
 /* 48bit far pointer */
-struct addr48 {
+static struct addr48 {
 	u_long		offset;
 	u_short		segment;
 } apm_addr;
 
-int apm_errno;
+static int apm_errno;
 
 inline
 int
 apm_int(u_long *eax, u_long *ebx, u_long *ecx)
 {
 	u_long cf;
-	__asm ("pushl	%%ebp
-		pushl	%%edx
-		pushl	%%esi
-		xorl	%3,%3
-		movl	%3,%%esi
+	__asm __volatile("
+		pushfl
+		cli
 		lcall	_apm_addr
+		movl	$0, %3
 		jnc	1f
 		incl	%3
-	1:	
-		popl	%%esi
-		popl	%%edx
-		popl	%%ebp"
+	1:
+		popfl
+		"
 		: "=a" (*eax), "=b" (*ebx), "=c" (*ecx), "=D" (cf)
 		: "0" (*eax),  "1" (*ebx),  "2" (*ecx)
-		);		
+		: "dx", "si", "memory"
+		);
 	apm_errno = ((*eax) >> 8) & 0xff;
 	return cf;
 }
 
 
 /* enable/disable power management */
-static int 
+static int
 apm_enable_disable_pm(struct apm_softc *sc, int enable)
 {
 	u_long eax, ebx, ecx;
@@ -203,54 +170,35 @@ apm_enable_disable_pm(struct apm_softc *sc, int enable)
 static void
 apm_driver_version(void)
 {
-	u_long eax, ebx, ecx, i;
-
-#ifdef APM_DEBUG
-	eax = (APM_BIOS<<8) | APM_INSTCHECK;
-	ebx  = 0x0;
-	ecx  = 0x0101;
-	i = apm_int(&eax, &ebx, &ecx);
-	printf("[%04lx %04lx %04lx %ld %02x]\n",
-		eax, ebx, ecx, i, apm_errno);
-#endif
+	u_long eax, ebx, ecx;
 
 	eax = (APM_BIOS << 8) | APM_DRVVERSION;
 	ebx  = 0x0;
 	ecx  = 0x0101;
-	if(!apm_int(&eax, &ebx, &ecx)) 
+	if(!apm_int(&eax, &ebx, &ecx))
 		apm_version = eax & 0xffff;
-
-#ifdef APM_DEBUG
-	eax = (APM_BIOS << 8) | APM_INSTCHECK;
-	ebx  = 0x0;
-	ecx  = 0x0101;
-	i = apm_int(&eax, &ebx, &ecx);
-	printf("[%04lx %04lx %04lx %ld %02x]\n",
-		eax, ebx, ecx, i, apm_errno);
-#endif
 }
 
 /* engage/disengage power management (APM 1.1 or later) */
-static int 
+static int
 apm_engage_disengage_pm(struct apm_softc *sc, int engage)
 {
-	u_long eax, ebx, ecx, i;
+	u_long eax, ebx, ecx;
 
 	eax = (APM_BIOS << 8) | APM_ENGAGEDISENGAGEPM;
 	ebx = PMDV_ALLDEV;
 	ecx = engage;
-	i = apm_int(&eax, &ebx, &ecx);
-	return i;
+	return(apm_int(&eax, &ebx, &ecx));
 }
 
 /* get PM event */
-static u_int 
+static u_int
 apm_getevent(struct apm_softc *sc)
 {
 	u_long eax, ebx, ecx;
 
 	eax = (APM_BIOS << 8) | APM_GETPMEVENT;
-	
+
 	ebx = 0;
 	ecx = 0;
 	if (apm_int(&eax, &ebx, &ecx))
@@ -260,23 +208,20 @@ apm_getevent(struct apm_softc *sc)
 }
 
 /* suspend entire system */
-static int 
+static int
 apm_suspend_system(struct apm_softc *sc)
 {
 	u_long eax, ebx, ecx;
-	
+
 	eax = (APM_BIOS << 8) | APM_SETPWSTATE;
 	ebx = PMDV_ALLDEV;
 	ecx = PMST_SUSPEND;
 
-	__asm("cli");
 	if (apm_int(&eax, &ebx, &ecx)) {
-		__asm("sti");
-		printf("Entire system suspend failure: errcode = %ld\n", 
+		printf("Entire system suspend failure: errcode = %ld\n",
 			0xff & (eax >> 8));
 		return 1;
 	}
-	__asm("sti");
 	return 0;
 }
 
@@ -286,16 +231,16 @@ apm_suspend_system(struct apm_softc *sc)
  * If your laptop can control the display via APM, please inform me.
  *                            HOSOKAWA, Tatsumi <hosokawa@mt.cs.keio.ac.jp>
  */
-int
+static int
 apm_display_off(void)
 {
 	u_long eax, ebx, ecx;
-	
+
 	eax = (APM_BIOS << 8) | APM_SETPWSTATE;
 	ebx = PMDV_2NDSTORAGE0;
 	ecx = PMST_STANDBY;
 	if (apm_int(&eax, &ebx, &ecx)) {
-		printf("Display off failure: errcode = %ld\n", 
+		printf("Display off failure: errcode = %ld\n",
 			0xff & (eax >> 8));
 		return 1;
 	}
@@ -304,7 +249,7 @@ apm_display_off(void)
 }
 
 /* APM Battery low handler */
-static void 
+static void
 apm_battery_low(struct apm_softc *sc)
 {
 	printf("\007\007 * * * BATTERY IS LOW * * * \007\007");
@@ -317,7 +262,7 @@ apm_add_hook(struct apmhook **list, struct apmhook *ah)
 	int s;
 	struct apmhook *p, *prev;
 
-#if 0
+#ifdef APM_DEBUG
 	printf("Add hook \"%s\"\n", ah->ah_name);
 #endif
 
@@ -343,7 +288,7 @@ apm_add_hook(struct apmhook **list, struct apmhook *ah)
 	return ah;
 }
 
-static void 
+static void
 apm_del_hook(struct apmhook **list, struct apmhook *ah)
 {
 	int s;
@@ -370,19 +315,19 @@ nosuchnode:
 
 
 /* APM driver calls some functions automatically */
-static void     
+static void
 apm_execute_hook(struct apmhook *list)
 {
 	struct apmhook *p;
 
 	for (p = list; p != NULL; p = p->ah_next) {
-#if 0
+#ifdef APM_DEBUG
 		printf("Execute APM hook \"%s.\"\n", p->ah_name);
 #endif
 		if ((*(p->ah_fun))(p->ah_arg)) {
 			printf("Warning: APM hook \"%s\" failed", p->ah_name);
-		}       
-	}       
+		}
+	}
 }
 
 
@@ -410,10 +355,9 @@ apm_hook_disestablish(int apmh, struct apmhook *ah)
 static struct timeval suspend_time;
 static struct timeval diff_time;
 
-static int 
-apm_default_resume(struct apm_softc *sc)
+static int
+apm_default_resume(void *arg)
 {
-#ifdef __FreeBSD__
 	int pl;
 	u_int second, minute, hour;
 	struct timeval resume_time, tmp_time;
@@ -433,14 +377,12 @@ apm_default_resume(struct apm_softc *sc)
 	second %= 60;
 	log(LOG_NOTICE, "resumed from suspended mode (slept %02d:%02d:%02d)\n",
 		hour, minute, second);
-#endif /* __FreeBSD__ */
 	return 0;
 }
 
-static int 
-apm_default_suspend(void)
+static int
+apm_default_suspend(void *arg)
 {
-#ifdef __FreeBSD__
 	int	pl;
 
 	pl = splsoftclock();
@@ -449,10 +391,6 @@ apm_default_suspend(void)
 	microtime(&suspend_time);
 	timevalsub(&diff_time, &suspend_time);
 	splx(pl);
-#if 0
-	printf("diff_time = %d:%d\n", diff_time.tv_sec, diff_time.tv_usec);
-#endif
-#endif /* __FreeBSD__ */
 	return 0;
 }
 
@@ -465,12 +403,11 @@ static void apm_processevent(struct apm_softc *);
  *
  */
 
-void 
+void
 apm_suspend(void)
 {
-	struct apm_softc *sc;
+	struct apm_softc *sc = &apm_softc;
 
-	sc = master_softc;      /* XXX */
 	if (!sc)
 		return;
 
@@ -484,9 +421,8 @@ apm_suspend(void)
 void
 apm_resume(void)
 {
-	struct apm_softc *sc;
+	struct apm_softc *sc = &apm_softc;
 
-	sc = master_softc;      /* XXX */
 	if (!sc)
 		return;
 
@@ -514,23 +450,27 @@ apm_get_info(struct apm_softc *sc, apm_info_t aip)
 	aip->ai_batt_life = ecx & 0xff;
 	aip->ai_major     = (u_int)sc->majorversion;
 	aip->ai_minor     = (u_int)sc->minorversion;
+	aip->ai_status    = (u_int)sc->active;
+
 	return 0;
 }
 
 
 /* inform APM BIOS that CPU is idle */
-void 
+void
 apm_cpu_idle(void)
 {
-	struct apm_softc *sc = master_softc;    /* XXX */
+	struct apm_softc *sc = &apm_softc;
 
-	if (sc->idle_cpu) {
-		if (sc->active) {
-			__asm ("movw $0x5305, %ax; lcall _apm_addr");
-		}
+	if (sc->active) {
+		u_long eax, ebx, ecx;
+
+		eax = (APM_BIOS <<8) | APM_CPUIDLE;
+		ecx = ebx = 0;
+		apm_int(&eax, &ebx, &ecx);
 	}
 	/*
-	 * Some APM implementation halts CPU in BIOS, whenever 
+	 * Some APM implementation halts CPU in BIOS, whenever
 	 * "CPU-idle" function are invoked, but swtch() of
 	 * FreeBSD halts CPU, therefore, CPU is halted twice
 	 * in the sched loop. It makes the interrupt latency
@@ -539,25 +479,28 @@ apm_cpu_idle(void)
 	 * "hlt" operation from swtch() and managed it under
 	 * APM driver.
 	 */
-	/*
-	 * UKAI Note: on NetBSD, idle() called from cpu_switch()
-	 * doesn't halt CPU, so halt_cpu may not need on NetBSD/i386
-	 * or only "sti" operation would be needed.
-	 */
-
-	if (!sc->active || sc->halt_cpu) {
-		__asm("sti ; hlt");	/* wait for interrupt */
+	if (!sc->active || sc->always_halt_cpu) {
+		__asm("hlt");	/* wait for interrupt */
 	}
 }
 
 /* inform APM BIOS that CPU is busy */
-void 
+void
 apm_cpu_busy(void)
 {
-	struct apm_softc *sc = master_softc;	/* XXX */
+	struct apm_softc *sc = &apm_softc;
 
-	if (sc->idle_cpu && sc->active) {
-		__asm("movw $0x5306, %ax; lcall _apm_addr");
+	/*
+	 * The APM specification says this is only necessary if your BIOS
+	 * slows down the processor in the idle task, otherwise it's not
+	 * necessary.
+	 */
+	if (sc->slow_idle_cpu && sc->active) {
+		u_long eax, ebx, ecx;
+
+		eax = (APM_BIOS <<8) | APM_CPUBUSY;
+		ecx = ebx = 0;
+		apm_int(&eax, &ebx, &ecx);
 	}
 }
 
@@ -568,17 +511,19 @@ apm_cpu_busy(void)
  * This routine is automatically called by timer once per second.
  */
 
-static void 
+static void
 apm_timeout(void *arg)
 {
 	struct apm_softc *sc = arg;
 
 	apm_processevent(sc);
-	timeout(apm_timeout, (void *)sc, hz - 1 );  /* More than 1 Hz */
+	if (sc->active == 1) {
+		timeout(apm_timeout, (void *)sc, hz - 1 );  /* More than 1 Hz */
+	}
 }
 
 /* enable APM BIOS */
-static void 
+static void
 apm_event_enable(struct apm_softc *sc)
 {
 #ifdef APM_DEBUG
@@ -591,7 +536,7 @@ apm_event_enable(struct apm_softc *sc)
 }
 
 /* disable APM BIOS */
-static void 
+static void
 apm_event_disable(struct apm_softc *sc)
 {
 #ifdef APM_DEBUG
@@ -604,46 +549,28 @@ apm_event_disable(struct apm_softc *sc)
 }
 
 /* halt CPU in scheduling loop */
-static void 
+static void
 apm_halt_cpu(struct apm_softc *sc)
 {
 	if (sc->initialized) {
-		sc->halt_cpu = 1;
+		sc->always_halt_cpu = 1;
 	}
-#ifdef APM_SLOWSTART
-	apm_slowstart = 0;
-#endif /* APM_SLOWSTART */
 }
 
 /* don't halt CPU in scheduling loop */
-static void 
+static void
 apm_not_halt_cpu(struct apm_softc *sc)
 {
 	if (sc->initialized) {
-		sc->halt_cpu = 0;
+		sc->always_halt_cpu = 0;
 	}
-#ifdef APM_SLOWSTART
-	apm_slowstart = apm_slowstart_p;
-#endif /* APM_SLOWSTART */
 }
 
 /* device driver definitions */
-#ifdef __FreeBSD__
-int apmprobe (struct isa_device *);
-int apmattach(struct isa_device *);
+static int apmprobe (struct isa_device *);
+static int apmattach(struct isa_device *);
 struct isa_driver apmdriver = {
 	apmprobe, apmattach, "apm" };
-#endif /* __FreeBSD__ */
-
-#ifdef MACH_KERNEL
-int apmprobe(vm_offset_t, struct bus_ctlr *);
-void apmattach(struct bus_device *);
-static struct bus_device *apminfo[NAPM];
-static vm_offset_t apmstd[NAPM] = { 0 };
-struct bus_driver apmdriver = {
-	apmprobe, 0, apmattach, 0, apmstd, "apm", apminfo, 0, 0, 0};
-#endif /* MACH_KERNEL */
-
 
 /*
  * probe APM (dummy):
@@ -654,33 +581,28 @@ struct bus_driver apmdriver = {
  * to use V86 mode in APM initialization.
  */
 
-int 
-#ifdef __FreeBSD__
-	apmprobe(struct isa_device *dvp)
-#endif /* __FreeBSD__ */
-#ifdef MACH_KERNEL
-	apmprobe(vm_offset_t port, struct bus_ctlr *devc)
-#endif /* MACH_KERNEL */
+static int
+apmprobe(struct isa_device *dvp)
 {
-#ifdef __FreeBSD__
-	int     unit = dvp->id_unit;
-#endif /* __FreeBSD__ */
-#ifdef MACH_KERNEL
-	int     unit = devc->unit;
-#endif /* MACH_KERNEL */
-	struct apm_softc *sc = &apm_softc[unit];
-
+	if ( dvp->id_unit > 0 ) {
+		printf("apm: Only one APM driver supported.\n");
+		return 0;
+	}
+	apm_registerdev(dvp);
 	switch (apm_version) {
 	case APMINI_CANTFIND:
 		/* silent */
 		return 0;
 	case APMINI_NOT32BIT:
-		printf("apm%d: 32bit connection is not supported.\n", unit);
+		printf("apm: 32bit connection is not supported.\n");
 		return 0;
 	case APMINI_CONNECTERR:
-		printf("apm%d: 32-bit connection error.\n", unit);
+		printf("apm: 32-bit connection error.\n");
 		return 0;
 	}
+#ifdef APM_BROKEN_STATCLOCK
+	statclock_disable = 1;
+#endif
 
 	return -1;
 }
@@ -694,14 +616,12 @@ apm_processevent(struct apm_softc *sc)
 
 #ifdef APM_DEBUG
 #  define OPMEV_DEBUGMESSAGE(symbol) case symbol: \
-	printf("Original APM Event: " #symbol "\n");
+	printf("Received APM Event: " #symbol "\n");
 #else
 #  define OPMEV_DEBUGMESSAGE(symbol) case symbol:
 #endif
-	while (1) {
+	do {
 		apm_event = apm_getevent(sc);
-		if (apm_event == PMEV_NOEVENT) 
-			break;
 		switch (apm_event) {
 		    OPMEV_DEBUGMESSAGE(PMEV_STANDBYREQ);
 			apm_suspend();
@@ -728,72 +648,43 @@ apm_processevent(struct apm_softc *sc)
 			apm_battery_low(sc);
 			apm_suspend();
 			break;
-
 		    OPMEV_DEBUGMESSAGE(PMEV_POWERSTATECHANGE);
 			break;
-
 		    OPMEV_DEBUGMESSAGE(PMEV_UPDATETIME);
 			inittodr(0);	/* adjust time to RTC */
 			break;
-
+		    OPMEV_DEBUGMESSAGE(PMEV_NOEVENT);
+			break;
 		    default:
 			printf("Unknown Original APM Event 0x%x\n", apm_event);
 			    break;
 		}
-	}
+	} while (apm_event != PMEV_NOEVENT);
 }
 
 /*
  * Attach APM:
  *
  * Initialize APM driver (APM BIOS itself has been initialized in locore.s)
- *
- * Now, unless I'm mad, (not quite ruled out yet), the APM-1.1 spec is bogus:
- * 
- * Appendix C says under the header "APM 1.0/APM 1.1 Modal BIOS Behavior"
- * that "When an APM Driver connects with an APM 1.1 BIOS, the APM 1.1 BIOS
- * will default to an APM 1.0 connection.  After an APM Driver calls the APM
- * Driver Version function, specifying that it supports APM 1.1, and [sic!]
- * APM BIOS will change its behavior to an APM 1.1 connection.  If the APM
- * BIOS is an APM 1.0 BIOS, the APM Driver Version function call will fail,
- * and the connection will remain an APM 1.0 connection."
- *
- * OK so I can establish a 1.0 connection, and then tell that I'm a 1.1
- * and maybe then the BIOS will tell that it too is a 1.1.
- * Fine.
- * Now how will I ever get the segment-limits for instance ?  There is no 
- * way I can see that I can get a 1.1 response back from an "APM Protected 
- * Mode 32-bit Interface Connect" function ???
- * 
- * Who made this,  Intel and Microsoft ?  -- How did you guess !
- *
- * /phk
  */
 
-#ifdef __FreeBSD__
-int 
+static int
 apmattach(struct isa_device *dvp)
-#endif /* __FreeBSD__ */
-#ifdef MACH_KERNEL
-void 
-apmattach(struct bus_device *dvp)
-#endif /* MACH_KERNEL */
 {
-#ifdef __FreeBSD__
-	int	unit = dvp->id_unit;
 #define APM_KERNBASE	KERNBASE
-#endif /* __FreeBSD__ */
-#ifdef MACH_KERNEL
-	int	unit = dvp->unit;
-#define APM_KERNBASE	VM_MIN_KERNEL_ADDRESS
-#endif /* MACH_KERNEL */
-	struct apm_softc	*sc = &apm_softc[unit];
-	int	i;
+	struct apm_softc	*sc = &apm_softc;
+#ifdef APM_DSVALUE_BUG
+	caddr_t apm_bios_work;
+
+	apm_bios_work = (caddr_t)malloc(apm_ds_limit, M_DEVBUF, M_NOWAIT);
+	bcopy((caddr_t)((apm_ds_base << 4) + APM_KERNBASE), apm_bios_work,
+		apm_ds_limit);
+#endif /* APM_DSVALUE_BUG */
 
-	master_softc = sc;	/* XXX */
 	sc->initialized = 0;
+
+	/* Must be externally enabled */
 	sc->active = 0;
-	sc->halt_cpu = 1;
 
 	/* setup APM parameters */
 	sc->cs16_base = (apm_cs32_base << 4) + APM_KERNBASE;
@@ -803,75 +694,92 @@ apmattach(struct bus_device *dvp)
 	sc->ds_limit = apm_ds_limit;
 	sc->cs_entry = apm_cs_entry;
 
-	sc->idle_cpu = ((apm_flags & APM_CPUIDLE_SLOW) != 0);
+#ifdef APM_DSVALUE_BUG
+	sc->ds_base = (u_int)apm_bios_work;
+#endif /* APM_DSVALUE_BUG */
+
+	/* Always call HLT in idle loop */
+	sc->always_halt_cpu = 1;
+
+	sc->slow_idle_cpu = ((apm_flags & APM_CPUIDLE_SLOW) != 0);
 	sc->disabled = ((apm_flags & APM_DISABLED) != 0);
 	sc->disengaged = ((apm_flags & APM_DISENGAGED) != 0);
 
-#ifdef APM_SLOWSTART
-	if (sc->idle_cpu) {
-		apm_slowstart = apm_slowstart_p = 1;
-	}
-#endif 
-
 	/* print bootstrap messages */
 #ifdef APM_DEBUG
-	printf(" found APM BIOS version %04x\n",  apm_version);
-	printf("apm%d: Code32 0x%08x, Code16 0x%08x, Data 0x%08x\n",
-		unit, sc->cs32_base, sc->cs16_base, sc->ds_base);
-	printf("apm%d: Code entry 0x%08x, Idling CPU %s, Management %s\n",
-		unit, sc->cs_entry, is_enabled(sc->idle_cpu), 
+	printf("apm: APM BIOS version %04x\n",  apm_version);
+	printf("apm: Code32 0x%08x, Code16 0x%08x, Data 0x%08x\n",
+		sc->cs32_base, sc->cs16_base, sc->ds_base);
+	printf("apm: Code entry 0x%08x, Idling CPU %s, Management %s\n",
+		sc->cs_entry, is_enabled(sc->slow_idle_cpu),
 		is_enabled(!sc->disabled));
-	printf("apm%d: CS_limit=%x, DS_limit=%x\n",
-		unit, sc->cs_limit, sc->ds_limit);
+	printf("apm: CS_limit=%x, DS_limit=%x\n", sc->cs_limit, sc->ds_limit);
 #endif /* APM_DEBUG */
 
+#if 0
+	/* Workaround for some buggy APM BIOS implementations */
 	sc->cs_limit = 0xffff;
 	sc->ds_limit = 0xffff;
+#endif
 
 	/* setup GDT */
-	setup_apm_gdt(sc->cs32_base, sc->cs16_base, sc->ds_base, 
+	setup_apm_gdt(sc->cs32_base, sc->cs16_base, sc->ds_base,
 			sc->cs_limit, sc->ds_limit);
 
 	/* setup entry point 48bit pointer */
-#ifdef __FreeBSD__
 	apm_addr.segment = GSEL(GAPMCODE32_SEL, SEL_KPL);
-#endif /* __FreeBSD__ */
-#ifdef MACH_KERNEL
-	apm_addr.segment = GAPMCODE32_SEL;
-#endif /* MACH_KERNEL */
 	apm_addr.offset  = sc->cs_entry;
 
-	/* Try to kick bios into 1.1 mode */
+#ifdef FORCE_APM10
+	apm_version = 0x100;
+	sc->majorversion = 1;
+	sc->minorversion = 0;
+	sc->intversion = INTVERSION(sc->majorversion, sc->minorversion);
+	printf("apm: running in APM 1.0 compatible mode\n");
+	kcd_apm.kdc_description =
+		"Advanced Power Management BIOS (1.0 compatability mode)",
+#else
+	/* Try to kick bios into 1.1 or greater mode */
 	apm_driver_version();
 	sc->minorversion = ((apm_version & 0x00f0) >>  4) * 10 +
 			((apm_version & 0x000f) >> 0);
-	sc->majorversion = ((apm_version & 0xf000) >> 12) * 10 + 
+	sc->majorversion = ((apm_version & 0xf000) >> 12) * 10 +
 			((apm_version & 0x0f00) >> 8);
 
 	sc->intversion = INTVERSION(sc->majorversion, sc->minorversion);
 
 	if (sc->intversion >= INTVERSION(1, 1)) {
-		printf("apm%d: Engaged control %s\n",
-			unit, is_enabled(!sc->disengaged));
+#ifdef APM_DEBUG
+		printf("apm: Engaged control %s\n", is_enabled(!sc->disengaged));
+#endif
 	}
 
-	printf(" found APM BIOS version %d.%d\n", 
+	printf("apm: found APM BIOS version %d.%d\n",
 		sc->majorversion, sc->minorversion);
-	printf("apm%d: Idling CPU %s\n", unit, is_enabled(sc->idle_cpu));
+#endif /* FORCE_APM10 */
+
+#ifdef APM_DEBUG
+	printf("apm: Slow Idling CPU %s\n", is_enabled(sc->slow_idle_cpu));
+#endif
 
 	/* enable power management */
 	if (sc->disabled) {
 		if (apm_enable_disable_pm(sc, 1)) {
-			printf("Warning: APM enable function failed! [%x]\n", 
+#ifdef APM_DEBUG
+			printf("apm: *Warning* enable function failed! [%x]\n",
 				apm_errno);
+#endif
 		}
 	}
 
 	/* engage power managment (APM 1.1 or later) */
 	if (sc->intversion >= INTVERSION(1, 1) && sc->disengaged) {
 		if (apm_engage_disengage_pm(sc, 1)) {
-			printf("Warning: APM engage function failed [%x]\n", 
+#ifdef APM_DEBUG
+			printf("apm: *Warning* engage function failed err=[%x]",
 				apm_errno);
+			printf(" (Docked or using external power?).\n");
+#endif
 		}
 	}
 
@@ -891,55 +799,48 @@ apmattach(struct bus_device *dvp)
         apm_hook_establish(APM_HOOK_RESUME , &sc->sc_resume);
 
 	apm_event_enable(sc);
+	kdc_apm.kdc_state = DC_IDLE;
 
 	sc->initialized = 1;
 
-#ifdef __FreeBSD__
 	return 0;
-#endif /* __FreeBSD__ */
 }
 
-#ifdef __FreeBSD__
-int 
-apmopen(dev_t dev, int flag, int fmt, struct proc *p) 
+int
+apmopen(dev_t dev, int flag, int fmt, struct proc *p)
 {
-	struct apm_softc *sc = &apm_softc[minor(dev)];
+	struct apm_softc *sc = &apm_softc;
 
-	if (minor(dev) >= NAPM) {
+	if (minor(dev) != 0 || !sc->initialized)
 		return (ENXIO);
-	}
-	if (!sc->initialized) {
-		return ENXIO;
-	}
+
 	return 0;
 }
 
-int 
+int
 apmclose(dev_t dev, int flag, int fmt, struct proc *p)
 {
 	return 0;
 }
 
-int 
+int
 apmioctl(dev_t dev, int cmd, caddr_t addr, int flag, struct proc *p)
 {
-	struct apm_softc *sc = &apm_softc[minor(dev)];
+	struct apm_softc *sc = &apm_softc;
 	int error = 0;
-	int pl;
 
+	if (minor(dev) != 0 || !sc->initialized)
+		return (ENXIO);
 #ifdef APM_DEBUG
-	printf("APM ioctl: minor = %d, cmd = 0x%x\n", minor(dev), cmd);
+	printf("APM ioctl: cmd = 0x%x\n", cmd);
 #endif
-
-	if (minor(dev) >= NAPM) {
-		return ENXIO;
-	}
-	if (!sc->initialized) {
-		return ENXIO;
-	}
 	switch (cmd) {
 	case APMIO_SUSPEND:
-		apm_suspend();
+		if ( sc->active) {
+			apm_suspend();
+		} else {
+			error = EINVAL;
+		}
 		break;
 	case APMIO_GETINFO:
 		if (apm_get_info(sc, (apm_info_t)addr)) {
@@ -947,9 +848,11 @@ apmioctl(dev_t dev, int cmd, caddr_t addr, int flag, struct proc *p)
 		}
 		break;
 	case APMIO_ENABLE:
+		kdc_apm.kdc_state = DC_BUSY;
 		apm_event_enable(sc);
 		break;
 	case APMIO_DISABLE:
+		kdc_apm.kdc_state = DC_IDLE;
 		apm_event_disable(sc);
 		break;
 	case APMIO_HALTCPU:
@@ -969,29 +872,3 @@ apmioctl(dev_t dev, int cmd, caddr_t addr, int flag, struct proc *p)
 	}
 	return error;
 }
-#endif /* __FreeBSD__ */
-
-#ifdef MACH_KERNEL
-io_return_t apmopen(dev_t dev, int flag, io_req_t ior)
-{
-	int result;
-
-	result = ENXIO;
-	return result;
-}
-
-io_return_t apmclose(dev_t dev, int flag)
-{
-	return 0;
-}
-
-io_return_t apmgetstat(dev_t dev, int flavor, int *data, u_int *count)
-{
-}
-
-io_return_t apmsetstat(dev_t dev, int flavor, int *data, u_int count)
-{
-}
-#endif /* MACH_KERNEL */
-
-#endif /* NAPM > 0 */
