@@ -42,7 +42,7 @@
  * SUCH DAMAGE.
  *
  *	from: @(#)conf.c	5.8 (Berkeley) 5/12/91
- *	$Id: conf.c,v 1.83 1995/04/14 15:13:26 dufault Exp $
+ * $FreeBSD: stable/2.1/sys/i386/i386/conf.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include <sys/param.h>
@@ -81,7 +81,7 @@ d_strategy_t swstrategy;
 #define nxstop		(d_stop_t *)enxio
 #define nxreset		(d_reset_t *)enxio
 #define nxselect	(d_select_t *)enxio
-#define nxmmap		(d_mmap_t *)enxio
+#define nxmmap		nommap		/* must return -1, not ENXIO */
 #define	nxdevtotty	(d_ttycv_t *)nullop
 
 #define nullopen	(d_open_t *)nullop
@@ -208,6 +208,42 @@ d_ioctl_t	stioctl;
 #define	stsize		zerosize
 #endif
 
+#include "od.h"
+#if NOD > 0
+d_open_t	odopen;
+d_close_t	odclose;
+d_strategy_t	odstrategy;
+d_ioctl_t	odioctl;
+#define	oddump	nxdump
+#else
+#define	odopen		nxopen
+#define	odclose		nxclose
+#define	odstrategy	nxstrategy
+#define	odioctl		nxioctl
+#define	oddump		nxdump
+#endif
+
+#include "ccd.h"
+#if NCCD > 0
+d_open_t        ccdopen;
+d_close_t       ccdclose;
+d_strategy_t    ccdstrategy;
+d_ioctl_t       ccdioctl;
+d_psize_t       ccdsize;
+d_read_t        ccdread;
+d_write_t       ccdwrite;
+#define	ccddump	nxdump
+#else
+#define	ccdopen		nxopen
+#define	ccdclose	nxclose
+#define	ccdstrategy	nxstrategy
+#define	ccdioctl	nxioctl
+#define	ccddump		nxdump
+#define	ccdsize		zerosize
+#define ccdread		nxread
+#define ccdwrite	nxwrite
+#endif
+
 #include "cd.h"
 #if NCD > 0
 d_open_t	cdopen;
@@ -294,6 +330,19 @@ d_psize_t	atasize;
 #define	atadump		nxdump
 #endif
 
+#include "wcd.h"
+#if NWCD > 0
+d_open_t	wcdopen;
+d_close_t	wcdclose;
+d_strategy_t	wcdstrategy;
+d_ioctl_t	wcdioctl;
+#else
+#define wcdopen		nxopen
+#define wcdclose	nxclose
+#define wcdstrategy	nxstrategy
+#define wcdioctl	nxioctl
+#endif
+
 #include "ch.h"
 #if NCH > 0
 d_open_t	chopen;
@@ -356,6 +405,38 @@ d_psize_t	vnsize;
 #define	vnsize		zerosize
 #endif
 
+/* Matrox Meteor capture card */
+#include "meteor.h"
+#if     NMETEOR > 0
+d_open_t        meteor_open; 
+d_close_t       meteor_close;
+d_read_t        meteor_read;
+d_write_t       meteor_write;
+d_ioctl_t       meteor_ioctl;
+d_mmap_t        meteor_mmap;
+#else 
+#define meteor_open     nxopen
+#define meteor_close    nxclose 
+#define meteor_read     nxread
+#define meteor_write    nxwrite
+#define meteor_ioctl    nxioctl
+#define meteor_mmap     nxmmap
+#endif
+
+/* Connectix QuickCam camera */
+#include "qcam.h"
+#if     NQCAM > 0
+d_open_t        qcam_open; 
+d_close_t       qcam_close;
+d_read_t        qcam_read;
+d_ioctl_t       qcam_ioctl;
+#else 
+#define qcam_open     nxopen
+#define qcam_close    nxclose 
+#define qcam_read     nxread
+#define qcam_ioctl    nxioctl
+#endif
+
 #define swopen		noopen
 #define swclose		noclose
 #define swioctl		noioc
@@ -404,7 +485,14 @@ struct bdevsw	bdevsw[] =
 	{ matcdopen,	matcdclose,	matcdstrategy,	matcdioctl,	/*17*/
 	  matcddump,	matcdsize,	0 },
 	{ ataopen,	ataclose,	atastrategy,	ataioctl,	/*18*/
-	  atadump,	atasize,	0 }
+	  atadump,	atasize,	0 },
+	{ wcdopen,      wcdclose,       wcdstrategy,    wcdioctl,       /*19*/
+	  nxdump,       zerosize,       0 },
+	{ odopen,	odclose,	odstrategy,	odioctl,	/*20*/
+	  oddump,	zerosize,	0 },
+	{ ccdopen,	ccdclose,	ccdstrategy,	ccdioctl,	/*21*/
+	  ccddump,	ccdsize,	0 },
+
 /*
  * If you need a bdev major number for a driver that you intend to donate
  * back to the group or release publically, please contact the FreeBSD team
@@ -422,9 +510,9 @@ int	nblkdev = sizeof (bdevsw) / sizeof (bdevsw[0]);
 #include "sc.h"
 #include "vt.h"
 #if NSC > 0
-# if NVT > 0
+# if NVT > 0 && !defined(LINT)
 #  error "sc0 and vt0 are mutually exclusive"
-# endif /* NVT > 0 */
+# endif
 d_open_t	scopen;
 d_close_t	scclose;
 d_rdwr_t	scread, scwrite;
@@ -460,6 +548,7 @@ d_open_t	mmopen;
 d_close_t	mmclose;
 d_rdwr_t	mmrw;
 d_mmap_t	memmmap;
+d_ioctl_t	mmioctl;
 #define	mmselect	seltrue
 
 #include "pty.h"
@@ -522,15 +611,15 @@ d_select_t	logselect;
 d_open_t	bquopen;
 d_close_t	bquclose;
 d_rdwr_t	bquread, bquwrite;
-d_select_t	bquselect; 
+d_select_t	bquselect;
 d_ioctl_t	bquioctl;
-#else 
-#define bquopen         nxopen 
+#else
+#define bquopen         nxopen
 #define bquclose        nxclose
 #define bquread         nxread
 #define bquwrite        nxwrite
 #define bquselect       nxselect
-#define bquioctl        nxioctl 
+#define bquioctl        nxioctl
 #endif
 
 #include "lpt.h"
@@ -593,22 +682,6 @@ d_select_t	sndselect;
 #define sndselect       seltrue
 #endif
 
-#include "vat_audio.h"		/* BSD audio driver emulator for voxware */
-#if     NVAT_AUDIO > 0		/* not general purpose, just vat */
-d_open_t	vaopen;
-d_close_t	vaclose;
-d_ioctl_t	vaioctl;
-d_rdwr_t	varead, vawrite;
-d_select_t	vaselect;
-#else
-#define vaopen          nxopen
-#define vaclose         nxclose
-#define vaioctl         nxioctl
-#define varead          nxread
-#define vawrite         nxwrite
-#define vaselect        seltrue
-#endif
-
 /* /dev/fd/NNN */
 d_open_t fdopen;
 
@@ -805,8 +878,8 @@ d_ttycv_t	cxdevtotty;
 d_open_t	gpopen;
 d_close_t	gpclose;
 d_rdwr_t	gpwrite;
-d_ioctl_t	gpioctl; 
-#else   
+d_ioctl_t	gpioctl;
+#else
 #define gpopen  	nxopen
 #define gpclose 	nxclose
 #define gpwrite 	nxwrite
@@ -818,14 +891,31 @@ d_ioctl_t	gpioctl;
 d_open_t	gscopen;
 d_close_t	gscclose;
 d_rdwr_t	gscread;
-d_ioctl_t	gscioctl; 
-#else   
+d_ioctl_t	gscioctl;
+#else
 #define gscopen		nxopen
 #define gscclose	nxclose
 #define gscread		nxread
 #define gscioctl	nxioctl
 #endif
 
+#include "crd.h"
+#if NCRD > 0
+d_open_t	crdopen;
+d_close_t	crdclose;
+d_rdwr_t	crdread, crdwrite;
+d_ioctl_t	crdioctl;
+d_select_t	crdselect;
+#else
+#define crdopen		nxopen
+#define crdclose	nxclose
+#define crdread		nxread
+#define crdwrite	nxwrite
+#define	crdioctl	nxioctl
+#define crdselect	nxselect
+
+#endif
+
 #include "joy.h"
 #if NJOY > 0
 d_open_t	joyopen;
@@ -839,6 +929,21 @@ d_ioctl_t	joyioctl;
 #define	joyioctl	nxioctl
 #endif
 
+#include "asc.h"
+#if NASC > 0
+d_open_t      ascopen;
+d_close_t     ascclose;
+d_rdwr_t      ascread;
+d_ioctl_t     ascioctl;
+d_select_t    ascselect;
+#else
+#define ascopen               nxopen
+#define ascclose      nxclose
+#define ascread               nxread
+#define ascioctl      nxioctl
+#define ascselect       nxselect
+#endif
+
 #include "tun.h"
 #if NTUN > 0
 d_open_t	tunopen;
@@ -855,8 +960,8 @@ d_select_t	tunselect;
 #define tunselect       nxselect
 #endif
 
-#include "spigot.h"     
-#if     NSPIGOT > 0     
+#include "spigot.h"
+#if     NSPIGOT > 0
 d_open_t        spigot_open;
 d_close_t       spigot_close;
 d_ioctl_t       spigot_ioctl;
@@ -864,12 +969,12 @@ d_rdwr_t        spigot_read, spigot_write;
 d_select_t      spigot_select;
 d_mmap_t        spigot_mmap;
 #else
-#define spigot_open     nxopen     
-#define spigot_close    nxclose     
-#define spigot_ioctl    nxioctl     
-#define spigot_read     nxread     
-#define spigot_write    nxwrite     
-#define spigot_select   seltrue     
+#define spigot_open     nxopen
+#define spigot_close    nxclose
+#define spigot_ioctl    nxioctl
+#define spigot_read     nxread
+#define spigot_write    nxwrite
+#define spigot_select   seltrue
 #define spigot_mmap     nommap
 #endif
 
@@ -899,6 +1004,49 @@ d_ttycv_t	cydevtotty;
 #define	cydevtotty	nxdevtotty
 #endif
 
+#include "dgb.h"      
+#if NDGB > 0
+d_open_t		dgbopen;     
+d_close_t		dgbclose;   
+d_rdwr_t		dgbread;
+d_rdwr_t		dgbwrite; 
+d_ioctl_t		dgbioctl;   
+d_stop_t		dgbstop;     
+#define	dgbreset	nxreset
+d_ttycv_t		dgbdevtotty;
+#else
+#define dgbopen		nxopen
+#define dgbclose	nxclose
+#define dgbread		nxread
+#define dgbwrite	nxwrite
+#define dgbioctl	nxioctl
+#define dgbstop		nxstop
+#define dgbreset	nxreset
+#define dgbdevtotty	nxdevtotty
+#endif
+
+/* Specialix serial driver */
+#include "si.h"
+#if	NSI > 0
+d_open_t        siopen;
+d_close_t       siclose;
+d_read_t        siread;
+d_write_t       siwrite;
+d_ioctl_t	siioctl;
+d_stop_t        sistop;
+d_ttycv_t	sidevtotty;
+#define sireset	nxreset
+#else
+#define	siopen		nxopen
+#define siclose		nxclose
+#define siread		nxread
+#define siwrite		nxwrite
+#define siioctl		nxioctl
+#define sistop		nxstop
+#define sireset		nxreset
+#define	sidevtotty	nxdevtotty
+#endif
+
 #include "ity.h"
 #if NITY > 0
 d_open_t	ityopen;
@@ -1016,6 +1164,59 @@ d_ioctl_t    labpcioctl;
 #define	labpcioctl		nxioctl
 #endif
 
+/* Stallion Multiport Serial Driver (cd1400 based) */
+#include "stl.h"
+#if NSTL > 0
+d_open_t	stlopen;
+d_close_t	stlclose;
+d_read_t	stlread;
+d_write_t	stlwrite;
+d_ioctl_t	stlioctl;
+d_stop_t	stlstop;
+d_ttycv_t	stldevtotty;
+#define stlreset	nxreset
+#define stlmmap		nxmmap
+#define stlstrategy	nxstrategy
+#else
+#define stlopen		nxopen
+#define stlclose	nxclose
+#define stlread		nxread
+#define stlwrite	nxwrite
+#define stlioctl	nxioctl
+#define stlstop		nxstop
+#define stlreset	nxreset
+#define stlmmap		nxmmap
+#define stlstrategy	nxstrategy
+#define stldevtotty	nxdevtotty
+#endif
+
+/* Stallion Intelligent Multiport Serial Driver */
+#include "stli.h"
+#if NSTLI > 0
+d_open_t	stliopen;
+d_close_t	stliclose;
+d_read_t	stliread;
+d_write_t	stliwrite;
+d_ioctl_t	stliioctl;
+d_stop_t	stlistop;
+d_ttycv_t	stlidevtotty;
+#define stlireset	nxreset
+#define stlimmap	nxmmap
+#define stlistrategy	nxstrategy
+#else
+#define stliopen	nxopen
+#define stliclose	nxclose
+#define stliread	nxread
+#define stliwrite	nxwrite
+#define stliioctl	nxioctl
+#define stlistop	nxstop
+#define stlireset	nxreset
+#define stlimmap	nxmmap
+#define stlistrategy	nxstrategy
+#define stlidevtotty	nxdevtotty
+#endif
+
+
 /* open, close, read, write, ioctl, stop, reset, ttys, select, mmap, strat */
 struct cdevsw	cdevsw[] =
 {
@@ -1026,7 +1227,7 @@ struct cdevsw	cdevsw[] =
 	  cttyioctl,	nullstop,	nullreset,	nodevtotty,/* tty */
 	  cttyselect,	nommap,		NULL },
 	{ mmopen,	mmclose,	mmrw,		mmrw,		/*2*/
-	  noioc,	nullstop,	nullreset,	nodevtotty,/* memory */
+	  mmioctl,	nullstop,	nullreset,	nodevtotty,/* memory */
 	  mmselect,	memmmap,	NULL },
 	{ wdopen,	wdclose,	rawread,	rawwrite,	/*3*/
 	  wdioctl,	nostop,		nullreset,	nodevtotty,/* wd */
@@ -1087,10 +1288,9 @@ struct cdevsw	cdevsw[] =
  * Otherwise, simply use the one reserved for local use.
  */
 	/* character device 20 is reserved for local use */
-	{ nxopen, nxclose, nxread,	/*20*/
-	  nxwrite, nxioctl, nxstop,
-	  nxreset, nxdevtotty, nxselect,
-	  nxmmap, NULL },
+	{ nxopen,	nxclose,	nxread,		nxwrite,	/*20*/
+	  nxioctl,	nxstop,		nxreset,	nxdevtotty,/* reserved */
+	  nxselect,	nxmmap,		NULL },
 	{ psmopen,	psmclose,	psmread,	nowrite,	/*21*/
 	  psmioctl,	nostop,		nullreset,	nodevtotty,/* psm mice */
 	  psmselect,	nommap,		NULL },
@@ -1103,9 +1303,9 @@ struct cdevsw	cdevsw[] =
  	{ pcaopen,      pcaclose,       noread,         pcawrite,       /*24*/
  	  pcaioctl,     nostop,         nullreset,      nodevtotty,/* pcaudio */
  	  pcaselect,	nommap,		NULL },
-	{ vaopen,	vaclose,	varead,		vawrite,	/*25*/
-  	  vaioctl,	nostop,		nullreset,	nodevtotty,/* vat */
-  	  vaselect,	nommap,		NULL },
+	{ nxopen,	nxclose,	nxread,		nxwrite,	/*25*/
+	  nxioctl,	nxstop,		nxreset,	nxdevtotty,/* was vat */
+	  nxselect,	nxmmap,		NULL },
 	{ spkropen,     spkrclose,      noread,         spkrwrite,      /*26*/
 	  spkrioctl,    nostop,         nullreset,      nodevtotty,/* spkr */
 	  seltrue,	nommap,		NULL },
@@ -1162,7 +1362,7 @@ struct cdevsw	cdevsw[] =
 	  seltrue,	nommap,		vnstrategy },
 	{ gpopen,	gpclose,	noread,		gpwrite,	/*44*/
 	  gpioctl,	nostop,		nullreset,	nodevtotty,/* GPIB */
-          seltrue,	nommap,		NULL },                 
+          seltrue,	nommap,		NULL },
 	{ scdopen,	scdclose,	rawread,	nowrite,	/*45*/
 	  scdioctl,	nostop,		nullreset,	nodevtotty,/* sony cd */
 	  seltrue,	nommap,		scdstrategy },
@@ -1178,9 +1378,9 @@ struct cdevsw	cdevsw[] =
 	{ sscopen,	sscclose,	sscread,	sscwrite,	/*49*/
 	  sscioctl,	nostop,		nullreset,	nodevtotty,/* scsi super */
 	  sscselect,	sscmmap,	sscstrategy },
-	{ nxopen,	nxclose,	nxread,		nxwrite,	/*50*/
-	  nxioctl,	nxstop,		nxreset,	nodevtotty,/* pcmcia */
-	  nxselect,	nxmmap,		NULL },
+	{ crdopen,	crdclose,	crdread,	crdwrite,	/*50*/
+	  crdioctl,	nostop,		nullreset,	nodevtotty,/* pcmcia */
+	  crdselect,	nommap,		NULL },
 	{ joyopen,	joyclose,	joyread,	nowrite,	/*51*/
 	  joyioctl,	nostop,		nullreset,	nodevtotty,/*joystick */
 	  seltrue,	nommap,		NULL},
@@ -1202,9 +1402,9 @@ struct cdevsw	cdevsw[] =
 	{ itelopen,	itelclose,	itelread,	itelwrite,	/*57*/
 	  itelioctl,	nostop,		nullreset,	nodevtotty,/* itel */
 	  seltrue,	nommap,		NULL },
-	{ nxopen,	nxclose,	nxread,		nxwrite,	/*58*/
-	  nxioctl,	nxstop,		nxreset,	nxdevtotty,/* unused */
-	  seltrue,	nxmmap,		NULL },
+	{ dgbopen,	dgbclose,	dgbread,	dgbwrite,	/*58*/
+	  dgbioctl,	dgbstop,	dgbreset,	dgbdevtotty, /* dgb */
+	  ttselect,	nommap,		NULL },
 	{ ispyopen,	ispyclose,	ispyread,	nowrite,	/*59*/
 	  ispyioctl,	nostop,		nullreset,	nodevtotty,/* ispy */
 	  seltrue,	nommap,         NULL },
@@ -1220,16 +1420,43 @@ struct cdevsw	cdevsw[] =
 	{ rcopen,       rcclose,        rcread,         rcwrite,        /*63*/
 	  rcioctl,      rcstop,         rcreset,        rcdevtotty,/* rc */
 	  ttselect,	nommap,		NULL },
-	{ nxopen, nxclose, nxread,                                      /*64*/
-	  nxwrite, nxioctl, nxstop,				   /* Talisman*/
-	  nxreset, nxdevtotty, nxselect,
-	  nxmmap, NULL },
+	{ nxopen,	nxclose,	nxread,		nxwrite,	/*64*/
+	  nxioctl,	nxstop,		nxreset,	nxdevtotty,/* Talisman */
+	  nxselect,	nxmmap,		NULL },
 	{ sctargopen,	sctargclose,	rawread,	rawwrite,	/*65*/
 	  sctargioctl,	nostop,		nullreset,	nodevtotty,/* sctarg */
 	  seltrue,	nommap,		sctargstrategy },
 	{ labpcopen,	labpcclose,	rawread,	rawwrite,	/*66*/
 	  labpcioctl,	nostop,		nullreset,	nodevtotty,/* labpc */
 	  seltrue,	nommap,		labpcstrategy },
+        { meteor_open,  meteor_close,   meteor_read,    meteor_write,   /*67*/
+          meteor_ioctl, nostop,         nullreset,      nodevtotty,/* Meteor */
+          seltrue, meteor_mmap, NULL },
+	{ siopen,	siclose,	siread,		siwrite,	/*68*/
+	  siioctl,	sistop,		sireset,	sidevtotty,/* slxos */
+	  ttselect,	nxmmap,		NULL },
+	{ wcdopen,      wcdclose,       rawread,        nowrite,        /*69*/
+	  wcdioctl,     nostop,         nullreset,      nodevtotty,/* atapi */
+	  seltrue,      nommap,         wcdstrategy },
+	{ odopen,	odclose,	rawread,	rawwrite,	/*70*/
+	  odioctl,	nostop,		nullreset,	nodevtotty,/* od */
+	  seltrue,	nommap,		odstrategy },
+	{ ascopen,      ascclose,       ascread,        nowrite,        /*71*/
+	  ascioctl,     nostop,         nullreset,      nodevtotty, /* asc */   
+	  ascselect,    nommap,         NULL },
+	{ stlopen,	stlclose,	stlread,	stlwrite,       /*72*/
+	  stlioctl,	stlstop,	stlreset,	stldevtotty, /*stli*/
+	  ttselect,	stlmmap,	stlstrategy },
+	{ qcam_open,    qcam_close,     qcam_read,      nowrite,        /*73*/
+	  qcam_ioctl,   nostop,         nullreset,      nodevtotty, /* qcam */
+	  noselect,     nommap,         NULL },
+	{ ccdopen,	ccdclose,	ccdread,	ccdwrite,	/*74*/
+	  ccdioctl,	nostop,		nullreset,	nodevtotty, /* ccd */
+	  seltrue,	nommap,		ccdstrategy },
+	{ stliopen,	stliclose,	stliread,	stliwrite,      /*75*/
+	  stliioctl,	stlistop,	stlireset,	stlidevtotty, /*stli*/
+	  ttselect,	stlimmap,	stlistrategy },
+
 };
 int	nchrdev = sizeof (cdevsw) / sizeof (cdevsw[0]);
 
@@ -1282,21 +1509,30 @@ isdisk(dev, type)
 {
 
 	switch (major(dev)) {
-	case 15:
+	case 15:		/* VBLK: vn, VCHR: cd */
 		return (1);
-	case 0:
-	case 2:
-	case 4:
-	case 6:
-	case 7:
+	case 0:			/* wd */
+	case 2:			/* fd */
+	case 4:			/* sd */
+	case 6:			/* cd */
+	case 7:			/* mcd */
+	case 16:		/* scd */
+	case 17:		/* matcd */
+	case 18:		/* ata */
+	case 19:		/* wcd */
+	case 20:		/* od */
 		if (type == VBLK)
 			return (1);
 		return (0);
-	case 3:
-	case 9:
-	case 13:
-	case 29:
-	case 43:
+	case 3:			/* wd */
+	case 9:			/* fd */
+	case 13:		/* sd */
+	case 29:		/* mcd */
+	case 43:		/* vn */
+	case 45:		/* scd */
+	case 46:		/* matcd */
+	case 69:		/* wcd */
+	case 70:		/* od */
 		if (type == VCHR)
 			return (1);
 		/* fall through */
@@ -1318,14 +1554,18 @@ chrtoblk(dev)
 	int blkmaj;
 
 	switch (major(dev)) {
-	case 3:		blkmaj = 0;  break;
-	case 9:		blkmaj = 2;  break;
-	case 10:	blkmaj = 3;  break;
-	case 13:	blkmaj = 4;  break;
-	case 14:	blkmaj = 5;  break;
-	case 15:	blkmaj = 6;  break;
-	case 29:	blkmaj = 7;  break;
-	case 43:	blkmaj = 15; break;
+	case 3:		blkmaj = 0;  break; /* wd */
+	case 9:		blkmaj = 2;  break; /* fd */
+	case 10:	blkmaj = 3;  break; /* wt */
+	case 13:	blkmaj = 4;  break; /* sd */
+	case 14:	blkmaj = 5;  break; /* st */
+	case 15:	blkmaj = 6;  break; /* cd */
+	case 29:	blkmaj = 7;  break; /* mcd */
+	case 43:	blkmaj = 15; break; /* vn */
+	case 45:	blkmaj = 16; break; /* scd */
+	case 46:	blkmaj = 17; break; /* matcd */
+	case 69:	blkmaj = 19; break; /* wcd */
+	case 70:	blkmaj = 20; break; /* od */
 	default:
 		return (NODEV);
 	}
