@@ -35,7 +35,7 @@
  * SUCH DAMAGE.
  *
  *	from: @(#)machdep.c	7.4 (Berkeley) 6/3/91
- *	$Id: machdep.c,v 1.126 1995/05/12 19:17:14 wollman Exp $
+ * $FreeBSD: stable/2.1/sys/i386/i386/machdep.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include "npx.h"
@@ -124,9 +124,9 @@
 #include <i386/isa/isa.h>
 #include <i386/isa/isa_device.h>
 #include <i386/isa/rtc.h>
+#include <machine/random.h>
 
 static void identifycpu(void);
-static void initcpu(void);
 
 char machine[] = "i386";
 char cpu_model[128];
@@ -179,7 +179,10 @@ long dumplo;
 extern int bootdev;
 int biosmem;
 
-vm_offset_t	phys_avail[6];
+vm_offset_t phys_avail[10];
+
+/* must be 2 less so 0 0 can signal end of chunks */
+#define PHYS_AVAIL_ARRAY_END ((sizeof(phys_avail) / sizeof(vm_offset_t)) - 2)
 
 int cpu_class;
 
@@ -192,6 +195,8 @@ vm_offset_t pager_sva, pager_eva;
 extern int pager_map_size;
 extern struct linker_set netisr_set;
 
+extern void dblfault_handler __P((void));
+
 #define offsetof(type, member)	((size_t)(&((type *)0)->member))
 
 void
@@ -201,7 +206,7 @@ cpu_startup()
 	register caddr_t v;
 	vm_offset_t maxaddr;
 	vm_size_t size = 0;
-	int firstaddr;
+	int firstaddr, indx;
 	vm_offset_t minaddr;
 
 	if (boothowto & RB_VERBOSE)
@@ -224,9 +229,27 @@ cpu_startup()
 	printf(version);
 	startrtclock();
 	identifycpu();
-	printf("real memory  = %d (%d pages)\n", ptoa(physmem), physmem);
-	if (badpages)
-		printf("bad memory   = %d (%d pages)\n", ptoa(badpages), badpages);
+	printf("real memory  = %d (%dK bytes)\n", ptoa(Maxmem), ptoa(Maxmem) / 1024);
+	/*
+	 * Display any holes after the first chunk of extended memory.
+	 */
+	if (badpages != 0) {
+		int indx = 1;
+
+		/*
+		 * XXX skip reporting ISA hole & unmanaged kernel memory
+		 */
+		if (phys_avail[0] == PAGE_SIZE)
+			indx += 2;
+
+		printf("Physical memory hole(s):\n");
+		for (; phys_avail[indx + 1] != 0; indx += 2) {
+			int size = phys_avail[indx + 1] - phys_avail[indx];
+
+			printf("0x%08x - 0x%08x, %d bytes (%d pages)\n", phys_avail[indx],
+			    phys_avail[indx + 1] - 1, size, size / PAGE_SIZE);
+		}
+	}
 
 	/*
 	 * Quickly wire in netisrs.
@@ -354,9 +377,17 @@ again:
 	for (i = 1; i < ncallout; i++)
 		callout[i-1].c_next = &callout[i];
 
-        if (boothowto & RB_CONFIG)
+#if defined(USERCONFIG_BOOT) && defined(USERCONFIG)
+	boothowto |= RB_CONFIG;
+#endif
+        if (boothowto & RB_CONFIG) {
+#ifdef USERCONFIG
 		userconfig();
-	printf("avail memory = %d (%d pages)\n", ptoa(cnt.v_free_count), cnt.v_free_count);
+		cninit();	/* the preferred console may have changed */
+#else
+		printf("Sorry! no userconfig in this kernel\n");
+#endif
+	}
 
 #ifdef BOUNCE_BUFFERS
 	/*
@@ -364,11 +395,8 @@ again:
 	 */
 	vm_bounce_init();
 #endif
-
-	/*
-	 * Set up CPU-specific registers, cache, etc.
-	 */
-	initcpu();
+	printf("avail memory = %d (%dK bytes)\n", ptoa(cnt.v_free_count),
+	    ptoa(cnt.v_free_count) / 1024);
 
 	/*
 	 * Set up buffers, so they can be used to read disk labels.
@@ -380,16 +408,35 @@ again:
 	 * Configure the system.
 	 */
 	configure();
+
+	/*
+	 * In verbose mode, print out the BIOS's idea of the disk geometries.
+	 */
 	if (bootverbose) {
 		printf("BIOS Geometries:\n");
-		for (i=0; i < N_BIOS_GEOM; i++) {
-			int j = bootinfo.bi_bios_geom[i];
-			if (j == 0x4f010f)
+		for (i = 0; i < N_BIOS_GEOM; i++) {
+			unsigned long bios_geom;
+			int max_cylinder, max_head, max_sector;
+
+			bios_geom = bootinfo.bi_bios_geom[i];
+
+			/*
+			 * XXX the bootstrap punts a 1200K floppy geometry
+			 * when the get-disk-geometry interrupt fails.  Skip
+			 * drives that have this geometry.
+			 */
+			if (bios_geom == 0x4f010f)
 				continue;
-			printf(" %x:%08x", i, j);
-			printf(" %d cyl, %d heads, %d sects\n",
-				j >> 16, (j >> 8) & 0xff, j & 0xff);
-				
+
+			printf(" %x:%08x ", i, bios_geom);
+			max_cylinder = bios_geom >> 16;
+			max_head = (bios_geom >> 8) & 0xff;
+			max_sector = bios_geom & 0xff;
+			printf(
+		"0..%d=%d cylinders, 0..%d=%d heads, 1..%d=%d sectors\n",
+			       max_cylinder, max_cylinder + 1,
+			       max_head, max_head + 1,
+			       max_sector, max_sector);
 		}
 		printf(" %d accounted for\n", bootinfo.bi_n_bios_used);
 	}
@@ -421,7 +468,7 @@ static void
 identifycpu()
 {
 	printf("CPU: ");
-	if (cpu >= 0 
+	if (cpu >= 0
 	    && cpu < (sizeof i386_cpus/sizeof(struct cpu_nameclass))) {
 		cpu_class = i386_cpus[cpu].cpu_class;
 		strncpy(cpu_model, i386_cpus[cpu].cpu_name, sizeof cpu_model);
@@ -442,8 +489,8 @@ identifycpu()
 			cpu_model[0] = '\0';
 
 			switch (cpu_id & 0x3000) {
-			case 0x1000: 
-				strcpy(cpu_model, "Overdrive "); 
+			case 0x1000:
+				strcpy(cpu_model, "Overdrive ");
 				break;
 			case 0x2000:
 				strcpy(cpu_model, "Dual ");
@@ -478,7 +525,7 @@ identifycpu()
 			case 0x480:
 				strcat(cpu_model, "DX4"); break;
 #if defined(I586_CPU)
-			case 0x510: 
+			case 0x510:
 				if (pentium_mhz == 60) {
 					strcat(cpu_model, "510\\60");
 				} else if (pentium_mhz == 66) {
@@ -534,8 +581,25 @@ identifycpu()
 	if (!strcmp(cpu_vendor, "GenuineIntel")) {
 		printf("  Stepping=%ld", cpu_id & 0xf);
 		if (cpu_high > 0) {
-#define FEATUREFMT "\020\001FPU\002VME\003PSE\004MCE\005CX8\006APIC"
-			printf("\n  Features=0x%b", cpu_feature, FEATUREFMT);
+			printf("\n  Features=0x%b", cpu_feature, 
+			"\020"
+			"\001FPU"
+			"\002VME"
+			"\003DE"
+			"\004PSE"
+			"\005TSC"
+			"\006MSR"
+			"\007PAE"
+			"\010MCE"
+			"\011CX8"
+			"\012APIC"
+			"\013<b10>"
+			"\014<b11>"
+			"\015MTRR"
+			"\016PGE"
+			"\017MCA"
+			"\020CMOV"
+			);
 		}
 	}
 	/* Avoid ugly blank lines: only print newline when we have to. */
@@ -630,7 +694,7 @@ sendsig(catcher, sig, mask, code)
 		return;
 	}
 
-	/* 
+	/*
 	 * Build the argument list for the signal handler.
 	 */
 	if (p->p_sysent->sv_sigtbl) {
@@ -803,35 +867,22 @@ int	waittime = -1;
 struct pcb dumppcb;
 
 __dead void
-boot(arghowto)
-	int arghowto;
+boot(howto)
+	int howto;
 {
-	register long dummy;		/* r12 is reserved */
-	register int howto;		/* r11 == how to boot */
-	register int devtype;		/* r10 == major of root dev */
-
-	if (cold) {
-		printf("hit reset please");
-		for(;;);
-	}
-	howto = arghowto;
-	if ((howto&RB_NOSYNC) == 0 && waittime < 0) {
+	if (!cold && (howto & RB_NOSYNC) == 0 && waittime < 0) {
 		register struct buf *bp;
 		int iter, nbusy;
 
 		waittime = 0;
 		printf("\nsyncing disks... ");
-		/*
-		 * Release inodes held by texts before update.
-		 */
-		if (panicstr == 0)
-			vnode_pager_umount(NULL);
+
 		sync(&proc0, NULL, NULL);
 
 		for (iter = 0; iter < 20; iter++) {
 			nbusy = 0;
 			for (bp = &buf[nbuf]; --bp >= buf; ) {
-				if ((bp->b_flags & (B_BUSY|B_INVAL)) == B_BUSY) {
+				if ((bp->b_flags & (B_BUSY | B_INVAL)) == B_BUSY) {
 					nbusy++;
 				}
 			}
@@ -858,25 +909,26 @@ boot(arghowto)
 		dev_shutdownall(FALSE);
 	}
 	splhigh();
-	devtype = major(rootdev);
-	if (howto&RB_HALT) {
+	if (howto & RB_HALT) {
 		printf("\n");
 		printf("The operating system has halted.\n");
 		printf("Please press any key to reboot.\n\n");
 		cngetc();
 	} else {
 		if (howto & RB_DUMP) {
-			savectx(&dumppcb, 0);
-			dumppcb.pcb_ptd = rcr3();
-			dumpsys();	
+			if (!cold) {
+				savectx(&dumppcb);
+				dumppcb.pcb_ptd = rcr3();
+				dumpsys();
+			}
 
 			if (PANIC_REBOOT_WAIT_TIME != 0) {
 				if (PANIC_REBOOT_WAIT_TIME != -1) {
 					int loop;
 					printf("Automatic reboot in %d seconds - press a key on the console to abort\n",
 						PANIC_REBOOT_WAIT_TIME);
-					for (loop = PANIC_REBOOT_WAIT_TIME; loop > 0; --loop) {
-						DELAY(1000 * 1000); /* one second */
+					for (loop = PANIC_REBOOT_WAIT_TIME * 10; loop > 0; --loop) {
+						DELAY(1000 * 100); /* 1/10th second */
 						if (cncheckc()) /* Did user type a key? */
 							break;
 					}
@@ -890,10 +942,6 @@ boot(arghowto)
 			cngetc();
 		}
 	}
-#ifdef lint
-	dummy = 0; dummy = dummy;
-	printf("howto %d, devtype %d\n", arghowto, devtype);
-#endif
 die:
 	printf("Rebooting...\n");
 	DELAY(1000000);	/* wait 1 sec for printf's to complete and be read */
@@ -953,11 +1001,6 @@ dumpsys()
 	}
 }
 
-static void
-initcpu()
-{
-}
-
 /*
  * Clear registers on exec
  */
@@ -998,6 +1041,7 @@ cpu_sysctl(name, namelen, oldp, oldlenp, newp, newlen, p)
 	size_t newlen;
 	struct proc *p;
 {
+	dev_t consdev;
 	int error;
 
 	/* all sysctl names at this level are terminal */
@@ -1006,8 +1050,9 @@ cpu_sysctl(name, namelen, oldp, oldlenp, newp, newlen, p)
 
 	switch (name[0]) {
 	case CPU_CONSDEV:
-		return (sysctl_rdstruct(oldp, oldlenp, newp, &cn_tty->t_dev,
-		   sizeof cn_tty->t_dev));
+		consdev = (cn_tty == NULL ? NODEV : cn_tty->t_dev);
+		return (sysctl_rdstruct(oldp, oldlenp, newp, &consdev,
+					sizeof consdev));
 	case CPU_ADJKERNTZ:
 		error = sysctl_int(oldp, oldlenp, newp, newlen, &adjkerntz);
 		if (!error && newp)
@@ -1035,7 +1080,8 @@ union descriptor gdt[NGDT];		/* global descriptor table */
 struct gate_descriptor idt[NIDT];	/* interrupt descriptor table */
 union descriptor ldt[NLDT];		/* local descriptor table */
 
-struct	i386tss	tss, panic_tss;
+static struct i386tss dblfault_tss;
+static char dblfault_stack[PAGE_SIZE];
 
 extern  struct user *proc0paddr;
 
@@ -1087,8 +1133,8 @@ struct soft_segment_descriptor gdt_segs[] = {
 	0,			/* default 32 vs 16 bit size */
 	0  			/* limit granularity (byte/page units)*/ },
 /* GPANIC_SEL	5 Panic Tss Descriptor */
-{	(int) &panic_tss,	/* segment base address  */
-	sizeof(tss)-1,		/* length - all address space */
+{	(int) &dblfault_tss,	/* segment base address  */
+	sizeof(struct i386tss)-1,/* length - all address space */
 	SDT_SYS386TSS,		/* segment type */
 	0,			/* segment descriptor priority level */
 	1,			/* segment descriptor present */
@@ -1097,7 +1143,7 @@ struct soft_segment_descriptor gdt_segs[] = {
 	0  			/* limit granularity (byte/page units)*/ },
 /* GPROC0_SEL	6 Proc 0 Tss Descriptor */
 {	(int) kstack,		/* segment base address  */
-	sizeof(tss)-1,		/* length - all address space */
+	sizeof(struct i386tss)-1,/* length - all address space */
 	SDT_SYS386TSS,		/* segment type */
 	0,			/* segment descriptor priority level */
 	1,			/* segment descriptor present */
@@ -1191,16 +1237,17 @@ struct soft_segment_descriptor ldt_segs[] = {
 };
 
 void
-setidt(idx, func, typ, dpl)
+setidt(idx, func, typ, dpl, selec)
 	int idx;
 	inthand_t *func;
 	int typ;
 	int dpl;
+	int selec;
 {
 	struct gate_descriptor *ip = idt + idx;
 
 	ip->gd_looffset = (int)func;
-	ip->gd_selector = 8;
+	ip->gd_selector = selec;
 	ip->gd_stkcpy = 0;
 	ip->gd_xx = 0;
 	ip->gd_type = typ;
@@ -1213,7 +1260,7 @@ setidt(idx, func, typ, dpl)
 
 extern inthand_t
 	IDTVEC(div), IDTVEC(dbg), IDTVEC(nmi), IDTVEC(bpt), IDTVEC(ofl),
-	IDTVEC(bnd), IDTVEC(ill), IDTVEC(dna), IDTVEC(dble), IDTVEC(fpusegm),
+	IDTVEC(bnd), IDTVEC(ill), IDTVEC(dna), IDTVEC(fpusegm),
 	IDTVEC(tss), IDTVEC(missing), IDTVEC(stk), IDTVEC(prot),
 	IDTVEC(page), IDTVEC(rsvd), IDTVEC(fpu), IDTVEC(align),
 	IDTVEC(syscall);
@@ -1248,7 +1295,7 @@ init386(first)
 	/* table descriptors - used to load tables by microp */
 	struct region_descriptor r_gdt, r_idt;
 	int	pagesinbase, pagesinext;
-	int	target_page;
+	int	target_page, pa_indx;
 
 	proc0.p_addr = proc0paddr;
 
@@ -1299,33 +1346,34 @@ init386(first)
 
 	/* exceptions */
 	for (x = 0; x < NIDT; x++)
-		setidt(x, &IDTVEC(rsvd), SDT_SYS386TGT, SEL_KPL);
-	setidt(0, &IDTVEC(div),  SDT_SYS386TGT, SEL_KPL);
-	setidt(1, &IDTVEC(dbg),  SDT_SYS386TGT, SEL_KPL);
-	setidt(2, &IDTVEC(nmi),  SDT_SYS386TGT, SEL_KPL);
- 	setidt(3, &IDTVEC(bpt),  SDT_SYS386TGT, SEL_UPL);
-	setidt(4, &IDTVEC(ofl),  SDT_SYS386TGT, SEL_UPL);
-	setidt(5, &IDTVEC(bnd),  SDT_SYS386TGT, SEL_KPL);
-	setidt(6, &IDTVEC(ill),  SDT_SYS386TGT, SEL_KPL);
-	setidt(7, &IDTVEC(dna),  SDT_SYS386TGT, SEL_KPL);
-	setidt(8, &IDTVEC(dble),  SDT_SYS386TGT, SEL_KPL);
-	setidt(9, &IDTVEC(fpusegm),  SDT_SYS386TGT, SEL_KPL);
-	setidt(10, &IDTVEC(tss),  SDT_SYS386TGT, SEL_KPL);
-	setidt(11, &IDTVEC(missing),  SDT_SYS386TGT, SEL_KPL);
-	setidt(12, &IDTVEC(stk),  SDT_SYS386TGT, SEL_KPL);
-	setidt(13, &IDTVEC(prot),  SDT_SYS386TGT, SEL_KPL);
-	setidt(14, &IDTVEC(page),  SDT_SYS386TGT, SEL_KPL);
-	setidt(15, &IDTVEC(rsvd),  SDT_SYS386TGT, SEL_KPL);
-	setidt(16, &IDTVEC(fpu),  SDT_SYS386TGT, SEL_KPL);
-	setidt(17, &IDTVEC(align), SDT_SYS386TGT, SEL_KPL);
-#ifdef COMPAT_LINUX
- 	setidt(0x80, &IDTVEC(linux_syscall),  SDT_SYS386TGT, SEL_UPL);
+		setidt(x, &IDTVEC(rsvd), SDT_SYS386TGT, SEL_KPL, GSEL(GCODE_SEL, SEL_KPL));
+	setidt(0, &IDTVEC(div),  SDT_SYS386TGT, SEL_KPL, GSEL(GCODE_SEL, SEL_KPL));
+	setidt(1, &IDTVEC(dbg),  SDT_SYS386TGT, SEL_KPL, GSEL(GCODE_SEL, SEL_KPL));
+	setidt(2, &IDTVEC(nmi),  SDT_SYS386TGT, SEL_KPL, GSEL(GCODE_SEL, SEL_KPL));
+ 	setidt(3, &IDTVEC(bpt),  SDT_SYS386TGT, SEL_UPL, GSEL(GCODE_SEL, SEL_KPL));
+	setidt(4, &IDTVEC(ofl),  SDT_SYS386TGT, SEL_UPL, GSEL(GCODE_SEL, SEL_KPL));
+	setidt(5, &IDTVEC(bnd),  SDT_SYS386TGT, SEL_KPL, GSEL(GCODE_SEL, SEL_KPL));
+	setidt(6, &IDTVEC(ill),  SDT_SYS386TGT, SEL_KPL, GSEL(GCODE_SEL, SEL_KPL));
+	setidt(7, &IDTVEC(dna),  SDT_SYS386TGT, SEL_KPL, GSEL(GCODE_SEL, SEL_KPL));
+	setidt(8, 0,  SDT_SYSTASKGT, SEL_KPL, GSEL(GPANIC_SEL, SEL_KPL));
+	setidt(9, &IDTVEC(fpusegm),  SDT_SYS386TGT, SEL_KPL, GSEL(GCODE_SEL, SEL_KPL));
+	setidt(10, &IDTVEC(tss),  SDT_SYS386TGT, SEL_KPL, GSEL(GCODE_SEL, SEL_KPL));
+	setidt(11, &IDTVEC(missing),  SDT_SYS386TGT, SEL_KPL, GSEL(GCODE_SEL, SEL_KPL));
+	setidt(12, &IDTVEC(stk),  SDT_SYS386TGT, SEL_KPL, GSEL(GCODE_SEL, SEL_KPL));
+	setidt(13, &IDTVEC(prot),  SDT_SYS386TGT, SEL_KPL, GSEL(GCODE_SEL, SEL_KPL));
+	setidt(14, &IDTVEC(page),  SDT_SYS386TGT, SEL_KPL, GSEL(GCODE_SEL, SEL_KPL));
+	setidt(15, &IDTVEC(rsvd),  SDT_SYS386TGT, SEL_KPL, GSEL(GCODE_SEL, SEL_KPL));
+	setidt(16, &IDTVEC(fpu),  SDT_SYS386TGT, SEL_KPL, GSEL(GCODE_SEL, SEL_KPL));
+	setidt(17, &IDTVEC(align), SDT_SYS386TGT, SEL_KPL, GSEL(GCODE_SEL, SEL_KPL));
+#if defined(COMPAT_LINUX) || defined(LINUX)
+ 	setidt(0x80, &IDTVEC(linux_syscall),  SDT_SYS386TGT, SEL_UPL, GSEL(GCODE_SEL, SEL_KPL));
 #endif
-  
+
 #include	"isa.h"
 #if	NISA >0
 	isa_defaultirq();
 #endif
+	rand_initialize();
 
 	r_gdt.rd_limit = sizeof(gdt) - 1;
 	r_gdt.rd_base =  (int) gdt;
@@ -1402,36 +1450,43 @@ init386(first)
 
 	/*
 	 * Maxmem isn't the "maximum memory", it's one larger than the
-	 * highest page of of the physical address space. It should be
-	 * called something like "Maxphyspage".
+	 * highest page of of the physical address space. It
 	 */
 	Maxmem = pagesinext + 0x100000/PAGE_SIZE;
 
 #ifdef MAXMEM
 	Maxmem = MAXMEM/4;
 #endif
-	/*
-	 * Calculate number of physical pages, but account for Maxmem
-	 *	adjustment above.
-	 */
-	physmem = pagesinbase + Maxmem - 0x100000/PAGE_SIZE;
 
 	/* call pmap initialization to make new kernel address space */
 	pmap_bootstrap (first, 0);
 
 	/*
-	 * Do a quick, non-destructive check over extended memory to verify
-	 * what the BIOS tells us agrees with reality. Adjust down Maxmem
-	 * if we find that the page can't be correctly written to/read from.
+	 * Size up each available chunk of physical memory.
 	 */
 
-	for (target_page = Maxmem - 1; target_page >= atop(first); target_page--) {
-		int tmp;
+	/*
+	 * We currently don't bother testing base memory.
+	 * XXX  ...but we probably should.
+	 */
+	pa_indx = 0;
+	badpages = 0;
+	if (pagesinbase > 1) {
+		phys_avail[pa_indx++] = PAGE_SIZE;	/* skip first page of memory */
+		phys_avail[pa_indx] = ptoa(pagesinbase);/* memory up to the ISA hole */
+		physmem = pagesinbase - 1;
+	} else {
+		/* point at first chunk end */
+		pa_indx++;
+	}
+
+	for (target_page = avail_start; target_page < ptoa(Maxmem); target_page += PAGE_SIZE) {
+		int tmp, page_bad = FALSE;
 
 		/*
 		 * map page into kernel: valid, read/write, non-cacheable
 		 */
-		*(int *)CMAP1 = PG_V | PG_KW | PG_N | ptoa(target_page);
+		*(int *)CMAP1 = PG_V | PG_KW | PG_N | target_page;
 		pmap_update();
 
 		tmp = *(int *)CADDR1;
@@ -1440,27 +1495,21 @@ init386(first)
 		 */
 		*(int *)CADDR1 = 0xaaaaaaaa;
 		if (*(int *)CADDR1 != 0xaaaaaaaa) {
-			Maxmem = target_page;
-			badpages++;
-			continue;
+			page_bad = TRUE;
 		}
 		/*
 		 * Test for alternating 0's and 1's
 		 */
 		*(int *)CADDR1 = 0x55555555;
 		if (*(int *)CADDR1 != 0x55555555) {
-			Maxmem = target_page;
-			badpages++;
-			continue;
+			page_bad = TRUE;
 		}
 		/*
 		 * Test for all 1's
 		 */
 		*(int *)CADDR1 = 0xffffffff;
 		if (*(int *)CADDR1 != 0xffffffff) {
-			Maxmem = target_page;
-			badpages++;
-			continue;
+			page_bad = TRUE;
 		}
 		/*
 		 * Test for all 0's
@@ -1470,35 +1519,65 @@ init386(first)
 			/*
 			 * test of page failed
 			 */
-			Maxmem = target_page;
-			badpages++;
-			continue;
+			page_bad = TRUE;
 		}
+		/*
+		 * Restore original value.
+		 */
 		*(int *)CADDR1 = tmp;
+
+		/*
+		 * Adjust array of valid/good pages.
+		 */
+		if (page_bad == FALSE) {
+			/*
+			 * If this good page is a continuation of the
+			 * previous set of good pages, then just increase
+			 * the end pointer. Otherwise start a new chunk.
+			 * Note that "end" points one higher than end,
+			 * making the range >= start and < end.
+			 */
+			if (phys_avail[pa_indx] == target_page) {
+				phys_avail[pa_indx] += PAGE_SIZE;
+			} else {
+				pa_indx++;
+				if (pa_indx == PHYS_AVAIL_ARRAY_END) {
+					printf("Too many holes in the physical address space, giving up\n");
+					pa_indx--;
+					break;
+				}
+				phys_avail[pa_indx++] = target_page;	/* start */
+				phys_avail[pa_indx] = target_page + PAGE_SIZE;	/* end */
+			}
+			physmem++;
+		} else {
+			badpages++;
+			page_bad = FALSE;
+		}
 	}
-	if (badpages != 0)
-		printf("WARNING: BIOS extended memory size and reality don't agree.\n");
 
 	*(int *)CMAP1 = 0;
 	pmap_update();
 
-	avail_end = (Maxmem << PAGE_SHIFT)
-		    - i386_round_page(sizeof(struct msgbuf));
-
 	/*
-	 * Initialize pointers to the two chunks of memory; for use
-	 *	later in vm_page_startup.
+	 * XXX
+	 * The last chunk must contain at leat one page plus the message
+	 * buffer to avoid complicating other code (message buffer address
+	 * calculation, etc.).
 	 */
-	/* avail_start is initialized in pmap_bootstrap */
-	x = 0;
-	if (pagesinbase > 1) {
-		phys_avail[x++] = NBPG;		/* skip first page of memory */
-		phys_avail[x++] = pagesinbase * NBPG;	/* memory up to the ISA hole */
+	while (phys_avail[pa_indx - 1] + PAGE_SIZE +
+	    round_page(sizeof(struct msgbuf)) >= phys_avail[pa_indx]) {
+		physmem -= atop(phys_avail[pa_indx] - phys_avail[pa_indx - 1]);
+		phys_avail[pa_indx--] = 0;
+		phys_avail[pa_indx--] = 0;
 	}
-	phys_avail[x++] = avail_start;	/* memory up to the end */
-	phys_avail[x++] = avail_end;
-	phys_avail[x++] = 0;		/* no more chunks */
-	phys_avail[x++] = 0;
+
+	Maxmem = atop(phys_avail[pa_indx]);
+
+	/* Trim off space for the message buffer. */
+	phys_avail[pa_indx] -= round_page(sizeof(struct msgbuf));
+
+	avail_end = phys_avail[pa_indx];
 
 	/* now running on new page tables, configured,and u/iom is accessible */
 
@@ -1507,8 +1586,20 @@ init386(first)
 	proc0.p_addr->u_pcb.pcb_tss.tss_ss0 = GSEL(GDATA_SEL, SEL_KPL) ;
 	gsel_tss = GSEL(GPROC0_SEL, SEL_KPL);
 
-	((struct i386tss *)gdt_segs[GPROC0_SEL].ssd_base)->tss_ioopt = 
-		(sizeof(tss))<<16;
+	dblfault_tss.tss_esp = dblfault_tss.tss_esp0 = dblfault_tss.tss_esp1 =
+	    dblfault_tss.tss_esp2 = (int) &dblfault_stack[sizeof(dblfault_stack)];
+	dblfault_tss.tss_ss = dblfault_tss.tss_ss0 = dblfault_tss.tss_ss1 =
+	    dblfault_tss.tss_ss2 = GSEL(GDATA_SEL, SEL_KPL);
+	dblfault_tss.tss_cr3 = IdlePTD;
+	dblfault_tss.tss_eip = (int) dblfault_handler;
+	dblfault_tss.tss_eflags = PSL_KERNEL;
+	dblfault_tss.tss_ds = dblfault_tss.tss_es = dblfault_tss.tss_fs = dblfault_tss.tss_gs =
+		GSEL(GDATA_SEL, SEL_KPL);
+	dblfault_tss.tss_cs = GSEL(GCODE_SEL, SEL_KPL);
+	dblfault_tss.tss_ldt = GSEL(GLDT_SEL, SEL_KPL);
+
+	((struct i386tss *)gdt_segs[GPROC0_SEL].ssd_base)->tss_ioopt =
+		(sizeof(struct i386tss))<<16;
 
 	ltr(gsel_tss);
 
