@@ -39,7 +39,7 @@
  * SUCH DAMAGE.
  *
  *	from:	@(#)pmap.c	7.7 (Berkeley)	5/12/91
- *	$Id: pmap.c,v 1.56 1995/04/09 05:40:38 davidg Exp $
+ * $FreeBSD: stable/2.1/sys/i386/i386/pmap.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -133,7 +133,6 @@ int protection_codes[8];
 struct pmap kernel_pmap_store;
 pmap_t kernel_pmap;
 
-vm_offset_t phys_avail[6];	/* 2 entries + 1 null */
 vm_offset_t avail_start;	/* PA of first available physical page */
 vm_offset_t avail_end;		/* PA of last available physical page */
 vm_size_t mem_size;		/* memory size in bytes */
@@ -167,7 +166,7 @@ struct msgbuf *msgbufp;
 
 #endif
 
-void 
+void
 init_pv_entries(int);
 
 /*
@@ -178,7 +177,7 @@ init_pv_entries(int);
  * [ what about induced faults -wfj]
  */
 
-inline pt_entry_t * const 
+inline pt_entry_t * const
 pmap_pte(pmap, va)
 	register pmap_t pmap;
 	vm_offset_t va;
@@ -1472,6 +1471,9 @@ pmap_object_init_pt(pmap, addr, object, offset, size)
 	if (!vm_object_lock_try(object))
 		return;
 
+	if ((size + offset) > object->size)
+		size = object->size - offset;
+
 	/*
 	 * if we are processing a major portion of the object, then scan the
 	 * entire thing.
@@ -1496,10 +1498,10 @@ pmap_object_init_pt(pmap, addr, object, offset, size)
 			    (p->bmapped == 0) &&
 				(p->busy == 0) &&
 			    (p->flags & (PG_BUSY | PG_FICTITIOUS | PG_CACHE)) == 0) {
-				vm_page_hold(p);
-				p->flags |= PG_MAPPED;
+				p->flags |= PG_BUSY;
 				pmap_enter_quick(pmap, addr + tmpoff, VM_PAGE_TO_PHYS(p));
-				vm_page_unhold(p);
+				p->flags |= PG_MAPPED;
+				PAGE_WAKEUP(p);
 			}
 			objbytes -= NBPG;
 		}
@@ -1513,10 +1515,10 @@ pmap_object_init_pt(pmap, addr, object, offset, size)
 			    (p->bmapped == 0) && (p->busy == 0) &&
 			    ((p->valid & VM_PAGE_BITS_ALL) == VM_PAGE_BITS_ALL) &&
 			    (p->flags & (PG_BUSY | PG_FICTITIOUS | PG_CACHE)) == 0) {
-				vm_page_hold(p);
-				p->flags |= PG_MAPPED;
+				p->flags |= PG_BUSY;
 				pmap_enter_quick(pmap, addr + tmpoff, VM_PAGE_TO_PHYS(p));
-				vm_page_unhold(p);
+				p->flags |= PG_MAPPED;
+				PAGE_WAKEUP(p);
 			}
 		}
 	}
@@ -2041,7 +2043,7 @@ pmap_mapdev(pa, size)
 
 	for (tmpva = va; size > 0;) {
 		pte = vtopte(tmpva);
-		*pte = (pt_entry_t) ((int) (pa | PG_RW | PG_V | PG_N));
+		*pte = (pt_entry_t) ((int) (pa | PG_RW | PG_V));
 		size -= PAGE_SIZE;
 		tmpva += PAGE_SIZE;
 		pa += PAGE_SIZE;
