@@ -35,7 +35,7 @@
  * SUCH DAMAGE.
  *
  *	from: @(#)trap.c	7.4 (Berkeley) 5/13/91
- *	$Id: trap.c,v 1.50 1995/03/21 07:02:51 davidg Exp $
+ * $FreeBSD: stable/2.1/sys/i386/i386/trap.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -67,11 +67,24 @@
 #include <machine/trap.h>
 #include <machine/../isa/isa_device.h>
 
+#ifdef POWERFAIL_NMI
+# include <syslog.h>
+# include <machine/clock.h>
+#endif
+
 #include "isa.h"
 #include "npx.h"
 
-int	trap_pfault	__P((struct trapframe *, int));
-void	trap_fatal	__P((struct trapframe *));
+extern void trap __P((struct trapframe frame));
+extern int trapwrite __P((unsigned addr));
+extern void syscall __P((struct trapframe frame));
+extern void linux_syscall __P((struct trapframe frame));
+
+static int trap_pfault __P((struct trapframe *, int));
+static void trap_fatal __P((struct trapframe *));
+void dblfault_handler __P((void));
+
+extern inthand_t IDTVEC(syscall);
 
 #define MAX_TRAP_MSG		27
 char *trap_msg[] = {
@@ -105,6 +118,9 @@ char *trap_msg[] = {
 	"stack fault",				/* 27 T_STKFLT */
 };
 
+static void userret __P((struct proc *p, struct trapframe *frame,
+			 u_quad_t oticks));
+
 static inline void
 userret(p, frame, oticks)
 	struct proc *p;
@@ -145,7 +161,9 @@ userret(p, frame, oticks)
 			addupc(frame->tf_eip, &p->p_stats->p_prof,
 			    ticks * profscale);
 #else
-			addupc(frame->tf_eip, &p->p_stats->p_prof, ticks);
+			extern int psratio;
+			addupc(frame->tf_eip, &p->p_stats->p_prof,
+			    ticks * psratio);
 #endif
 		}
 	}
@@ -153,14 +171,12 @@ userret(p, frame, oticks)
 }
 
 /*
- * trap(frame):
- *	Exception, fault, and trap interface to the FreeBSD kernel.
+ * Exception, fault, and trap interface to the FreeBSD kernel.
  * This common code is called from assembly language IDT gate entry
  * routines that prepare a suitable stack frame, and restore this
  * frame after the exception has been processed.
  */
 
-/*ARGSUSED*/
 void
 trap(frame)
 	struct trapframe frame;
@@ -174,7 +190,7 @@ trap(frame)
 
 	type = frame.tf_trapno;
 	code = frame.tf_err;
-	
+
 	if (ISPL(frame.tf_cs) == SEL_UPL) {
 		/* user trap */
 
@@ -234,16 +250,20 @@ trap(frame)
 
 #if NISA > 0
 		case T_NMI:
+#ifdef POWERFAIL_NMI
+			goto handle_powerfail;
+#else /* !POWERFAIL_NMI */
 #ifdef DDB
 			/* NMI can be hooked up to a pushbutton for debugging */
 			printf ("NMI ... going to debugger\n");
 			if (kdb_trap (type, 0, &frame))
 				return;
-#endif
+#endif /* DDB */
 			/* machine/parity/power fail/"kitchen sink" faults */
 			if (isa_nmi(code) == 0) return;
 			panic("NMI indicates hardware failure");
-#endif
+#endif /* POWERFAIL_NMI */
+#endif /* NISA > 0 */
 
 		case T_OFLOW:		/* integer overflow fault */
 			ucode = FPE_INTOVF_TRAP;
@@ -262,7 +282,7 @@ trap(frame)
 				return;
 #endif	/* NNPX > 0 */
 
-#if defined(MATH_EMULATE) || defined(GPL_MATH_EMULATE) 
+#if defined(MATH_EMULATE) || defined(GPL_MATH_EMULATE)
 			i = math_emulate(&frame);
 			if (i == 0) {
 				if (!(frame.tf_eflags & PSL_T))
@@ -340,31 +360,68 @@ trap(frame)
 			}
 			break;
 
-#ifdef DDB
+		case T_TRCTRAP:	 /* trace trap */
+			if (frame.tf_eip == (int)IDTVEC(syscall)) {
+				/*
+				 * We've just entered system mode via the
+				 * syscall lcall.  Continue single stepping
+				 * silently until the syscall handler has
+				 * saved the flags.
+				 */
+				return;
+			}
+			if (frame.tf_eip == (int)IDTVEC(syscall) + 1) {
+				/*
+				 * The syscall handler has now saved the
+				 * flags.  Stop single stepping it.
+				 */
+				frame.tf_eflags &= ~PSL_T;
+				return;
+			}
+			/*
+			 * Fall through.
+			 */
 		case T_BPTFLT:
-		case T_TRCTRAP:
+			/*
+			 * If DDB is enabled, let it handle the debugger trap.
+			 * Otherwise, debugger traps "can't happen".
+			 */
+#ifdef DDB
 			if (kdb_trap (type, 0, &frame))
 				return;
-			break;
-#else
-		case T_TRCTRAP:	 /* trace trap -- someone single stepping lcall's */
-			/* Q: how do we turn it on again? */
-			frame.tf_eflags &= ~PSL_T;
-			return;
 #endif
-	
+			break;
+
 #if NISA > 0
 		case T_NMI:
+#ifdef POWERFAIL_NMI
+#ifndef TIMER_FREQ
+#  define TIMER_FREQ 1193182
+#endif
+	handle_powerfail:
+		{
+		  static unsigned lastalert = 0;
+
+		  if(time.tv_sec - lastalert > 10)
+		    {
+		      log(LOG_WARNING, "NMI: power fail\n");
+		      sysbeep(TIMER_FREQ/880, hz);
+		      lastalert = time.tv_sec;
+		    }
+		  return;
+		}
+#else /* !POWERFAIL_NMI */
 #ifdef DDB
 			/* NMI can be hooked up to a pushbutton for debugging */
 			printf ("NMI ... going to debugger\n");
 			if (kdb_trap (type, 0, &frame))
 				return;
-#endif
+#endif /* DDB */
 			/* machine/parity/power fail/"kitchen sink" faults */
 			if (isa_nmi(code) == 0) return;
 			/* FALL THROUGH */
-#endif
+#endif /* POWERFAIL_NMI */
+#endif /* NISA > 0 */
 		}
 
 		trap_fatal(&frame);
@@ -376,7 +433,7 @@ trap(frame)
 #ifdef DEBUG
 	eva = rcr2();
 	if (type <= MAX_TRAP_MSG) {
-		uprintf("fatal process exception: %s", 
+		uprintf("fatal process exception: %s",
 			trap_msg[type]);
 		if ((type == T_PAGEFLT) || (type == T_PROTFLT))
 			uprintf(", fault VA = 0x%x", eva);
@@ -421,9 +478,9 @@ trap_pfault(frame, usermode)
 		vm_offset_t v;
 		vm_page_t ptepg;
 
-		if ((p == NULL) ||
+		if (p == NULL ||
 		    (!usermode && va < VM_MAXUSER_ADDRESS &&
-		    curpcb->pcb_onfault == NULL)) {
+		    (curpcb == NULL || curpcb->pcb_onfault == NULL))) {
 			trap_fatal(frame);
 			return (-1);
 		}
@@ -464,7 +521,8 @@ trap_pfault(frame, usermode)
 		v = (vm_offset_t) vtopte(va);
 
 		/* Fault the pte only if needed: */
-		*(volatile char *)v += 0;	
+		if (*((int *)vtopte(v)) == 0)
+			(void) vm_fault(map, trunc_page(v), VM_PROT_WRITE, FALSE);
 
 		pmap_use_pt( vm_map_pmap(map), va);
 
@@ -553,7 +611,7 @@ trap_pfault(frame, usermode)
 		ftype = VM_PROT_READ;
 
 	if (map != kernel_map) {
-		vm_offset_t v = (vm_offset_t) vtopte(va);
+		vm_offset_t v;
 		vm_page_t ptepg;
 
 		/*
@@ -578,9 +636,11 @@ trap_pfault(frame, usermode)
 		 * Check if page table is mapped, if not,
 		 *	fault it first
 		 */
+		v = (vm_offset_t) vtopte(va);
 
 		/* Fault the pte only if needed: */
-		*(volatile char *)v += 0;	
+		if (*((int *)vtopte(v)) == 0)
+			(void) vm_fault(map, trunc_page(v), VM_PROT_WRITE, FALSE);
 
 		pmap_use_pt( vm_map_pmap(map), va);
 
@@ -690,6 +750,33 @@ trap_fatal(frame)
 		panic("unknown/reserved trap");
 }
 
+/*
+ * Double fault handler. Called when a fault occurs while writing
+ * a frame for a trap/exception onto the stack. This usually occurs
+ * when the stack overflows (such is the case with infinite recursion,
+ * for example).
+ *
+ * XXX Note that the current PTD gets replaced by IdlePTD when the
+ * task switch occurs. This means that the stack that was active at
+ * the time of the double fault is not available at <kstack> unless
+ * the machine was idle when the double fault occurred. The downside
+ * of this is that "trace <ebp>" in ddb won't work.
+ */
+void
+dblfault_handler()
+{
+	struct pcb *pcb = curpcb;
+
+	if (pcb != NULL) {
+		printf("\nFatal double fault:\n");
+		printf("eip = 0x%x\n", pcb->pcb_tss.tss_eip);
+		printf("esp = 0x%x\n", pcb->pcb_tss.tss_esp);
+		printf("ebp = 0x%x\n", pcb->pcb_tss.tss_ebp);
+	}
+
+	panic("double fault");
+}
+
 /*
  * Compensate for 386 brain damage (missing URKR).
  * This is a little simpler than the pagefault handler in trap() because
@@ -754,11 +841,9 @@ int trapwrite(addr)
 }
 
 /*
- * syscall(frame):
- *	System call request from POSIX system call gate interface to kernel.
+ * System call request from POSIX system call gate interface to kernel.
  * Like trap(), argument is call by reference.
  */
-/*ARGSUSED*/
 void
 syscall(frame)
 	struct trapframe frame;
@@ -768,7 +853,7 @@ syscall(frame)
 	struct sysent *callp;
 	struct proc *p = curproc;
 	u_quad_t sticks;
-	int error, opc;
+	int error;
 	int args[8], rval[2];
 	u_int code;
 
@@ -776,14 +861,9 @@ syscall(frame)
 	if (ISPL(frame.tf_cs) != SEL_UPL)
 		panic("syscall");
 
-	code = frame.tf_eax;
 	p->p_md.md_regs = (int *)&frame;
-	params = (caddr_t)frame.tf_esp + sizeof (int) ;
-
-	/*
-	 * Reconstruct pc, assuming lcall $X,y is 7 bytes, as it is always.
-	 */
-	opc = frame.tf_eip - 7;
+	params = (caddr_t)frame.tf_esp + sizeof(int);
+	code = frame.tf_eax;
 	/*
 	 * Need to check if this is a 32 bit or 64 bit syscall.
 	 */
@@ -792,25 +872,25 @@ syscall(frame)
 		 * Code is first argument, followed by actual args.
 		 */
 		code = fuword(params);
-		params += sizeof (int);
+		params += sizeof(int);
 	} else if (code == SYS___syscall) {
 		/*
 		 * Like syscall, but code is a quad, so as to maintain
 		 * quad alignment for the rest of the arguments.
 		 */
-		code = fuword(params + _QUAD_LOWWORD * sizeof(int));
+		code = fuword(params);
 		params += sizeof(quad_t);
 	}
 
  	if (p->p_sysent->sv_mask)
- 		code = code & p->p_sysent->sv_mask;
- 
+ 		code &= p->p_sysent->sv_mask;
+
  	if (code >= p->p_sysent->sv_size)
  		callp = &p->p_sysent->sv_table[0];
   	else
  		callp = &p->p_sysent->sv_table[code];
 
-	if ((i = callp->sy_narg * sizeof (int)) &&
+	if ((i = callp->sy_narg * sizeof(int)) &&
 	    (error = copyin(params, (caddr_t)args, (u_int)i))) {
 #ifdef KTRACE
 		if (KTRPOINT(p, KTR_SYSCALL))
@@ -837,28 +917,37 @@ syscall(frame)
 		p = curproc;
 		frame.tf_eax = rval[0];
 		frame.tf_edx = rval[1];
-		frame.tf_eflags &= ~PSL_C;	/* carry bit */
+		frame.tf_eflags &= ~PSL_C;
 		break;
 
 	case ERESTART:
-		frame.tf_eip = opc;
+		/*
+		 * Reconstruct pc, assuming lcall $X,y is 7 bytes.
+		 */
+		frame.tf_eip -= 7;
 		break;
 
 	case EJUSTRETURN:
 		break;
 
 	default:
-	bad:
+bad:
  		if (p->p_sysent->sv_errsize)
  			if (error >= p->p_sysent->sv_errsize)
   				error = -1;	/* XXX */
-   			else 
+   			else
   				error = p->p_sysent->sv_errtbl[error];
 		frame.tf_eax = error;
-		frame.tf_eflags |= PSL_C;	/* carry bit */
+		frame.tf_eflags |= PSL_C;
 		break;
 	}
 
+	if (frame.tf_eflags & PSL_T) {
+		/* Traced syscall. */
+		frame.tf_eflags &= ~PSL_T;
+		trapsignal(p, SIGTRAP, 0);
+	}
+
 	userret(p, &frame, sticks);
 
 #ifdef KTRACE
@@ -868,77 +957,52 @@ syscall(frame)
 }
 
 #ifdef COMPAT_LINUX
-/*
- * linux_syscall(frame):
- */
-/*ARGSUSED*/
 void
 linux_syscall(frame)
 	struct trapframe frame;
 {
-	caddr_t params;
 	int i;
 	struct proc *p = curproc;
 	struct sysent *callp;
 	u_quad_t sticks;
-	int error, opc;
+	int error;
 	int rval[2];
-	int code;
+	u_int code;
 	struct linux_syscall_args {
-		int ebx;
-		int ecx;
-		int edx;
-		int esi;
-		int edi;
-		int ebp;
-		int eax;
+		int arg1;
+		int arg2;
+		int arg3;
+		int arg4;
+		int arg5;
 	} args;
 
-	args.ebx = frame.tf_ebx;
-	args.ecx = frame.tf_ecx;
-	args.edx = frame.tf_edx;
-	args.esi = frame.tf_esi;
-	args.edi = frame.tf_edi;
-	args.ebp = frame.tf_ebp;
-	args.eax = frame.tf_eax;
+	args.arg1 = frame.tf_ebx;
+	args.arg2 = frame.tf_ecx;
+	args.arg3 = frame.tf_edx;
+	args.arg4 = frame.tf_esi;
+	args.arg5 = frame.tf_edi;
 
 	sticks = p->p_sticks;
 	if (ISPL(frame.tf_cs) != SEL_UPL)
 		panic("linux syscall");
 
-	code = frame.tf_eax;
 	p->p_md.md_regs = (int *)&frame;
-	params = (caddr_t)frame.tf_esp + sizeof (int) ;
+	code = frame.tf_eax;
 
-	/*
-	 * Reconstruct pc, assuming lcall $X,y is 7 bytes, as it is always.
-	 * THIS IS WRONG FOR LINUX  XXX SOS
-	 * SIZE OF INT 0x80 (2??) NEEDED HERE !!!
-	 */
-	opc = frame.tf_eip - 2; /* was 7 */
-	if (code == 0) {
-		code = fuword(params);
-		params += sizeof (int);
-	}
 	if (p->p_sysent->sv_mask)
-		code = code & p->p_sysent->sv_mask;
+		code &= p->p_sysent->sv_mask;
 
-	if (code < 0 || code >= p->p_sysent->sv_size)
+	if (code >= p->p_sysent->sv_size)
 		callp = &p->p_sysent->sv_table[0];
 	else
 		callp = &p->p_sysent->sv_table[code];
 
 #ifdef KTRACE
 	if (KTRPOINT(p, KTR_SYSCALL))
-		ktrsyscall(p->p_tracep, code, callp->sy_narg, &args);
+		ktrsyscall(p->p_tracep, code, callp->sy_narg, (int *)&args);
 #endif
 
-#ifdef KTRACE
-	if (KTRPOINT(p, KTR_SYSCALL))
-		ktrsyscall(p->p_tracep, code, callp->sy_narg, &args);
-#endif
 	rval[0] = 0;
-	rval[1] = frame.tf_edx;
 
 	error = (*callp->sy_call)(p, &args, rval);
 
@@ -951,28 +1015,34 @@ linux_syscall(frame)
 		 */
 		p = curproc;
 		frame.tf_eax = rval[0];
-		frame.tf_eflags &= ~PSL_C;	/* carry bit */
+		frame.tf_eflags &= ~PSL_C;
 		break;
 
 	case ERESTART:
-		frame.tf_eip = opc;
+		/* Reconstruct pc, subtract size of int 0x80 */
+		frame.tf_eip -= 2;
 		break;
 
 	case EJUSTRETURN:
 		break;
 
 	default:
-	bad:
  		if (p->p_sysent->sv_errsize)
  			if (error >= p->p_sysent->sv_errsize)
   				error = -1;	/* XXX */
-   			else 
+   			else
   				error = p->p_sysent->sv_errtbl[error];
 		frame.tf_eax = -error;
-		frame.tf_eflags |= PSL_C;	/* carry bit */
+		frame.tf_eflags |= PSL_C;
 		break;
 	}
 
+	if (frame.tf_eflags & PSL_T) {
+		/* Traced syscall. */
+		frame.tf_eflags &= ~PSL_T;
+		trapsignal(p, SIGTRAP, 0);
+	}
+
 	userret(p, &frame, sticks);
 
 #ifdef KTRACE
