@@ -38,7 +38,7 @@
  *
  *	from: @(#)vm_machdep.c	7.3 (Berkeley) 5/13/91
  *	Utah $Hdr: vm_machdep.c 1.16.1.1 89/06/23$
- *	$Id: vm_machdep.c,v 1.37 1995/05/01 23:32:30 dyson Exp $
+ * $FreeBSD: stable/2.1/sys/i386/i386/vm_machdep.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include "npx.h"
@@ -232,7 +232,7 @@ more:
  * same as vm_bounce_kva -- but really allocate (but takes pages as arg)
  */
 vm_offset_t
-vm_bounce_kva_alloc(count) 
+vm_bounce_kva_alloc(count)
 int count;
 {
 	int i;
@@ -308,7 +308,7 @@ vm_bounce_alloc(bp)
  *		printf("size: %d, count: %d\n", bp->b_bufsize, bp->b_bcount);
  *	}
  */
-		
+
 
 	vastart = (vm_offset_t) bp->b_data;
 	vaend = (vm_offset_t) bp->b_data + bp->b_bufsize;
@@ -332,7 +332,7 @@ vm_bounce_alloc(bp)
 	if (dobounceflag == 0)
 		return;
 
-	if (bouncepages < dobounceflag) 
+	if (bouncepages < dobounceflag)
 		panic("Not enough bounce buffers!!!");
 
 /*
@@ -445,7 +445,7 @@ vm_bounce_free(bp)
 /*
  * free the bounce allocation
  */
-			
+
 /*
 			printf("(kva: %x, pa: %x)", bouncekva, mybouncepa);
 */
@@ -463,7 +463,7 @@ vm_bounce_free(bp)
 /*
  * add the old kva into the "to free" list
  */
-	
+
 	bouncekva= i386_trunc_page((vm_offset_t) bp->b_data);
 	bouncekvaend= i386_round_page((vm_offset_t)bp->b_data + bp->b_bufsize);
 
@@ -491,7 +491,7 @@ vm_bounce_init()
 
 	if (bouncepages == 0)
 		return;
-	
+
 	bounceallocarraysize = (bouncepages + BITS_IN_UNSIGNED - 1) / BITS_IN_UNSIGNED;
 	bounceallocarray = malloc(bounceallocarraysize * sizeof(unsigned), M_TEMP, M_NOWAIT);
 
@@ -552,7 +552,7 @@ cpu_fork(p1, p2)
 	int offset;
 
 	/*
-	 * Copy pcb and stack from proc p1 to p2. 
+	 * Copy pcb and stack from proc p1 to p2.
 	 * We do this as cheaply as possible, copying only the active
 	 * part of the stack.  The stack and pcb need to agree;
 	 * this is tricky, as the final pcb is constructed by savectx,
@@ -571,24 +571,16 @@ cpu_fork(p1, p2)
 	pmap_activate(&p2->p_vmspace->vm_pmap, &up->u_pcb);
 
 	/*
-	 * 
-	 * Arrange for a non-local goto when the new process
-	 * is started, to resume here, returning nonzero from setjmp.
+	 * Return (0) in parent, (1) in child.
 	 */
-	if (savectx(&up->u_pcb, 1)) {
-		/*
-		 * Return 1 in child.
-		 */
-		return (1);
-	}
-	return (0);
+	return (savectx(&up->u_pcb));
 }
 
 void
 cpu_exit(p)
 	register struct proc *p;
 {
-	
+
 #if NNPX > 0
 	npxexit(p);
 #endif	/* NNPX */
@@ -695,56 +687,31 @@ void
 vmapbuf(bp)
 	register struct buf *bp;
 {
-	register int npf;
-	register caddr_t addr;
-	int off;
-	vm_offset_t kva;
-	vm_offset_t pa, v;
+	register caddr_t addr, v, kva;
+	vm_offset_t pa;
 
 	if ((bp->b_flags & B_PHYS) == 0)
 		panic("vmapbuf");
 
-	/*
-	 * this is the kva that is to be used for
-	 * the temporary kernel mapping
-	 */
-	kva = (vm_offset_t) bp->b_saveaddr;
-
-	for (addr = (caddr_t)trunc_page(bp->b_data);
-		addr < bp->b_data + bp->b_bufsize;
-		addr += PAGE_SIZE) {
-
-/*
- * do the vm_fault if needed, do the copy-on-write thing when
- * reading stuff off device into memory.
- */
+	for (v = bp->b_saveaddr, addr = (caddr_t)trunc_page(bp->b_data);
+	    addr < bp->b_data + bp->b_bufsize;
+	    addr += PAGE_SIZE, v += PAGE_SIZE) {
+		/*
+		 * Do the vm_fault if needed; do the copy-on-write thing
+		 * when reading stuff off device into memory.
+		 */
 		vm_fault_quick(addr,
 			(bp->b_flags&B_READ)?(VM_PROT_READ|VM_PROT_WRITE):VM_PROT_READ);
-		pa = pmap_kextract((vm_offset_t) addr);
+		pa = trunc_page(pmap_kextract((vm_offset_t) addr));
 		if (pa == 0)
 			panic("vmapbuf: page not present");
-/*
- * hold the data page
- */
-#ifdef DIAGNOSTIC
-		if( VM_PAGE_TO_PHYS(PHYS_TO_VM_PAGE(pa)) != pa)
-			panic("vmapbuf: confused PHYS_TO_VM_PAGE mapping");
-#endif
 		vm_page_hold(PHYS_TO_VM_PAGE(pa));
+		pmap_kenter((vm_offset_t) v, pa);
 	}
 
-	addr = bp->b_saveaddr = bp->b_data;
-	off = (int)addr & PGOFSET;
-	npf = btoc(round_page(bp->b_bufsize + off));
-	bp->b_data = (caddr_t) (kva + off);
-	while (npf--) {
-		pa = pmap_kextract((vm_offset_t)addr);
-		if (pa == 0)
-			panic("vmapbuf: null page frame");
-		pmap_kenter(kva, trunc_page(pa));
-		addr += PAGE_SIZE;
-		kva += PAGE_SIZE;
-	}
+	kva = bp->b_saveaddr;
+	bp->b_saveaddr = bp->b_data;
+	bp->b_data = kva + (((vm_offset_t) bp->b_data) & PAGE_MASK);
 }
 
 /*
@@ -756,31 +723,20 @@ vunmapbuf(bp)
 	register struct buf *bp;
 {
 	register caddr_t addr;
-	vm_offset_t v,pa;
+	vm_offset_t pa;
 
 	if ((bp->b_flags & B_PHYS) == 0)
 		panic("vunmapbuf");
 
-	for (addr = (caddr_t)trunc_page((vm_offset_t) bp->b_data);
-		addr < bp->b_data + bp->b_bufsize;
-		addr += NBPG)
+	for (addr = (caddr_t)trunc_page(bp->b_data);
+	    addr < bp->b_data + bp->b_bufsize;
+	    addr += PAGE_SIZE) {
+		pa = trunc_page(pmap_kextract((vm_offset_t) addr));
 		pmap_kremove((vm_offset_t) addr);
-		
-	bp->b_data = bp->b_saveaddr;
-	bp->b_saveaddr = NULL;
-
-/*
- * unhold the pde, and data pages
- */
-	for (addr = (caddr_t)trunc_page((vm_offset_t) bp->b_data);
-		addr < bp->b_data + bp->b_bufsize;
-		addr += NBPG) {
-	/*
-	 * release the data page
-	 */
-		pa = pmap_kextract((vm_offset_t) addr);
 		vm_page_unhold(PHYS_TO_VM_PAGE(pa));
 	}
+
+	bp->b_data = bp->b_saveaddr;
 }
 
 /*
@@ -806,7 +762,7 @@ cpu_reset() {
 	bzero((caddr_t) PTD, NBPG);
 
 	/* "good night, sweet prince .... <THUNK!>" */
-	pmap_update(); 
+	pmap_update();
 	/* NOTREACHED */
 	while(1);
 }
