@@ -43,7 +43,7 @@
  * SUCH DAMAGE.
  *
  *	from:	@(#)fd.c	7.4 (Berkeley) 5/25/91
- *	$Id: fd.c,v 1.59 1995/05/06 19:34:25 joerg Exp $
+ * $FreeBSD: stable/2.1/sys/i386/isa/fd.c 50950 1999-09-05 08:39:22Z peter $
  *
  */
 
@@ -370,7 +370,7 @@ fdc_err(fdcu_t fdcu, const char *s)
 	else if(fdc_data[fdcu].fdc_errs == FDC_ERRMAX)
 		printf("fdc%d: too many errors, not logging any more\n",
 		       fdcu);
-	
+
 	return FD_FAILED;
 }
 
@@ -378,7 +378,7 @@ fdc_err(fdcu_t fdcu, const char *s)
  * fd_cmd: Send a command to the chip.  Takes a varargs with this structure:
  * Unit number,
  * # of output bytes, output bytes as ints ...,
- * # of input bytes, input bytes as ints ... 
+ * # of input bytes, input bytes as ints ...
  */
 
 int
@@ -604,7 +604,7 @@ fdattach(struct isa_device *dev)
 				unithasfd = 1;
 			if (ftattach(dev, fdup, unithasfd))
 				continue;
-			if (fdsu < DRVS_PER_CTLR) 
+			if (fdsu < DRVS_PER_CTLR)
 				fd->type = NO_TYPE;
 #endif
 			continue;
@@ -683,9 +683,9 @@ fdattach(struct isa_device *dev)
 					break; /* already probed succesfully */
 			}
 		}
-		
+
 		set_motor(fdcu, fdsu, TURNOFF);
-		
+
 		if (st0 & NE7_ST0_EC) /* no track 0 -> no drive present */
 			continue;
 
@@ -694,13 +694,13 @@ fdattach(struct isa_device *dev)
 		fd->fdsu = fdsu;
 		fd->options = 0;
 		printf("fd%d: ", fdu);
-		
+
 		fd_registerdev(fdcu, fdu);
 		switch (fdt) {
 		case RTCFDT_12M:
 			printf("1.2MB 5.25in\n");
 			fd->type = FD_1200;
-			kdc_fd[fdu].kdc_description = 
+			kdc_fd[fdu].kdc_description =
 				"1.2MB (1200K) 5.25in floppy disk drive";
 #ifdef	DEVFS
 			sprintf(name,"fd%d.1200",fdu);
@@ -716,6 +716,7 @@ fdattach(struct isa_device *dev)
 #endif	/* DEVFS */
 			break;
 		case RTCFDT_288M:
+		case RTCFDT_288M_1:
 			printf("2.88MB 3.5in - 1.44MB mode\n");
 			fd->type = FD_1440;
 			kdc_fd[fdu].kdc_description =
@@ -787,9 +788,9 @@ set_motor(fdcu_t fdcu, int fdsu, int turnon)
 {
 	int fdout = fdc_data[fdcu].fdout;
 	int needspecify = 0;
-	
+
 	if(turnon) {
-		fdout &= ~FDO_FDSEL;		
+		fdout &= ~FDO_FDSEL;
 		fdout |= (FDO_MOEN0 << fdsu) + fdsu;
 	} else
 		fdout &= ~(FDO_MOEN0 << fdsu);
@@ -832,6 +833,17 @@ fd_turnoff(void *arg1)
 	fd_p fd = fd_data + fdu;
 
 	TRACE1("[fd%d: turnoff]", fdu);
+
+	/*
+	 * Don't turn off the motor yet if the drive is active.
+	 * XXX shouldn't even schedule turnoff until drive is inactive
+	 * and nothing is queued on it.
+	 */
+	if (fd->fdc->state != DEVIDLE && fd->fdc->fdu == fdu) {
+		timeout(fd_turnoff, arg1, 4 * hz);
+		return;
+	}
+
 	s = splbio();
 	fd->flags &= ~FD_MOTOR;
 	set_motor(fd->fdc->fdcu, fd->fdsu, TURNOFF);
@@ -856,7 +868,7 @@ fd_motor_on(void *arg1)
 }
 
 static void
-fd_turnon(fdu_t fdu) 
+fd_turnon(fdu_t fdu)
 {
 	fd_p fd = fd_data + fdu;
 	if(!(fd->flags & FD_MOTOR))
@@ -871,7 +883,7 @@ static void
 fdc_reset(fdc_p fdc)
 {
 	fdcu_t fdcu = fdc->fdcu;
-	
+
 	/* Try a reset, keep motor on */
 	outb(fdc->baseport + FDOUT, fdc->fdout & ~(FDO_FRST|FDO_FDMAEN));
 	TRACE1("[0x%x->FDOUT]", fdc->fdout & ~(FDO_FRST|FDO_FDMAEN));
@@ -978,7 +990,7 @@ Fdopen(dev_t dev, int flags, int mode, struct proc *p)
 		return(ftopen(dev, flags));
 #endif
 	/* check bounds */
-	if (fdu >= NFD) 
+	if (fdu >= NFD)
 		return(ENXIO);
 	fdc = fd_data[fdu].fdc;
 	if ((fdc == NULL) || (fd_data[fdu].type == NO_TYPE))
@@ -1080,12 +1092,11 @@ fdstrategy(struct buf *bp)
 	fd = &fd_data[fdu];
 	fdc = fd->fdc;
 	fdcu = fdc->fdcu;
-	fdblk = 128 << (fd->ft->secsize);
 
 #if NFT > 0
 	if (FDTYPE(minor(bp->b_dev)) & F_TAPE_TYPE) {
 		/* ft tapes do not (yet) support strategy i/o */
-		bp->b_error = ENXIO;
+		bp->b_error = ENODEV;
 		bp->b_flags |= B_ERROR;
 		goto bad;
 	}
@@ -1093,9 +1104,10 @@ fdstrategy(struct buf *bp)
 	if (fdc->flags & FDC_TAPE_BUSY) {
 		bp->b_error = EBUSY;
 		bp->b_flags |= B_ERROR;
-		goto bad; 
+		goto bad;
 	}
 #endif
+	fdblk = 128 << (fd->ft->secsize);
 	if (!(bp->b_flags & B_FORMAT)) {
 		if ((fdu >= NFD) || (bp->b_blkno < 0)) {
 			printf(
@@ -1111,7 +1123,7 @@ fdstrategy(struct buf *bp)
 			goto bad;
 		}
 	}
-	
+
 	/*
 	 * Set up block calculations.
 	 */
@@ -1262,7 +1274,7 @@ fdstate(fdcu_t fdcu, fdc_p fdc)
 
 	dp = &(fdc->head);
 	bp = dp->b_actf;
-	if(!bp) 
+	if(!bp)
 	{
 		/***********************************************\
 		* nothing left for this controller to do	*
@@ -1469,7 +1481,7 @@ fdstate(fdcu_t fdcu, fdc_p fdc)
 		if(format)
 		{
 			/* formatting */
-			if(fd_cmd(fdcu, 6, 
+			if(fd_cmd(fdcu, 6,
 				  NE7CMD_FORMAT,
 				  head << 2 | fdu,
 				  finfo->fd_formb_secshift,
@@ -1482,7 +1494,7 @@ fdstate(fdcu_t fdcu, fdc_p fdc)
 				fdc->retry = 6;
 				return(retrier(fdcu));
 			}
-		}			
+		}
 		else
 		{
 			if (fd_cmd(fdcu, 9,
@@ -1574,6 +1586,13 @@ fdstate(fdcu_t fdcu, fdc_p fdc)
 		fdc->state = STARTRECAL;
 		break;
 	case STARTRECAL:
+		/* XXX clear the fdc results from the last reset, if any. */
+		{
+			int i;
+			for (i = 0; i < 4; i++)
+				(void)fd_sense_int(fdc, &st0, &cyl);
+		}
+
 		if(fd_cmd(fdcu,
 			  2, NE7CMD_RECAL, fdu,
 			  0)) /* Recalibrate Function */
@@ -1761,7 +1780,7 @@ fdformat(dev, finfo, p)
 
 	bp->b_bcount = sizeof(struct fd_idfield_data) * finfo->fd_formb_nsecs;
 	bp->b_un.b_addr = (caddr_t)finfo;
-	
+
 	/* now do the format */
 	fdstrategy(bp);
 
@@ -1774,7 +1793,7 @@ fdformat(dev, finfo, p)
 			break;
 	}
 	splx(s);
-	
+
 	if(rv == EWOULDBLOCK) {
 		/* timed out */
 		rv = EIO;
@@ -1890,7 +1909,7 @@ fdioctl(dev, cmd, addr, flag, p)
 	case FD_GOPTS:			/* get drive options */
 		*(int *)addr = fd_data[FDUNIT(minor(dev))].options;
 		break;
-		
+
 	case FD_SOPTS:			/* set drive options */
 		fd_data[FDUNIT(minor(dev))].options = *(int *)addr;
 		break;
