@@ -1,19 +1,40 @@
+/*
+ * Copyright (c) 1995, David Greenman
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice unmodified, this list of conditions, and the following
+ *    disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
+ *
+ * $FreeBSD: stable/2.1/sys/i386/isa/if_ed.c 50950 1999-09-05 08:39:22Z peter $
+ */
+
 /*
  * Device driver for National Semiconductor DS8390/WD83C690 based ethernet
  *   adapters. By David Greenman, 29-April-1993
  *
- * Copyright (C) 1993, David Greenman. This software may be used, modified,
- *   copied, distributed, and sold, in both source and binary form provided
- *   that the above copyright and these terms are retained. Under no
- *   circumstances is the author responsible for the proper functioning
- *   of this software, nor does the author assume any responsibility
- *   for damages incurred with its use.
- *
  * Currently supports the Western Digital/SMC 8003 and 8013 series,
  *   the SMC Elite Ultra (8216), the 3Com 3c503, the NE1000 and NE2000,
  *   and a variety of similar clones.
  *
- * $Id: if_ed.c,v 1.71 1995/05/12 22:39:40 davidg Exp $
  */
 
 #include "ed.h"
@@ -363,11 +384,11 @@ ed_probe_WD80x3(isa_dev)
 			isa16bit = 1;
 			memsize = 16384;
 			sc->type_str = "WD8013EP";
-			sc->kdc.kdc_description = 
+			sc->kdc.kdc_description =
 				"Ethernet adapter: WD 8013EP";
 		} else {
 			sc->type_str = "WD8003EP";
-			sc->kdc.kdc_description = 
+			sc->kdc.kdc_description =
 				"Ethernet adapter: WD 8003EP";
 		}
 		break;
@@ -390,25 +411,16 @@ ed_probe_WD80x3(isa_dev)
 		isa16bit = 1;
 		break;
 	case ED_TYPE_SMC8216C: /* 8216 has 16K shared mem -- 8416 has 8K */
-		(unsigned int) *(isa_dev->id_maddr+8192) = (unsigned int)0;
-		if ((unsigned int) *(isa_dev->id_maddr+8192)) {
-			sc->type_str = "SMC8416C/SMC8416BT";
-			sc->kdc.kdc_description =
-				"Ethernet adapter: SMC 8416C or 8416BT";
-			memsize = 8192;
-		} else {
+	case ED_TYPE_SMC8216T:
+		if (sc->type == ED_TYPE_SMC8216C) {
 			sc->type_str = "SMC8216/SMC8216C";
 			sc->kdc.kdc_description =
 				"Ethernet adapter: SMC 8216 or 8216C";
-			memsize = 16384;
+		} else {
+			sc->type_str = "SMC8216T";
+			sc->kdc.kdc_description =
+				"Ethernet adapter: SMC 8216T";
 		}
-		isa16bit = 1;
-		sc->is790 = 1;
-		break;
-	case ED_TYPE_SMC8216T:
-		sc->type_str = "SMC8216T";
-		sc->kdc.kdc_description = 
-			"Ethernet adapter: SMC 8216T";
 
 		outb(sc->asic_addr + ED_WD790_HWR,
 		    inb(sc->asic_addr + ED_WD790_HWR) | ED_WD790_HWR_SWH);
@@ -423,9 +435,16 @@ ed_probe_WD80x3(isa_dev)
 			memsize = 16384;
 			break;
 		case ED_WD790_RAR_SZ8:
-			sc->type_str = "SMC8416T";
-			sc->kdc.kdc_description =
-				"Ethernet adapter: SMC 8416T";
+			/* 8216 has 16K shared mem -- 8416 has 8K */
+			if (sc->type == ED_TYPE_SMC8216C) {
+				sc->type_str = "SMC8416C/SMC8416BT";
+				sc->kdc.kdc_description =
+					"Ethernet adapter: SMC 8416C or 8416BT";
+			} else {
+				sc->type_str = "SMC8416T";
+				sc->kdc.kdc_description =
+					"Ethernet adapter: SMC 8416T";
+			}
 			memsize = 8192;
 			break;
 		}
@@ -895,7 +914,7 @@ ed_probe_3Com(isa_dev)
 		outb(sc->asic_addr + ED_3COM_IDCFR, ED_3COM_IDCFR_IRQ5);
 		break;
 	default:
-		printf("ed%d: Invalid irq configuration (%d) must be 2-5 for 3c503\n",
+		printf("ed%d: Invalid irq configuration (%d) must be 3-5,9 for 3c503\n",
 		       isa_dev->id_unit, ffs(isa_dev->id_irq) - 1);
 		return (0);
 	}
@@ -1074,12 +1093,12 @@ ed_probe_Novell(isa_dev)
 		/* Search for the start of RAM. */
 		for (x = 1; x < 256; x++) {
 			ed_pio_readmem(sc, x * 256, tbuf, ED_PAGE_SIZE);
-			if (memcmp(pbuf0, tbuf, ED_PAGE_SIZE) == 0) {
+			if (bcmp(pbuf0, tbuf, ED_PAGE_SIZE) == 0) {
 				for (i = 0; i < ED_PAGE_SIZE; i++)
 					pbuf[i] = 255 - x;
 				ed_pio_writemem(sc, pbuf, x * 256, ED_PAGE_SIZE);
 				ed_pio_readmem(sc, x * 256, tbuf, ED_PAGE_SIZE);
-				if (memcmp(pbuf, tbuf, ED_PAGE_SIZE) == 0) {
+				if (bcmp(pbuf, tbuf, ED_PAGE_SIZE) == 0) {
 					mstart = x * ED_PAGE_SIZE;
 					msize = ED_PAGE_SIZE;
 					break;
@@ -1094,12 +1113,12 @@ ed_probe_Novell(isa_dev)
 		/* Search for the start of RAM. */
 		for (x = (mstart / ED_PAGE_SIZE) + 1; x < 256; x++) {
 			ed_pio_readmem(sc, x * 256, tbuf, ED_PAGE_SIZE);
-			if (memcmp(pbuf0, tbuf, ED_PAGE_SIZE) == 0) {
+			if (bcmp(pbuf0, tbuf, ED_PAGE_SIZE) == 0) {
 				for (i = 0; i < ED_PAGE_SIZE; i++)
 					pbuf[i] = 255 - x;
 				ed_pio_writemem(sc, pbuf, x * 256, ED_PAGE_SIZE);
 				ed_pio_readmem(sc, x * 256, tbuf, ED_PAGE_SIZE);
-				if (memcmp(pbuf, tbuf, ED_PAGE_SIZE) == 0)
+				if (bcmp(pbuf, tbuf, ED_PAGE_SIZE) == 0)
 					msize += ED_PAGE_SIZE;
 				else {
 					break;
@@ -1376,7 +1395,7 @@ ed_init(unit)
 	/*
 	 * Enable the following interrupts: receive/transmit complete,
 	 * receive/transmit error, and Receiver OverWrite.
-	 * 
+	 *
 	 * Counter overflow and Remote DMA complete are *not* enabled.
 	 */
 	outb(sc->nic_addr + ED_P0_IMR,
@@ -1682,7 +1701,8 @@ ed_rint(unit)
 			ed_pio_readmem(sc, packet_ptr, (char *) &packet_hdr,
 				       sizeof(packet_hdr));
 		len = packet_hdr.count;
-		if (len > ETHER_MAX_LEN+4) { /* len includes 4 byte header */
+		if (len > (ETHER_MAX_LEN + sizeof(struct ed_ring)) ||
+		    len < (ETHER_HDR_SIZE + sizeof(struct ed_ring))) {
 			/*
 			 * Length is a wild value. There's a good chance that
 			 * this was caused by the NIC being old and buggy.
@@ -1709,7 +1729,8 @@ ed_rint(unit)
 		 * the upper layer protocols can then figure out the length from
 		 * their own length field(s).
 		 */
-		if ((len <= MCLBYTES) &&
+		if ((len > sizeof(struct ed_ring)) &&
+		    (len <= MCLBYTES) &&
 		    (packet_hdr.next_packet >= sc->rec_page_start) &&
 		    (packet_hdr.next_packet < sc->rec_page_stop)) {
 			/*
@@ -2043,25 +2064,21 @@ ed_ioctl(ifp, command, data)
 	case SIOCSIFFLAGS:
 
 		/*
-		 * If interface is marked down and it is running, then stop it
+		 * If the interface is marked up and stopped, then start it.
+		 * If it is marked down and running, then stop it.
 		 */
-		if (((ifp->if_flags & IFF_UP) == 0) &&
-		    (ifp->if_flags & IFF_RUNNING)) {
-			ed_stop(ifp->if_unit);
-			ifp->if_flags &= ~IFF_RUNNING;
-		} else {
-
-			/*
-			 * If interface is marked up and it is stopped, then
-			 * start it
-			 */
-			if ((ifp->if_flags & IFF_UP) &&
-			    ((ifp->if_flags & IFF_RUNNING) == 0))
+		if (ifp->if_flags & IFF_UP) {
+			if ((ifp->if_flags & IFF_RUNNING) == 0)
 				ed_init(ifp->if_unit);
+		} else {
+			if (ifp->if_flags & IFF_RUNNING) {
+				ed_stop(ifp->if_unit);
+				ifp->if_flags &= ~IFF_RUNNING;
+			}
 		}
 		/* UP controls BUSY/IDLE */
-		sc->kdc.kdc_state = ((ifp->if_flags & IFF_UP) 
-				     ? DC_BUSY 
+		sc->kdc.kdc_state = ((ifp->if_flags & IFF_UP)
+				     ? DC_BUSY
 				     : DC_IDLE);
 
 #if NBPFILTER > 0
@@ -2124,6 +2141,42 @@ ed_ioctl(ifp, command, data)
 	return (error);
 }
 
+/*
+ * Given a source and destination address, copy 'amount' of a packet from
+ *	the ring buffer into a linear destination buffer. Takes into account
+ *	ring-wrap.
+ */
+static inline char *
+ed_ring_copy(sc, src, dst, amount)
+	struct ed_softc *sc;
+	char   *src;
+	char   *dst;
+	u_short amount;
+{
+	u_short tmp_amount;
+
+	/* does copy wrap to lower addr in ring buffer? */
+	if (src + amount > sc->mem_end) {
+		tmp_amount = sc->mem_end - src;
+
+		/* copy amount up to end of NIC memory */
+		if (sc->mem_shared)
+			bcopy(src, dst, tmp_amount);
+		else
+			ed_pio_readmem(sc, src, dst, tmp_amount);
+
+		amount -= tmp_amount;
+		src = sc->mem_ring;
+		dst += tmp_amount;
+	}
+	if (sc->mem_shared)
+		bcopy(src, dst, amount);
+	else
+		ed_pio_readmem(sc, src, dst, amount);
+
+	return (src + amount);
+}
+
 /*
  * Retreive packet from shared memory and send to the next level up via
  *	ether_input(). If there is a BPF listener, give a copy to BPF, too.
@@ -2145,13 +2198,21 @@ ed_get_packet(sc, buf, len, multicast)
 	m->m_pkthdr.rcvif = &sc->arpcom.ac_if;
 	m->m_pkthdr.len = m->m_len = len;
 
-	/* Attach an mbuf cluster */
-	MCLGET(m, M_DONTWAIT);
+	/*
+	 * We always put the received packet in a single buffer -
+	 * either with just an mbuf header or in a cluster attached
+	 * to the header. The +2 is to compensate for the alignment
+	 * fixup below.
+	 */
+	if ((len + 2) > MHLEN) {
+		/* Attach an mbuf cluster */
+		MCLGET(m, M_DONTWAIT);
 
-	/* Insist on getting a cluster */
-	if ((m->m_flags & M_EXT) == 0) {
-		m_freem(m);
-		return;
+		/* Insist on getting a cluster */
+		if ((m->m_flags & M_EXT) == 0) {
+			m_freem(m);
+			return;
+		}
 	}
 
 	/*
@@ -2327,7 +2388,7 @@ ed_pio_write_mbufs(sc, m, dst)
 	outb(sc->nic_addr + ED_P0_CR, ED_CR_RD1 | ED_CR_STA);
 
   /*
-   * Transfer the mbuf chain to the NIC memory. 
+   * Transfer the mbuf chain to the NIC memory.
    * 16-bit cards require that data be transferred as words, and only words.
    * So that case requires some extra code to patch over odd-length mbufs.
    */
@@ -2336,7 +2397,7 @@ ed_pio_write_mbufs(sc, m, dst)
 		/* NE1000s are easy */
 		while (m) {
 			if (m->m_len) {
-				outsb(sc->asic_addr + ED_NOVELL_DATA, 
+				outsb(sc->asic_addr + ED_NOVELL_DATA,
 				      m->m_data, m->m_len);
 			}
 			m = m->m_next;
@@ -2346,7 +2407,7 @@ ed_pio_write_mbufs(sc, m, dst)
 		unsigned char *data;
 		int len, wantbyte;
 		unsigned char savebyte[2];
-    
+
 		wantbyte = 0;
 
 		while (m) {
@@ -2363,7 +2424,7 @@ ed_pio_write_mbufs(sc, m, dst)
 				}
 				/* output contiguous words */
 				if (len > 1) {
-					outsw(sc->asic_addr + ED_NOVELL_DATA, 
+					outsw(sc->asic_addr + ED_NOVELL_DATA,
 					      data, len >> 1);
 					data += len & ~1;
 					len &= 1;
@@ -2400,42 +2461,6 @@ ed_pio_write_mbufs(sc, m, dst)
 	return (total_len);
 }
 
-/*
- * Given a source and destination address, copy 'amount' of a packet from
- *	the ring buffer into a linear destination buffer. Takes into account
- *	ring-wrap.
- */
-static inline char *
-ed_ring_copy(sc, src, dst, amount)
-	struct ed_softc *sc;
-	char   *src;
-	char   *dst;
-	u_short amount;
-{
-	u_short tmp_amount;
-
-	/* does copy wrap to lower addr in ring buffer? */
-	if (src + amount > sc->mem_end) {
-		tmp_amount = sc->mem_end - src;
-
-		/* copy amount up to end of NIC memory */
-		if (sc->mem_shared)
-			bcopy(src, dst, tmp_amount);
-		else
-			ed_pio_readmem(sc, src, dst, tmp_amount);
-
-		amount -= tmp_amount;
-		src = sc->mem_ring;
-		dst += tmp_amount;
-	}
-	if (sc->mem_shared)
-		bcopy(src, dst, amount);
-	else
-		ed_pio_readmem(sc, src, dst, amount);
-
-	return (src + amount);
-}
-
 void
 ed_setrcr(ifp, sc)
 	struct ifnet *ifp;
@@ -2556,7 +2581,7 @@ ds_getmcaf(sc, mcaf)
 			mcaf[1] = 0xffffffff;
 			return;
 		}
-		index = ds_crc(enm->enm_addrlo, 6) >> 26;
+		index = ds_crc(enm->enm_addrlo) >> 26;
 		af[index >> 3] |= 1 << (index & 7);
 
 		ETHER_NEXT_MULTI(step, enm);
