@@ -32,13 +32,13 @@
 
 /*
  *	Modified from the FreeBSD 1.1.5.1 version by:
- *		 	Andres Vega Garcia 
+ *		 	Andres Vega Garcia
  *			INRIA - Sophia Antipolis, France
  *			avega@sophia.inria.fr
  */
 
 /*
- *  $Id: if_ep.c,v 1.26 1995/04/12 20:47:49 wollman Exp $
+ * $FreeBSD: stable/2.1/sys/i386/isa/if_ep.c 50950 1999-09-05 08:39:22Z peter $
  *
  *  Promiscuous mode added and interrupt logic slightly changed
  *  to reduce the number of adapter failures. Transceiver select
@@ -59,6 +59,7 @@
 #if defined(__FreeBSD__)
 #include <sys/systm.h>
 #include <sys/kernel.h>
+#include <sys/conf.h>
 #include <sys/devconf.h>
 #endif
 #include <sys/mbuf.h>
@@ -82,6 +83,11 @@
 #include <netinet/if_ether.h>
 #endif
 
+#ifdef IPX
+#include <netipx/ipx.h>
+#include <netipx/ipx_if.h>
+#endif
+
 #ifdef NS
 #include <netns/ns.h>
 #include <netns/ns_if.h>
@@ -100,150 +106,157 @@
 #include <i386/isa/isa_device.h>
 #include <i386/isa/icu.h>
 #include <i386/isa/if_epreg.h>
+#include <i386/isa/elink.h>
 
-static int epprobe __P((struct isa_device *));
-static int epattach __P((struct isa_device *));
-static int epioctl __P((struct ifnet * ifp, int, caddr_t));
-static void epmbuffill __P((caddr_t, int));
-static void epmbufempty __P((struct ep_softc *));
+/* Exported variables */
+u_long	ep_unit;
+int	ep_boards;
+struct	ep_board ep_board[EP_MAX_BOARDS + 1];
 
-void epinit __P((int));
-void epintr __P((int));
-void epread __P((struct ep_softc *));
-void epreset __P((int));
-void epstart __P((struct ifnet *));
-void epstop __P((int));
-void epwatchdog __P((int));
+static	int eeprom_rdy __P((struct ep_softc *sc));
 
-static int send_ID_sequence __P((int));
-static int get_eeprom_data __P((int, int));
+static	int ep_isa_probe __P((struct isa_device *));
+static struct ep_board * ep_look_for_board_at __P((struct isa_device *is));
+static	int ep_isa_attach __P((struct isa_device *));
+static	void ep_isa_registerdev __P((struct ep_softc *sc,
+				     struct isa_device *id));
+static	int epioctl __P((struct ifnet * ifp, int, caddr_t));
+static	void epmbuffill __P((caddr_t, int));
+static	void epmbufempty __P((struct ep_softc *));
 
-struct ep_softc ep_softc[NEP];
+static	void epinit __P((struct ep_softc *));
+static	void epread __P((struct ep_softc *));
+void	epreset __P((int));
+static	void epstart __P((struct ifnet *));
+static	void epstop __P((struct ep_softc *));
+static	void epwatchdog __P((int unit));
+
+static	int send_ID_sequence __P((int));
+static	int get_eeprom_data __P((int, int));
+
+static	struct ep_softc* ep_softc[NEP];
+static	int ep_current_tag = EP_LAST_TAG + 1;
+static	char *ep_conn_type[] = {"UTP", "AUI", "???", "BNC"};
 
 #define ep_ftst(f) (sc->stat&(f))
 #define ep_fset(f) (sc->stat|=(f))
 #define ep_frst(f) (sc->stat&=~(f))
 
 struct isa_driver epdriver = {
-    epprobe,
-    epattach,
+    ep_isa_probe,
+    ep_isa_attach,
     "ep",
-	0
+    0
 };
 
-static struct kern_devconf kdc_ep[NEP] = { {
-      0, 0, 0,			/* filled in by dev_attach */
-      "ep", 0, { MDDT_ISA, 0, "net" },
-      isa_generic_externalize, 0, 0, ISA_EXTERNALLEN,
-      &kdc_isa0,		/* parent */
-      0,			/* parentdata */
-      DC_UNCONFIGURED,		/* state */
-      "3Com 3C509 Ethernet adapter",
-      DC_CLS_NETIF		/* class */
-} };
+static struct kern_devconf kdc_isa_ep = {
+    0, 0, 0,			/* filled in by dev_attach */
+    "ep", 0, { MDDT_ISA, 0, "net" },
+    isa_generic_externalize, 0, 0, ISA_EXTERNALLEN,
+    &kdc_isa0,		/* parent */
+    0,			/* parentdata */
+    DC_UNCONFIGURED,		/* state */
+    "3Com 3C509 Ethernet adapter",
+    DC_CLS_NETIF		/* class */
+};
 
-static inline void
-ep_registerdev(struct isa_device *id)
+static void
+ep_isa_registerdev(sc, id)
+    struct ep_softc *sc;
+    struct isa_device *id;
 {
-      if(id->id_unit)
-              kdc_ep[id->id_unit] = kdc_ep[0];
-      kdc_ep[id->id_unit].kdc_unit = id->id_unit;
-      kdc_ep[id->id_unit].kdc_parentdata = id;
-      dev_attach(&kdc_ep[id->id_unit]);
+    sc->kdc = (struct kern_devconf *)malloc(sizeof(struct kern_devconf),
+					    M_DEVBUF, M_NOWAIT);
+    if (!sc->kdc) {
+	printf("WARNING: ep_isa_registerdev unable to malloc! "
+	       "Device kdc will not be registerd\n");
+	return;
+    }
+    bcopy(&kdc_isa_ep, sc->kdc, sizeof(kdc_isa_ep));
+    sc->kdc->kdc_unit = sc->unit;
+    sc->kdc->kdc_parentdata = id;
+    dev_attach(sc->kdc);
 }
 
-int ep_current_tag = EP_LAST_TAG + 1;
-
-struct {
-	int epb_addr;	/* address of this board */
-	char epb_used;	/* was this entry already used for configuring ? */
-	}
-	ep_board[EP_MAX_BOARDS + 1];
-
 static int
-eeprom_rdy(is)
-    struct isa_device *is;
+eeprom_rdy(sc)
+    struct ep_softc *sc;
 {
     int i;
 
-    for (i = 0; is_eeprom_busy(IS_BASE) && i < MAX_EEPROMBUSY; i++);
+    for (i = 0; is_eeprom_busy(BASE) && i < MAX_EEPROMBUSY; i++);
     if (i >= MAX_EEPROMBUSY) {
-	printf("ep%d: eeprom failed to come ready.\n", is->id_unit);
+	printf("ep%d: eeprom failed to come ready.\n", sc->unit);
 	return (0);
     }
     return (1);
 }
 
-static int
+static struct ep_board *
 ep_look_for_board_at(is)
     struct isa_device *is;
 {
-    int data, i, j, io_base, id_port = EP_ID_PORT;
-    int nisa = 0, neisa = 0;
+    int data, i, j, io_base, id_port = ELINK_ID_PORT;
+    int count = 0;
 
     if (ep_current_tag == (EP_LAST_TAG + 1)) {
 	/* Come here just one time */
-   
-	/* Look for the EISA boards, leave them activated */
-	for(j = 1; j < 16; j++) {
-	    io_base = (j * EP_EISA_START) | EP_EISA_W0;
-	    if (inw(io_base + EP_W0_MFG_ID) != MFG_ID)
-		continue;
 
-	    /* we must found 0x1f if the board is EISA configurated */
-	    if ((inw(io_base + EP_W0_ADDRESS_CFG) & 0x1f) != 0x1f) 
-		continue;
-
-	    /* Reset and Enable the card */
-	    outb(io_base + EP_W0_CONFIG_CTRL, W0_P4_CMD_RESET_ADAPTER);
-	    DELAY(1000); /* we must wait at least 1 ms */
-	    outb(io_base + EP_W0_CONFIG_CTRL, W0_P4_CMD_ENABLE_ADAPTER);
-
-	    /*
-	     * Once activated, all the registers are mapped in the range
-	     * x000 - x00F, where x is the slot number.
-             */
-	    ep_board[neisa].epb_used = 0;
-	    ep_board[neisa++].epb_addr = j * EP_EISA_START;
-	}
 	ep_current_tag--;
 
         /* Look for the ISA boards. Init and leave them actived */
-	outb(id_port, 0xc0);	/* Global reset */
+	outb(id_port, 0);
+	outb(id_port, 0);
+
+	elink_idseq(0xCF);
+
+	elink_reset();
 	DELAY(10000);
 	for (i = 0; i < EP_MAX_BOARDS; i++) {
 	    outb(id_port, 0);
 	    outb(id_port, 0);
-	    send_ID_sequence(id_port);
+	    elink_idseq(0xCF);
 
 	    data = get_eeprom_data(id_port, EEPROM_MFG_ID);
 	    if (data != MFG_ID)
 		break;
 
 	    /* resolve contention using the Ethernet address */
-	    for (j = 0; j < 3; j++)
-		data = get_eeprom_data(id_port, j);
 
-	    ep_board[neisa+nisa].epb_used = 0;
-	    ep_board[neisa+nisa++].epb_addr =
-		(get_eeprom_data(id_port, EEPROM_ADDR_CFG) & 0x1f) * 0x10 + 0x200;
+	    for (j = 0; j < 3; j++)
+		 get_eeprom_data(id_port, j);
+
+	    /* and save this address for later use */
+
+	    for (j = 0; j < 3; j++)
+		 ep_board[ep_boards].eth_addr[j] = get_eeprom_data(id_port, j);
+
+	    ep_board[ep_boards].res_cfg =
+		get_eeprom_data(id_port, EEPROM_RESOURCE_CFG);
+
+	    ep_board[ep_boards].prod_id =
+		get_eeprom_data(id_port, EEPROM_PROD_ID);
+
+	    ep_board[ep_boards].epb_used = 0;
+	    ep_board[ep_boards].epb_addr =
+			(get_eeprom_data(id_port, EEPROM_ADDR_CFG) & 0x1f) * 0x10 + 0x200;
+
+	    if(ep_board[ep_boards].epb_addr > 0x3E0)
+		/* Board in EISA configuration mode */
+		continue;
+
 	    outb(id_port, ep_current_tag);	/* tags board */
 	    outb(id_port, ACTIVATE_ADAPTER_TO_CONFIG);
+	    ep_boards++;
+	    count++;
 	    ep_current_tag--;
 	}
 
-	ep_board[neisa+nisa].epb_addr = 0;
-	if (neisa) {
-	    printf("%d 3C5x9 board(s) on EISA found at", neisa);
+	ep_board[ep_boards].epb_addr = 0;
+	if (count) {
+	    printf("%d 3C5x9 board(s) on ISA found at", count);
 	    for (j = 0; ep_board[j].epb_addr; j++)
-		if (ep_board[j].epb_addr >= EP_EISA_START)
-		    printf(" 0x%x", ep_board[j].epb_addr);
-	    printf("\n");
-	}
-	if (nisa) {
-	    printf("%d 3C5x9 board(s) on ISA found at", nisa);
-	    for (j = 0; ep_board[j].epb_addr; j++)
-		if (ep_board[j].epb_addr < EP_EISA_START)
+		if (ep_board[j].epb_addr <= 0x3E0)
 		    printf(" 0x%x", ep_board[j].epb_addr);
 	    printf("\n");
 	}
@@ -266,7 +279,8 @@ ep_look_for_board_at(is)
 
 	IS_BASE=ep_board[i].epb_addr;
 	ep_board[i].epb_used=1;
-	return 1;
+
+	return &ep_board[i];
     } else {
 	for (i=0; ep_board[i].epb_addr && ep_board[i].epb_addr != IS_BASE; i++);
 
@@ -274,10 +288,11 @@ ep_look_for_board_at(is)
 	    return 0;
 
 	if (inw(IS_BASE + EP_W0_EEPROM_COMMAND) & EEPROM_TST_MODE)
-	    printf("ep%d: 3c5x9 at 0x%x in test mode. Erase pencil mark!\n",
+	    printf("ep%d: 3c5x9 at 0x%x in PnP mode. Disable PnP mode!\n",
 		   is->id_unit, IS_BASE);
 	ep_board[i].epb_used=1;
-	return 1;
+
+	return &ep_board[i];
     }
 }
 
@@ -285,43 +300,100 @@ ep_look_for_board_at(is)
  * get_e: gets a 16 bits word from the EEPROM. we must have set the window
  * before
  */
-static int
-get_e(is, offset)
-    struct isa_device *is;
+u_int16_t
+get_e(sc, offset)
+    struct ep_softc *sc;
     int offset;
 {
-    if (!eeprom_rdy(is))
+    if (!eeprom_rdy(sc))
 	return (0xffff);
-    outw(IS_BASE + EP_W0_EEPROM_COMMAND, EEPROM_CMD_RD | offset);
-    if (!eeprom_rdy(is))
+    outw(BASE + EP_W0_EEPROM_COMMAND, EEPROM_CMD_RD | offset);
+    if (!eeprom_rdy(sc))
 	return (0xffff);
-    return (inw(IS_BASE + EP_W0_EEPROM_DATA));
+    return (inw(BASE + EP_W0_EEPROM_DATA));
+}
+
+struct ep_softc *
+ep_alloc(unit, epb)
+    int	unit;
+    struct	ep_board *epb;
+{
+    struct	ep_softc *sc;
+
+    if (unit >= NEP) {
+	printf("ep: unit number (%d) too high\n", unit);
+	return NULL;
+    }
+ 
+    /*
+     * Allocate a storage area for us
+     */
+    if (ep_softc[unit]) {
+	printf("ep%d: unit number already allocated to another "
+	       "adaptor\n", unit);
+	return NULL;
+    }
+
+    sc = malloc(sizeof(struct ep_softc), M_DEVBUF, M_NOWAIT);
+    if(!sc) {
+	printf("ep%d: cannot malloc!\n", unit);
+	return NULL;
+    }
+    bzero(sc, sizeof(struct ep_softc));
+    ep_softc[unit] = sc;
+    sc->unit = unit;
+    sc->ep_io_addr = epb->epb_addr;
+    sc->epb = epb;
+
+    return(sc);
+}
+
+void
+ep_free(sc)
+    struct ep_softc *sc;
+{
+    ep_softc[sc->unit] = NULL;
+    free(sc, M_DEVBUF);
+    return;
 }
 
 int
-epprobe(is)
+ep_isa_probe(is)
     struct isa_device *is;
 {
-    struct ep_softc *sc = &ep_softc[is->id_unit];
+    struct ep_softc *sc;
+    struct ep_board *epb;
     u_short k;
     int i;
 
-    ep_registerdev(is);
-
-    if (!ep_look_for_board_at(is))
+    if(( epb=ep_look_for_board_at(is) )==0)
 	return (0);
+
+    /*
+     * Allocate a storage area for us
+     */
+    sc = ep_alloc(ep_unit, epb); 
+    if( !sc )
+	return (0);
+
+    is->id_unit = ep_unit++;
+
+    ep_isa_registerdev(sc, is);
+
     /*
      * The iobase was found and MFG_ID was 0x6d50. PROD_ID should be
      * 0x9[0-f]50
      */
     GO_WINDOW(0);
-    k = get_e(is, EEPROM_PROD_ID);
+    k = sc->epb->prod_id;
     if ((k & 0xf0ff) != (PROD_ID & 0xf0ff)) {
-	printf("epprobe: ignoring model %04x\n", k);
+	printf("ep_isa_probe: ignoring model %04x\n", k);
+	ep_free(sc);
 	return (0);
     }
 
-    k = get_e(is, EEPROM_RESOURCE_CFG);
+    k = sc->epb->res_cfg;
+
     k >>= 12;
 
     /* Now we have two cases again:
@@ -338,101 +410,109 @@ epprobe(is)
 	is->id_irq= 1 << ( (k==2) ? 9 : k );
     }
 
-    if (BASE >= EP_EISA_START) /* we have an EISA board, we allow 32 bits access */
-	sc->stat = F_ACCESS_32_BITS;
-    else
-	sc->stat = 0;
+    sc->stat = 0;	/* 16 bit access */
 
     /* By now, the adapter is already activated */
 
-    return (0x10);		/* 16 bytes of I/O space used. */
+    return (EP_IOSIZE);		/* 16 bytes of I/O space used. */
 }
 
-static char *ep_conn_type[] = {"UTP", "AUI", "???", "BNC"};
-
 static int
-epattach(is)
+ep_isa_attach(is)
     struct isa_device *is;
 {
-    struct ep_softc *sc = &ep_softc[is->id_unit];
-    struct ifnet *ifp = &sc->arpcom.ac_if;
-    u_short i, j, *p;
-    struct ifaddr *ifa;
-    struct sockaddr_dl *sdl;
+    struct ep_softc *sc = ep_softc[is->id_unit];
+    u_short config;
     int irq;
 
-    /* BASE = IS_BASE; */
-    sc->ep_io_addr = is->id_iobase;
-
-    printf("ep%d: ", is->id_unit);
-
     sc->ep_connectors = 0;
-    i = inw(IS_BASE + EP_W0_CONFIG_CTRL);
-    j = inw(IS_BASE + EP_W0_ADDRESS_CFG) >> ACF_CONNECTOR_BITS;
-    if (i & IS_AUI) {
-	printf("aui");
+    config = inw(IS_BASE + EP_W0_CONFIG_CTRL);
+    if (config & IS_AUI) {
 	sc->ep_connectors |= AUI;
     }
-    if (i & IS_BNC) {
-	if (sc->ep_connectors)
-	    printf("/");
-	printf("bnc");
+    if (config & IS_BNC) {
 	sc->ep_connectors |= BNC;
     }
-    if (i & IS_UTP) {
-	if (sc->ep_connectors)
-	    printf("/");
-	printf("utp");
+    if (config & IS_UTP) {
 	sc->ep_connectors |= UTP;
     }
     if (!(sc->ep_connectors & 7))
 	printf("no connectors!");
-    else
-	printf("[*%s*]", ep_conn_type[j]);
-
-    /*
-     * Read the station address from the eeprom
-     */
-    p = (u_short *) & sc->arpcom.ac_enaddr;
-    for (i = 0; i < 3; i++) {
-	GO_WINDOW(0);
-	p[i] = htons(get_e(is, i));
-	GO_WINDOW(2);
-	outw(BASE + EP_W2_ADDR_0 + (i * 2), ntohs(p[i]));
-    }
-    printf(" address %s", ether_sprintf(sc->arpcom.ac_enaddr));
-
+    sc->ep_connector = inw(BASE + EP_W0_ADDRESS_CFG) >> ACF_CONNECTOR_BITS;
     /*
      * Write IRQ value to board
      */
 
-    i=is->id_irq;
-    if(i==0) {
-	printf(" irq STRANGE\n");
+    irq = ffs(is->id_irq) - 1;
+    if(irq == -1) {
+	printf(" invalid irq... cannot attach\n");
 	return 0;
-	}
+    }
 
-    for(irq=0; !(i & 1) && irq<16 ; i>>=1, irq++);
-
-    if(irq==9)
-	irq=2;
-    printf(" irq %d\n",irq);
     GO_WINDOW(0);
+    if(irq == 9)
+	irq = 2;
     outw(BASE + EP_W0_RESOURCE_CFG, SET_IRQ(irq));
 
-    ifp->if_unit = is->id_unit;
+    ep_attach(sc);
+    return 1;
+}
+
+int
+ep_attach(sc)
+    struct ep_softc *sc;
+{
+    struct ifaddr *ifa;
+    struct ifnet *ifp = &sc->arpcom.ac_if;
+    struct sockaddr_dl *sdl;
+    u_short *p;
+    int i;
+
+    printf("ep%d: ", sc->unit);
+    /*
+     * Current media type
+     */
+    if(sc->ep_connectors & AUI) {
+	printf("aui");
+	if(sc->ep_connectors & ~AUI)
+		printf("/");
+    }
+    if(sc->ep_connectors & UTP) {
+	printf("utp");
+	if(sc->ep_connectors & BNC)
+		printf("/");
+    }
+    if(sc->ep_connectors & BNC) {
+	printf("bnc");
+    }
+
+    printf("[*%s*]", ep_conn_type[sc->ep_connector]);
+
+    /*
+     * Setup the station address
+     */
+    p = (u_short *) & sc->arpcom.ac_enaddr;
+    GO_WINDOW(2);
+    for (i = 0; i < 3; i++) {
+	p[i] = htons(sc->epb->eth_addr[i]);
+	outw(BASE + EP_W2_ADDR_0 + (i * 2), ntohs(p[i]));
+    }
+    printf(" address %s\n", ether_sprintf(sc->arpcom.ac_enaddr));
+
+    ifp->if_unit = sc->unit;
     ifp->if_name = "ep";
     ifp->if_mtu = ETHERMTU;
-    ifp->if_flags = IFF_BROADCAST | IFF_SIMPLEX | IFF_NOTRAILERS;
-    ifp->if_init = epinit;
+    ifp->if_flags = IFF_BROADCAST | IFF_SIMPLEX | IFF_MULTICAST;
     ifp->if_output = ether_output;
     ifp->if_start = epstart;
     ifp->if_ioctl = epioctl;
     ifp->if_watchdog = epwatchdog;
-	ifp->if_timer=1;
 
     if_attach(ifp);
-    kdc_ep[is->id_unit].kdc_state = DC_BUSY;
+    ether_ifattach(ifp);
+
+    /* device attach does transition from UNCONFIGURED to IDLE state */
+    sc->kdc->kdc_state=DC_IDLE;
 
     /*
      * Fill the hardware address into ifa_addr if we find an AF_LINK entry.
@@ -455,10 +535,10 @@ epattach(is)
     sc->rx_avg_pkt = 128;
 
     /*
-     * NOTE: In all this I multiply everything by 64. 
-     * W_s = the speed the CPU is able to write to the TX FIFO. 
+     * NOTE: In all this I multiply everything by 64.
+     * W_s = the speed the CPU is able to write to the TX FIFO.
      * T_s = the speed the board sends the info to the Ether.
-     * W_s/T_s = 16   (represents 16/64) =>    W_s = 25 % of T_s. 
+     * W_s/T_s = 16   (represents 16/64) =>    W_s = 25 % of T_s.
      * This will give us for a packet of 1500 bytes
      * tx_start_thresh=1125 and for a pkt of 64 bytes tx_start_threshold=48.
      * We prefer to start thinking the CPU is much slower than the Ethernet
@@ -479,7 +559,7 @@ epattach(is)
 #if NBPFILTER > 0
     bpfattach(&sc->bpf, ifp, DLT_EN10MB, sizeof(struct ether_header));
 #endif
-    return 1;
+    return 0;
 }
 
 
@@ -487,11 +567,10 @@ epattach(is)
  * The order in here seems important. Otherwise we may not receive
  * interrupts. ?!
  */
-void
-epinit(unit)
-    int unit;
+static void
+epinit(sc)
+    struct ep_softc *sc;
 {
-    register struct ep_softc *sc = &ep_softc[unit];
     register struct ifnet *ifp = &sc->arpcom.ac_if;
     int s, i, j;
 
@@ -571,10 +650,8 @@ epinit(unit)
 	outw(BASE + EP_W4_MEDIA_TYPE, ENABLE_UTP);
 	GO_WINDOW(1);
     } else {
-	GO_WINDOW(0);
-	j = inw(BASE + EP_W0_ADDRESS_CFG) >> ACF_CONNECTOR_BITS;
 	GO_WINDOW(1);
-	switch(j) {
+	switch(sc->ep_connector) {
 	    case ACF_CONNECTOR_UTP:
 		if(sc->ep_connectors & UTP) {
 		    GO_WINDOW(4);
@@ -593,7 +670,7 @@ epinit(unit)
 		break;
 	    default:
 		printf("ep%d: strange connector type in EEPROM: assuming AUI\n",
-		    unit);
+		    sc->unit);
 		break;
 	}
     }
@@ -647,34 +724,30 @@ epinit(unit)
 
 static const char padmap[] = {0, 3, 2, 1};
 
-void
+static void
 epstart(ifp)
     struct ifnet *ifp;
 {
-    register struct ep_softc *sc = &ep_softc[ifp->if_unit];
+    register struct ep_softc *sc = ep_softc[ifp->if_unit];
     register u_int len;
     register struct mbuf *m;
     struct mbuf *top;
     int s, pad;
 
     s = splimp();
-    if (sc->arpcom.ac_if.if_flags & IFF_OACTIVE) {
+    if (ifp->if_flags & IFF_OACTIVE) {
 	splx(s);
 	return;
     }
 startagain:
     /* Sneak a peek at the next packet */
-    m = sc->arpcom.ac_if.if_snd.ifq_head;
+    m = ifp->if_snd.ifq_head;
     if (m == 0) {
 	splx(s);
 	return;
     }
-#if 0
-    len = m->m_pkthdr.len;
-#else
     for (len = 0, top = m; m; m = m->m_next)
 	len += m->m_len;
-#endif
 
     pad = padmap[len & 3];
 
@@ -685,19 +758,19 @@ startagain:
      */
     if (len + pad > ETHER_MAX_LEN) {
 	/* packet is obviously too large: toss it */
-	++sc->arpcom.ac_if.if_oerrors;
-	IF_DEQUEUE(&sc->arpcom.ac_if.if_snd, m);
+	++ifp->if_oerrors;
+	IF_DEQUEUE(&ifp->if_snd, m);
 	m_freem(m);
 	goto readcheck;
     }
     if (inw(BASE + EP_W1_FREE_TX) < len + pad + 4) {
 	/* no room in FIFO */
 	outw(BASE + EP_COMMAND, SET_TX_AVAIL_THRESH | (len + pad + 4));
-	sc->arpcom.ac_if.if_flags |= IFF_OACTIVE;
+	ifp->if_flags |= IFF_OACTIVE;
 	splx(s);
 	return;
     }
-    IF_DEQUEUE(&sc->arpcom.ac_if.if_snd, m);
+    IF_DEQUEUE(&ifp->if_snd, m);
 
     outw(BASE + EP_W1_TX_PIO_WR_1, len);
     outw(BASE + EP_W1_TX_PIO_WR_1, 0x0);	/* Second dword meaningless */
@@ -721,7 +794,7 @@ startagain:
 		  m->m_len / 4);
 	    if (m->m_len & 3)
 		outsb(BASE + EP_W1_TX_PIO_WR_1,
-		      mtod(m, caddr_t) + m->m_len / 4,
+		      mtod(m, caddr_t) + (m->m_len & (~3)),
 		      m->m_len & 3);
 	} else {
 	    outsw(BASE + EP_W1_TX_PIO_WR_1, mtod(m, caddr_t), m->m_len / 2);
@@ -739,7 +812,8 @@ startagain:
     }
 #endif
 
-    sc->arpcom.ac_if.if_opackets++;
+    ifp->if_timer=2;
+    ifp->if_opackets++;
     m_freem(top);
     /*
      * Every 1024*4 packets we increment the tx_rate if we haven't had
@@ -759,7 +833,7 @@ readcheck:
 	 * we check if we have packets left, in that case we prepare to come
 	 * back later
 	 */
-	if (sc->arpcom.ac_if.if_snd.ifq_head) {
+	if (ifp->if_snd.ifq_head) {
 	    outw(BASE + EP_COMMAND, SET_TX_AVAIL_THRESH |
 		 sc->tx_start_thresh);
 	}
@@ -773,15 +847,25 @@ void
 epintr(unit)
     int unit;
 {
-    int i;
+    register struct ep_softc *sc = ep_softc[unit];
+    ep_intr(sc);
+}
+
+void
+ep_intr(arg)
+    void *arg;
+{
+    struct ep_softc *sc;
     register int status;
-    register struct ep_softc *sc = &ep_softc[unit];
-    struct ifnet *ifp = &sc->arpcom.ac_if;
-    struct mbuf *m;
+    struct ifnet *ifp;
     int x;
 
     x=splbio();
 
+    sc = (struct ep_softc *)arg;
+
+    ifp = &sc->arpcom.ac_if;
+
     outw(BASE + EP_COMMAND, SET_INTR_MASK); /* disable all Ints */
 
 rescan:
@@ -797,30 +881,39 @@ rescan:
 	}
 	if (status & S_TX_AVAIL) {
 	    /* we need ACK */
-	    sc->arpcom.ac_if.if_flags &= ~IFF_OACTIVE;
+	    ifp->if_timer=0;
+	    ifp->if_flags &= ~IFF_OACTIVE;
 	    GO_WINDOW(1);
 	    inw(BASE + EP_W1_FREE_TX);
-	    epstart(&sc->arpcom.ac_if);
+	    epstart(ifp);
 	}
 	if (status & S_CARD_FAILURE) {
+	    ifp->if_timer=0;
 #ifdef EP_LOCAL_STATS
-	    printf("\nep%d:\n\tStatus: %x\n", unit, status);
+	    printf("\nep%d:\n\tStatus: %x\n", sc->unit, status);
 	    GO_WINDOW(4);
 	    printf("\tFIFO Diagnostic: %x\n", inw(BASE + EP_W4_FIFO_DIAG));
 	    printf("\tStat: %x\n", sc->stat);
 	    printf("\tIpackets=%d, Opackets=%d\n",
-		sc->arpcom.ac_if.if_ipackets, sc->arpcom.ac_if.if_opackets);
+		ifp->if_ipackets, ifp->if_opackets);
 	    printf("\tNOF=%d, NOMB=%d, BPFD=%d, RXOF=%d, RXOL=%d, TXU=%d\n",
 		   sc->rx_no_first, sc->rx_no_mbuf, sc->rx_bpf_disc, sc->rx_overrunf,
 		   sc->rx_overrunl, sc->tx_underrun);
 #else
-	    printf("ep%d: Status: %x\n", unit, status); 
+
+#ifdef DIAGNOSTIC
+	    printf("ep%d: Status: %x (input buffer overflow)\n", sc->unit, status);
+#else
+	    ++ifp->if_ierrors;
 #endif
-	    epinit(unit);
+
+#endif
+	    epinit(sc);
 	    splx(x);
 	    return;
 	}
 	if (status & S_TX_COMPLETE) {
+	    ifp->if_timer=0;
 	    /* we  need ACK. we do it at the end */
 	    /*
 	     * We need to read TX_STATUS until we get a 0 status in order to
@@ -841,31 +934,31 @@ rescan:
 		    } else {
 			if (status & TXS_JABBER);
 			else	/* TXS_MAX_COLLISION - we shouldn't get here */
-			    ++sc->arpcom.ac_if.if_collisions;
+			    ++ifp->if_collisions;
 		    }
-		    ++sc->arpcom.ac_if.if_oerrors;
+		    ++ifp->if_oerrors;
 		    outw(BASE + EP_COMMAND, TX_ENABLE);
 		    /*
 		     * To have a tx_avail_int but giving the chance to the
 		     * Reception
 		     */
-		    if (sc->arpcom.ac_if.if_snd.ifq_head) {
+		    if (ifp->if_snd.ifq_head) {
 			outw(BASE + EP_COMMAND, SET_TX_AVAIL_THRESH | 8);
 		    }
 		}
 		outb(BASE + EP_W1_TX_STATUS, 0x0);	/* pops up the next
 							 * status */
 	    }			/* while */
-	    sc->arpcom.ac_if.if_flags &= ~IFF_OACTIVE;
+	    ifp->if_flags &= ~IFF_OACTIVE;
 	    GO_WINDOW(1);
 	    inw(BASE + EP_W1_FREE_TX);
-	    epstart(&sc->arpcom.ac_if);
+	    epstart(ifp);
 	}			/* end TX_COMPLETE */
     }
 
     outw(BASE + EP_COMMAND, C_INTR_LATCH);	/* ACK int Latch */
 
-    if ((status = inw(BASE + EP_STATUS)) & S_5_INTS) 
+    if ((status = inw(BASE + EP_STATUS)) & S_5_INTS)
 	goto rescan;
 
     /* re-enable Ints */
@@ -874,24 +967,26 @@ rescan:
     splx(x);
 }
 
-void
+static void
 epread(sc)
     register struct ep_softc *sc;
 {
     struct ether_header *eh;
     struct mbuf *top, *mcur, *m;
+    struct ifnet *ifp;
     int lenthisone;
 
     short rx_fifo2, status;
     register short delta;
     register short rx_fifo;
 
+    ifp = &sc->arpcom.ac_if;
     status = inw(BASE + EP_W1_RX_STATUS);
 
 read_again:
 
     if (status & ERR_RX) {
-	++sc->arpcom.ac_if.if_ierrors;
+	++ifp->if_ierrors;
 	if (status & ERR_RX_OVERRUN) {
 	    /*
 	     * we can think the rx latency is actually greather than we
@@ -1048,7 +1143,7 @@ all_pkt:
     if (delta < MIN_RX_EARLY_THRESHF)
 	delta = MIN_RX_EARLY_THRESHF;
     sc->rx_early_thresh = delta;
-    ++sc->arpcom.ac_if.if_ipackets;
+    ++ifp->if_ipackets;
     ep_fset(F_RX_FIRST);
     ep_frst(F_RX_TRAILER);
     top->m_pkthdr.rcvif = &sc->arpcom.ac_if;
@@ -1064,7 +1159,7 @@ all_pkt:
 	 * check if this packet is really ours.
 	 */
 	eh = mtod(top, struct ether_header *);
-	if ((sc->arpcom.ac_if.if_flags & IFF_PROMISC) &&
+	if ((ifp->if_flags & IFF_PROMISC) &&
 	    (eh->ether_dhost[0] & 1) == 0 &&
 	    bcmp(eh->ether_dhost, sc->arpcom.ac_enaddr,
 		 sizeof(eh->ether_dhost)) != 0 &&
@@ -1088,7 +1183,7 @@ all_pkt:
 
     eh = mtod(top, struct ether_header *);
     m_adj(top, sizeof(struct ether_header));
-    ether_input(&sc->arpcom.ac_if, eh, top);
+    ether_input(ifp, eh, top);
     if (!sc->mb[sc->next_mb])
 	epmbuffill((caddr_t) sc, 0);
     sc->top = 0;
@@ -1125,20 +1220,44 @@ epioctl(ifp, cmd, data)
     caddr_t data;
 {
     register struct ifaddr *ifa = (struct ifaddr *) data;
-    struct ep_softc *sc = &ep_softc[ifp->if_unit];
+    struct ep_softc *sc = ep_softc[ifp->if_unit];
     struct ifreq *ifr = (struct ifreq *) data;
     int s, error = 0;
 
+    s = splimp();
+
     switch (cmd) {
       case SIOCSIFADDR:
 	ifp->if_flags |= IFF_UP;
+
+	/* netifs are BUSY when UP */
+	sc->kdc->kdc_state=DC_BUSY;
+
 	switch (ifa->ifa_addr->sa_family) {
 #ifdef INET
 	  case AF_INET:
-	    epinit(ifp->if_unit);	/* before arpwhohas */
+	    epinit(sc);	/* before arpwhohas */
 	    arp_ifinit((struct arpcom *)ifp, ifa);
 	    break;
 #endif
+#ifdef IPX
+	  case AF_IPX:
+	    {
+		register struct ipx_addr *ina = &(IA_SIPX(ifa)->sipx_addr);
+
+		if (ipx_nullhost(*ina))
+		    ina->x_host =
+			*(union ipx_host *) (sc->arpcom.ac_enaddr);
+		else {
+		    ifp->if_flags &= ~IFF_RUNNING;
+		    bcopy((caddr_t) ina->x_host.c_host,
+			  (caddr_t) sc->arpcom.ac_enaddr,
+			  sizeof(sc->arpcom.ac_enaddr));
+		}
+		epinit(sc);
+		break;
+	    }
+#endif
 #ifdef NS
 	  case AF_NS:
 	    {
@@ -1153,41 +1272,42 @@ epioctl(ifp, cmd, data)
 			  (caddr_t) sc->arpcom.ac_enaddr,
 			  sizeof(sc->arpcom.ac_enaddr));
 		}
-		epinit(ifp->if_unit);
+		epinit(sc);
 		break;
 	    }
 #endif
 	  default:
-	    epinit(ifp->if_unit);
+	    epinit(sc);
 	    break;
 	}
 	break;
+      case SIOCGIFADDR:
+	{ 
+	  struct sockaddr *sa; 
+ 
+	  sa = (struct sockaddr *) & ifr->ifr_data;
+	  bcopy((caddr_t) sc->arpcom.ac_enaddr, 
+		(caddr_t) sa->sa_data, ETHER_ADDR_LEN);
+	}
+	break;
       case SIOCSIFFLAGS:
+	/* UP controls BUSY/IDLE */
+	sc->kdc->kdc_state= ( (ifp->if_flags & IFF_UP)
+		? DC_BUSY
+		: DC_IDLE );
+
 	if ((ifp->if_flags & IFF_UP) == 0 && ifp->if_flags & IFF_RUNNING) {
 	    ifp->if_flags &= ~IFF_RUNNING;
-	    epstop(ifp->if_unit);
+	    epstop(sc);
 	    epmbufempty(sc);
 	    break;
 	} else {
 	    /* reinitialize card on any parameter change */
-	    epinit(ifp->if_unit);
+	    epinit(sc);
 	    break;
 	}
 
 	/* NOTREACHED */
-
-	if (ifp->if_flags & IFF_UP && (ifp->if_flags & IFF_RUNNING) == 0)
-	    epinit(ifp->if_unit);
-
-	if ( (ifp->if_flags & IFF_PROMISC) &&  !ep_ftst(F_PROMISC) ) {
-	    ep_fset(F_PROMISC);
-	    epinit(ifp->if_unit);
-	    }
-	else if( !(ifp->if_flags & IFF_PROMISC) && ep_ftst(F_PROMISC) ) {
-	    ep_frst(F_PROMISC);
-	    epinit(ifp->if_unit);
-	    }
-
 	break;
 #ifdef notdef
       case SIOCGHWADDR:
@@ -1195,63 +1315,56 @@ epioctl(ifp, cmd, data)
 	      sizeof(sc->sc_addr));
 	break;
 #endif
-	case SIOCSIFMTU: 
+	case SIOCSIFMTU:
 
 		/*
 		 * Set the interface MTU.
 		 */
-		if (ifr->ifr_mtu > ETHERMTU) {  
-			error = EINVAL; 
+		if (ifr->ifr_mtu > ETHERMTU) {
+			error = EINVAL;
 		} else {
-			ifp->if_mtu = ifr->ifr_mtu; 
+			ifp->if_mtu = ifr->ifr_mtu;
 		}
-		break;  
-
+		break; 
+	case SIOCADDMULTI:
+	case SIOCDELMULTI:
+	    /* Now this driver has no support for programmable
+	     * multicast filters. If some day it will gain this
+	     * support this part of code must be extended.
+	     */
+	    error=0;
+	    break;
       default:
 		error = EINVAL;
     }
+
+    splx(s);
+
     return (error);
 }
 
-void
-epreset(unit)
-    int unit;
-{
-    int s = splimp();
-
-    epstop(unit);
-    epinit(unit);
-    splx(s);
-}
-
-void
+static void
 epwatchdog(unit)
     int unit;
 {
-    struct ep_softc *sc = &ep_softc[unit];
-    struct ifnet *ifp=&sc->arpcom.ac_if;
-
+    struct ep_softc *sc = ep_softc[unit];
+    struct ifnet *ifp = &sc->arpcom.ac_if;
     /*
     printf("ep: watchdog\n");
 
-    log(LOG_ERR, "ep%d: watchdog\n", unit);
-    ++sc->arpcom.ac_if.if_oerrors;
+    log(LOG_ERR, "ep%d: watchdog\n", ifp->if_unit);
+    ifp->if_oerrors++;
     */
 
-    /* epreset(unit); */
     ifp->if_flags &= ~IFF_OACTIVE;
     epstart(ifp);
-    epintr(unit);
-
-    ifp->if_timer=1;
+    ep_intr(sc);
 }
 
-void
-epstop(unit)
-    int unit;
+static void
+epstop(sc)
+    struct ep_softc *sc;
 {
-    struct ep_softc *sc = &ep_softc[unit];
-
     outw(BASE + EP_COMMAND, RX_DISABLE);
     outw(BASE + EP_COMMAND, RX_DISCARD_TOP_PACK);
     while (inw(BASE + EP_STATUS) & S_COMMAND_IN_PROGRESS);
