@@ -28,7 +28,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: if_ix.c,v 1.5 1995/03/02 08:17:14 rgrimes Exp $
+ * $FreeBSD: stable/2.1/sys/i386/isa/if_ix.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include "ix.h"
@@ -148,7 +148,7 @@ int ixattach(struct isa_device *);
 void ixinit(int);
 void ixinit_rfa(int);
 void ixinit_tfa(int);
-int ixintr(int);
+inthand2_t ixintr;
 static inline void ixintr_cx(int);
 static inline void ixintr_cx_free(int, cb_t *);
 static inline void ixintr_fr(int);
@@ -318,6 +318,7 @@ ixprobe(struct isa_device *dvp) {
 	/* ZZZ irq_translate should really be unsigned, but until
 	 * isa_device.h and all uses are fixed we have to live with it */
 	short	irq_translate[] = {0, IRQ9, IRQ3, IRQ4, IRQ5, IRQ10, IRQ11, 0};
+	char	irq_encode[] = { 0, 0, 0, 2, 3, 4, 0, 0, 0, 1, 5, 6, 0, 0, 0, 0 };
 
 	DEBUGBEGIN(DEBUGPROBE)
 	DEBUGDO(printf ("ixprobe:");)
@@ -431,7 +432,7 @@ ixprobe(struct isa_device *dvp) {
 			goto ixprobe_exit;
 		}
 	}
-	
+
 	if ((kvtop(dvp->id_maddr) < 0xC0000) ||
 	    (kvtop(dvp->id_maddr) + dvp->id_msize > 0xF0000)) {
 		printf("ixprobe mapped memory address out of range\n");
@@ -498,10 +499,20 @@ ixprobe(struct isa_device *dvp) {
 	 */
 	irq = ixeeprom_read(unit, eeprom_config1);
 	irq = (irq & IRQ) >> IRQ_SHIFT;
-	sc->irq_encoded = irq;
 	irq = irq_translate[irq];
-	if (irq != dvp->id_irq) {
-		printf("Warning board is configured for IRQ %d\n", irq);
+	if (dvp->id_irq > 0) {
+		if (irq != dvp->id_irq) {
+			printf("ix%d: WARNING: board is configured for IRQ %d, using %d\n",
+				unit, ffs(irq) - 1, ffs(dvp->id_irq) - 1);
+			irq = dvp->id_irq;
+		}
+	} else {
+		dvp->id_irq = irq;
+	}
+	sc->irq_encoded = irq_encode[ffs(irq) - 1];
+	if (sc->irq_encoded == 0) {
+		printf("ix%d: invalid irq (%d)\n", ffs(irq) - 1);
+		goto ixprobe_exit;
 	}
 
 	/*
@@ -573,7 +584,7 @@ ixattach(struct isa_device *dvp) {
 	ifp->if_mtu = ETHERMTU;
 	ifp->if_flags = IFF_NOTRAILERS | IFF_BROADCAST;
 	/*
-	 * This is commented out to save memory and cpu time 
+	 * This is commented out to save memory and cpu time
 	 * ifp->if_timer = 0;
 	 * ifp->if_metric = 0;
 	 * ifp->if_addrlist = 0;
@@ -592,7 +603,7 @@ ixattach(struct isa_device *dvp) {
 	ifp->if_reset = ixreset;
 	ifp->if_watchdog = ixwatchdog;
 	/*
-	 * This is commented out to save memory and cpu time 
+	 * This is commented out to save memory and cpu time
 	 * ifp->if_ipackets = 0;
 	 * ifp->if_ierrors = 0;
 	 * ifp->if_opackets = 0;
@@ -605,7 +616,7 @@ ixattach(struct isa_device *dvp) {
 	ifp->if_addrlen = ETHER_ADDRESS_LENGTH;
 	ifp->if_hdrlen = ETHER_HEADER_LENGTH;
 	/*
-	 * This is commented out to save memory and cpu time 
+	 * This is commented out to save memory and cpu time
 	 * ifp->if_index = 0;
 	 * ifp->if_lastchange.tv_sec = 0;
 	 * ifp->if_lastchange.tv_usec = 0;
@@ -770,7 +781,7 @@ ixinit(int unit) {
 					   collision detect source external */
 	cb_conf->byte[10] = 60;		/* minimum number of bytes is a frame */
 	cb_conf->byte[11] = 0;		/* unused */
-				
+
 	scb->command = SCB_CUC_START;
 	ixchannel_attention(unit);
 	status |= ix_cb_wait((cb_t *)cb_conf, "Configure");
@@ -859,7 +870,7 @@ ixinit_rfa(int unit) {
 	rbd = (rbd_t *)(sc->maddr +
 	                RFA_START +
                         (how_many_frames * sizeof(rfd_t)));
-	rb = sc->maddr + RFA_START + 
+	rb = sc->maddr + RFA_START +
 	     (how_many_frames * (sizeof(rfd_t) + sizeof(rbd_t)));
 	sc->rfd_head = rfd;
 	sc->rbd_head = rbd;
@@ -906,7 +917,7 @@ ixinit_rfa(int unit) {
 		rbd = (rbd_t *)(sc->maddr +
 		               RFA_START +
 		               (how_many_frames * sizeof(rfd_t)));
-		rb = sc->maddr + RFA_START + 
+		rb = sc->maddr + RFA_START +
 		     (how_many_frames * (sizeof(rfd_t) + sizeof(rbd_t)));
 		printf("  complete_frame_size = %d\n", complete_frame_size);
 		printf("  how_many_frames = %d\n", how_many_frames);
@@ -976,7 +987,7 @@ ixinit_tfa(int unit) {
 	tbd = (tbd_t *)(sc->maddr +
 	                TFA_START +
                         (TB_COUNT * sizeof(cb_transmit_t)));
-	tb = sc->maddr + TFA_START + 
+	tb = sc->maddr + TFA_START +
 	     (TB_COUNT * (sizeof(cb_transmit_t) + sizeof(tbd_t)));
 	sc->cb_head = (cb_t *)cb;
 	sc->tbd_head = tbd;
@@ -1013,7 +1024,7 @@ ixinit_tfa(int unit) {
 		tbd = (tbd_t *)(sc->maddr +
 		               TFA_START +
 		               (TB_COUNT * sizeof(cb_transmit_t)));
-		tb = sc->maddr + TFA_START + 
+		tb = sc->maddr + TFA_START +
 		     (TB_COUNT * (sizeof(cb_transmit_t) + sizeof(tbd_t)));
 		printf("  TB_COUNT = %d\n", TB_COUNT);
 		printf("  cb_head = %lx\t\tcb_tail = %lx\n",
@@ -1045,7 +1056,7 @@ ixinit_tfa(int unit) {
 	DEBUGEND
 }
 
-int
+void
 ixintr(int unit) {
 	ix_softc_t	*sc = &ix_softc[unit];
 	struct	ifnet	*ifp = &sc->arpcom.ac_if;
@@ -1109,12 +1120,11 @@ ixintr(int unit) {
 	if (check_queue && ifp->if_snd.ifq_head != 0) {
 		ixstart(ifp);	/* we have stuff on the queue */
 	}
-	
+
 ixintr_exit:
 	DEBUGBEGIN(DEBUGINTR)
 	DEBUGDO(printf(" ixintr exited\n");)
 	DEBUGEND
-	return(0 /* XXX Should be ??? */);
 }
 
 static inline void
@@ -1415,7 +1425,7 @@ ixstart(struct ifnet *ifp) {
 		IF_DEQUEUE(&ifp->if_snd, m);
 		length = 0;
 		for (m_temp = m; m_temp != 0; m_temp = m_temp->m_next) {
-			bcopy(mtod(m_temp, caddr_t), tb, m_temp->m_len); 
+			bcopy(mtod(m_temp, caddr_t), tb, m_temp->m_len);
 			tb += m_temp->m_len;
 			length += m_temp->m_len;
 		}
@@ -1427,7 +1437,7 @@ ixstart(struct ifnet *ifp) {
  			 * This should never ever happen, if it does
 			 * we probable screwed up all sorts of board data
 			 * in the above bcopy's and should probably shut
-			 * down, but for now just issue a warning that 
+			 * down, but for now just issue a warning that
 			 * something is real wrong
 			 */
 			printf("ix%d: ixstart: Packet length=%d > MTU=%d\n",
@@ -1517,13 +1527,16 @@ int
 ixioctl(struct ifnet *ifp, int cmd, caddr_t data) {
 	int	unit = ifp->if_unit;
 	int	status = 0;
+	int	s;
 	ix_softc_t	*sc = &ix_softc[unit];
 
 	DEBUGBEGIN(DEBUGIOCTL)
 	DEBUGDO(printf("ixioctl:");)
 	DEBUGEND
-	switch(cmd) {
 
+	s = splimp();
+
+	switch(cmd) {
 	case SIOCSIFADDR: {
 		struct ifaddr *ifa = (struct ifaddr *)data;
 
@@ -1575,6 +1588,7 @@ ixioctl(struct ifnet *ifp, int cmd, caddr_t data) {
 		break;
 	}
 	}
+	splx(s);
 
 	DEBUGBEGIN(DEBUGIOCTL)
 	DEBUGDO(printf("ixioctl exit\n");)
