@@ -34,7 +34,7 @@
  * SUCH DAMAGE.
  *
  *	from: @(#)isa.c	7.2 (Berkeley) 5/13/91
- *	$Id: isa.c,v 1.48 1995/05/11 05:20:43 jkh Exp $
+ * $FreeBSD: stable/2.1/sys/i386/isa/isa.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -108,7 +108,7 @@ struct kern_devconf kdc_isa0 = {
 	&kdc_cpu0,		/* parent is the CPU */
 	0,			/* no parentdata */
 	DC_BUSY,		/* busses are always busy */
-	"ISA or EISA bus",
+	"ISA bus",
 	DC_CLS_BUS		/* class */
 };
 
@@ -240,10 +240,6 @@ haveseen(dvp, tmpdvp, checkbits)
  * Search through all the isa_devtab_* tables looking for anything that
  * conflicts with the current device.
  */
-#include "eisa.h"
-#if NEISA > 0
-extern struct isa_device isa_devtab_eisa[];
-#endif
 
 int
 haveseen_isadev(dvp, checkbits)
@@ -273,13 +269,6 @@ haveseen_isadev(dvp, checkbits)
 		if (status)
 			return status;
 	}
-#if NEISA > 0
-	for (tmpdvp = isa_devtab_eisa; tmpdvp->id_driver; tmpdvp++) {
-		status |= haveseen(dvp, tmpdvp, checkbits);
-		if (status)
-			return status;
-	}
-#endif
 	return(status);
 }
 
@@ -293,8 +282,6 @@ isa_configure() {
 	dev_attach(&kdc_isa0);
 
 	splhigh();
-	enable_intr();
-	INTREN(IRQ_SLAVE);
 	printf("Probing for devices on the ISA bus:\n");
 	/* First probe all the sensitive probes */
 	for (dvp = isa_devtab_tty; dvp->id_driver; dvp++)
@@ -334,7 +321,7 @@ isa_configure() {
  * SLIPDISC.  No need to block out ALL ttys during a splimp when only one
  * of them is running slip.
  *
- * XXX actually, blocking all ttys during a splimp doesn't matter so much 
+ * XXX actually, blocking all ttys during a splimp doesn't matter so much
  * with sio because the serial interrupt layer doesn't use tty_imask.  Only
  * non-serial ttys suffer.  It's more stupid that ALL 'net's are blocked
  * during spltty.
@@ -372,8 +359,8 @@ isa_configure() {
 /*
  * Configure an ISA device.
  */
- 
- 
+
+
 static void
 config_isadev(isdp, mp)
      struct isa_device *isdp;
@@ -381,7 +368,7 @@ config_isadev(isdp, mp)
 {
 	config_isadev_c(isdp, mp, 0);
 }
- 
+
 void
 reconfig_isadev(isdp, mp)
 	struct isa_device *isdp;
@@ -400,7 +387,7 @@ config_isadev_c(isdp, mp, reconfig)
 	int id_alive;
 	int last_alive;
 	struct isa_driver *dp = isdp->id_driver;
- 
+
  	checkbits = 0;
 	checkbits |= CC_DRQ;
 	checkbits |= CC_IOADDR;
@@ -464,11 +451,11 @@ config_isadev_c(isdp, mp, reconfig)
 			}
 			printf("\n");
 			/*
-			 * Check for conflicts again.  The driver may have 
-			 * changed *dvp.  We should weaken the early check 
-			 * since the driver may have been able to change 
-			 * *dvp to avoid conflicts if given a chance.  We 
-			 * already skip the early check for IRQs and force 
+			 * Check for conflicts again.  The driver may have
+			 * changed *dvp.  We should weaken the early check
+			 * since the driver may have been able to change
+			 * *dvp to avoid conflicts if given a chance.  We
+			 * already skip the early check for IRQs and force
 			 * a check for IRQs in the next group of checks.
 		 	 */
 			checkbits |= CC_IRQ;
@@ -497,12 +484,12 @@ config_isadev_c(isdp, mp, reconfig)
 				}
 				printf("\n");
 			}
-		}	
+		}
 		else {
 			/* This code has not been tested.... */
 			if (isdp->id_irq) {
 				INTRDIS(isdp->id_irq);
-				unregister_intr(ffs(isdp->id_irq) - 1, 
+				unregister_intr(ffs(isdp->id_irq) - 1,
 						isdp->id_intr);
 				if (mp)
 					INTRUNMASK(*mp, isdp->id_irq);
@@ -526,27 +513,6 @@ isa_externalize(struct isa_device *id, void *userp, size_t *maxlen)
 	return copyout(id, userp, sizeof *id);
 }
 
-/*
- * Do the same thing for EISA information.  EISA information is currently
- * the same as ISA information plus a slot number, but could be extended in
- * the future.
- */
-int
-eisa_externalize(struct isa_device *id, int slot, void *userp, size_t *maxlen)
-{
-	int rv;
-
-	if(*maxlen < (sizeof *id) + (sizeof slot)) {
-		return ENOMEM;
-	}
-	*maxlen -= (sizeof *id) + (sizeof slot);
-
-	rv = copyout(id, userp, sizeof *id);
-	if(rv) return rv;
-
-	return copyout(&slot, (char *)userp + sizeof *id, sizeof slot);
-}
-
 /*
  * This is used to forcibly reconfigure an ISA device.  It currently just
  * returns an error 'cos you can't do that yet.  It is here to demonstrate
@@ -581,19 +547,12 @@ isa_generic_externalize(struct proc *p, struct kern_devconf *kdc,
 	return isa_externalize(kdc->kdc_isa, userp, &l);
 }
 
-int
-eisa_generic_externalize(struct proc *p, struct kern_devconf *kdc,
-			 void *userp, size_t l)
-{
-	return eisa_externalize(kdc->kdc_isa, -1, userp, &l);
-}
-
 /*
  * Fill in default interrupt table (in case of spuruious interrupt
  * during configuration of kernel, setup interrupt control unit
  */
 void
-isa_defaultirq() 
+isa_defaultirq()
 {
 	int i;
 
@@ -643,7 +602,7 @@ static short dmapageport[8] =
 void isa_dmacascade(unsigned chan)
 {
 	if (chan > 7)
-		panic("isa_dmacascade: impossible request"); 
+		panic("isa_dmacascade: impossible request");
 
 	/* set dma channel mode, and set dma channel mode */
 	if ((chan & 4) == 0) {
@@ -670,7 +629,7 @@ void isa_dmastart(int flags, caddr_t addr, unsigned nbytes, unsigned chan)
 	if (    chan > 7
 	    || (chan < 4 && nbytes > (1<<16))
 	    || (chan >= 4 && (nbytes > (1<<17) || (u_int)addr & 1)))
-		panic("isa_dmastart: impossible request"); 
+		panic("isa_dmastart: impossible request");
 
 	if (isa_dmarangecheck(addr, nbytes, chan)) {
 		if (dma_bounce[chan] == 0)
@@ -818,7 +777,7 @@ static void (*isaphysmemunblock)(); /* needs to be a list */
  */
 caddr_t
 isa_allocphysmem(caddr_t va, unsigned length, void (*func)()) {
-	
+
 	isaphysmemunblock = func;
 	while (isaphysmemflag & B_BUSY) {
 		isaphysmemflag |= B_WANTED;
@@ -844,7 +803,7 @@ isa_freephysmem(caddr_t va, unsigned length) {
 			(*isaphysmemunblock)();
 	}
 }
-	
+
 #define NMI_PARITY (1 << 7)
 #define NMI_IOCHAN (1 << 6)
 #define ENMI_WATCHDOG (1 << 7)
@@ -856,7 +815,7 @@ isa_freephysmem(caddr_t va, unsigned length) {
  * return true to panic system, false to ignore.
  */
 int
-isa_nmi(cd) 
+isa_nmi(cd)
 	int cd;
 {
 	int isa_port = inb(0x61);
@@ -930,7 +889,7 @@ struct isa_device *find_isadev(table, driverp, unit)
   while ((table->id_driver != driverp) || (table->id_unit != unit)) {
     if (table->id_driver == 0)
       return NULL;
-    
+
     table++;
   }
 
@@ -961,6 +920,7 @@ update_intr_masks(void)
 		if (intr==2) continue;
 		maskptr = intr_mptr[intr];
 		if (!maskptr) continue;
+		*maskptr |= 1 << intr;
 		mask = *maskptr;
 		if (mask != intr_mask[intr]) {
 #if 0
@@ -1003,7 +963,7 @@ register_intr(intr, device_id, flags, handler, maskptr, unit)
 	intr_unit[intr] = unit;
 	setidt(ICU_OFFSET + intr,
 	       flags & RI_FAST ? fastintr[intr] : slowintr[intr],
-	       SDT_SYS386IGT, SEL_KPL);
+	       SDT_SYS386IGT, SEL_KPL, GSEL(GCODE_SEL, SEL_KPL));
 	write_eflags(ef);
 	for (cp = intrnames, id = 0; id <= device_id; id++)
 		while (*cp++ != '\0')
@@ -1050,7 +1010,8 @@ unregister_intr(intr, handler)
 	intr_mptr[intr] = NULL;
 	intr_mask[intr] = HWI_MASK | SWI_MASK;
 	intr_unit[intr] = intr;
-	setidt(ICU_OFFSET + intr, slowintr[intr], SDT_SYS386IGT, SEL_KPL);
+	setidt(ICU_OFFSET + intr, slowintr[intr], SDT_SYS386IGT, SEL_KPL,
+	    GSEL(GCODE_SEL, SEL_KPL));
 	write_eflags(ef);
 	return (0);
 }
