@@ -12,25 +12,25 @@
  *    documentation and/or other materials provided with the distribution.
  * 3. All advertising materials mentioning features or use of this software
  *    must display the following acknowledgement:
- *	This software is a component of "386BSD" developed by 
+ *	This software is a component of "386BSD" developed by
  *	William F. Jolitz, TeleMuse.
  * 4. Neither the name of the developer nor the name "386BSD"
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
- * THIS SOFTWARE IS A COMPONENT OF 386BSD DEVELOPED BY WILLIAM F. JOLITZ 
- * AND IS INTENDED FOR RESEARCH AND EDUCATIONAL PURPOSES ONLY. THIS 
- * SOFTWARE SHOULD NOT BE CONSIDERED TO BE A COMMERCIAL PRODUCT. 
- * THE DEVELOPER URGES THAT USERS WHO REQUIRE A COMMERCIAL PRODUCT 
+ * THIS SOFTWARE IS A COMPONENT OF 386BSD DEVELOPED BY WILLIAM F. JOLITZ
+ * AND IS INTENDED FOR RESEARCH AND EDUCATIONAL PURPOSES ONLY. THIS
+ * SOFTWARE SHOULD NOT BE CONSIDERED TO BE A COMMERCIAL PRODUCT.
+ * THE DEVELOPER URGES THAT USERS WHO REQUIRE A COMMERCIAL PRODUCT
  * NOT MAKE USE OF THIS WORK.
  *
  * FOR USERS WHO WISH TO UNDERSTAND THE 386BSD SYSTEM DEVELOPED
- * BY WILLIAM F. JOLITZ, WE RECOMMEND THE USER STUDY WRITTEN 
- * REFERENCES SUCH AS THE  "PORTING UNIX TO THE 386" SERIES 
- * (BEGINNING JANUARY 1991 "DR. DOBBS JOURNAL", USA AND BEGINNING 
- * JUNE 1991 "UNIX MAGAZIN", GERMANY) BY WILLIAM F. JOLITZ AND 
- * LYNNE GREER JOLITZ, AS WELL AS OTHER BOOKS ON UNIX AND THE 
- * ON-LINE 386BSD USER MANUAL BEFORE USE. A BOOK DISCUSSING THE INTERNALS 
+ * BY WILLIAM F. JOLITZ, WE RECOMMEND THE USER STUDY WRITTEN
+ * REFERENCES SUCH AS THE  "PORTING UNIX TO THE 386" SERIES
+ * (BEGINNING JANUARY 1991 "DR. DOBBS JOURNAL", USA AND BEGINNING
+ * JUNE 1991 "UNIX MAGAZIN", GERMANY) BY WILLIAM F. JOLITZ AND
+ * LYNNE GREER JOLITZ, AS WELL AS OTHER BOOKS ON UNIX AND THE
+ * ON-LINE 386BSD USER MANUAL BEFORE USE. A BOOK DISCUSSING THE INTERNALS
  * OF 386BSD ENTITLED "386BSD FROM THE INSIDE OUT" WILL BE AVAILABLE LATE 1992.
  *
  * THIS SOFTWARE IS PROVIDED BY THE DEVELOPER ``AS IS'' AND
@@ -46,7 +46,7 @@
  * SUCH DAMAGE.
  *
  *	from: unknown origin, 386BSD 0.1
- *	$Id: lpt.c,v 1.28 1995/04/12 20:47:56 wollman Exp $
+ * $FreeBSD: stable/2.1/sys/i386/isa/lpt.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -62,8 +62,10 @@
  * This driver sends two bytes (0x08, 0x00) in front of each packet,
  * to allow us to distinguish another format later.
  *
+ * Now added an Linux/Crynwr compatibility mode which is enabled using
+ * IF_LINK0 - Tim Wilkinson.
+ *
  * TODO:
- *    Make Linux/Crynwr compatible mode, use IF_LLC0 to enable.
  *    Make HDLC/PPP mode, use IF_LLC1 to enable.
  *
  * Connect the two computers using a Laplink parallel cable to use this
@@ -93,30 +95,35 @@
  * Expect transfer-rates up to 75 kbyte/sec.
  *
  * If GCC could correctly grok
- *	register int port asm("edx") 
+ *	register int port asm("edx")
  * the code would be cleaner
  *
- * Poul-Henning Kamp <phk@login.dkuug.dk>
+ * Poul-Henning Kamp <phk@freebsd.org>
  */
 
 #include "lpt.h"
-#if NLPT > 0
 
 #include <sys/param.h>
 #include <sys/systm.h>
+#include <sys/conf.h>
 #include <sys/proc.h>
-#include <sys/user.h>
 #include <sys/buf.h>
 #include <sys/kernel.h>
 #include <sys/ioctl.h>
-#include <sys/tty.h>
 #include <sys/uio.h>
 #include <sys/syslog.h>
 #include <sys/devconf.h>
+#ifdef DEVFS
+#include <sys/devfsext.h>
+#endif /*DEVFS*/
 
 #include <machine/clock.h>
 #include <machine/lpt.h>
 
+#include <vm/vm.h>
+#include <vm/vm_param.h>
+#include <vm/pmap.h>
+
 #include <i386/isa/isa.h>
 #include <i386/isa/isa_device.h>
 #include <i386/isa/lptreg.h>
@@ -132,10 +139,18 @@
 #include <netinet/in_systm.h>
 #include <netinet/in_var.h>
 #include <netinet/ip.h>
+#include <netinet/if_ether.h>
+#include "bpfilter.h"
+#if NBPFILTER > 0
+#include <net/bpf.h>
+#include <net/bpfdesc.h>
+#endif
 #endif /* INET */
 
+
 #define	LPINITRDY	4	/* wait up to 4 seconds for a ready */
-#define	LPTOUTTIME	4	/* wait up to 4 seconds for a ready */
+#define	LPTOUTINITIAL	10	/* initial timeout to wait for ready 1/10 s */
+#define	LPTOUTMAX	1	/* maximal timeout 1 s */
 #define	LPPRI		(PZERO+8)
 #define	BUFSIZE		1024
 
@@ -156,9 +171,17 @@
 #define	LPMAXERRS	100
 #endif
 
+#define CLPIPHDRLEN	14	/* We send dummy ethernet addresses (two) + packet type in front of packet */
+#define	CLPIP_SHAKE	0x80	/* This bit toggles between nibble reception */
+#define MLPIPHDRLEN	CLPIPHDRLEN
+
 #define LPIPHDRLEN	2	/* We send 0x08, 0x00 in front of packet */
-#define	LPIPTBLSIZE	256	/* Size of octet translation table */
 #define	LPIP_SHAKE	0x40	/* This bit toggles between nibble reception */
+#if !defined(MLPIPHDRLEN) || LPIPHDRLEN > MLPIPHDRLEN
+#define MLPIPHDRLEN	LPIPHDRLEN
+#endif
+
+#define	LPIPTBLSIZE	256	/* Size of octet translation table */
 
 #endif /* INET */
 
@@ -178,7 +201,7 @@ int lptflag = 1;
 #define	LPTUNIT(s)	((s)&0x03)
 #define	LPTFLAGS(s)	((s)&0xfc)
 
-struct lpt_softc {
+static struct lpt_softc {
 	short	sc_port;
 	short	sc_state;
 	/* default case: negative prime, negative ack, handshake strobe,
@@ -199,13 +222,16 @@ struct lpt_softc {
 #define LP_HAS_IRQ	0x01	/* we have an irq available */
 #define LP_USE_IRQ	0x02	/* we are using our irq */
 #define LP_ENABLE_IRQ	0x04	/* enable IRQ on open */
+	u_char	sc_backoff ;	/* time to call lptout() again */
 
 #ifdef INET
 	struct  ifnet	sc_if;
 	u_char		*sc_ifbuf;
 	int		sc_iferrs;
 #endif /* ENDIF */
-
+#ifdef	DEVFS
+	void	*devfs_token;
+#endif
 } lpt_sc[NLPT] ;
 
 /* bits for state */
@@ -233,9 +259,8 @@ struct lpt_softc {
 #define	MAX_SPIN	20	/* Max delay for device ready in usecs */
 
 static void	lptout (struct lpt_softc * sc);
-int		lptprobe (struct isa_device *dvp);
-int		lptattach (struct isa_device *isdp);
-void		lptintr (int unit);
+static int	lptprobe (struct isa_device *dvp);
+static int	lptattach (struct isa_device *isdp);
 
 #ifdef INET
 
@@ -245,9 +270,14 @@ static u_char *txmith;
 #define trecvh (txmith+(2*LPIPTBLSIZE))
 #define trecvl (txmith+(3*LPIPTBLSIZE))
 
+static u_char *ctxmith;
+#define ctxmitl (ctxmith+(1*LPIPTBLSIZE))
+#define ctrecvh (ctxmith+(2*LPIPTBLSIZE))
+#define ctrecvl (ctxmith+(3*LPIPTBLSIZE))
+
 /* Functions for the lp# interface */
 static void lpattach(struct lpt_softc *,int);
-static int lpinittables();
+static int lpinittables(void);
 static int lpioctl(struct ifnet *, int, caddr_t);
 static int lpoutput(struct ifnet *, struct mbuf *, struct sockaddr *,
 	struct rtentry *);
@@ -258,6 +288,20 @@ struct	isa_driver lptdriver = {
 	lptprobe, lptattach, "lpt"
 };
 
+d_open_t	lptopen;
+d_close_t	lptclose;
+d_write_t	lptwrite;
+d_ioctl_t	lptioctl;
+void		lptintr(int unit);
+
+#define CDEV_MAJOR 16
+#if 0
+static struct cdevsw lpt_cdevsw = 
+	{ lptopen,	lptclose,	noread,		lptwrite,	/*16*/
+	  lptioctl,	nullstop,	nullreset,	nodevtotty,/* lpt */
+	  seltrue,	nommap,		nostrat,	"lpt",	NULL,	-1 };
+#endif
+
 static struct kern_devconf kdc_lpt[NLPT] = { {
 	0, 0, 0,		/* filled in by dev_attach */
 	"lpt", 0, { MDDT_ISA, 0, "tty" },
@@ -370,9 +414,9 @@ lptprobe(struct isa_device *dvp)
 				(*(BIOS_PORTS+next_bios_lpt) != 0) ) {
 			dvp->id_iobase = *(BIOS_PORTS+next_bios_lpt++);
 			goto end_probe;
-		} else 
+		} else
 			return (0);
-	} 
+	}
 
 	/* Port was explicitly specified */
 	/* This allows probing of ports unknown to the BIOS */
@@ -380,11 +424,11 @@ lptprobe(struct isa_device *dvp)
 	port = dvp->id_iobase + lpt_data;
 	mask = 0xff;
 	data = 0x55;				/* Alternating zeros */
-	if (!lpt_port_test(port, data, mask)) 
+	if (!lpt_port_test(port, data, mask))
 		{ status = 0 ; goto end_probe ; }
 
 	data = 0xaa;				/* Alternating ones */
-	if (!lpt_port_test(port, data, mask)) 
+	if (!lpt_port_test(port, data, mask))
 		{ status = 0 ; goto end_probe ; }
 
 	for (i = 0; i < 8; i++)	{		/* Walking zero */
@@ -395,7 +439,7 @@ lptprobe(struct isa_device *dvp)
 
 	for (i = 0; i < 8; i++)	{		/* Walking one */
 		data = (1 << i);
-		if (!lpt_port_test(port, data, mask)) 
+		if (!lpt_port_test(port, data, mask))
 			{ status = 0 ; goto end_probe ; }
 	}
 
@@ -412,8 +456,11 @@ int
 lptattach(struct isa_device *isdp)
 {
 	struct	lpt_softc	*sc;
+	int	unit;
+	char	name[32];
 
-	sc = lpt_sc + isdp->id_unit;
+	unit = isdp->id_unit;
+	sc = lpt_sc + unit;
 	sc->sc_port = isdp->id_iobase;
 	sc->sc_primed = 0;	/* not primed yet */
 	outb(sc->sc_port+lpt_control, LPC_NINIT);
@@ -422,17 +469,25 @@ lptattach(struct isa_device *isdp)
 	lprintf("oldirq %x\n", sc->sc_irq);
 	if (isdp->id_irq) {
 		sc->sc_irq = LP_HAS_IRQ | LP_USE_IRQ | LP_ENABLE_IRQ;
-		printf("lpt%d: Interrupt-driven port\n", isdp->id_unit);
+		printf("lpt%d: Interrupt-driven port\n", unit);
 #ifdef INET
-		lpattach(sc, isdp->id_unit);
+		lpattach(sc, unit);
 #endif
 	} else {
 		sc->sc_irq = 0;
-		lprintf("lpt%d: Polled port\n", isdp->id_unit);
+		lprintf("lpt%d: Polled port\n", unit);
 	}
 	lprintf("irq %x\n", sc->sc_irq);
 
-	kdc_lpt[isdp->id_unit].kdc_state = DC_IDLE;
+	kdc_lpt[unit].kdc_state = DC_IDLE;
+
+#ifdef DEVFS
+/* XXX */ /* what to do about the flags in the minor number? */
+	sprintf(name,"lpt%d",unit);
+			/*	path name devsw minor type  uid gid perm*/
+	sc->devfs_token = devfs_add_devsw( "/", name, &lpt_cdevsw, unit,
+					DV_CHR, 0, 0, 0600);
+#endif
 	return (1);
 }
 
@@ -443,7 +498,7 @@ lptattach(struct isa_device *isdp)
  */
 
 int
-lptopen (dev_t dev, int flag)
+lptopen (dev_t dev, int flags, int fmt, struct proc *p)
 {
 	struct lpt_softc *sc;
 	int s;
@@ -455,7 +510,7 @@ lptopen (dev_t dev, int flag)
 		return (ENXIO);
 
 #ifdef INET
-	if (sc->sc_if.if_flags & IFF_UP) 
+	if (sc->sc_if.if_flags & IFF_UP)
 		return(EBUSY);
 #endif
 
@@ -512,7 +567,7 @@ lptopen (dev_t dev, int flag)
 			splx(s);
 			return (EBUSY);
 		}
-		
+
 		/* is printer online and ready for output */
 	} while ((inb(port+lpt_status) & (LPS_SEL|LPS_OUT|LPS_NBSY|LPS_NERR)) !=
 		 (LPS_SEL|LPS_NBSY|LPS_NERR));
@@ -537,7 +592,8 @@ lptopen (dev_t dev, int flag)
 	lprintf("irq %x\n", sc->sc_irq);
 	if (sc->sc_irq & LP_USE_IRQ) {
 		sc->sc_state |= TOUT;
-		timeout ((timeout_func_t)lptout, (caddr_t)sc, hz/2);
+		timeout ((timeout_func_t)lptout, (caddr_t)sc,
+			 (sc->sc_backoff = hz/LPTOUTINITIAL));
 	}
 
 	lprintf("opened.\n");
@@ -549,9 +605,12 @@ lptout (struct lpt_softc * sc)
 {	int pl;
 
 	lprintf ("T %x ", inb(sc->sc_port+lpt_status));
-	if (sc->sc_state & OPEN)
-		timeout ((timeout_func_t)lptout, (caddr_t)sc, hz/2);
-	else
+	if (sc->sc_state & OPEN) {
+		sc->sc_backoff++;
+		if (sc->sc_backoff > hz/LPTOUTMAX)
+			sc->sc_backoff = sc->sc_backoff > hz/LPTOUTMAX;
+		timeout ((timeout_func_t)lptout, (caddr_t)sc, sc->sc_backoff);
+	} else
 		sc->sc_state &= ~TOUT;
 
 	if (sc->sc_state & ERROR)
@@ -577,12 +636,12 @@ lptout (struct lpt_softc * sc)
  */
 
 int
-lptclose(dev_t dev, int flag)
+lptclose(dev_t dev, int flags, int fmt, struct proc *p)
 {
 	struct lpt_softc *sc = lpt_sc + LPTUNIT(minor(dev));
 	int port = sc->sc_port;
 
-	if(sc->sc_flags & LP_BYPASS) 
+	if(sc->sc_flags & LP_BYPASS)
 		goto end_close;
 
 	sc->sc_state &= ~OPEN;
@@ -593,7 +652,7 @@ lptclose(dev_t dev, int flag)
 		while ((inb(port+lpt_status) & (LPS_SEL|LPS_OUT|LPS_NBSY|LPS_NERR)) !=
 			(LPS_SEL|LPS_NBSY|LPS_NERR) || sc->sc_xfercnt)
 			/* wait 1/4 second, give up if we get a signal */
-			if (tsleep ((caddr_t)sc, LPPRI|PCATCH, 
+			if (tsleep ((caddr_t)sc, LPPRI|PCATCH,
 				"lpclose", hz) != EWOULDBLOCK)
 				break;
 
@@ -615,8 +674,8 @@ end_close:
  *
  *	This code is only used when we are polling the port
  */
-static int 
-pushbytes(struct lpt_softc * sc) 
+static int
+pushbytes(struct lpt_softc * sc)
 {
 	int spin, err, tic;
 	char ch;
@@ -668,15 +727,15 @@ pushbytes(struct lpt_softc * sc)
 	return(0);
 }
 
-/* 
- * lptwrite --copy a line from user space to a local buffer, then call 
+/*
+ * lptwrite --copy a line from user space to a local buffer, then call
  * putc to get the chars moved to the output queue.
  *
  * Flagging of interrupted write added.
  */
 
 int
-lptwrite(dev_t dev, struct uio * uio)
+lptwrite(dev_t dev, struct uio * uio, int ioflag)
 {
 	register unsigned n;
 	int pl, err;
@@ -704,14 +763,14 @@ lptwrite(dev_t dev, struct uio * uio)
 			}
 			lprintf("W ");
 			if (sc->sc_state & OBUSY)
-				if ((err = tsleep ((caddr_t)sc, 
+				if ((err = tsleep ((caddr_t)sc,
 					 LPPRI|PCATCH, "lpwrite", 0))) {
 					sc->sc_state |= INTERRUPTED;
 					return(err);
 				}
 		}
 		/* check to see if we must do a polled write */
-		if(!(sc->sc_irq & LP_USE_IRQ) && (sc->sc_xfercnt)) { 
+		if(!(sc->sc_irq & LP_USE_IRQ) && (sc->sc_xfercnt)) {
 			lprintf("p");
 			if((err = pushbytes(sc)))
 				return(err);
@@ -732,6 +791,7 @@ lptintr(int unit)
 {
 	struct lpt_softc *sc = lpt_sc + unit;
 	int port = sc->sc_port, sts;
+	int i;
 
 #ifdef INET
 	if(sc->sc_if.if_flags & IFF_UP) {
@@ -740,14 +800,24 @@ lptintr(int unit)
 	}
 #endif /* INET */
 
-	/* is printer online and ready for output */
-	if (((sts=inb(port+lpt_status)) & RDY_MASK) == LP_READY) {
+	/*
+	 * Is printer online and ready for output?
+	 *
+	 * Avoid falling back to lptout() too quickly.  First spin-loop
+	 * to see if the printer will become ready ``really soon now''.
+	 */
+	for (i = 0;
+	     i < 100 &&
+	     ((sts=inb(port+lpt_status)) & RDY_MASK) != LP_READY;
+	     i++) ;
+	if ((sts & RDY_MASK) == LP_READY) {
 		sc->sc_state = (sc->sc_state | OBUSY) & ~ERROR;
+		sc->sc_backoff = hz/LPTOUTINITIAL;
 
 		if (sc->sc_xfercnt) {
 			/* send char */
 			/*lprintf("%x ", *sc->sc_cp); */
-			outb(port+lpt_data, *sc->sc_cp++) ; 
+			outb(port+lpt_data, *sc->sc_cp++) ;
 			outb(port+lpt_control, sc->sc_control|LPC_STB);
 			/* DELAY(X) */
 			outb(port+lpt_control, sc->sc_control);
@@ -769,12 +839,13 @@ lptintr(int unit)
 		if(((sts & (LPS_NERR | LPS_OUT) ) != LPS_NERR) &&
 				(sc->sc_state & OPEN))
 			sc->sc_state |= ERROR;
+		/* lptout() will jump in and try to restart. */
 	}
 	lprintf("sts %x ", sts);
 }
 
 int
-lptioctl(dev_t dev, int cmd, caddr_t data, int flag)
+lptioctl(dev_t dev, int cmd, caddr_t data, int flags, struct proc *p)
 {
 	int	error = 0;
         struct	lpt_softc *sc;
@@ -786,12 +857,12 @@ lptioctl(dev_t dev, int cmd, caddr_t data, int flag)
 	switch (cmd) {
 	case LPT_IRQ :
 		if(sc->sc_irq & LP_HAS_IRQ) {
-			/* 
-			 * NOTE: 
+			/*
+			 * NOTE:
 			 * If the IRQ status is changed,
 			 * this will only be visible on the
 			 * next open.
-			 * 
+			 *
 			 * If interrupt status changes,
 			 * this gets syslog'd.
 			 */
@@ -800,14 +871,14 @@ lptioctl(dev_t dev, int cmd, caddr_t data, int flag)
 				sc->sc_irq &= (~LP_ENABLE_IRQ);
 			else
 				sc->sc_irq |= LP_ENABLE_IRQ;
-			if (old_sc_irq != sc->sc_irq ) 
+			if (old_sc_irq != sc->sc_irq )
 				log(LOG_NOTICE, "lpt%c switched to %s mode\n",
-					(char)unit+'0', 
+					(char)unit+'0',
 					(sc->sc_irq & LP_ENABLE_IRQ)?
 					"interrupt-driven":"polled");
 		} else /* polled port */
 			error = EOPNOTSUPP;
-		break;	
+		break;
 	default:
 		error = ENODEV;
 	}
@@ -825,19 +896,23 @@ lpattach (struct lpt_softc *sc, int unit)
 	ifp->if_name = "lp";
 	ifp->if_unit = unit;
 	ifp->if_mtu = LPMTU;
-	ifp->if_flags = IFF_SIMPLEX | IFF_POINTOPOINT;
+	ifp->if_flags = IFF_SIMPLEX | IFF_POINTOPOINT | IFF_MULTICAST;
 	ifp->if_ioctl = lpioctl;
 	ifp->if_output = lpoutput;
-	ifp->if_type = IFT_SLIP;
+	ifp->if_type = IFT_PARA;
 	ifp->if_hdrlen = 0;
 	ifp->if_addrlen = 0;
 	ifp->if_snd.ifq_maxlen = IFQ_MAXLEN;
 	if_attach(ifp);
 	printf("lp%d: TCP/IP capable interface\n", unit);
+
+#if NBPFILTER > 0
+	bpfattach(&ifp->if_bpf, ifp, DLT_NULL, LPIPHDRLEN);
+#endif
 }
 /*
  * Build the translation tables for the LPIP (BSD unix) protocol.
- * We don't want to calculate these nasties in our tight loop, so we 
+ * We don't want to calculate these nasties in our tight loop, so we
  * precalculate them when we initialize.
  */
 static int
@@ -851,6 +926,19 @@ lpinittables (void)
     if (!txmith)
 	return 1;
 
+    if (!ctxmith)
+	ctxmith = malloc(4*LPIPTBLSIZE, M_DEVBUF, M_NOWAIT);
+
+    if (!ctxmith)
+	return 1;
+
+    for (i=0; i < LPIPTBLSIZE; i++) {
+	ctxmith[i] = (i & 0xF0) >> 4;
+	ctxmitl[i] = 0x10 | (i & 0x0F);
+	ctrecvh[i] = (i & 0x78) << 1;
+	ctrecvl[i] = (i & 0x78) >> 3;
+    }
+
     for (i=0; i < LPIPTBLSIZE; i++) {
 	txmith[i] = ((i & 0x80) >> 3) | ((i & 0x70) >> 4) | 0x08;
 	txmitl[i] = ((i & 0x08) << 1) | (i & 0x07);
@@ -871,7 +959,7 @@ lpioctl (struct ifnet *ifp, int cmd, caddr_t data)
 {
     struct lpt_softc *sc = lpt_sc + ifp->if_unit;
     struct ifaddr *ifa = (struct ifaddr *)data;
-    struct ifreq *ifr = (struct ifreq *)data; 
+    struct ifreq *ifr = (struct ifreq *)data;
     u_char *ptr;
 
     switch (cmd) {
@@ -892,7 +980,7 @@ lpioctl (struct ifnet *ifp, int cmd, caddr_t data)
 	if (((ifp->if_flags & IFF_UP)) && (!(ifp->if_flags & IFF_RUNNING))) {
 	    if (lpinittables())
 		return ENOBUFS;
-	    sc->sc_ifbuf = malloc(sc->sc_if.if_mtu + LPIPHDRLEN,
+	    sc->sc_ifbuf = malloc(sc->sc_if.if_mtu + MLPIPHDRLEN,
 				  M_DEVBUF, M_WAITOK);
 	    if (!sc->sc_ifbuf)
 		return ENOBUFS;
@@ -904,18 +992,35 @@ lpioctl (struct ifnet *ifp, int cmd, caddr_t data)
 
     case SIOCSIFMTU:
 	ptr = sc->sc_ifbuf;
-	sc->sc_ifbuf = malloc(ifr->ifr_metric+LPIPHDRLEN, M_DEVBUF, M_NOWAIT);
+	sc->sc_ifbuf = malloc(ifr->ifr_mtu+MLPIPHDRLEN, M_DEVBUF, M_NOWAIT);
 	if (!sc->sc_ifbuf) {
 	    sc->sc_ifbuf = ptr;
 	    return ENOBUFS;
-	} 
+	}
 	if (ptr)
 	    free(ptr,M_DEVBUF);
-	sc->sc_if.if_mtu = ifr->ifr_metric; 
+	sc->sc_if.if_mtu = ifr->ifr_mtu;
 	break;
 
     case SIOCGIFMTU:
-	ifr->ifr_metric = sc->sc_if.if_mtu;
+	ifr->ifr_mtu = sc->sc_if.if_mtu;
+	break;
+
+    case SIOCADDMULTI:
+    case SIOCDELMULTI:
+	if (ifr == 0) {
+	    return EAFNOSUPPORT;		/* XXX */
+	}
+	switch (ifr->ifr_addr.sa_family) {
+
+#ifdef INET
+	case AF_INET:
+	    break;
+#endif
+
+	default:
+	    return EAFNOSUPPORT;
+	}
 	break;
 
     default:
@@ -925,6 +1030,44 @@ lpioctl (struct ifnet *ifp, int cmd, caddr_t data)
     return 0;
 }
 
+static inline int
+clpoutbyte (u_char byte, int spin, int data_port, int status_port)
+{
+	outb(data_port, ctxmitl[byte]);
+	while (inb(status_port) & CLPIP_SHAKE)
+		if (--spin == 0) {
+			return 1;
+		}
+	outb(data_port, ctxmith[byte]);
+	while (!(inb(status_port) & CLPIP_SHAKE))
+		if (--spin == 0) {
+			return 1;
+		}
+	return 0;
+}
+
+static inline int
+clpinbyte (int spin, int data_port, int status_port)
+{
+	int c, cl;
+
+	while((inb(status_port) & CLPIP_SHAKE))
+	    if(!--spin) {
+		return -1;
+	    }
+	cl = inb(status_port);
+	outb(data_port, 0x10);
+
+	while(!(inb(status_port) & CLPIP_SHAKE))
+	    if(!--spin) {
+		return -1;
+	    }
+	c = inb(status_port);
+	outb(data_port, 0x00);
+
+	return (ctrecvl[cl] | ctrecvh[c]);
+}
+
 static void
 lpintr (int unit)
 {
@@ -939,6 +1082,59 @@ lpintr (int unit)
 
 	s = splhigh();
 
+	if (sc->sc_if.if_flags & IFF_LINK0) {
+
+	    /* Ack. the request */
+	    outb(lpt_data_port, 0x01);
+
+	    /* Get the packet length */
+	    j = clpinbyte(LPMAXSPIN2, lpt_data_port, lpt_stat_port);
+	    if (j == -1)
+		goto err;
+	    len = j;
+	    j = clpinbyte(LPMAXSPIN2, lpt_data_port, lpt_stat_port);
+	    if (j == -1)
+		goto err;
+	    len = len + (j << 8);
+	    if (len > sc->sc_if.if_mtu + MLPIPHDRLEN)
+		goto err;
+
+	    bp  = sc->sc_ifbuf;
+	
+	    while (len--) {
+	        j = clpinbyte(LPMAXSPIN2, lpt_data_port, lpt_stat_port);
+	        if (j == -1) {
+		    goto err;
+	        }
+	        *bp++ = j;
+	    }
+	    /* Get and ignore checksum */
+	    j = clpinbyte(LPMAXSPIN2, lpt_data_port, lpt_stat_port);
+	    if (j == -1) {
+	        goto err;
+	    }
+
+	    len = bp - sc->sc_ifbuf;
+	    if (len <= CLPIPHDRLEN)
+	        goto err;
+
+	    sc->sc_iferrs = 0;
+
+	    if (IF_QFULL(&ipintrq)) {
+	        lprintf("DROP");
+	        IF_DROP(&ipintrq);
+		goto done;
+	    }
+	    len -= CLPIPHDRLEN;
+	    sc->sc_if.if_ipackets++;
+	    sc->sc_if.if_ibytes += len;
+	    top = m_devget(sc->sc_ifbuf + CLPIPHDRLEN, len, 0, &sc->sc_if, 0);
+	    if (top) {
+	        IF_ENQUEUE(&ipintrq, top);
+	        schednetisr(NETISR_IP);
+	    }
+	    goto done;
+	}
 	while ((inb(lpt_stat_port) & LPIP_SHAKE)) {
 	    len = sc->sc_if.if_mtu + LPIPHDRLEN;
 	    bp  = sc->sc_ifbuf;
@@ -978,6 +1174,11 @@ lpintr (int unit)
 		IF_DROP(&ipintrq);
 		goto done;
 	    }
+#if NBPFILTER > 0
+	    if (sc->sc_if.if_bpf) {
+		bpf_tap(sc->sc_if.if_bpf, sc->sc_ifbuf, len);
+	    }
+#endif
 	    len -= LPIPHDRLEN;
 	    sc->sc_if.if_ipackets++;
 	    sc->sc_if.if_ibytes += len;
@@ -1014,20 +1215,19 @@ lpintr (int unit)
 static inline int
 lpoutbyte (u_char byte, int spin, int data_port, int status_port)
 {
-	outb(data_port, txmith[byte]);
-	while (!(inb(status_port) & LPIP_SHAKE))
-		if (--spin == 0)
-			return 1;
-	outb(data_port, txmitl[byte]);
-	while (inb(status_port) & LPIP_SHAKE)
-		if (--spin == 0)
-			return 1;
-
-	return 0;
+    outb(data_port, txmith[byte]);
+    while (!(inb(status_port) & LPIP_SHAKE))
+	if (--spin == 0)
+		return 1;
+    outb(data_port, txmitl[byte]);
+    while (inb(status_port) & LPIP_SHAKE)
+	if (--spin == 0)
+		return 1;
+    return 0;
 }
 
-static int 
-lpoutput (struct ifnet *ifp, struct mbuf *m, 
+static int
+lpoutput (struct ifnet *ifp, struct mbuf *m,
 	  struct sockaddr *dst, struct rtentry *rt)
 {
     register int lpt_data_port = lpt_sc[ifp->if_unit].sc_port + lpt_data;
@@ -1037,8 +1237,12 @@ lpoutput (struct ifnet *ifp, struct mbuf *m,
     int s, err;
     struct mbuf *mm;
     u_char *cp = "\0\0";
+    u_char chksum = 0;
+    int count = 0;
+    int i;
+    int spin;
 
-    /* We need a sensible value if we abort */	
+    /* We need a sensible value if we abort */
     cp++;
     ifp->if_flags |= IFF_RUNNING;
 
@@ -1049,27 +1253,106 @@ lpoutput (struct ifnet *ifp, struct mbuf *m,
     /* Suspend (on laptops) or receive-errors might have taken us offline */
     outb(lpt_ctrl_port, LPC_ENA);
 
+    if (ifp->if_flags & IFF_LINK0) {
+
+	if (!(inb(lpt_stat_port) & CLPIP_SHAKE)) {
+	    lprintf("&");
+	    lptintr(ifp->if_unit);
+	}
+
+	/* Alert other end to pending packet */
+	spin = LPMAXSPIN1;
+	outb(lpt_data_port, 0x08);
+	while ((inb(lpt_stat_port) & 0x08) == 0)
+		if (--spin == 0) {
+			goto nend;
+		}
+
+	/* Calculate length of packet, then send that */
+
+	count += 14;		/* Ethernet header len */
+
+	mm = m;
+	for (mm = m; mm; mm = mm->m_next) {
+		count += mm->m_len;
+	}
+	if (clpoutbyte(count & 0xFF, LPMAXSPIN1, lpt_data_port, lpt_stat_port))
+		goto nend;
+	if (clpoutbyte((count >> 8) & 0xFF, LPMAXSPIN1, lpt_data_port, lpt_stat_port))
+		goto nend;
+
+	/* Send dummy ethernet header */
+	for (i = 0; i < 12; i++) {
+		if (clpoutbyte(i, LPMAXSPIN1, lpt_data_port, lpt_stat_port))
+			goto nend;
+		chksum += i;
+	}
+
+	if (clpoutbyte(0x08, LPMAXSPIN1, lpt_data_port, lpt_stat_port))
+		goto nend;
+	if (clpoutbyte(0x00, LPMAXSPIN1, lpt_data_port, lpt_stat_port))
+		goto nend;
+	chksum += 0x08 + 0x00;		/* Add into checksum */
+
+	mm = m;
+	do {
+		cp = mtod(mm, u_char *);
+		while (mm->m_len--) {
+			chksum += *cp;
+			if (clpoutbyte(*cp++, LPMAXSPIN2, lpt_data_port, lpt_stat_port))
+				goto nend;
+		}
+	} while ((mm = mm->m_next));
+
+	/* Send checksum */
+	if (clpoutbyte(chksum, LPMAXSPIN2, lpt_data_port, lpt_stat_port))
+		goto nend;
+
+	/* Go quiescent */
+	outb(lpt_data_port, 0);
+
+	err = 0;			/* No errors */
+
+	nend:
+	if (err)  {				/* if we didn't timeout... */
+		ifp->if_oerrors++;
+		lprintf("X");
+	} else {
+		ifp->if_opackets++;
+		ifp->if_obytes += m->m_pkthdr.len;
+	}
+
+	m_freem(m);
+
+	if (!(inb(lpt_stat_port) & CLPIP_SHAKE)) {
+		lprintf("^");
+		lptintr(ifp->if_unit);
+	}
+	(void) splx(s);
+	return 0;
+    }
+
     if (inb(lpt_stat_port) & LPIP_SHAKE) {
-	lprintf("&");
-	lptintr(ifp->if_unit);
+        lprintf("&");
+        lptintr(ifp->if_unit);
     }
 
     if (lpoutbyte(0x08, LPMAXSPIN1, lpt_data_port, lpt_stat_port))
-	goto end;
+        goto end;
     if (lpoutbyte(0x00, LPMAXSPIN2, lpt_data_port, lpt_stat_port))
-	goto end;
+        goto end;
 
     mm = m;
     do {
-	cp = mtod(mm,u_char *);
-	while (mm->m_len--)
+        cp = mtod(mm,u_char *);
+        while (mm->m_len--)
 	    if (lpoutbyte(*cp++, LPMAXSPIN2, lpt_data_port, lpt_stat_port))
-		goto end;
+	        goto end;
     } while ((mm = mm->m_next));
 
     err = 0;				/* no errors were encountered */
 
-end:
+    end:
     --cp;
     outb(lpt_data_port, txmitl[*cp] ^ 0x17);
 
@@ -1079,6 +1362,25 @@ end:
     } else {
 	ifp->if_opackets++;
 	ifp->if_obytes += m->m_pkthdr.len;
+#if NBPFILTER > 0
+	if (ifp->if_bpf) {
+	    /*
+	     * We need to prepend the packet type as
+	     * a two byte field.  Cons up a dummy header
+	     * to pacify bpf.  This is safe because bpf
+	     * will only read from the mbuf (i.e., it won't
+	     * try to free it or keep a pointer to it).
+	     */
+	    struct mbuf m0;
+	    u_short hdr = 0x800;
+
+	    m0.m_next = m;
+	    m0.m_len = 2;
+	    m0.m_data = (char *)&hdr;
+
+	    bpf_mtap(ifp->if_bpf, &m0);
+	}
+#endif
     }
 
     m_freem(m);
@@ -1093,5 +1395,3 @@ end:
 }
 
 #endif /* INET */
-
-#endif	/* NLPT */
