@@ -41,18 +41,18 @@
 	    conditions even if it is unsupported, not a complete system
 	    and/or does not contain compiled code.
 	8.  These conditions will be in force for the full life of the
-	    copyright.  
+	    copyright.
 	9.  If all the above conditions are met, modifications to other
 	    parts of this file may be freely made, although any person
 	    or persons making changes do not receive the right to add their
 	    name or names to the copyright strings and notices in this
 	    software.  Persons making changes are encouraged to insert edit
 	    history in matcd.c and to put your name and details of the
-	    change there.  
+	    change there.
 	10. You must have prior written permission from the author to
 	    deviate from these terms.
 
-	Vendors who produce product(s) containing this code are encouraged 
+	Vendors who produce product(s) containing this code are encouraged
 	(but not required) to provide copies of the finished product(s) to
 	the author and to correspond with the author about development
 	activity relating to this code.   Donations of development hardware
@@ -204,7 +204,7 @@ static int matcd_pause(int ldrive, int cdrive, int controller, int action)
 
 	lockbus(controller, ldrive);	/*<16>Request bus*/
 	matcd_slowcmd(port,ldrive,cdrive,cmd);	/*<14>*/
-	i=waitforit(10*TICKRES,DTEN,port,"sense");	/*<14>*/
+	i=waitforit(10*TICKRES,DTEN,port,"matpau");	/*<25>*/
 	z=get_stat(port,ldrive);	/*<14>Read status byte*/
 	if ((z & MATCD_ST_ERROR)) {	/*<14>Something went wrong*/
 		i=get_error(port, ldrive, cdrive);	/*<14>*/
@@ -321,7 +321,7 @@ static int matcd_level(int ldrive, int cdrive, int controller,
 		cmd[1]=AUDIOPARM;	/*Audio/routing settings*/
 		lockbus(controller, ldrive);	/*<16>Request bus*/
 		matcd_slowcmd(port,ldrive,cdrive,cmd);
-		i=waitforit(10*TICKRES,DTEN,port,"sense");
+		i=waitforit(10*TICKRES,DTEN,port,"matlvl");	/*<25>*/
 		matcd_pread(port, 5, data);	/*Read data returned*/
 		z=get_stat(port,ldrive);/*Read status byte*/
 		unlockbus(controller, ldrive);	/*<16>Release bus*/
@@ -481,7 +481,7 @@ static int matcd_pitch(int ldrive, int cdrive, int controller,
 	"play slower", 0x0000 flat and 1 to 32767 being "play faster"
 	within the scale allowed by the device.  The value is scaled to fit
 	the allowed by the device and any excess is treated as being
-	the positive or negative limit.  No ioctl input value is considered 
+	the positive or negative limit.  No ioctl input value is considered
 	invalid.
 
 	This device has a +/- 13% playback pitch specified by a range
