@@ -32,7 +32,7 @@
  * SUCH DAMAGE.
  *
  *	from: @(#)npx.c	7.2 (Berkeley) 5/12/91
- *	$Id: npx.c,v 1.21 1995/03/05 04:06:44 wpaul Exp $
+ * $FreeBSD: stable/2.1/sys/i386/isa/npx.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include "npx.h"
@@ -205,8 +205,8 @@ npxprobe(dvp)
 	save_idt_npxtrap = idt[16];
 	outb(IO_ICU1 + 1, ~(IRQ_SLAVE | dvp->id_irq));
 	outb(IO_ICU2 + 1, ~(dvp->id_irq >> 8));
-	setidt(16, probetrap, SDT_SYS386TGT, SEL_KPL);
-	setidt(npx_intrno, probeintr, SDT_SYS386IGT, SEL_KPL);
+	setidt(16, probetrap, SDT_SYS386TGT, SEL_KPL, GSEL(GCODE_SEL, SEL_KPL));
+	setidt(npx_intrno, probeintr, SDT_SYS386IGT, SEL_KPL, GSEL(GCODE_SEL, SEL_KPL));
 	npx_idt_probeintr = idt[npx_intrno];
 	enable_intr();
 	result = npxprobe1(dvp);
@@ -280,7 +280,7 @@ npxprobe1(dvp)
 		/*
 		 * Good, now check for a proper control word.
 		 */
-		control = 0x5a5a;	
+		control = 0x5a5a;
 		fnstcw(&control);
 		if ((control & 0x1f3f) == 0x033f) {
 			hw_float = npx_exists = 1;
@@ -344,7 +344,7 @@ npxattach(dvp)
 		printf("npx%d: INT 16 interface\n", dvp->id_unit);
 	else if (npx_irq13)
 		;		/* higher level has printed "irq 13" */
-#if defined(MATH_EMULATE) || defined(GPL_MATH_EMULATE) 
+#if defined(MATH_EMULATE) || defined(GPL_MATH_EMULATE)
 	else if (npx_exists) {
 		printf("npx%d: error reporting broken; using 387 emulator\n",
 			dvp->id_unit);
