@@ -81,7 +81,9 @@
 #include <sys/syslog.h>
 #include <sys/malloc.h>
 #include <sys/time.h>
-
+#if PCVT_FREEBSD > 210
+#include <machine/random.h>
+#endif  /* PCVT_FREEBSD > 210 */
 #else /* ! PCVT_FREEBSD >= 200 */
 
 #include "param.h"
@@ -111,7 +113,7 @@
 #include "i386/cpufunc.h"
 #elif PCVT_FREEBSD >= 200
 #include <i386/isa/isa_device.h>
-#else 
+#else
 #include "i386/isa/isa_device.h"
 #endif
 
@@ -225,7 +227,7 @@ in the config file"
 #endif
 
 #else /* XSERVER */
-  
+
 #if PCVT_USL_VT_COMPAT
 #warning "Option PCVT_USL_VT_COMPAT meaningless without XSERVER"
 #undef PCVT_USL_VT_COMPAT
@@ -563,11 +565,11 @@ in the config file"
 /*---------------------------------------------------------------------------*
  *	VGA palette handling (output DAC palette)
  *---------------------------------------------------------------------------*/
- 
+
 #define VGA_DAC		0x3C6		/* vga dac address */
 #define VGA_PMSK	0x3F		/* palette mask, 64 distinct values */
 #define NVGAPEL 	256		/* number of palette entries */
- 
+
 /*---------------------------------------------------------------------------*
  *	function key labels
  *---------------------------------------------------------------------------*/
@@ -815,7 +817,7 @@ typedef struct video_state {
 	u_char	sc_row;			/* saved row */
 	u_char	sc_col;			/* saved col */
 	u_short sc_cur_offset;		/* saved cursor addr offset */
-	u_short	sc_attr;		/* saved attributes */	
+	u_short	sc_attr;		/* saved attributes */
 	u_char	sc_vtsgr;		/* saved sgr configuration */
 	u_char	sc_awm;			/* saved auto wrap mode */
 	u_char	sc_om;			/* saved origin mode */
@@ -824,7 +826,7 @@ typedef struct video_state {
 	u_short	*sc_G2;			/* save G2 ptr */
 	u_short	*sc_G3;			/* save G3 ptr */
 	u_short	*sc_GL;			/* save GL ptr */
-	u_short	*sc_GR;			/* save GR ptr */	
+	u_short	*sc_GR;			/* save GR ptr */
 	u_char	sc_sel;			/* selective erase state */
 	u_char	ufkl[8][17];		/* user fkey-labels */
 	u_char	sfkl[8][17];		/* system fkey-labels */
@@ -837,7 +839,7 @@ typedef struct video_state {
 	u_char	attribute;		/* attribute normal, tx only, local */
 	u_char	key;			/* fkey label no */
 	u_char	l_len;			/* buffer length's */
-	u_char	s_len;	
+	u_char	s_len;
 	u_char	m_len;
 	u_char	i;			/* help (got short of names ...) */
 	u_char	l_buf[MAX_LABEL+1]; 	/* buffers */
@@ -1143,14 +1145,14 @@ extern u_char		kbd_polling;
 extern u_char		keyboard_show;
 #endif /* PCVT_SHOWKEYS */
 
-u_char	cursor_pos_valid;
+extern	u_char	cursor_pos_valid;
 
-u_char	critical_scroll;
-int	switch_page;
+extern	u_char	critical_scroll;
+extern	int	switch_page;
 
 #if PCVT_SCREENSAVER
-u_char	reset_screen_saver;
-u_char	scrnsv_active;
+extern	u_char	reset_screen_saver;
+extern	u_char	scrnsv_active;
 #endif /* PCVT_SCREENSAVER */
 
 extern u_char		sgr_tab_color[];
@@ -1411,7 +1413,7 @@ static __inline void vt_selattr(struct video_state *svsp)
 	{ u_char x = inb(0x84); } \
 	{ u_char x = inb(0x84); } \
 	{ u_char x = inb(0x84); } \
-	{ u_char x = inb(0x84); }	
+	{ u_char x = inb(0x84); }
 
 #else /* PCVT_PORTIO_DELAY */
 				/* use system supplied delay function for */
