@@ -31,7 +31,7 @@
  * SUCH DAMAGE.
  *
  *	from: @(#)com.c	7.5 (Berkeley) 5/16/91
- *	$Id: sio.c,v 1.97 1995/05/07 23:00:02 ache Exp $
+ * $FreeBSD: stable/2.1/sys/i386/isa/sio.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include "sio.h"
@@ -46,9 +46,7 @@
 #include <sys/systm.h>
 #include <sys/reboot.h>
 #include <sys/ioctl.h>
-#define	TTYDEFCHARS		/* XXX TK2.0 */
 #include <sys/tty.h>
-#undef	TTYDEFCHARS
 #include <sys/proc.h>
 #include <sys/user.h>
 #include <sys/conf.h>
@@ -68,20 +66,6 @@
 #include <i386/isa/sioreg.h>
 #include <i386/isa/ic/ns16550.h>
 
-/*
- * XXX temporary kludges for 2.0 (XXX TK2.0).
- */
-#define	TSA_CARR_ON(tp)		((void *)&(tp)->t_rawq)
-#define	TSA_OCOMPLETE(tp)	((void *)&(tp)->t_outq)
-#define	TSA_OLOWAT(tp)		((void *)&(tp)->t_outq)
-void
-termioschars(t)
-	struct termios *t;
-{
-
-	bcopy(ttydefchars, t->c_cc, sizeof t->c_cc);
-}
-
 #define	LOTS_OF_EVENTS	64	/* helps separate urgent events from input */
 #define	RB_I_HIGH_WATER	(TTYHOG - 2 * RS_IBUFSIZE)
 #define	RS_IBUFSIZE	256
@@ -103,6 +87,7 @@ termioschars(t)
 #define	COM_NOTAST4(dev)	((dev)->id_flags & 0x04)
 #endif /* COM_MULTIPORT */
 
+#define	COM_LOSESOUTINTS(dev)	((dev)->id_flags & 0x08)
 #define	COM_NOFIFO(dev)		((dev)->id_flags & 0x02)
 #define	COM_VERBOSE(dev)	((dev)->id_flags & 0x80)
 
@@ -127,8 +112,9 @@ termioschars(t)
  * off the low bits.
  *
  * The following com and tty flags correspond closely:
- *	CS_BUSY		= TS_BUSY (maintained by comstart() and comflush())
- *	CS_TTGO		= ~TS_TTSTOP (maintained by comstart() and siostop())
+ *	CS_BUSY		= TS_BUSY (maintained by comstart(), siopoll() and
+ *				   siostop())
+ *	CS_TTGO		= ~TS_TTSTOP (maintained by comparam() and comstart())
  *	CS_CTS_OFLOW	= CCTS_OFLOW (maintained by comparam())
  *	CS_RTS_IFLOW	= CRTS_IFLOW (maintained by comparam())
  * TS_FLUSH is not used.
@@ -160,6 +146,14 @@ static	char const * const	error_desc[] = {
 typedef u_int	Port_t;		/* hardware port */
 typedef u_char	bool_t;		/* boolean */
 
+/* queue of linear buffers */
+struct lbq {
+	u_char	*l_head;	/* next char to process */
+	u_char	*l_tail;	/* one past the last char to process */
+	struct lbq *l_next;	/* next in queue */
+	bool_t	l_queued;	/* nonzero if queued */
+};
+
 /* com device structure */
 struct com_s {
 	u_char	state;		/* miscellaneous flag bits */
@@ -169,12 +163,14 @@ struct com_s {
 	u_char	ftl_init;	/* ftl_max for next open() */
 	u_char	ftl_max;	/* maximum ftl for curent open() */
 	bool_t	hasfifo;	/* nonzero for 16550 UARTs */
+	bool_t	loses_outints;	/* nonzero if device loses output interrupts */
 	u_char	mcr_image;	/* copy of value written to MCR */
 #ifdef COM_MULTIPORT
 	bool_t	multiport;	/* is this unit part of a multiport device? */
 #endif /* COM_MULTIPORT */
 	bool_t	no_irq;		/* nonzero if irq is not attached */
 	bool_t	poll;		/* nonzero if polling is required */
+	bool_t	poll_output;	/* nonzero if polling for output is required */
 	int	unit;		/* unit	number */
 	int	dtr_wait;	/* time to hold DTR down on close (* 1/hz) */
 	u_int	tx_fifo_size;
@@ -195,8 +191,8 @@ struct com_s {
 	u_char	*ihighwater;	/* threshold in input buffer */
 	u_char	*iptr;		/* next free spot in input buffer */
 
-	u_char	*obufend;	/* end of output buffer */
-	u_char	*optr;		/* next char to output */
+	struct lbq	obufq;	/* head of queue of output buffers */
+	struct lbq	obufs[2];	/* output buffers */
 
 	Port_t	data_port;	/* i/o ports */
 	Port_t	int_id_port;
@@ -232,9 +228,11 @@ struct com_s {
 	u_char	ibuf2[2 * RS_IBUFSIZE];
 
 	/*
-	 * Output buffer.  Someday we should avoid copying.  Twice.
+	 * Data area for output buffers.  Someday we should build the output
+	 * buffer queue without copying data.
 	 */
-	u_char	obuf[256];
+	u_char	obuf1[256];
+	u_char	obuf2[256];
 };
 
 /*
@@ -264,17 +262,17 @@ struct tty *siodevtotty	__P((dev_t dev));
 
 static	int	sioattach	__P((struct isa_device *dev));
 static	timeout_t siodtrwakeup;
-static	void	comflush	__P((struct com_s *com));
 static	void	comhardclose	__P((struct com_s *com));
 static	void	siointr1	__P((struct com_s *com));
-static	void	commctl		__P((struct com_s *com, int bits, int how));
+static	int	commctl		__P((struct com_s *com, int bits, int how));
 static	int	comparam	__P((struct tty *tp, struct termios *t));
 static	int	sioprobe	__P((struct isa_device *dev));
 static	void	sioregisterdev	__P((struct isa_device *id));
+static	void	siosettimeout	__P((void));
 static	void	comstart	__P((struct tty *tp));
 static	timeout_t comwakeup;
-static	int	tiocm_xxx2mcr	__P((int tiocm_xxx));
-static	void	disc_optim	__P((struct tty	*tp, struct termios *t,	struct com_s *com));
+static	void	disc_optim	__P((struct tty	*tp, struct termios *t,
+				     struct com_s *com));
 
 #ifdef DSI_SOFT_MODEM
 static  int 	LoadSoftModem   __P((int unit,int base_io, u_long size, u_char *ptr));
@@ -290,23 +288,17 @@ struct isa_driver	siodriver = {
 	sioprobe, sioattach, "sio"
 };
 
-#ifdef COMCONSOLE
-#undef COMCONSOLE
-#define	COMCONSOLE	1
-#else
-#define	COMCONSOLE	0
-#endif
-
-static	int	comconsole = CONUNIT;
+static	int	comconsole = -1;
 static	speed_t	comdefaultrate = TTYDEF_SPEED;
 static	u_int	com_events;	/* input chars + weighted output completions */
 static	int	commajor;
-#if 0 /* XXX TK2.0 */
-struct tty	*sio_tty[NSIO];
+static	int	sio_timeout;
+static	int	sio_timeouts_until_log;
+#if 0 /* XXX */
+static struct tty	*sio_tty[NSIO];
 #else
-/* must be not static, used by pstat -t */
-struct tty	sio_tty[NSIO];
-int             nsio_tty = NSIO;
+static struct tty	sio_tty[NSIO];
+static	int	nsio_tty = NSIO;
 #endif
 
 #ifdef KGDB
@@ -339,9 +331,6 @@ static	struct speedtab comspeedtab[] = {
 	-1,	-1
 };
 
-/* XXX - configure this list */
-static Port_t likely_com_ports[] = { 0x3f8, 0x2f8, 0x3e8, 0x2e8, };
-
 static struct kern_devconf kdc_sio[NSIO] = { {
 	0, 0, 0,		/* filled in by dev_attach */
 	"sio", 0, { MDDT_ISA, 0, "tty" },
@@ -372,13 +361,13 @@ sioprobe(dev)
 	struct isa_device	*dev;
 {
 	static bool_t	already_init;
-	Port_t		*com_ptr;
 	bool_t		failures[10];
 	int		fn;
 	struct isa_device	*idev;
 	Port_t		iobase;
 	u_char		mcr_image;
 	int		result;
+	struct isa_device	*xdev;
 
 	sioregisterdev(dev);
 
@@ -389,11 +378,9 @@ sioprobe(dev)
 		 * from any used port that shares the interrupt vector.
 		 * XXX the gate enable is elsewhere for some multiports.
 		 */
-		for (com_ptr = likely_com_ports;
-		     com_ptr < &likely_com_ports[sizeof likely_com_ports
-						 / sizeof likely_com_ports[0]];
-		     ++com_ptr)
-			outb(*com_ptr + com_mcr, 0);
+		for (xdev = isa_devtab_tty; xdev->id_driver != NULL; xdev++)
+			if (xdev->id_driver == &siodriver && xdev->id_enabled)
+				outb(xdev->id_iobase + com_mcr, 0);
 		already_init = TRUE;
 	}
 
@@ -456,7 +443,7 @@ sioprobe(dev)
 	 * XXX what about the UART bug avoided by waiting in comparam()?
 	 * We don't want to to wait long enough to drain at 2 bps.
 	 */
-	outb(iobase + com_cfcr, CFCR_DLAB);
+	outb(iobase + com_cfcr, CFCR_DLAB | CFCR_8BITS);
 	outb(iobase + com_dlbl, COMBRD(9600) & 0xff);
 	outb(iobase + com_dlbh, (u_int) COMBRD(9600) >> 8);
 	outb(iobase + com_cfcr, CFCR_8BITS);
@@ -562,7 +549,6 @@ sioattach(isdp)
 	struct isa_device	*isdp;
 {
 	struct com_s	*com;
-	static bool_t	comwakeup_started = FALSE;
 	Port_t		iobase;
 	int		s;
 	int		unit;
@@ -590,11 +576,15 @@ sioattach(isdp)
 	com->unit = unit;
 	com->cfcr_image = CFCR_8BITS;
 	com->dtr_wait = 3 * hz;
+	com->loses_outints = COM_LOSESOUTINTS(isdp) != 0;
 	com->no_irq = isdp->id_irq == 0;
 	com->tx_fifo_size = 1;
 	com->iptr = com->ibuf = com->ibuf1;
 	com->ibufend = com->ibuf1 + RS_IBUFSIZE;
 	com->ihighwater = com->ibuf1 + RS_IHIGHWATER;
+	com->obufs[0].l_head = com->obuf1;
+	com->obufs[1].l_head = com->obuf2;
+
 	com->iobase = iobase;
 	com->data_port = iobase + com_data;
 	com->int_id_port = iobase + com_iir;
@@ -613,7 +603,7 @@ sioattach(isdp)
 	com->it_in.c_oflag = 0;
 	com->it_in.c_cflag = TTYDEF_CFLAG;
 	com->it_in.c_lflag = 0;
-	if (unit == comconsole && (COMCONSOLE || boothowto & RB_SERIAL)) {
+	if (unit == comconsole) {
 		com->it_in.c_iflag = TTYDEF_IFLAG;
 		com->it_in.c_oflag = TTYDEF_OFLAG;
 		com->it_in.c_cflag = TTYDEF_CFLAG | CLOCAL;
@@ -630,7 +620,7 @@ sioattach(isdp)
 #ifdef DSI_SOFT_MODEM
 	if((inb(iobase+7) ^ inb(iobase+7)) & 0x80) {
 	    printf(" Digicom Systems, Inc. SoftModem");
-	    kdc_sio[unit].kdc_description = 
+	    kdc_sio[unit].kdc_description =
 	      "Serial port: Digicom Systems SoftModem";
 	goto determined_type;
 	}
@@ -706,13 +696,11 @@ determined_type: ;
 #endif /* COM_MULTIPORT */
 	printf("\n");
 
-	kdc_sio[unit].kdc_state =
-		(unit == comconsole && (COMCONSOLE || boothowto & RB_SERIAL))
-		? DC_BUSY : DC_IDLE;
+	kdc_sio[unit].kdc_state = (unit == comconsole) ? DC_BUSY : DC_IDLE;
 
 #ifdef KGDB
 	if (kgdb_dev == makedev(commajor, unit)) {
-		if (unit == comconsole && (COMCONSOLE || boothowto & RB_SERIAL))
+		if (unit == comconsole)
 			kgdb_dev = -1;	/* can't debug over console port */
 		else {
 			int	divisor;
@@ -728,7 +716,7 @@ determined_type: ;
 			outb(iobase + com_dlbl, divisor & 0xFF);
 			outb(iobase + com_dlbh, (u_int) divisor >> 8);
 			outb(iobase + com_cfcr, CFCR_8BITS);
-			outb(com->modem_status_port,
+			outb(com->modem_ctl_port,
 			     com->mcr_image |= MCR_DTR | MCR_RTS);
 
 			if (kgdb_debug_init) {
@@ -747,14 +735,9 @@ determined_type: ;
 	s = spltty();
 	com_addr(unit) = com;
 	splx(s);
-	if (!comwakeup_started) {
-		comwakeup((void *)NULL);
-		comwakeup_started = TRUE;
-	}
 	return (1);
 }
 
-/* ARGSUSED */
 int
 sioopen(dev, flag, mode, p)
 	dev_t		dev;
@@ -776,7 +759,7 @@ sioopen(dev, flag, mode, p)
 		return (ENXIO);
 	if (mynor & CONTROL_MASK)
 		return (0);
-#if 0 /* XXX TK2.0 */
+#if 0 /* XXX */
 	tp = com->tp = sio_tty[unit] = ttymalloc(sio_tty[unit]);
 #else
 	tp = com->tp = &sio_tty[unit];
@@ -832,9 +815,10 @@ open_top:
 		tp->t_dev = dev;
 		tp->t_termios = mynor & CALLOUT_MASK
 				? com->it_out : com->it_in;
-		commctl(com, MCR_DTR | MCR_RTS, DMSET);
+		(void)commctl(com, TIOCM_DTR | TIOCM_RTS, DMSET);
 		com->ftl_max = com->ftl_init;
 		com->poll = com->no_irq;
+		com->poll_output = com->loses_outints;
 		++com->wopeners;
 		error = comparam(tp, &tp->t_termios);
 		--com->wopeners;
@@ -873,8 +857,8 @@ open_top:
 		disable_intr();
 		(void) inb(com->line_status_port);
 		(void) inb(com->data_port);
-			com->prev_modem_status =
-			com->last_modem_status = inb(com->modem_status_port);
+		com->prev_modem_status = com->last_modem_status
+		    = inb(com->modem_status_port);
 		outb(iobase + com_ier, IER_ERXRDY | IER_ETXRDY | IER_ERLS
 				       | IER_EMSC);
 		enable_intr();
@@ -884,6 +868,13 @@ open_top:
 		 * callin opens get woken up and resume sleeping on "siobi"
 		 * instead of "siodcd".
 		 */
+		/*
+		 * XXX `mynor & CALLOUT_MASK' should be
+		 * `tp->t_cflag & (SOFT_CARRIER | TRAPDOOR_CARRIER) where
+		 * TRAPDOOR_CARRIER is the default initial state for callout
+		 * devices and SOFT_CARRIER is like CLOCAL except it hides
+		 * the true carrier.
+		 */
 		if (com->prev_modem_status & MSR_DCD || mynor & CALLOUT_MASK)
 			(*linesw[tp->t_line].l_modem)(tp, 1);
 	}
@@ -900,9 +891,10 @@ open_top:
 		goto open_top;
 	}
 	error =	(*linesw[tp->t_line].l_open)(dev, tp);
-	disc_optim(tp, &(tp->t_termios), com);
+	disc_optim(tp, &tp->t_termios, com);
 	if (tp->t_state & TS_ISOPEN && mynor & CALLOUT_MASK)
 		com->active_out = TRUE;
+	siosettimeout();
 out:
 	splx(s);
 	if (!(tp->t_state & TS_ISOPEN) && com->wopeners == 0)
@@ -910,7 +902,6 @@ out:
 	return (error);
 }
 
-/*ARGSUSED*/
 int
 sioclose(dev, flag, mode, p)
 	dev_t		dev;
@@ -930,10 +921,11 @@ sioclose(dev, flag, mode, p)
 	tp = com->tp;
 	s = spltty();
 	(*linesw[tp->t_line].l_close)(tp, flag);
-	disc_optim(tp, &(tp->t_termios), com);
+	disc_optim(tp, &tp->t_termios, com);
 	siostop(tp, FREAD | FWRITE);
 	comhardclose(com);
 	ttyclose(tp);
+	siosettimeout();
 	splx(s);
 	return (0);
 }
@@ -951,6 +943,7 @@ comhardclose(com)
 	iobase = com->iobase;
 	s = spltty();
 	com->poll = FALSE;
+	com->poll_output = FALSE;
 	com->do_timestamp = 0;
 	outb(iobase + com_cfcr, com->cfcr_image &= ~CFCR_SBREAK);
 #ifdef KGDB
@@ -972,18 +965,25 @@ comhardclose(com)
 		       && !(com->prev_modem_status & MSR_DCD)
 		       && !(com->it_in.c_cflag & CLOCAL)
 		    || !(tp->t_state & TS_ISOPEN)) {
-			commctl(com, MCR_RTS, DMSET);
+			(void)commctl(com, TIOCM_DTR, DMBIC);
 			if (com->dtr_wait != 0) {
 				timeout(siodtrwakeup, com, com->dtr_wait);
 				com->state |= CS_DTR_OFF;
 			}
 		}
 	}
+	if (com->hasfifo) {
+		/*
+		 * Disable fifos so that they are off after controlled
+		 * reboots.  Some BIOSes fail to detect 16550s when the
+		 * fifos are enabled.
+		 */
+		outb(iobase + com_fifo, 0);
+	}
 	com->active_out = FALSE;
 	wakeup(&com->active_out);
 	wakeup(TSA_CARR_ON(tp));	/* restart any wopeners */
-	if (!(com->state & CS_DTR_OFF)
-	    && !(unit == comconsole && (COMCONSOLE || boothowto & RB_SERIAL)))
+	if (!(com->state & CS_DTR_OFF) && unit != comconsole)
 		kdc_sio[unit].kdc_state = DC_IDLE;
 	splx(s);
 }
@@ -1026,8 +1026,7 @@ siowrite(dev, uio, flag)
 	 * is not the console.  In that situation we don't need/want the X
 	 * server taking over the console.
 	 */
-	if (constty && unit == comconsole
-	    && (COMCONSOLE || boothowto & RB_SERIAL))
+	if (constty != NULL && unit == comconsole)
 		constty = NULL;
 	return ((*linesw[tp->t_line].l_write)(tp, uio, flag));
 }
@@ -1040,7 +1039,7 @@ siodtrwakeup(chan)
 
 	com = (struct com_s *)chan;
 	com->state &= ~CS_DTR_OFF;
-	if (!(com->unit == comconsole && (COMCONSOLE || boothowto & RB_SERIAL)))
+	if (com->unit != comconsole)
 		kdc_sio[com->unit].kdc_state = DC_IDLE;
 	wakeup(&com->dtr_wait);
 }
@@ -1097,7 +1096,7 @@ siointr(unit)
 
 static void
 siointr1(com)
-	struct com_s *com;
+	struct com_s	*com;
 {
 	u_char	line_status;
 	u_char	modem_status;
@@ -1120,8 +1119,7 @@ siointr1(com)
 			if (line_status & (LSR_PE|LSR_FE|LSR_BI)) {
 #ifdef DDB
 #ifdef BREAK_TO_DEBUGGER
-				if (   (line_status & LSR_BI)
-				    && (COMCONSOLE || boothowto	& RB_SERIAL)
+				if (line_status & LSR_BI
 				    && com->unit == comconsole)	{
 					Debugger("serial console break");
 					goto cont;
@@ -1165,6 +1163,7 @@ siointr1(com)
 				CE_RECORD(com, CE_INTERRUPT_BUF_OVERFLOW);
 			else {
 				++com_events;
+				schedsofttty();
 #if 0 /* for testing input latency vs efficiency */
 if (com->iptr - com->ibuf == 8)
 	setsofttty();
@@ -1179,7 +1178,7 @@ if (com->iptr - com->ibuf == 8)
 				if (line_status & LSR_OE)
 					CE_RECORD(com, CE_OVERRUN);
 			}
-		cont:
+cont:
 			/*
 			 * "& 0x7F" is to avoid the gcc-1.40 generating a slow
 			 * jump from the top of the loop to here
@@ -1214,12 +1213,12 @@ if (com->iptr - com->ibuf == 8)
 
 		/* output queued and everything ready? */
 		if (line_status & LSR_TXRDY
-		    && com->state >= (CS_ODEVREADY | CS_BUSY | CS_TTGO)) {
-			ioptr = com->optr;
+		    && com->state >= (CS_BUSY | CS_TTGO | CS_ODEVREADY)) {
+			ioptr = com->obufq.l_head;
 			if (com->tx_fifo_size > 1) {
 				u_int	ocount;
 
-				ocount = com->obufend - ioptr;
+				ocount = com->obufq.l_tail - ioptr;
 				if (ocount > com->tx_fifo_size)
 					ocount = com->tx_fifo_size;
 				com->bytes_out += ocount;
@@ -1230,12 +1229,26 @@ if (com->iptr - com->ibuf == 8)
 				outb(com->data_port, *ioptr++);
 				++com->bytes_out;
 			}
-			com->optr = ioptr;
-			if (ioptr >= com->obufend) {
-				/* output just completed */
-				com_events += LOTS_OF_EVENTS;
-				com->state ^= (CS_ODONE | CS_BUSY);
-				setsofttty();	/* handle at high level ASAP */
+			com->obufq.l_head = ioptr;
+			if (ioptr >= com->obufq.l_tail) {
+				struct lbq	*qp;
+
+				qp = com->obufq.l_next;
+				qp->l_queued = FALSE;
+				qp = qp->l_next;
+				if (qp != NULL) {
+					com->obufq.l_head = qp->l_head;
+					com->obufq.l_tail = qp->l_tail;
+					com->obufq.l_next = qp;
+				} else {
+					/* output just completed */
+					com->state &= ~CS_BUSY;
+				}
+				if (!(com->state & CS_ODONE)) {
+					com_events += LOTS_OF_EVENTS;
+					com->state |= CS_ODONE;
+					setsofttty();	/* handle at high level ASAP */
+				}
 			}
 		}
 
@@ -1247,20 +1260,6 @@ if (com->iptr - com->ibuf == 8)
 	}
 }
 
-static int
-tiocm_xxx2mcr(tiocm_xxx)
-	int	tiocm_xxx;
-{
-	int	mcr;
-
-	mcr = 0;
-	if (tiocm_xxx & TIOCM_DTR)
-		mcr |= MCR_DTR;
-	if (tiocm_xxx & TIOCM_RTS)
-		mcr |= MCR_RTS;
-	return (mcr);
-}
-
 int
 sioioctl(dev, cmd, data, flag, p)
 	dev_t		dev;
@@ -1272,11 +1271,8 @@ sioioctl(dev, cmd, data, flag, p)
 	struct com_s	*com;
 	int		error;
 	Port_t		iobase;
-	int		mcr;
-	int		msr;
 	int		mynor;
 	int		s;
-	int		tiocm_xxx;
 	struct tty	*tp;
 #if defined(COMPAT_43) || defined(COMPAT_SUNOS)
 	int		oldcmd;
@@ -1326,15 +1322,15 @@ sioioctl(dev, cmd, data, flag, p)
 
 			pi = (u_char*)(*(caddr_t*)data);
 			error = copyin(pi,&l,sizeof l);
-			if(error) 
+			if(error)
 				{return error;};
 			pi += sizeof l;
 
 			p = malloc(l,M_TEMP,M_NOWAIT);
-			if(!p) 
+			if(!p)
 				{return ENOBUFS;}
 			error = copyin(pi,p,l);
-			if(error) 
+			if(error)
 				{free(p,M_TEMP); return error;};
 			if(error = LoadSoftModem(
 			    MINOR_TO_UNIT(mynor),iobase,l,p))
@@ -1384,7 +1380,7 @@ sioioctl(dev, cmd, data, flag, p)
 		return (error);
 	s = spltty();
 	error = ttioctl(tp, cmd, data, flag);
-	disc_optim(tp, &(tp->t_termios), com);
+	disc_optim(tp, &tp->t_termios, com);
 	if (error >= 0) {
 		splx(s);
 		return (error);
@@ -1397,42 +1393,22 @@ sioioctl(dev, cmd, data, flag, p)
 		outb(iobase + com_cfcr, com->cfcr_image &= ~CFCR_SBREAK);
 		break;
 	case TIOCSDTR:
-		commctl(com, MCR_DTR, DMBIS);
+		(void)commctl(com, TIOCM_DTR, DMBIS);
 		break;
 	case TIOCCDTR:
-		commctl(com, MCR_DTR, DMBIC);
+		(void)commctl(com, TIOCM_DTR, DMBIC);
 		break;
 	case TIOCMSET:
-		commctl(com, tiocm_xxx2mcr(*(int *)data), DMSET);
+		(void)commctl(com, *(int *)data, DMSET);
 		break;
 	case TIOCMBIS:
-		commctl(com, tiocm_xxx2mcr(*(int *)data), DMBIS);
+		(void)commctl(com, *(int *)data, DMBIS);
 		break;
 	case TIOCMBIC:
-		commctl(com, tiocm_xxx2mcr(*(int *)data), DMBIC);
+		(void)commctl(com, *(int *)data, DMBIC);
 		break;
 	case TIOCMGET:
-		tiocm_xxx = TIOCM_LE;	/* XXX - always enabled while open */
-		mcr = com->mcr_image;
-		if (mcr & MCR_DTR)
-			tiocm_xxx |= TIOCM_DTR;
-		if (mcr & MCR_RTS)
-			tiocm_xxx |= TIOCM_RTS;
-		msr = com->prev_modem_status;
-		if (msr & MSR_CTS)
-			tiocm_xxx |= TIOCM_CTS;
-		if (msr & MSR_DCD)
-			tiocm_xxx |= TIOCM_CD;
-		if (msr & MSR_DSR)
-			tiocm_xxx |= TIOCM_DSR;
-		/*
-		 * XXX - MSR_RI is naturally volatile, and we make MSR_TERI
-		 * more volatile by reading the modem status a lot.  Perhaps
-		 * we should latch both bits until the status is read here.
-		 */
-		if (msr & (MSR_RI | MSR_TERI))
-			tiocm_xxx |= TIOCM_RI;
-		*(int *)data = tiocm_xxx;
+		*(int *)data = commctl(com, 0, DMGET);
 		break;
 	case TIOCMSDTRWAIT:
 		/* must be root since the wait applies to following logins */
@@ -1458,19 +1434,6 @@ sioioctl(dev, cmd, data, flag, p)
 	return (0);
 }
 
-/* cancel pending output */
-static void
-comflush(com)
-	struct com_s	*com;
-{
-	disable_intr();
-	if (com->state & CS_ODONE)
-		com_events -= LOTS_OF_EVENTS;
-	com->state &= ~(CS_ODONE | CS_BUSY);
-	enable_intr();
-	com->tp->t_state &= ~TS_BUSY;
-}
-
 void
 siopoll()
 {
@@ -1565,36 +1528,16 @@ repeat:
 					(tp, com->prev_modem_status & MSR_DCD);
 		}
 		if (com->state & CS_ODONE) {
-			comflush(com);
+			disable_intr();
+			com_events -= LOTS_OF_EVENTS;
+			com->state &= ~CS_ODONE;
+			if (!(com->state & CS_BUSY))
+				com->tp->t_state &= ~TS_BUSY;
+			enable_intr();
 			(*linesw[tp->t_line].l_start)(tp);
 		}
 		if (incc <= 0 || !(tp->t_state & TS_ISOPEN))
 			continue;
-		/*
-		 * XXX only do this when we bypass ttyinput.
-		 */
-		if (tp->t_rawq.c_cc + incc >= RB_I_HIGH_WATER
-		    && (com->state & CS_RTS_IFLOW || tp->t_iflag & IXOFF)
-		    && !(tp->t_state & TS_TBLOCK)
-		    /*
-		     * XXX - need flow control for all line disciplines.
-		     * Only have it in standard one now.
-		     */
-		    && linesw[tp->t_line].l_rint == ttyinput) {
-			int	putc_status = FALSE;
-
-			if ((tp->t_iflag & IXOFF
-			     && tp->t_cc[VSTOP] != _POSIX_VDISABLE
-			     && (putc_status = putc(tp->t_cc[VSTOP],
-						    &tp->t_outq)) == 0)
-			    || com->state & CS_RTS_IFLOW) {
-				tp->t_state |= TS_TBLOCK;
-				ttstart(tp);
-				if (putc_status != 0)
-					/* Try again later. */
-					tp->t_state &= ~TS_TBLOCK;
-			}
-		}
 		/*
 		 * Avoid the grotesquely inefficient lineswitch routine
 		 * (ttyinput) in "raw" mode.  It usually takes about 450
@@ -1603,6 +1546,11 @@ repeat:
 		 * call overhead).
 		 */
 		if (tp->t_state & TS_CAN_BYPASS_L_RINT) {
+			if (tp->t_rawq.c_cc + incc >= RB_I_HIGH_WATER
+			    && (com->state & CS_RTS_IFLOW
+				|| tp->t_iflag & IXOFF)
+			    && !(tp->t_state & TS_TBLOCK))
+				ttyblock(tp);
 			tk_nin += incc;
 			tk_rawcc += incc;
 			tp->t_rawcc += incc;
@@ -1614,7 +1562,7 @@ repeat:
 				|| tp->t_cc[VSTART] == tp->t_cc[VSTOP])) {
 				tp->t_state &= ~TS_TTSTOP;
 				tp->t_lflag &= ~FLUSHO;
-				ttstart(tp);
+				comstart(tp);
 			}
 		} else {
 			do {
@@ -1653,15 +1601,20 @@ comparam(tp, t)
 	int		cflag;
 	struct com_s	*com;
 	int		divisor;
+	u_char		dlbh;
+	u_char		dlbl;
 	int		error;
 	Port_t		iobase;
 	int		s;
 	int		unit;
+	int		txtimeout;
+
+	/* do historical conversions */
+	if (t->c_ispeed == 0)
+		t->c_ispeed = t->c_ospeed;
 
 	/* check requested parameters */
 	divisor = ttspeedtab(t->c_ospeed, comspeedtab);
-	if (t->c_ispeed == 0)
-		t->c_ispeed = t->c_ospeed;
 	if (divisor < 0 || divisor > 0 && t->c_ispeed != t->c_ospeed)
 		return (EINVAL);
 
@@ -1671,9 +1624,9 @@ comparam(tp, t)
 	iobase = com->iobase;
 	s = spltty();
 	if (divisor == 0)
-		commctl(com, MCR_DTR, DMBIC);	/* hang up line */
+		(void)commctl(com, TIOCM_DTR, DMBIC);	/* hang up line */
 	else
-		commctl(com, MCR_DTR, DMBIS);
+		(void)commctl(com, TIOCM_DTR, DMBIS);
 	cflag = t->c_cflag;
 	switch (cflag & CSIZE) {
 	case CS5:
@@ -1697,7 +1650,7 @@ comparam(tp, t)
 	if (cflag & CSTOPB)
 		cfcr |= CFCR_STOPB;
 
-	if (com->hasfifo) {
+	if (com->hasfifo && divisor != 0) {
 		/*
 		 * Use a fifo trigger level low enough so that the input
 		 * latency from the fifo is less than about 16 msec and
@@ -1724,11 +1677,18 @@ comparam(tp, t)
 	disable_intr();
 retry:
 	com->state &= ~CS_TTGO;
+	txtimeout = tp->t_timeout;
 	enable_intr();
 	while ((inb(com->line_status_port) & (LSR_TSRE | LSR_TXRDY))
 	       != (LSR_TSRE | LSR_TXRDY)) {
+		tp->t_state |= TS_SO_OCOMPLETE;
 		error = ttysleep(tp, TSA_OCOMPLETE(tp), TTIPRI | PCATCH,
 				 "siotx", hz / 100);
+		if (   txtimeout != 0
+		    && (!error || error	== EAGAIN)
+		    && (txtimeout -= hz	/ 100) <= 0
+		   )
+			error = EIO;
 		if (error != 0 && error != EAGAIN) {
 			if (!(tp->t_state & TS_TTSTOP)) {
 				disable_intr();
@@ -1754,8 +1714,18 @@ retry:
 
 	if (divisor != 0) {
 		outb(iobase + com_cfcr, cfcr | CFCR_DLAB);
-		outb(iobase + com_dlbl, divisor & 0xFF);
-		outb(iobase + com_dlbh, (u_int) divisor >> 8);
+		/*
+		 * Only set the divisor registers if they would change,
+		 * since on some 16550 incompatibles (UMC8669F), setting
+		 * them while input is arriving them loses sync until
+		 * data stops arriving.
+		 */
+		dlbl = divisor & 0xFF;
+		if (inb(iobase + com_dlbl) != dlbl)
+			outb(iobase + com_dlbl, dlbl);
+		dlbh = (u_int) divisor >> 8;
+		if (inb(iobase + com_dlbh) != dlbh)
+			outb(iobase + com_dlbh, dlbh);
 	}
 	outb(iobase + com_cfcr, com->cfcr_image = cfcr);
 	if (!(tp->t_state & TS_TTSTOP))
@@ -1770,13 +1740,14 @@ retry:
 	 * XXX - worth handling MDMBUF (DCD) flow control at the lowest level?
 	 * Now has 10+ msec latency, while CTS flow has 50- usec latency.
 	 */
-	com->state &= ~CS_CTS_OFLOW;
 	com->state |= CS_ODEVREADY;
+	com->state &= ~CS_CTS_OFLOW;
 	if (cflag & CCTS_OFLOW) {
 		com->state |= CS_CTS_OFLOW;
 		if (!(com->last_modem_status & MSR_CTS))
 			com->state &= ~CS_ODEVREADY;
 	}
+	/* XXX shouldn't call functions while intrs are disabled. */
 	disc_optim(tp, t, com);
 	/*
 	 * Recover from fiddling with CS_TTGO.  We used to call siointr1()
@@ -1820,36 +1791,61 @@ comstart(tp)
 			outb(com->modem_ctl_port, com->mcr_image |= MCR_RTS);
 	}
 	enable_intr();
-	if (tp->t_state & (TS_TIMEOUT | TS_TTSTOP))
-		goto out;
-#if 0 /* XXX TK2.0 */
-	if (tp->t_state & (TS_SO_OCOMPLETE | TS_SO_OLOWAT) || tp->t_wsel)
-		ttwwakeup(tp);
-#else
-	if (tp->t_outq.c_cc <= tp->t_lowat) {
-		if (tp->t_state & TS_ASLEEP) {
-			tp->t_state &= ~TS_ASLEEP;
-			wakeup(TSA_OLOWAT(tp));
-		}
-		selwakeup(&tp->t_wsel);
+	if (tp->t_state & (TS_TIMEOUT | TS_TTSTOP)) {
+		splx(s);
+		return;
 	}
-#endif
-	if (tp->t_state & TS_BUSY) {
-		disable_intr();
-		siointr1(com);
-		enable_intr();
-	} else if (tp->t_outq.c_cc != 0) {
-		u_int	ocount;
+	if (tp->t_outq.c_cc != 0) {
+		struct lbq	*qp;
+		struct lbq	*next;
 
+		if (!com->obufs[0].l_queued) {
+			com->obufs[0].l_tail
+			    = com->obuf1 + q_to_b(&tp->t_outq, com->obuf1,
+						  sizeof com->obuf1);
+			com->obufs[0].l_next = NULL;
+			com->obufs[0].l_queued = TRUE;
+			disable_intr();
+			if (com->state & CS_BUSY) {
+				qp = com->obufq.l_next;
+				while ((next = qp->l_next) != NULL)
+					qp = next;
+				qp->l_next = &com->obufs[0];
+			} else {
+				com->obufq.l_head = com->obufs[0].l_head;
+				com->obufq.l_tail = com->obufs[0].l_tail;
+				com->obufq.l_next = &com->obufs[0];
+				com->state |= CS_BUSY;
+			}
+			enable_intr();
+		}
+		if (tp->t_outq.c_cc != 0 && !com->obufs[1].l_queued) {
+			com->obufs[1].l_tail
+			    = com->obuf2 + q_to_b(&tp->t_outq, com->obuf2,
+						  sizeof com->obuf2);
+			com->obufs[1].l_next = NULL;
+			com->obufs[1].l_queued = TRUE;
+			disable_intr();
+			if (com->state & CS_BUSY) {
+				qp = com->obufq.l_next;
+				while ((next = qp->l_next) != NULL)
+					qp = next;
+				qp->l_next = &com->obufs[1];
+			} else {
+				com->obufq.l_head = com->obufs[1].l_head;
+				com->obufq.l_tail = com->obufs[1].l_tail;
+				com->obufq.l_next = &com->obufs[1];
+				com->state |= CS_BUSY;
+			}
+			enable_intr();
+		}
 		tp->t_state |= TS_BUSY;
-		ocount = q_to_b(&tp->t_outq, com->obuf, sizeof com->obuf);
-		disable_intr();
-		com->obufend = (com->optr = com->obuf) + ocount;
-		com->state |= CS_BUSY;
-		siointr1(com);	/* fake interrupt to start output */
-		enable_intr();
 	}
-out:
+	disable_intr();
+	if (com->state >= (CS_BUSY | CS_TTGO))
+		siointr1(com);	/* fake interrupt to start output */
+	enable_intr();
+	ttwwakeup(tp);
 	splx(s);
 }
 
@@ -1861,18 +1857,23 @@ siostop(tp, rw)
 	struct com_s	*com;
 
 	com = com_addr(DEV_TO_UNIT(tp->t_dev));
-	if (rw & FWRITE)
-		comflush(com);
 	disable_intr();
+	if (rw & FWRITE) {
+		com->obufs[0].l_queued = FALSE;
+		com->obufs[1].l_queued = FALSE;
+		if (com->state & CS_ODONE)
+			com_events -= LOTS_OF_EVENTS;
+		com->state &= ~(CS_ODONE | CS_BUSY);
+		com->tp->t_state &= ~TS_BUSY;
+	}
 	if (rw & FREAD) {
 		com_events -= (com->iptr - com->ibuf);
 		com->iptr = com->ibuf;
 	}
-	if (tp->t_state & TS_TTSTOP)
-		com->state &= ~CS_TTGO;
-	else
-		com->state |= CS_TTGO;
 	enable_intr();
+	comstart(tp);
+
+	/* XXX should clear h/w fifos too. */
 }
 
 struct tty *
@@ -1891,26 +1892,95 @@ siodevtotty(dev)
 	return (&sio_tty[unit]);
 }
 
-static void
+static int
 commctl(com, bits, how)
 	struct com_s	*com;
 	int		bits;
 	int		how;
 {
+	int	mcr;
+	int	msr;
+
+	if (how == DMGET) {
+		bits = TIOCM_LE;	/* XXX - always enabled while open */
+		mcr = com->mcr_image;
+		if (mcr & MCR_DTR)
+			bits |= TIOCM_DTR;
+		if (mcr & MCR_RTS)
+			bits |= TIOCM_RTS;
+		msr = com->prev_modem_status;
+		if (msr & MSR_CTS)
+			bits |= TIOCM_CTS;
+		if (msr & MSR_DCD)
+			bits |= TIOCM_CD;
+		if (msr & MSR_DSR)
+			bits |= TIOCM_DSR;
+		/*
+		 * XXX - MSR_RI is naturally volatile, and we make MSR_TERI
+		 * more volatile by reading the modem status a lot.  Perhaps
+		 * we should latch both bits until the status is read here.
+		 */
+		if (msr & (MSR_RI | MSR_TERI))
+			bits |= TIOCM_RI;
+		return (bits);
+	}
+	mcr = 0;
+	if (bits & TIOCM_DTR)
+		mcr |= MCR_DTR;
+	if (bits & TIOCM_RTS)
+		mcr |= MCR_RTS;
 	disable_intr();
 	switch (how) {
 	case DMSET:
 		outb(com->modem_ctl_port,
-		     com->mcr_image = bits | (com->mcr_image & MCR_IENABLE));
+		     com->mcr_image = mcr | (com->mcr_image & MCR_IENABLE));
 		break;
 	case DMBIS:
-		outb(com->modem_ctl_port, com->mcr_image |= bits);
+		outb(com->modem_ctl_port, com->mcr_image |= mcr);
 		break;
 	case DMBIC:
-		outb(com->modem_ctl_port, com->mcr_image &= ~bits);
+		outb(com->modem_ctl_port, com->mcr_image &= ~mcr);
 		break;
 	}
 	enable_intr();
+	return (0);
+}
+
+static void
+siosettimeout()
+{
+	struct com_s	*com;
+	bool_t		someopen;
+	int		unit;
+
+	/*
+	 * Set our timeout period to 1 second if no polled devices are open.
+	 * Otherwise set it to max(1/200, 1/hz).
+	 * Enable timeouts iff some device is open.
+	 */
+	untimeout(comwakeup, (void *)NULL);
+	sio_timeout = hz;
+	someopen = FALSE;
+	for (unit = 0; unit < NSIO; ++unit) {
+		com = com_addr(unit);
+		if (com != NULL && com->tp != NULL
+		    && com->tp->t_state & TS_ISOPEN) {
+			someopen = TRUE;
+			if (com->poll || com->poll_output) {
+				sio_timeout = hz > 200 ? hz / 200 : 1;
+				break;
+			}
+		}
+	}
+	if (someopen) {
+		sio_timeouts_until_log = hz / sio_timeout;
+		timeout(comwakeup, (void *)NULL, sio_timeout);
+	} else {
+		/* Flush error messages, if any. */
+		sio_timeouts_until_log = 1;
+		comwakeup((void *)NULL);
+		untimeout(comwakeup, (void *)NULL);
+	}
 }
 
 static void
@@ -1918,18 +1988,9 @@ comwakeup(chan)
 	void	*chan;
 {
 	struct com_s	*com;
-	static	int	log_countdown = 1;
 	int		unit;
 
-	timeout(comwakeup, (caddr_t)NULL, hz > 200 ? hz / 200 : 1);
-
-	if (com_events != 0) {
-		int	s;
-
-		s = splsofttty();
-		siopoll();
-		splx(s);
-	}
+	timeout(comwakeup, (void *)NULL, sio_timeout);
 
 	/*
 	 * Recover from lost output interrupts.
@@ -1948,9 +2009,9 @@ comwakeup(chan)
 	/*
 	 * Check for and log errors, but not too often.
 	 */
-	if (--log_countdown > 0)
+	if (--sio_timeouts_until_log > 0)
 		return;
-	log_countdown = hz > 200 ? 200 : hz;
+	sio_timeouts_until_log = hz / sio_timeout;
 	for (unit = 0; unit < NSIO; ++unit) {
 		int	errnum;
 
@@ -2005,14 +2066,11 @@ disc_optim(tp, t, com)
 	struct termios	*t;
 	struct com_s	*com;
 {
-
-	if (!(t->c_iflag & (ICRNL | IGNCR | IMAXBEL | INLCR | ISTRIP
-			   | IXOFF | IXON))
+	if (!(t->c_iflag & (ICRNL | IGNCR | IMAXBEL | INLCR | ISTRIP | IXON))
 	    && (!(t->c_iflag & BRKINT) || (t->c_iflag & IGNBRK))
-	    && (!(t->c_iflag & PARMRK) ||
-		(t->c_iflag & (IGNPAR|IGNBRK)) == (IGNPAR|IGNBRK))
-	    && !(t->c_lflag & (ECHO | ECHONL | ICANON | IEXTEN | ISIG
-			   | PENDIN))
+	    && (!(t->c_iflag & PARMRK)
+		|| (t->c_iflag & (IGNPAR | IGNBRK)) == (IGNPAR | IGNBRK))
+	    && !(t->c_lflag & (ECHO | ICANON | IEXTEN | ISIG | PENDIN))
 	    && linesw[tp->t_line].l_rint == ttyinput)
 		tp->t_state |= TS_CAN_BYPASS_L_RINT;
 	else
@@ -2069,6 +2127,8 @@ siocnopen(sp)
 	struct siocnstate	*sp;
 {
 	int	divisor;
+	u_char	dlbh;
+	u_char	dlbl;
 	Port_t	iobase;
 
 	/*
@@ -2081,12 +2141,22 @@ siocnopen(sp)
 	outb(iobase + com_ier, 0);	/* spltty() doesn't stop siointr() */
 	siocntxwait();
 	sp->cfcr = inb(iobase + com_cfcr);
-	outb(iobase + com_cfcr, CFCR_DLAB);
+	outb(iobase + com_cfcr, CFCR_DLAB | CFCR_8BITS);
 	sp->dlbl = inb(iobase + com_dlbl);
 	sp->dlbh = inb(iobase + com_dlbh);
+	/*
+	 * Only set the divisor registers if they would change, since on
+	 * some 16550 incompatibles (Startech), setting them clears the
+	 * data input register.  This also reduces the effects of the
+	 * UMC8669F bug.
+	 */
 	divisor = ttspeedtab(comdefaultrate, comspeedtab);
-	outb(iobase + com_dlbl, divisor & 0xFF);
-	outb(iobase + com_dlbh, (u_int) divisor >> 8);
+	dlbl = divisor & 0xFF;
+	if (sp->dlbl != dlbl)
+		outb(iobase + com_dlbl, dlbl);
+	dlbh = (u_int) divisor >> 8;
+	if (sp->dlbh != dlbh)
+		outb(iobase + com_dlbh, dlbh);
 	outb(iobase + com_cfcr, CFCR_8BITS);
 	sp->mcr = inb(iobase + com_mcr);
 	/*
@@ -2108,9 +2178,11 @@ siocnclose(sp)
 	 */
 	siocntxwait();
 	iobase = siocniobase;
-	outb(iobase + com_cfcr, CFCR_DLAB);
-	outb(iobase + com_dlbl, sp->dlbl);
-	outb(iobase + com_dlbh, sp->dlbh);
+	outb(iobase + com_cfcr, CFCR_DLAB | CFCR_8BITS);
+	if (sp->dlbl != inb(iobase + com_dlbl))
+		outb(iobase + com_dlbl, sp->dlbl);
+	if (sp->dlbh != inb(iobase + com_dlbh))
+		outb(iobase + com_dlbh, sp->dlbh);
 	outb(iobase + com_cfcr, sp->cfcr);
 	/*
 	 * XXX damp oscillations of MCR_DTR and MCR_RTS by not restoring them.
@@ -2125,12 +2197,22 @@ siocnprobe(cp)
 {
 	int	unit;
 
+	/*
+	 * If this has already been done, don't do it again
+	 * because cdevsw[maj] has been changed to cnopen and
+	 * we won't find the right table entry.
+	 */
+	if (cp->cn_dev > 0)
+	    return;
+ 
 	/* locate the major number */
 	/* XXX - should be elsewhere since KGDB uses it */
 	for (commajor = 0; commajor < nchrdev; commajor++)
 		if (cdevsw[commajor].d_open == sioopen)
 			break;
 
+	/* XXX - we should panic() if we don't find it */
+
 	/* XXX: ick */
 	unit = DEV_TO_UNIT(CONUNIT);
 	siocniobase = CONADDR;
@@ -2139,12 +2221,11 @@ siocnprobe(cp)
 
 	/* initialize required fields */
 	cp->cn_dev = makedev(commajor, unit);
-
-	if (COMCONSOLE || boothowto & RB_SERIAL)
-		cp->cn_pri = CN_REMOTE;	/* Force a serial port console */
-	else
-		cp->cn_pri = CN_NORMAL;
-
+#ifdef COMCONSOLE
+	cp->cn_pri = CN_REMOTE;		/* Force a serial port console */
+#else
+	cp->cn_pri = (boothowto & RB_SERIAL) ? CN_REMOTE : CN_NORMAL;
+#endif
 }
 
 void
@@ -2229,7 +2310,7 @@ LoadSoftModem(int unit, int base_io, u_long size, u_char *ptr)
     int int_c,int_k;
     int data_0188, data_0187;
 
-    /* 
+    /*
      * First see if it is a DSI SoftModem
      */
     if(!((inb(base_io+7) ^ inb(base_io+7) & 0x80)))
@@ -2248,8 +2329,8 @@ LoadSoftModem(int unit, int base_io, u_long size, u_char *ptr)
 	DSI_ERROR("dsp bus not granted");
 
     if(0x01 != (inb(base_io+7) & 0x01)) {
-	outb(base_io+7,0x18); 
-	outb(base_io+7,0x19); 
+	outb(base_io+7,0x18);
+	outb(base_io+7,0x19);
 	if(0x01 != (inb(base_io+7) & 0x01))
 	    DSI_ERROR("program mem not granted");
     }
@@ -2275,14 +2356,14 @@ LoadSoftModem(int unit, int base_io, u_long size, u_char *ptr)
  	outb(base_io+7,0x19);
 	if(0x00 != (inb(base_io+7) & 0x01))
 	    DSI_ERROR("program data not granted");
-	
+
 	for(int_k = 0 ; int_k < 0x800; int_k++) {
 	    outb(base_io+1,*ptr++);
 	    outb(base_io+2,0);
 	    outb(base_io+1,*ptr++);
 	    outb(base_io+2,*ptr++);
 	}
-	
+
 	size -= 0x1800;
 
 	while(size > 0x1800) {
@@ -2299,7 +2380,7 @@ LoadSoftModem(int unit, int base_io, u_long size, u_char *ptr)
 		outb(base_io+2,*ptr++);
 	    }
 	}
-	
+
     } else if (size > 0) {
 	if(int_c == 7) {
 	    outb(base_io+7,0x18);
@@ -2319,7 +2400,7 @@ LoadSoftModem(int unit, int base_io, u_long size, u_char *ptr)
 		outb(base_io+2,*ptr++);
 	    }
 	}
-    }	
+    }
     outb(base_io+7,0x11);
     outb(base_io+7,3);
 
