@@ -98,7 +98,7 @@
  * the unit worked).  I suggest saving money and buying the bare TW523
  * rather than the TW523 development kit (what I bought), because if you
  * are running FreeBSD you don't really care about the DOS binaries.
- * 
+ *
  * The interface to the TW-523 consists of four wires on the RJ-11 connector,
  * which are jumpered to somewhat more wires on the DB-25 connector, which
  * in turn is intended to plug into the PC parallel printer port.  I dismantled
@@ -135,7 +135,6 @@
 #include "buf.h"
 #include "kernel.h"
 #include "ioctl.h"
-#include "tty.h"
 #include "uio.h"
 #include "syslog.h"
 #include "select.h"
@@ -355,7 +354,7 @@ int twopen(dev, flag, mode, p)
   struct tw_sc *sc = &tw_sc[TWUNIT(dev)];
   int s;
   int port;
-  
+
   s = spltty();
   if(sc->sc_state == 0) {
     sc->sc_state = TWS_OPEN;
@@ -384,9 +383,10 @@ int twclose(dev, flag, mode, p)
   return(0);
 }
 
-int twread(dev, uio)
+int twread(dev, uio, ioflag)
      dev_t dev;
      struct uio *uio;
+     int ioflag;
 {
   u_char buf[3];
   struct tw_sc *sc = &tw_sc[TWUNIT(dev)];
@@ -401,9 +401,10 @@ int twread(dev, uio)
   return(error);
 }
 
-int twwrite(dev, uio)
+int twwrite(dev, uio, ioflag)
      dev_t dev;
      struct uio *uio;
+     int ioflag;
 {
   struct tw_sc *sc;
   int house, key, reps;
@@ -663,7 +664,7 @@ int h, k, cnt;
  * which is more than the 11 cycles it takes to transmit a full
  * X-10 packet.
  */
- 
+
 static int wait_for_zero(sc)
 struct tw_sc *sc;
 {
@@ -929,7 +930,7 @@ int unit;
     sc->sc_state &= ~TWS_RCVING;
     twputpkt(sc, pkt);
     untimeout((timeout_func_t)twabortrcv, (caddr_t)sc);
-    if(sc->sc_flags & TW_RCV_ERROR) 
+    if(sc->sc_flags & TW_RCV_ERROR)
       log(LOG_ERR, "TWRCV: invalid packet: (%d, %x)\n",
 	  sc->sc_rcount, sc->sc_bits);
     wakeup((caddr_t)sc);
