@@ -22,7 +22,7 @@
  * today: Fri Jun  2 17:21:03 EST 1994
  * added 24F support  ++sg
  *
- *      $Id: ultra14f.c,v 1.31 1995/05/07 06:54:01 bde Exp $
+ * $FreeBSD: stable/2.1/sys/i386/isa/ultra14f.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include <sys/types.h>
@@ -173,7 +173,7 @@ struct uha_bits
 #define U24_ABORT_ACK		0x18	/* same */
 #define U24_ICM_ACK		0x02	/* 24F acknowledge ICM and clear */
 
-/* 
+/*
  * UHA_CONF1 bits (read only)
  */
 
@@ -265,18 +265,18 @@ struct uha_data {
 int     uhaprobe();
 int     uha_attach();
 int     uhaintr();
-int32   uha_scsi_cmd();
+int32_t uha_scsi_cmd();
 timeout_t uha_timeout;
 void	uha_free_mscp();
 int     uha_abort();
 void    uhaminphys();
 void    uha_done();
-u_int32 uha_adapter_info();
+u_int32_t uha_adapter_info();
 struct mscp *uha_mscp_phys_kv();
 int	uha_init __P((int unit));
 int	uha24_init __P((int unit));
 
-struct mscp *cheat;
+static	struct mscp *cheat;
 unsigned long int scratch;
 #define	EISA_MAX_SLOTS	16	/* XXX This should go into a comon header */
 static	uha_slot = 0;		/* slot last board was found in */
@@ -370,10 +370,10 @@ uha_send_mbox(int unit, struct mscp *mscp)
 		DELAY(100);
 	}
 	if (spincount == 0) {
-		printf("uha%d: uha_send_mbox, board not responding\n", unit);
+		printf("uha%d: uha_send_mbox, board is not responding\n", unit);
 		Debugger("ultra14f");
 	}
-	outl(ur->ogmptr, KVTOPHYS(mscp)); 
+	outl(ur->ogmptr, KVTOPHYS(mscp));
 	if (uha->flags & UHA_24F) outb(ur->ogmcmd, 1);
 	outb(ur->lint, ub->ogmint);
 	splx(s);
@@ -399,7 +399,7 @@ uha_abort(int unit, struct mscp *mscp)
 		DELAY(10);
 	}
 	if (spincount == 0) {
-		printf("uha%d: uha_abort, board not responding\n", unit);
+		printf("uha%d: uha_abort, board is not responding\n", unit);
 		Debugger("ultra14f");
 	}
 	outl(ur->ogmptr,KVTOPHYS(mscp));
@@ -412,7 +412,7 @@ uha_abort(int unit, struct mscp *mscp)
 		DELAY(10);
 	}
 	if (abortcount == 0) {
-		printf("uha%d: uha_abort, board not responding\n", unit);
+		printf("uha%d: uha_abort, board is not responding\n", unit);
 		Debugger("ultra14f");
 	}
 	if ((inb(ur->sint) & 0x10) != 0) {
@@ -447,7 +447,7 @@ uha_poll(int unit, int wait)
 		DELAY(1000);	/* 1 mSec per loop */
 	}
 	if (wait == 0) {
-		printf("uha%d: uha_poll, board not responding\n", unit);
+		printf("uha%d: uha_poll, board is not responding\n", unit);
 		return (EIO);
 	}
 	uhaintr(unit);
@@ -542,6 +542,7 @@ uha_attach(dev)
 {
 	int     unit = dev->id_unit;
 	struct uha_data *uha = uhadata[unit];
+	struct scsibus_data *scbus;
 
 	/*
 	 * fill in the prototype scsi_link.
@@ -552,11 +553,20 @@ uha_attach(dev)
 	uha->sc_link.device = &uha_dev;
 	uha->sc_link.flags = SDEV_BOUNCE;
 
+	/*
+	 * Prepare the scsibus_data area for the upperlevel
+	 * scsi code.
+	 */
+	scbus = scsi_alloc_bus();
+	if(!scbus)
+                return 0;
+	scbus->adapter_link = &uha->sc_link;
+
 	kdc_uha[unit].kdc_state = DC_BUSY;
 	/*
 	 * ask the adapter what subunits are present
 	 */
-	scsi_attachdevs(&(uha->sc_link));
+	scsi_attachdevs(scbus);
 
 	return 1;
 }
@@ -565,7 +575,7 @@ uha_attach(dev)
  * Return some information to the caller about
  * the adapter and it's capabilities
  */
-u_int32 
+u_int32_t
 uha_adapter_info(unit)
 	int     unit;
 {
@@ -925,7 +935,7 @@ uha_init(unit)
 	uha->flags |= UHA_INIT;
 	return (0);
 }
-  
+
 
 /*
  *  Initialize an Ultrastor 24F
@@ -975,7 +985,7 @@ int	unit;
 	ub->icm_ack	= U24_ICM_ACK;
 
 	/*
-	 * Make sure an EISA card is installed in this slot, 
+	 * Make sure an EISA card is installed in this slot,
 	 * and if it is make sure that the card is enabled.
 	 */
 	outb(ur->id, 0xff);
@@ -1060,7 +1070,7 @@ uhaminphys(bp)
  * start a scsi operation given the command and the data address.  Also
  * needs the unit, target and lu.
  */
-int32 
+int32_t
 uha_scsi_cmd(xs)
 	struct scsi_xfer *xs;
 {
@@ -1208,8 +1218,8 @@ uha_scsi_cmd(xs)
 				nextphys = thisphys;
 				while ((datalen) && (thisphys == nextphys))
 					/*
-					 * This page is contiguous (physically) with 
-					 * the the last, just extend the length 
+					 * This page is contiguous (physically) with
+					 * the the last, just extend the length
 					 */
 				{
 					/* how far to the end of the page */
@@ -1338,7 +1348,7 @@ uha_timeout(arg1)
 	if ((uha_abort(unit, mscp) != 1) || (mscp->flags = MSCP_ABORTED)) {
 		printf("AGAIN");
 		mscp->xs->retries = 0;	/* I MEAN IT ! */
-		uha_done(unit, mscp, FAIL);
+		uha_done(unit, mscp);
 	} else {		/* abort the operation that has timed out */
 		printf("\n");
 		timeout(uha_timeout, (caddr_t)mscp, 2 * hz);
