@@ -34,7 +34,7 @@
  * SUCH DAMAGE.
  *
  *	from: @(#)wd.c	7.2 (Berkeley) 5/9/91
- *	$Id: wd.c,v 1.80 1995/05/09 12:26:00 rgrimes Exp $
+ * $FreeBSD: stable/2.1/sys/i386/isa/wd.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /* TODO:
@@ -56,7 +56,12 @@
  */
 
 #include "wd.h"
-#if	NWDC > 0
+#ifdef  NWDC
+#undef  NWDC
+#endif
+
+#include "wdc.h"
+#if     NWDC > 0
 
 #include <sys/param.h>
 #include <sys/dkbad.h>
@@ -84,6 +89,10 @@
 #include <sys/dkstat.h>
 #include <vm/vm.h>
 
+#ifdef ATAPI
+#include <i386/isa/atapi.h>
+#endif
+
 #define TIMEOUT		10000
 #define	RETRIES		5	/* number of retries before giving up */
 #define RECOVERYTIME	500000	/* usec for controller to recover after err */
@@ -113,7 +122,7 @@ static struct kern_devconf kdc_wd[NWD] = { {
 	DC_CLS_DISK		/* class */
 } };
 
-static struct kern_devconf kdc_wdc[NWDC] = { {
+struct kern_devconf kdc_wdc[NWDC] = { {
 	0, 0, 0,		/* filled in by kern_devconf.c */
 	"wdc", 0, { MDDT_ISA, 0 },
 	isa_generic_externalize, 0, wdc_goaway, ISA_EXTERNALLEN,
@@ -239,7 +248,7 @@ static struct buf rwdbuf[NWD];	/* buffers for raw IO */
 static int wdprobe(struct isa_device *dvp);
 static int wdattach(struct isa_device *dvp);
 static void wdustart(struct disk *du);
-static void wdstart(int ctrlr);
+void wdstart(int ctrlr);
 static int wdcontrol(struct buf *bp);
 static int wdcommand(struct disk *du, u_int cylinder, u_int head,
 		     u_int sector, u_int count, u_int command);
@@ -290,12 +299,36 @@ wdprobe(struct isa_device *dvp)
 	wdc_registerdev(dvp);
 
 	/* check if we have registers that work */
+	outb(du->dk_port + wd_sdh, WDSD_IBM);   /* set unit 0 */
 	outb(du->dk_port + wd_cyl_lo, 0xa5);	/* wd_cyl_lo is read/write */
-	if (inb(du->dk_port + wd_cyl_lo) == 0xff)	/* XXX too weak */
-		goto nodevice;
+	if (inb(du->dk_port + wd_cyl_lo) == 0xff) {     /* XXX too weak */
+#ifdef ATAPI
+		/* There is no master, try the ATAPI slave. */
+		outb(du->dk_port + wd_sdh, WDSD_IBM | 0x10);
+		outb(du->dk_port + wd_cyl_lo, 0xa5);
+		if (inb(du->dk_port + wd_cyl_lo) == 0xff)
+#endif
+			goto nodevice;
+	}
 
-	if (wdreset(du) != 0 && (DELAY(RECOVERYTIME), wdreset(du)) != 0)
+	if (wdreset(du) == 0)
+		goto reset_ok;
+#ifdef ATAPI
+	/* test for ATAPI signature */
+	outb(du->dk_port + wd_sdh, WDSD_IBM);           /* master */
+	if (inb(du->dk_port + wd_cyl_lo) == 0x14 &&
+	    inb(du->dk_port + wd_cyl_hi) == 0xeb)
+		goto reset_ok;
+	du->dk_unit = 1;
+	outb(du->dk_port + wd_sdh, WDSD_IBM | 0x10); /* slave */
+	if (inb(du->dk_port + wd_cyl_lo) == 0x14 &&
+	    inb(du->dk_port + wd_cyl_hi) == 0xeb)
+		goto reset_ok;
+#endif
+	DELAY(RECOVERYTIME);
+	if (wdreset(du) != 0)
 		goto nodevice;
+reset_ok:
 
 	/* execute a controller only command */
 	if (wdcommand(du, 0, 0, 0, 0, WDCC_DIAGNOSE) != 0
@@ -449,7 +482,19 @@ wdattach(struct isa_device *dvp)
 			wddrives[lunit] = NULL;
 		}
 	}
-
+#ifdef ATAPI
+	/*
+	 * Probe all free IDE units, searching for ATAPI drives.
+	 */
+	for (unit=0; unit<2; ++unit) {
+		for (lunit=0; lunit<NWD && wddrives[lunit]; ++lunit)
+			if (wddrives[lunit]->dk_ctrlr == dvp->id_unit &&
+			    wddrives[lunit]->dk_unit == unit)
+				goto next;
+		atapi_attach (dvp->id_unit, unit, dvp->id_iobase,
+			&kdc_wdc[dvp->id_unit]);
+next:   }
+#endif
 	/*
 	 * Discard any interrupts generated by wdgetctlr().  wdflushirq()
 	 * doesn't work now because the ambient ipl is too high.
@@ -612,7 +657,7 @@ wdustart(register struct disk *du)
  * 1. The transfer length must be an exact multiple of the sector size.
  */
 
-static void
+void
 wdstart(int ctrlr)
 {
 	register struct disk *du;
@@ -624,11 +669,23 @@ wdstart(int ctrlr)
 	int	lunit;
 	int	count;
 
+#ifdef ATAPI
+	if (wdtab[ctrlr].b_active == 2)
+		wdtab[ctrlr].b_active = 0;
+	if (wdtab[ctrlr].b_active)
+		return;
+#endif
 loop:
 	/* is there a drive for the controller to do a transfer with? */
 	bp = wdtab[ctrlr].b_actf;
-	if (bp == NULL)
+	if (bp == NULL) {
+#ifdef ATAPI
+		if (atapi_start (ctrlr))
+			/* mark controller active in ATAPI mode */
+			wdtab[ctrlr].b_active = 3;
+#endif
 		return;
+	}
 
 	/* obtain controller and drive information */
 	lunit = dkunit(bp->b_dev);
@@ -854,7 +911,18 @@ wdintr(int unit)
 #endif
 		return;
 	}
-
+#ifdef ATAPI
+	if (wdtab[unit].b_active == 3) {
+		/* process an ATAPI interrupt */
+		if (atapi_intr (unit))
+			/* ATAPI op continues */
+			return;
+		/* controller is free, start new op */
+		wdtab[unit].b_active = 0;
+		wdstart (unit);
+		return;
+	}
+#endif
 	bp = wdtab[unit].b_actf;
 	du = wddrives[dkunit(bp->b_dev)];
 	dp = &wdutab[du->dk_lunit];
@@ -1016,7 +1084,10 @@ done: ;
 	/* anything more on drive queue? */
 	wdustart(du);
 	/* anything more for controller to do? */
+#ifndef ATAPI
+	/* This is not valid in ATAPI mode. */
 	if (wdtab[unit].b_actf)
+#endif
 		wdstart(unit);
 }
 
@@ -1855,18 +1926,26 @@ wdflushirq(struct disk *du, int old_ipl)
 static int
 wdreset(struct disk *du)
 {
-	int	wdc;
+	int     wdc, err = 0;
 
 	wdc = du->dk_port;
 	(void)wdwait(du, 0, TIMEOUT);
 	outb(wdc + wd_ctlr, WDCTL_IDS | WDCTL_RST);
 	DELAY(10 * 1000);
 	outb(wdc + wd_ctlr, WDCTL_IDS);
+#ifdef ATAPI
+	if (wdwait(du, WDCS_READY | WDCS_SEEKCMPLT, TIMEOUT) != 0)
+		err = 1;                /* no IDE drive found */
+	du->dk_error = inb(wdc + wd_error);
+	if (du->dk_error != 0x01)
+		err = 1;                /* the drive is incompatible */
+#else
 	if (wdwait(du, WDCS_READY | WDCS_SEEKCMPLT, TIMEOUT) != 0
 	    || (du->dk_error = inb(wdc + wd_error)) != 0x01)
 		return (1);
+#endif
 	outb(wdc + wd_ctlr, WDCTL_4BIT);
-	return (0);
+	return (err);
 }
 
 /*
@@ -1979,6 +2058,18 @@ wdwait(struct disk *du, u_char bits_wanted, int timeout)
 			min_retries[du->dk_ctrlr] = timeout;
 #endif
 		du->dk_status = status = inb(wdc + wd_status);
+#ifdef ATAPI
+		/*
+		 * Atapi drives have a very interesting feature, when attached
+		 * as a slave on the IDE bus, and there is no master.
+		 * They release the bus after getting the command.
+		 * We should reselect the drive here to get the status.
+		 */
+		if (status == 0xff) {
+			outb(wdc + wd_sdh, WDSD_IBM | du->dk_unit << 4);
+			du->dk_status = status = inb(wdc + wd_status);
+		}
+#endif
 		if (!(status & WDCS_BUSY)) {
 			if (status & WDCS_ERR) {
 				du->dk_error = inb(wdc + wd_error);
