@@ -19,26 +19,32 @@
  * the original CMU copyright notice.
  *
  * Version 1.3, Thu Nov 11 12:09:13 MSK 1993
- * $Id: wt.c,v 1.15 1995/03/28 07:55:56 bde Exp $
+ * $FreeBSD: stable/2.1/sys/i386/isa/wt.c 50950 1999-09-05 08:39:22Z peter $
  *
  */
 
+/*
+ * Code for MTERASE added by John Lind (john@starfire.mn.org) 95/09/02.
+ * This was very easy due to the excellent structure and clear coding
+ * of the original driver.
+ */
+
 /*
  * Copyright (c) 1989 Carnegie-Mellon University.
  * All rights reserved.
  *
  * Authors: Robert Baron
- * 
+ *
  * Permission to use, copy, modify and distribute this software and
  * its documentation is hereby granted, provided that both the copyright
  * notice and this permission notice appear in all copies of the
  * software, derivative works or modified versions, and any portions
  * thereof, and that both notices appear in supporting documentation.
- * 
- * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS" 
- * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND 
+ *
+ * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
+ * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
  * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
- * 
+ *
  * Carnegie Mellon requests users of this software to return to
  *
  *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
@@ -258,13 +264,13 @@ int wtattach (struct isa_device *id)
 
 struct isa_driver wtdriver = { wtprobe, wtattach, "wt", };
 
-int wtdump (int dev)
+int wtdump (dev_t dev)
 {
 	/* Not implemented */
 	return (EINVAL);
 }
 
-int wtsize (int dev)
+int wtsize (dev_t dev)
 {
 	/* Not implemented */
 	return (-1);
@@ -273,7 +279,7 @@ int wtsize (int dev)
 /*
  * Open routine, called on every device open.
  */
-int wtopen (int dev, int flag)
+int wtopen (dev_t dev, int flag, int fmt, struct proc *p)
 {
 	int u = minor (dev) & T_UNIT;
 	wtinfo_t *t = wttab + u;
@@ -351,7 +357,7 @@ int wtopen (int dev, int flag)
 /*
  * Close routine, called on last device close.
  */
-int wtclose (int dev)
+int wtclose (dev_t dev, int flags, int fmt, struct proc *p)
 {
 	int u = minor (dev) & T_UNIT;
 	wtinfo_t *t = wttab + u;
@@ -395,13 +401,11 @@ done:
 
 /*
  * Ioctl routine.  Compatible with BSD ioctls.
- * Direct QIC-02 commands ERASE and RETENSION added.
- * There are three possible ioctls:
+ * There are two possible ioctls:
  * ioctl (int fd, MTIOCGET, struct mtget *buf)  -- get status
  * ioctl (int fd, MTIOCTOP, struct mtop *buf)   -- do BSD-like op
- * ioctl (int fd, WTQICMD, int qicop)           -- do QIC op
  */
-int wtioctl (int dev, int cmd, void *arg, int mode)
+int wtioctl (dev_t dev, int cmd, caddr_t arg, int flags, struct proc *p)
 {
 	int u = minor (dev) & T_UNIT;
 	wtinfo_t *t = wttab + u;
@@ -413,32 +417,6 @@ int wtioctl (int dev, int cmd, void *arg, int mode)
 	switch (cmd) {
 	default:
 		return (EINVAL);
-	case WTQICMD:                   /* direct QIC command */
-		op = (int) *(void**)arg;
-		switch (op) {
-		default:
-			return (EINVAL);
-		case QIC_ERASE:         /* erase the whole tape */
-			if (! (t->flags & TPWRITE) || (t->flags & TPWP))
-				return (EACCES);
-			if (error = wtwait (t, PCATCH, "wterase"))
-				return (error);
-			break;
-		case QIC_RETENS:        /* retension the tape */
-			if (error = wtwait (t, PCATCH, "wtretens"))
-				return (error);
-			break;
-		}
-		/* Both ERASE and RETENS operations work like REWIND. */
-		/* Simulate the rewind operation here. */
-		t->flags &= ~(TPRO | TPWO | TPVOL);
-		if (! wtcmd (t, op))
-			return (EIO);
-		t->flags |= TPSTART | TPREW;
-		if (op == QIC_ERASE)
-			t->flags |= TPWANY;
-		wtclock (t);
-		return (0);
 	case MTIOCIEOT:         /* ignore EOT errors */
 	case MTIOCEEOT:         /* enable EOT errors */
 		return (0);
@@ -488,6 +466,28 @@ int wtioctl (int dev, int cmd, void *arg, int mode)
 		if (error = wtwritefm (t))
 			return (error);
 		return (0);
+	case MTRETENS:		/* re-tension tape */
+		if (error = wtwait (t, PCATCH, "wtretens"))
+			return (error);
+		op = QIC_RETENS;
+		goto erase_retens;
+		
+	case MTERASE:		/* erase to EOM */
+		if (! (t->flags & TPWRITE) || (t->flags & TPWP))
+			return (EACCES);
+		if (error = wtwait (t, PCATCH, "wterase"))
+			return (error);
+		op = QIC_ERASE;
+	erase_retens:
+		/* ERASE and RETENS operations work like REWIND. */
+		/* Simulate the rewind operation here. */
+		t->flags &= ~(TPRO | TPWO | TPVOL);
+		if (! wtcmd (t, op))
+			return (EIO);
+		t->flags |= TPSTART | TPREW;
+		t->flags |= TPWANY;
+		wtclock (t);
+		return (0);
 	}
 	return (EINVAL);
 }
@@ -740,7 +740,7 @@ static int wtcmd (wtinfo_t *t, int cmd)
 	        splx(x);
 		return (0);
 	}
-	
+
 	outb (t->CMDPORT, cmd);                         /* output the command */
 
 	outb (t->CTLPORT, t->REQUEST | t->ONLINE);      /* set request */
