@@ -36,7 +36,7 @@
  * SUCH DAMAGE.
  *
  *	@(#)cd9660_vnops.c	8.3 (Berkeley) 1/23/94
- * $Id: cd9660_vnops.c,v 1.12 1995/02/13 06:12:32 phk Exp $
+ * $FreeBSD: stable/2.1/sys/isofs/cd9660/cd9660_vnops.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include <sys/param.h>
@@ -81,10 +81,10 @@ cd9660_mknod(ndp, vap, cred, p)
 	struct iso_node *ip;
 	struct iso_dnode *dp;
 	int error;
-	
+
 	vp = ndp->ni_vp;
 	ip = VTOI(vp);
-	
+
 	if (ip->i_mnt->iso_ftype != ISO_FTYPE_RRIP
 	    || vap->va_type != vp->v_type
 	    || (vap->va_type != VCHR && vap->va_type != VBLK)) {
@@ -93,7 +93,7 @@ cd9660_mknod(ndp, vap, cred, p)
 		vput(ndp->ni_vp);
 		return EINVAL;
 	}
-	
+
 	dp = iso_dmap(ip->i_dev,ip->i_number,1);
 	if (ip->inode.iso_rdev == vap->va_rdev || vap->va_rdev == VNOVAL) {
 		/* same as the unmapped one, delete the mapping */
@@ -102,7 +102,7 @@ cd9660_mknod(ndp, vap, cred, p)
 	} else
 		/* enter new mapping */
 		dp->d_dev = vap->va_rdev;
-	
+
 	/*
 	 * Remove inode so that it will be reloaded by iget and
 	 * checked to see if it is an alias of an existing entry
@@ -116,6 +116,43 @@ cd9660_mknod(ndp, vap, cred, p)
 }
 #endif
 
+/*
+ * Setattr call. Only allowed for block and character special devices.
+ */
+int
+cd9660_setattr(ap)
+	struct vop_setattr_args /* {
+		struct vnodeop_desc *a_desc;
+		struct vnode *a_vp;
+		struct vattr *a_vap;
+		struct ucred *a_cred;
+		struct proc *a_p;
+	} */ *ap;
+{
+	struct vnode *vp = ap->a_vp;
+	struct vattr *vap = ap->a_vap;
+
+  	if (vap->va_flags != VNOVAL || vap->va_uid != (uid_t)VNOVAL ||
+	    vap->va_gid != (gid_t)VNOVAL || vap->va_atime.ts_sec != VNOVAL ||
+	    vap->va_mtime.ts_sec != VNOVAL || vap->va_mode != (mode_t)VNOVAL)
+		return (EROFS);
+	if (vap->va_size != VNOVAL) {
+ 		switch (vp->v_type) {
+ 		case VDIR:
+ 			return (EISDIR);
+		case VLNK:
+		case VREG:
+			return (EROFS);
+ 		case VCHR:
+ 		case VBLK:
+ 		case VSOCK:
+ 		case VFIFO:
+			return (0);
+		}
+	}
+	return (EOPNOTSUPP);
+}
+
 /*
  * Open called.
  *
@@ -167,6 +204,22 @@ cd9660_access(ap)
 		struct proc *a_p;
 	} */ *ap;
 {
+	/*
+	 * Disallow write attempts on read-only file systems;
+	 * unless the file is a socket, fifo, or a block or
+	 * character device resident on the file system.
+	 */
+	if (ap->a_mode & VWRITE) {
+		switch (ap->a_vp->v_type) {
+		case VDIR:
+		case VLNK:
+		case VREG:
+			if (ap->a_vp->v_mount->mnt_flag & MNT_RDONLY)
+				return (EROFS);
+			break;
+		}
+	}
+
 	return (0);
 }
 
@@ -237,7 +290,7 @@ cd9660_read(ap)
 	off_t diff;
 	int rasize, error = 0;
 	long size, n, on;
-	
+
 	if (uio->uio_resid == 0)
 		return (0);
 	if (uio->uio_offset < 0)
@@ -260,7 +313,7 @@ cd9660_read(ap)
 			if (iso_lblktosize(imp, rablock) <= ip->i_size)
 				error = cluster_read(vp, (off_t)ip->i_size,
 						     lbn, size, NOCRED, &bp);
-			else 
+			else
 				error = bread(vp, lbn, size, NOCRED, &bp);
 		} else {
 			if (vp->v_lastr + 1 == lbn &&
@@ -380,25 +433,25 @@ iso_uiodir(idp,dp,off)
 	off_t off;
 {
 	int error;
-	
+
 	dp->d_name[dp->d_namlen] = 0;
 	dp->d_reclen = DIRSIZ(dp);
-	
+
 	if (idp->uio->uio_resid < dp->d_reclen) {
 		idp->eof = 0;
 		return -1;
 	}
-	
+
 	if (idp->cookiep) {
 		if (idp->ncookies <= 0) {
 			idp->eof = 0;
 			return -1;
 		}
-		
+
 		*idp->cookiep++ = off;
 		--idp->ncookies;
 	}
-	
+
 	if ((error = uiomove((caddr_t)dp,dp->d_reclen,idp->uio)))
 		return error;
 	idp->uio_off = off;
@@ -413,7 +466,7 @@ iso_shipdir(idp)
 	int cl, sl, assoc;
 	int error;
 	char *cname, *sname;
-	
+
 	cl = idp->current.d_namlen;
 	cname = idp->current.d_name;
 assoc = (cl > 1) && (*cname == ASSOCCHAR);
@@ -421,7 +474,7 @@ assoc = (cl > 1) && (*cname == ASSOCCHAR);
 		cl--;
 		cname++;
 	}
-	
+
 	dp = &idp->saveent;
 	sname = dp->d_name;
 	if (!(sl = dp->d_namlen)) {
@@ -475,6 +528,7 @@ cd9660_readdir(ap)
 	int endsearch;
 	struct iso_directory_record *ep;
 	u_short elen;
+	int namlen;
 	int reclen;
 	int isoflags;
 	struct iso_mnt *imp;
@@ -483,10 +537,10 @@ cd9660_readdir(ap)
 	u_short tmplen;
 	int ncookies = 0;
 	u_int *cookies = NULL;
-	
+
 	ip = VTOI(ap->a_vp);
 	imp = ip->i_mnt;
-	
+
 	MALLOC(idp,struct isoreaddir *,sizeof(*idp),M_TEMP,M_WAITOK);
 	idp->saveent.d_namlen = 0;
 	idp->assocent.d_namlen = 0;
@@ -503,7 +557,7 @@ cd9660_readdir(ap)
 		idp->cookiep = 0;
 	idp->eof = 0;
 	idp->curroff = uio->uio_offset;
-	
+
 	entryoffsetinblock = iso_blkoff(imp, idp->curroff);
 	if (entryoffsetinblock != 0) {
 		if ((error = iso_blkatoff(ip, idp->curroff, &bp))) {
@@ -511,16 +565,16 @@ cd9660_readdir(ap)
 			return (error);
 		}
 	}
-	
+
 	endsearch = ip->i_size;
-	
+
 	while (idp->curroff < endsearch) {
 		/*
 		 * If offset is on a block boundary,
 		 * read the next directory block.
 		 * Release previous if it exists.
 		 */
-		
+
 		if (iso_blkoff(imp, idp->curroff) == 0) {
 			if (bp != NULL)
 				brelse(bp);
@@ -531,48 +585,49 @@ cd9660_readdir(ap)
 		/*
 		 * Get pointer to next entry.
 		 */
-		
+
 		ep = (struct iso_directory_record *)
 			(bp->b_un.b_addr + entryoffsetinblock);
-		
+
 		reclen = isonum_711 (ep->length);
-		isoflags = isonum_711(imp->iso_ftype == ISO_FTYPE_HIGH_SIERRA?
-				      &ep->date[6]: ep->flags);
 		if (reclen == 0) {
 			/* skip to next block, if any */
 			idp->curroff = roundup (idp->curroff,
 						imp->logical_block_size);
 			continue;
 		}
-		
+
 		if (reclen < ISO_DIRECTORY_RECORD_SIZE) {
 			error = EINVAL;
 			/* illegal entry, stop */
 			break;
 		}
-		
+
 		if (entryoffsetinblock + reclen > imp->logical_block_size) {
 			error = EINVAL;
 			/* illegal directory, so stop looking */
 			break;
 		}
-		
+
+		namlen = isonum_711 (ep->name_len);
+		if (reclen < ISO_DIRECTORY_RECORD_SIZE + namlen) {
+			error = EINVAL;
+			/* illegal entry, stop */
+			break;
+		}
+
 		/* XXX: be more intelligent if we can */
 		idp->current.d_type = DT_UNKNOWN;
 
-		idp->current.d_namlen = isonum_711 (ep->name_len);
+		idp->current.d_namlen = namlen;
+		isoflags = isonum_711(imp->iso_ftype == ISO_FTYPE_HIGH_SIERRA?
+				      &ep->date[6]: ep->flags);
 		if (isoflags & 2)
 			isodirino(&idp->current.d_fileno,ep,imp);
 		else
 			idp->current.d_fileno = dbtob(bp->b_blkno) +
 				idp->curroff;
-		
-		if (reclen < ISO_DIRECTORY_RECORD_SIZE + idp->current.d_namlen) {
-			error = EINVAL;
-			/* illegal entry, stop */
-			break;
-		}
-		
+
 		idp->curroff += reclen;
 		/*
 		 *
@@ -612,10 +667,10 @@ cd9660_readdir(ap)
 		}
 		if (error)
 			break;
-		
+
 		entryoffsetinblock += reclen;
 	}
-	
+
 	if (!error && imp->iso_ftype == ISO_FTYPE_DEFAULT) {
 		idp->current.d_namlen = 0;
 		error = iso_shipdir(idp);
@@ -634,16 +689,16 @@ cd9660_readdir(ap)
 			*ap->a_cookies = cookies;
 		}
 	}
-	
+
 	if (bp)
 		brelse (bp);
 
 	uio->uio_offset = idp->uio_off;
 	if (ap->a_eofflag)
 	    *ap->a_eofflag = idp->eof;
-	
+
 	FREE(idp,M_TEMP);
-	
+
 	return (error);
 }
 
@@ -665,19 +720,19 @@ cd9660_readlink(ap)
 	} */ *ap;
 {
 	ISONODE	*ip;
-	ISODIR	*dirp;			 
+	ISODIR	*dirp;
 	ISOMNT	*imp;
 	struct	buf *bp;
 	u_short	symlen;
 	int	error;
 	char	*symname;
-	
+
 	ip  = VTOI(ap->a_vp);
 	imp = ip->i_mnt;
-	
+
 	if (imp->iso_ftype != ISO_FTYPE_RRIP)
 		return EINVAL;
-	
+
 	/*
 	 * Get parents directory record block that this inode included.
 	 */
@@ -706,7 +761,7 @@ cd9660_readlink(ap)
 	       ip->i_number,
 	       ip->i_number & imp->im_bmask );
 #endif
-	
+
 	/*
 	 * Just make sure, we have a right one....
 	 *   1: Check not cross boundary on block
@@ -716,13 +771,13 @@ cd9660_readlink(ap)
 		brelse(bp);
 		return EINVAL;
 	}
-	
+
 	/*
 	 * Now get a buffer
 	 * Abuse a namei buffer for now.
 	 */
 	MALLOC(symname,char *,MAXPATHLEN,M_NAMEI,M_WAITOK);
-	
+
 	/*
 	 * Ok, we just gathering a symbolic name in SL record.
 	 */
@@ -735,14 +790,14 @@ cd9660_readlink(ap)
 	 * Don't forget before you leave from home ;-)
 	 */
 	brelse(bp);
-	
+
 	/*
 	 * return with the symbolic name to caller's.
 	 */
 	error = uiomove(symname,symlen,ap->a_uio);
-	
+
 	FREE(symname,M_NAMEI);
-	
+
 	return error;
 }
 
@@ -877,8 +932,6 @@ cd9660_enotsupp()
 #define cd9660_create \
 	((int (*) __P((struct  vop_create_args *)))cd9660_enotsupp)
 #define cd9660_mknod ((int (*) __P((struct  vop_mknod_args *)))cd9660_enotsupp)
-#define cd9660_setattr \
-	((int (*) __P((struct  vop_setattr_args *)))cd9660_enotsupp)
 #define cd9660_write ((int (*) __P((struct  vop_write_args *)))cd9660_enotsupp)
 #define cd9660_fsync ((int (*) __P((struct  vop_fsync_args *)))nullop)
 #define cd9660_remove \
