@@ -30,7 +30,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id$
+ * $FreeBSD: stable/2.1/sys/kern/kern_lkm.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -174,11 +174,12 @@ lkmcclose(dev, flag, mode, p)
 
 /*ARGSUSED*/
 int
-lkmcioctl(dev, cmd, data, flag)
+lkmcioctl(dev, cmd, data, flag, p)
 	dev_t dev;
 	int cmd;
 	caddr_t data;
 	int flag;
+	struct proc *p;
 {
 	int err = 0;
 	int i;
@@ -190,7 +191,8 @@ lkmcioctl(dev, cmd, data, flag)
 
 	switch(cmd) {
 	case LMRESERV:		/* reserve pages for a module */
-		if ((flag & FWRITE) == 0) /* only allow this if writing */
+		if ((flag & FWRITE) == 0 || securelevel > 0)
+			/* only allow this if writing */
 			return EPERM;
 
 		resrvp = (struct lmc_resrv *)data;
@@ -230,7 +232,8 @@ lkmcioctl(dev, cmd, data, flag)
 		break;
 
 	case LMLOADBUF:		/* Copy in; stateful, follows LMRESERV */
-		if ((flag & FWRITE) == 0) /* only allow this if writing */
+		if ((flag & FWRITE) == 0 || securelevel > 0)
+			/* only allow this if writing */
 			return EPERM;
 
 		loadbufp = (struct lmc_loadbuf *)data;
@@ -265,7 +268,8 @@ lkmcioctl(dev, cmd, data, flag)
 		break;
 
 	case LMUNRESRV:		/* discard reserved pages for a module */
-		if ((flag & FWRITE) == 0) /* only allow this if writing */
+		if ((flag & FWRITE) == 0 || securelevel > 0)
+			/* only allow this if writing */
 			return EPERM;
 
 		lkmunreserve();	/* coerce state to LKM_IDLE */
@@ -275,7 +279,8 @@ lkmcioctl(dev, cmd, data, flag)
 		break;
 
 	case LMREADY:		/* module loaded: call entry */
-		if ((flag & FWRITE) == 0) /* only allow this if writing */
+		if ((flag & FWRITE) == 0 || securelevel > 0)
+			/* only allow this if writing */
 			return EPERM;
 
 		switch (lkm_state) {
@@ -337,7 +342,8 @@ lkmcioctl(dev, cmd, data, flag)
 		break;
 
 	case LMUNLOAD:		/* unload a module */
-		if ((flag & FWRITE) == 0) /* only allow this if writing */
+		if ((flag & FWRITE) == 0 || securelevel > 0)
+			/* only allow this if writing */
 			return EPERM;
 
 		unloadp = (struct lmc_unload *)data;
@@ -442,7 +448,7 @@ lkmcioctl(dev, cmd, data, flag)
 		statp->size	= curp->size / PAGESIZE;
 		statp->private	= (unsigned long)curp->private.lkm_any;
 		statp->ver	= curp->private.lkm_any->lkm_ver;
-		copystr(curp->private.lkm_any->lkm_name, 
+		copystr(curp->private.lkm_any->lkm_name,
 			  statp->name,
 			  MAXLKMNAME - 2,
 			  NULL);
@@ -464,10 +470,13 @@ lkmcioctl(dev, cmd, data, flag)
  * Place holder for system call slots reserved for loadable modules.
  */
 int
-lkmnosys()
+lkmnosys(p, args, retval)
+	struct proc *p;
+	struct nosys_args *args;
+	int *retval;
 {
 
-	return(nosys());
+	return(nosys(p, args, retval));
 }
 
 /*
@@ -478,7 +487,11 @@ lkmnosys()
  * Place holder for device switch slots reserved for loadable modules.
  */
 int
-lkmenodev()
+lkmenodev(dev, flags, fmt, p)
+	dev_t dev;
+	int flags;
+	int fmt;
+	struct proc *p;
 {
 
 	return(enodev());
@@ -547,11 +560,11 @@ _lkm_syscall(lkmtp, cmd)
 
 		/* save old */
 		bcopy(&aout_sysvec.sv_table[i],
-		      &(args->lkm_oldent), 
+		      &(args->lkm_oldent),
 		      sizeof(struct sysent));
 
 		/* replace with new */
-		bcopy(args->lkm_sysent, 
+		bcopy(args->lkm_sysent,
 		      &aout_sysvec.sv_table[i],
 		      sizeof(struct sysent));
 
@@ -565,8 +578,8 @@ _lkm_syscall(lkmtp, cmd)
 		i = args->lkm_offset;
 
 		/* replace current slot contents with old contents */
-		bcopy(&(args->lkm_oldent), 
-		      &aout_sysvec.sv_table[i], 
+		bcopy(&(args->lkm_oldent),
+		      &aout_sysvec.sv_table[i],
 		      sizeof(struct sysent));
 
 		break;
@@ -623,7 +636,7 @@ _lkm_vfs(lkmtp, cmd)
 		for(i = 0; args->lkm_vnodeops->ls_items[i]; i++) {
 			struct vnodeopv_desc *opv =
 			  (struct vnodeopv_desc *)args->lkm_vnodeops->ls_items[i];
-			*(opv->opv_desc_vector_p) = NULL; 
+			*(opv->opv_desc_vector_p) = NULL;
 		}
 		vfs_opv_init((struct vnodeopv_desc **)args->lkm_vnodeops->ls_items);
 
@@ -813,10 +826,9 @@ _lkm_exec(lkmtp, cmd)
 	struct lkm_exec *args = lkmtp->private.lkm_exec;
 	int i;
 	int err = 0;
-	const struct execsw **execsw = 
+	const struct execsw **execsw =
 		(const struct execsw **)&execsw_set.ls_items[0];
 
-#if 1
 	switch(cmd) {
 	case LKM_E_LOAD:
 		/* don't load twice! */
@@ -862,9 +874,6 @@ _lkm_exec(lkmtp, cmd)
 	case LKM_E_STAT:	/* no special handling... */
 		break;
 	}
-#else
-	err = EINVAL;
-#endif
 	return(err);
 }
 
@@ -882,7 +891,7 @@ TEXT_SET(execsw_set, lkm_exec_dummy);
  */
 int
 lkmdispatch(lkmtp, cmd)
-	struct lkm_table *lkmtp;	
+	struct lkm_table *lkmtp;
 	int cmd;
 {
 	int err = 0;		/* default = success */
