@@ -36,7 +36,7 @@
  * SUCH DAMAGE.
  *
  *	@(#)kern_sig.c	8.7 (Berkeley) 4/18/94
- * $Id: kern_sig.c,v 1.9 1995/01/29 01:19:12 ats Exp $
+ * $FreeBSD: stable/2.1/sys/kern/kern_sig.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #define	SIGPROP		/* include signal properties table */
@@ -266,7 +266,7 @@ sigprocmask(p, uap, retval)
 	case SIG_SETMASK:
 		p->p_sigmask = uap->mask &~ sigcantmask;
 		break;
-	
+
 	default:
 		error = EINVAL;
 		break;
@@ -507,13 +507,13 @@ killpg1(cp, signum, pgid, all)
 	register struct pcred *pc = cp->p_cred;
 	struct pgrp *pgrp;
 	int nfound = 0;
-	
-	if (all)	
-		/* 
-		 * broadcast 
+
+	if (all)
+		/*
+		 * broadcast
 		 */
 		for (p = (struct proc *)allproc; p != NULL; p = p->p_next) {
-			if (p->p_pid <= 1 || p->p_flag & P_SYSTEM || 
+			if (p->p_pid <= 1 || p->p_flag & P_SYSTEM ||
 			    p == cp || !CANSIGNAL(cp, pc, p, signum))
 				continue;
 			nfound++;
@@ -521,8 +521,8 @@ killpg1(cp, signum, pgid, all)
 				psignal(p, signum);
 		}
 	else {
-		if (pgid == 0)		
-			/* 
+		if (pgid == 0)
+			/*
 			 * zero pgid means send to my process group.
 			 */
 			pgrp = cp->p_pgrp;
@@ -650,7 +650,7 @@ trapsignal(p, signum, code)
 		p->p_stats->p_ru.ru_nsignals++;
 #ifdef KTRACE
 		if (KTRPOINT(p, KTR_PSIG))
-			ktrpsig(p->p_tracep, signum, ps->ps_sigact[signum], 
+			ktrpsig(p->p_tracep, signum, ps->ps_sigact[signum],
 				p->p_sigmask, code);
 #endif
 		sendsig(ps->ps_sigact[signum], signum, p->p_sigmask, code);
@@ -1109,9 +1109,8 @@ killproc(p, why)
 	struct proc *p;
 	char *why;
 {
-
-	log(LOG_ERR, "pid %d was killed: %s\n", p->p_pid, why);
-	uprintf("sorry, pid %d was killed: %s\n", p->p_pid, why);
+	log(LOG_ERR, "pid %d (%s), uid %d, was killed: %s\n", p->p_pid, p->p_comm,
+		p->p_cred && p->p_ucred ? p->p_ucred->cr_uid : -1, why);
 	psignal(p, SIGKILL);
 }
 
@@ -1134,12 +1133,14 @@ sigexit(p, signum)
 		p->p_sigacts->ps_sig = signum;
 		/*
 		 * Log signals which would cause core dumps
-		 * (Log as LOG_INFO to appease those who don't want 
+		 * (Log as LOG_INFO to appease those who don't want
 		 * these messages.)
 		 * XXX : Todo, as well as euid, write out ruid too
 		 */
-		log(LOG_INFO, "pid %d: %s: uid %d: exited on signal %d\n",
-			p->p_pid, p->p_comm, p->p_ucred->cr_uid, signum);
+		log(LOG_INFO, "pid %d (%s), uid %d: exited on signal %d\n",
+			p->p_pid, p->p_comm,
+			p->p_cred && p->p_ucred ? p->p_ucred->cr_uid : -1,
+			signum);
 		if (coredump(p) == 0)
 			signum |= WCOREFLAG;
 	}
@@ -1156,15 +1157,14 @@ coredump(p)
 	register struct proc *p;
 {
 	register struct vnode *vp;
-	register struct pcred *pcred = p->p_cred;
-	register struct ucred *cred = pcred->pc_ucred;
+	register struct ucred *cred = p->p_cred->pc_ucred;
 	register struct vmspace *vm = p->p_vmspace;
 	struct nameidata nd;
 	struct vattr vattr;
 	int error, error1;
 	char name[MAXCOMLEN+6];		/* progname.core */
 
-	if (pcred->p_svuid != pcred->p_ruid || pcred->p_svgid != pcred->p_rgid)
+	if (p->p_flag & P_SUGID)
 		return (EFAULT);
 	if (ctob(UPAGES + vm->vm_dsize + vm->vm_ssize) >=
 	    p->p_rlimit[RLIMIT_CORE].rlim_cur)
