@@ -36,7 +36,7 @@
  * SUCH DAMAGE.
  *
  *	@(#)kern_synch.c	8.6 (Berkeley) 1/21/94
- * $Id: kern_synch.c,v 1.9 1994/12/12 06:04:27 davidg Exp $
+ * $FreeBSD: stable/2.1/sys/kern/kern_synch.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include <sys/param.h>
@@ -108,7 +108,7 @@ roundrobin(arg)
  * We now need to prove two things:
  *	1) Given factor ** (5 * loadavg) ~= .1, prove factor == b/(b+1)
  *	2) Given b/(b+1) ** power ~= .1, prove power == (5 * loadavg)
- *	
+ *
  * Facts:
  *         For x close to zero, exp(x) =~ 1 + x, since
  *              exp(x) = 0! + x**1/1! + x**2/2! + ... .
@@ -577,7 +577,7 @@ mi_switch()
 		rlim = &p->p_rlimit[RLIMIT_CPU];
 		if (s >= rlim->rlim_cur) {
 			if (s >= rlim->rlim_max)
-				psignal(p, SIGKILL);
+				killproc(p, "exceeded maximum CPU limit");
 			else {
 				psignal(p, SIGXCPU);
 				if (rlim->rlim_cur < rlim->rlim_max)
