@@ -34,7 +34,7 @@
  * SUCH DAMAGE.
  *
  *	@(#)kern_sysctl.c	8.4 (Berkeley) 4/14/94
- * $Id: kern_sysctl.c,v 1.23 1995/03/16 18:12:37 bde Exp $
+ * $FreeBSD: stable/2.1/sys/kern/kern_sysctl.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -178,6 +178,12 @@ char kernelname[MAXPATHLEN] = "/kernel";	/* XXX bloat */
 extern int vfs_update_wakeup;
 extern int vfs_update_interval;
 extern int osreldate;
+extern int exec_ps_strings;
+extern int exec_usrstack;
+extern int somaxconn;
+extern u_long sb_max;
+extern u_long sb_efficiency;
+extern int sominqueue;
 
 /*
  * kernel related system variables.
@@ -192,7 +198,7 @@ kern_sysctl(name, namelen, oldp, oldlenp, newp, newlen, p)
 	size_t newlen;
 	struct proc *p;
 {
-	int error, level, inthostid;
+	int error, level, inthostid, intsb_max, intsb_efficiency;
 	dev_t ndumpdev;
 
 	/* all sysctl names at this level are terminal */
@@ -230,7 +236,7 @@ kern_sysctl(name, namelen, oldp, oldlenp, newp, newlen, p)
 		 * `tsleep' takes a timeout argument of 0 as meaning
 		 * `no timeout'.
 		 */
-		error = sysctl_int(oldp, oldlenp, newp, newlen, 
+		error = sysctl_int(oldp, oldlenp, newp, newlen,
 				   &vfs_update_interval);
 		if(!error) {
 			wakeup(&vfs_update_wakeup);
@@ -243,21 +249,23 @@ kern_sysctl(name, namelen, oldp, oldlenp, newp, newlen, p)
 		if ((error = sysctl_int(oldp, oldlenp, newp, newlen, &level)) ||
 		    newp == NULL)
 			return (error);
-		if (level < securelevel && p->p_pid != 1)
+		if (level < securelevel)
 			return (EPERM);
 		securelevel = level;
 		return (0);
 	case KERN_HOSTNAME:
 		error = sysctl_string(oldp, oldlenp, newp, newlen,
 		    hostname, sizeof(hostname));
-		if (newp && !error)
-			hostnamelen = newlen;
+		if (newp)
+			if (error == 0 || error == ENOMEM)
+				hostnamelen = newlen;
 		return (error);
 	case KERN_DOMAINNAME:
 		error = sysctl_string(oldp, oldlenp, newp, newlen,
 		    domainname, sizeof(domainname));
-		if (newp && !error)
-			domainnamelen = newlen;
+		if (newp)
+			if (error == 0 || error == ENOMEM)
+				domainnamelen = newlen;
 		return (error);
 	case KERN_HOSTID:
 		inthostid = hostid;  /* XXX assumes sizeof long <= sizeof int */
@@ -303,6 +311,24 @@ kern_sysctl(name, namelen, oldp, oldlenp, newp, newlen, p)
 			error = setdumpdev(ndumpdev);
 		}
 		return error;
+	case KERN_SOMAXCONN:
+		return(sysctl_int(oldp, oldlenp, newp, newlen, &somaxconn));
+	case KERN_MAXSOCKBUF:
+		intsb_max = sb_max;  /* XXX assumes sizeof long <= sizeof int */
+		error = sysctl_int(oldp, oldlenp, newp, newlen, &intsb_max);
+		sb_max = intsb_max;
+		return (error);
+	case KERN_PS_STRINGS:
+		return(sysctl_int(oldp, oldlenp, newp, newlen, &exec_ps_strings));
+	case KERN_USRSTACK:
+		return(sysctl_int(oldp, oldlenp, newp, newlen, &exec_usrstack));
+	case KERN_SOCKBUF_WASTE:
+		intsb_efficiency = sb_efficiency;  /* XXX assumes sizeof long <= sizeof int */
+		error = sysctl_int(oldp, oldlenp, newp, newlen, &intsb_efficiency);
+		sb_efficiency = intsb_efficiency;
+		return (error);
+	case KERN_SOMINQUEUE:
+		return(sysctl_int(oldp, oldlenp, newp, newlen, &sominqueue));
 	default:
 		return (EOPNOTSUPP);
 	}
@@ -458,22 +484,28 @@ sysctl_string(oldp, oldlenp, newp, newlen, str, maxlen)
 	char *str;
 	int maxlen;
 {
-	int len, error = 0;
+	int len, error = 0, rval = 0;
 
 	len = strlen(str) + 1;
-	if (oldp && *oldlenp < len)
+	if (oldp && *oldlenp < len) {
 		len = *oldlenp;
+		rval = ENOMEM;
+	}
 	if (newp && newlen >= maxlen)
 		return (EINVAL);
 	if (oldp) {
 		*oldlenp = len;
 		error = copyout(str, oldp, len);
+		if (error)
+			rval = error;
 	}
-	if (error == 0 && newp) {
+	if ((error == 0 || error == ENOMEM) && newp) {
 		error = copyin(newp, str, newlen);
+		if (error)
+			rval = error;
 		str[newlen] = 0;
 	}
-	return (error);
+	return (rval);
 }
 
 /*
@@ -486,17 +518,21 @@ sysctl_rdstring(oldp, oldlenp, newp, str)
 	void *newp;
 	char *str;
 {
-	int len, error = 0;
+	int len, error = 0, rval = 0;
 
 	len = strlen(str) + 1;
-	if (oldp && *oldlenp < len)
-		return (ENOMEM);
+	if (oldp && *oldlenp < len) {
+		len = *oldlenp;
+		rval = ENOMEM;
+	}
 	if (newp)
 		return (EPERM);
 	*oldlenp = len;
 	if (oldp)
 		error = copyout(str, oldp, len);
-	return (error);
+		if (error)
+			rval = error;
+	return (rval);
 }
 
 /*
@@ -615,17 +651,24 @@ sysctl_doproc(name, namelen, where, sizep)
 	size_t *sizep;
 {
 	register struct proc *p;
-	register struct kinfo_proc *dp = (struct kinfo_proc *)where;
-	register int needed = 0;
-	int buflen = where != NULL ? *sizep : 0;
+	register struct kinfo_proc *dp;
+	register int needed;
+	int buflen;
 	int doingzomb;
 	struct eproc eproc;
 	int error = 0;
 
 	if (namelen != 2 && !(namelen == 1 && name[0] == KERN_PROC_ALL))
 		return (EINVAL);
-	p = (struct proc *)allproc;
+restart:
 	doingzomb = 0;
+	p = (struct proc *)allproc;
+	if (where != NULL)
+		buflen = *sizep;
+	else
+		buflen = 0;
+	dp = (struct kinfo_proc *)where;
+	needed = 0;
 again:
 	for (; p != NULL; p = p->p_next) {
 		/*
@@ -670,15 +713,37 @@ again:
 			break;
 		}
 		if (buflen >= sizeof(struct kinfo_proc)) {
+			pid_t pid = p->p_pid;
+
 			fill_eproc(p, &eproc);
-			error = copyout((caddr_t)p, &dp->kp_proc, 
+			error = copyout((caddr_t)p, &dp->kp_proc,
 			    sizeof(struct proc));
 			if (error)
 				return (error);
+			/*
+			 * Since copyout can block, our cached struct proc * may
+			 * no longer be valid. For allproc, restart from the
+			 * beginning if the process no longer exists. For
+			 * zombproc, just stop if it's gone.
+			 */
+			if (!doingzomb) {
+				if (pid && (pfind(pid) != p))
+					goto restart;
+			} else {
+				if (zpfind(pid) != p)
+					break;
+			}
 			error = copyout((caddr_t)&eproc, &dp->kp_eproc,
 			    sizeof(eproc));
 			if (error)
 				return (error);
+			if (!doingzomb) {
+				if (pid && (pfind(pid) != p))
+					goto restart;
+			} else {
+				if (zpfind(pid) != p)
+					break;
+			}
 			dp++;
 			buflen -= sizeof(struct kinfo_proc);
 		}
@@ -713,11 +778,12 @@ fill_eproc(p, ep)
 	bzero(ep, sizeof(*ep));
 
 	ep->e_paddr = p;
-	if (p->p_cred)
+	if (p->p_cred) {
 		ep->e_pcred = *p->p_cred;
-	if (p->p_ucred)
-		ep->e_ucred = *p->p_ucred;
-	if (p->p_stat != SIDL && p->p_stat != SZOMB) {
+		if (p->p_ucred)
+			ep->e_ucred = *p->p_ucred;
+	}
+	if (p->p_stat != SIDL && p->p_stat != SZOMB && p->p_vmspace != NULL) {
 		register struct vmspace *vm = p->p_vmspace;
 
 #ifdef pmap_resident_count
@@ -735,9 +801,17 @@ fill_eproc(p, ep)
 	if (p->p_pptr)
 		ep->e_ppid = p->p_pptr->p_pid;
 	if (p->p_pgrp) {
-		ep->e_sess = p->p_pgrp->pg_session;
 		ep->e_pgid = p->p_pgrp->pg_id;
 		ep->e_jobc = p->p_pgrp->pg_jobc;
+		ep->e_sess = p->p_pgrp->pg_session;
+
+		if (ep->e_sess) {
+			bcopy(ep->e_sess->s_login, ep->e_login, sizeof(ep->e_login));
+			if (ep->e_sess->s_ttyvp)
+				ep->e_flag = EPROC_CTTY;
+			if (p->p_session && SESS_LEADER(p))
+				ep->e_flag |= EPROC_SLEADER;
+		}
 	}
 	if ((p->p_flag & P_CONTROLT) &&
 	    (ep->e_sess != NULL) &&
@@ -747,10 +821,6 @@ fill_eproc(p, ep)
 		ep->e_tsess = tp->t_session;
 	} else
 		ep->e_tdev = NODEV;
-	if (ep->e_sess && ep->e_sess->s_ttyvp) 
-		ep->e_flag = EPROC_CTTY;
-	if (SESS_LEADER(p))
-		ep->e_flag |= EPROC_SLEADER;
 	if (p->p_wmesg) {
 		strncpy(ep->e_wmesg, p->p_wmesg, WMESGLEN);
 		ep->e_wmesg[WMESGLEN] = 0;
@@ -767,6 +837,53 @@ fill_eproc(p, ep)
 #define	KINFO_LOADAVG		(5<<8)
 #define	KINFO_CLOCKRATE		(6<<8)
 
+/* Non-standard BSDI extension - only present on their 4.3 net-2 releases */
+#define	KINFO_BSDI_SYSINFO	(101<<8)
+
+/*
+ * XXX this is bloat, but I hope it's better here than on the potentially
+ * limited kernel stack...  -Peter
+ */
+
+struct {
+	int	bsdi_machine;		/* "i386" on BSD/386 */
+/*      ^^^ this is an offset to the string, relative to the struct start */
+	char	*pad0;
+	long	pad1;
+	long	pad2;
+	long	pad3;
+	u_long	pad4;
+	u_long	pad5;
+	u_long	pad6;
+
+	int	bsdi_ostype;		/* "BSD/386" on BSD/386 */
+	int	bsdi_osrelease;		/* "1.1" on BSD/386 */
+	long	pad7;
+	long	pad8;
+	char	*pad9;
+
+	long	pad10;
+	long	pad11;
+	int	pad12;
+	long	pad13;
+	quad_t	pad14;
+	long	pad15;
+
+	struct	timeval pad16;
+	/* we dont set this, because BSDI's uname used gethostname() instead */
+	int	bsdi_hostname;		/* hostname on BSD/386 */
+
+	/* the actual string data is appended here */
+
+} bsdi_si;
+/*
+ * this data is appended to the end of the bsdi_si structure during copyout.
+ * The "char *" offsets are relative to the base of the bsdi_si struct.
+ * This contains "FreeBSD\02.0-BUILT-nnnnnn\0i386\0", and these strings
+ * should not exceed the length of the buffer here... (or else!! :-)
+ */
+char bsdi_strings[80];	/* It had better be less than this! */
+
 struct getkerninfo_args {
 	int	op;
 	char	*where;
@@ -830,6 +947,70 @@ ogetkerninfo(p, uap, retval)
 		error = kern_sysctl(name, 1, uap->where, &size, NULL, 0, p);
 		break;
 
+	case KINFO_BSDI_SYSINFO: {
+		/*
+		 * this is pretty crude, but it's just enough for uname()
+		 * from BSDI's 1.x libc to work.
+		 *
+		 * In particular, it doesn't return the same results when
+		 * the supplied buffer is too small.  BSDI's version apparently
+		 * will return the amount copied, and set the *size to how
+		 * much was needed.  The emulation framework here isn't capable
+		 * of that, so we just set both to the amount copied.
+		 * BSDI's 2.x product apparently fails with ENOMEM in this
+		 * scenario.
+		 */
+
+		u_int needed;
+		u_int left;
+		char *s;
+
+		bzero((char *)&bsdi_si, sizeof(bsdi_si));
+		bzero(bsdi_strings, sizeof(bsdi_strings));
+
+		s = bsdi_strings;
+
+		bsdi_si.bsdi_ostype = (s - bsdi_strings) + sizeof(bsdi_si);
+		strcpy(s, ostype);
+		s += strlen(s) + 1;
+
+		bsdi_si.bsdi_osrelease = (s - bsdi_strings) + sizeof(bsdi_si);
+		strcpy(s, osrelease);
+		s += strlen(s) + 1;
+
+		bsdi_si.bsdi_machine = (s - bsdi_strings) + sizeof(bsdi_si);
+		strcpy(s, machine);
+		s += strlen(s) + 1;
+
+		needed = sizeof(bsdi_si) + (s - bsdi_strings);
+
+		if (uap->where == NULL) {
+			/* process is asking how much buffer to supply.. */
+			size = needed;
+			error = 0;
+			break;
+		}
+
+
+		/* if too much buffer supplied, trim it down */
+		if (size > needed)
+			size = needed;
+
+		/* how much of the buffer is remaining */
+		left = size;
+
+		if ((error = copyout((char *)&bsdi_si, uap->where, left)) != 0)
+			break;
+
+		/* is there any point in continuing? */
+		if (left > sizeof(bsdi_si)) {
+			left -= sizeof(bsdi_si);
+			error = copyout(&bsdi_strings,
+					uap->where + sizeof(bsdi_si), left);
+		}
+		break;
+	}
+
 	default:
 		return (EOPNOTSUPP);
 	}
