@@ -31,7 +31,7 @@
  * SUCH DAMAGE.
  *
  *	@(#)uipc_socket.c	8.3 (Berkeley) 4/15/94
- * $Id: uipc_socket.c,v 1.8 1995/02/07 02:01:14 wollman Exp $
+ * $FreeBSD: stable/2.1/sys/kern/uipc_socket.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include <sys/param.h>
@@ -48,6 +48,11 @@
 #include <sys/resourcevar.h>
 #include <sys/signalvar.h>
 
+/*
+ * Exported to userland via sysctl
+ */
+int somaxconn = SOMAXCONN;
+
 /*
  * Socket operation routines.
  * These routines are called by the routines in
@@ -125,9 +130,9 @@ solisten(so, backlog)
 	}
 	if (so->so_q == 0)
 		so->so_options |= SO_ACCEPTCONN;
-	if (backlog < 0)
-		backlog = 0;
-	so->so_qlimit = min(backlog, SOMAXCONN);
+	if (backlog < 0 || backlog > somaxconn)
+		backlog = somaxconn;
+	so->so_qlimit = backlog;
 	splx(s);
 	return (0);
 }
@@ -784,8 +789,7 @@ dontblock:
 			so->so_rcv.sb_mb = nextrecord;
 		if (pr->pr_flags & PR_WANTRCVD && so->so_pcb)
 			(*pr->pr_usrreq)(so, PRU_RCVD, (struct mbuf *)0,
-			    (struct mbuf *)flags, (struct mbuf *)0,
-			    (struct mbuf *)0);
+			    (struct mbuf *)flags, (struct mbuf *)0);
 	}
 	if (orig_resid == uio->uio_resid && orig_resid &&
 	    (flags & MSG_EOR) == 0 && (so->so_state & SS_CANTRCVMORE) == 0) {
@@ -793,7 +797,7 @@ dontblock:
 		splx(s);
 		goto restart;
 	}
-		
+
 	if (flagsp)
 		*flagsp |= flags;
 release:
