@@ -31,7 +31,7 @@
  * SUCH DAMAGE.
  *
  *	@(#)uipc_syscalls.c	8.4 (Berkeley) 2/21/94
- * $Id: uipc_syscalls.c,v 1.4 1994/10/02 17:35:35 phk Exp $
+ * $FreeBSD: stable/2.1/sys/kern/uipc_syscalls.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include <sys/param.h>
@@ -238,7 +238,7 @@ accept1(p, uap, retval)
 	return (error);
 }
 
-#ifdef COMPAT_OLDSOCK 
+#ifdef COMPAT_OLDSOCK
 int
 accept(p, uap, retval)
 	struct proc *p;
@@ -398,7 +398,7 @@ sendit(p, s, mp, flags, retsize)
 #ifdef KTRACE
 	struct iovec *ktriov = NULL;
 #endif
-	
+
 	error = getsock(p->p_fd, s, &fp);
 	if (error)
 		return (error);
@@ -564,7 +564,7 @@ osendmsg(p, uap, retval)
 		if ((u_int)msg.msg_iovlen >= UIO_MAXIOV)
 			return (EMSGSIZE);
 		MALLOC(iov, struct iovec *,
-		      sizeof(struct iovec) * (u_int)msg.msg_iovlen, M_IOV, 
+		      sizeof(struct iovec) * (u_int)msg.msg_iovlen, M_IOV,
 		      M_WAITOK);
 	} else
 		iov = aiov;
@@ -649,7 +649,7 @@ recvit(p, s, mp, namelenp, retsize)
 #ifdef KTRACE
 	struct iovec *ktriov = NULL;
 #endif
-	
+
 	error = getsock(p->p_fd, s, &fp);
 	if (error)
 		return (error);
@@ -1003,8 +1003,8 @@ getsockopt(p, uap, retval)
 	int *retval;
 {
 	struct file *fp;
-	struct mbuf *m = NULL;
-	int valsize, error;
+	struct mbuf *m = NULL, *m0;
+	int op, i, valsize, error;
 
 	error = getsock(p->p_fd, uap->s, &fp);
 	if (error)
@@ -1018,9 +1018,16 @@ getsockopt(p, uap, retval)
 		valsize = 0;
 	if ((error = sogetopt((struct socket *)fp->f_data, uap->level,
 	    uap->name, &m)) == 0 && uap->val && valsize && m != NULL) {
-		if (valsize > m->m_len)
-			valsize = m->m_len;
-		error = copyout(mtod(m, caddr_t), uap->val, (u_int)valsize);
+		op = 0;
+		while (m && !error && op < valsize) {
+			i = min(m->m_len, (valsize - op));
+			error = copyout(mtod(m, caddr_t), uap->val, (u_int)i);
+			op += i;
+			uap->val += i;
+			m0 = m;
+			MFREE(m0,m);
+		}
+		valsize = op;
 		if (error == 0)
 			error = copyout((caddr_t)&valsize,
 			    (caddr_t)uap->avalsize, sizeof (valsize));
@@ -1096,7 +1103,7 @@ struct getsockname_args {
 #endif
 };
 
-#ifndef COMPAT_OLDSOCK 
+#ifndef COMPAT_OLDSOCK
 #define	getsockname1	getsockname
 #endif
 
@@ -1178,7 +1185,7 @@ struct getpeername_args {
 };
 
 
-#ifndef COMPAT_OLDSOCK 
+#ifndef COMPAT_OLDSOCK
 #define	getpeername1	getpeername
 #endif
 
