@@ -18,7 +18,7 @@
  * 5. Modifications may be freely made to this file if the above conditions
  *    are met.
  *
- * $Id: vfs_bio.c,v 1.44 1995/05/11 19:26:29 rgrimes Exp $
+ * $FreeBSD: stable/2.1/sys/kern/vfs_bio.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -383,7 +383,7 @@ brelse(struct buf * bp)
 
 	/* anyone need this block? */
 	if (bp->b_flags & B_WANTED) {
-		bp->b_flags &= ~B_WANTED | B_AGE;
+		bp->b_flags &= ~(B_WANTED | B_AGE);
 		wakeup((caddr_t) bp);
 	} else if (bp->b_flags & B_VMIO) {
 		bp->b_flags &= ~B_WANTED;
@@ -399,7 +399,7 @@ brelse(struct buf * bp)
 		if (((bp->b_flags & B_VMIO) == 0) && bp->b_vp)
 			brelvp(bp);
 	}
-	
+
 	/*
 	 * VMIO buffer rundown.  It is not very necessary to keep a VMIO buffer
 	 * constituted, so the B_INVAL flag is used to *invalidate* the buffer,
@@ -411,72 +411,86 @@ brelse(struct buf * bp)
 		vm_object_t obj;
 		int i, resid;
 		vm_page_t m;
+		struct vnode *vp;
 		int iototal = bp->b_bufsize;
 
-		foff = 0;
-		obj = 0;
+		vp = bp->b_vp;
+		if (!vp) 
+			panic("brelse: missing vp");
+		if (!vp->v_mount)
+			panic("brelse: missing mount info");
+
 		if (bp->b_npages) {
-			if (bp->b_vp && bp->b_vp->v_mount) {
-				foff = bp->b_vp->v_mount->mnt_stat.f_iosize * bp->b_lblkno;
-			} else {
-				/*
-				 * vnode pointer has been ripped away --
-				 * probably file gone...
-				 */
-				foff = bp->b_pages[0]->offset;
-			}
-		}
-		for (i = 0; i < bp->b_npages; i++) {
-			m = bp->b_pages[i];
-			if (m == bogus_page) {
-				m = vm_page_lookup(obj, foff);
-				if (!m) {
-					panic("brelse: page missing\n");
+			obj = (vm_object_t) vp->v_vmdata;
+			foff = trunc_page(vp->v_mount->mnt_stat.f_iosize * bp->b_lblkno);
+			for (i = 0; i < bp->b_npages; i++) {
+				m = bp->b_pages[i];
+				if (m == bogus_page) {
+					m = vm_page_lookup(obj, foff);
+					if (!m) {
+						panic("brelse: page missing\n");
+					}
+					bp->b_pages[i] = m;
+					pmap_qenter(trunc_page(bp->b_data), bp->b_pages, bp->b_npages);
 				}
-				bp->b_pages[i] = m;
-				pmap_qenter(trunc_page(bp->b_data), bp->b_pages, bp->b_npages);
-			}
-			resid = (m->offset + PAGE_SIZE) - foff;
-			if (resid > iototal)
-				resid = iototal;
-			if (resid > 0) {
-				/*
-				 * Don't invalidate the page if the local machine has already
-				 * modified it.  This is the lesser of two evils, and should
-				 * be fixed.
-				 */
-				if (bp->b_flags & (B_NOCACHE | B_ERROR)) {
-					vm_page_test_dirty(m);
-					if (m->dirty == 0) {
-						vm_page_set_invalid(m, foff, resid);
-						if (m->valid == 0) 
-							vm_page_protect(m, VM_PROT_NONE);
+				resid = (m->offset + PAGE_SIZE) - foff;
+				if (resid > iototal)
+					resid = iototal;
+				if (resid > 0) {
+					/*
+					 * Don't invalidate the page if the local machine has already
+					 * modified it.  This is the lesser of two evils, and should
+					 * be fixed.
+					 */
+					if (bp->b_flags & (B_NOCACHE | B_ERROR)) {
+						vm_page_test_dirty(m);
+						if (m->dirty == 0) {
+							vm_page_set_invalid(m, foff, resid);
+							if (m->valid == 0)
+								vm_page_protect(m, VM_PROT_NONE);
+						}
 					}
 				}
+				foff += resid;
+				iototal -= resid;
 			}
-			foff += resid;
-			iototal -= resid;
 		}
 
 		if (bp->b_flags & (B_INVAL | B_RELBUF)) {
-			for(i=0;i<bp->b_npages;i++) {
+			for(i = 0; i < bp->b_npages; i++) {
 				m = bp->b_pages[i];
 				--m->bmapped;
 				if (m->bmapped == 0) {
 					if (m->flags & PG_WANTED) {
-						wakeup((caddr_t) m);
+						wakeup(m);
 						m->flags &= ~PG_WANTED;
 					}
-					vm_page_test_dirty(m);
-					if ((m->dirty & m->valid) == 0 &&
-						(m->flags & PG_REFERENCED) == 0 &&
-							!pmap_is_referenced(VM_PAGE_TO_PHYS(m))) {
-						vm_page_cache(m);
-					} else if ((m->flags & PG_ACTIVE) == 0) {
-						vm_page_activate(m);
-						m->act_count = 0;
+					if ((m->busy == 0) && ((m->flags & PG_BUSY) == 0)) {
+						vm_page_test_dirty(m);
+						/*
+						 * if page isn't valid, no sense in keeping it around
+						 */
+						if (m->valid == 0) {
+							vm_page_protect(m, VM_PROT_NONE);
+							vm_page_free(m);
+						/*
+						 * if page isn't dirty and hasn't been referenced by
+						 * a process, then cache it
+						 */
+						} else if ((m->dirty & m->valid) == 0 &&
+						    (m->flags & PG_REFERENCED) == 0 &&
+						    !pmap_is_referenced(VM_PAGE_TO_PHYS(m))) {
+							vm_page_cache(m);
+						/*
+						 * otherwise activate it
+						 */
+						} else if ((m->flags & PG_ACTIVE) == 0) {
+							vm_page_activate(m);
+							m->act_count = 0;
+						}
 					}
 				}
+				bp->b_pages[i] = 0;
 			}
 			bufspace -= bp->b_bufsize;
 			pmap_qremove(trunc_page((vm_offset_t) bp->b_data), bp->b_npages);
@@ -546,7 +560,8 @@ vfs_bio_awrite(struct buf * bp)
 
 		for (i = 1; i < MAXPHYS / size; i++) {
 			if ((bpa = incore(vp, lblkno + i)) &&
-			    ((bpa->b_flags & (B_BUSY | B_DELWRI | B_BUSY | B_CLUSTEROK | B_INVAL)) == B_DELWRI | B_CLUSTEROK) &&
+			    ((bpa->b_flags & (B_BUSY | B_DELWRI | B_CLUSTEROK | B_INVAL)) ==
+			    (B_DELWRI | B_CLUSTEROK)) &&
 			    (bpa->b_bufsize == size)) {
 				if ((bpa->b_blkno == bpa->b_lblkno) ||
 				    (bpa->b_blkno != bp->b_blkno + (i * size) / DEV_BSIZE))
@@ -594,7 +609,8 @@ start:
 	/* can we constitute a new buffer? */
 	if ((bp = bufqueues[QUEUE_EMPTY].tqh_first)) {
 		if (bp->b_qindex != QUEUE_EMPTY)
-			panic("getnewbuf: inconsistent EMPTY queue");
+			panic("getnewbuf: inconsistent EMPTY queue, qindex=%d",
+			    bp->b_qindex);
 		bremfree(bp);
 		goto fillbuf;
 	}
@@ -606,10 +622,12 @@ trytofreespace:
 	 */
 	if ((bp = bufqueues[QUEUE_AGE].tqh_first)) {
 		if (bp->b_qindex != QUEUE_AGE)
-			panic("getnewbuf: inconsistent AGE queue");
+			panic("getnewbuf: inconsistent AGE queue, qindex=%d",
+			    bp->b_qindex);
 	} else if ((bp = bufqueues[QUEUE_LRU].tqh_first)) {
 		if (bp->b_qindex != QUEUE_LRU)
-			panic("getnewbuf: inconsistent LRU queue");
+			panic("getnewbuf: inconsistent LRU queue, qindex=%d",
+			    bp->b_qindex);
 	}
 	if (!bp) {
 		/* wait for a free buffer of any kind */
@@ -644,12 +662,16 @@ trytofreespace:
 	if (bp->b_vp)
 		brelvp(bp);
 
-	/* we are not free, nor do we contain interesting data */
-	if (bp->b_rcred != NOCRED)
-		crfree(bp->b_rcred);
-	if (bp->b_wcred != NOCRED)
-		crfree(bp->b_wcred);
 fillbuf:
+	/* we are not free, nor do we contain interesting data */
+	if (bp->b_rcred != NOCRED) {
+		crfree(bp->b_rcred);
+		bp->b_rcred = NOCRED;
+	}
+	if (bp->b_wcred != NOCRED) {
+		crfree(bp->b_wcred);
+		bp->b_wcred = NOCRED;
+	}
 	bp->b_flags |= B_BUSY;
 	LIST_REMOVE(bp, b_hash);
 	LIST_INSERT_HEAD(&invalhash, bp, b_hash);
@@ -666,7 +688,6 @@ fillbuf:
 	bp->b_resid = 0;
 	bp->b_bcount = 0;
 	bp->b_npages = 0;
-	bp->b_wcred = bp->b_rcred = NOCRED;
 	bp->b_data = buffers_kva + (bp - buf) * MAXBSIZE;
 	bp->b_dirtyoff = bp->b_dirtyend = 0;
 	bp->b_validoff = bp->b_validend = 0;
@@ -825,7 +846,7 @@ loop:
 			bp->b_flags |= B_WANTED;
 			if (!tsleep((caddr_t) bp, PRIBIO | slpflag, "getblk", slptimeo))
 				goto loop;
-			
+
 			splx(s);
 			return (struct buf *) NULL;
 		}
@@ -835,12 +856,13 @@ loop:
 		 * check for size inconsistancies
 		 */
 		if (bp->b_bcount != size) {
-#if defined(VFS_BIO_DEBUG)
-			printf("getblk: invalid buffer size: %ld\n", bp->b_bcount);
-#endif
-			bp->b_flags |= B_NOCACHE;
-			(void) VOP_BWRITE(bp);
-			goto loop;
+			if (bp->b_flags & B_VMIO) {
+				allocbuf(bp, size);
+			} else {
+				bp->b_flags |= B_NOCACHE;
+				VOP_BWRITE(bp);
+				goto loop;
+			}
 		}
 		splx(s);
 		return (bp);
@@ -854,20 +876,20 @@ loop:
 			doingvmio = 0;
 		}
 		if ((bp = getnewbuf(slpflag, slptimeo, doingvmio)) == 0) {
-			if (slpflag || slptimeo)
+			if (slpflag || slptimeo) {
+				splx(s);
 				return NULL;
+			}
 			goto loop;
 		}
 
 		/*
 		 * This code is used to make sure that a buffer is not
-		 * created while the getnewbuf routine is blocked. 
+		 * created while the getnewbuf routine is blocked.
 		 * Normally the vnode is locked so this isn't a problem.
 		 * VBLK type I/O requests, however, don't lock the vnode.
-		 * VOP_ISLOCKED would be much better but is also much
-		 * slower.
 		 */
-		if ((vp->v_type == VBLK) && incore(vp, blkno)) {
+		if (!VOP_ISLOCKED(vp) && incore(vp, blkno)) {
 			bp->b_flags |= B_INVAL;
 			brelse(bp);
 			goto loop;
@@ -932,6 +954,9 @@ allocbuf(struct buf * bp, int size)
 	int newbsize, mbsize;
 	int i;
 
+	if (!(bp->b_flags & B_BUSY))
+		panic("allocbuf: buffer not busy");
+
 	if ((bp->b_flags & B_VMIO) == 0) {
 		/*
 		 * Just get anonymous memory from the kernel
@@ -939,21 +964,16 @@ allocbuf(struct buf * bp, int size)
 		mbsize = ((size + DEV_BSIZE - 1) / DEV_BSIZE) * DEV_BSIZE;
 		newbsize = round_page(size);
 
-		if (newbsize == bp->b_bufsize) {
-			bp->b_bcount = size;
-			return 1;
-		} else if (newbsize < bp->b_bufsize) {
+		if (newbsize < bp->b_bufsize) {
 			vm_hold_free_pages(
 			    bp,
 			    (vm_offset_t) bp->b_data + newbsize,
 			    (vm_offset_t) bp->b_data + bp->b_bufsize);
-			bufspace -= (bp->b_bufsize - newbsize);
 		} else if (newbsize > bp->b_bufsize) {
 			vm_hold_load_pages(
 			    bp,
 			    (vm_offset_t) bp->b_data + bp->b_bufsize,
 			    (vm_offset_t) bp->b_data + newbsize);
-			bufspace += (newbsize - bp->b_bufsize);
 		}
 	} else {
 		vm_page_t m;
@@ -962,10 +982,7 @@ allocbuf(struct buf * bp, int size)
 		newbsize = ((size + DEV_BSIZE - 1) / DEV_BSIZE) * DEV_BSIZE;
 		desiredpages = round_page(newbsize) / PAGE_SIZE;
 
-		if (newbsize == bp->b_bufsize) {
-			bp->b_bcount = size;
-			return 1;
-		} else if (newbsize < bp->b_bufsize) {
+		if (newbsize < bp->b_bufsize) {
 			if (desiredpages < bp->b_npages) {
 				pmap_qremove((vm_offset_t) trunc_page(bp->b_data) +
 				    desiredpages * PAGE_SIZE, (bp->b_npages - desiredpages));
@@ -990,9 +1007,8 @@ allocbuf(struct buf * bp, int size)
 					bp->b_pages[i] = NULL;
 				}
 				bp->b_npages = desiredpages;
-				bufspace -= (bp->b_bufsize - newbsize);
 			}
-		} else {
+		} else if (newbsize > bp->b_bufsize) {
 			vm_object_t obj;
 			vm_offset_t tinc, off, toff, objoff;
 			int pageindex, curbpnpages;
@@ -1008,8 +1024,8 @@ allocbuf(struct buf * bp, int size)
 				if (tinc > bsize)
 					tinc = bsize;
 				off = bp->b_lblkno * bsize;
-				curbpnpages = bp->b_npages;
 		doretry:
+				curbpnpages = bp->b_npages;
 				bp->b_flags |= B_CACHE;
 				for (toff = 0; toff < newbsize; toff += tinc) {
 					int mask;
@@ -1045,12 +1061,12 @@ allocbuf(struct buf * bp, int size)
 								PAGE_WAKEUP(bp->b_pages[j]);
 							}
 							VM_WAIT;
-							curbpnpages = bp->b_npages;
 							goto doretry;
 						}
 						vm_page_activate(m);
 						m->act_count = 0;
 						m->valid = 0;
+						bp->b_flags &= ~B_CACHE;
 					} else if (m->flags & PG_BUSY) {
 						int j;
 
@@ -1063,7 +1079,6 @@ allocbuf(struct buf * bp, int size)
 						tsleep(m, PRIBIO, "pgtblk", 0);
 						splx(s);
 
-						curbpnpages = bp->b_npages;
 						goto doretry;
 					} else {
 						int pb;
@@ -1087,29 +1102,19 @@ allocbuf(struct buf * bp, int size)
 					bp->b_pages[pageindex] = m;
 					curbpnpages = pageindex + 1;
 				}
-				if (bsize >= PAGE_SIZE) {
-					for (i = bp->b_npages; i < curbpnpages; i++) {
-						m = bp->b_pages[i];
-						if (m->valid == 0) {
-							bp->b_flags &= ~B_CACHE;
-						}
-						m->bmapped++;
-						PAGE_WAKEUP(m);
-					}
-				} else {
-					if (!vm_page_is_valid(bp->b_pages[0], off, bsize))
-						bp->b_flags &= ~B_CACHE;
-					bp->b_pages[0]->bmapped++;
-					PAGE_WAKEUP(bp->b_pages[0]);
+				for (i = bp->b_npages; i < curbpnpages; i++) {
+					m = bp->b_pages[i];
+					m->bmapped++;
+					PAGE_WAKEUP(m);
 				}
 				bp->b_npages = curbpnpages;
 				bp->b_data = buffers_kva + (bp - buf) * MAXBSIZE;
 				pmap_qenter((vm_offset_t) bp->b_data, bp->b_pages, bp->b_npages);
 				bp->b_data += off % PAGE_SIZE;
 			}
-			bufspace += (newbsize - bp->b_bufsize);
 		}
 	}
+	bufspace += (newbsize - bp->b_bufsize);
 	bp->b_bufsize = newbsize;
 	bp->b_bcount = size;
 	return 1;
@@ -1149,6 +1154,9 @@ biodone(register struct buf * bp)
 	int s;
 
 	s = splbio();
+	if (!(bp->b_flags & B_BUSY))
+		panic("biodone: buffer not busy");
+
 	if (bp->b_flags & B_DONE) {
 		splx(s);
 		printf("biodone: buffer already done\n");
@@ -1183,7 +1191,7 @@ biodone(register struct buf * bp)
 		foff = vp->v_mount->mnt_stat.f_iosize * bp->b_lblkno;
 		obj = (vm_object_t) vp->v_vmdata;
 		if (!obj) {
-			return;
+			panic("biodone: no object");
 		}
 #if defined(VFS_BIO_DEBUG)
 		if (obj->paging_in_progress < bp->b_npages) {
@@ -1236,16 +1244,18 @@ biodone(register struct buf * bp)
 				    "off: %ld, foff: %ld, "
 				    "resid: %d, index: %d\n",
 				    m->offset, foff, resid, i);
-				printf(" iosize: %ld, lblkno: %ld\n",
+				printf(" iosize: %ld, lblkno: %ld, flags: 0x%x, npages: %d\n",
 				    bp->b_vp->v_mount->mnt_stat.f_iosize,
-				    bp->b_lblkno);
+				    bp->b_lblkno, bp->b_flags, bp->b_npages);
 				printf(" valid: 0x%x, dirty: 0x%x, mapped: %d\n",
 				    m->valid, m->dirty, m->bmapped);
 				panic("biodone: page busy < 0\n");
 			}
 			--m->busy;
-			if( (m->busy == 0) && (m->flags & PG_WANTED))
+			if ((m->busy == 0) && (m->flags & PG_WANTED)) {
+				m->flags &= ~PG_WANTED;
 				wakeup((caddr_t) m);
+			}
 			--obj->paging_in_progress;
 			foff += resid;
 			iosize -= resid;
@@ -1314,13 +1324,13 @@ vfs_unbusy_pages(struct buf * bp)
 		vm_object_t obj = (vm_object_t) vp->v_vmdata;
 		vm_offset_t foff;
 
-		foff = vp->v_mount->mnt_stat.f_iosize * bp->b_lblkno;
+		foff = trunc_page(vp->v_mount->mnt_stat.f_iosize * bp->b_lblkno);
 
 		for (i = 0; i < bp->b_npages; i++) {
 			vm_page_t m = bp->b_pages[i];
 
 			if (m == bogus_page) {
-				m = vm_page_lookup(obj, foff);
+				m = vm_page_lookup(obj, foff + i * PAGE_SIZE);
 				if (!m) {
 					panic("vfs_unbusy_pages: page missing\n");
 				}
@@ -1329,8 +1339,10 @@ vfs_unbusy_pages(struct buf * bp)
 			}
 			--obj->paging_in_progress;
 			--m->busy;
-			if( (m->busy == 0) && (m->flags & PG_WANTED))
+			if ((m->busy == 0) && (m->flags & PG_WANTED)) {
+				m->flags &= ~PG_WANTED;
 				wakeup((caddr_t) m);
+			}
 		}
 		if (obj->paging_in_progress == 0 &&
 		    (obj->flags & OBJ_PIPWNT)) {
@@ -1367,8 +1379,8 @@ vfs_busy_pages(struct buf * bp, int clear_modify)
 				resid = iocount;
 			obj->paging_in_progress++;
 			m->busy++;
+			vm_page_protect(m, VM_PROT_NONE);
 			if (clear_modify) {
-				vm_page_protect(m, VM_PROT_READ);
 				vm_page_set_valid(m,
 					foff & (PAGE_SIZE-1), resid);
 				vm_page_set_clean(m,
@@ -1431,7 +1443,7 @@ vfs_bio_clrbuf(struct buf *bp) {
 			}
 			bp->b_resid = 0;
 			return;
-		} 
+		}
 		for(i=0;i<bp->b_npages;i++) {
 			if( bp->b_pages[i]->valid == VM_PAGE_BITS_ALL)
 				continue;
@@ -1440,7 +1452,7 @@ vfs_bio_clrbuf(struct buf *bp) {
 			} else {
 				int j;
 				for(j=0;j<PAGE_SIZE/DEV_BSIZE;j++) {
-					if( (bp->b_pages[i]->valid & (1<<j)) == 0) 
+					if( (bp->b_pages[i]->valid & (1<<j)) == 0)
 						bzero(bp->b_data + i * PAGE_SIZE + j * DEV_BSIZE, DEV_BSIZE);
 				}
 			}
@@ -1458,14 +1470,14 @@ vfs_bio_clrbuf(struct buf *bp) {
  * not associated with a file object.
  */
 void
-vm_hold_load_pages(struct buf * bp, vm_offset_t froma, vm_offset_t toa)
+vm_hold_load_pages(struct buf * bp, vm_offset_t from, vm_offset_t to)
 {
 	vm_offset_t pg;
 	vm_page_t p;
-	vm_offset_t from = round_page(froma);
-	vm_offset_t to = round_page(toa);
 
-	for (pg = from; pg < to; pg += PAGE_SIZE) {
+	to = round_page(to);
+
+	for (pg = round_page(from); pg < to; pg += PAGE_SIZE) {
 
 tryagain:
 
@@ -1477,23 +1489,26 @@ tryagain:
 		}
 		vm_page_wire(p);
 		pmap_kenter(pg, VM_PAGE_TO_PHYS(p));
-		bp->b_pages[((caddr_t) pg - bp->b_data) / PAGE_SIZE] = p;
+		bp->b_pages[(pg - trunc_page(bp->b_data)) >> PAGE_SHIFT] = p;
 		PAGE_WAKEUP(p);
 		bp->b_npages++;
 	}
 }
 
 void
-vm_hold_free_pages(struct buf * bp, vm_offset_t froma, vm_offset_t toa)
+vm_hold_free_pages(struct buf * bp, vm_offset_t from, vm_offset_t to)
 {
 	vm_offset_t pg;
 	vm_page_t p;
-	vm_offset_t from = round_page(froma);
-	vm_offset_t to = round_page(toa);
+	int index;
 
-	for (pg = from; pg < to; pg += PAGE_SIZE) {
-		p = bp->b_pages[((caddr_t) pg - bp->b_data) / PAGE_SIZE];
-		bp->b_pages[((caddr_t) pg - bp->b_data) / PAGE_SIZE] = 0;
+	from = round_page(from);
+	to = round_page(to);
+	index = (from - trunc_page(bp->b_data)) >> PAGE_SHIFT;
+
+	for (pg = from; pg < to; pg += PAGE_SIZE, index++) {
+		p = bp->b_pages[index];
+		bp->b_pages[index] = 0;
 		pmap_kremove(pg);
 		vm_page_free(p);
 		--bp->b_npages;
