@@ -33,7 +33,7 @@
  * SUCH DAMAGE.
  *
  *	@(#)vfs_cluster.c	8.7 (Berkeley) 2/13/94
- * $Id: vfs_cluster.c,v 1.14 1995/03/19 13:28:54 davidg Exp $
+ * $FreeBSD: stable/2.1/sys/kern/vfs_cluster.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include <sys/param.h>
@@ -140,7 +140,7 @@ cluster_read(vp, filesize, lblkno, size, cred, bpp)
 			if ( vp->v_maxra >= (origlblkno + vp->v_ralen))
 				return 0;
 			lblkno = vp->v_maxra;
-		} 
+		}
 		bp = NULL;
 	} else {
 		/*
@@ -422,7 +422,7 @@ cluster_write(bp, filesize)
 		if (vp->v_clen != 0) {
 			/*
 			 * Next block is not sequential.
-			 * 
+			 *
 			 * If we are not writing at end of file, the process
 			 * seeked to another point in the file since its last
 			 * write, or we have reached our maximum cluster size,
@@ -550,7 +550,8 @@ redo:
 	 * potentially pull it back up if the cluster was terminated
 	 * prematurely--too much hassle.
 	 */
-	if (tbp->b_bcount != tbp->b_bufsize) {
+	if (((tbp->b_flags & B_VMIO) == 0) ||
+		(tbp->b_bcount != tbp->b_bufsize)) {
 		relpbuf(pb);
 		++start_lbn;
 		--len;
