@@ -36,7 +36,7 @@
  * SUCH DAMAGE.
  *
  *	@(#)vfs_subr.c	8.13 (Berkeley) 4/18/94
- * $Id: vfs_subr.c,v 1.29 1995/05/12 04:24:53 davidg Exp $
+ * $FreeBSD: stable/2.1/sys/kern/vfs_subr.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -45,6 +45,7 @@
 
 #include <sys/param.h>
 #include <sys/systm.h>
+#include <sys/file.h>
 #include <sys/proc.h>
 #include <sys/mount.h>
 #include <sys/time.h>
@@ -620,11 +621,14 @@ reassignbuf(bp, newvp)
 	register struct vnode *newvp;
 {
 	register struct buflists *listheadp;
+	int s;
 
 	if (newvp == NULL) {
 		printf("reassignbuf: NULL");
 		return;
 	}
+
+	s = splbio();
 	/*
 	 * Delete from old vnode list, if on one.
 	 */
@@ -650,6 +654,7 @@ reassignbuf(bp, newvp)
 		listheadp = &newvp->v_cleanblkhd;
 		bufinsvn(bp, listheadp);
 	}
+	splx(s);
 }
 
 /*
@@ -998,7 +1003,7 @@ vclean(vp, flags)
 	 */
 	if (active) {
 		if (flags & DOCLOSE)
-			VOP_CLOSE(vp, IO_NDELAY, NOCRED, NULL);
+			VOP_CLOSE(vp, FNONBLOCK, NOCRED, NULL);
 		VOP_INACTIVE(vp);
 	}
 	/*
@@ -1313,12 +1318,16 @@ again:
 				goto again;
 			}
 			if (bp + VPTRSZ + VNODESZ > ewhere) {
+				vfs_unbusy(mp);
 				*sizep = bp - where;
 				return (ENOMEM);
 			}
-			if ((error = copyout((caddr_t) &vp, bp, VPTRSZ)) ||
-			    (error = copyout((caddr_t) vp, bp + VPTRSZ, VNODESZ)))
+			if ((error = copyout(&vp, bp, VPTRSZ)) ||
+			    (error = copyout(vp, bp + VPTRSZ, VNODESZ))) {
+				vfs_unbusy(mp);
+				*sizep = bp - where;
 				return (error);
+			}
 			bp += VPTRSZ + VNODESZ;
 		}
 		vfs_unbusy(mp);
@@ -1389,7 +1398,7 @@ vfs_hang_addrlist(mp, nep, argp)
 		saddr->sa_len = argp->ex_addrlen;
 	if (argp->ex_masklen) {
 		smask = (struct sockaddr *) ((caddr_t) saddr + argp->ex_addrlen);
-		error = copyin(argp->ex_addr, (caddr_t) smask, argp->ex_masklen);
+		error = copyin(argp->ex_mask, (caddr_t) smask, argp->ex_masklen);
 		if (error)
 			goto out;
 		if (smask->sa_len > argp->ex_masklen)
