@@ -35,7 +35,7 @@
  *
  *	@(#)fdesc_vnops.c	8.9 (Berkeley) 1/21/94
  *
- * $Id: fdesc_vnops.c,v 1.7 1994/10/06 21:06:42 davidg Exp $
+ * $FreeBSD: stable/2.1/sys/miscfs/fdesc/fdesc_vnops.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -138,7 +138,7 @@ loop:
 	/*
 	 * otherwise lock the array while we call getnewvnode
 	 * since that can block.
-	 */ 
+	 */
 	if (fdcache_lock & FDL_LOCKED) {
 		fdcache_lock |= FDL_WANT;
 		(void) tsleep((caddr_t) &fdcache_lock, PINOD, "fdalvp", 0);
@@ -146,10 +146,18 @@ loop:
 	}
 	fdcache_lock |= FDL_LOCKED;
 
+	/*
+	 * Do the MALLOC before the getnewvnode since doing so afterward
+	 * might cause a bogus v_data pointer to get dereferenced
+	 * elsewhere if MALLOC should block.
+	 */
+	MALLOC(fd, struct fdescnode *, sizeof(struct fdescnode), M_TEMP, M_WAITOK);
+
 	error = getnewvnode(VT_FDESC, mp, fdesc_vnodeop_p, vpp);
-	if (error)
+	if (error) {
+		FREE(fd, M_TEMP);
 		goto out;
-	MALLOC(fd, void *, sizeof(struct fdescnode), M_TEMP, M_WAITOK);
+	}
 	(*vpp)->v_data = fd;
 	fd->fd_vnode = *vpp;
 	fd->fd_type = ftype;
@@ -192,10 +200,16 @@ fdesc_lookup(ap)
 	struct vnode *fvp;
 	char *ln;
 
+	if (ap->a_cnp->cn_nameiop == DELETE || 
+	    ap->a_cnp->cn_nameiop == RENAME) {
+		error = EROFS;
+		goto bad;
+	}
+
 	pname = ap->a_cnp->cn_nameptr;
 	if (ap->a_cnp->cn_namelen == 1 && *pname == '.') {
 		*vpp = dvp;
-		VREF(dvp);	
+		VREF(dvp);
 		VOP_LOCK(dvp);
 		return (0);
 	}
@@ -325,7 +339,7 @@ fdesc_open(ap)
 	case Fdesc:
 		/*
 		 * XXX Kludge: set p->p_dupfd to contain the value of the
-		 * the file descriptor being sought for duplication. The error 
+		 * the file descriptor being sought for duplication. The error
 		 * return ensures that the vnode for this device will be
 		 * released by vn_open. Open will detect this special error and
 		 * take the actions in dupfdopen.  Other callers of vn_open or
@@ -467,7 +481,7 @@ fdesc_getattr(ap)
 
 	default:
 		panic("fdesc_getattr");
-		break;	
+		break;
 	}
 
 	if (error == 0)
@@ -586,7 +600,7 @@ fdesc_readdir(ap)
 				break;
 			}
 			i++;
-			
+
 			switch (dt->d_fileno) {
 			case FD_CTTY:
 				if (cttyvp(uio->uio_procp) == NULL)
@@ -690,7 +704,7 @@ fdesc_read(ap)
 		error = EOPNOTSUPP;
 		break;
 	}
-	
+
 	return (error);
 }
 
@@ -714,7 +728,7 @@ fdesc_write(ap)
 		error = EOPNOTSUPP;
 		break;
 	}
-	
+
 	return (error);
 }
 
@@ -741,7 +755,7 @@ fdesc_ioctl(ap)
 		error = EOPNOTSUPP;
 		break;
 	}
-	
+
 	return (error);
 }
 
@@ -766,7 +780,7 @@ fdesc_select(ap)
 		error = EOPNOTSUPP;
 		break;
 	}
-	
+
 	return (error);
 }
 
