@@ -35,7 +35,7 @@
  *
  *	@(#)portal_vnops.c	8.8 (Berkeley) 1/21/94
  *
- * $Id: portal_vnops.c,v 1.4 1994/09/21 03:47:05 wollman Exp $
+ * $FreeBSD: stable/2.1/sys/miscfs/portal/portal_vnops.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -109,15 +109,21 @@ portal_lookup(ap)
 		return (0);
 	}
 
-
-	error = getnewvnode(VT_PORTAL, ap->a_dvp->v_mount, portal_vnodeop_p, &fvp);
-	if (error)
-		goto bad;
-	fvp->v_type = VREG;
-	MALLOC(fvp->v_data, void *, sizeof(struct portalnode),
+	/*
+	 * Do the MALLOC before the getnewvnode since doing so afterward
+	 * might cause a bogus v_data pointer to get dereferenced
+	 * elsewhere if MALLOC should block.
+	 */
+	MALLOC(pt, struct portalnode *, sizeof(struct portalnode),
 		M_TEMP, M_WAITOK);
 
-	pt = VTOPORTAL(fvp);
+	error = getnewvnode(VT_PORTAL, ap->a_dvp->v_mount, portal_vnodeop_p, &fvp);
+	if (error) {
+		FREE(pt, M_TEMP);
+		goto bad;
+	}
+	fvp->v_type = VREG;
+	fvp->v_data = pt;
 	/*
 	 * Save all of the remaining pathname and
 	 * advance the namei next pointer to the end
@@ -272,7 +278,7 @@ portal_open(ap)
 		error = so->so_error;
 		goto bad;
 	}
-		
+
 	/*
 	 * Set miscellaneous flags
 	 */
@@ -377,7 +383,7 @@ portal_open(ap)
 	}
 
 	/*
-	 * Check that the mode the file is being opened for is a subset 
+	 * Check that the mode the file is being opened for is a subset
 	 * of the mode of the existing descriptor.
 	 */
  	fp = p->p_fd->fd_ofiles[fd];
@@ -399,7 +405,7 @@ bad:;
 	/*
 	 * And discard the control message.
 	 */
-	if (cm) { 
+	if (cm) {
 		m_freem(cm);
 	}
 
