@@ -36,7 +36,7 @@
  *
  *	@(#)procfs_vnops.c	8.6 (Berkeley) 2/7/94
  *
- *	$Id: procfs_vnops.c,v 1.12 1995/05/11 19:26:33 rgrimes Exp $
+ * $FreeBSD: stable/2.1/sys/miscfs/procfs/procfs_vnops.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -108,16 +108,21 @@ procfs_open(ap)
 	struct vop_open_args *ap;
 {
 	struct pfsnode *pfs = VTOPFS(ap->a_vp);
+	struct proc *p1 = ap->a_p, *p2 = PFIND(pfs->pfs_pid);
+
+	if (p2 == NULL)
+		return ENOENT;
 
 	switch (pfs->pfs_type) {
 	case Pmem:
-		if (PFIND(pfs->pfs_pid) == 0)
-			return (ENOENT);	/* was ESRCH, jsp */
-
 		if (((pfs->pfs_flags & FWRITE) && (ap->a_mode & O_EXCL)) ||
 			((pfs->pfs_flags & O_EXCL) && (ap->a_mode & FWRITE)))
 			return (EBUSY);
 
+		if (!CHECKIO(p1, p2) &&
+		    (p1->p_cred->pc_ucred->cr_gid != KMEM_GROUP))
+			return EPERM;
+
 
 		if (ap->a_mode & FWRITE)
 			pfs->pfs_flags = ap->a_mode & (FWRITE|O_EXCL);
@@ -164,7 +169,6 @@ int
 procfs_ioctl(ap)
 	struct vop_ioctl_args *ap;
 {
-
 	return (ENOTTY);
 }
 
@@ -311,7 +315,10 @@ procfs_getattr(ap)
 	struct proc *procp;
 	int error;
 
-	/* first check the process still exists */
+	/*
+	 * First make sure that the process and its credentials 
+	 * still exist.
+	 */
 	switch (pfs->pfs_type) {
 	case Proot:
 		procp = 0;
@@ -319,7 +326,8 @@ procfs_getattr(ap)
 
 	default:
 		procp = PFIND(pfs->pfs_pid);
-		if (procp == 0)
+		if (procp == 0 || procp->p_cred == NULL ||
+		    procp->p_ucred == NULL)
 			return (ENOENT);
 	}
 
@@ -342,6 +350,7 @@ procfs_getattr(ap)
 	 * that only root can gain access.
 	 */
 	switch (pfs->pfs_type) {
+	case Pctl:
 	case Pregs:
 	case Pfpregs:
 		if (procp->p_flag & P_SUGID)
@@ -532,6 +541,11 @@ procfs_lookup(ap)
 	pfstype pfs_type;
 	int i;
 
+	*vpp = NULL;
+
+	if (cnp->cn_nameiop == DELETE || cnp->cn_nameiop == RENAME)
+		return (EROFS);
+
 	if (cnp->cn_namelen == 1 && *pname == '.') {
 		*vpp = dvp;
 		VREF(dvp);
@@ -539,8 +553,6 @@ procfs_lookup(ap)
 		return (0);
 	}
 
-	*vpp = NULL;
-
 	pfs = VTOPFS(dvp);
 	switch (pfs->pfs_type) {
 	case Proot:
@@ -666,7 +678,7 @@ procfs_readdir(ap)
 				break;
 
 			dt = &procent[i];
-			
+
 			dp->d_reclen = UIO_MX;
 			dp->d_fileno = PROCFS_FILENO(pfs->pfs_pid, dt->d_pfstype);
 			dp->d_type = DT_REG;
