@@ -31,7 +31,7 @@
  * SUCH DAMAGE.
  *
  *	@(#)spec_vnops.c	8.6 (Berkeley) 4/9/94
- * $Id: spec_vnops.c,v 1.10 1995/02/03 06:46:21 davidg Exp $
+ * $FreeBSD: stable/2.1/sys/miscfs/specfs/spec_vnops.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include <sys/param.h>
@@ -473,6 +473,7 @@ loop:
 #ifdef DIAGNOSTIC
 		if (vp->v_dirtyblkhd.lh_first) {
 			vprint("spec_fsync: dirty", vp);
+			splx(s);
 			goto loop;
 		}
 #endif
