@@ -35,7 +35,7 @@
  * SUCH DAMAGE.
  *
  *	@(#)union_subr.c	8.4 (Berkeley) 2/17/94
- * $Id: union_subr.c,v 1.4 1994/10/06 21:06:48 davidg Exp $
+ * $FreeBSD: stable/2.1/sys/miscfs/union/union_subr.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include <sys/param.h>
@@ -130,7 +130,7 @@ union_updatevp(un, uppervp, lowervp)
 
 		LIST_REMOVE(un, un_cache);
 		union_list_unlock(ohash);
-	} else {	
+	} else {
 		while (union_list_lock(nhash))
 			continue;
 	}
@@ -372,7 +372,7 @@ loop:
 	/*
 	 * otherwise lock the vp list while we call getnewvnode
 	 * since that can block.
-	 */ 
+	 */
 	hash = UNION_HASH(uppervp, lowervp);
 
 	if (union_list_lock(hash))
@@ -716,7 +716,7 @@ union_vn_close(vp, fmode, cred, p)
 {
 	if (fmode & FWRITE)
 		--vp->v_writecount;
-	return (VOP_CLOSE(vp, fmode));
+	return (VOP_CLOSE(vp, fmode, cred, p));
 }
 
 void
