@@ -1,4 +1,4 @@
-/*	$Id: msdosfs_vnops.c,v 1.14 1995/04/11 18:32:17 ache Exp $ */
+/* $FreeBSD: stable/2.1/sys/msdosfs/msdosfs_vnops.c 50950 1999-09-05 08:39:22Z peter $ */
 /*	$NetBSD: msdosfs_vnops.c,v 1.20 1994/08/21 18:44:13 ws Exp $	*/
 
 /*-
@@ -34,17 +34,17 @@
  */
 /*
  * Written by Paul Popelka (paulp@uts.amdahl.com)
- * 
+ *
  * You can do anything you want with this software, just don't say you wrote
  * it, and don't remove this notice.
- * 
+ *
  * This software is provided "as is".
- * 
+ *
  * The author supplies this software to be publicly redistributed on the
  * understanding that the author is not responsible for the correct
  * functioning of this software in any circumstances and is not liable for
  * any damages caused by this software.
- * 
+ *
  * October 1992
  */
 
@@ -73,7 +73,7 @@
 #include <msdosfs/fat.h>
 /*
  * Some general notes:
- * 
+ *
  * In the ufs filesystem the inodes, superblocks, and indirect blocks are
  * read/written using the vnode for the filesystem. Blocks that represent
  * the contents of a file are read/written using the vnode for the file
@@ -156,7 +156,7 @@ msdosfs_mknod(ap)
 	} */ *ap;
 {
 	int error;
-	
+
 	switch (ap->a_vap->va_type) {
 	case VDIR:
 		error = msdosfs_mkdir((struct vop_mkdir_args *)ap);
@@ -204,64 +204,6 @@ msdosfs_close(ap)
 	return 0;
 }
 
-/*
- * This routine will go into sys/kern/vfs_subr.c one day!
- *
- * Do the usual access checking.
- * file_node, uid and gid are from the vnode in question,
- * while acc_mode and cred are from the VOP_ACCESS parameter list.
- */
-static int
-vaccess(file_mode, uid, gid, acc_mode, cred)
-	mode_t file_mode;
-	uid_t uid;
-	gid_t gid;
-	mode_t acc_mode;
-	struct ucred *cred;
-{
-	mode_t mask;
-	int i;
-	register gid_t *gp;
-	
-	/* User id 0 always gets access. */
-	if (cred->cr_uid == 0)
-		return 0;
-	
-	mask = 0;
-	
-	/* Otherwise, check the owner. */
-	if (cred->cr_uid == uid) {
-		if (acc_mode & VEXEC)
-			mask |= S_IXUSR;
-		if (acc_mode & VREAD)
-			mask |= S_IRUSR;
-		if (acc_mode & VWRITE)
-			mask |= S_IWUSR;
-		return (file_mode & mask) == mask ? 0 : EACCES;
-	}
-	
-	/* Otherwise, check the groups. */
-	for (i = 0, gp = cred->cr_groups; i < cred->cr_ngroups; i++, gp++)
-		if (gid == *gp) {
-			if (acc_mode & VEXEC)
-				mask |= S_IXGRP;
-			if (acc_mode & VREAD)
-				mask |= S_IRGRP;
-			if (acc_mode & VWRITE)
-				mask |= S_IWGRP;
-			return (file_mode & mask) == mask ? 0 : EACCES;
-		}
-	
-	/* Otherwise, check everyone else. */
-	if (acc_mode & VEXEC)
-		mask |= S_IXOTH;
-	if (acc_mode & VREAD)
-		mask |= S_IROTH;
-	if (acc_mode & VWRITE)
-		mask |= S_IWOTH;
-	return (file_mode & mask) == mask ? 0 : EACCES;
-}
-
 int
 msdosfs_access(ap)
 	struct vop_access_args /* {
@@ -271,15 +213,71 @@ msdosfs_access(ap)
 		struct proc *a_p;
 	} */ *ap;
 {
-	mode_t dosmode;
+	struct vnode *vp = ap->a_vp;
 	struct denode *dep = VTODE(ap->a_vp);
 	struct msdosfsmount *pmp = dep->de_pmp;
-	
-	dosmode = (S_IXUSR|S_IXGRP|S_IXOTH) | (S_IRUSR|S_IRGRP|S_IROTH) |
+	struct ucred *cred = ap->a_cred;
+	mode_t mask, file_mode, mode = ap->a_mode;
+	register gid_t *gp;
+	int i;
+
+	file_mode = (S_IXUSR|S_IXGRP|S_IXOTH) | (S_IRUSR|S_IRGRP|S_IROTH) |
 	    ((dep->de_Attributes & ATTR_READONLY) ? 0 : (S_IWUSR|S_IWGRP|S_IWOTH));
-	dosmode &= pmp->pm_mask;
-	
-	return vaccess(dosmode, pmp->pm_uid, pmp->pm_gid, ap->a_mode, ap->a_cred);
+	file_mode &= pmp->pm_mask;
+
+	/*
+	 * Disallow write attempts on read-only file systems;
+	 * unless the file is a socket, fifo, or a block or
+	 * character device resident on the file system.
+	 */
+	if (mode & VWRITE) {
+		switch (vp->v_type) {
+		case VDIR:
+		case VLNK:
+		case VREG:
+			if (vp->v_mount->mnt_flag & MNT_RDONLY)
+				return (EROFS);
+			break;
+		}
+	}
+
+	/* User id 0 always gets access. */
+	if (cred->cr_uid == 0)
+		return 0;
+
+	mask = 0;
+
+	/* Otherwise, check the owner. */
+	if (cred->cr_uid == pmp->pm_uid) {
+		if (mode & VEXEC)
+			mask |= S_IXUSR;
+		if (mode & VREAD)
+			mask |= S_IRUSR;
+		if (mode & VWRITE)
+			mask |= S_IWUSR;
+		return (file_mode & mask) == mask ? 0 : EACCES;
+	}
+
+	/* Otherwise, check the groups. */
+	for (i = 0, gp = cred->cr_groups; i < cred->cr_ngroups; i++, gp++)
+		if (pmp->pm_gid == *gp) {
+			if (mode & VEXEC)
+				mask |= S_IXGRP;
+			if (mode & VREAD)
+				mask |= S_IRGRP;
+			if (mode & VWRITE)
+				mask |= S_IWGRP;
+			return (file_mode & mask) == mask ? 0 : EACCES;
+		}
+
+	/* Otherwise, check everyone else. */
+	if (mode & VEXEC)
+		mask |= S_IXOTH;
+	if (mode & VREAD)
+		mask |= S_IROTH;
+	if (mode & VWRITE)
+		mask |= S_IWOTH;
+	return (file_mode & mask) == mask ? 0 : EACCES;
 }
 
 int
@@ -348,85 +346,27 @@ msdosfs_setattr(ap)
 		struct proc *a_p;
 	} */ *ap;
 {
-	int error = 0;
+	struct vnode *vp = ap->a_vp;
 	struct denode *dep = VTODE(ap->a_vp);
 	struct vattr *vap = ap->a_vap;
 	struct ucred *cred = ap->a_cred;
-	
-#ifdef MSDOSFS_DEBUG
-	printf("msdosfs_setattr(): vp %08x, vap %08x, cred %08x, p %08x\n",
-	       ap->a_vp, vap, cred, ap->a_p);
-#endif
-	if ((vap->va_type != VNON) ||
-	    (vap->va_nlink != VNOVAL) ||
-	    (vap->va_fsid != VNOVAL) ||
-	    (vap->va_fileid != VNOVAL) ||
-	    (vap->va_blocksize != VNOVAL) ||
-	    (vap->va_rdev != VNOVAL) ||
-	    (vap->va_bytes != VNOVAL) ||
-	    (vap->va_gen != VNOVAL)) {
-#ifdef MSDOSFS_DEBUG
-		printf("msdosfs_setattr(): returning EINVAL\n");
-		printf("    va_type %d, va_nlink %x, va_fsid %x, va_fileid %x\n",
-		       vap->va_type, vap->va_nlink, vap->va_fsid, vap->va_fileid);
-		printf("    va_blocksize %x, va_rdev %x, va_bytes %x, va_gen %x\n",
-		       vap->va_blocksize, vap->va_rdev, vap->va_bytes, vap->va_gen);
-#endif
-		return EINVAL;
-	}
-
-	if (vap->va_uid != (uid_t)VNOVAL || vap->va_gid != (uid_t)VNOVAL) {
-		if ((cred->cr_uid != dep->de_pmp->pm_uid ||
-		     vap->va_uid != dep->de_pmp->pm_uid ||
-		     (vap->va_gid != dep->de_pmp->pm_gid &&
-		      !groupmember(vap->va_gid, cred))) &&
-		    (error = suser(cred, &ap->a_p->p_acflag)))
-			return error;
-		if (vap->va_uid != dep->de_pmp->pm_uid ||
-		    vap->va_gid != dep->de_pmp->pm_gid)
-			return EINVAL;
-	}
-	if (vap->va_size != VNOVAL) {
-		if (ap->a_vp->v_type == VDIR)
-			return EISDIR;
-		error = detrunc(dep, vap->va_size, 0, cred, ap->a_p);
-		if (error)
-			return error;
-	}
-	if (vap->va_mtime.ts_sec != VNOVAL) {
-		if (cred->cr_uid != dep->de_pmp->pm_uid &&
-		    (error = suser(cred, &ap->a_p->p_acflag)) &&
-		    ((vap->va_vaflags & VA_UTIMES_NULL) == 0 || 
-		    (error = VOP_ACCESS(ap->a_vp, VWRITE, cred, &ap->a_p))))
-			return error;
-		dep->de_flag |= DE_UPDATE;
-		error = deupdat(dep, &vap->va_mtime, 1);
-		if (error)
-			return error;
-	}
+	int error = 0;
 
 	/*
-	 * DOS files only have the ability to have their writability
-	 * attribute set, so we use the owner write bit to set the readonly
-	 * attribute.
+	 * Check for unsettable attributes.
 	 */
-	if (vap->va_mode != (u_short) VNOVAL) {
-		if (cred->cr_uid != dep->de_pmp->pm_uid &&
-		    (error = suser(cred, &ap->a_p->p_acflag)))
-			return error;
-
-		/* We ignore the read and execute bits */
-		if (vap->va_mode & VWRITE)
-			dep->de_Attributes &= ~ATTR_READONLY;
-		else
-			dep->de_Attributes |= ATTR_READONLY;
-		dep->de_flag |= DE_MODIFIED;
+	if ((vap->va_type != VNON) || (vap->va_nlink != VNOVAL) ||
+	    (vap->va_fsid != VNOVAL) || (vap->va_fileid != VNOVAL) ||
+	    (vap->va_blocksize != VNOVAL) || (vap->va_rdev != VNOVAL) ||
+	    (vap->va_bytes != VNOVAL) || (vap->va_gen != VNOVAL)) {
+		return (EINVAL);
 	}
-
 	if (vap->va_flags != VNOVAL) {
+		if (vp->v_mount->mnt_flag & MNT_RDONLY)
+			return (EROFS);
 		if (cred->cr_uid != dep->de_pmp->pm_uid &&
 		    (error = suser(cred, &ap->a_p->p_acflag)))
-			return error;
+			return (error);
 		/*
 		 * We are very inconsistent about handling unsupported
 		 * attributes.  We ignored the the access time and the
@@ -451,6 +391,73 @@ msdosfs_setattr(ap)
 			dep->de_Attributes |= ATTR_ARCHIVE;
 		dep->de_flag |= DE_MODIFIED;
 	}
+
+	if (vap->va_uid != (uid_t)VNOVAL || vap->va_gid != (uid_t)VNOVAL) {
+		if (vp->v_mount->mnt_flag & MNT_RDONLY)
+			return (EROFS);
+		if ((cred->cr_uid != dep->de_pmp->pm_uid ||
+		     vap->va_uid != dep->de_pmp->pm_uid ||
+		     (vap->va_gid != dep->de_pmp->pm_gid &&
+		      !groupmember(vap->va_gid, cred))) &&
+		    (error = suser(cred, &ap->a_p->p_acflag)))
+			return error;
+		if (vap->va_uid != dep->de_pmp->pm_uid ||
+		    vap->va_gid != dep->de_pmp->pm_gid)
+			return EINVAL;
+	}
+	if (vap->va_size != VNOVAL) {
+		/*
+		 * Disallow write attempts on read-only file systems;
+		 * unless the file is a socket, fifo, or a block or
+		 * character device resident on the file system.
+		 */
+		switch (vp->v_type) {
+		case VDIR:
+			return (EISDIR);
+		case VLNK:
+		case VREG:
+			if (vp->v_mount->mnt_flag & MNT_RDONLY)
+				return (EROFS);
+			break;
+		}
+		error = detrunc(dep, vap->va_size, 0, cred, ap->a_p);
+		if (error)
+			return error;
+	}
+	if (vap->va_mtime.ts_sec != VNOVAL) {
+		if (vp->v_mount->mnt_flag & MNT_RDONLY)
+			return (EROFS);
+		if (cred->cr_uid != dep->de_pmp->pm_uid &&
+		    (error = suser(cred, &ap->a_p->p_acflag)) &&
+		    ((vap->va_vaflags & VA_UTIMES_NULL) == 0 ||
+		    (error = VOP_ACCESS(vp, VWRITE, cred, ap->a_p))))
+			return error;
+		dep->de_flag |= DE_UPDATE;
+		error = deupdat(dep, &vap->va_mtime, 1);
+		if (error)
+			return error;
+	}
+
+	/*
+	 * DOS files only have the ability to have their writability
+	 * attribute set, so we use the owner write bit to set the readonly
+	 * attribute.
+	 */
+	error = 0;
+	if (vap->va_mode != (u_short) VNOVAL) {
+		if (vp->v_mount->mnt_flag & MNT_RDONLY)
+			return (EROFS);
+		if (cred->cr_uid != dep->de_pmp->pm_uid &&
+		    (error = suser(cred, &ap->a_p->p_acflag)))
+			return error;
+
+		/* We ignore the read and execute bits */
+		if (vap->va_mode & VWRITE)
+			dep->de_Attributes &= ~ATTR_READONLY;
+		else
+			dep->de_Attributes |= ATTR_READONLY;
+		dep->de_flag |= DE_MODIFIED;
+	}
 	return error;
 }
 
@@ -574,7 +581,7 @@ msdosfs_write(ap)
 	struct msdosfsmount *pmp = dep->de_pmp;
 	struct ucred *cred = ap->a_cred;
 	struct timespec ts;
-	
+
 #ifdef MSDOSFS_DEBUG
 	printf("msdosfs_write(vp %08x, uio %08x, ioflag %08x, cred %08x\n",
 	       vp, uio, ioflag, cred);
@@ -658,7 +665,7 @@ msdosfs_write(ap)
 		lastcn = dep->de_fc[FC_LASTFC].fc_frcn;
 	} else
 		lastcn = de_clcount(pmp, osize) - 1;
-	
+
 	do {
 		bn = de_blk(pmp, uio->uio_offset);
 		if (isadir) {
@@ -669,7 +676,7 @@ msdosfs_write(ap)
 			error = ENOSPC;
 			break;
 		}
-		
+
 		if ((uio->uio_offset & pmp->pm_crbomask) == 0
 		    && (de_blk(pmp, uio->uio_offset + uio->uio_resid) > de_blk(pmp, uio->uio_offset)
 			|| uio->uio_offset + uio->uio_resid >= dep->de_FileSize)) {
@@ -687,7 +694,7 @@ msdosfs_write(ap)
 			if (!isadir) {
 				if (bp->b_blkno == bp->b_lblkno) {
 					error = pcbmap(dep, bp->b_lblkno,
-							   &bp->b_blkno, 0); 
+							   &bp->b_blkno, 0);
 					if (error)
 						bp->b_blkno = -1;
 				}
@@ -802,7 +809,7 @@ msdosfs_mmap(ap)
 
 /*
  * Flush the blocks of a file to disk.
- * 
+ *
  * This function is worthless for vnodes that represent directories. Maybe we
  * could just do a sync if they try an fsync on a directory file.
  */
@@ -917,9 +924,9 @@ msdosfs_link(ap)
  * Renames on files require moving the denode to a new hash queue since the
  * denode's location is used to compute which hash queue to put the file
  * in. Unless it is a rename in place.  For example "mv a b".
- * 
+ *
  * What follows is the basic algorithm:
- * 
+ *
  * if (file move) {
  *	if (dest file exists) {
  *		remove dest file
@@ -946,18 +953,16 @@ msdosfs_link(ap)
  *		be sure dest is not a child of src directory
  *		write entry in dest directory
  *		update "." and ".." in moved directory
- *		update offset and dirclust in denode
- *		move denode to new hash chain
  *		clear old directory entry for moved directory
  *	}
  * }
- * 
+ *
  * On entry:
  *	source's parent directory is unlocked
  *	source file or directory is unlocked
  *	destination's parent directory is locked
  *	destination file or directory is locked if it exists
- * 
+ *
  * On exit:
  *	all denodes should be released
  *
@@ -981,8 +986,6 @@ msdosfs_rename(ap)
 	int error;
 	int newparent = 0;
 	int sourceisadirectory = 0;
-	u_long to_dirclust;
-	u_long to_diroffset;
 	u_long cn;
 	daddr_t bn;
 	struct vnode *tvp = ap->a_tvp;
@@ -1034,7 +1037,8 @@ msdosfs_rename(ap)
 		if ((ap->a_fcnp->cn_namelen == 1
 		     && ap->a_fcnp->cn_nameptr[0] == '.')
 		    || fddep == fdep
-		    || (ap->a_fcnp->cn_flags & ISDOTDOT)) {
+		    || (ap->a_fcnp->cn_flags | ap->a_tcnp->cn_flags)
+		       & ISDOTDOT) {
 			VOP_ABORTOP(ap->a_tdvp, ap->a_tcnp);
 			vput(ap->a_tdvp);
 			if (tvp)
@@ -1091,25 +1095,18 @@ msdosfs_rename(ap)
 				error = ENOTEMPTY;
 				goto bad;
 			}
+			cache_purge(DETOV(tddep));
 		} else {		/* destination is file */
 			if (sourceisadirectory) {
 				error = EISDIR;
 				goto bad;
 			}
 		}
-		to_dirclust = tdep->de_dirclust;
-		to_diroffset = tdep->de_diroffset;
 		error = removede(tddep,tdep);
 		if (error)
 			goto bad;
 		vput(ap->a_tvp);
 		tdep = NULL;
-
-		/*
-		 * Remember where the slot was for createde().
-		 */
-		tddep->de_fndclust = to_dirclust;
-		tddep->de_fndoffset = to_diroffset;
 	}
 
 	/*
@@ -1181,9 +1178,11 @@ msdosfs_rename(ap)
 			VOP_UNLOCK(ap->a_fdvp);
 			goto bad;
 		}
-		fdep->de_dirclust = tddep->de_fndclust;
-		fdep->de_diroffset = tddep->de_fndoffset;
-		reinsert(fdep);
+		if (!sourceisadirectory) {
+			fdep->de_dirclust = tddep->de_fndclust;
+			fdep->de_diroffset = tddep->de_fndoffset;
+			reinsert(fdep);
+		}
 		VOP_UNLOCK(ap->a_fdvp);
 	}
 	/* fdep is still locked here */
@@ -1559,11 +1558,8 @@ msdosfs_readdir(ap)
 		on = (uio->uio_offset - bias) & pmp->pm_crbomask;
 		n = min((u_long) (pmp->pm_bpcluster - on), uio->uio_resid);
 		diff = dep->de_FileSize - (uio->uio_offset - bias);
-		if (diff <= 0) {
-			if(ap->a_eofflag)
-				*ap->a_eofflag = 1;
-			return 0;
-		}
+		if (diff <= 0)
+			break;
 		if (diff < n)
 			n = diff;
 		error = pcbmap(dep, lbn, &bn, &cn);
@@ -1646,7 +1642,7 @@ msdosfs_readdir(ap)
 				ncookies++;
 			}
 			dentp++;
-			
+
 			crnt = (struct dirent *) ((char *) crnt + sizeof(struct direntry));
 			pushout = 1;
 
@@ -1722,7 +1718,7 @@ out:	;
 			*ap->a_eofflag = 1;
 		else
 			*ap->a_eofflag = 0;
-	
+
 	return error;
 }
 
