@@ -4,7 +4,7 @@
  *
  * This code is derived from the Stanford/CMU enet packet filter,
  * (net/enet.c) distributed as part of 4.3BSD, and code contributed
- * to Berkeley by Steven McCanne and Van Jacobson both of Lawrence 
+ * to Berkeley by Steven McCanne and Van Jacobson both of Lawrence
  * Berkeley Laboratory.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -37,7 +37,7 @@
  *
  *      @(#)bpf.c	8.2 (Berkeley) 3/28/94
  *
- * $Id: bpf.c,v 1.5 1995/03/14 09:14:10 davidg Exp $
+ * $FreeBSD: stable/2.1/sys/net/bpf.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include "bpfilter.h"
@@ -52,6 +52,7 @@
 
 #include <sys/param.h>
 #include <sys/systm.h>
+#include <machine/cpu.h>	/* for bootverbose */
 #include <sys/mbuf.h>
 #include <sys/buf.h>
 #include <sys/time.h>
@@ -63,7 +64,6 @@
 #if defined(sparc) && BSD < 199103
 #include <sys/stream.h>
 #endif
-#include <sys/tty.h>
 #include <sys/uio.h>
 
 #include <sys/protosw.h>
@@ -192,10 +192,10 @@ bpf_movein(uio, linktype, mp, sockp, datlen)
 	if ((unsigned)len > MCLBYTES)
 		return (EIO);
 
-	MGET(m, M_WAIT, MT_DATA);
+	MGETHDR(m, M_WAIT, MT_DATA);
 	if (m == 0)
 		return (ENOBUFS);
-	if (len > MLEN) {
+	if (len > MHLEN) {
 #if BSD >= 199103
 		MCLGET(m, M_WAIT);
 		if ((m->m_flags & M_EXT) == 0) {
@@ -207,7 +207,8 @@ bpf_movein(uio, linktype, mp, sockp, datlen)
 			goto bad;
 		}
 	}
-	m->m_len = len;
+	m->m_pkthdr.len = m->m_len = len;
+	m->m_pkthdr.rcvif = NULL;
 	*mp = m;
 	/*
 	 * Make room for link header.
@@ -309,9 +310,11 @@ bpf_detachd(d)
  */
 /* ARGSUSED */
 int
-bpfopen(dev, flag)
+bpfopen(dev, flags, fmt, p)
 	dev_t dev;
-	int flag;
+	int flags;
+	int fmt;
+	struct proc *p;
 {
 	register struct bpf_d *d;
 
@@ -328,6 +331,7 @@ bpfopen(dev, flag)
 	/* Mark "free" and do most initialization. */
 	bzero((char *)d, sizeof(*d));
 	d->bd_bufsize = bpf_bufsize;
+	d->bd_sig = SIGIO;
 
 	return (0);
 }
@@ -338,9 +342,11 @@ bpfopen(dev, flag)
  */
 /* ARGSUSED */
 int
-bpfclose(dev, flag)
+bpfclose(dev, flags, fmt, p)
 	dev_t dev;
-	int flag;
+	int flags;
+	int fmt;
+	struct proc *p;
 {
 	register struct bpf_d *d = &bpf_dtab[minor(dev)];
 	register int s;
@@ -408,9 +414,10 @@ bpf_sleep(d)
  *  bpfread - read next chunk of packets from buffers
  */
 int
-bpfread(dev, uio)
+bpfread(dev, uio, ioflag)
 	dev_t dev;
 	register struct uio *uio;
+	int ioflag;
 {
 	register struct bpf_d *d = &bpf_dtab[minor(dev)];
 	int error;
@@ -439,8 +446,11 @@ bpfread(dev, uio)
 			ROTATE_BUFFERS(d);
 			break;
 		}
-		error = BPF_SLEEP((caddr_t)d, PRINET|PCATCH, "bpf",
-				  d->bd_rtout);
+		if (d->bd_rtout != -1)
+			error = BPF_SLEEP((caddr_t)d, PRINET|PCATCH, "bpf",
+					  d->bd_rtout);
+		else
+			error = EWOULDBLOCK; /* User requested non-blocking I/O */
 		if (error == EINTR || error == ERESTART) {
 			splx(s);
 			return (error);
@@ -496,7 +506,15 @@ static inline void
 bpf_wakeup(d)
 	register struct bpf_d *d;
 {
+	struct proc *p;
+
 	wakeup((caddr_t)d);
+	if (d->bd_async && d->bd_sig)
+		if (d->bd_pgid > 0)
+			gsignal (d->bd_pgid, d->bd_sig);
+		else if (p = pfind (-d->bd_pgid))
+			psignal (p, d->bd_sig);
+
 #if BSD >= 199103
 	selwakeup(&d->bd_sel);
 	/* XXX */
@@ -511,9 +529,10 @@ bpf_wakeup(d)
 }
 
 int
-bpfwrite(dev, uio)
+bpfwrite(dev, uio, ioflag)
 	dev_t dev;
 	struct uio *uio;
+	int ioflag;
 {
 	register struct bpf_d *d = &bpf_dtab[minor(dev)];
 	struct ifnet *ifp;
@@ -587,11 +606,12 @@ reset_d(d)
  */
 /* ARGSUSED */
 int
-bpfioctl(dev, cmd, addr, flag)
+bpfioctl(dev, cmd, addr, flags, p)
 	dev_t dev;
 	int cmd;
 	caddr_t addr;
-	int flag;
+	int flags;
+	struct proc *p;
 {
 	register struct bpf_d *d = &bpf_dtab[minor(dev)];
 	int s, error = 0;
@@ -782,6 +802,48 @@ bpfioctl(dev, cmd, addr, flag)
 			bv->bv_minor = BPF_MINOR_VERSION;
 			break;
 		}
+
+
+	case FIONBIO:		/* Non-blocking I/O */
+		if (*(int *)addr)
+			d->bd_rtout = -1;
+		else
+			d->bd_rtout = 0;
+		break;
+
+	case FIOASYNC:		/* Send signal on receive packets */
+		d->bd_async = *(int *)addr;
+		break;
+
+/* N.B.  ioctl (FIOSETOWN) and fcntl (F_SETOWN) both end up doing the
+   equivalent of a TIOCSPGRP and hence end up here.  *However* TIOCSPGRP's arg
+   is a process group if it's positive and a process id if it's negative.  This
+   is exactly the opposite of what the other two functions want!  Therefore
+   there is code in ioctl and fcntl to negate the arg before calling here. */
+
+	case TIOCSPGRP:		/* Process or group to send signals to */
+		d->bd_pgid = *(int *)addr;
+		break;
+
+	case TIOCGPGRP:
+		*(int *)addr = d->bd_pgid;
+		break;
+
+	case BIOCSRSIG:		/* Set receive signal */
+		{
+		 	u_int sig;
+
+			sig = *(u_int *)addr;
+
+			if (sig >= NSIG)
+				error = EINVAL;
+			else
+				d->bd_sig = sig;
+			break;
+		}
+	case BIOCGRSIG:
+		*(u_int *)addr = d->bd_sig;
+		break;
 	}
 	return (error);
 }
@@ -920,6 +982,7 @@ bpf_ifname(ifp, ifr)
 
 	while (*d++ = *s++)
 		continue;
+	d--; /* back to the null */
 	/* XXX Assume that unit number is less than 10. */
 	*d++ = ifp->if_unit + '0';
 	*d = '\0';
@@ -1244,7 +1307,8 @@ bpfattach(driverp, ifp, dlt, hdrlen)
 		for (i = 0; i < NBPFILTER; ++i)
 			D_MARKFREE(&bpf_dtab[i]);
 
-	printf("bpf: %s%d attached\n", ifp->if_name, ifp->if_unit);
+	if (bootverbose)
+		printf("bpf: %s%d attached\n", ifp->if_name, ifp->if_unit);
 }
 
 #if BSD >= 199103
@@ -1261,14 +1325,14 @@ ifpromisc(ifp, pswitch)
 	int pswitch;
 {
 	struct ifreq ifr;
-	/*
-	 * If the device is not configured up, we cannot put it in
-	 * promiscuous mode.
-	 */
-	if ((ifp->if_flags & IFF_UP) == 0)
-		return (ENETDOWN);
 
 	if (pswitch) {
+		/*
+		 * If the device is not configured up, we cannot put it in
+		 * promiscuous mode.
+		 */
+		if ((ifp->if_flags & IFF_UP) == 0)
+			return (ENETDOWN);
 		if (ifp->if_pcount++ != 0)
 			return (0);
 		ifp->if_flags |= IFF_PROMISC;
