@@ -31,7 +31,7 @@
  * SUCH DAMAGE.
  *
  *	@(#)if.c	8.3 (Berkeley) 1/4/94
- * $Id: if.c,v 1.13 1995/02/24 11:47:31 davidg Exp $
+ * $FreeBSD: stable/2.1/sys/net/if.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include <sys/param.h>
@@ -112,6 +112,7 @@ if_attach(ifp)
 		p = &((*p)->if_next);
 	*p = ifp;
 	ifp->if_index = ++if_index;
+	microtime(&ifp->if_lastchange);
 	if (ifnet_addrs == 0 || if_index >= if_indexlim) {
 		unsigned n = (if_indexlim <<= 1) * sizeof(ifa);
 		struct ifaddr **q = (struct ifaddr **)
@@ -203,7 +204,7 @@ ifa_ifwithdstaddr(addr)
 	register struct ifnet *ifp;
 	register struct ifaddr *ifa;
 
-	for (ifp = ifnet; ifp; ifp = ifp->if_next) 
+	for (ifp = ifnet; ifp; ifp = ifp->if_next)
 	    if (ifp->if_flags & IFF_POINTOPOINT)
 		for (ifa = ifp->if_addrlist; ifa; ifa = ifa->ifa_next) {
 			if (ifa->ifa_addr->sa_family != addr->sa_family)
@@ -237,15 +238,14 @@ ifa_ifwithnet(addr)
 		for (ifa = ifp->if_addrlist; ifa; ifa = ifa->ifa_next) {
 			register char *cp, *cp2, *cp3;
 
-			if (ifa->ifa_addr->sa_family != af || ifa->ifa_netmask == 0)
+			if (ifa->ifa_addr->sa_family != af)
 				next: continue;
-#ifdef P2P_LOCALADDR_SHARE
 			if (ifp->if_flags & IFF_POINTOPOINT) {
-				if (equal(addr, ifa->ifa_addr))
+				if (equal(addr, ifa->ifa_dstaddr))
  					return (ifa);
-			} else
-#endif /* P2P_LOCALADDR_SHARE */
-			{
+			} else {
+				if (ifa->ifa_netmask == 0)
+					continue;
 				cp = addr_data;
 				cp2 = ifa->ifa_addr->sa_data;
 				cp3 = ifa->ifa_netmask->sa_data;
@@ -307,13 +307,10 @@ ifaof_ifpforaddr(addr, ifp)
 				return (ifa);
 			continue;
 		}
-#ifdef P2P_LOCALADDR_SHARE
 		if (ifp->if_flags & IFF_POINTOPOINT) {
 			if (equal(addr, ifa->ifa_dstaddr))
 				return (ifa);
-		} else
-#endif /* P2P_LOCALADDR_SHARE */
-		{
+		} else {
 			cp = addr->sa_data;
 			cp2 = ifa->ifa_addr->sa_data;
 			cp3 = ifa->ifa_netmask->sa_data;
@@ -370,6 +367,7 @@ if_down(ifp)
 	register struct ifaddr *ifa;
 
 	ifp->if_flags &= ~IFF_UP;
+	microtime(&ifp->if_lastchange);
 	for (ifa = ifp->if_addrlist; ifa; ifa = ifa->ifa_next)
 		pfctlinput(PRC_IFDOWN, ifa->ifa_addr);
 	if_qflush(&ifp->if_snd);
@@ -387,6 +385,7 @@ if_up(ifp)
 {
 
 	ifp->if_flags |= IFF_UP;
+	microtime(&ifp->if_lastchange);
 #ifdef notyet
 	register struct ifaddr *ifa;
 	/* this has no effect on IP, and will kill all iso connections XXX */
@@ -537,6 +536,7 @@ ifioctl(so, cmd, data, p)
 			(ifr->ifr_flags &~ IFF_CANTCHANGE);
 		if (ifp->if_ioctl)
 			(void) (*ifp->if_ioctl)(ifp, cmd, data);
+		microtime(&ifp->if_lastchange);
 		break;
 
 	case SIOCSIFMETRIC:
@@ -544,14 +544,19 @@ ifioctl(so, cmd, data, p)
 		if (error)
 			return (error);
 		ifp->if_metric = ifr->ifr_metric;
+		microtime(&ifp->if_lastchange);
 		break;
 
 	case SIOCSIFPHYS:
 		error = suser(p->p_ucred, &p->p_acflag);
-		if (error) return error;
-
-		if (!ifp->if_ioctl) return EOPNOTSUPP;
-		return ifp->if_ioctl(ifp, cmd, data);
+		if (error)
+		        return error;
+		if (!ifp->if_ioctl)
+		        return EOPNOTSUPP;
+		error = (*ifp->if_ioctl)(ifp, cmd, data);
+		if (error == 0)
+			microtime(&ifp->if_lastchange);
+		return(error);
 
 	case SIOCSIFMTU:
 		error = suser(p->p_ucred, &p->p_acflag);
@@ -559,13 +564,16 @@ ifioctl(so, cmd, data, p)
 			return (error);
 		if (ifp->if_ioctl == NULL)
 			return (EOPNOTSUPP);
-		/* 
+		/*
 		 * 72 was chosen below because it is the size of a TCP/IP
 		 * header (40) + the minimum mss (32).
 		 */
 		if (ifr->ifr_mtu < 72 || ifr->ifr_mtu > 65535)
 			return (EINVAL);
-		return ((*ifp->if_ioctl)(ifp, cmd, data));
+		error = (*ifp->if_ioctl)(ifp, cmd, data);
+		if (error == 0)
+			microtime(&ifp->if_lastchange);
+		return(error);
 
 	case SIOCADDMULTI:
 	case SIOCDELMULTI:
@@ -574,7 +582,10 @@ ifioctl(so, cmd, data, p)
 			return (error);
 		if (ifp->if_ioctl == NULL)
 			return (EOPNOTSUPP);
-		return ((*ifp->if_ioctl)(ifp, cmd, data));
+		error = (*ifp->if_ioctl)(ifp, cmd, data);
+		if (error == 0 )
+		    	microtime(&ifp->if_lastchange);
+		return(error);
 
 	default:
 		if (so->so_proto == 0)
@@ -659,7 +670,7 @@ ifconf(cmd, data)
 	for (; space > sizeof (ifr) && ifp; ifp = ifp->if_next) {
 		char workbuf[12], *unitname;
 		int unitlen, ifnlen;
-	       
+
 		unitname = sprint_d(ifp->if_unit, workbuf, sizeof workbuf);
 		unitlen = strlen(unitname);
 		ifnlen = strlen(ifp->if_name);
@@ -677,7 +688,7 @@ ifconf(cmd, data)
 			if (error)
 				break;
 			space -= sizeof (ifr), ifrp++;
-		} else 
+		} else
 		    for ( ; space > sizeof (ifr) && ifa; ifa = ifa->ifa_next) {
 			register struct sockaddr *sa = ifa->ifa_addr;
 #ifdef COMPAT_43
