@@ -31,7 +31,7 @@
  * SUCH DAMAGE.
  *
  *	@(#)if_ethersubr.c	8.1 (Berkeley) 6/10/93
- * $Id: if_ethersubr.c,v 1.6 1995/03/16 18:14:25 bde Exp $
+ * $FreeBSD: stable/2.1/sys/net/if_ethersubr.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include <sys/param.h>
@@ -110,12 +110,11 @@ ether_output(ifp, m0, dst, rt0)
 
 	if ((ifp->if_flags & (IFF_UP|IFF_RUNNING)) != (IFF_UP|IFF_RUNNING))
 		senderr(ENETDOWN);
-	ifp->if_lastchange = time;
 	if (rt = rt0) {
 		if ((rt->rt_flags & RTF_UP) == 0) {
 			if (rt0 = rt = rtalloc1(dst, 1, 0UL))
 				rt->rt_refcnt--;
-			else 
+			else
 				senderr(EHOSTUNREACH);
 		}
 		if (rt->rt_flags & RTF_GATEWAY) {
@@ -123,7 +122,7 @@ ether_output(ifp, m0, dst, rt0)
 				goto lookup;
 			if (((rt = rt->rt_gwroute)->rt_flags & RTF_UP) == 0) {
 				rtfree(rt); rt = rt0;
-			lookup: rt->rt_gwroute = rtalloc1(rt->rt_gateway, 1, 
+			lookup: rt->rt_gwroute = rtalloc1(rt->rt_gateway, 1,
 							  0UL);
 				if ((rt = rt->rt_gwroute) == 0)
 					senderr(EHOSTUNREACH);
@@ -206,7 +205,7 @@ ether_output(ifp, m0, dst, rt0)
 #ifdef	LLC
 /*	case AF_NSAP: */
 	case AF_CCITT: {
-		register struct sockaddr_dl *sdl = 
+		register struct sockaddr_dl *sdl =
 			(struct sockaddr_dl *) rt -> rt_gateway;
 
 		if (sdl && sdl->sdl_family == AF_LINK
@@ -234,14 +233,14 @@ ether_output(ifp, m0, dst, rt0)
 			printf("ether_output: sending LLC2 pkt to: ");
 			for (i=0; i<6; i++)
 				printf("%x ", edst[i] & 0xff);
-			printf(" len 0x%x dsap 0x%x ssap 0x%x control 0x%x\n", 
+			printf(" len 0x%x dsap 0x%x ssap 0x%x control 0x%x\n",
 			       type & 0xff, l->llc_dsap & 0xff, l->llc_ssap &0xff,
 			       l->llc_control & 0xff);
 
 		}
 #endif /* LLC_DEBUG */
 		} break;
-#endif /* LLC */	
+#endif /* LLC */
 
 	case AF_UNSPEC:
 		eh = (struct ether_header *)dst->sa_data;
@@ -318,7 +317,6 @@ ether_input(ifp, eh, m)
 		m_freem(m);
 		return;
 	}
-	ifp->if_lastchange = time;
 	ifp->if_ibytes += m->m_pkthdr.len + sizeof (*eh);
 	if (bcmp((caddr_t)etherbroadcastaddr, (caddr_t)eh->ether_dhost,
 	    sizeof(etherbroadcastaddr)) == 0)
@@ -356,7 +354,7 @@ ether_input(ifp, eh, m)
 		l = mtod(m, struct llc *);
 		switch (l->llc_dsap) {
 #ifdef	ISO
-		case LLC_ISO_LSAP: 
+		case LLC_ISO_LSAP:
 			switch (l->llc_control) {
 			case LLC_UI:
 				/* LLC_UI_P forbidden in class 1 service */
@@ -380,7 +378,7 @@ ether_input(ifp, eh, m)
 					break;
 				}
 				goto dropanyway;
-				
+
 			case LLC_XID:
 			case LLC_XID_P:
 				if(m->m_len < 6)
@@ -408,7 +406,7 @@ ether_input(ifp, eh, m)
 				eh2 = (struct ether_header *)sa.sa_data;
 				for (i = 0; i < 6; i++) {
 					eh2->ether_shost[i] = c = eh->ether_dhost[i];
-					eh2->ether_dhost[i] = 
+					eh2->ether_dhost[i] =
 						eh->ether_dhost[i] = eh->ether_shost[i];
 					eh->ether_shost[i] = c;
 				}
@@ -430,7 +428,7 @@ ether_input(ifp, eh, m)
 			if (m == 0)
 				return;
 			if ( !sdl_sethdrif(ifp, eh->ether_shost, LLC_X25_LSAP,
-					    eh->ether_dhost, LLC_X25_LSAP, 6, 
+					    eh->ether_dhost, LLC_X25_LSAP, 6,
 					    mtod(m, struct sdl_hdr *)))
 				panic("ETHER cons addr failure");
 			mtod(m, struct sdl_hdr *)->sdlhdr_len = ether_type;
@@ -497,6 +495,8 @@ ether_ifattach(ifp)
 	ifp->if_addrlen = 6;
 	ifp->if_hdrlen = 14;
 	ifp->if_mtu = ETHERMTU;
+	if (ifp->if_baudrate == 0)
+	    ifp->if_baudrate = 10000000;
 	for (ifa = ifp->if_addrlist; ifa; ifa = ifa->ifa_next)
 		if ((sdl = (struct sockaddr_dl *)ifa->ifa_addr) &&
 		    sdl->sdl_family == AF_LINK) {
@@ -523,6 +523,7 @@ ether_addmulti(ifr, ac)
 	struct sockaddr_in *sin;
 	u_char addrlo[6];
 	u_char addrhi[6];
+        int set_allmulti = 0;
 	int s = splimp();
 
 	switch (ifr->ifr_addr.sa_family) {
@@ -543,6 +544,7 @@ ether_addmulti(ifr, ac)
 			 */
 			bcopy(ether_ipmulticast_min, addrlo, 6);
 			bcopy(ether_ipmulticast_max, addrhi, 6);
+                      set_allmulti = 1;
 		}
 		else {
 			ETHER_MAP_IP_MULTICAST(&sin->sin_addr, addrlo);
@@ -592,6 +594,9 @@ ether_addmulti(ifr, ac)
 	ac->ac_multiaddrs = enm;
 	ac->ac_multicnt++;
 	splx(s);
+        if (set_allmulti)
+        	ac->ac_if.if_flags |= IFF_ALLMULTI;
+
 	/*
 	 * Return ENETRESET to inform the driver that the list has changed
 	 * and its reception filter should be adjusted accordingly.
@@ -612,6 +617,7 @@ ether_delmulti(ifr, ac)
 	struct sockaddr_in *sin;
 	u_char addrlo[6];
 	u_char addrhi[6];
+      int unset_allmulti = 0;
 	int s = splimp();
 
 	switch (ifr->ifr_addr.sa_family) {
@@ -632,6 +638,7 @@ ether_delmulti(ifr, ac)
 			 */
 			bcopy(ether_ipmulticast_min, addrlo, 6);
 			bcopy(ether_ipmulticast_max, addrhi, 6);
+                      unset_allmulti = 1;
 		}
 		else {
 			ETHER_MAP_IP_MULTICAST(&sin->sin_addr, addrlo);
@@ -671,6 +678,9 @@ ether_delmulti(ifr, ac)
 	free(enm, M_IFMADDR);
 	ac->ac_multicnt--;
 	splx(s);
+      if (unset_allmulti)
+              ac->ac_if.if_flags &= ~IFF_ALLMULTI;
+
 	/*
 	 * Return ENETRESET to inform the driver that the list has changed
 	 * and its reception filter should be adjusted accordingly.
