@@ -69,7 +69,7 @@
  * Paul Mackerras (paulus@cs.anu.edu.au).
  */
 
-/* $Id: if_ppp.c,v 1.11 1995/03/20 19:20:42 wollman Exp $ */
+/* $FreeBSD: stable/2.1/sys/net/if_ppp.c 50950 1999-09-05 08:39:22Z peter $ */
 /* from if_sl.c,v 1.11 84/10/04 12:54:47 rick Exp */
 
 #include "ppp.h"
@@ -146,9 +146,14 @@ int	pppstart __P((struct tty *tp));
 
 static struct linesw pppdisc = {
 	pppopen, pppclose, pppread, pppwrite, ppptioctl,
-	pppinput, pppstart, nullmodem
+	pppinput, pppstart, ttymodem
 };
 
+extern struct	ppp_softc *pppalloc __P((pid_t pid));
+extern void	pppdealloc __P((struct ppp_softc *sc));
+extern struct	mbuf *ppp_dequeue __P((struct ppp_softc *sc));
+extern int	ppppktin __P((struct ppp_softc *sc, struct mbuf *m, int ilen));
+
 static int	pppasyncstart __P((struct ppp_softc *));
 static u_short	pppfcs __P((u_short fcs, u_char *cp, int len));
 static int	pppgetm __P((struct ppp_softc *sc));
@@ -200,7 +205,7 @@ pppattach()
 	sc->sc_if.if_name = "ppp";
 	sc->sc_if.if_unit = i++;
 	sc->sc_if.if_mtu = PPP_MTU;
-	sc->sc_if.if_flags = IFF_POINTOPOINT;
+	sc->sc_if.if_flags = IFF_POINTOPOINT | IFF_MULTICAST;
 	sc->sc_if.if_type = IFT_PPP;
 	sc->sc_if.if_hdrlen = PPP_HDRLEN;
 	sc->sc_if.if_ioctl = pppioctl;
@@ -243,6 +248,7 @@ pppalloc(pid)
     sl_compress_init(&sc->sc_comp);
 #endif
     sc->sc_if.if_flags |= IFF_RUNNING;
+    microtime(&sc->sc_if.if_lastchange);
 
     return sc;
 }
@@ -272,8 +278,9 @@ pppdealloc(sc)
 	m_freem(m);
     }
     sc->sc_if.if_flags &= ~(IFF_UP|IFF_RUNNING);
+    microtime(&sc->sc_if.if_lastchange);
 }
-    
+
 /*
  * Line specific open routine for async tty devices.
  * Attach the given tty to the first available ppp unit.
@@ -304,9 +311,10 @@ pppopen(dev, tp)
 	m_freem(sc->sc_outm);
 	sc->sc_outm = NULL;
     }
-	
+
     if (pppgetm(sc) == 0) {
 	sc->sc_if.if_flags &= ~(IFF_UP|IFF_RUNNING);
+	microtime(&sc->sc_if.if_lastchange);
 	return (ENOBUFS);
     }
 
@@ -347,9 +355,9 @@ pppclose(tp, flag)
     struct mbuf *m;
     int s;
 
-    if (ttywflush(tp))
-	ttyflush(tp, FREAD | FWRITE);
+    ttyflush(tp, FREAD | FWRITE);
     s = splimp();		/* paranoid; splnet probably ok */
+    clist_free_cblocks(&tp->t_canq);
     clist_free_cblocks(&tp->t_outq);
     tp->t_line = 0;
     sc = (struct ppp_softc *)tp->t_sc;
@@ -381,7 +389,7 @@ pppread(tp, uio, flag)
     register int s;
     int error = 0;
 
-    if ((tp->t_state & TS_CARR_ON) == 0 && (tp->t_cflag & CLOCAL) == 0)
+    if ((tp->t_state & TS_CONNECTED) == 0)
 	return 0;		/* end of file */
     if (sc == NULL || tp != (struct tty *) sc->sc_devp)
 	return 0;
@@ -391,7 +399,7 @@ pppread(tp, uio, flag)
 	    splx(s);
 	    return (EWOULDBLOCK);
 	}
-	error = ttysleep(tp, (caddr_t)&tp->t_rawq, TTIPRI|PCATCH, ttyin, 0);
+	error = ttysleep(tp, TSA_HUP_OR_INPUT(tp), TTIPRI | PCATCH, "pppin", 0);
 	if (error)
 	    return error;
     }
@@ -429,7 +437,7 @@ pppwrite(tp, uio, flag)
     struct ppp_header *ph1, *ph2;
     int len, error;
 
-    if ((tp->t_state & TS_CARR_ON) == 0 && (tp->t_cflag & CLOCAL) == 0)
+    if ((tp->t_state & TS_CONNECTED) == 0)
 	return 0;		/* wrote 0 bytes */
     if (tp->t_line != PPPDISC)
 	return (EINVAL);
@@ -550,7 +558,10 @@ ppptioctl(tp, cmd, data, flag, p)
 		error = ENOBUFS;
 		sc->sc_mru = PPP_MRU;
 		if (pppgetm(sc) == 0)
+		{
 		    sc->sc_if.if_flags &= ~IFF_UP;
+		    microtime(&sc->sc_if.if_lastchange);
+		}
 	    }
 	}
 	break;
@@ -670,7 +681,7 @@ pppoutput(ifp, m0, dst, rt)
 	    error = ENETDOWN;
 	    goto bad;
 	}
-	
+
 	/*
 	 * If this is a TCP packet to or from an "interactive" port,
 	 * put the packet on the fastq instead.
@@ -680,6 +691,15 @@ pppoutput(ifp, m0, dst, rt)
 	    if (INTERACTIVE(p & 0xffff) || INTERACTIVE(p >> 16))
 		ifq = &sc->sc_fastq;
 	}
+
+      /*
+       * Most high-priority services set IPTOS_LOWDELAY.  This is a much more
+       * generic mechanism than specific port numbers.  This same mechanism
+       * is used by slip.
+       */
+      if (ip->ip_tos & IPTOS_LOWDELAY)
+          ifq = &sc->sc_fastq;
+
 	break;
 #endif
 #ifdef NS
@@ -882,7 +902,7 @@ pppstart(tp)
     int n, s, ndone, done;
     struct mbuf *m2;
 
-    if ((tp->t_state & TS_CARR_ON) == 0 && (tp->t_cflag & CLOCAL) == 0) {
+    if ((tp->t_state & TS_CONNECTED) == 0) {
 	/* sorry, I can't talk now */
 	return 0;
     }
@@ -893,11 +913,11 @@ pppstart(tp)
 
     for (;;) {
 	/*
-	 * If there is more in the output queue, just send it now.
+	 * Call output process whether or not there is any output.
 	 * We are being called in lieu of ttstart and must do what
 	 * it would.
 	 */
-	if (CCOUNT(&tp->t_outq) != 0 && tp->t_oproc != NULL) {
+	if (tp->t_oproc != NULL) {
 	    (*tp->t_oproc)(tp);
 	    if (CCOUNT(&tp->t_outq) > PPP_HIWAT)
 		return 0;
@@ -1123,7 +1143,10 @@ ppp_btom(sc)
     if (pppgetm(sc) == 0) {
 	m_freem(top);
 	if (pppgetm(sc) == 0)
+	{
 	    sc->sc_if.if_flags &= ~IFF_UP;
+	    microtime(&sc->sc_if.if_lastchange);
+	}
 	return (NULL);
     }
 
@@ -1284,7 +1307,7 @@ pppinput(c, tp)
 
     ++sc->sc_bytesrcvd;
 
-    if (!(tp->t_state & TS_CARR_ON) && !(tp->t_cflag & CLOCAL)) {
+    if ((tp->t_state & TS_CONNECTED) == 0) {
 	if (sc->sc_flags & SC_DEBUG)
 	    printf("ppp%d: no carrier\n", sc->sc_if.if_unit);
 	goto flush;
@@ -1314,7 +1337,7 @@ pppinput(c, tp)
 	sc->sc_ilen = 0;
 	sc->sc_if.if_ibytes = sc->sc_bytesrcvd;
 
-	if (sc->sc_rawin_count > 0) 
+	if (sc->sc_rawin_count > 0)
 	    ppplogchar(sc, -1);
 
 	/*
@@ -1526,6 +1549,18 @@ pppioctl(ifp, cmd, data)
     case SIOCGIFMTU:
 	ifr->ifr_mtu = sc->sc_if.if_mtu;
 	break;
+    case SIOCADDMULTI:
+    case SIOCDELMULTI:
+	switch(ifr->ifr_addr.sa_family) {
+#ifdef INET
+	case AF_INET:
+		break;
+#endif
+	default:
+		error = EAFNOSUPPORT;
+		break;
+	}
+	break;
 
     default:
 	error = EINVAL;
