@@ -31,7 +31,7 @@
  * SUCH DAMAGE.
  *
  *	@(#)if_sl.c	8.6 (Berkeley) 2/1/94
- * $Id: if_sl.c,v 1.18 1995/04/02 00:11:48 bde Exp $
+ * $FreeBSD: stable/2.1/sys/net/if_sl.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -60,7 +60,7 @@
  * pinging you can use up all your bandwidth).  Made low clist behavior
  * more robust and slightly less likely to hang serial line.
  * Sped up a bunch of things.
- * 
+ *
  * Note that splimp() is used throughout to block both (tty) input
  * interrupts and network activity; thus, splimp must be >= spltty.
  */
@@ -190,7 +190,7 @@ static timeout_t sl_outfill;
 
 static struct linesw slipdisc =
 	{ slopen, slclose, ttyerrio, ttyerrio, sltioctl,
-	  slinput, slstart, nullmodem };
+	  slinput, slstart, ttymodem };
 
 /*
  * Called from boot code to establish sl interfaces.
@@ -236,7 +236,6 @@ slinit(sc)
 			sc->sc_ep = (u_char *)p + SLBUFSIZE;
 		else {
 			printf("sl%d: can't allocate buffer\n", sc - sl_softc);
-			sc->sc_if.if_flags &= ~IFF_UP;
 			return (0);
 		}
 	}
@@ -259,7 +258,7 @@ slopen(dev, tp)
 	struct proc *p = curproc;		/* XXX */
 	register struct sl_softc *sc;
 	register int nsl;
-	int error;
+	int s, error;
 
 	error = suser(p->p_ucred, &p->p_acflag);
 	if (error)
@@ -290,10 +289,13 @@ slopen(dev, tp)
 			 */
 			clist_alloc_cblocks(&tp->t_canq, 0, 0);
 			clist_alloc_cblocks(&tp->t_outq,
-					    sc->sc_if.if_mtu + SLIP_HIWAT,
-					    sc->sc_if.if_mtu + SLIP_HIWAT);
+			    SLIP_HIWAT + 2 * sc->sc_if.if_mtu + 1,
+			    SLIP_HIWAT + 2 * sc->sc_if.if_mtu + 1);
 			clist_alloc_cblocks(&tp->t_rawq, 0, 0);
 
+			s = splnet();
+			if_up(&sc->sc_if);
+			splx(s);
 			return (0);
 		}
 	return (ENXIO);
@@ -311,8 +313,7 @@ slclose(tp,flag)
 	register struct sl_softc *sc;
 	int s;
 
-	if (ttywflush(tp))
-		ttyflush(tp, FREAD | FWRITE);
+	ttyflush(tp, FREAD | FWRITE);
 	/*
 	 * XXX the placement of the following spl is misleading.  tty
 	 * interrupts must be blocked across line discipline switches
@@ -438,12 +439,11 @@ sloutput(ifp, m, dst, rtp)
 		return (EAFNOSUPPORT);
 	}
 
-	if (sc->sc_ttyp == NULL) {
+	if (sc->sc_ttyp == NULL || !(ifp->if_flags & IFF_UP)) {
 		m_freem(m);
-		return (ENETDOWN);	/* sort of */
+		return (ENETDOWN);
 	}
-	if ((sc->sc_ttyp->t_state & TS_CARR_ON) == 0 &&
-	    (sc->sc_ttyp->t_cflag & CLOCAL) == 0) {
+	if ((sc->sc_ttyp->t_state & TS_CONNECTED) == 0) {
 		m_freem(m);
 		return (EHOSTUNREACH);
 	}
@@ -464,7 +464,6 @@ sloutput(ifp, m, dst, rtp)
 		return (ENOBUFS);
 	}
 	IF_ENQUEUE(ifq, m);
-	sc->sc_if.if_lastchange = time;
 	if (sc->sc_ttyp->t_outq.c_cc == 0)
 		slstart(sc->sc_ttyp);
 	splx(s);
@@ -493,17 +492,19 @@ slstart(tp)
 
 	for (;;) {
 		/*
-		 * If there is more in the output queue, just send it now.
-		 * We are being called in lieu of ttstart and must do what
-		 * it would.
+		 * Call output process whether or not there is more in the
+		 * output queue.  We are being called in lieu of ttstart
+		 * and must do what it would.
 		 */
+		(*tp->t_oproc)(tp);
+
 		if (tp->t_outq.c_cc != 0) {
 			if (sc != NULL)
 				sc->sc_flags &= ~SC_OUTWAIT;
-			(*tp->t_oproc)(tp);
 			if (tp->t_outq.c_cc > SLIP_HIWAT)
 				return 0;
 		}
+
 		/*
 		 * This happens briefly when the line shuts down.
 		 */
@@ -522,7 +523,6 @@ slstart(tp)
 		splx(s);
 		if (m == NULL)
 			return 0;
-		sc->sc_flags &= ~SC_OUTWAIT;
 
 		/*
 		 * We do the header compression here rather than in sloutput
@@ -570,7 +570,6 @@ slstart(tp)
 			bpf_tap(sc->sc_bpf, bpfbuf, len + SLIP_HDRLEN);
 		}
 #endif
-		sc->sc_if.if_lastchange = time;
 
 		/*
 		 * If system is getting low on clists, just flush our
@@ -586,6 +585,7 @@ slstart(tp)
 			continue;
 		}
 
+		sc->sc_flags &= ~SC_OUTWAIT;
 		/*
 		 * The extra FRAME_END will start up a new packet, and thus
 		 * will flush any accumulated garbage.  We do this whenever
@@ -729,8 +729,7 @@ slinput(c, tp)
 	sc = (struct sl_softc *)tp->t_sc;
 	if (sc == NULL)
 		return 0;
-	if (c & TTY_ERRORMASK || ((tp->t_state & TS_CARR_ON) == 0 &&
-	    (tp->t_cflag & CLOCAL) == 0)) {
+	if (c & TTY_ERRORMASK || (tp->t_state & TS_CONNECTED) == 0) {
 		sc->sc_flags |= SC_ERROR;
 		return 0;
 	}
@@ -852,7 +851,12 @@ slinput(c, tp)
 			goto error;
 
 		sc->sc_if.if_ipackets++;
-		sc->sc_if.if_lastchange = time;
+
+		if ((sc->sc_if.if_flags & IFF_UP) == 0) {
+			m_freem(m);
+			goto newpack;
+		}
+
 		s = splimp();
 		if (IF_QFULL(&ipintrq)) {
 			IF_DROP(&ipintrq);
@@ -900,11 +904,26 @@ slioctl(ifp, cmd, data)
 
 	switch (cmd) {
 
+	case SIOCSIFFLAGS:
+		/*
+		 * if.c will set the interface up even if we
+		 * don't want it to.
+		 */
+		if (sl_softc[ifp->if_unit].sc_ttyp == NULL) {
+			ifp->if_flags &= ~IFF_UP;
+		}
+		break;
 	case SIOCSIFADDR:
-		if (ifa->ifa_addr->sa_family == AF_INET)
-			ifp->if_flags |= IFF_UP;
-		else
+		/*
+		 * This is "historical" - set the interface up when
+		 * setting the address.
+		 */
+		if (ifa->ifa_addr->sa_family == AF_INET) {
+			if (sl_softc[ifp->if_unit].sc_ttyp != NULL)
+				if_up(ifp);
+		} else {
 			error = EAFNOSUPPORT;
+		}
 		break;
 
 	case SIOCSIFDSTADDR:
@@ -944,8 +963,8 @@ slioctl(ifp, cmd, data)
 			tp = sl_softc[ifp->if_unit].sc_ttyp;
 			if (tp != NULL)
 				clist_alloc_cblocks(&tp->t_outq,
-						    ifp->if_mtu + SLIP_HIWAT,
-						    ifp->if_mtu + SLIP_HIWAT);
+				    SLIP_HIWAT + 2 * ifp->if_mtu + 1,
+				    SLIP_HIWAT + 2 * ifp->if_mtu + 1);
 		}
 		break;
 
