@@ -31,7 +31,7 @@
  * SUCH DAMAGE.
  *
  *	@(#)route.c	8.2 (Berkeley) 11/15/93
- *	$Id: route.c,v 1.20 1995/03/23 18:07:29 wollman Exp $
+ * $FreeBSD: stable/2.1/sys/net/route.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include <sys/param.h>
@@ -146,7 +146,7 @@ void
 rtfree(rt)
 	register struct rtentry *rt;
 {
-	register struct radix_node_head *rnh = 
+	register struct radix_node_head *rnh =
 		rt_tables[rt_key(rt)->sa_family];
 	register struct ifaddr *ifa;
 
@@ -237,7 +237,7 @@ rtredirect(dst, gateway, netmask, flags, src, rtp)
 		goto create;
 	/*
 	 * Don't listen to the redirect if it's
-	 * for a route to an interface. 
+	 * for a route to an interface.
 	 */
 	if (rt->rt_flags & RTF_GATEWAY) {
 		if (((rt->rt_flags & RTF_HOST) == 0) && (flags & RTF_HOST)) {
@@ -393,6 +393,11 @@ rtrequest(req, dst, gateway, netmask, flags, ret_nrt)
 					       rt_fixdelete, rt);
 		}
 
+		if (rt->rt_gwroute) {
+			rt = rt->rt_gwroute; RTFREE(rt);
+			(rt = (struct rtentry *)rn)->rt_gwroute = 0;
+		}
+
 		/*
 		 * NB: RTF_UP must be set during the search above,
 		 * because we might delete the last ref, causing
@@ -400,10 +405,6 @@ rtrequest(req, dst, gateway, netmask, flags, ret_nrt)
 		 */
 		rt->rt_flags &= ~RTF_UP;
 
-		if (rt->rt_gwroute) {
-			rt = rt->rt_gwroute; RTFREE(rt);
-			(rt = (struct rtentry *)rn)->rt_gwroute = 0;
-		}
 		if ((ifa = rt->rt_ifa) && ifa->ifa_rtrequest)
 			ifa->ifa_rtrequest(RTM_DELETE, rt, SA(0));
 		rttrash++;
@@ -449,18 +450,51 @@ rtrequest(req, dst, gateway, netmask, flags, ret_nrt)
 			rt_maskedcopy(dst, ndst, netmask);
 		} else
 			Bcopy(dst, ndst, dst->sa_len);
+
+		/*
+		 * This moved from below so that rnh->rnh_addaddr() can
+		 * examine the ifa and ifp if it so desires.
+		 */
+		ifa->ifa_refcnt++;
+		rt->rt_ifa = ifa;
+		rt->rt_ifp = ifa->ifa_ifp;
+
 		rn = rnh->rnh_addaddr((caddr_t)ndst, (caddr_t)netmask,
 					rnh, rt->rt_nodes);
+		if (rn == 0) {
+			struct rtentry *rt2;
+			/*
+			 * Uh-oh, we already have one of these in the tree.
+			 * We do a special hack: if the route that's already
+			 * there was generated by the protocol-cloning
+			 * mechanism, then we just blow it away and retry
+			 * the insertion of the new one.
+			 */
+			rt2 = rtalloc1(dst, 0, RTF_PRCLONING);
+			if (rt2 && rt2->rt_parent) {
+				rtrequest(RTM_DELETE, 
+					  (struct sockaddr *)rt_key(rt2),
+					  rt2->rt_gateway,
+					  rt_mask(rt2), rt2->rt_flags, 0);
+				RTFREE(rt2);
+				rn = rnh->rnh_addaddr((caddr_t)ndst,
+						      (caddr_t)netmask,
+						      rnh, rt->rt_nodes);
+			} else if (rt2) {
+				RTFREE(rt2);
+			}
+		}
+
 		if (rn == 0) {
 			if (rt->rt_gwroute)
 				rtfree(rt->rt_gwroute);
+			if (rt->rt_ifa) {
+				IFAFREE(rt->rt_ifa);
+			}
 			Free(rt_key(rt));
 			Free(rt);
 			senderr(EEXIST);
 		}
-		ifa->ifa_refcnt++;
-		rt->rt_ifa = ifa;
-		rt->rt_ifp = ifa->ifa_ifp;
 		rt->rt_parent = 0;
 
 		if (req == RTM_RESOLVE) {
@@ -477,7 +511,7 @@ rtrequest(req, dst, gateway, netmask, flags, ret_nrt)
 		 * it doesn't fire when we call it there because the node
 		 * hasn't been added to the tree yet.
 		 */
-		if (!(rt->rt_flags & RTF_HOST)) {
+		if (!(rt->rt_flags & RTF_HOST) && rt_mask(rt) != 0) {
 			struct rtfc_arg arg;
 			arg.rnh = rnh;
 			arg.rt0 = rt;
@@ -510,7 +544,7 @@ rt_fixdelete(struct radix_node *rn, void *vp)
 	struct rtentry *rt0 = vp;
 
 	if (rt->rt_parent == rt0 && !(rt->rt_flags & RTF_PINNED)) {
-		return rtrequest(RTM_DELETE, rt_key(rt), 
+		return rtrequest(RTM_DELETE, rt_key(rt),
 				 (struct sockaddr *)0, rt_mask(rt),
 				 rt->rt_flags, (struct rtentry **)0);
 	}
@@ -519,13 +553,13 @@ rt_fixdelete(struct radix_node *rn, void *vp)
 
 /*
  * This routine is called from rt_setgate() to do the analogous thing for
- * adds and changes.  There is the added complication in this case of a 
+ * adds and changes.  There is the added complication in this case of a
  * middle insert; i.e., insertion of a new network route between an older
  * network route and (cloned) host routes.  For this reason, a simple check
  * of rt->rt_parent is insufficient; each candidate route must be tested
  * against the (mask, value) of the new route (passed as before in vp)
  * to see if the new route matches it.  Unfortunately, this has the obnoxious
- * property of also triggering for insertion /above/ a pre-existing network 
+ * property of also triggering for insertion /above/ a pre-existing network
  * route and clones.  Sigh.  This may be fixed some day.
  *
  * XXX - it may be possible to do fixdelete() for changes and reserve this
@@ -562,7 +596,7 @@ rt_fixchange(struct radix_node *rn, void *vp)
 #ifdef DEBUG
 		if(rtfcdebug) printf("parent match\n");
 #endif
-		return rtrequest(RTM_DELETE, rt_key(rt), 
+		return rtrequest(RTM_DELETE, rt_key(rt),
 				 (struct sockaddr *)0, rt_mask(rt),
 				 rt->rt_flags, (struct rtentry **)0);
 	}
@@ -570,7 +604,7 @@ rt_fixchange(struct radix_node *rn, void *vp)
 	/*
 	 * There probably is a function somewhere which does this...
 	 * if not, there should be.
-	 */	
+	 */
 	len = imin(((struct sockaddr *)rt_key(rt0))->sa_len,
 		   ((struct sockaddr *)rt_key(rt))->sa_len);
 
@@ -655,7 +689,7 @@ rt_setgate(rt0, dst, gate)
 		struct rtfc_arg arg;
 		arg.rnh = rnh;
 		arg.rt0 = rt;
-		rnh->rnh_walktree_from(rnh, rt_key(rt), rt_mask(rt), 
+		rnh->rnh_walktree_from(rnh, rt_key(rt), rt_mask(rt),
 				       rt_fixchange, &arg);
 	}
 
