@@ -31,7 +31,7 @@
  * SUCH DAMAGE.
  *
  *	@(#)if_ether.c	8.1 (Berkeley) 6/10/93
- * $Id: if_ether.c,v 1.15 1995/05/09 13:35:44 davidg Exp $
+ * $FreeBSD: stable/2.1/sys/netinet/if_ether.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -147,16 +147,6 @@ arp_rtrequest(req, rt, sa)
 		if ((rt->rt_flags & RTF_HOST) == 0 &&
 		    SIN(rt_mask(rt))->sin_addr.s_addr != 0xffffffff)
 			rt->rt_flags |= RTF_CLONING;
-#if 0
-		/*
-		 * Actually, all IP gateway routes should have the cloning
-		 * flag turned on.  We can't do this yet because the expiration
-		 * stuff isn't working yet.
-		 */
-		if (rt->rt_flags & RTF_GATEWAY) {
-			rt->rt_flags |= RTF_CLONING;
-		}
-#endif
 		if (rt->rt_flags & RTF_CLONING) {
 			/*
 			 * Case 1: This route should come from a route to iface.
@@ -324,7 +314,8 @@ arpresolve(ac, rt, m, dst, desten, rt0)
 			rt = la->la_rt;
 	}
 	if (la == 0 || rt == 0) {
-		log(LOG_DEBUG, "arpresolve: can't allocate llinfo\n");
+		log(LOG_DEBUG, "arpresolve: can't allocate llinfo for %s\n",
+			inet_ntoa(SIN(dst)->sin_addr));
 		m_freem(m);
 		return (0);
 	}
@@ -516,7 +507,7 @@ reply:
 			(void)memcpy(ea->arp_sha, ac->ac_enaddr, sizeof(ea->arp_sha));
 			rtfree(rt);
 #ifdef DEBUG_PROXY
-			printf("arp: proxying for %s\n", 
+			printf("arp: proxying for %s\n",
 			       inet_ntoa(itaddr));
 #endif
 #else
@@ -614,7 +605,8 @@ arp_ifinit(ac, ifa)
 	struct ifaddr *ifa;
 {
 	ac->ac_ipaddr = IA_SIN(ifa)->sin_addr;
-	arpwhohas(ac, &ac->ac_ipaddr);
+	if (ntohl(IA_SIN(ifa)->sin_addr.s_addr) != INADDR_ANY)
+		arpwhohas(ac, &ac->ac_ipaddr);
 	ifa->ifa_rtrequest = arp_rtrequest;
 	ifa->ifa_flags |= RTF_CLONING;
 }
