@@ -31,7 +31,7 @@
  * SUCH DAMAGE.
  *
  *	@(#)in.c	8.2 (Berkeley) 11/15/93
- * $Id: in.c,v 1.12 1995/04/25 19:50:20 wollman Exp $
+ * $FreeBSD: stable/2.1/sys/netinet/in.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include <sys/param.h>
@@ -186,7 +186,7 @@ in_control(so, cmd, data, ifp)
 	struct in_ifaddr *oia;
 	struct in_aliasreq *ifra = (struct in_aliasreq *)data;
 	struct sockaddr_in oldaddr;
-	int error, hostIsNew, maskIsNew;
+	int error, hostIsNew, maskIsNew, s;
 	u_long i;
 	struct multi_kludge *mk;
 
@@ -209,11 +209,11 @@ in_control(so, cmd, data, ifp)
 				    ifra->ifra_addr.sin_addr.s_addr)
 					break;
 			}
-			if ((ifp->if_flags & IFF_POINTOPOINT) 
+			if ((ifp->if_flags & IFF_POINTOPOINT)
 			    && (cmd == SIOCAIFADDR)
 			    && (ifra->ifra_dstaddr.sin_addr.s_addr
 				== INADDR_ANY)) {
-				return EADDRNOTAVAIL;
+				return EDESTADDRREQ;
 			}
 		}
 		if (cmd == SIOCDIFADDR && ia == 0)
@@ -234,6 +234,12 @@ in_control(so, cmd, data, ifp)
 				return (ENOBUFS);
 			bzero((caddr_t)oia, sizeof *oia);
 			ia = in_ifaddr;
+			/*
+			 * Protect from ipintr() traversing address list
+			 * while we're modifying it.
+			 */
+			s = splnet();
+
 			if (ia) {
 				for ( ; ia->ia_next; ia = ia->ia_next)
 					continue;
@@ -261,6 +267,7 @@ in_control(so, cmd, data, ifp)
 			ia->ia_ifp = ifp;
 			if (!(ifp->if_flags & IFF_LOOPBACK))
 				in_interfaces++;
+			splx(s);
 		}
 		break;
 
@@ -372,6 +379,12 @@ in_control(so, cmd, data, ifp)
 			return ENOBUFS;
 
 		in_ifscrub(ifp, ia);
+		/*
+		 * Protect from ipintr() traversing address list
+		 * while we're modifying it.
+		 */
+		s = splnet();
+
 		if ((ifa = ifp->if_addrlist) == (struct ifaddr *)ia)
 			ifp->if_addrlist = ifa->ifa_next;
 		else {
@@ -398,6 +411,7 @@ in_control(so, cmd, data, ifp)
 		if (!oia->ia_multiaddrs.lh_first) {
 			IFAFREE(&oia->ia_ifa);
 			FREE(mk, M_IPMADDR);
+			splx(s);
 			break;
 		}
 
@@ -411,7 +425,7 @@ in_control(so, cmd, data, ifp)
 		IFP_TO_IA(oia->ia_ifp, ia);
 		if (ia) {	/* there is another address */
 			struct in_multi *inm;
-			for(inm = oia->ia_multiaddrs.lh_first; inm; 
+			for(inm = oia->ia_multiaddrs.lh_first; inm;
 			    inm = inm->inm_entry.le_next) {
 				IFAFREE(&inm->inm_ia->ia_ifa);
 				ia->ia_ifa.ifa_refcnt++;
@@ -426,7 +440,7 @@ in_control(so, cmd, data, ifp)
 			LIST_INIT(&mk->mk_head);
 			mk->mk_ifp = ifp;
 
-			for(inm = oia->ia_multiaddrs.lh_first; inm; 
+			for(inm = oia->ia_multiaddrs.lh_first; inm;
 			    inm = inm->inm_entry.le_next) {
 				LIST_INSERT_HEAD(&mk->mk_head, inm, inm_entry);
 			}
@@ -437,8 +451,9 @@ in_control(so, cmd, data, ifp)
 				FREE(mk, M_IPMADDR);
 			}
 		}
-		
+
 		IFAFREE((&oia->ia_ifa));
+		splx(s);
 		break;
 
 	default:
@@ -551,20 +566,20 @@ in_ifinit(ifp, ia, sin, scrub)
 
 		/*
 		 * Continuation of multicast address hack:
-		 * If there was a multicast group list previously saved 
+		 * If there was a multicast group list previously saved
 		 * for this interface, then we re-attach it to the first
 		 * address configured on the i/f.
 		 */
 		for(mk = in_mk.lh_first; mk; mk = mk->mk_entry.le_next) {
 			if(mk->mk_ifp == ifp) {
 				struct in_multi *inm;
-				
-				for(inm = mk->mk_head.lh_first; inm; 
+
+				for(inm = mk->mk_head.lh_first; inm;
 				    inm = inm->inm_entry.le_next) {
 					IFAFREE(&inm->inm_ia->ia_ifa);
 					ia->ia_ifa.ifa_refcnt++;
 					inm->inm_ia = ia;
-					LIST_INSERT_HEAD(&ia->ia_multiaddrs, 
+					LIST_INSERT_HEAD(&ia->ia_multiaddrs,
 							 inm, inm_entry);
 				}
 				LIST_REMOVE(mk, mk_entry);
@@ -609,7 +624,13 @@ in_broadcast(in, ifp)
 		     /*
 		      * Check for old-style (host 0) broadcast.
 		      */
-		     t == ia->ia_subnet || t == ia->ia_net))
+		     t == ia->ia_subnet || t == ia->ia_net) &&
+		     /*
+		      * Check for an all one subnetmask. These
+		      * only exist when an interface gets a secondary
+		      * address.
+		      */
+		     ia->ia_subnetmask != (u_long)0xffffffff)
 			    return 1;
 	return (0);
 #undef ia
