@@ -31,7 +31,7 @@
  * SUCH DAMAGE.
  *
  *	@(#)ip_output.c	8.3 (Berkeley) 1/21/94
- * $Id: ip_output.c,v 1.17 1995/04/26 18:10:55 pst Exp $
+ * $FreeBSD: stable/2.1/sys/netinet/ip_output.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include <sys/param.h>
@@ -85,6 +85,23 @@ ip_output(m0, opt, ro, flags, imo)
 	register struct mbuf *m = m0;
 	register int hlen = sizeof (struct ip);
 	int len, off, error = 0;
+	/*
+	 * It might seem obvious at first glance that one could easily
+	 * make a one-behind cache out of this by simply making `iproute'
+	 * static and eliminating the bzero() below.  However, this turns
+	 * out not to work, for two reasons:
+	 *
+	 * 1) This routine needs to be reentrant.  It can be called
+	 * recursively from encapsulating network interfaces, and it
+	 * is always called recursively from ip_mforward().
+	 *
+	 * 2) You turn out not to gain much.  There is already a one-
+	 * behind cache implemented for the specific case of forwarding,
+	 * and sends on a connected socket will use a route associated
+	 * with the PCB.  The only cases left are sends on unconnected
+	 * and raw sockets, and if these cases are really significant,
+	 * something is seriously wrong.
+	 */
 	struct route iproute;
 	struct sockaddr_in *dst;
 	struct in_ifaddr *ia;
@@ -188,15 +205,20 @@ ip_output(m0, opt, ro, flags, imo)
 			ip->ip_ttl = imo->imo_multicast_ttl;
 			if (imo->imo_multicast_ifp != NULL)
 				ifp = imo->imo_multicast_ifp;
+			if (imo->imo_multicast_vif != -1)
+				ip->ip_src.s_addr =
+				    ip_mcast_src(imo->imo_multicast_vif);
 		} else
 			ip->ip_ttl = IP_DEFAULT_MULTICAST_TTL;
 		/*
 		 * Confirm that the outgoing interface supports multicast.
 		 */
-		if ((ifp->if_flags & IFF_MULTICAST) == 0) {
-			ipstat.ips_noroute++;
-			error = ENETUNREACH;
-			goto bad;
+		if ((imo == NULL) || (imo->imo_multicast_vif == -1)) {
+			if ((ifp->if_flags & IFF_MULTICAST) == 0) {
+				ipstat.ips_noroute++;
+				error = ENETUNREACH;
+				goto bad;
+			}
 		}
 		/*
 		 * If source address not specified yet, use address
@@ -242,7 +264,7 @@ ip_output(m0, opt, ro, flags, imo)
 				 * is multicast and not just sent down one link
 				 * as prescribed by rsvpd.
 				 */
-				if (ip_rsvpd == NULL)
+				if (!rsvp_on)
 				  imo = NULL;
 				if (ip_mforward(ip, ifp, m, imo) != 0) {
 					m_freem(m);
@@ -308,6 +330,14 @@ ip_output(m0, opt, ro, flags, imo)
 		m->m_flags &= ~M_BCAST;
 
 sendit:
+	/*
+	 * Check with the firewall...
+	 */
+	if (!(*ip_fw_chk_ptr)(m,ip,ifp,1)) {
+		error = EACCES;
+		goto done;
+	}
+
 	/*
 	 * If small enough for interface, can just send directly.
 	 */
@@ -326,6 +356,20 @@ sendit:
 	 */
 	if (ip->ip_off & IP_DF) {
 		error = EMSGSIZE;
+#if 1
+		/*
+		 * This case can happen if the user changed the MTU
+		 * of an interface after enabling IP on it.  Because
+		 * most netifs don't keep track of routes pointing to
+		 * them, there is no way for one to update all its
+		 * routes when the MTU is changed.
+		 */
+		if ((ro->ro_rt->rt_flags & (RTF_UP | RTF_HOST))
+		    && !(ro->ro_rt->rt_rmx.rmx_locks & RTV_MTU)
+		    && (ro->ro_rt->rt_rmx.rmx_mtu > ifp->if_mtu)) {
+			ro->ro_rt->rt_rmx.rmx_mtu = ifp->if_mtu;
+		}
+#endif
 		ipstat.ips_cantfrag++;
 		goto bad;
 	}
@@ -412,16 +456,6 @@ sendorfree:
 done:
 	if (ro == &iproute && (flags & IP_ROUTETOIF) == 0 && ro->ro_rt)
 		RTFREE(ro->ro_rt);
-	/*
-	 * Count outgoing packet,here we count both our packets and
-	 * those we forward.
-	 * Here we want to convert ip_len to host byte order when counting
-	 * so we set 3rd arg to 1.
-	 * This is locally generated packet so it has not
-	 * incoming interface.
-	 */
-	if (ip_acct_cnt_ptr!=NULL)
-		(*ip_acct_cnt_ptr)(ip,NULL,ip_acct_chain,1);
 
 	return (error);
 bad:
@@ -433,6 +467,8 @@ bad:
  * Insert IP options into preformed packet.
  * Adjust IP destination as required for IP source routing,
  * as indicated by a non-zero in_addr at the start of the options.
+ *
+ * XXX This routine assumes that the packet has no options in place.
  */
 static struct mbuf *
 ip_insertoptions(m, opt, phlen)
@@ -471,6 +507,7 @@ ip_insertoptions(m, opt, phlen)
 	ip = mtod(m, struct ip *);
 	(void)memcpy(ip + 1, p->ipopt_list, (unsigned)optlen);
 	*phlen = sizeof(struct ip) + optlen;
+	ip->ip_hl = *phlen >> 2;
 	ip->ip_len += optlen;
 	return (m);
 }
@@ -549,7 +586,7 @@ ip_ctloutput(op, so, level, optname, mp)
 		case IP_RECVOPTS:
 		case IP_RECVRETOPTS:
 		case IP_RECVDSTADDR:
-			if (m->m_len != sizeof(int))
+			if (m == 0 || m->m_len != sizeof(int))
 				error = EINVAL;
 			else {
 				optval = *mtod(m, int *);
@@ -805,7 +842,7 @@ ip_setmoptions(optname, imop, m)
 			return (ENOBUFS);
 		*imop = imo;
 		imo->imo_multicast_ifp = NULL;
-		imo->imo_multicast_vif = 0;
+		imo->imo_multicast_vif = -1;
 		imo->imo_multicast_ttl = IP_DEFAULT_MULTICAST_TTL;
 		imo->imo_multicast_loop = IP_DEFAULT_MULTICAST_LOOP;
 		imo->imo_num_memberships = 0;
@@ -823,7 +860,7 @@ ip_setmoptions(optname, imop, m)
 			break;
 		}
 		i = *(mtod(m, int *));
-		if (!legal_vif_num(i)) {
+		if (!legal_vif_num(i) && (i != -1)) {
 			error = EINVAL;
 			break;
 		}
@@ -907,7 +944,7 @@ ip_setmoptions(optname, imop, m)
 		 * the route to the given multicast address.
 		 */
 		if (mreq->imr_interface.s_addr == INADDR_ANY) {
-			ro.ro_rt = NULL;
+			bzero((caddr_t)&ro, sizeof(ro));
 			dst = (struct sockaddr_in *)&ro.ro_dst;
 			dst->sin_len = sizeof(*dst);
 			dst->sin_family = AF_INET;
@@ -1036,7 +1073,7 @@ ip_setmoptions(optname, imop, m)
 	 * If all options have default values, no need to keep the mbuf.
 	 */
 	if (imo->imo_multicast_ifp == NULL &&
-	    imo->imo_multicast_vif == 0 &&
+	    imo->imo_multicast_vif == -1 &&
 	    imo->imo_multicast_ttl == IP_DEFAULT_MULTICAST_TTL &&
 	    imo->imo_multicast_loop == IP_DEFAULT_MULTICAST_LOOP &&
 	    imo->imo_num_memberships == 0) {
@@ -1069,7 +1106,7 @@ ip_getmoptions(optname, imo, mp)
 		if (imo != NULL)
 			*(mtod(*mp, int *)) = imo->imo_multicast_vif;
 		else
-			*(mtod(*mp, int *)) = 7890;
+			*(mtod(*mp, int *)) = -1;
 		(*mp)->m_len = sizeof(int);
 		return(0);
 
