@@ -31,7 +31,7 @@
  * SUCH DAMAGE.
  *
  *	From: @(#)tcp_input.c	8.5 (Berkeley) 4/10/94
- *	$Id: tcp_input.c,v 1.23 1995/05/09 12:32:06 olah Exp $
+ * $FreeBSD: stable/2.1/sys/netinet/tcp_input.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #ifndef TUBA_INCLUDE
@@ -44,6 +44,7 @@
 #include <sys/socketvar.h>
 #include <sys/errno.h>
 #include <sys/queue.h>
+#include <sys/kernel.h>
 
 #include <net/if.h>
 #include <net/route.h>
@@ -217,8 +218,6 @@ present:
 	ti = tp->seg_next;
 	if (ti == (struct tcpiphdr *)tp || ti->ti_seq != tp->rcv_nxt)
 		return (0);
-	if (tp->t_state == TCPS_SYN_RECEIVED && ti->ti_len)
-		return (0);
 	do {
 		tp->rcv_nxt += ti->ti_len;
 		flags = ti->ti_flags & TH_FIN;
@@ -318,23 +317,6 @@ tcp_input(m, iphlen)
 		}
 		optlen = off - sizeof (struct tcphdr);
 		optp = mtod(m, caddr_t) + sizeof (struct tcpiphdr);
-		/* 
-		 * Do quick retrieval of timestamp options ("options
-		 * prediction?").  If timestamp is the only option and it's
-		 * formatted as recommended in RFC 1323 appendix A, we
-		 * quickly get the values now and not bother calling
-		 * tcp_dooptions(), etc.
-		 */
-		if ((optlen == TCPOLEN_TSTAMP_APPA ||
-		     (optlen > TCPOLEN_TSTAMP_APPA &&
-			optp[TCPOLEN_TSTAMP_APPA] == TCPOPT_EOL)) &&
-		     *(u_long *)optp == htonl(TCPOPT_TSTAMP_HDR) &&
-		     (ti->ti_flags & TH_SYN) == 0) {
-			to.to_flag |= TOF_TS;
-			to.to_tsval = ntohl(*(u_long *)(optp + 4));
-			to.to_tsecr = ntohl(*(u_long *)(optp + 8));
-			optp = NULL;	/* we've parsed the options */
-		}
 	}
 	tiflags = ti->ti_flags;
 
@@ -382,7 +364,7 @@ findpcb:
 		goto dropwithreset;
 	if (tp->t_state == TCPS_CLOSED)
 		goto drop;
-	
+
 	/* Unscale the window into a 32-bit value. */
 	if ((tiflags & TH_SYN) == 0)
 		tiwin = ti->ti_win << tp->snd_scale;
@@ -399,9 +381,65 @@ findpcb:
 #endif
 		if (so->so_options & SO_ACCEPTCONN) {
 			register struct tcpcb *tp0 = tp;
-			so = sonewconn(so, 0);
-			if (so == 0)
+			struct socket *so2;
+			if ((tiflags & (TH_RST|TH_ACK|TH_SYN)) != TH_SYN) {
+				/*
+				 * Note: dropwithreset makes sure we don't
+				 * send a RST in response to a RST.
+				 */
+				if (tiflags & TH_ACK) {
+					tcpstat.tcps_badsyn++;
+					goto dropwithreset;
+				}
 				goto drop;
+			}
+			so2 = sonewconn(so, 0);
+			if (so2 == 0) {
+				unsigned int i, j, qlen;
+
+				static int rnd;
+				static long old_mono_secs;
+				static unsigned int cur_cnt, old_cnt;
+
+				tcpstat.tcps_listendrop++;
+
+				/*
+				 * Keep a decaying average of the number
+				 * of overruns we've been getting.
+				 */
+				if ((i = (mono_time.tv_sec -
+					  old_mono_secs)) != 0) {
+					old_mono_secs = mono_time.tv_sec;
+					old_cnt = cur_cnt / i;
+					cur_cnt = 0;
+				}
+
+				so2 = so->so_q0;
+				if (so2 == 0)
+					goto drop;
+
+				/*
+				 * If we've been getting a lot of hits,
+				 * random drop an incomplete connection
+				 * from the queue, otherwise, fall through
+				 * so we head-drop from the queue.
+				 */
+				qlen = so->so_q0len;
+				if (++cur_cnt > qlen || old_cnt > qlen) {
+					rnd = (314159 * rnd + 66329) & 0xffff;
+					j = ((qlen + 1) * rnd) >> 16;
+
+					while (j-- && so2)
+						so2 = so2->so_q0;
+				}
+				if (so2) {
+					tcp_drop(sototcpcb(so2), ETIMEDOUT);
+					so2 = sonewconn(so, 0);
+				}
+				if (!so2)
+					goto drop;
+			}
+			so = so2;
 			/*
 			 * This is ugly, but ....
 			 *
@@ -437,17 +475,17 @@ findpcb:
 	 * Reset idle time and keep-alive timer.
 	 */
 	tp->t_idle = 0;
-	tp->t_timer[TCPT_KEEP] = tcp_keepidle;
+	if (TCPS_HAVEESTABLISHED(tp->t_state))
+		tp->t_timer[TCPT_KEEP] = tcp_keepidle;
 
 	/*
 	 * Process options if not in LISTEN state,
 	 * else do it below (after getting remote address).
 	 */
-	if (optp && tp->t_state != TCPS_LISTEN)
-		tcp_dooptions(tp, optp, optlen, ti,
-			&to);
+	if (tp->t_state != TCPS_LISTEN)
+		tcp_dooptions(tp, optp, optlen, ti, &to);
 
-	/* 
+	/*
 	 * Header prediction: check for the two common cases
 	 * of a uni-directional data xfer.  If the packet has
 	 * no control flags, is in-sequence, the window didn't
@@ -480,7 +518,7 @@ findpcb:
 	    tiwin && tiwin == tp->snd_wnd &&
 	    tp->snd_nxt == tp->snd_max) {
 
-		/* 
+		/*
 		 * If last ACK falls within this segment's sequence numbers,
 		 * record the timestamp.
 		 * NOTE that the test is modified according to the latest
@@ -580,7 +618,7 @@ findpcb:
 	win = sbspace(&so->so_rcv);
 	if (win < 0)
 		win = 0;
-	tp->rcv_wnd = max(win, (int)(tp->rcv_adv - tp->rcv_nxt));
+	tp->rcv_wnd = imax(win, (int)(tp->rcv_adv - tp->rcv_nxt));
 	}
 
 	switch (tp->t_state) {
@@ -645,9 +683,7 @@ findpcb:
 			taop = &tao_noncached;
 			bzero(taop, sizeof(*taop));
 		}
-		if (optp)
-			tcp_dooptions(tp, optp, optlen, ti,
-				&to);
+		tcp_dooptions(tp, optp, optlen, ti, &to);
 		if (iss)
 			tp->iss = iss;
 		else
@@ -666,7 +702,7 @@ findpcb:
 		tp->cc_recv = to.to_cc;
 		/*
 		 * Perform TAO test on incoming CC (SEG.CC) option, if any.
-		 * - compare SEG.CC against cached CC from the same host, 
+		 * - compare SEG.CC against cached CC from the same host,
 		 *	if any.
 		 * - if SEG.CC > chached value, SYN must be new and is accepted
 		 *	immediately: save new CC in the cache, mark the socket
@@ -678,7 +714,9 @@ findpcb:
 		 * - otherwise do a normal 3-way handshake.
 		 */
 		if ((to.to_flag & TOF_CC) != 0) {
-		    if (taop->tao_cc != 0 && CC_GT(to.to_cc, taop->tao_cc)) {
+		    if (((tp->t_flags & TF_NOPUSH) != 0) &&
+			taop->tao_cc != 0 && CC_GT(to.to_cc, taop->tao_cc)) {
+
 			taop->tao_cc = to.to_cc;
 			tp->t_state = TCPS_ESTABLISHED;
 
@@ -694,10 +732,16 @@ findpcb:
 				tp->t_flags |= (TF_DELACK | TF_NEEDSYN);
 			else
 				tp->t_flags |= (TF_ACKNOW | TF_NEEDSYN);
-			tp->rcv_adv += tp->rcv_wnd;
+
+			/*
+			 * Limit the `virtual advertised window' to TCP_MAXWIN
+			 * here.  Even if we requested window scaling, it will
+			 * become effective only later when our SYN is acked.
+			 */
+			tp->rcv_adv += min(tp->rcv_wnd, TCP_MAXWIN);
 			tcpstat.tcps_connects++;
 			soisconnected(so);
-			tp->t_timer[TCPT_KEEP] = TCPTV_KEEP_INIT;
+			tp->t_timer[TCPT_KEEP] = tcp_keepinit;
 			dropsocket = 0;		/* committed to socket */
 			tcpstat.tcps_accepts++;
 			goto trimthenstep6;
@@ -716,13 +760,15 @@ findpcb:
 		 */
 		tp->t_flags |= TF_ACKNOW;
 		tp->t_state = TCPS_SYN_RECEIVED;
-		tp->t_timer[TCPT_KEEP] = TCPTV_KEEP_INIT;
+		tp->t_timer[TCPT_KEEP] = tcp_keepinit;
 		dropsocket = 0;		/* committed to socket */
 		tcpstat.tcps_accepts++;
 		goto trimthenstep6;
 		}
 
 	/*
+	 * If the state is SYN_RECEIVED:
+	 *	do just the ack and RST checks from SYN_SENT state.
 	 * If the state is SYN_SENT:
 	 *	if seg contains an ACK, but not for our SYN, drop the input.
 	 *	if seg contains a RST, then drop the connection.
@@ -734,6 +780,7 @@ findpcb:
 	 *	arrange for segment to be acked (eventually)
 	 *	continue processing rest of data/controls, beginning with URG
 	 */
+	case TCPS_SYN_RECEIVED:
 	case TCPS_SYN_SENT:
 		if ((taop = tcp_gettaocache(inp)) == NULL) {
 			taop = &tao_noncached;
@@ -761,6 +808,8 @@ findpcb:
 				tp = tcp_drop(tp, ECONNREFUSED);
 			goto drop;
 		}
+		if (tp->t_state == TCPS_SYN_RECEIVED)
+			break;
 		if ((tiflags & TH_SYN) == 0)
 			goto drop;
 		tp->snd_wnd = ti->ti_win;	/* initial send window */
@@ -768,7 +817,23 @@ findpcb:
 
 		tp->irs = ti->ti_seq;
 		tcp_rcvseqinit(tp);
-		if (tiflags & TH_ACK && SEQ_GT(ti->ti_ack, tp->iss)) {
+		if (tiflags & TH_ACK) {
+			/*
+			 * Our SYN was acked.  If segment contains CC.ECHO
+			 * option, check it to make sure this segment really
+			 * matches our SYN.  If not, just drop it as old
+			 * duplicate, but send an RST if we're still playing
+			 * by the old rules.  If no CC.ECHO option, make sure
+			 * we don't get fooled into using T/TCP.
+			 */
+			if (to.to_flag & TOF_CCECHO) {
+				if (tp->cc_send != to.to_ccecho)
+					if (taop->tao_ccsent != 0)
+						goto drop;
+					else
+						goto dropwithreset;
+			} else
+				tp->t_flags &= ~TF_RCVD_CC;
 			tcpstat.tcps_connects++;
 			soisconnected(so);
 			/* Do window scaling on this connection? */
@@ -777,24 +842,10 @@ findpcb:
 				tp->snd_scale = tp->requested_s_scale;
 				tp->rcv_scale = tp->request_r_scale;
 			}
-			/*
-			 * Our SYN was acked.  If segment contains CC.ECHO
-			 * option, check it to make sure this segment really
-			 * matches our SYN.  If not, just drop it as old
-			 * duplicate, but send an RST if we're still playing
-			 * by the old rules.
-			 */
-			if ((to.to_flag & TOF_CCECHO) &&
-			    tp->cc_send != to.to_ccecho) {
-				if (taop->tao_ccsent != 0)
-					goto drop;
-				else
-					goto dropwithreset;
-			}
 			/* Segment is acceptable, update cache if undefined. */
 			if (taop->tao_ccsent == 0)
 				taop->tao_ccsent = to.to_ccecho;
-			
+
 			tp->rcv_adv += tp->rcv_wnd;
 			tp->snd_una++;		/* SYN is acked */
 			/*
@@ -815,26 +866,27 @@ findpcb:
 				tp->t_state = TCPS_FIN_WAIT_1;
 				tp->t_flags &= ~TF_NEEDFIN;
 				tiflags &= ~TH_SYN;
-			} else
+			} else {
 				tp->t_state = TCPS_ESTABLISHED;
-
+				tp->t_timer[TCPT_KEEP] = tcp_keepidle;
+			}
 		} else {
 		/*
 		 *  Received initial SYN in SYN-SENT[*] state => simul-
 		 *  taneous open.  If segment contains CC option and there is
-		 *  a cached CC, apply TAO test; if it succeeds, connection is 
+		 *  a cached CC, apply TAO test; if it succeeds, connection is
 		 *  half-synchronized.  Otherwise, do 3-way handshake:
 		 *        SYN-SENT -> SYN-RECEIVED
 		 *        SYN-SENT* -> SYN-RECEIVED*
 		 *  If there was no CC option, clear cached CC value.
 		 */
 			tp->t_flags |= TF_ACKNOW;
-			tp->t_timer[TCPT_REXMT] = 0;		    
+			tp->t_timer[TCPT_REXMT] = 0;
 			if (to.to_flag & TOF_CC) {
 				if (taop->tao_cc != 0 &&
 				    CC_GT(to.to_cc, taop->tao_cc)) {
 					/*
-					 * update cache and make transition: 
+					 * update cache and make transition:
 					 *        SYN-SENT -> ESTABLISHED*
 					 *        SYN-SENT* -> FIN-WAIT-1*
 					 */
@@ -842,8 +894,10 @@ findpcb:
 					if (tp->t_flags & TF_NEEDFIN) {
 						tp->t_state = TCPS_FIN_WAIT_1;
 						tp->t_flags &= ~TF_NEEDFIN;
-					} else
+					} else {
 						tp->t_state = TCPS_ESTABLISHED;
+						tp->t_timer[TCPT_KEEP] = tcp_keepidle;
+					}
 					tp->t_flags |= TF_NEEDSYN;
 				} else
 					tp->t_state = TCPS_SYN_RECEIVED;
@@ -852,7 +906,7 @@ findpcb:
 				taop->tao_cc = 0;
 				tp->t_state = TCPS_SYN_RECEIVED;
 			}
-		}				
+		}
 
 trimthenstep6:
 		/*
@@ -877,7 +931,7 @@ trimthenstep6:
 		 *  our data will be ACK'd; if so, enter normal data segment
 		 *  processing in the middle of step 5, ack processing.
 		 *  Otherwise, goto step 6.
-		 */ 
+		 */
  		if (tiflags & TH_ACK)
 			goto process_ACK;
 		goto step6;
@@ -893,8 +947,8 @@ trimthenstep6:
 	 *                  segment, hoping to find new TCPCB in LISTEN state;
 	 *
 	 *		else must be old SYN; drop it.
-	 *      else do normal processing. 
-	 */      
+	 *      else do normal processing.
+	 */
 	case TCPS_LAST_ACK:
 	case TCPS_CLOSING:
 	case TCPS_TIME_WAIT:
@@ -917,7 +971,7 @@ trimthenstep6:
 	 * States other than LISTEN or SYN_SENT.
 	 * First check timestamp, if present.
 	 * Then check the connection count, if present.
-	 * Then check that at least some bytes of segment are within 
+	 * Then check that at least some bytes of segment are within
 	 * receive window.  If segment begins before rcv_nxt,
 	 * drop leading data (and SYN); if nothing left, just ack.
 	 *
@@ -965,7 +1019,7 @@ trimthenstep6:
 		if (tiflags & TH_SYN) {
 			tiflags &= ~TH_SYN;
 			ti->ti_seq++;
-			if (ti->ti_urp > 1) 
+			if (ti->ti_urp > 1)
 				ti->ti_urp--;
 			else
 				tiflags &= ~TH_URG;
@@ -1064,7 +1118,7 @@ trimthenstep6:
 	 * NOTE that the test is modified according to the latest
 	 * proposal of the tcplw@cray.com list (Braden 1993/04/26).
 	 */
-	if ((to.to_flag & TOF_TS) != 0 && 
+	if ((to.to_flag & TOF_TS) != 0 &&
 	    SEQ_LEQ(ti->ti_seq, tp->last_ack_sent)) {
 		tp->ts_recent_age = tcp_now;
 		tp->ts_recent = to.to_tsval;
@@ -1125,7 +1179,7 @@ trimthenstep6:
 		else
 			goto drop;
 	}
-	
+
 	/*
 	 * Ack processing.
 	 */
@@ -1159,16 +1213,18 @@ trimthenstep6:
 			taop->tao_cc = tp->cc_recv;
 
 		/*
-		 * Make transitions:  
+		 * Make transitions:
 		 *      SYN-RECEIVED  -> ESTABLISHED
 		 *      SYN-RECEIVED* -> FIN-WAIT-1
 		 */
 		if (tp->t_flags & TF_NEEDFIN) {
 			tp->t_state = TCPS_FIN_WAIT_1;
 			tp->t_flags &= ~TF_NEEDFIN;
-		} else
+		} else {
 			tp->t_state = TCPS_ESTABLISHED;
-		/* 
+			tp->t_timer[TCPT_KEEP] = tcp_keepidle;
+		}
+		/*
 		 * If segment contains data or ACK, will call tcp_reass()
 		 * later; if not, do so now to pass queued data to user.
 		 */
@@ -1216,7 +1272,7 @@ trimthenstep6:
 				 * the new ssthresh).
 				 *
 				 * Dup acks mean that packets have left the
-				 * network (they're now cached at the receiver) 
+				 * network (they're now cached at the receiver)
 				 * so bump cwnd by the amount in the receiver
 				 * to keep a constant cwnd packets in the
 				 * network.
@@ -1265,18 +1321,25 @@ trimthenstep6:
 			goto dropafterack;
 		}
 		/*
-		 *  If we reach this point, ACK is not a duplicate, 
+		 *  If we reach this point, ACK is not a duplicate,
 		 *     i.e., it ACKs something we sent.
 		 */
 		if (tp->t_flags & TF_NEEDSYN) {
-			/* 
-			 *   T/TCP: Connection was half-synchronized, and our
-			 *   SYN has been ACK'd (so connection is now fully
-			 *   synchronized).  Go to non-starred state and
-			 *   increment snd_una for ACK of SYN.
+			/*
+			 * T/TCP: Connection was half-synchronized, and our
+			 * SYN has been ACK'd (so connection is now fully
+			 * synchronized).  Go to non-starred state,
+			 * increment snd_una for ACK of SYN, and check if
+			 * we can do window scaling.
 			 */
 			tp->t_flags &= ~TF_NEEDSYN;
 			tp->snd_una++;
+			/* Do window scaling? */
+			if ((tp->t_flags & (TF_RCVD_SCALE|TF_REQ_SCALE)) ==
+				(TF_RCVD_SCALE|TF_REQ_SCALE)) {
+				tp->snd_scale = tp->requested_s_scale;
+				tp->rcv_scale = tp->request_r_scale;
+			}
 		}
 
 process_ACK:
@@ -1422,7 +1485,7 @@ step6:
 	 * Don't look at window if no ACK: TAC's send garbage on first SYN.
 	 */
 	if ((tiflags & TH_ACK) &&
-	    (SEQ_LT(tp->snd_wl1, ti->ti_seq) || 
+	    (SEQ_LT(tp->snd_wl1, ti->ti_seq) ||
 	    (tp->snd_wl1 == ti->ti_seq && (SEQ_LT(tp->snd_wl2, ti->ti_ack) ||
 	     (tp->snd_wl2 == ti->ti_ack && tiwin > tp->snd_wnd))))) {
 		/* keep track of pure window updates */
@@ -1457,14 +1520,14 @@ step6:
 		 * If this segment advances the known urgent pointer,
 		 * then mark the data stream.  This should not happen
 		 * in CLOSE_WAIT, CLOSING, LAST_ACK or TIME_WAIT STATES since
-		 * a FIN has been received from the remote side. 
+		 * a FIN has been received from the remote side.
 		 * In these states we ignore the URG.
 		 *
 		 * According to RFC961 (Assigned Protocols),
 		 * the urgent pointer points to the last octet
 		 * of urgent data.  We continue, however,
 		 * to consider it to indicate the first octet
-		 * of data past the urgent section as the original 
+		 * of data past the urgent section as the original
 		 * spec states (in one of two places).
 		 */
 		if (SEQ_GT(ti->ti_seq+ti->ti_urp, tp->rcv_up)) {
@@ -1529,14 +1592,14 @@ dodata:							/* XXX */
 			socantrcvmore(so);
 			/*
 			 *  If connection is half-synchronized
-			 *  (ie SEND_SYN flag on) then delay ACK,
+			 *  (ie NEEDSYN flag on) then delay ACK,
 			 *  so it may be piggybacked when SYN is sent.
 			 *  Otherwise, since we received a FIN then no
-			 *  more input can be expected, send ACK now. 
+			 *  more input can be expected, send ACK now.
 			 */
 			if (tp->t_flags & TF_NEEDSYN)
 				tp->t_flags |= TF_DELACK;
-			else 
+			else
 				tp->t_flags |= TF_ACKNOW;
 			tp->rcv_nxt++;
 		}
@@ -1561,7 +1624,7 @@ dodata:							/* XXX */
 
 	 	/*
 		 * In FIN_WAIT_2 state enter the TIME_WAIT state,
-		 * starting the time-wait timer, turning off the other 
+		 * starting the time-wait timer, turning off the other
 		 * standard timers.
 		 */
 		case TCPS_FIN_WAIT_2:
@@ -1714,7 +1777,7 @@ tcp_dooptions(tp, cp, cnt, ti, to)
 			    (char *)&to->to_tsecr, sizeof(to->to_tsecr));
 			NTOHL(to->to_tsecr);
 
-			/* 
+			/*
 			 * A timestamp received in a SYN makes
 			 * it ok to send timestamp requests and replies.
 			 */
@@ -1731,7 +1794,7 @@ tcp_dooptions(tp, cp, cnt, ti, to)
 			bcopy((char *)cp + 2,
 			    (char *)&to->to_cc, sizeof(to->to_cc));
 			NTOHL(to->to_cc);
-			/* 
+			/*
 			 * A CC or CC.new option received in a SYN makes
 			 * it ok to send CC in subsequent segments.
 			 */
@@ -1747,7 +1810,7 @@ tcp_dooptions(tp, cp, cnt, ti, to)
 			bcopy((char *)cp + 2,
 			    (char *)&to->to_cc, sizeof(to->to_cc));
 			NTOHL(to->to_cc);
-			/* 
+			/*
 			 * A CC or CC.new option received in a SYN makes
 			 * it ok to send CC in subsequent segments.
 			 */
@@ -1782,7 +1845,7 @@ tcp_pulloutofband(so, ti, m)
 	register struct mbuf *m;
 {
 	int cnt = ti->ti_urp - 1;
-	
+
 	while (cnt >= 0) {
 		if (m->m_len > cnt) {
 			char *cp = mtod(m, caddr_t) + cnt;
@@ -1814,6 +1877,7 @@ tcp_xmit_timer(tp, rtt)
 	register short delta;
 
 	tcpstat.tcps_rttupdated++;
+	tp->t_rttupdated++;
 	if (tp->t_srtt != 0) {
 		/*
 		 * srtt is stored as fixed point with 3 bits after the
@@ -1841,7 +1905,7 @@ tcp_xmit_timer(tp, rtt)
 		if ((tp->t_rttvar += delta) <= 0)
 			tp->t_rttvar = 1;
 	} else {
-		/* 
+		/*
 		 * No rtt measurement yet - use the unsmoothed rtt.
 		 * Set the variance to half the rtt (so our first
 		 * retransmit happens at 3*rtt).
@@ -1865,7 +1929,7 @@ tcp_xmit_timer(tp, rtt)
 	 */
 	TCPT_RANGESET(tp->t_rxtcur, TCP_REXMTVAL(tp),
 	    tp->t_rttmin, TCPTV_REXMTMAX);
-	
+
 	/*
 	 * We received an ack for a packet that wasn't retransmitted;
 	 * it is probably safe to discard any error indications we've
@@ -1949,7 +2013,6 @@ tcp_mss(tp, offer)
 		offer = max(offer, 64);
 	taop->tao_mssopt = offer;
 
-#ifdef RTV_MTU	/* if route characteristics exist ... */
 	/*
 	 * While we're here, check if there's an initial rtt
 	 * or rttvar.  Convert from the route-table units
@@ -1963,13 +2026,16 @@ tcp_mss(tp, offer)
 		if (rt->rt_rmx.rmx_locks & RTV_RTT)
 			tp->t_rttmin = rtt / (RTM_RTTUNIT / PR_SLOWHZ);
 		tp->t_srtt = rtt / (RTM_RTTUNIT / (PR_SLOWHZ * TCP_RTT_SCALE));
-		if (rt->rt_rmx.rmx_rttvar)
+		tcpstat.tcps_usedrtt++;
+		if (rt->rt_rmx.rmx_rttvar) {
 			tp->t_rttvar = rt->rt_rmx.rmx_rttvar /
 			    (RTM_RTTUNIT / (PR_SLOWHZ * TCP_RTTVAR_SCALE));
-		else
+			tcpstat.tcps_usedrttvar++;
+		} else {
 			/* default variation is +- 1 rtt */
 			tp->t_rttvar =
 			    tp->t_srtt * TCP_RTTVAR_SCALE / TCP_RTT_SCALE;
+		}
 		TCPT_RANGESET(tp->t_rxtcur,
 		    ((tp->t_srtt >> 2) + tp->t_rttvar) >> 1,
 		    tp->t_rttmin, TCPTV_REXMTMAX);
@@ -1980,7 +2046,6 @@ tcp_mss(tp, offer)
 	if (rt->rt_rmx.rmx_mtu)
 		mss = rt->rt_rmx.rmx_mtu - sizeof(struct tcpiphdr);
 	else
-#endif /* RTV_MTU */
 	{
 		mss = ifp->if_mtu - sizeof(struct tcpiphdr);
 		if (!in_localaddr(inp->inp_faddr))
@@ -2053,7 +2118,6 @@ tcp_mss(tp, offer)
 	if (!in_localaddr(inp->inp_faddr))
 		tp->snd_cwnd = mss;
 
-#ifdef RTV_SSTHRESH
 	if (rt->rt_rmx.rmx_ssthresh) {
 		/*
 		 * There's some sort of gateway or interface
@@ -2062,8 +2126,8 @@ tcp_mss(tp, offer)
 		 * threshold to no less than 2*mss.
 		 */
 		tp->snd_ssthresh = max(2 * mss, rt->rt_rmx.rmx_ssthresh);
+		tcpstat.tcps_usedssthresh++;
 	}
-#endif
 }
 
 /*
@@ -2079,12 +2143,6 @@ tcp_mssopt(tp)
 	if (rt == NULL)
 		return tcp_mssdflt;
 
-	/*
-	 * if there's an mtu associated with the route, use it
-	 */
-	if (rt->rt_rmx.rmx_mtu)
-		return rt->rt_rmx.rmx_mtu - sizeof(struct tcpiphdr);
-
 	return rt->rt_ifp->if_mtu - sizeof(struct tcpiphdr);
 }
 #endif /* TUBA_INCLUDE */
