@@ -31,7 +31,7 @@
  * SUCH DAMAGE.
  *
  *	From: @(#)tcp_usrreq.c	8.2 (Berkeley) 1/3/94
- *	$Id: tcp_usrreq.c,v 1.12 1995/03/16 18:15:06 bde Exp $
+ * $FreeBSD: stable/2.1/sys/netinet/tcp_usrreq.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include <sys/param.h>
@@ -93,7 +93,7 @@ tcp_usrreq(so, req, m, nam, control)
 #endif
 
 	if (req == PRU_CONTROL)
-		return (in_control(so, (int)m, (caddr_t)nam,
+		return (in_control(so, (u_long)m, (caddr_t)nam,
 			(struct ifnet *)control));
 	if (control && control->m_len) {
 		m_freem(control);
@@ -111,6 +111,18 @@ tcp_usrreq(so, req, m, nam, control)
 	 */
 	if (inp == 0 && req != PRU_ATTACH) {
 		splx(s);
+#if 0
+		/*
+		 * The following corrects an mbuf leak under rare
+		 * circumstances, but has not been fully tested.
+		 */
+		if (m && req != PRU_SENSE)
+			m_freem(m);
+#else
+		/* safer version of fix for mbuf leak */
+		if (m && (req == PRU_SEND || req == PRU_SENDOOB))
+			m_freem(m);
+#endif
 		return (EINVAL);		/* XXX */
 	}
 	if (inp) {
@@ -206,7 +218,7 @@ tcp_usrreq(so, req, m, nam, control)
 			error = EAFNOSUPPORT;
 			break;
 		}
-			
+
 		if ((error = tcp_connect(tp, nam)) != 0)
 			break;
 		error = tcp_output(tp);
@@ -338,6 +350,19 @@ tcp_usrreq(so, req, m, nam, control)
 		 * Otherwise, snd_up should be one lower.
 		 */
 		sbappend(&so->so_snd, m);
+		if (nam && tp->t_state < TCPS_SYN_SENT) {
+			/*
+			 * Do implied connect if not yet connected,
+			 * initialize window to default value, and
+			 * initialize maxseg/maxopd using peer's cached
+			 * MSS.
+			 */
+			error = tcp_connect(tp, nam);
+			if (error)
+				break;
+			tp->snd_wnd = TTCP_CLIENT_SND_WND;
+			tcp_mss(tp, -1);
+		}
 		tp->snd_up = tp->snd_una + so->so_snd.sb_cc;
 		tp->t_force = 1;
 		error = tcp_output(tp);
@@ -395,6 +420,8 @@ tcp_connect(tp, nam)
 	struct sockaddr_in *sin = mtod(nam, struct sockaddr_in *);
 	struct sockaddr_in *ifaddr;
 	int error;
+	struct rmxp_tao *taop;
+	struct rmxp_tao tao_noncached;
 
 	if (inp->inp_lport == 0) {
 		error = in_pcbbind(inp, NULL);
@@ -402,12 +429,14 @@ tcp_connect(tp, nam)
 			return error;
 	}
 
-	/* 
+	/*
 	 * Cannot simply call in_pcbconnect, because there might be an
 	 * earlier incarnation of this same connection still in
 	 * TIME_WAIT state, creating an ADDRINUSE error.
 	 */
 	error = in_pcbladdr(inp, nam, &ifaddr);
+	if (error)
+		return error;
 	oinp = in_pcblookup(inp->inp_pcbinfo->listhead,
 	    sin->sin_addr, sin->sin_port,
 	    inp->inp_laddr.s_addr != INADDR_ANY ? inp->inp_laddr
@@ -442,10 +471,27 @@ tcp_connect(tp, nam)
 	soisconnecting(so);
 	tcpstat.tcps_connattempt++;
 	tp->t_state = TCPS_SYN_SENT;
-	tp->t_timer[TCPT_KEEP] = TCPTV_KEEP_INIT;
+	tp->t_timer[TCPT_KEEP] = tcp_keepinit;
 	tp->iss = tcp_iss; tcp_iss += TCP_ISSINCR/2;
 	tcp_sendseqinit(tp);
+
+	/*
+	 * Generate a CC value for this connection and
+	 * check whether CC or CCnew should be used.
+	 */
+	if ((taop = tcp_gettaocache(tp->t_inpcb)) == NULL) {
+		taop = &tao_noncached;
+		bzero(taop, sizeof(*taop));
+	}
+
 	tp->cc_send = CC_INC(tcp_ccgen);
+	if (taop->tao_ccsent != 0 &&
+	    CC_GEQ(tp->cc_send, taop->tao_ccsent)) {
+		taop->tao_ccsent = tp->cc_send;
+	} else {
+		taop->tao_ccsent = 0;
+		tp->t_flags |= TF_SENDCCNEW;
+	}
 
 	return 0;
 }
@@ -660,8 +706,12 @@ tcp_usrclosed(tp)
 		tp->t_state = TCPS_LAST_ACK;
 		break;
 	}
-	if (tp && tp->t_state >= TCPS_FIN_WAIT_2)
+	if (tp && tp->t_state >= TCPS_FIN_WAIT_2) {
 		soisdisconnected(tp->t_inpcb->inp_socket);
+		/* To prevent the connection hanging in FIN_WAIT_2 forever. */
+		if (tp->t_state == TCPS_FIN_WAIT_2)
+			tp->t_timer[TCPT_2MSL] = tcp_maxidle;
+	}
 	return (tp);
 }
 
@@ -697,7 +747,7 @@ tcp_sysctl(name, namelen, oldp, oldlenp, newp, newlen)
 	case TCPCTL_RTTDFLT:
 		return (sysctl_int(oldp, oldlenp, newp, newlen, &tcp_rttdflt));
 	case TCPCTL_KEEPIDLE:
-		return (sysctl_int(oldp, oldlenp, newp, newlen, 
+		return (sysctl_int(oldp, oldlenp, newp, newlen,
 				   &tcp_keepidle));
 	case TCPCTL_KEEPINTVL:
 		return (sysctl_int(oldp, oldlenp, newp, newlen,
@@ -708,6 +758,9 @@ tcp_sysctl(name, namelen, oldp, oldlenp, newp, newlen)
 	case TCPCTL_RECVSPACE:
 		return (sysctl_int(oldp, oldlenp, newp, newlen,
 				   (int *)&tcp_recvspace)); /* XXX */
+	case TCPCTL_KEEPINIT:
+		return (sysctl_int(oldp, oldlenp, newp, newlen,
+				   &tcp_keepinit));
 	default:
 		return (ENOPROTOOPT);
 	}
