@@ -34,7 +34,7 @@
  * SUCH DAMAGE.
  *
  *	@(#)nfs_bio.c	8.5 (Berkeley) 1/4/94
- * $Id: nfs_bio.c,v 1.12 1995/04/16 05:05:25 davidg Exp $
+ * $FreeBSD: stable/2.1/sys/nfs/nfs_bio.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include <sys/param.h>
@@ -185,7 +185,7 @@ nfs_bioread(vp, uio, ioflag, cred)
 			error = nfs_readdirrpc(vp, uio, cred);
 			break;
 		default:
-			printf(" NQNFSNONCACHE: type %x unexpected\n",	
+			printf(" NQNFSNONCACHE: type %x unexpected\n",
 				vp->v_type);
 			break;
 		};
@@ -204,7 +204,7 @@ nfs_bioread(vp, uio, ioflag, cred)
 		if (nfs_numasync > 0 && nmp->nm_readahead > 0 &&
 		    lbn == vp->v_lastr + 1) {
 		    for (nra = 0; nra < nmp->nm_readahead &&
-			(lbn + 1 + nra) * biosize < np->n_size; nra++) {
+			(off_t)(lbn + 1 + nra) * biosize < np->n_size; nra++) {
 			rabn = lbn + 1 + nra;
 			if (!incore(vp, rabn)) {
 			    rabp = nfs_getcacheblk(vp, rabn, biosize, p);
@@ -233,7 +233,8 @@ nfs_bioread(vp, uio, ioflag, cred)
 		 */
 again:
 		bufsize = biosize;
-		if ((lbn + 1) * biosize > np->n_size) {
+		if ((off_t)(lbn + 1) * biosize > np->n_size &&
+		    (off_t)(lbn + 1) * biosize - np->n_size < biosize) {
 			bufsize = np->n_size - lbn * biosize;
 			bufsize = (bufsize + DEV_BSIZE - 1) & ~(DEV_BSIZE - 1);
 		}
@@ -242,6 +243,7 @@ again:
 			return (EINTR);
 		if ((bp->b_flags & B_CACHE) == 0) {
 			bp->b_flags |= B_READ;
+			bp->b_flags &= ~(B_DONE | B_ERROR | B_INVAL);
 			not_readin = 0;
 			vfs_busy_pages(bp, 0);
 			error = nfs_doio(bp, cred, p);
@@ -694,7 +696,7 @@ nfs_asyncio(bp, cred)
 				bp->b_wcred = cred;
 			}
 		}
-	
+
 		TAILQ_INSERT_TAIL(&nfs_bufq, bp, b_freelist);
 		nfs_iodwant[i] = (struct proc *)0;
 		wakeup((caddr_t)&nfs_iodwant[i]);
