@@ -34,7 +34,7 @@
  * SUCH DAMAGE.
  *
  *	@(#)nfs_serv.c	8.3 (Berkeley) 1/12/94
- * $Id: nfs_serv.c,v 1.12 1995/03/19 12:04:11 davidg Exp $
+ * $FreeBSD: stable/2.1/sys/nfs/nfs_serv.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -75,6 +75,7 @@
 #include <nfs/nfsm_subs.h>
 #include <nfs/nqnfs.h>
 
+#ifdef NFS_SERVER
 /* Defs */
 #define	TRUE	1
 #define	FALSE	0
@@ -86,6 +87,12 @@ extern u_long nfs_false, nfs_true;
 nfstype nfs_type[9] = { NFNON, NFREG, NFDIR, NFBLK, NFCHR, NFLNK, NFNON,
 		      NFCHR, NFNON };
 
+#ifdef NFS_ASYNC
+int nfs_async = 1;
+#else
+int nfs_async;
+#endif
+
 /*
  * nqnfs access service
  */
@@ -575,7 +582,7 @@ nfsrv_write(nfsd, mrep, md, dpos, cred, nam, mrq)
 	register long t1;
 	caddr_t bpos;
 	int error = 0, rdonly, cache, siz, len, xfer;
-	int ioflags = IO_SYNC | IO_NODELOCKED;
+	int ioflags;
 	char *cp2;
 	struct mbuf *mb, *mb2, *mreq;
 	struct vnode *vp;
@@ -585,6 +592,11 @@ nfsrv_write(nfsd, mrep, md, dpos, cred, nam, mrq)
 	off_t off;
 	u_quad_t frev;
 
+	if (nfs_async)
+		ioflags = IO_NODELOCKED;
+	else
+		ioflags = IO_SYNC | IO_NODELOCKED;
+
 	fhp = &nfh.fh_generic;
 	nfsm_srvmtofh(fhp);
 	nfsm_dissect(tl, u_long *, 4 * NFSX_UNSIGNED);
@@ -966,7 +978,7 @@ nfsrv_rename(nfsd, mrep, md, dpos, cred, nam, mrq)
 	fromnd.ni_cnd.cn_cred = cred;
 	fromnd.ni_cnd.cn_nameiop = DELETE;
 	fromnd.ni_cnd.cn_flags = WANTPARENT | SAVESTART;
-	error = nfs_namei(&fromnd, ffhp, len, nfsd->nd_slp, nam, &md, &dpos, 
+	error = nfs_namei(&fromnd, ffhp, len, nfsd->nd_slp, nam, &md, &dpos,
 		nfsd->nd_procp);
 	if(error)
 		nfsm_reply(0);
@@ -1029,6 +1041,7 @@ out:
 			nqsrv_getl(tvp, NQL_WRITE);
 			if ((tvp->v_flag & VVMIO) && tvp->v_vmdata)
 				deallocobjto = 1;
+			(void) vnode_pager_uncache(tvp);
 		}
 		if ((fvp->v_flag & VVMIO) && fvp->v_vmdata)
 			deallocobjfrom = 1;
@@ -1038,7 +1051,7 @@ out:
 			vm_object_deallocate((vm_object_t) fvp->v_vmdata);
 		if (deallocobjto)
 			vm_object_deallocate((vm_object_t) tvp->v_vmdata);
-		
+
 	} else {
 		VOP_ABORTOP(tond.ni_dvp, &tond.ni_cnd);
 		if (tdvp == tvp)
@@ -1124,14 +1137,9 @@ nfsrv_link(nfsd, mrep, md, dpos, cred, nam, mrq)
 		error = EXDEV;
 out:
 	if (!error) {
-		int deallocobj = 0;
 		nqsrv_getl(vp, NQL_WRITE);
 		nqsrv_getl(xp, NQL_WRITE);
-		if ((vp->v_flag & VVMIO) && vp->v_vmdata)
-			deallocobj = 1;
 		error = VOP_LINK(nd.ni_dvp, vp, &nd.ni_cnd);
-		if (error == 0 && deallocobj)
-			vm_object_deallocate((vm_object_t) vp->v_vmdata);
 	} else {
 		VOP_ABORTOP(nd.ni_dvp, &nd.ni_cnd);
 		if (nd.ni_dvp == nd.ni_vp)
@@ -1574,7 +1582,7 @@ again:
 			nfsm_clget;
 			*tl = txdr_unsigned(nlen);
 			bp += NFSX_UNSIGNED;
-	
+
 			/* And loop around copying the name */
 			xfer = nlen;
 			cp = dp->d_name;
@@ -1594,7 +1602,7 @@ again:
 			for (i = 0; i < rem; i++)
 				*bp++ = '\0';
 			nfsm_clget;
-	
+
 			/* Finish off the record */
 			*tl = txdr_unsigned(*cookiep);
 			bp += NFSX_UNSIGNED;
@@ -1761,7 +1769,7 @@ again:
 		if (dp->d_fileno != 0) {
 			nlen = dp->d_namlen;
 			rem = nfsm_rndup(nlen)-nlen;
-	
+
 			/*
 			 * For readdir_and_lookup get the vnode using
 			 * the file number.
@@ -1827,7 +1835,7 @@ again:
 			nfsm_clget;
 			*tl = txdr_unsigned(nlen);
 			bp += NFSX_UNSIGNED;
-	
+
 			/* And loop around copying the name */
 			xfer = nlen;
 			cp = dp->d_name;
@@ -1847,7 +1855,7 @@ again:
 			for (i = 0; i < rem; i++)
 				*bp++ = '\0';
 			nfsm_clget;
-	
+
 			/* Finish off the record */
 			*tl = txdr_unsigned(*cookiep);
 			bp += NFSX_UNSIGNED;
@@ -2021,3 +2029,4 @@ nfsrv_access(vp, flags, cred, rdonly, p)
 		return (error);
 	return (0);
 }
+#endif /* NFS_SERVER */
