@@ -34,7 +34,7 @@
  * SUCH DAMAGE.
  *
  *	@(#)nfs_socket.c	8.3 (Berkeley) 1/12/94
- * $Id: nfs_socket.c,v 1.5 1994/10/17 17:47:35 phk Exp $
+ * $FreeBSD: stable/2.1/sys/nfs/nfs_socket.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -159,6 +159,7 @@ void	nfs_rcvunlock(), nqnfs_serverd(), nqnfs_clientlease();
 struct mbuf *nfsm_rpchead();
 int nfsrtton = 0;
 struct nfsrtt nfsrtt;
+#ifdef NFS_SERVER
 
 int	nfsrv_null(),
 	nfsrv_getattr(),
@@ -208,6 +209,8 @@ int (*nfsrv_procs[NFS_NPROCS])() = {
 	nqnfsrv_access,
 };
 
+#endif /* NFS_SERVER */
+
 /*
  * Initialize sockets and congestion for a new NFS connection.
  * We do not free the sockaddr if error.
@@ -226,7 +229,7 @@ nfs_connect(nmp, rep)
 
 	nmp->nm_so = (struct socket *)0;
 	saddr = mtod(nmp->nm_nam, struct sockaddr *);
-	error = socreate(saddr->sa_family, &nmp->nm_so, nmp->nm_sotype, 
+	error = socreate(saddr->sa_family, &nmp->nm_so, nmp->nm_sotype,
 		nmp->nm_soproto);
 	if (error)
 		goto bad;
@@ -529,7 +532,7 @@ tryagain:
 		}
 		so = rep->r_nmp->nm_so;
 		if (!so) {
-			error = nfs_reconnect(rep); 
+			error = nfs_reconnect(rep);
 			if (error) {
 				nfs_sndunlock(&rep->r_nmp->nm_flag);
 				return (error);
@@ -739,7 +742,7 @@ nfs_reply(myrep)
 		}
 		if (nam)
 			m_freem(nam);
-	
+
 		/*
 		 * Get the xid and check that it is an rpc reply
 		 */
@@ -867,7 +870,7 @@ nfs_request(vp, mrest, procnum, procp, cred, mrp, mdp, dposp)
 	struct mbuf **mdp;
 	caddr_t *dposp;
 {
-	register struct mbuf *m, *mrep;
+	register struct mbuf *m, *mrep, *m2;
 	register struct nfsreq *rep;
 	register u_long *tl;
 	register int i;
@@ -975,8 +978,8 @@ tryagain:
 		if (nmp->nm_soflags & PR_CONNREQUIRED)
 			error = nfs_sndlock(&nmp->nm_flag, rep);
 		if (!error) {
-			m = m_copym(m, 0, M_COPYALL, M_WAIT);
-			error = nfs_send(nmp->nm_so, nmp->nm_nam, m, rep);
+			m2 = m_copym(m, 0, M_COPYALL, M_WAIT);
+			error = nfs_send(nmp->nm_so, nmp->nm_nam, m2, rep);
 			if (nmp->nm_soflags & PR_CONNREQUIRED)
 				nfs_sndunlock(&nmp->nm_flag);
 		}
@@ -1225,6 +1228,8 @@ nfs_rephead(siz, nd, err, cache, frev, mrq, mbp, bposp)
 	return (0);
 }
 
+#ifdef NFS_SERVER
+
 /*
  * Nfs timer routine
  * Scan the nfsreq list and retranmit any requests that have timed out
@@ -1342,6 +1347,8 @@ nfs_timer(arg)
 	timeout(nfs_timer, (void *)0, hz / NFS_HZ);
 }
 
+#endif /* NFS_SERVER */
+
 /*
  * Test for a termination condition pending on the process.
  * This is used for NFSMNT_INT mounts.
@@ -1493,7 +1500,7 @@ nfs_realign(m, hsiz)
 		tcp = mtod(m, caddr_t);
 		mnew = m;
 		m2 = m->m_next;
-	
+
 		/*
 		 * If possible, only put the first invariant part
 		 * of the RPC header in the first mbuf.
@@ -1501,7 +1508,7 @@ nfs_realign(m, hsiz)
 		mlen = M_TRAILINGSPACE(m);
 		if (olen <= hsiz && mlen > hsiz)
 			mlen = hsiz;
-	
+
 		/*
 		 * Loop through the mbuf list consolidating data.
 		 */
@@ -1534,7 +1541,7 @@ nfs_realign(m, hsiz)
 				fcp = mtod(m, caddr_t);
 			}
 		}
-	
+
 		/*
 		 * Finally, set m_len == 0 for any trailing mbufs that have
 		 * been copied out of.
@@ -1549,6 +1556,7 @@ nfs_realign(m, hsiz)
 	}
 }
 
+#ifdef NFS_SERVER
 /*
  * Socket upcall routine for the nfsd sockets.
  * The caddr_t arg is a pointer to the "struct nfssvc_sock".
@@ -1806,6 +1814,7 @@ nfsrv_dorec(slp, nd)
 	return (0);
 }
 
+#endif /* NFS_SERVER */
 /*
  * Parse an RPC request
  * - verify it
@@ -1960,6 +1969,8 @@ nfsmout:
 	return (error);
 }
 
+#ifdef NFS_SERVER
+
 /*
  * Search for a sleeping nfsd and wake it up.
  * SIDE EFFECT: If none found, set NFSD_CHECKSLP flag, so that one of the
@@ -1988,6 +1999,8 @@ nfsrv_wakenfsd(slp)
 	nfsd_head_flag |= NFSD_CHECKSLP;
 }
 
+#endif /* NFS_SERVER */
+
 int
 nfs_msg(p, server, msg)
 	struct proc *p;
