@@ -34,7 +34,7 @@
  * SUCH DAMAGE.
  *
  *	@(#)nfs_subs.c	8.3 (Berkeley) 1/4/94
- * $Id: nfs_subs.c,v 1.13 1995/04/21 02:58:49 dyson Exp $
+ * $FreeBSD: stable/2.1/sys/nfs/nfs_subs.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -137,7 +137,7 @@ nfsm_reqh(vp, procid, hsiz, bposp)
 		MCLGET(mb, M_WAIT);
 	mb->m_len = 0;
 	bpos = mtod(mb, caddr_t);
-	
+
 	/*
 	 * For NQNFS, add lease request.
 	 */
@@ -627,6 +627,7 @@ nfs_init()
 		nfs_iodwant[i] = (struct proc *)0;
 	TAILQ_INIT(&nfs_bufq);
 	nfs_nhinit();			/* Init the nfsnode table */
+#ifdef NFS_SERVER
 	nfsrv_init(0);			/* Init server data structures */
 	nfsrv_initcache();		/* Init the server request cache */
 
@@ -655,6 +656,7 @@ nfs_init()
 	 */
 	lease_check = nfs_lease_check;
 	lease_updatetime = nfs_lease_updatetime;
+#endif /* NFS_SERVER */
 	vfsconf[MOUNT_NFS]->vfc_refcount++; /* make us non-unloadable */
 #ifdef VFS_LKM
 	sysent[SYS_nfssvc].sy_narg = 2;
@@ -712,7 +714,26 @@ nfs_loadattrcache(vpp, mdp, dposp, vaper)
 	fp = (struct nfsv2_fattr *)cp2;
 	vtyp = nfstov_type(fp->fa_type);
 	vmode = fxdr_unsigned(u_short, fp->fa_mode);
-	if (vtyp == VNON || vtyp == VREG)
+	/*
+	 * XXX
+	 *
+	 * The duplicate information returned in fa_type and fa_mode
+	 * is an ambiguity in the NFS version 2 protocol.
+	 *
+	 * VREG should be taken literally as a regular file.  If a
+	 * server intents to return some type information differently
+	 * in the upper bits of the mode field (e.g. for sockets, or
+	 * FIFOs), NFSv2 mandates fa_type to be VNON.  Anyway, we
+	 * leave the examination of the mode bits even in the VREG
+	 * case to avoid breakage for bogus servers, but we make sure
+	 * that there are actually type bits set in the upper part of
+	 * fa_mode (and failing that, trust the va_type field).
+	 *
+	 * NFSv3 cleared the issue, and requires fa_mode to not
+	 * contain any type information (while also introduing sockets
+	 * and FIFOs for fa_type).
+	 */
+	if (vtyp == VNON || (vtyp == VREG && (vmode & S_IFMT) != 0))
 		vtyp = IFTOVT(vmode);
 	if (isnq) {
 		rdev = fxdr_unsigned(long, fp->fa_nqrdev);
@@ -723,7 +744,7 @@ nfs_loadattrcache(vpp, mdp, dposp, vaper)
 	}
 	/*
 	 * If v_type == VNON it is a new node, so fill in the v_type,
-	 * n_mtime fields. Check to see if it represents a special 
+	 * n_mtime fields. Check to see if it represents a special
 	 * device, and if so, check for a possible alias. Once the
 	 * correct vnode has been obtained, fill in the rest of the
 	 * information.
@@ -783,7 +804,8 @@ nfs_loadattrcache(vpp, mdp, dposp, vaper)
 	} else {
 		vap->va_size = fxdr_unsigned(u_long, fp->fa_nfssize);
 		vap->va_blocksize = fxdr_unsigned(long, fp->fa_nfsblocksize);
-		vap->va_bytes = fxdr_unsigned(long, fp->fa_nfsblocks) * NFS_FABLKSIZE;
+		vap->va_bytes = (u_quad_t)fxdr_unsigned(long, fp->fa_nfsblocks)
+		    * NFS_FABLKSIZE;
 		vap->va_fileid = fxdr_unsigned(long, fp->fa_nfsfileid);
 		fxdr_nfstime(&fp->fa_nfsatime, &vap->va_atime);
 		vap->va_flags = 0;
@@ -889,6 +911,7 @@ nfs_getattrcache(vp, vaper)
 	return (0);
 }
 
+#ifdef NFS_SERVER 
 /*
  * Set up nameidata for a lookup() call and do it
  */
@@ -1005,6 +1028,8 @@ out:
 	return (error);
 }
 
+#endif /* NFS_SERVER */
+
 /*
  * A fiddled version of m_adj() that ensures null fill to a long
  * boundary and only trims off the back end
@@ -1067,6 +1092,7 @@ nfsm_adj(mp, len, nul)
 		m->m_len = 0;
 }
 
+#ifdef NFS_SERVER
 /*
  * nfsrv_fhtovp() - convert a fh to a vnode ptr (optionally locked)
  * 	- look up fsid in mount list (if not found ret error)
@@ -1132,6 +1158,8 @@ nfsrv_fhtovp(fhp, lockflag, vpp, cred, slp, nam, rdonlyp)
 	return (0);
 }
 
+#endif /* NFS_SERVER */
+
 /*
  * This function compares two net addresses by family and returns TRUE
  * if they are the same host.
@@ -1175,6 +1203,8 @@ netaddr_match(family, haddr, nam)
 	return (0);
 }
 
+#ifdef NFS_SERVER
+
 int
 nfsrv_vmio( struct vnode *vp) {
 	vm_object_t object;
@@ -1184,18 +1214,20 @@ nfsrv_vmio( struct vnode *vp) {
 		return 1;
 
 retry:
-	if( (vp->v_flag & VVMIO) == 0) {
-		pager = (vm_pager_t) vnode_pager_alloc((caddr_t) vp, 0, 0, 0);
-		object = (vm_object_t) vp->v_vmdata;
-		if( object->pager != pager)
-			panic("nfsrv_vmio: pager/object mismatch");
-		(void) vm_object_lookup( pager);
-		pager_cache( object, TRUE);
+	if ((vp->v_flag & VVMIO) == 0) {
+		struct vattr vat;
+		struct proc *p = curproc;
+
+		if (VOP_GETATTR(vp, &vat, p->p_ucred, p) != 0)
+			panic("nfsrv_vmio: VOP_GETATTR failed");
+
+		(void) vnode_pager_alloc(vp, vat.va_size, 0, 0);
+
 		vp->v_flag |= VVMIO;
 	} else {
-		if( (object = (vm_object_t)vp->v_vmdata) &&
+		if ((object = (vm_object_t)vp->v_vmdata) &&
 			(object->flags & OBJ_DEAD)) {
-			tsleep( (caddr_t) object, PVM, "nfdead", 0);
+			tsleep(object, PVM, "nfdead", 0);
 			goto retry;
 		}
 		if( !object)
@@ -1227,3 +1259,5 @@ nfsrv_vrele( struct vnode *vp) {
 	}
 	return 0;
 }
+
+#endif /* NFS_SERVER */
