@@ -34,7 +34,7 @@
  * SUCH DAMAGE.
  *
  *	@(#)nfs_vfsops.c	8.3 (Berkeley) 1/4/94
- * $Id: nfs_vfsops.c,v 1.12 1995/03/16 20:23:46 wollman Exp $
+ * $FreeBSD: stable/2.1/sys/nfs/nfs_vfsops.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include <sys/param.h>
@@ -72,6 +72,54 @@ struct nfsstats nfsstats;
 static int nfs_sysctl(int *, u_int, void *, size_t *, void *, size_t,
 		      struct proc *);
 
+/*
+ * Prototypes for NFS mount operations
+ */
+static int	nfs_mount __P((
+		struct mount *mp,
+		char *path,
+		caddr_t data,
+		struct nameidata *ndp,
+		struct proc *p));
+static int	nfs_start __P((
+		struct mount *mp,
+		int flags,
+		struct proc *p));
+static int	nfs_unmount __P((
+		struct mount *mp,
+		int mntflags,
+		struct proc *p));
+static int	nfs_root __P((
+		struct mount *mp,
+		struct vnode **vpp));
+static int	nfs_quotactl __P((
+		struct mount *mp,
+		int cmds,
+		uid_t uid,
+		caddr_t arg,
+		struct proc *p));
+static int	nfs_statfs __P((
+		struct mount *mp,
+		struct statfs *sbp,
+		struct proc *p));
+static int	nfs_sync __P((
+		struct mount *mp,
+		int waitfor,
+		struct ucred *cred,
+		struct proc *p));
+static int	nfs_fhtovp __P((
+		struct mount *mp,
+		struct fid *fhp,
+		struct mbuf *nam,
+		struct vnode **vpp,
+		int *exflagsp,
+		struct ucred **credanonp));
+static int	nfs_vptofh __P((
+		struct vnode *vp,
+		struct fid *fhp));
+
+static int	nfs_vget __P((struct mount *, ino_t, struct vnode **));
+
 /*
  * nfs vfs operations.
  */
@@ -198,7 +246,7 @@ nfs_mountroot()
 	if (time.tv_sec == 0)
 		time.tv_sec = 1;
 
-	/* 
+	/*
 	 * XXX splnet, so networks will receive...
 	 */
 	splnet();
@@ -230,11 +278,11 @@ nfs_mountroot()
 	 */
 
 	for (i = strlen(nd->myif.ifra_name) - 1;
-		nd->myif.ifra_name[i] >= '0' && 
+		nd->myif.ifra_name[i] >= '0' &&
 		nd->myif.ifra_name[i] <= '9';
 		nd->myif.ifra_name[i] ++) {
 		error = ifioctl(so, SIOCAIFADDR, (caddr_t)&nd->myif, p);
-		if(!error) 
+		if(!error)
 			break;
 	}
 	if (error)
@@ -260,6 +308,10 @@ nfs_mountroot()
 	}
 
 	if (nd->swap_nblks) {
+
+		/* Convert to DEV_BSIZE instead of Kilobyte */
+		nd->swap_nblks *= 2;
+
 		/*
 		 * Create a fake mount point just for the swap vnode so that the
 		 * swap file can be on a different server from the rootfs.
@@ -273,6 +325,9 @@ nfs_mountroot()
 		(void) nfs_mountdiskless(buf, "/swap", 0,
 		    &nd->swap_saddr, &nd->swap_args, &vp);
 
+		VTONFS(vp)->n_size = VTONFS(vp)->n_vattr.va_size = 
+				nd->swap_nblks * DEV_BSIZE ;
+		
 		/*
 		 * Since the swap file is not the root dir of a file system,
 		 * hack it to a regular file.
@@ -280,8 +335,8 @@ nfs_mountroot()
 		vp->v_type = VREG;
 		vp->v_flag = 0;
 		VREF(vp);
-		swaponvp(p, vp, NODEV, nd->swap_nblks * 2);
-	} 
+		swaponvp(p, vp, NODEV, nd->swap_nblks);
+	}
 
 	/*
 	 * Create the rootfs mount point.
