@@ -21,36 +21,51 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $Id: dc21040.h,v 1.3 1995/05/05 19:44:34 thomas Exp $
- *
- * $Log: dc21040.h,v $
- * Revision 1.3  1995/05/05  19:44:34  thomas
- * cogent em100 support
- *
- * Revision 1.1  1994/10/01  20:16:45  wollman
- * Add Matt Thomas's DC21040 PCI Ethernet driver.  (This is turning out
- * to be quite a popular chip, so expect to see a number of products
- * based on it.)
- *
- * Revision 1.2  1994/08/15  20:42:25  thomas
- * misc additions
- *
- * Revision 1.1  1994/08/12  21:02:46  thomas
- * Initial revision
- *
- * Revision 1.8  1994/08/05  20:20:54  thomas
- * Enable change log
- *
- * Revision 1.7  1994/08/05  20:20:14  thomas
- * *** empty log message ***
+ * $FreeBSD: stable/2.1/sys/pci/dc21040.h 50950 1999-09-05 08:39:22Z peter $
  *
  */
 
 #if !defined(_DC21040_H)
 #define _DC21040_H
 
-typedef	  signed int tulip_sint32_t;
-typedef	unsigned int tulip_uint32_t;
+typedef unsigned char  tulip_uint8_t;
+typedef	unsigned short tulip_uint16_t;
+typedef	unsigned int   tulip_uint32_t;
+
+/*
+ * The various controllers support.  Technically the DE425 is just
+ * a 21040 on EISA.  But since it remarkably difference from normal
+ * 21040s, we give it its own chip id.
+ */
+
+typedef enum {
+    TULIP_DC21040, TULIP_DE425,
+    TULIP_DC21041,
+    TULIP_DC21140, TULIP_DC21140A, TULIP_DC21142,
+    TULIP_DC21143,
+    TULIP_CHIPID_UNKNOWN
+} tulip_chipid_t;
+
+/*
+ * Various physical media types supported.
+ * BNCAUI is BNC or AUI since on the 21040 you can't really tell
+ * which is in use.
+ */
+typedef enum {
+    TULIP_MEDIA_UNKNOWN,
+    TULIP_MEDIA_10BASET,
+    TULIP_MEDIA_BNC,
+    TULIP_MEDIA_AUI,
+    TULIP_MEDIA_BNCAUI,
+    TULIP_MEDIA_10BASET_FD,
+    TULIP_MEDIA_100BASETX,
+    TULIP_MEDIA_100BASETX_FD,
+    TULIP_MEDIA_100BASET4,
+    TULIP_MEDIA_100BASEFX,
+    TULIP_MEDIA_100BASEFX_FD,
+    TULIP_MEDIA_MAX
+} tulip_media_t;
+
 
 #if defined(BYTE_ORDER) && BYTE_ORDER == BIG_ENDIAN
 #define	TULIP_BITFIELD2(a, b)		      b, a
@@ -145,6 +160,7 @@ typedef struct {
 #define	TULIP_STS_ERR_PARITY	0x00000000L		/*        000 - Parity Error (Perform Reset) */
 #define	TULIP_STS_ERR_MASTER	0x00800000L		/*        001 - Master Abort */
 #define	TULIP_STS_ERR_TARGET	0x01000000L		/*        010 - Target Abort */
+#define	TULIP_STS_ERR_SHIFT	23
 #define	TULIP_STS_TXSTATEMASK	0x00700000L		/* ( R)  Transmission Process State */
 #define	TULIP_STS_TXS_RESET	0x00000000L		/*        000 - Rset or transmit jabber expired */
 #define	TULIP_STS_TXS_FETCH	0x00100000L		/*        001 - Fetching transmit descriptor */
@@ -182,6 +198,7 @@ typedef struct {
 #define	TULIP_STS_RXNOBUF	0x00000080L		/* (RW)  Receive Buffer Unavailable */
 #define	TULIP_STS_RXINTR	0x00000040L		/* (RW)  Receive Interrupt */
 #define	TULIP_STS_TXUNDERFLOW	0x00000020L		/* (RW)  Transmit Underflow */
+#define	TULIP_STS_LINKPASS	0x00000010L		/* (RW)  LinkPass (DC21041) */
 #define	TULIP_STS_TXBABBLE	0x00000008L		/* (RW)  Transmit Jabber Timeout */
 #define	TULIP_STS_TXNOBUF	0x00000004L		/* (RW)  Transmit Buffer Unavailable */
 #define	TULIP_STS_TXSTOPPED	0x00000002L		/* (RW)  Transmit Process Stopped */
@@ -197,6 +214,7 @@ typedef struct {
 #define	TULIP_CMD_STOREFWD	0x00200000L		/* (RW)  Store and Foward (DC21140) */
 #define	TULIP_CMD_NOHEARTBEAT	0x00080000L		/* (RW)  No Heartbeat (DC21140) */
 #define	TULIP_CMD_PORTSELECT	0x00040000L		/* (RW)  Post Select (100Mb) (DC21140) */
+#define	TULIP_CMD_ENHCAPTEFFCT	0x00040000L		/* (RW)  Enhanced Capture Effecty (DC21041) */
 #define	TULIP_CMD_CAPTREFFCT	0x00020000L		/* (RW)  Capture Effect (!802.3) */
 #define	TULIP_CMD_BACKPRESSURE	0x00010000L		/* (RW)  Back Pressure (!802.3) (DC21040) */
 #define	TULIP_CMD_THRESHOLDCTL	0x0000C000L		/* (RW)  Threshold Control */
@@ -207,7 +225,7 @@ typedef struct {
 #define	TULIP_CMD_TXRUN 	0x00002000L		/* (RW)  Start/Stop Transmitter */
 #define	TULIP_CMD_FORCECOLL	0x00001000L		/* (RW)  Force Collisions */
 #define	TULIP_CMD_OPERMODE	0x00000C00L		/* (RW)  Operating Mode */
-#define	TULIP_CMD_FULLDULPEX	0x00000200L		/* (RW)  Full Duplex Mode */
+#define	TULIP_CMD_FULLDUPLEX	0x00000200L		/* (RW)  Full Duplex Mode */
 #define	TULIP_CMD_FLAKYOSCDIS	0x00000100L		/* (RW)  Flakey Oscillator Disable */
 #define	TULIP_CMD_ALLMULTI	0x00000080L		/* (RW)  Pass All Multicasts */
 #define	TULIP_CMD_PROMISCUOUS	0x00000040L		/* (RW)  Promiscuous Mode */
@@ -218,13 +236,30 @@ typedef struct {
 #define	TULIP_CMD_RXRUN		0x00000002L		/* (RW)  Start/Stop Receive Filtering */
 #define	TULIP_CMD_HASHPRFCTFLTR	0x00000001L		/* (R )  Hash/Perfect Receive Filtering */
 
-
+#define TULIP_SIASTS_OTHERRXACTIVITY	0x00000200L
+#define TULIP_SIASTS_RXACTIVITY		0x00000100L
 #define	TULIP_SIASTS_LINKFAIL		0x00000004L
 #define	TULIP_SIACONN_RESET		0x00000000L
 
 #define	TULIP_SIACONN_AUI		0x0000000DL
 #define	TULIP_SIACONN_10BASET		0x00000005L
 
+#define	TULIP_DC21041_SIACONN_10BASET	0x0000EF01L
+#define	TULIP_DC21041_SIATXRX_10BASET	0x0000FF3FL
+#define	TULIP_DC21041_SIATXRX_10BASET_FD 0x0000FF3DL
+#define	TULIP_DC21041_SIAGEN_10BASET	0x00000000L
+
+#define	TULIP_DC21041_SIACONN_AUI	0x0000EF09L
+#define	TULIP_DC21041_SIATXRX_AUI	0x0000F73DL
+#define	TULIP_DC21041_SIAGEN_AUI	0x0000000EL
+
+#define	TULIP_DC21041_SIACONN_BNC	0x0000EF09L
+#define	TULIP_DC21041_SIATXRX_BNC	0x0000F73DL
+#define	TULIP_DC21041_SIAGEN_BNC	0x00000006L
+
+#define	TULIP_WATCHDOG_TXDISABLE	0x00000001L
+#define	TULIP_WATCHDOG_RXDISABLE	0x00000010L
+
 #define	TULIP_BUSMODE_SWRESET		0x00000001L
 #define	TULIP_BUSMODE_DESCSKIPLEN_MASK	0x0000007CL
 #define	TULIP_BUSMODE_BIGENDIAN		0x00000080L
@@ -241,43 +276,89 @@ typedef struct {
 #define	TULIP_BUSMODE_CACHE_ALIGN16	0x00008000L
 #define	TULIP_BUSMODE_CACHE_ALIGN32	0x0000C000L
 #define	TULIP_BUSMODE_TXPOLL_NEVER	0x00000000L
-#define	TULIP_BUSMODE_TXPOLL_200us	0x00020000L
-#define	TULIP_BUSMODE_TXPOLL_800us	0x00040000L
-#define	TULIP_BUSMODE_TXPOLL_1600us	0x00060000L
+#define	TULIP_BUSMODE_TXPOLL_200000ns	0x00020000L
+#define	TULIP_BUSMODE_TXPOLL_800000ns	0x00040000L
+#define	TULIP_BUSMODE_TXPOLL_1600000ns	0x00060000L
+#define	TULIP_BUSMODE_TXPOLL_12800ns	0x00080000L	/* DC21041 only */
+#define	TULIP_BUSMODE_TXPOLL_25600ns	0x000A0000L	/* DC21041 only */
+#define	TULIP_BUSMODE_TXPOLL_51200ns	0x000C0000L	/* DC21041 only */
+#define	TULIP_BUSMODE_TXPOLL_102400ns	0x000E0000L	/* DC21041 only */
+#define	TULIP_BUSMODE_DESC_BIGENDIAN	0x00100000L	/* DC21041 only */
 
+#define	TULIP_REG_CFDA			0x40
+#define	TULIP_CFDA_SLEEP		0x80000000L
+#define	TULIP_CFDA_SNOOZE		0x40000000L
 
 /*
  * These are the defintitions used for the DEC DC21140
  * evaluation board.
  */
-#define	TULIP_GP_EB_PINS	0x0000011F	/* General Purpose Pin directions */
-#define	TULIP_GP_EB_OK10	0x00000080	/* 10 Mb/sec Signal Detect gep<7> */
-#define	TULIP_GP_EB_OK100	0x00000040	/* 100 Mb/sec Signal Detect gep<6> */
-#define	TULIP_GP_EB_INIT	0x0000000B	/* No loopback --- point-to-point */
+#define	TULIP_GP_EB_PINS		0x0000011F	/* General Purpose Pin directions */
+#define	TULIP_GP_EB_OK10		0x00000080	/* 10 Mb/sec Signal Detect gep<7> */
+#define	TULIP_GP_EB_OK100		0x00000040	/* 100 Mb/sec Signal Detect gep<6> */
+#define	TULIP_GP_EB_INIT		0x0000000B	/* No loopback --- point-to-point */
 
 /*
- * There are the definitions used for the DEC DE500-XA
- * 10/100 board
+ * These are the defintitions used for the SMC9332 (DC21140) board.
  */
-#define	TULIP_GP_DE500_PINS		0x0000010FL
-#define	TULIP_GP_DE500_NOTOK_10		0x00000080L
-#define	TULIP_GP_DE500_NOTOK_100	0x00000040L
+#define	TULIP_GP_SMC_9332_PINS		0x0000013F	/* General Purpose Pin directions */
+#define	TULIP_GP_SMC_9332_OK10		0x00000080	/* 10 Mb/sec Signal Detect gep<7> */
+#define	TULIP_GP_SMC_9332_OK100		0x00000040	/* 100 Mb/sec Signal Detect gep<6> */
+#define	TULIP_GP_SMC_9332_INIT		0x00000009	/* No loopback --- point-to-point */
+
+#define	TULIP_OUI_SMC_0			0x00
+#define	TULIP_OUI_SMC_1			0x00
+#define	TULIP_OUI_SMC_2			0xC0
+
+/*
+ * There are the definitions used for the DEC DE500
+ * 10/100 family of boards
+ */
+#define	TULIP_GP_DE500_PINS		0x0000011FL
+#define	TULIP_GP_DE500_LINK_PASS	0x00000080L
+#define	TULIP_GP_DE500_SYM_LINK		0x00000040L
+#define	TULIP_GP_DE500_SIGNAL_DETECT	0x00000020L
+#define	TULIP_GP_DE500_PHY_RESET	0x00000010L
 #define	TULIP_GP_DE500_HALFDUPLEX	0x00000008L
+#define	TULIP_GP_DE500_PHY_LOOPBACK	0x00000004L
+#define	TULIP_GP_DE500_FORCE_LED	0x00000002L
 #define	TULIP_GP_DE500_FORCE_100	0x00000001L
 
 /*
  * These are the defintitions used for the Cogent EM100
  * DC21140 board.
  */
-#define	TULIP_GP_EM100_PINS	0x0000013F	/* General Purpose Pin directions */
-#define	TULIP_GP_EM100_INIT	0x00000009	/* No loopback --- point-to-point */
-#define	TULIP_OUI_COGENT_0	0x00
-#define	TULIP_OUI_COGENT_1	0x00
-#define	TULIP_OUI_COGENT_2	0x94
-#define	TULIP_COGENT_EM100_ID	0x12
+#define	TULIP_GP_EM100_PINS		0x0000013F	/* General Purpose Pin directions */
+#define	TULIP_GP_EM100_INIT		0x00000009	/* No loopback --- point-to-point */
+#define	TULIP_OUI_COGENT_0		0x00
+#define	TULIP_OUI_COGENT_1		0x00
+#define	TULIP_OUI_COGENT_2		0x92
+#define	TULIP_COGENT_EM100_ID		0x12
+
+
+/*
+ * These are the defintitions used for the Znyx ZX342
+ * 10/100 board
+ */
+#define	TULIP_GP_ZX34X_PINS		0x0000011F	/* General Purpose Pin directions */
+#define	TULIP_GP_ZX34X_OK10		0x00000080	/* 10 Mb/sec Signal Detect gep<7> */
+#define	TULIP_GP_ZX34X_OK100		0x00000040	/* 100 Mb/sec Signal Detect gep<6> */
+#define	TULIP_GP_ZX34X_INIT		0x00000009	
+#define	TULIP_OUI_ZNYX_0		0x00
+#define	TULIP_OUI_ZNYX_1		0xC0
+#define	TULIP_OUI_ZNYX_2		0x95
+
+/*
+ * Compex's OUI.  We need to twiddle a bit on their DC21041 card.
+ */
+#define	TULIP_OUI_COMPEX_0		0x00
+#define	TULIP_OUI_COMPEX_1		0x80
+#define	TULIP_OUI_COMPEX_2		0x48
+#define	TULIP_DC21041_COMPEX_XREGDATA	1
 /*
  * SROM definitions for the DC21140 and DC21041.
  */
+#define	SROMXREG	0x0400
 #define SROMSEL         0x0800
 #define SROMRD          0x4000
 #define SROMWR          0x2000
@@ -296,4 +377,138 @@ typedef struct {
 #define	SROMCMD_RD	6
 
 #define	SROM_BITWIDTH	6
+
+/*
+ * MII Definitions for the DC21041 and DC21140/DC21140A/DC21142
+ */
+#define	MII_PREAMBLE		(~0)
+#define	MII_TEST		0xAAAAAAAA
+#define	MII_RDCMD		0xF6		/* 1111.0110 */
+#define	MII_WRCMD		0xF5		/* 1111.0101 */
+#define	MII_DIN			0x00080000
+#define	MII_RD			0x00040000
+#define	MII_WR			0x00000000
+#define	MII_DOUT		0x00020000
+#define	MII_CLK			0x00010000
+#define	MII_CLKON		MII_CLK
+#define	MII_CLKOFF		MII_CLK
+
+#define	PHYREG_CONTROL			0
+#define	PHYREG_STATUS			1
+#define	PHYREG_IDLOW			2
+#define	PHYREG_IDHIGH			3
+#define	PHYREG_AUTONEG_ADVERTISEMENT	4
+#define	PHYREG_AUTONEG_ABILITIES	5
+#define	PHYREG_AUTONEG_EXPANSION	6
+#define	PHYREG_AUTONEG_NEXTPAGE		7
+
+#define	PHYSTS_100BASET4	0x8000
+#define	PHYSTS_100BASETX_FD	0x4000
+#define	PHYSTS_100BASETX	0x2000
+#define	PHYSTS_10BASET_FD	0x1000
+#define	PHYSTS_10BASET		0x0800
+#define	PHYSTS_AUTONEG_DONE	0x0020
+#define	PHYSTS_REMOTE_FAULT	0x0010
+#define	PHYSTS_CAN_AUTONEG	0x0008
+#define	PHYSTS_LINK_UP		0x0004
+#define	PHYSTS_JABBER_DETECT	0x0002
+#define	PHYSTS_EXTENDED_REGS	0x0001
+
+#define	PHYCTL_RESET		0x8000
+#define	PHYCTL_SELECT_100MB	0x2000
+#define	PHYCTL_AUTONEG_ENABLE	0x1000
+#define	PHYCTL_ISOLATE		0x0400
+#define	PHYCTL_AUTONEG_RESTART	0x0200
+#define	PHYCTL_FULL_DUPLEX	0x0100
+
+
+#define MII_RD          0x00040000
+#define MII_WR          0x00000000
+#define MII_DIN         0x00080000
+#define MII_DOUT        0x00020000
+#define MII_DOUTON      MII_DOUT
+#define MII_DOUTOFF     MII_DOUT
+#define MII_CLK		0x00010000
+#define MII_CLKON       MII_CLK
+#define MII_CLKOFF      MII_CLK
+
+/*
+ * Definitions for the DE425.
+ */
+#define	DE425_CFID		0x08	/* Configuration Id */
+#define	DE425_CFCS		0x0C	/* Configuration Command-Status */
+#define	DE425_CFRV		0x18	/* Configuration Revision */
+#define	DE425_CFLT		0x1C	/* Configuration Latency Timer */
+#define	DE425_CBIO		0x28	/* Configuration Base IO Address */
+#define	DE425_CFDA		0x2C	/* Configuration Driver Area */
+#define	DE425_ENETROM_OFFSET	0xC90	/* Offset in I/O space for ENETROM */
+#define	DE425_CFG0		0xC88	/* IRQ register */
+#define	DE425_EISAID		0x10a34250 /* EISA device id */
+#define	DE425_EISA_IOSIZE	0x100
+
+#define	DEC_VENDORID		0x1011
+#define	DC21040_CHIPID		0x0002
+#define	DC21140_CHIPID		0x0009
+#define	DC21041_CHIPID		0x0014
+#define	PCI_VENDORID(x)		((x) & 0xFFFF)
+#define	PCI_CHIPID(x)		(((x) >> 16) & 0xFFFF)
+
+/*
+ * Generic SROM Format
+ *
+ *
+ */
+
+typedef struct {
+    tulip_uint8_t sh_idbuf[18];
+    tulip_uint8_t sh_version;
+    tulip_uint8_t sh_adapter_count;
+    tulip_uint8_t sh_ieee802_address[6];
+} tulip_srom_header_t;
+
+typedef enum {
+    TULIP_SROM_CONNTYPE_10BASET			=0x0000,
+    TULIP_SROM_CONNTYPE_BNC			=0x0001,
+    TULIP_SROM_CONNTYPE_AUI			=0x0002,
+    TULIP_SROM_CONNTYPE_100BASETX		=0x0003,
+    TULIP_SROM_CONNTYPE_100BASET4		=0x0006,
+    TULIP_SROM_CONNTYPE_100BASEFX		=0x0007,
+    TULIP_SROM_CONNTYPE_MII_10BASET		=0x0009,
+    TULIP_SROM_CONNTYPE_MII_100BASETX		=0x000D,
+    TULIP_SROM_CONNTYPE_MII_100BASET4		=0x000F,
+    TULIP_SROM_CONNTYPE_MII_100BASEFX		=0x0010,
+    TULIP_SROM_CONNTYPE_10BASET_NWAY		=0x0100,
+    TULIP_SROM_CONNTYPE_10BASET_FD		=0x0204,
+    TULIP_SROM_CONNTYPE_MII_10BASET_FD		=0x020A,
+    TULIP_SROM_CONNTYPE_100BASETX_FD		=0x020E,
+    TULIP_SROM_CONNTYPE_MII_100BASETX_FD	=0x0211,
+    TULIP_SROM_CONNTYPE_10BASET_NOLINKPASS	=0x0400,
+    TULIP_SROM_CONNTYPE_AUTOSENSE		=0x0800,
+    TULIP_SROM_CONNTYPE_AUTOSENSE_POWERUP	=0x8800,
+    TULIP_SROM_CONNTYPE_AUTOSENSE_NWAY		=0x9000,
+    TULIP_SROM_CONNTYPE_NOT_USED		=0xFFFF
+} tulip_srom_connection_t;
+
+typedef enum {
+    TULIP_SROM_MEDIA_10BASET			=0x0000,
+    TULIP_SROM_MEDIA_BNC			=0x0001,
+    TULIP_SROM_MEDIA_AUI			=0x0002,
+    TULIP_SROM_MEDIA_100BASETX			=0x0003,
+    TULIP_SROM_MEDIA_10BASET_FD			=0x0004,
+    TULIP_SROM_MEDIA_100BASETX_FD		=0x0005,
+    TULIP_SROM_MEDIA_100BASET4			=0x0006,
+    TULIP_SROM_MEDIA_100BASEFX			=0x0007,
+    TULIP_SROM_MEDIA_100BASEFX_FD		=0x0008
+} tulip_srom_media_t;
+
+#define	TULIP_SROM_DC21041_EXTENDED	0x40
+
+#define	TULIP_SROM_DC2114X_NOINDICATOR		0x8000
+#define	TULIP_SROM_DC2114X_DEFAULT		0x4000
+#define	TULIP_SROM_DC2114X_POLARITY		0x0080
+#define	TULIP_SROM_DC2114X_CMDBITS(n)		(((n) & 0x0071) << 18)
+#define	TULIP_SROM_DC2114X_BITPOS(b)		(1 << (((b) & 0x0E) >> 1))
+
+
+
 #endif /* !defined(_DC21040_H) */
