@@ -1,6 +1,6 @@
 /**************************************************************************
 **
-**  $Id: pci.c,v 1.21 1995/03/23 06:14:44 davidg Exp $
+** $FreeBSD: stable/2.1/sys/pci/pci.c 50950 1999-09-05 08:39:22Z peter $
 **
 **  General subroutines for the PCI bus.
 **  pci_configure ()
@@ -36,9 +36,7 @@
 ***************************************************************************
 */
 
-#define __PCI_C__ "pl13 95/03/21"
-
-#include <pci.h>
+#include "pci.h"
 #if NPCI > 0
 
 /*========================================================
@@ -113,7 +111,7 @@ struct pcicb {
 
 static int
 pci_externalize (struct proc *, struct kern_devconf *, void *, size_t);
- 
+
 static int
 pci_internalize (struct proc *, struct kern_devconf *, void *, size_t);
 
@@ -123,9 +121,12 @@ not_supported (pcici_t tag, u_long type);
 static void
 pci_bus_config (void);
 
-static void
+static int
 pci_bridge_config (void);
 
+static int
+pci_mfdev (int bus, int device);
+
 /*========================================================
 **
 **	Variables
@@ -137,7 +138,7 @@ pci_bridge_config (void);
 **      log2 of safe burst len (in words)
 */
 
-unsigned pci_max_burst_len = 2;
+unsigned pci_max_burst_len = 2; /* 2=16Byte, 3=32Byte, 4=64Byte, ... */
 unsigned pci_mechanism     = 0;
 unsigned pci_maxdevice     = 0;
 struct pcibus* pcibus;
@@ -157,60 +158,39 @@ static	struct pcicb	pcibus0 = {
     0, 0, 0, 0,
     0, 0, 0, 0, 0, 0, 0, 0,	/* real allocation */
     0, 0xFFFF,			/* iobase/limit */
-    0x4000000, 0xFFFFFFFFu,	/* nonprefetch membase/limit */
-    0x4000000, 0xFFFFFFFFu	/* prefetch membase/limit */
+    0x2000000, 0xFFFFFFFFu,	/* nonprefetch membase/limit */
+    0x2000000, 0xFFFFFFFFu	/* prefetch membase/limit */
 };
 static	struct pcicb   *pcicb;
 
-/*========================================================
+/*-----------------------------------------------------------------
 **
-**	pci_configure ()
+**	The following functions are provided for the device driver
+**	to read/write the configuration space.
 **
-**      Autoconfiguration of pci devices.
+**	pci_conf_read():
+**		Read a long word from the pci configuration space.
+**		Requires a tag (from pcitag) and the register
+**		number (should be a long word alligned one).
 **
-**      May be called more than once.
-**      Any device is attached only once.
+**	pci_conf_write():
+**		Writes a long word to the pci configuration space.
+**		Requires a tag (from pcitag), the register number
+**		(should be a long word alligned one), and a value.
 **
-**      Has to take care of mirrored devices, which are
-**      entailed by incomplete decoding of pci address lines.
-**
-**========================================================
+**-----------------------------------------------------------------
 */
 
-void pci_configure()
+u_long
+pci_conf_read  (pcici_t tag, u_long reg)
 {
-	int i;
+    return (pcibus->pb_read (tag, reg));
+}
 
-	/*
-	**	check pci bus present
-	*/
-
-	for (i=0; i<pcibus_set.ls_length; i++) {
-		if (pci_maxdevice) break;
-		pcibus = (struct pcibus*) pcibus_set.ls_items[i];
-		pcibus->pb_setup ();
-	}
-	if (!pci_maxdevice) return;
-
-	/*
-	**	hello world ..
-	*/
-
-	for (pcicb = &pcibus0; pcicb != NULL;) {
-		pci_bus_config ();
-
-		if (pcicb->pcicb_down) {
-			pcicb = pcicb->pcicb_down;
-			continue;
-		};
-
-		while (pcicb && !pcicb->pcicb_next)
-			pcicb = pcicb->pcicb_up;
-
-		if (pcicb)
-			pcicb = pcicb->pcicb_next;
-	}
-	pci_conf_count++;
+void
+pci_conf_write (pcici_t tag, u_long reg, u_long data)
+{
+    pcibus->pb_write (tag, reg, data);
 }
 
 /*========================================================
@@ -224,8 +204,9 @@ static void
 pci_register_io (struct pcicb * cb, u_int base, u_int limit)
 {
 #ifdef PCI_BRIDGE_DEBUG
-	printf ("register_io:  bus=%d base=%x limit=%x\n",
-		cb->pcicb_bus, base, limit);
+	if (bootverbose)
+		printf ("register_io:  bus=%d base=%x limit=%x\n",
+			cb->pcicb_bus, base, limit);
 #endif
 
 	if (!cb->pcicb_pfrom || base < cb->pcicb_pfrom)
@@ -234,7 +215,7 @@ pci_register_io (struct pcicb * cb, u_int base, u_int limit)
 		cb->pcicb_pupto = limit;
 
 	/*
-	**	XXX should set bridge io mapping here 
+	**	XXX should set bridge io mapping here
 	**	but it can be mapped in 4k blocks only,
 	**	leading to conflicts with isa/eisa ..
 	*/
@@ -244,8 +225,9 @@ static void
 pci_register_memory (struct pcicb * cb, u_int base, u_int limit)
 {
 #ifdef PCI_BRIDGE_DEBUG
-	printf ("register_mem: bus=%d base=%x limit=%x\n",
-		cb->pcicb_bus, base, limit);
+	if (bootverbose)
+		printf ("register_mem: bus=%d base=%x limit=%x\n",
+			cb->pcicb_bus, base, limit);
 #endif
 
 	if (!cb->pcicb_mfrom || base < cb->pcicb_mfrom)
@@ -262,10 +244,11 @@ pci_register_memory (struct pcicb * cb, u_int base, u_int limit)
 			PCI_PCI_BRIDGE_MEM_REG,
 			(cb->pcicb_memlimit & 0xffff0000) |
 			(cb->pcicb_membase >> 16));
-		printf ("\t[pci%d uses memory from %x to %x]\n",
-			cb->pcicb_bus,
-			(unsigned) cb->pcicb_membase,
-			(unsigned) cb->pcicb_memlimit);
+		if (bootverbose)
+			printf ("\t[pci%d uses memory from %x to %x]\n",
+				cb->pcicb_bus,
+				(unsigned) cb->pcicb_membase,
+				(unsigned) cb->pcicb_memlimit);
 	}
 }
 
@@ -280,8 +263,9 @@ pci_memalloc (struct pcicb * cb, u_int addr, u_int size)
 {
 	u_int result = 0, limit=0, newbase=0;
 #ifdef PCI_BRIDGE_DEBUG
-	printf ("memalloc:  bus=%d addr=%x size=%x ..\n",
-		cb->pcicb_bus, addr, size);
+	if (bootverbose)
+		printf ("memalloc:  bus=%d addr=%x size=%x ..\n",
+			cb->pcicb_bus, addr, size);
 #endif
 
 	if (!cb) goto done;
@@ -333,6 +317,70 @@ done:
 	return (result);
 }
 
+/*========================================================
+**
+**	pci_bridge_config()
+**
+**	Configuration of a pci bridge.
+**
+**========================================================
+*/
+
+static int
+pci_bridge_config (void)
+{
+	pcici_t tag;
+	struct pcicb* parent;
+
+	tag = pcicb->pcicb_bridge;
+	if (tag.tag) {
+
+	    if (!pcicb->pcicb_bus) {
+		u_int data;
+		/*
+		**	Get the lowest available bus number.
+		*/
+		pcicb->pcicb_bus = ++pcibus0.pcicb_subordinate;
+
+		/*
+		**	and configure the bridge
+		*/
+		data = pci_conf_read (tag, PCI_PCI_BRIDGE_BUS_REG);
+		data = PCI_PRIMARY_BUS_INSERT(data, pcicb->pcicb_up->pcicb_bus);
+		data = PCI_SECONDARY_BUS_INSERT(data, pcicb->pcicb_bus);
+		data = PCI_SUBORDINATE_BUS_INSERT(data, pcicb->pcicb_bus);
+		pci_conf_write (tag, PCI_PCI_BRIDGE_BUS_REG, data);
+
+		/*
+		**	Propagate the new upper bus number limit.
+		*/
+		for (parent = pcicb->pcicb_up; parent != NULL;
+			parent = parent->pcicb_up)
+		{
+			if (parent->pcicb_subordinate >= pcicb->pcicb_bus)
+				continue;
+			parent->pcicb_subordinate = pcicb->pcicb_bus;
+			if (!parent->pcicb_bridge.tag)
+				continue;
+			data = pci_conf_read
+				(parent->pcicb_bridge, PCI_PCI_BRIDGE_BUS_REG);
+			data = PCI_SUBORDINATE_BUS_INSERT
+				(data, pcicb->pcicb_bus);
+			pci_conf_write (parent->pcicb_bridge,
+				PCI_PCI_BRIDGE_BUS_REG, data);
+		}
+	    }
+
+	    if (!pcicb->pcicb_membase) {
+		u_int size = 0x100000;
+		pcicb->pcicb_membase = pci_memalloc (pcicb->pcicb_up, 0, size);
+		if (pcicb->pcicb_membase)
+			pcicb->pcicb_memlimit = pcicb->pcicb_membase+size-1;
+	    }
+	}
+	return pcicb->pcicb_bus;
+}
+
 /*========================================================
 **
 **	pci_bus_config()
@@ -342,45 +390,87 @@ done:
 **========================================================
 */
 
+static int
+pci_mfdev (int bus, int device)
+{
+    pcici_t tag0,tag1;
+    int	pci_id0, pci_id1;
+
+    /*
+    ** Detect a multi-function device that complies to the PCI 2.0 spec
+    */
+    tag0 = pcibus->pb_tag  (bus, device, 0);
+    if (pci_conf_read (tag0, PCI_HEADER_MISC) & PCI_HEADER_MULTIFUNCTION)
+	return 1;
+
+    /*
+    ** Well, as always: Theory and implementation of PCI ...
+    **
+    ** If there is a valid device ID returned for function 1 AND
+    **		the device ID of function 0 and 1 is different OR
+    **		the first mapping register of 0 and 1 differs,
+    ** then assume a multi-function device anyway ...
+    **
+    ** Example of such a broken device: ISA and IDE chip i83371FB (Triton)
+    */
+    tag1 = pcibus->pb_tag  (bus, device, 1);
+    pci_id1 = pci_conf_read (tag1, PCI_ID_REG);
+
+    if (pci_id1 != 0xffffffff) {
+
+	pci_id0 = pci_conf_read (tag0, PCI_ID_REG);
+
+	if (pci_id0 != pci_id1)
+	    return 1;
+
+	if (pci_conf_read (tag0, PCI_MAP_REG_START) 
+			!= pci_conf_read (tag1, PCI_MAP_REG_START))
+	    return 1;
+    }
+    return 0;
+}
+
 static void
 pci_bus_config (void)
 {
+	int	bus_no;
 	u_char  device;
 	u_char	reg;
 	pcici_t tag, mtag;
 	pcidi_t type;
 	u_long  data;
 	int     unit;
-	int     pciint;
-	int     irq;
-	char*   name=0;
+	u_char	pciint;
+	u_char	irq;
 
-	int     dvi;
-	struct pci_device *dvp=0;
+	struct	pci_device *dvp;
 
-	struct pci_devconf *pdcp;
+	struct	pci_devconf *pdcp;
 
 	/*
 	**	first initialize the bridge (bus controller chip)
 	*/
-	pci_bridge_config ();
+	bus_no = pci_bridge_config ();
 
+	printf ("Probing for devices on PCI bus %d:\n", bus_no);
 #ifndef PCI_QUIET
-	printf ("Probing for devices on the %s%d bus:\n",
-		pcibus->pb_name, pcicb->pcicb_bus);
-	if (!pci_info_done) {
+	if (bootverbose && !pci_info_done) {
 		pci_info_done=1;
 		printf ("\tconfiguration mode %d allows %d devices.\n",
 			pci_mechanism, pci_maxdevice);
 	};
 #endif
 	for (device=0; device<pci_maxdevice; device ++) {
+	    char *name = NULL;
+	    struct pci_device **dvpp;
+	    int func, maxfunc = 0;
 
-		if ((pcicb->pcicb_seen >> device) & 1)
-			continue;
+	    if ((pcicb->pcicb_seen >> device) & 1)
+	    	continue;
 
-		tag  = pcibus->pb_tag  (pcicb->pcicb_bus, device, 0);
-		type = pcibus->pb_read (tag, PCI_ID_REG);
+	    for (func=0; func <= maxfunc; func++) {
+		tag  = pcibus->pb_tag  (bus_no, device, func);
+		type = pci_conf_read (tag, PCI_ID_REG);
 
 		if ((!type) || (type==0xfffffffful)) continue;
 
@@ -388,52 +478,79 @@ pci_bus_config (void)
 		**	lookup device in ioconfiguration:
 		*/
 
-		for (dvi=0; dvi<pcidevice_set.ls_length; dvi++) {
-			dvp = (struct pci_device*) pcidevice_set.ls_items[dvi];
-			if ((name=(*dvp->pd_probe)(tag, type)))
-				break;
-			dvp = NULL;
-		};
+		dvpp = (struct pci_device **)pcidevice_set.ls_items;
 
+		while (dvp = *dvpp++) {
+			if (dvp->pd_probe) {
+				if (name=(*dvp->pd_probe)(tag, type))
+					break;
+			}
+		};
 		/*
 		**	check for mirrored devices.
 		*/
+		if (func != 0) {
+			goto real_device;
+		}
 		if (device & 0x10) {
-			mtag=pcibus->pb_tag (pcicb->pcicb_bus,
+			mtag=pcibus->pb_tag (bus_no,
 				(u_char)(device & ~0x10), 0);
 		} else if (device & 0x08) {
-			mtag=pcibus->pb_tag (pcicb->pcicb_bus,
+			mtag=pcibus->pb_tag (bus_no,
 				(u_char)(device & ~0x08), 0);
 		} else goto real_device;
 
-		if (type!=pcibus->pb_read (mtag, PCI_ID_REG))
+		if (type!=pci_conf_read (mtag, PCI_ID_REG))
 			goto real_device;
 
 		for (reg=PCI_MAP_REG_START;reg<PCI_MAP_REG_END;reg+=4)
-			if (pcibus->pb_read(tag,reg)!=pcibus->pb_read(mtag,reg))
+			if (pci_conf_read(tag,reg)!=pci_conf_read(mtag,reg))
 				goto real_device;
 
 #ifndef PCI_QUIET
 		if (dvp==NULL) continue;
-		printf ("%s? <%s> mirrored on pci%d:%d\n",
-			dvp->pd_name, name, pcicb->pcicb_bus, device);
+		if (bootverbose)
+			printf ("%s? <%s> mirrored on pci%d:%d\n",
+				dvp->pd_name, name, bus_no, device);
 #endif
 		continue;
 
 	real_device:
 
+#ifndef PCI_QUIET
+#ifdef PCI_BRIDGE_DEBUG
+		if (bootverbose) {
+		    printf ("\tconfig header: 0x%08x 0x%08x 0x%08x 0x%08x\n",
+			    pci_conf_read (tag, 0),
+			    pci_conf_read (tag, 4),
+			    pci_conf_read (tag, 8),
+			    pci_conf_read (tag, 12));
+		}
+#endif
+#endif
+
+		if (func == 0 && pci_mfdev (bus_no, device)) {
+			maxfunc = 7;
+		}
+
 		if (dvp==NULL) {
 #ifndef PCI_QUIET
 			if (pci_conf_count)
 				continue;
-			printf("%s%d:%d: ", pcibus->pb_name,
-				pcicb->pcicb_bus, device);
+
+			if (maxfunc == 0)
+				printf("%s%d:%d:    ", 
+				       pcibus->pb_name, bus_no, device);
+			else
+				printf("%s%d:%d:%d: ", 
+				       pcibus->pb_name, bus_no, device, func);
 			not_supported (tag, type);
 #endif
 			continue;
 		};
 
 		pcicb->pcicb_seen |= (1ul << device);
+
 		/*
 		**	Get and increment the unit.
 		*/
@@ -458,7 +575,7 @@ pci_bus_config (void)
 		**	from the pci configuration space.
 		*/
 
-		data = pcibus->pb_read (tag, PCI_INTERRUPT_REG);
+		data = pci_conf_read (tag, PCI_INTERRUPT_REG);
 		pciint = PCI_INTERRUPT_PIN_EXTRACT(data);
 
 		if (pciint) {
@@ -472,13 +589,16 @@ pci_bus_config (void)
 			**	and we cannot bind the pci interrupt.
 			*/
 
-			if (irq)
+			if (irq && (irq != 0xff))
 				printf ("%d", irq);
 			else
 				printf ("??");
 		};
 
-		printf (" on pci%d:%d\n", pcicb->pcicb_bus, device);
+		if (maxfunc == 0)
+			printf (" on pci%d:%d\n", bus_no, device);
+		else
+			printf (" on pci%d:%d:%d\n", bus_no, device, func);
 
 		/*
 		**	Read the current mapping,
@@ -494,13 +614,13 @@ pci_bus_config (void)
 				continue;
 			};
 
-			map = pcibus->pb_read (tag, reg);
+			map = pci_conf_read (tag, reg);
 			if (!(map & PCI_MAP_MEMORY_ADDRESS_MASK))
 				continue;
 
-			pcibus->pb_write (tag, reg, 0xffffffff);
-			data = pcibus->pb_read (tag, reg);
-			pcibus->pb_write (tag, reg, map);
+			pci_conf_write (tag, reg, 0xffffffff);
+			data = pci_conf_read (tag, reg);
+			pci_conf_write (tag, reg, map);
 
 			switch (data & 7) {
 
@@ -508,8 +628,9 @@ pci_bus_config (void)
 				continue;
 			case 1:
 			case 5:
-				size = -(data & PCI_MAP_IO_ADDRESS_MASK);
 				addr = map & PCI_MAP_IO_ADDRESS_MASK;
+				size = -(data & PCI_MAP_IO_ADDRESS_MASK);
+				size &= ~(addr ^ -addr);
 
 				pci_register_io (pcicb, addr, addr+size-1);
 				pcicb->pcicb_pamount += size;
@@ -529,10 +650,9 @@ pci_bus_config (void)
 				};
 				break;
 			};
-			if (!bootverbose)
-				continue;
-			printf ("\tmapreg[%02x] type=%d addr=%08x size=%04x.\n",
-				reg, map&7, addr, size);
+			if (bootverbose)
+				printf ("\tmapreg[%02x] type=%d addr=%08x size=%04x.\n",
+					reg, map&7, addr, size);
 		};
 
 		/*
@@ -546,8 +666,9 @@ pci_bus_config (void)
 			malloc (sizeof (struct pci_devconf),M_DEVBUF,M_WAITOK);
 		bzero(pdcp, sizeof(struct pci_devconf));
 
-		pdcp -> pdc_pi.pi_bus    = pcicb->pcicb_bus;
+		pdcp -> pdc_pi.pi_bus    = bus_no;
 		pdcp -> pdc_pi.pi_device = device;
+		pdcp -> pdc_pi.pi_func   = func;
 
 		pdcp -> pdc_kdc.kdc_name = dvp->pd_name;
 		pdcp -> pdc_kdc.kdc_unit = unit;
@@ -598,20 +719,27 @@ pci_bus_config (void)
 			secondary   = PCI_SECONDARY_BUS_EXTRACT(data);
 			subordinate = PCI_SUBORDINATE_BUS_EXTRACT(data);
 #ifndef PCI_QUIET
-			printf ("\tbridge from pci%d to pci%d through %d.\n",
+			if (bootverbose) {
+			    printf ("\tbridge from pci%d to pci%d through %d.\n",
 				primary, secondary, subordinate);
+			    printf ("\tmapping regs: io:%08lx mem:%08lx pmem:%08lx\n",
+				pci_conf_read (tag, PCI_PCI_BRIDGE_IO_REG),
+				pci_conf_read (tag, PCI_PCI_BRIDGE_MEM_REG),
+				pci_conf_read (tag, PCI_PCI_BRIDGE_PMEM_REG));
+			}
 #endif
 			/*
 			**	check for uninitialized bridge.
 			*/
-			if (secondary == 0 || secondary < primary ||
-				pcicb->pcicb_bus != primary)
+			if (!(primary < secondary 
+			      && secondary <= subordinate
+			      && bus_no == primary))
 			{
 				printf ("\tINCORRECTLY or NEVER CONFIGURED.\n");
 				/*
 				**	disable this bridge
 				*/
-				pcibus->pb_write (tag, PCI_COMMAND_STATUS_REG,
+				pci_conf_write (tag, PCI_COMMAND_STATUS_REG,
 							0xffff0000);
 				secondary   = 0;
 				subordinate = 0;
@@ -621,7 +749,8 @@ pci_bus_config (void)
 			**  allocate bus descriptor for bus behind the bridge
 			*/
 			link = &pcicb->pcicb_down;
-			while (*link) link = &(*link)->pcicb_next;
+			while (*link && (*link)->pcicb_bus < secondary)
+				link = &(*link)->pcicb_next;
 
 			this = malloc (sizeof (*this), M_DEVBUF, M_WAITOK);
 
@@ -631,6 +760,7 @@ pci_bus_config (void)
 			**	scanning the bus behind the bridge.
 			*/
 			bzero (this, sizeof(*this));
+			this->pcicb_next        = *link;
 			this->pcicb_up		= pcicb;
 			this->pcicb_bridge      = tag;
 			this->pcicb_bus 	= secondary;
@@ -643,23 +773,14 @@ pci_bus_config (void)
 				**	Bridge was configured by the bios.
 				**	Read out the mapped io region.
 				*/
-				u_int reg, data, mask;
+				unsigned reg;
 
 				reg = pci_conf_read (tag,
 					PCI_PCI_BRIDGE_IO_REG);
-				pci_conf_write(tag,
-					PCI_PCI_BRIDGE_IO_REG, 0xFFFF);
-				data = pci_conf_read (tag,
-					PCI_PCI_BRIDGE_IO_REG);
-				pci_conf_write(tag,
-					PCI_PCI_BRIDGE_IO_REG, reg & 0xffff);
-
-				mask = (0xFF00 ^ (data & 0xFF00)) | 0xFF;
-
 				this->pcicb_iobase  =
 					PCI_PPB_IOBASE_EXTRACT (reg);
 				this->pcicb_iolimit =
-					PCI_PPB_IOLIMIT_EXTRACT(reg) | mask;
+					PCI_PPB_IOLIMIT_EXTRACT(reg);
 
 				/*
 				**	Note the used io space.
@@ -674,26 +795,17 @@ pci_bus_config (void)
 				**	Bridge was configured by the bios.
 				**	Read out the mapped memory regions.
 				*/
-				u_int reg, data, mask;
+				unsigned reg;
 
 				/*
 				**	non prefetchable memory
 				*/
 				reg = pci_conf_read (tag,
 					PCI_PCI_BRIDGE_MEM_REG);
-				pci_conf_write(tag,
-					PCI_PCI_BRIDGE_MEM_REG, 0xFFFFFFFF);
-				data = pci_conf_read (tag,
-					PCI_PCI_BRIDGE_MEM_REG);
-				pci_conf_write(tag,
-					PCI_PCI_BRIDGE_MEM_REG, reg);
-				mask = (0xFFFF0000 ^ (data & 0xFFFF0000))
-					| 0xFFFF;
-
 				this->pcicb_membase  =
 					PCI_PPB_MEMBASE_EXTRACT (reg);
 				this->pcicb_memlimit =
-					PCI_PPB_MEMLIMIT_EXTRACT(reg) | mask;
+					PCI_PPB_MEMLIMIT_EXTRACT(reg);
 
 				/*
 				**	Register used memory space.
@@ -707,19 +819,10 @@ pci_bus_config (void)
 				*/
 				reg = pci_conf_read (tag,
 					PCI_PCI_BRIDGE_PMEM_REG);
-				pci_conf_write(tag,
-					PCI_PCI_BRIDGE_PMEM_REG, 0xFFFFFFFF);
-				data = pci_conf_read (tag,
-					PCI_PCI_BRIDGE_PMEM_REG);
-				pci_conf_write(tag,
-					PCI_PCI_BRIDGE_PMEM_REG, reg);
-
-				mask = (0xFFFF0000 ^ (data & 0xFFFF0000))
-					| 0xFFFF;
 				this->pcicb_p_membase=
 					PCI_PPB_MEMBASE_EXTRACT (reg);
 				this->pcicb_p_memlimit=
-					PCI_PPB_MEMLIMIT_EXTRACT(reg) | mask;
+					PCI_PPB_MEMLIMIT_EXTRACT(reg);
 
 				/*
 				**	Register used memory space.
@@ -742,117 +845,79 @@ pci_bus_config (void)
 			if (subordinate > pcicb->pcicb_bupto)
 				pcicb->pcicb_bupto = subordinate;
 
-  			break;
+			break;
 		}
+	    }
 	}
 
 #ifndef PCI_QUIET
-	if (pcicb->pcicb_mamount)
+	if (bootverbose) {
+	    if (pcicb->pcicb_mamount)
 		printf ("%s%d: uses %d bytes of memory from %x upto %x.\n",
-			pcibus->pb_name, pcicb->pcicb_bus,
+			pcibus->pb_name, bus_no,
 			pcicb->pcicb_mamount,
 			pcicb->pcicb_mfrom, pcicb->pcicb_mupto);
-	if (pcicb->pcicb_pamount)
+	    if (pcicb->pcicb_pamount)
 		printf ("%s%d: uses %d bytes of I/O space from %x upto %x.\n",
-			pcibus->pb_name, pcicb->pcicb_bus,
+			pcibus->pb_name, bus_no,
 			pcicb->pcicb_pamount,
 			pcicb->pcicb_pfrom, pcicb->pcicb_pupto);
-	if (pcicb->pcicb_bfrom)
+	    if (pcicb->pcicb_bfrom)
 		printf ("%s%d: subordinate busses from %x upto %x.\n",
-			pcibus->pb_name, pcicb->pcicb_bus,
+			pcibus->pb_name, bus_no,
 			pcicb->pcicb_bfrom, pcicb->pcicb_bupto);
+	}
 #endif
 }
 
 /*========================================================
 **
-**	pci_bridge_config()
+**	pci_configure ()
 **
-**	Configuration of a pci bridge.
+**      Autoconfiguration of pci devices.
+**
+**      May be called more than once.
+**      Any device is attached only once.
+**
+**      Has to take care of mirrored devices, which are
+**      entailed by incomplete decoding of pci address lines.
 **
 **========================================================
 */
 
-static void
-pci_bridge_config (void)
+void pci_configure()
 {
-	pcici_t tag;
-	struct pcicb* parent;
+	struct pcibus **pbp = (struct pcibus**) pcibus_set.ls_items;
 
-	tag = pcicb->pcicb_bridge;
-	if (!tag.tag) return;
+	/*
+	**	check pci bus present
+	*/
 
-	if (!pcicb->pcicb_bus) {
-		u_int data;
-		/*
-		**	Get the lowest available bus number.
-		*/
-		pcicb->pcicb_bus = ++pcibus0.pcicb_subordinate;
-
-		/*
-		**	and configure the bridge
-		*/
-		data = pci_conf_read (tag, PCI_PCI_BRIDGE_BUS_REG);
-		data = PCI_PRIMARY_BUS_INSERT(data, pcicb->pcicb_up->pcicb_bus);
-		data = PCI_SECONDARY_BUS_INSERT(data, pcicb->pcicb_bus);
-		data = PCI_SUBORDINATE_BUS_INSERT(data, pcicb->pcicb_bus);
-		pci_conf_write (tag, PCI_PCI_BRIDGE_BUS_REG, data);
-
-		/*
-		**	Propagate the new upper bus number limit.
-		*/
-		for (parent = pcicb->pcicb_up; parent != NULL;
-			parent = parent->pcicb_up)
-		{
-			if (parent->pcicb_subordinate >= pcicb->pcicb_bus)
-				continue;
-			parent->pcicb_subordinate = pcicb->pcicb_bus;
-			if (!parent->pcicb_bridge.tag)
-				continue;
-			data = pci_conf_read
-				(parent->pcicb_bridge, PCI_PCI_BRIDGE_BUS_REG);
-			data = PCI_SUBORDINATE_BUS_INSERT
-				(data, pcicb->pcicb_bus);
-			pci_conf_write (parent->pcicb_bridge,
-				PCI_PCI_BRIDGE_BUS_REG, data);
-		}
+	while (!pci_maxdevice && (pcibus = *pbp++)) {
+		(*pcibus->pb_setup)();
 	}
 
-	if (!pcicb->pcicb_membase) {
-		u_int size = 0x100000;
-		pcicb->pcicb_membase = pci_memalloc (pcicb->pcicb_up, 0, size);
-		if (pcicb->pcicb_membase)
-			pcicb->pcicb_memlimit = pcicb->pcicb_membase+size-1;
+	if (!pci_maxdevice) return;
+
+	/*
+	**	hello world ..
+	*/
+
+	for (pcicb = &pcibus0; pcicb != NULL;) {
+		pci_bus_config ();
+
+		if (pcicb->pcicb_down) {
+			pcicb = pcicb->pcicb_down;
+			continue;
+		};
+
+		while (pcicb && !pcicb->pcicb_next)
+			pcicb = pcicb->pcicb_up;
+
+		if (pcicb)
+			pcicb = pcicb->pcicb_next;
 	}
-}
-/*-----------------------------------------------------------------
-**
-**	The following functions are provided for the device driver
-**	to read/write the configuration space.
-**
-**	pci_conf_read():
-**		Read a long word from the pci configuration space.
-**		Requires a tag (from pcitag) and the register
-**		number (should be a long word alligned one).
-**
-**	pci_conf_write():
-**		Writes a long word to the pci configuration space.
-**		Requires a tag (from pcitag), the register number
-**		(should be a long word alligned one), and a value.
-**
-**-----------------------------------------------------------------
-*/
-
-u_long
-pci_conf_read  (pcici_t tag, u_long reg)
-{
-	return (pcibus->pb_read (tag, reg));
-}
-
-void
-pci_conf_write (pcici_t tag, u_long reg, u_long data)
-{
-	pcibus->pb_write (tag, reg, data);
+	pci_conf_count++;
 }
 
 /*-----------------------------------------------------------------------
@@ -896,15 +961,15 @@ int pci_map_port (pcici_t tag, u_long reg, u_short* pa)
 	**	n-2 bits are hardwired as 0.
 	*/
 
-	ioaddr = pcibus->pb_read (tag, reg) & PCI_MAP_IO_ADDRESS_MASK;
-	if (!ioaddr || ioaddr > 0xfffful) {
+	ioaddr = pci_conf_read (tag, reg) & PCI_MAP_IO_ADDRESS_MASK;
+	if (!ioaddr) {
 		printf ("pci_map_port failed: not configured by bios.\n");
 		return (0);
 	};
 
-	pcibus->pb_write (tag, reg, 0xfffffffful);
-	data = pcibus->pb_read (tag, reg);
-	pcibus->pb_write (tag, reg, ioaddr);
+	pci_conf_write (tag, reg, 0xfffffffful);
+	data = pci_conf_read (tag, reg);
+	pci_conf_write (tag, reg, ioaddr);
 
 	if ((data & 0x03) != PCI_MAP_IO) {
 		printf ("pci_map_port failed: bad port type=0x%x\n",
@@ -912,8 +977,9 @@ int pci_map_port (pcici_t tag, u_long reg, u_short* pa)
 		return (0);
 	};
 	iosize = -(data &  PCI_MAP_IO_ADDRESS_MASK);
+	iosize &= ~(ioaddr ^ -ioaddr);
 	if (ioaddr < pcicb->pcicb_iobase
-		|| ioaddr + iosize > pcicb->pcicb_iolimit) {
+		|| ioaddr + iosize -1 > pcicb->pcicb_iolimit) {
 		printf ("pci_map_port failed: device's iorange 0x%x-0x%x "
 			"is incompatible with its bridge's range 0x%x-0x%x\n",
 			(unsigned) ioaddr, (unsigned) ioaddr + iosize - 1,
@@ -923,8 +989,9 @@ int pci_map_port (pcici_t tag, u_long reg, u_short* pa)
 	}
 
 #ifndef PCI_QUIET
-	printf ("\treg%d: ioaddr=0x%x size=0x%x\n",
-		(unsigned) reg, (unsigned) ioaddr, (unsigned) iosize);
+	if (bootverbose)
+		printf ("\treg%d: ioaddr=0x%x size=0x%x\n",
+			(unsigned) reg, (unsigned) ioaddr, (unsigned) iosize);
 #endif
 	/*
 	**	set the configuration register of and
@@ -934,9 +1001,9 @@ int pci_map_port (pcici_t tag, u_long reg, u_short* pa)
 	*/
 
 	for (;;) {
-		data =	pcibus->pb_read (tag, PCI_COMMAND_STATUS_REG) & 0xffff;
+		data =	pci_conf_read (tag, PCI_COMMAND_STATUS_REG) & 0xffff;
 		data |= PCI_COMMAND_IO_ENABLE | PCI_COMMAND_MASTER_ENABLE;
-		(void)	pcibus->pb_write(tag, PCI_COMMAND_STATUS_REG, data);
+		(void)	pci_conf_write(tag, PCI_COMMAND_STATUS_REG, data);
 		if ((link = link->pcicb_up) == NULL)
 			break;
 		tag = link->pcicb_bridge;
@@ -984,10 +1051,10 @@ int pci_map_mem (pcici_t tag, u_long reg, vm_offset_t* va, vm_offset_t* pa)
 	**	n-4 bits are read as 0.
 	*/
 
-	paddr = pcibus->pb_read (tag, reg) & PCI_MAP_MEMORY_ADDRESS_MASK;
-	pcibus->pb_write (tag, reg, 0xfffffffful);
-	data = pcibus->pb_read (tag, reg);
-	pcibus->pb_write (tag, reg, paddr);
+	paddr = pci_conf_read (tag, reg) & PCI_MAP_MEMORY_ADDRESS_MASK;
+	pci_conf_write (tag, reg, 0xfffffffful);
+	data = pci_conf_read (tag, reg);
+	pci_conf_write (tag, reg, paddr);
 
 	/*
 	**	check the type
@@ -1022,9 +1089,13 @@ int pci_map_mem (pcici_t tag, u_long reg, vm_offset_t* va, vm_offset_t* pa)
 			(unsigned) (paddr + psize - 1),
 			(unsigned) pcicb->pcicb_membase,
 			(unsigned) pcicb->pcicb_memlimit);
-		return (0);
+/*		return (0);*/
+/* ACHTUNG: Ist der Code richtig, wenn eine PCI-PCI-Bridge fuer
+ * die PCI-Slots verwendet wird, aber die Onboard-Devices direkt 
+ * an der CPU-PCI-Bridge haengen (Siehe Compaq Prolinea Problem) ???
+ */
 	}
-	pcibus->pb_write (tag, reg, paddr);
+	pci_conf_write (tag, reg, paddr);
 
 	/*
 	**	Truncate paddr to page boundary.
@@ -1043,8 +1114,9 @@ int pci_map_mem (pcici_t tag, u_long reg, vm_offset_t* va, vm_offset_t* pa)
 	**	display values.
 	*/
 
-	printf ("\treg%d: virtual=0x%lx physical=0x%lx size=0x%lx\n",
-		(unsigned) reg, (u_long)vaddr, (u_long)paddr, (u_long)psize);
+	if (bootverbose)
+		printf ("\treg%d: virtual=0x%lx physical=0x%lx size=0x%lx\n",
+		 (unsigned) reg, (u_long)vaddr, (u_long)paddr, (u_long)psize);
 #endif
 	/*
 	**      set the configuration register and
@@ -1054,9 +1126,9 @@ int pci_map_mem (pcici_t tag, u_long reg, vm_offset_t* va, vm_offset_t* pa)
 	*/
 
 	for (;;) {
-		data =  pcibus->pb_read (tag, PCI_COMMAND_STATUS_REG) & 0xffff;
+		data =  pci_conf_read (tag, PCI_COMMAND_STATUS_REG) & 0xffff;
 		data |= PCI_COMMAND_MEM_ENABLE | PCI_COMMAND_MASTER_ENABLE;
-		(void)  pcibus->pb_write(tag, PCI_COMMAND_STATUS_REG, data);
+		(void)  pci_conf_write(tag, PCI_COMMAND_STATUS_REG, data);
 		if ((link = link->pcicb_up) == NULL)
 			break;
 		tag = link->pcicb_bridge;
@@ -1087,18 +1159,18 @@ pci_externalize (struct proc *p, struct kern_devconf *kdcp, void *u, size_t l)
 		return ENOMEM;
 	};
 
-	tag = pcibus->pb_tag (pip->pi_bus, pip->pi_device, 0);
+	tag = pcibus->pb_tag (pip->pi_bus, pip->pi_device, pip->pi_func);
 
 	buffer.peb_pci_info	= *pip;
 
 	for (i=0; i<PCI_EXT_CONF_LEN; i++) {
-		buffer.peb_config[i] = pcibus->pb_read (tag, i*4);
+		buffer.peb_config[i] = pci_conf_read (tag, i*4);
 	};
 
 	return copyout(&buffer, u, sizeof buffer);
 }
- 
- 
+
+
 static int
 pci_internalize (struct proc *p, struct kern_devconf *kdcp, void *u, size_t s)
 {
@@ -1142,7 +1214,7 @@ pci_int (int irq)
 	int i, n;
 #endif
 	if (irq<0 || irq >= PCI_MAX_IRQ) {
-		printf ("pci_int(%d)\n", irq);
+		printf ("pci_int: irq %d out of range, ignored\n", irq);
 		return;
 	};
 
@@ -1184,18 +1256,18 @@ getirq (pcici_t tag)
 {
 	u_int irq;
 
-        irq = PCI_INTERRUPT_LINE_EXTRACT(
-                pcibus->pb_read (tag, PCI_INTERRUPT_REG));
+	irq = PCI_INTERRUPT_LINE_EXTRACT(
+		pci_conf_read (tag, PCI_INTERRUPT_REG));
 
-        if (irq <= 0) {
+	if (irq <= 0) {
 		printf ("\tint line register not set by bios\n");
 		return (0);
-        }
+	}
 
-        if (irq >= pcibus->pb_maxirq || irq >= PCI_MAX_IRQ) {
+	if (irq >= pcibus->pb_maxirq || irq >= PCI_MAX_IRQ) {
 		printf ("\tirq %d invalid.\n", irq);
 		return (0);
-        }
+	}
 
 	return (irq);
 }
@@ -1205,9 +1277,9 @@ getintdescbytag (u_int irq, pcici_t tag)
 {
 	struct pci_int_desc *p, **pp;
 
-        pp=&pci_int_desc[irq];
+	pp=&pci_int_desc[irq];
 	while (((p=*pp)) && !sametag(p->pcid_tag,tag))
-                pp=&p->pcid_next;
+		pp=&p->pcid_next;
 
 	if (!p) return (NULL);
 
@@ -1245,18 +1317,18 @@ int pci_map_int (pcici_t tag, int(*func)(), void* arg, unsigned* maskptr)
 	**	and check for consistency.
 	*/
 
-        irq = getirq (tag);
+	irq = getirq (tag);
 	if (irq >= PCI_MAX_IRQ) {
 		printf ("\tillegal irq %d.\n", irq);
 		return (0);
 	};
 	mask= 1ul << irq;
 
-        /*
-        **      disable this interrupt.
-        */
+	/*
+	**      disable this interrupt.
+	*/
 
-        oldspl = splq (mask);
+	oldspl = splq (mask);
 
 	/*
 	**	If handler for this tag already installed,
@@ -1266,7 +1338,7 @@ int pci_map_int (pcici_t tag, int(*func)(), void* arg, unsigned* maskptr)
 	if (getintdescbytag (irq, tag) != NULL)
 		pci_unmap_int (tag);
 
-        /*
+	/*
 	**	If this irq not yet included in the mask, include it.
 	*/
 
@@ -1275,7 +1347,7 @@ int pci_map_int (pcici_t tag, int(*func)(), void* arg, unsigned* maskptr)
 		result = pcibus->pb_imaskinc (irq, maskptr);
 		if (result)
 			goto conflict;
-        };
+	};
 
 	/*
 	**	Allocate descriptor and initialize it.
@@ -1283,7 +1355,7 @@ int pci_map_int (pcici_t tag, int(*func)(), void* arg, unsigned* maskptr)
 
 	tail = pci_int_desc[irq];
 
-        new = malloc (sizeof (*new), M_DEVBUF, M_WAITOK);
+	new = malloc (sizeof (*new), M_DEVBUF, M_WAITOK);
 	bzero (new, sizeof (*new));
 
 	new->pcid_next	   = tail;
@@ -1311,12 +1383,13 @@ int pci_map_int (pcici_t tag, int(*func)(), void* arg, unsigned* maskptr)
 #ifdef NO_SHARED_IRQ
 	} else goto conflict;
 #else
-        } else if (!tail->pcid_next) {
+	} else if (!tail->pcid_next) {
 		/*
 		**	Second handler for this irq.
 		*/
 
-                printf ("\tusing shared irq %d.\n", irq);
+		if (bootverbose)
+			printf ("\tusing shared irq %d.\n", irq);
 
 		/*
 		**	replace old handler by shared-int-handler.
@@ -1331,15 +1404,15 @@ int pci_map_int (pcici_t tag, int(*func)(), void* arg, unsigned* maskptr)
 			printf ("\tCANNOT ATTACH SHARED INT HANDLER.\n");
 			goto fail;
 		};
-        }
+	}
 #endif
 	/*
 	**	Link new descriptor, reenable ints and done.
 	*/
 
-        pci_int_desc[irq]  = new;
+	pci_int_desc[irq]  = new;
 	splx (oldspl);
-        return (1);
+	return (1);
 
 	/*
 	**	Handle some problems.
@@ -1353,10 +1426,10 @@ fail:
 	**	If included in mask, remove it.
 	*/
 
-        if (free) free (new, M_DEVBUF);
+	if (new) free(new, M_DEVBUF);
 	if (!mdp) (void) pcibus->pb_imaskexc (irq, maskptr);
 	splx (oldspl);
-        return (0);
+	return (0);
 }
 
 /*-----------------------------------------------------------------------
@@ -1377,7 +1450,7 @@ int pci_unmap_int (pcici_t tag)
 	**	and check for consistency.
 	*/
 
-        irq = getirq (tag);
+	irq = getirq (tag);
 	if (irq >= PCI_MAX_IRQ) {
 		printf ("\tillegal irq %d.\n", irq);
 		return (0);
@@ -1394,7 +1467,7 @@ int pci_unmap_int (pcici_t tag)
 		return (0);
 	};
 
-        this = *hook;
+	this = *hook;
 	*hook= this->pcid_next;
 
 	/*
@@ -1405,16 +1478,16 @@ int pci_unmap_int (pcici_t tag)
 		irq, this->pcid_handler, this->pcid_argument,
 		this->pcid_tag.tag, this->pcid_tally);
 
-        /*
+	/*
 	**	If this irq no longer included in the mask, remove it.
 	*/
 
 	if (!getintdescbymptr (irq, this->pcid_maskptr))
 		(void) pcibus->pb_imaskexc (irq, this->pcid_maskptr);
 
-        tail = pci_int_desc[irq];
+	tail = pci_int_desc[irq];
 
-        if (tail == NULL) {
+	if (tail == NULL) {
 
 		/*
 		**	Remove the old handler.
@@ -1433,22 +1506,22 @@ int pci_unmap_int (pcici_t tag)
 
 		oldspl = splq (1ul << irq);
 
-                result = pcibus->pb_idetach (irq, pci_int);
+		result = pcibus->pb_idetach (irq, pci_int);
 		if (result)
-                        printf ("\tirq %d: cannot remove handler.\n", irq);
+			printf ("\tirq %d: cannot remove handler.\n", irq);
 
 		result = pcibus->pb_iattach (irq,
 				(void(*)()) tail->pcid_handler,
 				(int) tail->pcid_argument,
 				tail->pcid_maskptr);
 
-                if (result)
+		if (result)
 			printf ("\tirq %d: cannot install handler.\n", irq);
 
 		splx (oldspl);
-        };
+	};
 
-        free (this, M_DEVBUF);
+	free (this, M_DEVBUF);
 	return (1);
 }
 
@@ -1464,26 +1537,137 @@ struct vt {
 };
 
 static struct vt VendorTable[] = {
-	{0x1002, "ATI TECHNOLOGIES INC"},
-	{0x1011, "DIGITAL EQUIPMENT CORPORATION"},
+	{0x0e11, "Compaq"},
+	{0x1000, "NCR/Symbios"},
+	{0x1002, "ATI Technologies Inc."},
+	{0x1004, "VLSI"},
+	{0x100B, "National Semiconductor"},
+	{0x100E, "Weitek"},
+	{0x1011, "Digital Equipment Corporation"},
+	{0x1013, "Cirrus Logic"},
 	{0x101A, "NCR"},
-	{0x102B, "MATROX"},
+	{0x1022, "AMD"},
+	{0x102B, "Matrox"},
+	{0x102C, "Chips & Technologies"},
+	{0x1039, "Silicon Integrated Systems"},
+	{0x1042, "SMC"},
+	{0x1044, "DPT"},
 	{0x1045, "OPTI"},
-	{0x5333, "S3 INC."},
-	{0x8086, "INTEL CORPORATION"},
+	{0x104B, "Bus Logic"},
+	{0x1060, "UMC"},
+	{0x1080, "Contaq"},
+	{0x1095, "CMD"},
+	{0x1106, "VIA Technologies"},
+	{0x5333, "S3 Inc."},
+	{0x8086, "Intel Corporation"},
+	{0x9004, "Adaptec"},
 	{0,0}
 };
 
-static const char *const majclasses[] = {
-	"old", "storage", "network", "display",
-	"multimedia", "memory", "bridge" 
+typedef struct {
+	const int	subclass;
+	const char	*name;
+} subclass_name;
+
+/* 0x00 prehistoric subclasses */
+static const subclass_name old_subclasses[] =
+{
+	{ 0x00, "misc"	},
+	{ 0x01, "vga"	},
+	{ 0x00, NULL	}
 };
 
+/* 0x01 mass storage subclasses */
+static const subclass_name storage_subclasses[] =
+{
+	{ 0x00, "scsi"	},
+	{ 0x01, "ide"	},
+	{ 0x02, "floppy"},
+	{ 0x03, "ipi"	},
+	{ 0x80, "misc"	},
+	{ 0x00, NULL	}
+};
+
+/* 0x02 network subclasses */
+static const subclass_name network_subclasses[] =
+{
+	{ 0x00, "ethernet"	},
+	{ 0x01, "tokenring"	},
+	{ 0x02, "fddi"	},
+	{ 0x80, "misc"	},
+	{ 0x00, NULL	}
+};
+
+/* 0x03 display subclasses */
+static const subclass_name display_subclasses[] =
+{
+	{ 0x00, "vga"	},
+	{ 0x01, "xga"	},
+	{ 0x80, "misc"	},
+	{ 0x00, NULL	}
+};
+
+/* 0x04 multimedia subclasses */
+static const subclass_name multimedia_subclasses[] =
+{
+	{ 0x00, "video"	},
+	{ 0x01, "audio"	},
+	{ 0x80, "misc"	},
+	{ 0x00, NULL	}
+};
+
+/* 0x05 memory subclasses */
+static const subclass_name memory_subclasses[] =
+{
+	{ 0x00, "ram"	},
+	{ 0x01, "flash"	},
+	{ 0x80, "misc"	},
+	{ 0x00, NULL	}
+};
+
+/* 0x06 bridge subclasses */
+static const subclass_name bridge_subclasses[] =
+{
+	{ 0x00, "host"	},
+	{ 0x01, "isa"	},
+	{ 0x02, "eisa"	},
+	{ 0x03, "mc"	},
+	{ 0x04, "pci"	},
+	{ 0x05, "pcmcia"},
+	{ 0x80, "misc"	},
+	{ 0x00, NULL	}
+};
+
+static const subclass_name *const subclasses[] = {
+	old_subclasses, 
+	storage_subclasses, 
+	network_subclasses, 
+	display_subclasses,
+	multimedia_subclasses,
+	memory_subclasses, 
+	bridge_subclasses,
+};
+
+static const char *const majclasses[] = {
+	"old", 
+	"storage", 
+	"network", 
+	"display",
+	"multimedia", 
+	"memory", 
+	"bridge"
+};
+
+
 void not_supported (pcici_t tag, u_long type)
 {
-	u_char	reg;
+	u_long	reg;
 	u_long	data;
+	u_char	class;
+	u_char	subclass;
 	struct vt * vp;
+	int	pciint;
+	int	irq;
 
 	/*
 	**	lookup the names.
@@ -1498,39 +1682,92 @@ void not_supported (pcici_t tag, u_long type)
 	*/
 
 	if (vp->ident) printf (vp->name);
-		else   printf ("vendor=0x%lx", type & 0xffff);
+		else   printf ("vendor=0x%04lx", type & 0xffff);
 
-	printf (", device=0x%lx", type >> 16);
+	printf (", device=0x%04lx", type >> 16);
 
-	data = (pcibus->pb_read(tag, PCI_CLASS_REG) >> 24) & 0xff;
-	if (data < sizeof(majclasses) / sizeof(majclasses[0]))
-		printf(", class=%s", majclasses[data]);
+	data = pci_conf_read(tag, PCI_CLASS_REG);
+	class = (data >> 24) & 0xff;
+	subclass = (data >> 16) & 0xff;
 
-	printf (" [not supported]\n");
+	if (class < sizeof(majclasses) / sizeof(majclasses[0])) {
+		printf(", class=%s", majclasses[class]);
+	} else {
+		printf(", class=0x%02x", class);
+	}
 
-	for (reg=PCI_MAP_REG_START; reg<PCI_MAP_REG_END; reg+=4) {
-		data = pcibus->pb_read (tag, reg);
-		if ((data&~7)==0) continue;
-		switch (data&7) {
+	if (class < sizeof(subclasses) / sizeof(subclasses[0])) {
+		const subclass_name *p = subclasses[class];
+		while (p->name && (p->subclass != subclass)) 
+			p++;
+		if (p->name) {
+			printf(" (%s)", p->name);
+		} else {
+			printf(" (unknown subclass 0x%02lx)", subclass);
+		}
+	} else {
+		printf(", subclass=0x%02x", subclass);
+	}
+
+	data = pci_conf_read (tag, PCI_INTERRUPT_REG);
+	pciint = PCI_INTERRUPT_PIN_EXTRACT(data);
+
+	if (pciint) {
+
+		printf (" int %c irq ", 0x60+pciint);
+
+		irq = PCI_INTERRUPT_LINE_EXTRACT(data);
+
+		/*
+		**	If it's zero, the isa irq number is unknown,
+		**	and we cannot bind the pci interrupt.
+		*/
+
+		if (irq && (irq != 0xff))
+			printf ("%d", irq);
+		else
+			printf ("??");
+	};
+
+	if (class != (PCI_CLASS_BRIDGE >> 24))
+	    printf (" [no driver assigned]");
+	printf ("\n");
+
+	if (bootverbose) {
+	    if (class == (PCI_CLASS_BRIDGE >> 24)) {
+		printf ("configuration space registers:");
+		for (reg = 0; reg < 0x100; reg+=4) {
+		    if ((reg & 0x0f) == 0) printf ("\n%02x:\t", reg);
+		    printf ("%08x ", pci_conf_read (tag, reg));
+		}
+		printf ("\n");
+	    } else {
+		for (reg=PCI_MAP_REG_START; reg<PCI_MAP_REG_END; reg+=4) {
+		    data = pci_conf_read (tag, reg);
+		    if ((data&~7)==0) continue;
+		    switch (data&7) {
 
 		case 1:
 		case 5:
-			printf ("	map(%x): io(%lx)\n",
+			printf ("\tmap(%x): io(%04lx)\n",
 				reg, data & ~3);
 			break;
 		case 0:
-			printf ("	map(%x): mem32(%lx)\n",
+			printf ("\tmap(%x): mem32(%08lx)\n",
 				reg, data & ~7);
 			break;
 		case 2:
-			printf ("	map(%x): mem20(%lx)\n",
+			printf ("\tmap(%x): mem20(%05lx)\n",
 				reg, data & ~7);
 			break;
 		case 4:
-			printf ("	map(%x): mem64(%lx)\n",
-				reg, data & ~7);
+			printf ("\tmap(%x): mem64(%08x%08lx)\n",
+				reg, pci_conf_read (tag, reg +4), data & ~7);
+			reg += 4;
 			break;
+		    }
 		}
+	    }
 	}
 }
 #endif /* NPCI */
