@@ -14,7 +14,7 @@
  *
  * Ported to run under 386BSD by Julian Elischer (julian@tfs.com) Sept 1992
  *
- *      $Id: cd.c,v 1.39 1995/04/23 22:07:48 gibbs Exp $
+ * $FreeBSD: stable/2.1/sys/scsi/cd.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #define SPLCD splbio
@@ -43,20 +43,20 @@
 
 /* static function prototypes */
 static errval cd_get_parms __P((int, int));
-static errval cd_get_mode __P((u_int32, struct cd_mode_data *, u_int32));
-static errval cd_set_mode __P((u_int32 unit, struct cd_mode_data *));
-static errval cd_read_toc __P((u_int32, u_int32, u_int32, struct cd_toc_entry *,
-			  u_int32));
+static errval cd_get_mode __P((u_int32_t, struct cd_mode_data *, u_int32_t));
+static errval cd_set_mode __P((u_int32_t unit, struct cd_mode_data *));
+static errval cd_read_toc __P((u_int32_t, u_int32_t, u_int32_t, struct cd_toc_entry *,
+			  u_int32_t));
 
-static errval cd_pause __P((u_int32, u_int32));
-static errval cd_reset __P((u_int32));
-static errval cd_play_msf __P((u_int32, u_int32, u_int32, u_int32, u_int32, u_int32, u_int32));
-static errval cd_play __P((u_int32, u_int32, u_int32));
-static errval cd_play_tracks __P((u_int32, u_int32, u_int32, u_int32, u_int32));
-static errval cd_read_subchannel __P((u_int32, u_int32, u_int32, int, struct cd_sub_channel_info *, u_int32));
-static errval cd_getdisklabel __P((u_int8));
+static errval cd_pause __P((u_int32_t, u_int32_t));
+static errval cd_reset __P((u_int32_t));
+static errval cd_play_msf __P((u_int32_t, u_int32_t, u_int32_t, u_int32_t, u_int32_t, u_int32_t, u_int32_t));
+static errval cd_play __P((u_int32_t, u_int32_t, u_int32_t));
+static errval cd_play_tracks __P((u_int32_t, u_int32_t, u_int32_t, u_int32_t, u_int32_t));
+static errval cd_read_subchannel __P((u_int32_t, u_int32_t, u_int32_t, int, struct cd_sub_channel_info *, u_int32_t));
+static errval cd_getdisklabel __P((u_int8_t));
 
-int32   cdstrats, cdqueues;
+int32_t   cdstrats, cdqueues;
 
 #define CDUNIT(DEV)      ((minor(DEV)&0xF8) >> 3)    /* 5 bit unit */
 #define CDSETUNIT(DEV, U) makedev(major(DEV), ((U) << 3))
@@ -69,20 +69,20 @@ int32   cdstrats, cdqueues;
 #define PARTITION(z)	(minor(z) & 0x07)
 #define RAW_PART        2
 
-void	cdstart(u_int32 unit, u_int32 flags);
+void	cdstart(u_int32_t unit, u_int32_t flags);
 
 struct scsi_data {
-	u_int32 flags;
+	u_int32_t flags;
 #define	CDINIT		0x04	/* device has been init'd */
 	struct cd_parms {
-		u_int32 blksize;
+		u_int32_t blksize;
 		u_long  disksize;	/* total number sectors */
 	} params;
 	struct disklabel disklabel;
-	u_int32 partflags[MAXPARTITIONS];	/* per partition flags */
+	u_int32_t partflags[MAXPARTITIONS];	/* per partition flags */
 #define CDOPEN	0x01
-	u_int32 openparts;	/* one bit for each open partition */
-	u_int32 xfer_block_wait;
+	u_int32_t openparts;	/* one bit for each open partition */
+	u_int32_t xfer_block_wait;
 	struct buf buf_queue;
 	int dkunit;
 };
@@ -128,7 +128,7 @@ struct scsi_device cd_switch =
 #define CD_EJECT	-2
 
 static int
-cd_externalize(struct proc *p, struct kern_devconf *kdc, void *userp, 
+cd_externalize(struct proc *p, struct kern_devconf *kdc, void *userp,
 	       size_t len)
 {
 	return scsi_externalize(SCSI_LINK(&cd_switch, kdc->kdc_unit),
@@ -170,10 +170,10 @@ cd_registerdev(int unit)
  * The routine called by the low level scsi routine when it discovers
  * A device suitable for this driver
  */
-int 
+int
 cdattach(struct scsi_link *sc_link)
 {
-	u_int32 unit;
+	u_int32_t unit;
 	struct cd_parms *dp;
 	struct scsi_data *cd = sc_link->sd;
 
@@ -211,12 +211,12 @@ cdattach(struct scsi_link *sc_link)
 /*
  * open the device. Make sure the partition info is a up-to-date as can be.
  */
-errval 
+errval
 cd_open(dev_t dev, int flags, int fmt, struct proc *p,
 struct scsi_link *sc_link)
 {
 	errval  errcode = 0;
-	u_int32 unit, part;
+	u_int32_t unit, part;
 	struct scsi_data *cd;
 
 	unit = CDUNIT(dev);
@@ -268,7 +268,7 @@ struct scsi_link *sc_link)
 	scsi_prevent(sc_link, PR_PREVENT, SCSI_SILENT);
 	SC_DEBUG(sc_link, SDEV_DB3, ("'start' attempted "));
 	/*
-	 * Load the physical device parameters 
+	 * Load the physical device parameters
 	 */
 	if (cd_get_parms(unit, 0)) {
 		errcode = ENXIO;
@@ -325,11 +325,11 @@ struct scsi_link *sc_link)
  * close the device.. only called if we are the LAST
  * occurence of an open device
  */
-errval 
+errval
 cd_close(dev_t dev, int flag, int fmt, struct proc *p,
         struct scsi_link *sc_link)
 {
-	u_int8  unit, part;
+	u_int8_t  unit, part;
 	struct scsi_data *cd;
 
 	unit = CDUNIT(dev);
@@ -356,12 +356,12 @@ cd_close(dev_t dev, int flag, int fmt, struct proc *p,
  * understand.  The transfer is described by a buf and will include only one
  * physical transfer.
  */
-void 
+void
 cd_strategy(struct buf *bp, struct scsi_link *sc_link)
 {
 	struct buf *dp;
-	u_int32 opri;
-	u_int32 unit = CDUNIT((bp->b_dev));
+	u_int32_t opri;
+	u_int32_t unit = CDUNIT((bp->b_dev));
 	struct scsi_data *cd = sc_link->sd;
 
 	cdstrats++;
@@ -447,15 +447,15 @@ cd_strategy(struct buf *bp, struct scsi_link *sc_link)
  * must be called at the correct (highish) spl level
  * cdstart() is called at SPLCD  from cdstrategy and scsi_done
  */
-void 
+void
 cdstart(unit, flags)
-	u_int32 unit;
-	u_int32 flags;
+	u_int32_t unit;
+	u_int32_t flags;
 {
 	register struct buf *bp = 0;
 	register struct buf *dp;
 	struct scsi_rw_big cmd;
-	u_int32 blkno, nblk;
+	u_int32_t blkno, nblk;
 	struct partition *p;
 	struct scsi_link *sc_link = SCSI_LINK(&cd_switch, unit);
 	struct scsi_data *cd = sc_link->sd;
@@ -484,7 +484,7 @@ cdstart(unit, flags)
 		goto bad;	/* no I/O.. media changed or something */
 	}
 	/*
-	 * We have a buf, now we should make a command 
+	 * We have a buf, now we should make a command
 	 *
 	 * First, translate the block to absolute and put it in terms of the
 	 * logical blocksize of the device.  Really a bit silly until we have
@@ -533,7 +533,7 @@ cdstart(unit, flags)
 		return;
 	}
 	cdqueues++;
-	if(cd->dkunit) {
+	if(cd->dkunit >= 0) {
 		dk_xfer[cd->dkunit]++;
 		dk_seek[cd->dkunit]++; /* don't know */
 		dk_wds[cd->dkunit] += bp->b_bcount >> 6;
@@ -544,12 +544,12 @@ cdstart(unit, flags)
  * Perform special action on behalf of the user.
  * Knows about the internals of this device
  */
-errval 
+errval
 cd_ioctl(dev_t dev, int cmd, caddr_t addr, int flag, struct proc *p,
 struct scsi_link *sc_link)
 {
 	errval  error = 0;
-	u_int8  unit, part;
+	u_int8_t  unit, part;
 	register struct scsi_data *cd;
 
 	/*
@@ -582,7 +582,7 @@ struct scsi_link *sc_link)
 		break;
 
 		/*
-		 * a bit silly, but someone might want to test something on a 
+		 * a bit silly, but someone might want to test something on a
 		 * section of cdrom.
 		 */
 	case DIOCWDINFO:
@@ -665,7 +665,7 @@ struct scsi_link *sc_link)
 			struct ioc_read_subchannel *args
 			= (struct ioc_read_subchannel *) addr;
 			struct cd_sub_channel_info data;
-			u_int32 len = args->data_len;
+			u_int32_t len = args->data_len;
 			if (len > sizeof(data) ||
 			    len < sizeof(struct cd_sub_channel_header)) {
 				error = EINVAL;
@@ -675,7 +675,7 @@ struct scsi_link *sc_link)
 				args->data_format, args->track, &data, len);
 			if (error)
 				break;
-			len = min(len, ((data.header.data_len[0] << 8) + 
+			len = min(len, ((data.header.data_len[0] << 8) +
 				data.header.data_len[1] +
 				sizeof(struct cd_sub_channel_header)));
 			if (copyout(&data, args->data, len) != 0) {
@@ -686,7 +686,7 @@ struct scsi_link *sc_link)
 	case CDIOREADTOCHEADER:
 		{		/* ??? useless bcopy? XXX */
 			struct ioc_toc_header th;
-			error = cd_read_toc(unit, 0, 0, 
+			error = cd_read_toc(unit, 0, 0,
 					(struct cd_toc_entry *)&th, sizeof th);
 			if (error)
 				break;
@@ -703,7 +703,7 @@ struct scsi_link *sc_link)
 			struct ioc_read_toc_entry *te =
 			(struct ioc_read_toc_entry *) addr;
 			struct ioc_toc_header *th;
-			u_int32 len = te->data_len;
+			u_int32_t len = te->data_len;
 			th = &data.header;
 
 			if (len > sizeof(data.entries) || len < sizeof(struct cd_toc_entry)) {
@@ -775,7 +775,7 @@ struct scsi_link *sc_link)
 			error = cd_get_mode(unit, &data, AUDIO_PAGE);
 			if (error)
 				break;
-			data.page.audio.port[LEFT_PORT].channels = LEFT_CHANNEL | RIGHT_CHANNEL | 4 | 8;
+			data.page.audio.port[LEFT_PORT].channels = LEFT_CHANNEL | RIGHT_CHANNEL;
 			data.page.audio.port[RIGHT_PORT].channels = LEFT_CHANNEL | RIGHT_CHANNEL;
 			data.page.audio.port[2].channels = 0;
 			data.page.audio.port[3].channels = 0;
@@ -784,7 +784,7 @@ struct scsi_link *sc_link)
 				break;
 		}
 		break;
-	case CDIOCSETSTERIO:
+	case CDIOCSETSTEREO:
 		{
 			struct cd_mode_data data;
 			error = cd_get_mode(unit, &data, AUDIO_PAGE);
@@ -887,13 +887,13 @@ struct scsi_link *sc_link)
 /*
  * Load the label information on the named device
  * Actually fabricate a disklabel
- * 
+ *
  * EVENTUALLY take information about different
  * data tracks from the TOC and put it in the disklabel
  */
-errval 
+errval
 cd_getdisklabel(unit)
-	u_int8  unit;
+	u_int8_t  unit;
 {
 	/*unsigned int n, m; */
 	struct scsi_data *cd;
@@ -940,15 +940,15 @@ cd_getdisklabel(unit)
 /*
  * Find out from the device what it's capacity is
  */
-u_int32 
+u_int32_t
 cd_size(unit, flags)
 	int unit;
 	int flags;
 {
 	struct scsi_read_cd_cap_data rdcap;
 	struct scsi_read_cd_capacity scsi_cmd;
-	u_int32 size;
-	u_int32 blksize;
+	u_int32_t size;
+	u_int32_t blksize;
 	struct scsi_link *sc_link = SCSI_LINK(&cd_switch, unit);
 	struct scsi_data *cd = sc_link->sd;
 
@@ -997,11 +997,11 @@ cd_size(unit, flags)
 /*
  * Get the requested page into the buffer given
  */
-static errval 
+static errval
 cd_get_mode(unit, data, page)
-	u_int32 unit;
+	u_int32_t unit;
 	struct cd_mode_data *data;
-	u_int32 page;
+	u_int32_t page;
 {
 	struct scsi_mode_sense scsi_cmd;
 	errval  retval;
@@ -1026,9 +1026,9 @@ cd_get_mode(unit, data, page)
 /*
  * Get the requested page into the buffer given
  */
-errval 
+errval
 cd_set_mode(unit, data)
-	u_int32 unit;
+	u_int32_t unit;
 	struct cd_mode_data *data;
 {
 	struct scsi_mode_select scsi_cmd;
@@ -1038,6 +1038,12 @@ cd_set_mode(unit, data)
 	scsi_cmd.byte2 |= SMS_PF;
 	scsi_cmd.length = sizeof(*data) & 0xff;
 	data->header.data_length = 0;
+	/*
+	 * SONY drives do not allow a mode select with a medium_type
+	 * value that has just been returned by a mode sense; use a
+	 * medium_type of 0 (Default) instead.
+	 */
+	data->header.medium_type = 0;
 	return (scsi_scsi_cmd(SCSI_LINK(&cd_switch, unit),
 		(struct scsi_generic *) &scsi_cmd,
 		sizeof(scsi_cmd),
@@ -1052,9 +1058,9 @@ cd_set_mode(unit, data)
 /*
  * Get scsi driver to send a "start playing" command
  */
-errval 
+errval
 cd_play(unit, blk, len)
-	u_int32 unit, blk, len;
+	u_int32_t unit, blk, len;
 {
 	struct scsi_play scsi_cmd;
 
@@ -1080,9 +1086,9 @@ cd_play(unit, blk, len)
 /*
  * Get scsi driver to send a "start playing" command
  */
-errval 
+errval
 cd_play_big(unit, blk, len)
-	u_int32 unit, blk, len;
+	u_int32_t unit, blk, len;
 {
 	struct scsi_play_big scsi_cmd;
 
@@ -1110,9 +1116,9 @@ cd_play_big(unit, blk, len)
 /*
  * Get scsi driver to send a "start playing" command
  */
-errval 
+errval
 cd_play_tracks(unit, strack, sindex, etrack, eindex)
-	u_int32 unit, strack, sindex, etrack, eindex;
+	u_int32_t unit, strack, sindex, etrack, eindex;
 {
 	struct scsi_play_track scsi_cmd;
 
@@ -1136,9 +1142,9 @@ cd_play_tracks(unit, strack, sindex, etrack, eindex)
 /*
  * Get scsi driver to send a "play msf" command
  */
-errval 
+errval
 cd_play_msf(unit, startm, starts, startf, endm, ends, endf)
-	u_int32 unit, startm, starts, startf, endm, ends, endf;
+	u_int32_t unit, startm, starts, startf, endm, ends, endf;
 {
 	struct scsi_play_msf scsi_cmd;
 
@@ -1165,9 +1171,9 @@ cd_play_msf(unit, startm, starts, startf, endm, ends, endf)
 /*
  * Get scsi driver to send a "start up" command
  */
-errval 
+errval
 cd_pause(unit, go)
-	u_int32 unit, go;
+	u_int32_t unit, go;
 {
 	struct scsi_pause scsi_cmd;
 
@@ -1189,9 +1195,9 @@ cd_pause(unit, go)
 /*
  * Get scsi driver to send a "RESET" command
  */
-errval 
+errval
 cd_reset(unit)
-	u_int32 unit;
+	u_int32_t unit;
 {
 	return scsi_reset_target(SCSI_LINK(&cd_switch, unit));
 }
@@ -1199,12 +1205,12 @@ cd_reset(unit)
 /*
  * Read subchannel
  */
-errval 
+errval
 cd_read_subchannel(unit, mode, format, track, data, len)
-	u_int32 unit, mode, format;
+	u_int32_t unit, mode, format;
 	int track;
 	struct cd_sub_channel_info *data;
-	u_int32 len;
+	u_int32_t len;
 {
 	struct scsi_read_subchannel scsi_cmd;
 
@@ -1232,14 +1238,14 @@ cd_read_subchannel(unit, mode, format, track, data, len)
 /*
  * Read table of contents
  */
-static errval 
+static errval
 cd_read_toc(unit, mode, start, data, len)
-	u_int32 unit, mode, start;
+	u_int32_t unit, mode, start;
 	struct cd_toc_entry *data;
-	u_int32 len;
+	u_int32_t len;
 {
 	struct scsi_read_toc scsi_cmd;
-	u_int32 ntoc;
+	u_int32_t ntoc;
 
 	bzero(&scsi_cmd, sizeof(scsi_cmd));
 	/*if(len!=sizeof(struct ioc_toc_header))
@@ -1270,7 +1276,7 @@ cd_read_toc(unit, mode, start, data, len)
  * Get the scsi driver to send a full inquiry to the device and use the
  * results to fill out the disk parameter structure.
  */
-static errval 
+static errval
 cd_get_parms(unit, flags)
 	int unit;
 	int flags;
@@ -1284,7 +1290,7 @@ cd_get_parms(unit, flags)
 		return (0);
 	/*
 	 * give a number of sectors so that sec * trks * cyls
-	 * is <= disk_size 
+	 * is <= disk_size
 	 */
 	if (cd_size(unit, flags)) {
 		sc_link->flags |= SDEV_MEDIA_LOADED;
