@@ -2,13 +2,13 @@
  * Written By Julian ELischer
  * Copyright julian Elischer 1993.
  * Permission is granted to use or redistribute this file in any way as long
- * as this notice remains. Julian Elischer does not guarantee that this file 
- * is totally correct for any given task and users of this file must 
+ * as this notice remains. Julian Elischer does not guarantee that this file
+ * is totally correct for any given task and users of this file must
  * accept responsibility for any damage that occurs from the application of this
  * file.
- * 
+ *
  * Written by Julian Elischer (julian@dialix.oz.au)
- *      $Id: scsi_base.c,v 1.27 1995/04/14 15:10:31 dufault Exp $
+ * $FreeBSD: stable/2.1/sys/scsi/scsi_base.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #define SPLSD splbio
@@ -34,7 +34,7 @@ struct scsi_xfer *next_free_xs;
 
 /*
  * Get a scsi transfer structure for the caller. Charge the structure
- * to the device that is referenced by the sc_link structure. If the 
+ * to the device that is referenced by the sc_link structure. If the
  * sc_link structure has no 'credits' then the device already has the
  * maximum number or outstanding operations under way. In this stage,
  * wait on the structure so that when one is freed, we are awoken again
@@ -46,10 +46,10 @@ struct scsi_xfer *next_free_xs;
 struct scsi_xfer *
 get_xs(sc_link, flags)
 	struct	scsi_link *sc_link;	/* who to charge the xs to */
-	u_int32	flags;			/* if this call can sleep */
+	u_int32_t	flags;			/* if this call can sleep */
 {
 	struct	scsi_xfer *xs;
-	u_int32	s;
+	u_int32_t	s;
 
 	SC_DEBUG(sc_link, SDEV_DB3, ("get_xs\n"));
 	s = splbio();
@@ -88,11 +88,11 @@ get_xs(sc_link, flags)
  * return the struct to the free pool and credit the device with it
  * If another process is waiting for an xs, do a wakeup, let it proceed
  */
-void 
+void
 free_xs(xs, sc_link, flags)
 	struct scsi_xfer *xs;
 	struct scsi_link *sc_link;	/* who to credit for returning it */
-	u_int32 flags;
+	u_int32_t flags;
 {
 	xs->next = next_free_xs;
 	next_free_xs = xs;
@@ -117,15 +117,15 @@ free_xs(xs, sc_link, flags)
 /*
  * Find out from the device what its capacity is.
  */
-u_int32 
+u_int32_t
 scsi_read_capacity(sc_link, blk_size, flags)
 	struct scsi_link *sc_link;
-	u_int32 *blk_size;
-	u_int32 flags;
+	u_int32_t *blk_size;
+	u_int32_t flags;
 {
 	struct scsi_read_cap_data rdcap;
 	struct scsi_read_capacity scsi_cmd;
-	u_int32 size;
+	u_int32_t size;
 
 	/*
 	 * make up a scsi command and ask the scsi driver to do
@@ -143,8 +143,8 @@ scsi_read_capacity(sc_link, blk_size, flags)
 		sizeof(scsi_cmd),
 		(u_char *) & rdcap,
 		sizeof(rdcap),
-		2,
-		20000,
+		4,
+		5000000,	/* WORMs tend to take a HUGE amount of time */
 		NULL,
 		flags | SCSI_DATA_IN) != 0) {
 
@@ -198,10 +198,10 @@ scsi_target_mode(sc_link, on_off)
 /*
  * Get scsi driver to send a "are you ready?" command
  */
-errval 
+errval
 scsi_test_unit_ready(sc_link, flags)
 	struct scsi_link *sc_link;
-	u_int32 flags;
+	u_int32_t flags;
 {
 	struct scsi_test_unit_ready scsi_cmd;
 
@@ -222,10 +222,10 @@ scsi_test_unit_ready(sc_link, flags)
 /*
  * Do a scsi operation, asking a device to run as SCSI-II if it can.
  */
-errval 
+errval
 scsi_change_def(sc_link, flags)
 	struct scsi_link *sc_link;
-	u_int32 flags;
+	u_int32_t flags;
 {
 	struct scsi_changedef scsi_cmd;
 
@@ -248,11 +248,11 @@ scsi_change_def(sc_link, flags)
  * Do a scsi operation asking a device what it is
  * Use the scsi_cmd routine in the switch table.
  */
-errval 
+errval
 scsi_inquire(sc_link, inqbuf, flags)
 	struct scsi_link *sc_link;
 	struct scsi_inquiry_data *inqbuf;
-	u_int32 flags;
+	u_int32_t flags;
 {
 	struct scsi_inquiry scsi_cmd;
 
@@ -274,10 +274,10 @@ scsi_inquire(sc_link, inqbuf, flags)
 /*
  * Prevent or allow the user to remove the media
  */
-errval 
+errval
 scsi_prevent(sc_link, type, flags)
 	struct scsi_link *sc_link;
-	u_int32 type, flags;
+	u_int32_t type, flags;
 {
 	struct scsi_prevent scsi_cmd;
 
@@ -298,10 +298,10 @@ scsi_prevent(sc_link, type, flags)
 /*
  * Get scsi driver to send a "start up" command
  */
-errval 
+errval
 scsi_start_unit(sc_link, flags)
 	struct scsi_link *sc_link;
-	u_int32 flags;
+	u_int32_t flags;
 {
 	struct scsi_start_stop scsi_cmd;
 
@@ -323,11 +323,11 @@ scsi_start_unit(sc_link, flags)
 /*
  * Get scsi driver to send a "stop" command
  */
-errval 
+errval
 scsi_stop_unit(sc_link, eject, flags)
 	struct scsi_link *sc_link;
-	u_int32 eject;
-	u_int32 flags;
+	u_int32_t eject;
+	u_int32_t flags;
 {
 	struct scsi_start_stop scsi_cmd;
 
@@ -351,7 +351,7 @@ scsi_stop_unit(sc_link, eject, flags)
 /*
  * This routine is called by the scsi interrupt when the transfer is complete.
  */
-void 
+void
 scsi_done(xs)
 	struct scsi_xfer *xs;
 {
@@ -395,7 +395,7 @@ scsi_done(xs)
 		}
 		/* XXX: This isn't used anywhere. Do you have plans for it,
 		 * Julian? (dufault@hda.com).
-		 * This allows a private 'done' handler to 
+		 * This allows a private 'done' handler to
 		 * resubmit the command if it wants to retry,
 		 * In this case the xs must NOT be freed. (julian)
 		 */
@@ -435,22 +435,22 @@ scsi_done(xs)
  * long the data is supposed to be. If we have  a buf
  * to associate with the transfer, we need that too.
  */
-errval 
+errval
 scsi_scsi_cmd(sc_link, scsi_cmd, cmdlen, data_addr, datalen,
     retries, timeout, bp, flags)
 	struct scsi_link *sc_link;
 	struct scsi_generic *scsi_cmd;
-	u_int32 cmdlen;
+	u_int32_t cmdlen;
 	u_char *data_addr;
-	u_int32 datalen;
-	u_int32 retries;
-	u_int32 timeout;
+	u_int32_t datalen;
+	u_int32_t retries;
+	u_int32_t timeout;
 	struct buf *bp;
-	u_int32 flags;
+	u_int32_t flags;
 {
 	struct scsi_xfer *xs;
 	errval  retval;
-	u_int32 s;
+	u_int32_t s;
 
 	/*
 	 * Illegal command lengths will wedge host adapter software.
@@ -463,8 +463,8 @@ scsi_scsi_cmd(sc_link, scsi_cmd, cmdlen, data_addr, datalen,
 			return EFAULT;
 		else
 		{
-			static u_int8 sizes[] = {6, 10, 10, 0, 0, 12, 0, 0 };
-			u_int8 size = sizes[((scsi_cmd->opcode) >> 5)];
+			static u_int8_t sizes[] = {6, 10, 10, 0, 0, 12, 0, 0 };
+			u_int8_t size = sizes[((scsi_cmd->opcode) >> 5)];
 			if (size && (size != cmdlen))
 				return EIO;
 		}
@@ -502,7 +502,7 @@ scsi_scsi_cmd(sc_link, scsi_cmd, cmdlen, data_addr, datalen,
 #ifdef BOUNCE_BUFFERS
 		xs->data = (caddr_t) vm_bounce_kva_alloc( (datalen + PAGE_SIZE - 1)/PAGE_SIZE);
 #else
-		xs->data = malloc(datalen, M_TEMP, M_WAITOK); 
+		xs->data = malloc(datalen, M_TEMP, M_WAITOK);
 #endif
 		/* I think waiting is ok *//*XXX */
 		switch ((int)(flags & (SCSI_DATA_IN | SCSI_DATA_OUT))) {
@@ -533,12 +533,12 @@ retry:
 	 * Do the transfer. If we are polling we will return:
 	 * COMPLETE,  Was poll, and scsi_done has been called
 	 * TRY_AGAIN_LATER, Adapter short resources, try again
-	 * 
+	 *
 	 * if under full steam (interrupts) it will return:
 	 * SUCCESSFULLY_QUEUED, will do a wakeup when complete
 	 * TRY_AGAIN_LATER, (as for polling)
 	 * After the wakeup, we must still check if it succeeded
-	 * 
+	 *
 	 * If we have a bp however, all the error proccessing
 	 * and the buffer code both expect us to return straight
 	 * to them, so as soon as the command is queued, return
@@ -599,7 +599,9 @@ retry:
 	 * check if anyone else needs to be started up.
 	 */
 bad:
+	s = splbio();
 	free_xs(xs, sc_link, flags);	/* includes the 'start' op */
+	splx(s);
 	if (bp && retval) {
 		bp->b_error = retval;
 		bp->b_flags |= B_ERROR;
@@ -710,13 +712,13 @@ errval scsi_retry(xs,delay)
 
 /*
  * handle checking for errors..
- * called at interrupt time from scsi_done() and 
+ * called at interrupt time from scsi_done() and
  * at user time from scsi_scsi_cmd(), depending on whether
  * there was a bp  (basically, if there is a bp, there may be no
  * associated process at the time. (it could be an async operation))
  * lower level routines shouldn't know about xs->bp.. we are the lowest.
  */
-static errval 
+static errval
 sc_err1(xs)
 	struct scsi_xfer *xs;
 {
@@ -740,6 +742,7 @@ sc_err1(xs)
 		return sc_done(xs, SCSIRET_DO_RETRY);
 
 		/* fall through */
+	case XS_SELTIMEOUT:
 	case XS_DRIVER_STUFFUP:
 		return sc_done(xs, EIO);
 
@@ -782,8 +785,8 @@ void scsi_sense_print(xs)
 {
 	struct scsi_sense_data_new *sense;
 	struct scsi_sense_extended *ext;
-	u_int32 key;
-	u_int32 info;
+	u_int32_t key;
+	u_int32_t info;
 	int asc, ascq;
 
 	/* This sense key text now matches what is in the SCSI spec
@@ -870,12 +873,12 @@ void scsi_sense_print(xs)
 
 			printf(" %s", desc);
 		}
-			
+
 		if (ext->extra_len >= 7 && ext->fru) {
 			printf(" field replaceable unit: %x", ext->fru);
 		}
 
-		if (ext->extra_len >= 10 && 
+		if (ext->extra_len >= 10 &&
 		(ext->sense_key_spec_1 & SSD_SCS_VALID)) {
 			printf(" sks:%x,%x", ext->sense_key_spec_1,
 			(ext->sense_key_spec_2 |
@@ -907,14 +910,14 @@ void scsi_sense_print(xs)
  *
  * THIS IS THE DEFAULT SENSE HANDLER
  */
-static errval 
+static errval
 scsi_interpret_sense(xs)
 	struct scsi_xfer *xs;
 {
 	struct scsi_sense_data *sense;
 	struct scsi_link *sc_link = xs->sc_link;
-	u_int32 key;
-	u_int32 silent;
+	u_int32_t key;
+	u_int32_t silent;
 	errval  errcode;
 	int error_code;
 
@@ -929,7 +932,7 @@ scsi_interpret_sense(xs)
 #ifdef	SCSIDEBUG
 	if (sc_link->flags & SDEV_DB1) {
 
-		u_int32 count = 0;
+		u_int32_t count = 0;
 		printf("code%x valid%x ",
 		    sense->error_code & SSD_ERRCODE,
 		    sense->error_code & SSD_ERRCODE_VALID ? 1 : 0);
@@ -1083,13 +1086,13 @@ scsi_interpret_sense(xs)
  */
 
 /*
- * convert a physical address to 3 bytes, 
+ * convert a physical address to 3 bytes,
  * MSB at the lowest address,
  * LSB at the highest.
  */
 void
 scsi_uto3b(val, bytes)
-	u_int32	val;
+	u_int32_t	val;
 	u_char	*bytes;
 {
 	*bytes++ = (val & 0xff0000) >> 16;
@@ -1097,43 +1100,43 @@ scsi_uto3b(val, bytes)
 	*bytes = val & 0xff;
 }
 
-u_int32
+u_int32_t
 scsi_3btou(bytes)
 	u_char *bytes;
 {
-	u_int32 rc;
+	u_int32_t rc;
 	rc = (*bytes++ << 16);
 	rc += (*bytes++ << 8);
 	rc += *bytes;
 	return rc;
 }
 
-int32
+int32_t
 scsi_3btoi(bytes)
 	u_char *bytes;
 {
-	u_int32 rc = scsi_3btou(bytes);
+	u_int32_t rc = scsi_3btou(bytes);
 
 	if (rc & 0x00800000)
 		rc |= 0xff000000;
 
-	return (int32) rc;
+	return (int32_t) rc;
 }
 
 void
 scsi_uto2b(val, bytes)
-	u_int32	val;
+	u_int32_t	val;
 	u_char	*bytes;
 {
 	*bytes++ = (val & 0xff00) >> 8;
 	*bytes =    val & 0xff;
 }
 
-u_int32
+u_int32_t
 scsi_2btou(bytes)
 	u_char *bytes;
 {
-	u_int32 rc;
+	u_int32_t rc;
 	rc  = (*bytes++ << 8);
 	rc +=  *bytes;
 	return rc;
@@ -1141,7 +1144,7 @@ scsi_2btou(bytes)
 
 void
 scsi_uto4b(val, bytes)
-	u_int32	val;
+	u_int32_t	val;
 	u_char	*bytes;
 {
 	*bytes++ = (val & 0xff000000) >> 24;
@@ -1150,11 +1153,11 @@ scsi_uto4b(val, bytes)
 	*bytes =    val & 0xff;
 }
 
-u_int32
+u_int32_t
 scsi_4btou(bytes)
 	u_char *bytes;
 {
-	u_int32 rc;
+	u_int32_t rc;
 	rc  = (*bytes++ << 24);
 	rc += (*bytes++ << 16);
 	rc += (*bytes++ << 8);
@@ -1279,9 +1282,9 @@ show_scsi_cmd(struct scsi_xfer *xs)
 void
 show_mem(address, num)
 	unsigned char *address;
-	u_int32 num;
+	u_int32_t num;
 {
-	u_int32 y;
+	u_int32_t y;
 	printf("------------------------------");
 	for (y = 0; y < num; y += 1) {
 		if (!(y % 16))
