@@ -16,7 +16,7 @@
  *
  * New configuration setup: dufault@hda.com
  *
- *      $Id: scsiconf.c,v 1.28 1995/04/23 07:47:10 bde Exp $
+ * $FreeBSD: stable/2.1/sys/scsi/scsiconf.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include <sys/types.h>
@@ -36,6 +36,8 @@
 #include "st.h"
 #include "cd.h"
 #include "ch.h"
+#include "worm.h"
+#include "od.h"
 
 #include "su.h"
 #include "sctarg.h"
@@ -60,6 +62,8 @@ struct extend_array
 	void **ps;
 };
 
+static errval scsi_attach_sctarg __P((void));
+
 static void *extend_alloc(size_t s)
 {
 	void *p = malloc(s, M_DEVBUF, M_NOWAIT);
@@ -145,7 +149,7 @@ struct extend_array *scbusses;
  * The structure of known drivers for autoconfiguration
  */
 struct scsidevs {
-	u_int32 type;
+	u_int32_t type;
 	boolean removable;
 	char   *manufacturer;
 	char   *model;
@@ -153,7 +157,7 @@ struct scsidevs {
 	char   *devname;
 	char    flags;		/* 1 show my comparisons during boot(debug) */
 #ifdef NEW_SCSICONF
-	u_int16	quirks;
+	u_int16_t	quirks;
 	void   *devmodes;
 #endif
 };
@@ -162,14 +166,14 @@ struct scsidevs {
 #define	SC_ONE_LU	0x00
 #define	SC_MORE_LUS	0x02
 
-static struct scsidevs unknowndev = 
+static struct scsidevs unknowndev =
 	{
-		T_UNKNOWN, 0, "*", "*", "*", 
+		T_UNKNOWN, 0, "*", "*", "*",
 		"uk", SC_MORE_LUS
 	};
 
 #ifdef NEW_SCSICONF
-static st_modes mode_tandberg3600 = 
+static st_modes mode_tandberg3600 =
 	{
 	    {0, 0, 0},					/* minor 0,1,2,3 */
 	    {0, ST_Q_FORCE_VAR_MODE, QIC_525},		/* minor 4,5,6,7 */
@@ -216,13 +220,38 @@ static st_modes mode_unktape =
 static struct scsidevs knowndevs[] =
 #ifdef NEW_SCSICONF
 {
+/* od's must be probed before sd's since some of them identify as T_DIRECT */
+#if NOD > 0
+        {
+		T_OPTICAL, T_REMOV, "MATSHITA", "PD-1 LF-1000", "*",
+		"od", SC_MORE_LUS
+	},
+	/*
+	 * The SONY SMO is not really supported here, since it
+	 * identifies as T_DIRECT, and thus doesn't get assigned
+	 * to the od driver.
+	 * You need to upgrade to the FreeBSD 2.2 line for this.
+	 */
+	{
+		T_OPTICAL, T_REMOV, "SONY", "SMO-*", "*",
+		"od", SC_MORE_LUS
+	},
+	{
+		T_OPTICAL, T_REMOV, "*", "*", "*",
+		"od", SC_ONE_LU
+	},
+#endif  /* NOD */
 #if NSD > 0
 	{
-		T_DIRECT, T_FIXED, "MAXTOR", "XT-4170S", "B5A", 
+		T_DIRECT, T_FIXED, "MAXTOR", "XT-4170S", "B5A",
 		"mx1", SC_ONE_LU
 	},
 	{
-		T_DIRECT, T_FIXED, "*", "*", "*", 
+		T_DIRECT, T_FIXED, "EMULEX", "MD21*" , "*",
+		"sd", SC_MORE_LUS
+	},
+	{
+		T_DIRECT, T_FIXED, "*", "*", "*",
 		"sd", SC_ONE_LU
 	},
 #endif	/* NSD */
@@ -248,34 +277,72 @@ static struct scsidevs knowndevs[] =
 		"st", SC_ONE_LU, 0, mode_wangdat1300
 	},
 	{
-		T_SEQUENTIAL, T_REMOV, "*", "*", "*", 
+		T_SEQUENTIAL, T_REMOV, "*", "*", "*",
 		"st", SC_ONE_LU, 0, mode_unktape
 	},
 #endif	/* NST */
 #if NCH > 0
 	{
-		T_CHANGER, T_REMOV, "*", "*", "*", 
+		T_CHANGER, T_REMOV, "*", "*", "*",
 		"ch", SC_ONE_LU
 	},
 #endif	/* NCH */
 #if NCD > 0
 #ifndef UKTEST	/* make cdroms unrecognised to test the uk driver */
 	{
-		T_READONLY, T_REMOV, "SONY", "CD-ROM CDU-8012", "3.1a", 
+		T_READONLY, T_REMOV, "SONY",    "CD-ROM CDU-8012", "3.1a",
 		"cd", SC_ONE_LU
 	},
 	{
 		T_READONLY, T_REMOV, "PIONEER", "CD-ROM DRM-600", "*",
 		"cd", SC_MORE_LUS
 	},
-#endif
+	{
+		T_READONLY, T_REMOV, "PIONEER", "CD-ROM DRM-602X" ,"*",
+		"cd", SC_MORE_LUS
+	},
+	{
+		T_READONLY, T_REMOV, "NRC",	"MBR-7", "*",
+		"cd", SC_MORE_LUS
+	},
+	{
+		T_READONLY, T_REMOV, "CHINON",  "CD-ROM CDS-535","*",
+		"cd", SC_ONE_LU
+	},
+#endif /* !UKTEST */
 #endif	/* NCD */
+#if NWORM > 0
+	{
+		T_WORM, T_REMOV, "YAMAHA", "CDR100", "*",
+		"worm", SC_ONE_LU
+	},
+	{
+		T_WORM, T_REMOV, "*", "*", "*",
+		"worm", SC_ONE_LU
+	},
+#endif /* NWORM */
 	{
 		0
 	}
 };
-#else
+#else /* !NEW_SCSICONF */
 {
+/* od's must be probed before sd's since some of them identify as T_DIRECT */
+#if NOD > 0
+	{
+		T_OPTICAL, T_REMOV, "standard", "any"
+		    ,"any", "od", SC_ONE_LU
+	},
+	{
+		T_OPTICAL, T_REMOV, "MATSHITA", "PD-1 LF-1000"
+		    ,"any", "od", SC_MORE_LUS
+	},
+	/* See comments under NEW_SCSICONF above. */
+	{
+		T_OPTICAL, T_REMOV, "SONY", "SMO-C501-09"
+		    ,"*", "od", SC_MORE_LUS
+	},
+#endif  /* NOD */
 #if NSD > 0
 	{
 		T_DIRECT, T_FIXED, "standard", "any"
@@ -301,15 +368,33 @@ static struct scsidevs knowndevs[] =
 #if NCD > 0
 #ifndef UKTEST	/* make cdroms unrecognised to test the uk driver */
 	{
-		T_READONLY, T_REMOV, "SONY    ", "CD-ROM CDU-8012 "
+		T_READONLY, T_REMOV, "SONY",   "CD-ROM CDU-8012"
 		    ,"3.1a", "cd", SC_ONE_LU
 	},
 	{
-		T_READONLY, T_REMOV, "PIONEER ", "CD-ROM DRM-600  "
+		T_READONLY, T_REMOV, "PIONEER", "CD-ROM DRM-600"
 		    ,"any", "cd", SC_MORE_LUS
 	},
-#endif
+	{
+		T_READONLY, T_REMOV, "PIONEER", "CD-ROM DRM-602X"
+		    ,"any", "cd", SC_MORE_LUS
+	},
+	{
+		T_READONLY, T_REMOV, "NRC",	"MBR-7"
+		    ,"any", "cd", SC_MORE_LUS
+	},
+	{
+		T_READONLY, T_REMOV, "CHINON",  "CD-ROM CDS-535"
+		    ,"any", "cd", SC_ONE_LU
+	},
+#endif /* !UKTEST */
 #endif	/* NCD */
+#if NWORM > 0
+	{
+		T_WORM, T_REMOV, "YAMAHA", "CDR100"
+		    ,"any", "worm", SC_ONE_LU
+	},
+#endif /* NWORM */
 	{
 		0
 	}
@@ -434,7 +519,7 @@ scsi_device_lookup_by_name(char *name)
 /* scsi_init: Do all the one time processing.  This initializes the
  * type drivers and initializes the configuration.
  */
-static void 
+static void
 scsi_init(void)
 {
 	static int done = 0;
@@ -459,10 +544,10 @@ scsi_init(void)
 		 * specified in config:
 		 */
 		for (i = 0; scsi_cinit[i].driver; i++)
-			if (IS_SPECIFIED(scsi_cinit[i].unit) &&
-			  free_bus <= scsi_cinit[i].unit)
-				free_bus = scsi_cinit[i].unit + 1;
-	
+			if (IS_SPECIFIED(scsi_cinit[i].scbus) &&
+			  free_bus <= scsi_cinit[i].scbus)
+				free_bus = scsi_cinit[i].scbus + 1;
+
 		/* Lowest free unit for each type for auto-configure is one
 		 * more than the first one not specified in the config file:
 		 */
@@ -500,22 +585,42 @@ scsi_bus_conf(sc_link_proto)
 
 	bus = SCCONF_UNSPEC;
 	for (i = 0; scsi_cinit[i].driver; i++) {
-		if (IS_SPECIFIED(scsi_cinit[i].unit))
+		if (IS_SPECIFIED(scsi_cinit[i].scbus))
 		{
-			if (!strcmp(sc_link_proto->adapter->name, scsi_cinit[i].driver) &&
-			(sc_link_proto->adapter_unit == scsi_cinit[i].unit) )
+			if (!strcmp(sc_link_proto->adapter->name, scsi_cinit[i].driver)
+			  &&(sc_link_proto->adapter_unit == scsi_cinit[i].unit))
 			{
-				bus = scsi_cinit[i].bus;
-				if (bootverbose)
-					printf("Choosing drivers for scbus configured at %d\n",
-					bus);
-				break;
+			  if (IS_SPECIFIED(scsi_cinit[i].bus)) {
+			     if (sc_link_proto->adapter_bus==scsi_cinit[i].bus){
+				bus = scsi_cinit[i].scbus;
+			   	break;
+			     }
+			  }
+			  else if (sc_link_proto->adapter_bus == 0) {
+			     /* Backwards compatibility for single bus cards */
+			     bus = scsi_cinit[i].scbus;
+			     break;
+			  }
+			  else {
+			     printf("Ambiguous scbus configuration for %s%d "
+				    "bus %d, cannot wire down.  The kernel "
+				    "config entry for scbus%d should specify "
+				    "a controller bus.\n"
+				    "Scbus will be assigned dynamically.\n",
+				    sc_link_proto->adapter->name,
+				    sc_link_proto->adapter_unit,
+				    sc_link_proto->adapter_bus);
+			     break;
+			  }
 			}
 		}
 	}
 
+			
 	if (bus == SCCONF_UNSPEC)
 		bus = free_bus++;
+	else if (bootverbose)
+		printf("Choosing drivers for scbus configured at %d\n", bus);
 
 	return bus;
 }
@@ -586,8 +691,6 @@ void scsi_configure_start(void)
 
 void scsi_configure_finish(void)
 {
-
-	static errval scsi_attach_sctarg __P((void));
 #if NSCTARG > 0
 	scsi_attach_sctarg();
 #endif
@@ -599,11 +702,11 @@ void scsi_configure_finish(void)
  * to get all their devices configured in.
  */
 void
-scsi_attachdevs(sc_link_proto)
-	struct scsi_link *sc_link_proto;
+scsi_attachdevs(scbus)
+	struct scsibus_data *scbus;
 {
 	int scsibus;
-	struct scsibus_data *scbus;
+	struct scsi_link *sc_link_proto = scbus->adapter_link;
 
 	if ( (scsibus = scsi_bus_conf(sc_link_proto)) == -1) {
 		return;
@@ -616,12 +719,17 @@ scsi_attachdevs(sc_link_proto)
 		sc_link_proto->opennings = 1;
 	}
 	sc_link_proto->scsibus = scsibus;
-	scbus = malloc(sizeof(struct scsibus_data), M_TEMP, M_NOWAIT);
-	if(scbus == 0 || extend_set(scbusses, scsibus, scbus) == 0) {
+	/*
+	 * Allocate our target-lun space.
+	 */
+	scbus->sc_link = (struct scsi_link *(*)[][8])malloc(
+		sizeof(struct scsi_link *[scbus->maxtarg + 1][8]),
+		M_TEMP, M_NOWAIT);
+	if(scbus == 0 || scbus->sc_link == 0
+	   || extend_set(scbusses, scsibus, scbus) == 0) {
 		panic("scsi_attachdevs: malloc");
 	}
-	bzero(scbus, sizeof(struct scsibus_data));
-	scbus->adapter_link = sc_link_proto;
+	bzero(scbus->sc_link, sizeof(struct scsi_link*[scbus->maxtarg + 1][8]));
 #if defined(SCSI_DELAY) && SCSI_DELAY > 2
 	printf("%s%d waiting for scsi devices to settle\n",
 	    sc_link_proto->adapter->name, sc_link_proto->adapter_unit);
@@ -663,7 +771,7 @@ scsi_probe_busses(int bus, int targ, int lun)
 static int
 scsi_alloc_unit(struct scsi_link *sc_link)
 {
-	u_int32 unit;
+	u_int32_t unit;
 	struct scsi_data *sd;
 	struct scsi_device *dsw;
 
@@ -823,6 +931,31 @@ static errval
 }
 #endif /* NSCTARG > 0 */
 
+/*
+ * Allocate a scsibus_data structure
+ * The target/lun area is dynamically allocated in scsi_attachdevs after
+ * the controller driver has a chance to update the maxtarg field.
+ */
+struct scsibus_data*
+scsi_alloc_bus()
+{
+	struct scsibus_data *scbus;
+	/*
+	 * Prepare the scsibus_data area for the upperlevel
+	 * scsi code.
+	 */
+	scbus = malloc(sizeof(struct scsibus_data), M_TEMP, M_NOWAIT);
+	if(!scbus) {
+                printf("scsi_alloc_bus: - cannot malloc!\n");
+                return NULL;
+	}
+	bzero(scbus, sizeof(struct scsibus_data));
+	/* Setup the defaults */
+	scbus->maxtarg = 7;
+	scbus->maxlun = 7;
+	return scbus;
+}
+
 /*
  * Probe the requested scsi bus. It must be already set up.
  * targ and lun optionally narrow the search if not -1
@@ -833,7 +966,7 @@ scsi_probe_bus(int bus, int targ, int lun)
 	struct scsibus_data *scsibus_data ;
 	int	maxtarg,mintarg,maxlun,minlun;
 	struct scsi_link *sc_link_proto;
-	u_int8  scsi_addr ;
+	u_int8_t  scsi_addr ;
 	struct scsidevs *bestmatch = NULL;
 	struct scsi_link *sc_link = NULL;
 	boolean maybe_more;
@@ -847,18 +980,18 @@ scsi_probe_bus(int bus, int targ, int lun)
 	sc_link_proto = scsibus_data->adapter_link;
 	scsi_addr = sc_link_proto->adapter_targ;
 	if(targ == -1){
-		maxtarg = 7;
+		maxtarg = scsibus_data->maxtarg;
 		mintarg = 0;
 	} else {
-		if((targ < 0 ) || (targ > 7)) return EINVAL;
+		if((targ < 0 ) || (targ > scsibus_data->maxtarg)) return EINVAL;
 		maxtarg = mintarg = targ;
 	}
 
 	if(lun == -1){
-		maxlun = 7;
+		maxlun = scsibus_data->maxlun;
 		minlun = 0;
 	} else {
-		if((lun < 0 ) || (lun > 7)) return EINVAL;
+		if((lun < 0 ) || (lun > scsibus_data->maxlun)) return EINVAL;
 		maxlun = minlun = lun;
 	}
 
@@ -872,7 +1005,7 @@ scsi_probe_bus(int bus, int targ, int lun)
 			 * The spot appears to already have something
 			 * linked in, skip past it. Must be doing a 'reprobe'
 			 */
-			if(scsibus_data->sc_link[targ][lun])
+			if((*scsibus_data->sc_link)[targ][lun])
 			{/* don't do this one, but check other luns */
 				maybe_more = 1;
 				continue;
@@ -903,11 +1036,11 @@ scsi_probe_bus(int bus, int targ, int lun)
 				sc_link->device = scsi_device_lookup(type);
 
 				(void)scsi_assign_unit(sc_link);
-				
+
 				if (scsi_alloc_unit(sc_link)) {
 
 					if (scsi_device_attach(sc_link) == 0) {
-						scsibus_data->sc_link[targ][lun] = sc_link;
+						(*scsibus_data->sc_link)[targ][lun] = sc_link;
 						sc_link = NULL;		/* it's been used */
 					}
 					else
@@ -937,7 +1070,7 @@ scsi_link_get(bus, targ, lun)
 {
 	struct scsibus_data *scsibus_data =
 	 (struct scsibus_data *)extend_get(scbusses, bus);
-	return (scsibus_data) ? scsibus_data->sc_link[targ][lun] : 0;
+	return (scsibus_data) ? (*scsibus_data->sc_link)[targ][lun] : 0;
 }
 
 /* make_readable: Make the inquiry data readable.  Anything less than a ' '
@@ -975,21 +1108,21 @@ scsi_probedev(sc_link, maybe_more, type_p)
 	struct scsi_link *sc_link;
 	int *type_p;
 {
-	u_int8  target = sc_link->target;
-	u_int8  lu = sc_link->lun;
+	u_int8_t  target = sc_link->target;
+	u_int8_t  lu = sc_link->lun;
 	struct scsidevs *bestmatch = (struct scsidevs *) 0;
 	int    dtype = 0;
 	char   *desc;
 	char   *qtype;
 	struct scsi_inquiry_data *inqbuf;
-	u_int32 len, qualifier, type;
+	u_int32_t len, qualifier, type;
 	boolean remov;
 	char    manu[8 + 1];
 	char    model[16 + 1];
 	char    version[4 + 1];
 
  	inqbuf = &sc_link->inqbuf;
- 
+
  	bzero(inqbuf, sizeof(*inqbuf));
 	/*
 	 * Ask the device what it is
@@ -1117,7 +1250,7 @@ scsi_probedev(sc_link, maybe_more, type_p)
 
 	/*
 	 * Try make as good a match as possible with
-	 * available sub drivers       
+	 * available sub drivers
 	 */
 	bestmatch = (scsi_selectdev(
 		qualifier, type, remov ? T_REMOV : T_FIXED, manu, model, version));
@@ -1159,7 +1292,7 @@ scsi_dev_lookup(d_open)
 #ifdef NEW_SCSICONF
 /*
  * Compare name with pattern, return 0 on match.
- * Short pattern matches trailing blanks in name, 
+ * Short pattern matches trailing blanks in name,
  * wildcard '*' in pattern matches rest of name
  */
 int
@@ -1183,11 +1316,11 @@ match(pattern, name)
 
 /*
  * Try make as good a match as possible with
- * available sub drivers       
+ * available sub drivers
  */
 struct scsidevs *
 scsi_selectdev(qualifier, type, remov, manu, model, rev)
-	u_int32 qualifier, type;
+	u_int32_t qualifier, type;
 	boolean remov;
 	char   *manu, *model, *rev;
 {
@@ -1225,9 +1358,9 @@ scsi_selectdev(qualifier, type, remov, manu, model, rev)
 		break;
 	}
 #else
-	u_int32 numents = (sizeof(knowndevs) / sizeof(struct scsidevs)) - 1;
-	u_int32 count = 0;
-	u_int32 bestmatches = 0;
+	u_int32_t numents = (sizeof(knowndevs) / sizeof(struct scsidevs)) - 1;
+	u_int32_t count = 0;
+	u_int32_t bestmatches = 0;
 	struct scsidevs *bestmatch = (struct scsidevs *) 0;
 	struct scsidevs *thisentry = knowndevs;
 
@@ -1251,7 +1384,7 @@ scsi_selectdev(qualifier, type, remov, manu, model, rev)
 			bestmatch = thisentry;
 		}
 		if (thisentry->flags & SC_SHOWME)
-			printf("\n%s-\n%s-", thisentry->manufacturer, manu);
+			printf("'%s'-'%s'\n", thisentry->manufacturer, manu);
 		if (strcmp(thisentry->manufacturer, manu)) {
 			continue;
 		}
@@ -1260,7 +1393,7 @@ scsi_selectdev(qualifier, type, remov, manu, model, rev)
 			bestmatch = thisentry;
 		}
 		if (thisentry->flags & SC_SHOWME)
-			printf("\n%s-\n%s-", thisentry->model, model);
+			printf("'%s'-'%s'\n", thisentry->model, model);
 		if (strcmp(thisentry->model, model)) {
 			continue;
 		}
@@ -1269,7 +1402,7 @@ scsi_selectdev(qualifier, type, remov, manu, model, rev)
 			bestmatch = thisentry;
 		}
 		if (thisentry->flags & SC_SHOWME)
-			printf("\n%s-\n%s-", thisentry->version, rev);
+			printf("'%s'-'%s'\n", thisentry->version, rev);
 		if (strcmp(thisentry->version, rev)) {
 			continue;
 		}
