@@ -14,7 +14,7 @@
  *
  * Ported to run under 386BSD by Julian Elischer (julian@dialix.oz.au) Sept 1992
  *
- *      $Id: sd.c,v 1.63 1995/05/03 18:09:17 dufault Exp $
+ * $FreeBSD: stable/2.1/sys/scsi/sd.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #define SPLSD splbio
@@ -43,7 +43,7 @@
 #include <machine/md_var.h>
 #include <i386/i386/cons.h>		/* XXX */
 
-u_int32 sdstrats, sdqueues;
+u_int32_t sdstrats, sdqueues;
 
 #define SECSIZE 512
 #define	SDOUTSTANDING	4
@@ -61,17 +61,17 @@ errval	sd_get_parms __P((int unit, int flags));
 static	void	sdstrategy1 __P((struct buf *));
 
 int		sd_sense_handler __P((struct scsi_xfer *));
-void    sdstart __P((u_int32, u_int32));
+void    sdstart __P((u_int32_t, u_int32_t));
 
 struct scsi_data {
-	u_int32 flags;
+	u_int32_t flags;
 #define	SDINIT		0x04	/* device has been init'd */
 	struct disk_parms {
 		u_char  heads;	/* Number of heads */
-		u_int16 cyls;	/* Number of cylinders */
+		u_int16_t cyls;	/* Number of cylinders */
 		u_char  sectors;	/*dubious *//* Number of sectors/track */
-		u_int16 secsiz;	/* Number of bytes/sector */
-		u_int32 disksize;	/* total number sectors */
+		u_int16_t secsiz;	/* Number of bytes/sector */
+		u_int32_t disksize;	/* total number sectors */
 	} params;
 	struct diskslices *dk_slices;	/* virtual drives */
 	struct buf buf_queue;
@@ -117,7 +117,7 @@ struct scsi_device sd_switch =
 static struct scsi_xfer sx;
 
 static int
-sd_externalize(struct proc *p, struct kern_devconf *kdc, void *userp, 
+sd_externalize(struct proc *p, struct kern_devconf *kdc, void *userp,
 	       size_t len)
 {
 	return scsi_externalize(SCSI_LINK(&sd_switch, kdc->kdc_unit),
@@ -158,10 +158,10 @@ sd_registerdev(int unit)
  * The routine called by the low level scsi routine when it discovers
  * a device suitable for this driver.
  */
-errval 
+errval
 sdattach(struct scsi_link *sc_link)
 {
-	u_int32 unit;
+	u_int32_t unit;
 	struct disk_parms *dp;
 
 	struct scsi_data *sd = sc_link->sd;
@@ -170,7 +170,7 @@ sdattach(struct scsi_link *sc_link)
 
 	dp = &(sd->params);
 
-	if (sc_link->opennings > SDOUTSTANDING) 
+	if (sc_link->opennings > SDOUTSTANDING)
 		sc_link->opennings = SDOUTSTANDING;
 	/*
 	 * Use the subdriver to request information regarding
@@ -216,7 +216,7 @@ sd_open(dev, mode, fmt, p, sc_link)
 	struct scsi_link *sc_link;
 {
 	errval  errcode = 0;
-	u_int32 unit;
+	u_int32_t unit;
 	struct disklabel label;
 	struct scsi_data *sd;
 
@@ -237,7 +237,7 @@ sd_open(dev, mode, fmt, p, sc_link)
 		dev, unit, PARTITION(dev)));
 
 	/*
-	 * "unit attention" errors should occur here if the 
+	 * "unit attention" errors should occur here if the
 	 * drive has been restarted or the pack changed.
 	 * just ingnore the result, it's a decoy instruction
 	 * The error code will act on the error though
@@ -280,7 +280,7 @@ sd_open(dev, mode, fmt, p, sc_link)
 	SC_DEBUG(sc_link, SDEV_DB3, ("device ok\n"));
 
 	/*
-	 * Load the physical device parameters 
+	 * Load the physical device parameters
 	 */
 	sd_get_parms(unit, 0);	/* sets SDEV_MEDIA_LOADED */
 	if (sd->params.secsiz != SECSIZE) {	/* XXX One day... */
@@ -330,7 +330,7 @@ bad:
  * close the device.. only called if we are the LAST occurence of an open
  * device.  Convenient now but usually a pain.
  */
-errval 
+errval
 sd_close(dev, fflag, fmt, p, sc_link)
 	dev_t	dev;
 	int	fflag;
@@ -358,9 +358,9 @@ void
 sd_strategy(struct buf *bp, struct scsi_link *sc_link)
 {
 	struct buf *dp;
-	u_int32 opri;
+	u_int32_t opri;
 	struct scsi_data *sd;
-	u_int32 unit;
+	u_int32_t unit;
 
 	sdstrats++;
 	unit = SDUNIT((bp->b_dev));
@@ -389,9 +389,9 @@ sd_strategy(struct buf *bp, struct scsi_link *sc_link)
 	opri = SPLSD();
 	dp = &sd->buf_queue;
 
-	/*      
+	/*
 	 * Use a bounce buffer if necessary
-	 */      
+	 */
 #ifdef BOUNCE_BUFFERS
 	if (sc_link->flags & SDEV_BOUNCE)
 		vm_bounce_alloc(bp);
@@ -448,15 +448,15 @@ sdstrategy1(struct buf *bp)
  * must be called at the correct (highish) spl level
  * sdstart() is called at SPLSD  from sdstrategy and scsi_done
  */
-void 
-sdstart(u_int32 unit, u_int32 flags)
+void
+sdstart(u_int32_t unit, u_int32_t flags)
 {
 	register struct	scsi_link *sc_link = SCSI_LINK(&sd_switch, unit);
 	register struct scsi_data *sd = sc_link->sd;
 	struct buf *bp = 0;
 	struct buf *dp;
 	struct scsi_rw_big cmd;
-	u_int32 blkno, nblk;
+	u_int32_t blkno, nblk;
 
 	SC_DEBUG(sc_link, SDEV_DB2, ("sdstart "));
 	/*
@@ -465,7 +465,7 @@ sdstart(u_int32 unit, u_int32 flags)
 	while (sc_link->opennings) {
 
 		/*
-		 * there is excess capacity, but a special waits 
+		 * there is excess capacity, but a special waits
 		 * It'll need the adapter as soon as we clear out of the
 		 * way and let it run (user level wait).
 		 */
@@ -547,7 +547,7 @@ bad:
  * Perform special action on behalf of the user
  * Knows about the internals of this device
  */
-errval 
+errval
 sd_ioctl(dev_t dev, int cmd, caddr_t addr, int flag, struct proc *p,
 	 struct scsi_link *sc_link)
 {
@@ -587,13 +587,13 @@ sd_ioctl(dev_t dev, int cmd, caddr_t addr, int flag, struct proc *p,
 /*
  * Find out from the device what it's capacity is
  */
-u_int32 
+u_int32_t
 sd_size(unit, flags)
 	int	unit, flags;
 {
 	struct scsi_read_cap_data rdcap;
 	struct scsi_read_capacity scsi_cmd;
-	u_int32 size;
+	u_int32_t size;
 	struct scsi_link *sc_link = SCSI_LINK(&sd_switch, unit);
 
 	/*
@@ -630,7 +630,7 @@ sd_size(unit, flags)
 /*
  * Tell the device to map out a defective block
  */
-errval 
+errval
 sd_reassign_blocks(unit, block)
 	int	unit, block;
 {
@@ -663,10 +663,10 @@ sd_reassign_blocks(unit, block)
 
 /*
  * Get the scsi driver to send a full inquiry to the
- * device and use the results to fill out the disk 
+ * device and use the results to fill out the disk
  * parameter structure.
  */
-errval 
+errval
 sd_get_parms(unit, flags)
 	int	unit, flags;
 {
@@ -679,7 +679,7 @@ sd_get_parms(unit, flags)
 		struct blk_desc blk_desc;
 		union disk_pages pages;
 	} scsi_sense;
-	u_int32 sectors;
+	u_int32_t sectors;
 
 	/*
 	 * First check if we have it all loaded
@@ -734,7 +734,7 @@ sd_get_parms(unit, flags)
 		/*
 		 * KLUDGE!!(for zone recorded disks)
 		 * give a number of sectors so that sec * trks * cyls
-		 * is <= disk_size 
+		 * is <= disk_size
 		 * can lead to wasted space! THINK ABOUT THIS !
 		 */
 		disk_parms->heads = scsi_sense.pages.rigid_geometry.nheads;
@@ -767,7 +767,7 @@ sdsize(dev_t dev)
 {
 	struct scsi_data *sd;
 
-	sd = SCSI_DATA(&sd_switch, (u_int32) SDUNIT(dev));
+	sd = SCSI_DATA(&sd_switch, (u_int32_t) SDUNIT(dev));
 	if (sd == NULL)
 		return (-1);
 	return (dssize(dev, &sd->dk_slices, sdopen, sdclose));
@@ -780,7 +780,7 @@ sdsize(dev_t dev)
  * This will issue a retry when the device returns a
  * non-media hardware failure.  The CDC-WREN IV does this
  * when you access it during thermal calibrarion, so the drive
- * is pretty useless without this.  
+ * is pretty useless without this.
  *
  * In general, you probably almost always would like to issue a retry
  * for your disk I/O.  It can't hurt too much (the caller only retries
@@ -835,10 +835,10 @@ sddump(dev_t dev)
 	struct disklabel *lp;
 	register struct scsi_data *sd;	/* disk unit to do the IO */
 	struct scsi_link *sc_link;
-	int32	num;		/* number of sectors to write */
-	u_int32	unit, part;
-	int32	blkoff, blknum, blkcnt = MAXTRANSFER;
-	int32	nblocks;
+	int32_t	num;		/* number of sectors to write */
+	u_int32_t	unit, part;
+	int32_t	blkoff, blknum, blkcnt = MAXTRANSFER;
+	int32_t	nblocks;
 	char	*addr;
 	struct	scsi_rw_big cmd;
 	static	int sddoingadump = 0;
@@ -874,7 +874,7 @@ sddump(dev_t dev)
 		return (ENXIO);
 
 	/* Convert to disk sectors */
-	num = (u_int32) num * NBPG / sd->params.secsiz;	/* XXX it must be 512 */
+	num = (u_int32_t) num * NBPG / sd->params.secsiz;	/* XXX it must be 512 */
 
 	/* check if controller active */
 	if (sddoingadump)
@@ -910,7 +910,7 @@ sddump(dev_t dev)
 		/*
 		 * Fill out the scsi_xfer structure
 		 *    Note: we cannot sleep as we may be an interrupt
-		 * don't use scsi_scsi_cmd() as it may want 
+		 * don't use scsi_scsi_cmd() as it may want
 		 * to wait for an xs.
 		 */
 		bzero(xs, sizeof(sx));
