@@ -12,7 +12,7 @@
  * on the understanding that TFS is not responsible for the correct
  * functioning of this software in any circumstances.
  *
- * $Id: st.c,v 1.34 1995/04/29 21:30:29 joerg Exp $
+ * $FreeBSD: stable/2.1/sys/scsi/st.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -46,7 +46,7 @@
 #include <scsi/scsiconf.h>
 #include <sys/devconf.h>
 
-u_int32 ststrats, stqueues;
+u_int32_t ststrats, stqueues;
 
 /* Defines for device specific stuff */
 #define		PAGE_0_SENSE_DATA_SIZE	12
@@ -70,8 +70,8 @@ u_int32 ststrats, stqueues;
  * and note how they are bad, so we can correct for them
  */
 struct modes {
-	u_int32 blksiz;
-	u_int32 quirks;		/* same definitions as in rogues */
+	u_int32_t blksiz;
+	u_int32_t quirks;		/* same definitions as in rogues */
 	char    density;
 	char    spare[3];
 };
@@ -81,7 +81,7 @@ struct rogues {
 	char   *manu;
 	char   *model;
 	char   *version;
-	u_int32 quirks;		/* valid for all modes */
+	u_int32_t quirks;		/* valid for all modes */
 	struct modes modes[4];
 };
 
@@ -153,22 +153,22 @@ static struct rogues gallery[] =	/* ends with an all-null entry */
 };
 #endif /* NEW_SCSICONF */
 
-errval	st_space __P((u_int32 unit, int32 number, u_int32 what, u_int32 flags));
-errval	st_rewind __P((u_int32 unit, boolean immed, u_int32 flags));
-errval	st_erase __P((u_int32 unit, boolean immed, u_int32 flags));
-static errval st_mode_sense __P((u_int32 unit, u_int32 flags, \
-	struct tape_pages *page, u_int32 pagelen, u_int32 pagecode));
-errval	st_decide_mode __P((u_int32 unit, boolean first_read));
-errval	st_rd_blk_lim __P((u_int32 unit, u_int32 flags));
-errval	st_touch_tape __P((u_int32 unit));
-errval	st_write_filemarks __P((u_int32 unit, int32 number, u_int32 flags));
-errval	st_load __P((u_int32 unit, u_int32 type, u_int32 flags));
-errval	st_mode_select __P((u_int32 unit, u_int32 flags, \
-	struct tape_pages *page, u_int32 pagelen));
-errval	st_comp __P((u_int32 unit, u_int32 mode));
+errval	st_space __P((u_int32_t unit, int32_t number, u_int32_t what, u_int32_t flags));
+errval	st_rewind __P((u_int32_t unit, boolean immed, u_int32_t flags));
+errval	st_erase __P((u_int32_t unit, boolean immed, u_int32_t flags));
+static errval st_mode_sense __P((u_int32_t unit, u_int32_t flags, \
+	struct tape_pages *page, u_int32_t pagelen, u_int32_t pagecode));
+errval	st_decide_mode __P((u_int32_t unit, boolean first_read));
+errval	st_rd_blk_lim __P((u_int32_t unit, u_int32_t flags));
+errval	st_touch_tape __P((u_int32_t unit));
+errval	st_write_filemarks __P((u_int32_t unit, int32_t number, u_int32_t flags));
+errval	st_load __P((u_int32_t unit, u_int32_t type, u_int32_t flags));
+errval	st_mode_select __P((u_int32_t unit, u_int32_t flags, \
+	struct tape_pages *page, u_int32_t pagelen));
+errval	st_comp __P((u_int32_t unit, u_int32_t mode));
 void    ststrategy();
-int32   st_chkeod();
-void	ststart(u_int32	unit, u_int32 flags);
+int32_t   st_chkeod();
+void	ststart(u_int32_t	unit, u_int32_t flags);
 void	st_unmount();
 errval	st_mount_tape();
 void	st_loadquirks();
@@ -183,31 +183,31 @@ errval  st_interpret_sense();
 
 struct scsi_data {
 /*--------------------present operating parameters, flags etc.----------------*/
-	u_int32 flags;		/* see below                          */
-	u_int32 blksiz;		/* blksiz we are using                */
-	u_int32 density;	/* present density                    */
-	u_int32 comp;		/* present compression mode           */
-	u_int32 quirks;		/* quirks for the open mode           */
-	u_int32 last_dsty;	/* last density openned               */
+	u_int32_t flags;		/* see below                          */
+	u_int32_t blksiz;		/* blksiz we are using                */
+	u_int32_t density;	/* present density                    */
+	u_int32_t comp;		/* present compression mode           */
+	u_int32_t quirks;		/* quirks for the open mode           */
+	u_int32_t last_dsty;	/* last density openned               */
 /*--------------------parameters reported by the device ----------------------*/
-	u_int32 blkmin;		/* min blk size                       */
-	u_int32 blkmax;		/* max blk size                       */
+	u_int32_t blkmin;		/* min blk size                       */
+	u_int32_t blkmax;		/* max blk size                       */
 #ifndef NEW_SCSICONF
 	struct rogues *rogues;	/* if we have a rogue entry           */
 #endif
 /*--------------------parameters reported by the device for this media--------*/
-	u_int32 numblks;	/* nominal blocks capacity            */
-	u_int32 media_blksiz;	/* 0 if not ST_FIXEDBLOCKS            */
-	u_int32 media_density;	/* this is what it said when asked    */
+	u_int32_t numblks;	/* nominal blocks capacity            */
+	u_int32_t media_blksiz;	/* 0 if not ST_FIXEDBLOCKS            */
+	u_int32_t media_density;	/* this is what it said when asked    */
 /*--------------------quirks for the whole drive------------------------------*/
-	u_int32 drive_quirks;	/* quirks of this drive               */
+	u_int32_t drive_quirks;	/* quirks of this drive               */
 /*--------------------How we should set up when openning each minor device----*/
 #ifdef NEW_SCSICONF
 	st_modes modes; /* plus more for each mode            */
 #else
 	struct modes modes[4];	/* plus more for each mode            */
 #endif
-	u_int8  modeflags[4];	/* flags for the modes                */
+	u_int8_t  modeflags[4];	/* flags for the modes                */
 #define DENSITY_SET_BY_USER	0x01
 #define DENSITY_SET_BY_QUIRK	0x02
 #define BLKSIZE_SET_BY_USER	0x04
@@ -221,7 +221,7 @@ struct scsi_data {
 					 */
 	struct buf *buf_queue;		/* the queue of pending IO operations */
 	struct scsi_xfer scsi_xfer;	/* scsi xfer struct for this drive */
-	u_int32 xfer_block_wait;	/* is a process waiting? */
+	u_int32_t xfer_block_wait;	/* is a process waiting? */
 };
 
 static int stunit(dev_t dev) { return STUNIT(dev); }
@@ -278,6 +278,7 @@ struct scsi_device st_switch =
 #define	ST_BLANK_READ	0x800	/* BLANK CHECK encountered already */
 #define	ST_2FM_AT_EOD	0x1000	/* write 2 file marks at EOD */
 #define	ST_MOUNTED	0x2000	/* Device is presently mounted */
+#define	ST_SENSE_READ	0x4000	/* mode sense read from drive */
 
 #define	ST_PER_ACTION	(ST_AT_FILEMARK | ST_EIO_PENDING | ST_BLANK_READ)
 #define	ST_PER_MOUNT	(ST_INFO_VALID | ST_BLOCK_SET | ST_WRITTEN | \
@@ -285,7 +286,7 @@ struct scsi_device st_switch =
 			ST_FM_WRITTEN | ST_2FM_AT_EOD | ST_PER_ACTION)
 
 static int
-st_externalize(struct proc *p, struct kern_devconf *kdc, void *userp, 
+st_externalize(struct proc *p, struct kern_devconf *kdc, void *userp,
 	       size_t len)
 {
 	return scsi_externalize(SCSI_LINK(&st_switch, kdc->kdc_unit),
@@ -319,10 +320,10 @@ st_registerdev(int unit)
  * a device suitable for this driver
  */
 
-errval 
+errval
 stattach(struct scsi_link *sc_link)
 {
-	u_int32 unit;
+	u_int32_t unit;
 
 	struct scsi_data *st = sc_link->sd;
 
@@ -376,7 +377,7 @@ stattach(struct scsi_link *sc_link)
  */
 void
 st_identify_drive(unit)
-	u_int32 unit;
+	u_int32_t unit;
 {
 	struct scsi_link *sc_link = SCSI_LINK(&st_switch, unit);
 	struct scsi_data *st = sc_link->sd;
@@ -385,7 +386,7 @@ st_identify_drive(unit)
 	char    model[32];
 	char    model2[32];
 	char    version[32];
-	u_int32 model_len;
+	u_int32_t model_len;
  	struct scsi_inquiry_data *inqbuf = &sc_link->inqbuf;
 
 	/*
@@ -507,11 +508,11 @@ st_loadquirks(sc_link)
 /*
  * open the device.
  */
-errval 
+errval
 st_open(dev_t dev, int flags, int fmt, struct proc *p,
 struct scsi_link *sc_link)
 {
-	u_int32 unit, mode, dsty;
+	u_int32_t unit, mode, dsty;
 	errval  errno = 0;
 	struct scsi_data *st;
 
@@ -568,7 +569,7 @@ struct scsi_link *sc_link)
 		st_unmount(unit, NOEJECT);
 	}
 	/*
-	 * If we are not mounted, then we should start a new 
+	 * If we are not mounted, then we should start a new
 	 * mount session.
 	 */
 	if (!(st->flags & ST_MOUNTED)) {
@@ -593,7 +594,7 @@ struct scsi_link *sc_link)
  * close the device.. only called if we are the LAST
  * occurence of an open device
  */
-errval 
+errval
 st_close(dev_t dev, int flag, int fmt, struct proc *p,
         struct scsi_link *sc_link)
 {
@@ -633,9 +634,9 @@ st_close(dev_t dev, int flag, int fmt, struct proc *p,
 errval
 st_mount_tape(dev, flags)
 	dev_t   dev;
-	u_int32 flags;
+	u_int32_t flags;
 {
-	u_int32 unit, mode, dsty;
+	u_int32_t unit, mode, dsty;
 	struct scsi_data *st;
 	struct scsi_link *sc_link;
 	errval  errno = 0;
@@ -724,7 +725,7 @@ st_mount_tape(dev, flags)
 	scsi_prevent(sc_link, PR_PREVENT, 0);	/* who cares if it fails? */
 	st->flags &= ~ST_NEW_MOUNT;
 	st->flags |= ST_MOUNTED;
-	sc_link->flags |= SDEV_MEDIA_LOADED;	/* move earlier? */
+	sc_link->flags |= SDEV_MEDIA_LOADED;
 
 	return 0;
 }
@@ -740,7 +741,7 @@ st_unmount(int unit, boolean eject)
 {
 	struct scsi_link *sc_link = SCSI_LINK(&st_switch, unit);
 	struct scsi_data *st = sc_link->sd;
-	int32   nmarks;
+	int32_t   nmarks;
 
 	if (!(st->flags & ST_MOUNTED))
 		return;
@@ -760,9 +761,9 @@ st_unmount(int unit, boolean eject)
  * initial operation, make a decision as to how we should be set
  * to run (regarding blocking and EOD marks)
  */
-errval 
+errval
 st_decide_mode(unit, first_read)
-	u_int32	unit;
+	u_int32_t	unit;
 	boolean	first_read;
 {
 	struct scsi_link *sc_link = SCSI_LINK(&st_switch, unit);
@@ -828,6 +829,7 @@ st_decide_mode(unit, first_read)
 	case QIC_150:
 	case QIC_525:
 	case QIC_1320:
+	case QIC_3080:
 		st->flags |= ST_FIXEDBLOCKS;
 		if (st->media_blksiz > 0) {
 			st->blksiz = st->media_blksiz;
@@ -882,6 +884,7 @@ done:
 	case QIC_150:
 	case QIC_525:
 	case QIC_1320:
+	case QIC_3080:
 		st->flags &= ~ST_2FM_AT_EOD;
 		break;
 	default:
@@ -896,12 +899,12 @@ done:
  * The transfer is described by a buf and will include
  * only one physical transfer.
  */
-void 
+void
 st_strategy(struct buf *bp, struct scsi_link *sc_link)
 {
 	struct buf **dp;
-	unsigned char unit;	/* XXX Everywhere else unit is "u_int32". Please int? */
-	u_int32 opri;
+	unsigned char unit;	/* XXX Everywhere else unit is "u_int32_t". Please int? */
+	u_int32_t opri;
 	struct scsi_data *st;
 
 	ststrats++;
@@ -935,9 +938,9 @@ st_strategy(struct buf *bp, struct scsi_link *sc_link)
 	}
 	opri = splbio();
 
-	/*      
+	/*
 	 * Use a bounce buffer if necessary
-	 */      
+	 */
 #ifdef BOUNCE_BUFFERS
 	if (sc_link->flags & SDEV_BOUNCE)
 		vm_bounce_alloc(bp);
@@ -988,10 +991,10 @@ done:
  * continues to be drained.
  * ststart() is called at splbio
  */
-void 
+void
 ststart(unit, flags)
-	u_int32	unit;
-	u_int32 flags;
+	u_int32_t	unit;
+	u_int32_t flags;
 {
 	struct scsi_link *sc_link = SCSI_LINK(&st_switch, unit);
 	struct scsi_data *st = sc_link->sd;
@@ -1017,7 +1020,7 @@ ststart(unit, flags)
 		st->buf_queue = bp->b_actf;
 
 		/*
-		 * if the device has been unmounted byt the user
+		 * if the device has been unmounted by the user
 		 * then throw away all requests until done
 		 */
 		if ((!(st->flags & ST_MOUNTED))
@@ -1101,7 +1104,7 @@ ststart(unit, flags)
 			0,	/* can't retry a read on a tape really */
 			100000,
 			bp,
-			flags | SCSI_NOSLEEP) == SUCCESSFULLY_QUEUED) {
+			flags) == SUCCESSFULLY_QUEUED) {
 			stqueues++;
 		} else {
 badnews:
@@ -1117,17 +1120,17 @@ badnews:
  * Perform special action on behalf of the user;
  * knows about the internals of this device
  */
-errval 
+errval
 st_ioctl(dev_t dev, int cmd, caddr_t arg, int	flag,
 struct proc *p, struct scsi_link *sc_link)
 {
 	errval  errcode = 0;
 	unsigned char unit;
-	u_int32 number, flags, dsty;
+	u_int32_t number, flags, dsty;
 	struct scsi_data *st;
-	u_int32 hold_blksiz;
-	u_int32 hold_density;
-	int32   nmarks;
+	u_int32_t hold_blksiz;
+	u_int32_t hold_density;
+	int32_t   nmarks;
 	struct mtop *mt = (struct mtop *) arg;
 
 	/*
@@ -1205,6 +1208,10 @@ struct proc *p, struct scsi_link *sc_link)
 			case MTERASE:	/* erase */
 				errcode = st_erase(unit, FALSE, flags);
 				break;
+			case MTRETENS:	/* re-tension tape */
+				errcode = st_load(unit, LD_LOAD|LD_RETEN,
+						  flags);
+				break;
 			case MTOFFL:	/* rewind and put the drive offline */
 				st_unmount(unit, EJECT);
 				break;
@@ -1282,7 +1289,7 @@ try_new_value:
 	/*
 	 * As the drive liked it, if we are setting a new default,
 	 * set it into the structures as such.
-	 * 
+	 *
 	 * The means for deciding this are not finalised yet
 	 */
 	if (IS_CTLMODE(dev)) {
@@ -1305,9 +1312,9 @@ try_new_value:
 /*
  * Do a synchronous read.
  */
-errval 
+errval
 st_read(unit, buf, size, flags)
-	u_int32 unit, size, flags;
+	u_int32_t unit, size, flags;
 	char   *buf;
 {
 	struct scsi_link *sc_link = SCSI_LINK(&st_switch, unit);
@@ -1348,9 +1355,9 @@ st_read(unit, buf, size, flags)
 /*
  * Ask the drive what it's min and max blk sizes are.
  */
-errval 
+errval
 st_rd_blk_lim(unit, flags)
-	u_int32 unit, flags;
+	u_int32_t unit, flags;
 {
 	struct scsi_link *sc_link = SCSI_LINK(&st_switch, unit);
 	struct scsi_data *st = sc_link->sd;
@@ -1394,7 +1401,7 @@ st_rd_blk_lim(unit, flags)
 
 /*
  * Get the scsi driver to send a full inquiry to the
- * device and use the results to fill out the global 
+ * device and use the results to fill out the global
  * parameter structure.
  *
  * called from:
@@ -1402,13 +1409,13 @@ st_rd_blk_lim(unit, flags)
  * open
  * ioctl (to reset original blksize)
  */
-static errval 
+static errval
 st_mode_sense(unit, flags, page, pagelen, pagecode)
-	u_int32 unit, flags;
+	u_int32_t unit, flags;
 	struct tape_pages *page;
-	u_int32 pagelen,pagecode;
+	u_int32_t pagelen,pagecode;
 {
-	u_int32 dat_len;
+	u_int32_t dat_len;
 	errval  errno;
 	struct scsi_mode_sense scsi_cmd;
 	struct {
@@ -1427,6 +1434,7 @@ st_mode_sense(unit, flags, page, pagelen, pagecode)
 	struct scsi_link *sc_link = SCSI_LINK(&st_switch, unit);
 	struct scsi_data *st = sc_link->sd;
 
+	st->flags &= ~ST_SENSE_READ;
 	/*
 	 * Check if we need to use a default page..
 	 */
@@ -1442,7 +1450,7 @@ st_mode_sense(unit, flags, page, pagelen, pagecode)
 		+ sizeof(struct blk_desc)
 		+ (page ? pagelen : 0);
 	/*
-	 * Set up a mode sense 
+	 * Set up a mode sense
 	 */
 	bzero(&scsi_cmd, sizeof(scsi_cmd));
 	scsi_cmd.op_code = MODE_SENSE;
@@ -1450,7 +1458,7 @@ st_mode_sense(unit, flags, page, pagelen, pagecode)
 	scsi_cmd.length = dat_len;
 
 	/*
-	 * do the command, 
+	 * do the command,
 	 * use the results to set blksiz, numblks and density
 	 * or if we need it as a template for the mode select
 	 * store it away.
@@ -1481,7 +1489,7 @@ st_mode_sense(unit, flags, page, pagelen, pagecode)
 	if (page) {
 		bcopy(&dat.page, page, pagelen);
 	}
-	sc_link->flags |= SDEV_MEDIA_LOADED;
+	st->flags |= ST_SENSE_READ;
 	return 0;
 }
 
@@ -1489,13 +1497,13 @@ st_mode_sense(unit, flags, page, pagelen, pagecode)
  * Send a filled out parameter structure to the drive to
  * set it into the desire modes etc.
  */
-errval 
+errval
 st_mode_select(unit, flags, page, pagelen)
-	u_int32 unit, flags;
+	u_int32_t unit, flags;
 	struct tape_pages *page;
-	u_int32 pagelen;
+	u_int32_t pagelen;
 {
-	u_int32 dat_len;
+	u_int32_t dat_len;
 	struct scsi_mode_select scsi_cmd;
 	struct {
 		struct scsi_mode_header header;
@@ -1557,7 +1565,7 @@ int noisy_st = 0;
 	still doesn't work! grrr!
 \***************************************************************/
 errval	st_comp(unit,mode)
-u_int32 unit,mode;
+u_int32_t unit,mode;
 {
 	struct tape_pages       page;
 	int	pagesize;
@@ -1604,10 +1612,10 @@ u_int32 unit,mode;
 /*
  * skip N blocks/filemarks/seq filemarks/eom
  */
-errval 
+errval
 st_space(unit, number, what, flags)
-	u_int32 unit, what, flags;
-	int32   number;
+	u_int32_t unit, what, flags;
+	int32_t   number;
 {
 	errval  error;
 	struct scsi_space scsi_cmd;
@@ -1689,10 +1697,10 @@ st_space(unit, number, what, flags)
 /*
  * write N filemarks
  */
-errval 
+errval
 st_write_filemarks(unit, number, flags)
-	u_int32 unit, flags;
-	int32   number;
+	u_int32_t unit, flags;
+	int32_t   number;
 {
 	struct scsi_write_filemarks scsi_cmd;
 	struct scsi_link *sc_link = SCSI_LINK(&st_switch, unit);
@@ -1741,12 +1749,12 @@ st_write_filemarks(unit, number, flags)
  * nmarks returns the number of marks to skip (or, if position
  * true, which were skipped) to get back original position.
  */
-int32 
+int32_t
 st_chkeod(unit, position, nmarks, flags)
-	u_int32 unit;
+	u_int32_t unit;
 	boolean position;
-	int32  *nmarks;
-	u_int32 flags;
+	int32_t  *nmarks;
+	u_int32_t flags;
 {
 	errval  error;
 	struct scsi_data *st = SCSI_DATA(&st_switch, unit);
@@ -1771,9 +1779,9 @@ st_chkeod(unit, position, nmarks, flags)
 /*
  * load/unload (with retension if true)
  */
-errval 
+errval
 st_load(unit, type, flags)
-	u_int32 unit, type, flags;
+	u_int32_t unit, type, flags;
 {
 	struct scsi_load scsi_cmd;
 	struct scsi_link *sc_link = SCSI_LINK(&st_switch, unit);
@@ -1782,7 +1790,7 @@ st_load(unit, type, flags)
 	bzero(&scsi_cmd, sizeof(scsi_cmd));
 	if (type != LD_LOAD) {
 		errval  error;
-		int32   nmarks;
+		int32_t   nmarks;
 
 		error = st_chkeod(unit, FALSE, &nmarks, flags);
 		if (error != ESUCCESS)
@@ -1807,16 +1815,16 @@ st_load(unit, type, flags)
 /*
  *  Rewind the device
  */
-errval 
+errval
 st_rewind(unit, immed, flags)
-	u_int32 unit, flags;
+	u_int32_t unit, flags;
 	boolean immed;
 {
 	struct scsi_rewind scsi_cmd;
 	struct scsi_link *sc_link = SCSI_LINK(&st_switch, unit);
 	struct scsi_data *st = sc_link->sd;
 	errval  error;
-	int32   nmarks;
+	int32_t   nmarks;
 
 	error = st_chkeod(unit, FALSE, &nmarks, flags);
 	if (error != ESUCCESS)
@@ -1831,24 +1839,24 @@ st_rewind(unit, immed, flags)
 		0,
 		0,
 		ST_RETRIES,
-		immed ? 5000 : 300000,	/* 5 sec or 5 min */
+		immed ? 5000 : (200 * 60 * 1000),	/* 5 sec or 200 min */
 		NULL,
 		flags));
 }
 
 /*
 **  Erase the device
-*/ 
-errval 
+*/
+errval
 st_erase(unit, immed, flags)
-	u_int32 unit, flags;
+	u_int32_t unit, flags;
 	boolean immed;
 {
 	struct scsi_erase scsi_cmd;
 	struct scsi_link *sc_link = SCSI_LINK(&st_switch, unit);
 	struct scsi_data *st = sc_link->sd;
 	errval  error;
-	int32   nmarks;
+	int32_t   nmarks;
 
 	error = st_chkeod(unit, FALSE, &nmarks, flags);
 	if (error != ESUCCESS)
@@ -1883,23 +1891,23 @@ st_erase(unit, immed, flags)
  * The unix error number to pass back... (0 = report no error)
  *                            (SCSIRET_CONTINUE = continue processing)
  */
-errval 
+errval
 st_interpret_sense(xs)
 	struct scsi_xfer *xs;
 {
 	struct scsi_link *sc_link = xs->sc_link;
 	struct scsi_sense_data *sense = &(xs->sense);
 	boolean silent = xs->flags & SCSI_SILENT;
-	u_int32 unit = sc_link->dev_unit;
+	u_int32_t unit = sc_link->dev_unit;
 	struct scsi_data *st = SCSI_DATA(&st_switch, unit);
-	u_int32 key;
-	int32   info;
+	u_int32_t key;
+	int32_t   info;
 
 	/*
 	 * Get the sense fields and work out what code
 	 */
 	if (sense->error_code & SSD_ERRCODE_VALID) {
-		info = ntohl(*((int32 *) sense->ext.extended.info));
+		info = ntohl(*((int32_t *) sense->ext.extended.info));
 	} else {
 		if (st->flags & ST_FIXEDBLOCKS) {
 			info = xs->datalen / st->blksiz;
@@ -1929,7 +1937,7 @@ st_interpret_sense(xs)
 				    	,info);
 				/*XXX*/ /* is this how it works ? */
 				/* check def of ILI for fixed blk tapes */
-	
+
 				/*
 			 	 * This quirk code helps the drive read
 			 	 * the first tape block, regardless of
@@ -1938,7 +1946,7 @@ st_interpret_sense(xs)
 			 	 * information.
 			 	 */
 				if ((st->quirks & ST_Q_SNS_HLP) &&
-			    	!(sc_link->flags & SDEV_MEDIA_LOADED)) {
+				    !(st->flags & ST_SENSE_READ)) {
 					st->blksiz -= 512;
 				}
 			}
@@ -1994,7 +2002,7 @@ st_interpret_sense(xs)
 		 * MODE SENSE information.
 		 */
 		if ((st->quirks & ST_Q_SNS_HLP) &&
-		    !(sc_link->flags & SDEV_MEDIA_LOADED)) {
+		    !(st->flags & ST_SENSE_READ)) {
 			/* still starting */
 			st->blksiz -= 512;
 		} else if (!(st->flags & (ST_2FM_AT_EOD | ST_BLANK_READ))) {
@@ -2024,11 +2032,11 @@ st_interpret_sense(xs)
  */
 errval
 st_touch_tape(unit)
-	u_int32	unit;
+	u_int32_t	unit;
 {
 	struct scsi_data *st = SCSI_DATA(&st_switch, unit);
 	char   *buf;
-	u_int32 readsiz;
+	u_int32_t readsiz;
 	errval  errno;
 
 	buf = malloc(1024, M_TEMP, M_NOWAIT);
