@@ -1,8 +1,8 @@
-/* 
+/*
  * Driver for a device we can't identify.
  * by Julian Elischer (julian@tfs.com)
  *
- *      $Id: uk.c,v 1.7 1995/03/01 22:24:47 dufault Exp $
+ * $FreeBSD: stable/2.1/sys/scsi/uk.c 50950 1999-09-05 08:39:22Z peter $
  *
  * If you find that you are adding any code to this file look closely
  * at putting it in "scsi_driver.c" instead.
@@ -23,7 +23,7 @@ struct scsi_device uk_switch =
     "uk",
     0,
 	{0, 0},
-	SDEV_ONCE_ONLY,	/* Only one open allowed */
+	SDEV_ONCE_ONLY|SDEV_UK,	/* Only one open allowed */
 	0,
 	"Unknown",
 	ukopen,
