@@ -31,7 +31,7 @@
  * SUCH DAMAGE.
  *
  *	@(#)mount.h	8.13 (Berkeley) 3/27/94
- *	$Id: mount.h,v 1.17 1995/04/20 03:18:17 julian Exp $
+ * $FreeBSD: stable/2.1/sys/sys/mount.h 50950 1999-09-05 08:39:22Z peter $
  */
 
 #ifndef _SYS_MOUNT_H_
@@ -298,7 +298,7 @@ struct vfsops {
 	}; \
 	DATA_SET(vfs_set,_fs_vfsconf)
 #endif /* VFS_LKM */
-				
+
 #endif /* KERNEL */
 
 /*
@@ -401,7 +401,7 @@ struct iso_args {
 #define ISOFSMNT_EXTATT		0x00000004 /* enable extended attributes */
 #endif /* CD9660 */
 
-#ifdef NFS
+#if defined(NFS) || defined(NFS_SERVER) || defined(NFS_CLIENT)
 /*
  * File Handle (32 bytes for version 2), variable up to 1024 for version 3
  */
