@@ -31,7 +31,7 @@
  * SUCH DAMAGE.
  *
  *	@(#)ffs_alloc.c	8.8 (Berkeley) 2/21/94
- * $Id: ffs_alloc.c,v 1.12 1995/03/26 23:29:09 davidg Exp $
+ * $FreeBSD: stable/2.1/sys/ufs/ffs/ffs_alloc.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include <sys/param.h>
@@ -52,8 +52,6 @@
 #include <ufs/ffs/fs.h>
 #include <ufs/ffs/ffs_extern.h>
 
-extern u_long nextgennumber;
-
 static daddr_t	ffs_alloccg __P((struct inode *, int, daddr_t, int));
 static daddr_t	ffs_alloccgblk __P((struct fs *, struct cg *, daddr_t));
 static daddr_t	ffs_clusteralloc __P((struct inode *, int, daddr_t, int));
@@ -69,7 +67,7 @@ void		ffs_clusteracct	__P((struct fs *, struct cg *, daddr_t, int));
 
 /*
  * Allocate a block in the file system.
- * 
+ *
  * The size of the requested block is given, which must be some
  * multiple of fs_fsize and <= fs_bsize.
  * A preference may be optionally specified. If a preference is given
@@ -100,8 +98,8 @@ ffs_alloc(ip, lbn, bpref, size, cred, bnp)
 #ifdef QUOTA
 	int error;
 #endif
-	
-	
+
+
 	*bnp = 0;
 	fs = ip->i_fs;
 #ifdef DIAGNOSTIC
@@ -169,7 +167,7 @@ ffs_realloccg(ip, lbprev, bpref, osize, nsize, cred, bpp)
 	struct buf *bp;
 	int cg, request, error;
 	daddr_t bprev, bno;
-	
+
 	*bpp = 0;
 	fs = ip->i_fs;
 #ifdef DIAGNOSTIC
@@ -206,7 +204,7 @@ ffs_realloccg(ip, lbprev, bpref, osize, nsize, cred, bpp)
 			panic("ffs_realloccg: lbprev out of range");
 		bp->b_blkno = fsbtodb(fs, bprev);
 	}
-		
+
 #ifdef QUOTA
 	error = chkdq(ip, (long)btodb(nsize - osize), cred, 0);
 	if (error) {
@@ -238,8 +236,8 @@ ffs_realloccg(ip, lbprev, bpref, osize, nsize, cred, bpp)
 	switch ((int)fs->fs_optim) {
 	case FS_OPTSPACE:
 		/*
-		 * Allocate an exact sized fragment. Although this makes 
-		 * best use of space, we will waste time relocating it if 
+		 * Allocate an exact sized fragment. Although this makes
+		 * best use of space, we will waste time relocating it if
 		 * the file continues to grow. If the fragmentation is
 		 * less than half of the minimum free reserve, we choose
 		 * to begin optimizing for time.
@@ -432,7 +430,7 @@ ffs_reallocblks(ap)
 	 * Next we must write out the modified inode and indirect blocks.
 	 * For strict correctness, the writes should be synchronous since
 	 * the old block values may have been written to disk. In practise
-	 * they are almost never written, but if we are concerned about 
+	 * they are almost never written, but if we are concerned about
 	 * strict correctness, the `doasyncfree' flag should be set to zero.
 	 *
 	 * The test on `doasyncfree' should be changed to test a flag
@@ -477,7 +475,7 @@ fail:
 
 /*
  * Allocate an inode in the file system.
- * 
+ *
  * If allocating a directory, use ffs_dirpref to select the inode.
  * If allocating in a directory, the following hierarchy is followed:
  *   1) allocate the preferred inode.
@@ -506,7 +504,7 @@ ffs_valloc(ap)
 	mode_t mode = ap->a_mode;
 	ino_t ino, ipref;
 	int cg, error;
-	
+
 	*ap->a_vpp = NULL;
 	pip = VTOI(pvp);
 	fs = pip->i_fs;
@@ -543,9 +541,8 @@ ffs_valloc(ap)
 	/*
 	 * Set up a new generation number for this inode.
 	 */
-	if (++nextgennumber < (u_long)time.tv_sec)
-		nextgennumber = time.tv_sec;
-	ip->i_gen = nextgennumber;
+	if (ip->i_gen == 0 || ++(ip->i_gen) == 0)
+		ip->i_gen = random() / 2 + 1;
 	return (0);
 noinodes:
 	ffs_fserr(fs, ap->a_cred->cr_uid, "out of inodes");
@@ -582,7 +579,7 @@ ffs_dirpref(fs)
  * Select the desired position for the next block in a file.  The file is
  * logically divided into sections. The first section is composed of the
  * direct blocks. Each additional section contains fs_maxbpg blocks.
- * 
+ *
  * If no blocks have been allocated in the first section, the policy is to
  * request a block in the same cylinder group as the inode that describes
  * the file. If no blocks have been allocated in any other section, the
@@ -596,7 +593,7 @@ ffs_dirpref(fs)
  * indirect block, the information on the previous allocation is unavailable;
  * here a best guess is made based upon the logical block number being
  * allocated.
- * 
+ *
  * If a section is already partially allocated, the policy is to
  * contiguously allocate fs_maxcontig blocks.  The end of one of these
  * contiguous blocks and the beginning of the next is physically separated
@@ -726,7 +723,7 @@ ffs_hashalloc(ip, cg, pref, size, allocator)
 /*
  * Determine whether a fragment can be extended.
  *
- * Check to see if the necessary fragments are available, and 
+ * Check to see if the necessary fragments are available, and
  * if they are, allocate them.
  */
 static daddr_t
@@ -844,7 +841,7 @@ ffs_alloccg(ip, cg, bpref, size)
 			break;
 	if (allocsiz == fs->fs_frag) {
 		/*
-		 * no fragments were available, so a block will be 
+		 * no fragments were available, so a block will be
 		 * allocated, and hacked up
 		 */
 		if (cgp->cg_cs.cs_nbfree == 0) {
@@ -921,7 +918,7 @@ ffs_alloccgblk(fs, cgp, bpref)
 		/*
 		 * Block layout information is not available.
 		 * Leaving bpref unchanged means we take the
-		 * next available free block following the one 
+		 * next available free block following the one
 		 * we just allocated. Hopefully this will at
 		 * least hit a track cache on drives of unknown
 		 * geometry (e.g. SCSI).
@@ -935,7 +932,7 @@ ffs_alloccgblk(fs, cgp, bpref)
 	if (cg_blktot(cgp)[cylno] == 0)
 		goto norot;
 	/*
-	 * check the summary information to see if a block is 
+	 * check the summary information to see if a block is
 	 * available in the requested cylinder starting at the
 	 * requested rotational position and proceeding around.
 	 */
@@ -1172,7 +1169,7 @@ gotit:
  * Free a block or fragment.
  *
  * The specified block or fragment is placed back in the
- * free map. If a fragment is deallocated, a possible 
+ * free map. If a fragment is deallocated, a possible
  * block reassembly is checked.
  */
 void
@@ -1486,7 +1483,7 @@ ffs_clusteracct(fs, cgp, blkno, cnt)
 
 /*
  * Fserr prints the name of a file system with an error diagnostic.
- * 
+ *
  * The form of the error message is:
  *	fs: error message
  */
