@@ -31,7 +31,7 @@
  * SUCH DAMAGE.
  *
  *	@(#)ffs_vfsops.c	8.8 (Berkeley) 4/18/94
- * $Id: ffs_vfsops.c,v 1.19 1995/05/15 08:39:37 davidg Exp $
+ * $FreeBSD: stable/2.1/sys/ufs/ffs/ffs_vfsops.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 #include <sys/param.h>
@@ -85,8 +85,6 @@ struct vfsops ufs_vfsops = {
 
 VFS_SET(ufs_vfsops, ufs, MOUNT_UFS, 0);
 
-extern u_long nextgennumber;
-
 /*
  * Called by main() when ufs is going to be mounted as root.
  *
@@ -103,7 +101,7 @@ ffs_mountroot()
 	struct ufsmount *ump;
 	u_int size;
 	int error;
-	
+
 	/*
 	 * Get vnode for rootdev.
 	 */
@@ -246,7 +244,7 @@ ffs_mount(mp, path, data, ndp, p)
 	bzero(fs->fs_fsmnt + size, sizeof(fs->fs_fsmnt) - size);
 	bcopy((caddr_t)fs->fs_fsmnt, (caddr_t)mp->mnt_stat.f_mntonname,
 	    MNAMELEN);
-	(void) copyinstr(args.fspec, mp->mnt_stat.f_mntfromname, MNAMELEN - 1, 
+	(void) copyinstr(args.fspec, mp->mnt_stat.f_mntfromname, MNAMELEN - 1,
 	    &size);
 	bzero(mp->mnt_stat.f_mntfromname + size, MNAMELEN - size);
 	(void)ffs_statfs(mp, &mp->mnt_stat, p);
@@ -463,7 +461,10 @@ ffs_mountfs(devvp, mp, p)
 	}
 	mp->mnt_data = (qaddr_t)ump;
 	mp->mnt_stat.f_fsid.val[0] = (long)dev;
-	mp->mnt_stat.f_fsid.val[1] = MOUNT_UFS;
+	if (fs->fs_id[0] != 0 && fs->fs_id[1] != 0)
+		mp->mnt_stat.f_fsid.val[1] = fs->fs_id[1];
+	else
+		mp->mnt_stat.f_fsid.val[1] = MOUNT_UFS;
 	mp->mnt_maxsymlinklen = fs->fs_maxsymlinklen;
 	mp->mnt_flag |= MNT_LOCAL;
 	ump->um_mountp = mp;
@@ -633,8 +634,7 @@ ffs_statfs(mp, sbp, p)
 	sbp->f_blocks = fs->fs_dsize;
 	sbp->f_bfree = fs->fs_cstotal.cs_nbfree * fs->fs_frag +
 		fs->fs_cstotal.cs_nffree;
-	sbp->f_bavail = (fs->fs_dsize * (100 - fs->fs_minfree) / 100) -
-		(fs->fs_dsize - sbp->f_bfree);
+	sbp->f_bavail = freespace(fs, fs->fs_minfree);
 	sbp->f_files =  fs->fs_ncg * fs->fs_ipg - ROOTINO;
 	sbp->f_ffree = fs->fs_cstotal.cs_nifree;
 	if (sbp != &mp->mnt_stat) {
@@ -706,7 +706,7 @@ loop:
 						0, 0, 0);
 			vput(vp);
 		}
-			
+
 		if ((((ip->i_flag &
 		    (IN_ACCESS | IN_CHANGE | IN_MODIFIED | IN_UPDATE)) == 0)) &&
 		    vp->v_dirtyblkhd.lh_first == NULL)
@@ -741,6 +741,8 @@ loop:
  * return the inode locked.  Detection and handling of mount points must be
  * done by the calling routine.
  */
+int ffs_inode_hash_lock;
+
 int
 ffs_vget(mp, ino, vpp)
 	struct mount *mp;
@@ -757,17 +759,44 @@ ffs_vget(mp, ino, vpp)
 
 	ump = VFSTOUFS(mp);
 	dev = ump->um_dev;
+restart:
 	if ((*vpp = ufs_ihashget(dev, ino)) != NULL)
 		return (0);
 
+	/*
+	 * Lock out the creation of new entries in the FFS hash table in
+	 * case getnewvnode() or MALLOC() blocks, otherwise a duplicate
+	 * may occur!
+	 */
+	if (ffs_inode_hash_lock) {
+		while (ffs_inode_hash_lock) {
+			ffs_inode_hash_lock = -1;
+			tsleep(&ffs_inode_hash_lock, PVM, "ffsvgt", 0);
+		}
+		goto restart;
+	}
+	ffs_inode_hash_lock = 1;
+
+	/*
+	 * If this MALLOC() is performed after the getnewvnode()
+	 * it might block, leaving a vnode with a NULL v_data to be
+	 * found by ffs_sync() if a sync happens to fire right then,
+	 * which will cause a panic because ffs_sync() blindly
+	 * dereferences vp->v_data (as well it should).
+	 */
+	type = ump->um_devvp->v_tag == VT_MFS ? M_MFSNODE : M_FFSNODE; /* XXX */
+	MALLOC(ip, struct inode *, sizeof(struct inode), type, M_WAITOK);
+
 	/* Allocate a new vnode/inode. */
 	error = getnewvnode(VT_UFS, mp, ffs_vnodeop_p, &vp);
 	if (error) {
+		if (ffs_inode_hash_lock < 0)
+			wakeup(&ffs_inode_hash_lock);
+		ffs_inode_hash_lock = 0;
 		*vpp = NULL;
+		FREE(ip, type);
 		return (error);
 	}
-	type = ump->um_devvp->v_tag == VT_MFS ? M_MFSNODE : M_FFSNODE; /* XXX */
-	MALLOC(ip, struct inode *, sizeof(struct inode), type, M_WAITOK);
 	bzero((caddr_t)ip, sizeof(struct inode));
 	vp->v_data = ip;
 	ip->i_vnode = vp;
@@ -776,9 +805,9 @@ ffs_vget(mp, ino, vpp)
 	ip->i_number = ino;
 #ifdef QUOTA
 	{
-	int i;
-	for (i = 0; i < MAXQUOTAS; i++)
-		ip->i_dquot[i] = NODQUOT;
+		int i;
+		for (i = 0; i < MAXQUOTAS; i++)
+			ip->i_dquot[i] = NODQUOT;
 	}
 #endif
 	/*
@@ -789,6 +818,10 @@ ffs_vget(mp, ino, vpp)
 	 */
 	ufs_ihashins(ip);
 
+	if (ffs_inode_hash_lock < 0)
+		wakeup(&ffs_inode_hash_lock);
+	ffs_inode_hash_lock = 0;
+
 	/* Read in the disk contents for the inode, copy into the inode. */
 	error = bread(ump->um_devvp, fsbtodb(fs, ino_to_fsba(fs, ino)),
 	    (int)fs->fs_bsize, NOCRED, &bp);
@@ -827,9 +860,7 @@ ffs_vget(mp, ino, vpp)
 	 * already have one. This should only happen on old filesystems.
 	 */
 	if (ip->i_gen == 0) {
-		if (++nextgennumber < (u_long)time.tv_sec)
-			nextgennumber = time.tv_sec;
-		ip->i_gen = nextgennumber;
+		ip->i_gen = random() / 2 + 1;
 		if ((vp->v_mount->mnt_flag & MNT_RDONLY) == 0)
 			ip->i_flag |= IN_MODIFIED;
 	}
