@@ -39,7 +39,7 @@
  * from: Utah $Hdr: swap_pager.c 1.4 91/04/30$
  *
  *	@(#)swap_pager.c	8.9 (Berkeley) 3/21/94
- * $Id: swap_pager.c,v 1.39 1995/05/14 03:00:08 davidg Exp $
+ * $FreeBSD: stable/2.1/sys/vm/swap_pager.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -135,12 +135,12 @@ int require_swap_init;
 void swap_pager_finish();
 int dmmin, dmmax;
 
-static inline void 
+static inline void
 swapsizecheck()
 {
 	if (vm_swap_size < 128 * btodb(PAGE_SIZE)) {
 		if (swap_pager_full == 0)
-			printf("swap_pager: out of space\n");
+			printf("swap_pager: out of swap space\n");
 		swap_pager_full = 1;
 	} else if (vm_swap_size > 192 * btodb(PAGE_SIZE))
 		swap_pager_full = 0;
@@ -566,9 +566,6 @@ swap_pager_copy(srcpager, srcoffset, dstpager, dstoffset, offset)
 	if (vm_swap_size)
 		no_swap_space = 0;
 
-	if (no_swap_space)
-		return;
-
 	srcswp = (sw_pager_t) srcpager->pg_data;
 	origsize = srcswp->sw_allocsize;
 	dstswp = (sw_pager_t) dstpager->pg_data;
@@ -793,7 +790,7 @@ swap_pager_putpage(pager, m, sync)
 }
 
 static inline int
-const 
+const
 swap_pager_block_index(swp, offset)
 	sw_pager_t swp;
 	vm_offset_t offset;
@@ -802,7 +799,7 @@ swap_pager_block_index(swp, offset)
 }
 
 static inline int
-const 
+const
 swap_pager_block_offset(swp, offset)
 	sw_pager_t swp;
 	vm_offset_t offset;
@@ -1077,7 +1074,6 @@ swap_pager_input(swp, m, count, reqpage)
 	bp->b_bufsize = PAGE_SIZE * count;
 
 	pbgetvp(swapdev_vp, bp);
-	swp->sw_piip++;
 
 	cnt.v_swapin++;
 	cnt.v_swappgsin += count;
@@ -1102,11 +1098,6 @@ swap_pager_input(swp, m, count, reqpage)
 		rv = VM_PAGER_OK;
 	}
 
-	--swp->sw_piip;
-	if (swp->sw_piip == 0)
-		wakeup((caddr_t) swp);
-
-
 	/*
 	 * relpbuf does this, but we maintain our own buffer list also...
 	 */
