@@ -38,7 +38,7 @@
  * from: Utah $Hdr: vm_mmap.c 1.6 91/10/21$
  *
  *	@(#)vm_mmap.c	8.4 (Berkeley) 1/12/94
- * $Id: vm_mmap.c,v 1.22 1995/04/16 12:56:18 davidg Exp $
+ * $FreeBSD: stable/2.1/sys/vm/vm_mmap.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -181,7 +181,7 @@ mmap(p, uap, retval)
 	/*
 	 * XXX if no hint provided for a non-fixed mapping place it after the
 	 * end of the largest possible heap.
-	 * 
+	 *
 	 * There should really be a pmap call to determine a reasonable location.
 	 */
 	if (addr == 0 && (flags & MAP_FIXED) == 0)
@@ -609,21 +609,13 @@ vm_mmap(map, addr, size, prot, maxprot, flags, handle, foff)
 	struct vnode *vp = NULL;
 	int type;
 	int rv = KERN_SUCCESS;
+	vm_size_t objsize;
+	struct proc *p = curproc;
 
 	if (size == 0)
 		return (0);
 
-	size = round_page(size);
-
-	if ((flags & MAP_FIXED) == 0) {
-		fitit = TRUE;
-		*addr = round_page(*addr);
-	} else {
-		if (*addr != trunc_page(*addr))
-			return (EINVAL);
-		fitit = FALSE;
-		(void) vm_map_remove(map, *addr, *addr + size);
-	}
+	objsize = size = round_page(size);
 
 	/*
 	 * We currently can only deal with page aligned file offsets.
@@ -636,6 +628,16 @@ vm_mmap(map, addr, size, prot, maxprot, flags, handle, foff)
 	if (foff & PAGE_MASK)
 		return (EINVAL);
 
+	if ((flags & MAP_FIXED) == 0) {
+		fitit = TRUE;
+		*addr = round_page(*addr);
+	} else {
+		if (*addr != trunc_page(*addr))
+			return (EINVAL);
+		fitit = FALSE;
+		(void) vm_map_remove(map, *addr, *addr + size);
+	}
+
 	/*
 	 * Lookup/allocate pager.  All except an unnamed anonymous lookup gain
 	 * a reference to ensure continued existance of the object. (XXX the
@@ -653,10 +655,18 @@ vm_mmap(map, addr, size, prot, maxprot, flags, handle, foff)
 		if (vp->v_type == VCHR) {
 			type = PG_DEVICE;
 			handle = (caddr_t) vp->v_rdev;
-		} else
+		} else {
+			struct vattr vat;
+			int error;
+
+			error = VOP_GETATTR(vp, &vat, p->p_ucred, p);
+			if (error)
+				return (error);
+			objsize = vat.va_size;
 			type = PG_VNODE;
+		}
 	}
-	pager = vm_pager_allocate(type, handle, size, prot, foff);
+	pager = vm_pager_allocate(type, handle, objsize, prot, foff);
 	if (pager == NULL)
 		return (type == PG_DEVICE ? EINVAL : ENOMEM);
 	/*
@@ -760,7 +770,7 @@ vm_mmap(map, addr, size, prot, maxprot, flags, handle, foff)
 				vm_object_deallocate(user_object);
 				goto out;
 			}
-			
+
 			/*
 			 * this is a consistancy check, gets the map entry, and should
 			 * never fail
