@@ -65,7 +65,7 @@
  * any improvements or extensions that they make and grant Carnegie the
  * rights to redistribute these changes.
  *
- * $Id: vm_pageout.c,v 1.49 1995/05/10 18:56:06 davidg Exp $
+ * $FreeBSD: stable/2.1/sys/vm/vm_pageout.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -110,6 +110,8 @@ int vm_pageout_page_count = VM_PAGEOUT_PAGE_COUNT;
 
 int vm_page_max_wired;		/* XXX max # of wired pages system-wide */
 
+static void vm_req_vmdaemon __P((void));
+
 /*
  * vm_pageout_clean:
  * 	cleans a vm_page
@@ -121,10 +123,10 @@ vm_pageout_clean(m, sync)
 {
 	/*
 	 * Clean the page and remove it from the laundry.
-	 * 
+	 *
 	 * We set the busy bit to cause potential page faults on this page to
 	 * block.
-	 * 
+	 *
 	 * And we set pageout-in-progress to keep the object from disappearing
 	 * during pageout.  This guarantees that the page won't move from the
 	 * inactive queue.  (However, any other page on the inactive queue may
@@ -160,7 +162,7 @@ vm_pageout_clean(m, sync)
 	    (cnt.v_free_count + cnt.v_cache_count) < cnt.v_pageout_free_min)
 		return 0;
 
-	if ((!sync && m->hold_count != 0) ||
+	if ((!sync && m->hold_count != 0) || m->wire_count ||
 	    ((m->busy != 0) || (m->flags & PG_BUSY)))
 		return 0;
 
@@ -234,20 +236,20 @@ vm_pageout_clean(m, sync)
 				pageout_count += b_pageout_count;
 			}
 		}
-			
+
 		/*
 		 * we allow reads during pageouts...
 		 */
 		for (i = 0; i < pageout_count; i++) {
 			ms[i]->flags |= PG_BUSY;
-			vm_page_protect(ms[i], VM_PROT_READ);
+			vm_page_protect(ms[i], VM_PROT_NONE);
 		}
 		object->paging_in_progress += pageout_count;
 	} else {
 
 		m->flags |= PG_BUSY;
 
-		vm_page_protect(m, VM_PROT_READ);
+		vm_page_protect(m, VM_PROT_NONE);
 
 		object->paging_in_progress++;
 
@@ -339,176 +341,158 @@ vm_pageout_clean(m, sync)
 	return anyok;
 }
 
+#ifndef NO_SWAPPING
 /*
  *	vm_pageout_object_deactivate_pages
  *
- *	deactivate enough pages to satisfy the inactive target
- *	requirements or if vm_page_proc_limit is set, then
- *	deactivate all of the pages in the object and its
- *	shadows.
+ *	Deactivate pages in the object chain until desired count
+ *	is reached.
  *
  *	The object and map must be locked.
  */
-int
-vm_pageout_object_deactivate_pages(map, object, count, map_remove_only)
+static void
+vm_pageout_object_deactivate_pages(map, object, desired, map_remove_only)
 	vm_map_t map;
 	vm_object_t object;
-	int count;
+	u_int desired;
 	int map_remove_only;
 {
 	register vm_page_t p, next;
 	int rcount;
-	int dcount;
-
-	dcount = 0;
-	if (count == 0)
-		count = 1;
+	int s;
 
 	if (object->pager && (object->pager->pg_type == PG_DEVICE))
-		return 0;
+		return;
 
-	if (object->shadow) {
-		if (object->shadow->ref_count == 1)
-			dcount += vm_pageout_object_deactivate_pages(map, object->shadow, count / 2 + 1, map_remove_only);
-		else
-			vm_pageout_object_deactivate_pages(map, object->shadow, count, 1);
-	}
-	if (object->paging_in_progress || !vm_object_lock_try(object))
-		return dcount;
+	while (object) {
+		if (vm_map_pmap(map)->pm_stats.resident_count <= desired)
+			return;
+		if (object->paging_in_progress)
+			return;
 
-	/*
-	 * scan the objects entire memory queue
-	 */
-	rcount = object->resident_page_count;
-	p = object->memq.tqh_first;
-	while (p && (rcount-- > 0)) {
-		next = p->listq.tqe_next;
-		cnt.v_pdpages++;
-		vm_page_lock_queues();
-		if (p->wire_count != 0 ||
-		    p->hold_count != 0 ||
-		    p->busy != 0 ||
-		    !pmap_page_exists(vm_map_pmap(map), VM_PAGE_TO_PHYS(p))) {
-			p = next;
-			continue;
-		}
 		/*
-		 * if a page is active, not wired and is in the processes
-		 * pmap, then deactivate the page.
+		 * scan the objects entire memory queue
 		 */
-		if ((p->flags & (PG_ACTIVE | PG_BUSY)) == PG_ACTIVE) {
-			if (!pmap_is_referenced(VM_PAGE_TO_PHYS(p)) &&
-			    (p->flags & (PG_REFERENCED|PG_WANTED)) == 0) {
-				p->act_count -= min(p->act_count, ACT_DECLINE);
-				/*
-				 * if the page act_count is zero -- then we
-				 * deactivate
-				 */
-				if (!p->act_count) {
+		rcount = object->resident_page_count;
+		p = object->memq.tqh_first;
+		while (p && (rcount-- > 0)) {
+			if (vm_map_pmap(map)->pm_stats.resident_count <= desired)
+				return;
+			next = p->listq.tqe_next;
+			cnt.v_pdpages++;
+			if (p->wire_count != 0 ||
+			    p->hold_count != 0 ||
+			    p->busy != 0 ||
+			    (p->flags & PG_BUSY) ||
+			    !pmap_page_exists(vm_map_pmap(map), VM_PAGE_TO_PHYS(p))) {
+				p = next;
+				continue;
+			}
+
+			if (pmap_is_referenced(VM_PAGE_TO_PHYS(p))) {
+				pmap_clear_reference(VM_PAGE_TO_PHYS(p));
+				p->flags |= PG_REFERENCED;
+			}
+
+			if ((p->flags & PG_INACTIVE) &&
+				(p->flags & PG_REFERENCED)) {
+				vm_page_activate(p);
+			}
+
+			/*
+			 * if a page is active, not wired and is in the processes
+			 * pmap, then deactivate the page.
+			 */
+			if (p->flags & PG_ACTIVE) {
+				if ((p->flags & PG_REFERENCED) == 0) {
+					p->act_count -= min(p->act_count, ACT_DECLINE);
+					vm_page_protect(p, VM_PROT_NONE);
 					if (!map_remove_only)
 						vm_page_deactivate(p);
-					vm_page_protect(p, VM_PROT_NONE);
-					/*
-					 * else if on the next go-around we
-					 * will deactivate the page we need to
-					 * place the page on the end of the
-					 * queue to age the other pages in
-					 * memory.
-					 */
 				} else {
+					p->flags &= ~PG_REFERENCED;
+					s = splbio();
 					TAILQ_REMOVE(&vm_page_queue_active, p, pageq);
 					TAILQ_INSERT_TAIL(&vm_page_queue_active, p, pageq);
+					splx(s);
 				}
-				/*
-				 * see if we are done yet
-				 */
-				if (p->flags & PG_INACTIVE) {
-					--count;
-					++dcount;
-					if (count <= 0 &&
-					    cnt.v_inactive_count > cnt.v_inactive_target) {
-						vm_page_unlock_queues();
-						vm_object_unlock(object);
-						return dcount;
-					}
-				}
-			} else {
-				/*
-				 * Move the page to the bottom of the queue.
-				 */
-				pmap_clear_reference(VM_PAGE_TO_PHYS(p));
-				p->flags &= ~PG_REFERENCED;
-				if (p->act_count < ACT_MAX)
-					p->act_count += ACT_ADVANCE;
-
-				TAILQ_REMOVE(&vm_page_queue_active, p, pageq);
-				TAILQ_INSERT_TAIL(&vm_page_queue_active, p, pageq);
+			} else if (p->flags & PG_INACTIVE) {
+				vm_page_protect(p, VM_PROT_NONE);
 			}
-		} else if ((p->flags & (PG_INACTIVE | PG_BUSY)) == PG_INACTIVE) {
-			vm_page_protect(p, VM_PROT_NONE);
+			p = next;
 		}
-		vm_page_unlock_queues();
-		p = next;
+		object = object->shadow;
 	}
-	vm_object_unlock(object);
-	return dcount;
+	return;
 }
 
-
 /*
  * deactivate some number of pages in a map, try to do it fairly, but
  * that is really hard to do.
  */
-
-void
-vm_pageout_map_deactivate_pages(map, entry, count, freeer)
+static void
+vm_pageout_map_deactivate_pages(map, desired)
 	vm_map_t map;
-	vm_map_entry_t entry;
-	int *count;
-	int (*freeer) (vm_map_t, vm_object_t, int);
+	u_int desired;
 {
-	vm_map_t tmpm;
 	vm_map_entry_t tmpe;
-	vm_object_t obj;
+	vm_object_t obj, bigobj;
 
-	if (*count <= 0)
-		return;
 	vm_map_reference(map);
-	if (!lock_try_read(&map->lock)) {
+	if (!lock_try_write(&map->lock)) {
 		vm_map_deallocate(map);
 		return;
 	}
-	if (entry == 0) {
-		tmpe = map->header.next;
-		while (tmpe != &map->header && *count > 0) {
-			vm_pageout_map_deactivate_pages(map, tmpe, count, freeer, 0);
-			tmpe = tmpe->next;
-		};
-	} else if (entry->is_sub_map || entry->is_a_map) {
-		tmpm = entry->object.share_map;
-		tmpe = tmpm->header.next;
-		while (tmpe != &tmpm->header && *count > 0) {
-			vm_pageout_map_deactivate_pages(tmpm, tmpe, count, freeer, 0);
-			tmpe = tmpe->next;
-		};
-	} else if ((obj = entry->object.vm_object) != 0) {
-		*count -= (*freeer) (map, obj, *count);
+
+	bigobj = NULL;
+
+	/*
+	 * first, search out the biggest object, and try to free pages from
+	 * that.
+	 */
+	tmpe = map->header.next;
+	while (tmpe != &map->header) {
+		if ((tmpe->is_sub_map == 0) && (tmpe->is_a_map == 0)) {
+			obj = tmpe->object.vm_object;
+			if ((obj != NULL) && ((bigobj == NULL) ||
+				 (bigobj->resident_page_count < obj->resident_page_count))) {
+				bigobj = obj;
+			}
+		}
+		tmpe = tmpe->next;
 	}
-	lock_read_done(&map->lock);
+
+	if (bigobj)
+		vm_pageout_object_deactivate_pages(map, bigobj, desired, 0);
+
+	/*
+	 * Next, hunt around for other pages to deactivate.  We actually
+	 * do this search sort of wrong -- .text first is not the best idea.
+	 */
+	tmpe = map->header.next;
+	while (tmpe != &map->header) {
+		if (vm_map_pmap(map)->pm_stats.resident_count <= desired)
+			break;
+		if ((tmpe->is_sub_map == 0) && (tmpe->is_a_map == 0)) {
+			obj = tmpe->object.vm_object;
+			if (obj)
+				vm_pageout_object_deactivate_pages(map, obj, desired, 0);
+		}
+		tmpe = tmpe->next;
+	};
+
+	/*
+	 * Remove all mappings if a process is swapped out, this will free page
+	 * table pages.
+	 */
+	if (desired == 0)
+		pmap_remove(vm_map_pmap(map),
+			VM_MIN_ADDRESS, VM_MAXUSER_ADDRESS);
+	vm_map_unlock(map);
 	vm_map_deallocate(map);
 	return;
 }
-
-void
-vm_req_vmdaemon()
-{
-	static int lastrun = 0;
-
-	if ((ticks > (lastrun + hz / 10)) || (ticks < lastrun)) {
-		wakeup((caddr_t) &vm_daemon_needed);
-		lastrun = ticks;
-	}
-}
+#endif
 
 /*
  *	vm_pageout_scan does the dirty work for the pageout daemon.
@@ -525,6 +509,7 @@ vm_pageout_scan()
 	vm_object_t object;
 	int force_wakeup = 0;
 	int vnodes_skipped = 0;
+	int s;
 
 	pages_freed = 0;
 
@@ -535,6 +520,7 @@ vm_pageout_scan()
 	 * them.
 	 */
 
+rescan0:
 	maxlaunder = (cnt.v_inactive_target > MAXLAUNDER) ?
 	    MAXLAUNDER : cnt.v_inactive_target;
 
@@ -548,19 +534,18 @@ rescan1:
 		cnt.v_pdpages++;
 		next = m->pageq.tqe_next;
 
-#if defined(VM_DIAGNOSE)
 		if ((m->flags & PG_INACTIVE) == 0) {
-			printf("vm_pageout_scan: page not inactive?\n");
-			break;
+			goto rescan1;
 		}
-#endif
 
 		/*
 		 * dont mess with busy pages
 		 */
 		if (m->hold_count || m->busy || (m->flags & PG_BUSY)) {
+			s = splbio();
 			TAILQ_REMOVE(&vm_page_queue_inactive, m, pageq);
 			TAILQ_INSERT_TAIL(&vm_page_queue_inactive, m, pageq);
+			splx(s);
 			m = next;
 			continue;
 		}
@@ -593,9 +578,6 @@ rescan1:
 					vm_page_cache(m);
 				}
 				++pages_freed;
-			} else {
-				m = next;
-				continue;
 			}
 		} else if (maxlaunder > 0) {
 			int written;
@@ -616,6 +598,18 @@ rescan1:
 					m = next;
 					continue;
 				}
+				/*
+				 * We might have blocked above, so make sure
+				 * the page didn't move on us.
+				 */
+				if ((m->flags & PG_INACTIVE) == 0 || m->dirty == 0) {
+					vput(vp);
+					vm_object_unlock(object);
+					if (object->flags & OBJ_WRITEABLE)
+						++vnodes_skipped;
+					m = next;
+					continue;
+				}
 			}
 
 			/*
@@ -635,13 +629,6 @@ rescan1:
 				break;
 			}
 			maxlaunder -= written;
-			/*
-			 * if the next page has been re-activated, start
-			 * scanning again
-			 */
-			if ((next->flags & PG_INACTIVE) == 0) {
-				goto rescan1;
-			}
 		}
 		m = next;
 	}
@@ -661,6 +648,7 @@ rescan1:
 			page_shortage = 1;
 		}
 	}
+rescan_active:
 	maxscan = MAXSCAN;
 	pcount = cnt.v_active_count;
 	m = vm_page_queue_active.tqh_first;
@@ -669,14 +657,19 @@ rescan1:
 		cnt.v_pdpages++;
 		next = m->pageq.tqe_next;
 
+		if ((m->flags & PG_ACTIVE) == 0)
+			break;
+
 		/*
 		 * Don't deactivate pages that are busy.
 		 */
 		if ((m->busy != 0) ||
 		    (m->flags & PG_BUSY) ||
 		    (m->hold_count != 0)) {
+			s = splbio();
 			TAILQ_REMOVE(&vm_page_queue_active, m, pageq);
 			TAILQ_INSERT_TAIL(&vm_page_queue_active, m, pageq);
+			splx(s);
 			m = next;
 			continue;
 		}
@@ -689,8 +682,10 @@ rescan1:
 			if (m->act_count < ACT_MAX) {
 				m->act_count += ACT_ADVANCE;
 			}
+			s = splbio();
 			TAILQ_REMOVE(&vm_page_queue_active, m, pageq);
 			TAILQ_INSERT_TAIL(&vm_page_queue_active, m, pageq);
+			splx(s);
 		} else {
 			m->flags &= ~PG_REFERENCED;
 			pmap_clear_reference(VM_PAGE_TO_PHYS(m));
@@ -700,22 +695,13 @@ rescan1:
 			 * if the page act_count is zero -- then we deactivate
 			 */
 			if (!m->act_count && (page_shortage > 0)) {
-				if (m->object->ref_count == 0) {
-					--page_shortage;
-					vm_page_test_dirty(m);
-					if ((m->bmapped == 0) && (m->dirty == 0) ) {
-						m->act_count = 0;
-						vm_page_cache(m);
-					} else {
-						vm_page_deactivate(m);
-					}
-				} else {
-					vm_page_deactivate(m);
-					--page_shortage;
-				}
+				vm_page_deactivate(m);
+				--page_shortage;
 			} else if (m->act_count) {
+				s = splbio();
 				TAILQ_REMOVE(&vm_page_queue_active, m, pageq);
 				TAILQ_INSERT_TAIL(&vm_page_queue_active, m, pageq);
+				splx(s);
 			}
 		}
 		maxscan--;
@@ -747,6 +733,7 @@ rescan1:
 				wakeup((caddr_t) &vfs_update_wakeup);
 			}
 		}
+#ifndef NO_SWAPPING
 		/*
 		 * now swap processes out if we are in low memory conditions
 		 */
@@ -755,12 +742,15 @@ rescan1:
 			vm_pageout_req_swapout = 1;
 			vm_req_vmdaemon();
 		}
+#endif
 	}
 
+#ifndef NO_SWAPPING
 	if ((cnt.v_inactive_count + cnt.v_free_count + cnt.v_cache_count) <
 	    (cnt.v_inactive_target + cnt.v_free_min)) {
 		vm_req_vmdaemon();
 	}
+#endif
 
 	/*
 	 * make sure that we have swap space -- if we are low on memory and
@@ -799,8 +789,7 @@ rescan1:
 			}
 		}
 		if (bigproc != NULL) {
-			printf("Process %lu killed by vm_pageout -- out of swap\n", (u_long) bigproc->p_pid);
-			psignal(bigproc, SIGKILL);
+			killproc(bigproc, "out of swap space");
 			bigproc->p_estcpu = 0;
 			bigproc->p_nice = PRIO_MIN;
 			resetpriority(bigproc);
@@ -877,12 +866,26 @@ vm_pageout()
 	}
 }
 
+#ifndef NO_SWAPPING
+static void
+vm_req_vmdaemon()
+{
+	static int lastrun = 0;
+
+	if ((ticks > (lastrun + hz)) || (ticks < lastrun)) {
+		wakeup((caddr_t) &vm_daemon_needed);
+		lastrun = ticks;
+	}
+}
+
 void
 vm_daemon()
 {
 	vm_object_t object;
 	struct proc *p;
 
+	(void) spl0();
+
 	while (TRUE) {
 		tsleep((caddr_t) &vm_daemon_needed, PUSER, "psleep", 0);
 		if( vm_pageout_req_swapout) {
@@ -895,7 +898,6 @@ vm_daemon()
 		 */
 
 		for (p = (struct proc *) allproc; p != NULL; p = p->p_next) {
-			int overage;
 			quad_t limit;
 			vm_offset_t size;
 
@@ -929,32 +931,10 @@ vm_daemon()
 
 			size = p->p_vmspace->vm_pmap.pm_stats.resident_count * NBPG;
 			if (limit >= 0 && size >= limit) {
-				overage = (size - limit) / NBPG;
 				vm_pageout_map_deactivate_pages(&p->p_vmspace->vm_map,
-				    (vm_map_entry_t) 0, &overage, vm_pageout_object_deactivate_pages);
+					(u_int) (limit >> PAGE_SHIFT));
 			}
 		}
 	}
-
-	/*
-	 * we remove cached objects that have no RSS...
-	 */
-restart:
-	vm_object_cache_lock();
-	object = vm_object_cached_list.tqh_first;
-	while (object) {
-		vm_object_cache_unlock();
-		/*
-		 * if there are no resident pages -- get rid of the object
-		 */
-		if (object->resident_page_count == 0) {
-			if (object != vm_object_lookup(object->pager))
-				panic("vm_object_cache_trim: I'm sooo confused.");
-			pager_cache(object, FALSE);
-			goto restart;
-		}
-		object = object->cached_list.tqe_next;
-		vm_object_cache_lock();
-	}
-	vm_object_cache_unlock();
 }
+#endif /* !NO_SWAPPING */
