@@ -37,7 +37,7 @@
  * SUCH DAMAGE.
  *
  *	from: @(#)vnode_pager.c	7.5 (Berkeley) 4/20/91
- *	$Id: vnode_pager.c,v 1.38 1995/05/10 18:56:09 davidg Exp $
+ * $FreeBSD: stable/2.1/sys/vm/vnode_pager.c 50950 1999-09-05 08:39:22Z peter $
  */
 
 /*
@@ -124,11 +124,8 @@ vnode_pager_alloc(handle, size, prot, offset)
 {
 	register vm_pager_t pager;
 	register vn_pager_t vnp;
-	vm_object_t object, tobject;
-	struct vattr vattr;
+	vm_object_t object;
 	struct vnode *vp;
-	struct proc *p = curproc;	/* XXX */
-	int rtval;
 
 	/*
 	 * Pageout to vnode, no can do yet.
@@ -136,14 +133,24 @@ vnode_pager_alloc(handle, size, prot, offset)
 	if (handle == NULL)
 		return (NULL);
 
-	/*
-	 * Vnodes keep a pointer to any associated pager so no need to lookup
-	 * with vm_pager_lookup.
-	 */
 	vp = (struct vnode *) handle;
-	while ((object = (vm_object_t) vp->v_vmdata) &&
-		(object->flags & OBJ_DEAD))
-		tsleep((caddr_t) object, PVM, "vadead", 0);
+
+	/*
+	 * Prevent race condition when allocating the object. This
+	 * can happen with NFS vnodes since the nfsnode isn't locked.
+	 */
+	while (vp->v_flag & VOLOCK) {
+		vp->v_flag |= VOWANT;
+		tsleep(vp, PVM, "vnpobj", 0);
+	}
+	vp->v_flag |= VOLOCK;
+
+	/*
+	 * If the object is being terminated, wait for it to
+	 * go away.
+	 */
+	while (((object = (vm_object_t) vp->v_vmdata) != NULL) && (object->flags & OBJ_DEAD))
+		tsleep(object, PVM, "vadead", 0);
 
 	pager = NULL;
 	if (object != NULL)
@@ -154,26 +161,15 @@ vnode_pager_alloc(handle, size, prot, offset)
 		 * Allocate pager structures
 		 */
 		pager = (vm_pager_t) malloc(sizeof *pager, M_VMPAGER, M_WAITOK);
-		if (pager == NULL)
-			return (NULL);
 		vnp = (vn_pager_t) malloc(sizeof *vnp, M_VMPGDATA, M_WAITOK);
-		if (vnp == NULL) {
-			free((caddr_t) pager, M_VMPAGER);
-			return (NULL);
-		}
+
 		/*
 		 * And an object of the appropriate size
 		 */
-		if ((rtval = VOP_GETATTR(vp, &vattr, p->p_ucred, p)) == 0) {
-			object = vm_object_allocate(round_page(vattr.va_size));
-			object->flags = OBJ_CANPERSIST;
-			vm_object_enter(object, pager);
-			object->pager = pager;
-		} else {
-			free((caddr_t) vnp, M_VMPGDATA);
-			free((caddr_t) pager, M_VMPAGER);
-			return (NULL);
-		}
+		object = vm_object_allocate(round_page(size));
+		object->flags = OBJ_CANPERSIST;
+		vm_object_enter(object, pager);
+		object->pager = pager;
 
 		/*
 		 * Hold a reference to the vnode and initialize pager data.
@@ -181,7 +177,7 @@ vnode_pager_alloc(handle, size, prot, offset)
 		VREF(vp);
 		vnp->vnp_flags = 0;
 		vnp->vnp_vp = vp;
-		vnp->vnp_size = vattr.va_size;
+		vnp->vnp_size = size;
 
 		TAILQ_INSERT_TAIL(&vnode_pager_list, pager, pg_list);
 		pager->pg_handle = handle;
@@ -197,8 +193,15 @@ vnode_pager_alloc(handle, size, prot, offset)
 		 */
 		(void) vm_object_lookup(pager);
 	}
-	if( vp->v_type == VREG)
+
+	if (vp->v_type == VREG)
 		vp->v_flag |= VVMIO;
+
+	vp->v_flag &= ~VOLOCK;
+	if (vp->v_flag & VOWANT) {
+		vp->v_flag &= ~VOWANT;
+		wakeup(vp);
+	}
 	return (pager);
 }
 
@@ -314,11 +317,11 @@ vnode_pager_haspage(pager, offset)
 	/*
 	 * Read the index to find the disk block to read from.  If there is no
 	 * block, report that we don't have this data.
-	 * 
+	 *
 	 * Assumes that the vnode has whole page or nothing.
 	 */
 	err = VOP_BMAP(vp, block, (struct vnode **) 0, &bn, 0);
-	if (err) 
+	if (err)
 		return (TRUE);
 	return ((long) bn < 0 ? FALSE : TRUE);
 }
