@@ -1,5 +1,5 @@
 #ifndef lint
-static char rcsid[] = "$Id: dig.c,v 1.2 1994/09/22 21:51:49 pst Exp $";
+static char rcsid[] = "$FreeBSD: stable/2.1/usr.bin/dig/dig.c 50952 1999-09-05 11:54:50Z peter $";
 #endif
 
 /*
@@ -7,7 +7,7 @@ static char rcsid[] = "$Id: dig.c,v 1.2 1994/09/22 21:51:49 pst Exp $";
  * -
  * Copyright (c) 1989
  *    The Regents of the University of California.  All rights reserved.
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -23,7 +23,7 @@ static char rcsid[] = "$Id: dig.c,v 1.2 1994/09/22 21:51:49 pst Exp $";
  * 4. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
- * 
+ *
  * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -37,14 +37,14 @@ static char rcsid[] = "$Id: dig.c,v 1.2 1994/09/22 21:51:49 pst Exp $";
  * SUCH DAMAGE.
  * -
  * Portions Copyright (c) 1993 by Digital Equipment Corporation.
- * 
+ *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
  * copyright notice and this permission notice appear in all copies, and that
  * the name of Digital Equipment Corporation not be used in advertising or
  * publicity pertaining to distribution of the document or software without
  * specific, written prior permission.
- * 
+ *
  * THE SOFTWARE IS PROVIDED "AS IS" AND DIGITAL EQUIPMENT CORP. DISCLAIMS ALL
  * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
  * OF MERCHANTABILITY AND FITNESS.   IN NO EVENT SHALL DIGITAL EQUIPMENT
@@ -157,14 +157,14 @@ static char rcsid[] = "$Id: dig.c,v 1.2 1994/09/22 21:51:49 pst Exp $";
 #include <netdb.h>
 #include <stdio.h>
 #include <resolv.h>
-#include <ctype.h> 
+#include <ctype.h>
 #include <errno.h>
 #include <string.h>
 #include <setjmp.h>
 #include <fcntl.h>
 #include <stdlib.h>
 
-#include "../../usr.sbin/nslookup/res.h"
+#include "res.h"
 
 #define PRF_DEF		0x2ff9
 #define PRF_MIN		0xA930
@@ -197,8 +197,7 @@ jmp_buf		env;
 HostInfo	*defaultPtr = NULL;
 HostInfo	curHostInfo, defaultRec;
 int		curHostValid = FALSE;
-int		queryType = T_A;
-int		queryClass = C_IN;
+int		queryType, queryClass;
 extern int	StringToClass(), StringToType();	/* subr.c */
 #if defined(BSD) && BSD >= 199006 && !defined(RISCOS_BSD)
 FILE		*yyin = NULL;
@@ -221,7 +220,7 @@ stackarg(y, l)
 		case '\t':
 		case ' ':
 			l++;    break;
-		case NULL:
+		case '\0':
 		case '\n':
 			done++;
 			*y = NULL;
@@ -246,7 +245,12 @@ main(argc, argv)
 {
 	struct hostent *hp;
 	short port = htons(NAMESERVER_PORT);
-	u_char packet[PACKETSZ];
+	/* Wierd stuff for SPARC alignment, hurts nothing else. */
+	union {
+		HEADER header_;
+		u_char packet_[PACKETSZ];
+	} packet_;
+#define	packet  (packet_.packet_)
 	u_char answer[8*1024];
 	int n;
 	char doping[90];
@@ -260,7 +264,7 @@ main(argc, argv)
 	int anyflag = 0;
 	int sticky = 0;
 	int tmp; 
-	int qtype = 1, qclass = 1;
+	int qtypeSet;
 	int addrflag = 0;
 	int zone = 0;
         int bytes_out, bytes_in;
@@ -283,6 +287,7 @@ main(argc, argv)
 
 	res_init();
 	_res.pfcode = PRF_DEF;
+	qtypeSet = 0;
 	gethostname(myhostname, (sizeof myhostname));
 	defsrv = strcat(defbuf, inet_ntoa(_res.nsaddr.sin_addr));
 	res_x = _res;
@@ -296,7 +301,7 @@ main(argc, argv)
 	if ((((afile = (char *) getenv("LOCALDEF")) != (char *) NULL) &&
 	     ((fp = open(afile, O_RDONLY)) > 0)) ||
 	    ((fp = open(SAVEENV, O_RDONLY)) > 0)) {
-		read(fp, &res_x, (sizeof res_x));
+		read(fp, (char *)&res_x, (sizeof res_x));
 		close(fp);
 		_res = res_x;
 	}
@@ -340,8 +345,8 @@ main(argc, argv)
  **                while !EOF if batch mode
  */
 	*fileq = '\0';
-	while ((dofile && (fgets(fileq,100,qfp) != NULL)) || 
-	       ((!dofile) && (once--))) 
+	while ((dofile && (fgets(fileq,100,qfp) != NULL)) ||
+	       ((!dofile) && (once--)))
 	{
 		if ((*fileq=='\n') || (*fileq=='#') || (*fileq==';')) {
 			continue; /* ignore blank lines & comments */
@@ -362,7 +367,8 @@ main(argc, argv)
 		stackarg(ay, qptr);
 
 		/* defaults */
-		qtype = qclass = 1;
+		queryType = T_NS;
+		queryClass = C_IN;
 		zone = 0;
 		*pingstr = 0;
 		srv = NULL;
@@ -374,7 +380,7 @@ main(argc, argv)
  * More cmd-line options than anyone should ever have to
  * deal with ....
  */
-		while (*(++argv) != NULL && **argv != '\0') { 
+		while (*(++argv) != NULL && **argv != '\0') {
 			strcat(cmd,*argv); strcat(cmd," ");
 			if (**argv == '@') {
 				srv = (*argv+1);
@@ -386,7 +392,7 @@ main(argc, argv)
 				SetOption(*argv+1);
 				continue;
 			}
-	 
+
 			if (strncmp(*argv,"-nost",5) == 0) {
 				sticky = 0;
 				continue;
@@ -402,31 +408,33 @@ main(argc, argv)
 			}
 
 			if (**argv == '-') {
-				switch (argv[0][1]) { 
+				switch (argv[0][1]) {
 				case 'T': wait = atoi(*++argv);
 					break;
-				case 'c': 
+				case 'c':
 					if ((tmp = atoi(*++argv))
 					    || *argv[0]=='0') {
-						qclass = tmp;
+						queryClass = tmp;
 					} else if (tmp = StringToClass(*argv,
 								       0, NULL)
 						   ) {
-						qclass = tmp;
+						queryClass = tmp;
 					} else {
 						printf(
 						  "; invalid class specified\n"
 						       );
 					}
 					break;
-				case 't': 
+				case 't':
 					if ((tmp = atoi(*++argv))
 					    || *argv[0]=='0') {
-						qtype = tmp;
+						queryType = tmp;
+						qtypeSet++;
 					} else if (tmp = StringToType(*argv,
 								      0, NULL)
 						   ) {
-						qtype = tmp;
+						queryType = tmp;
+						qtypeSet++;
 					} else {
 						printf(
 						   "; invalid type specified\n"
@@ -434,8 +442,10 @@ main(argc, argv)
 						}
 					break;
 				case 'x':
-					if (qtype == T_A)
-						qtype = T_ANY;
+					if (!qtypeSet) {
+						queryType = T_ANY;
+						qtypeSet++;
+					}
 					if (!(addrc = *++argv)) {
 						printf(
 						       "; no arg for -x?\n"
@@ -472,18 +482,19 @@ main(argc, argv)
 
 			if ((tmp = StringToType(*argv, -1, NULL)) != -1) { 
 				if ((T_ANY == tmp) && anyflag++) {  
-					qclass = C_ANY; 	
+					queryClass = C_ANY; 	
 					continue; 
 				}
 				if (T_AXFR == tmp) {
 					_res.pfcode = PRF_ZONE;
 					zone++;
 				} else {
-					qtype = tmp; 
+					queryType = tmp; 
+					qtypeSet++;
 				}
 			} else if ((tmp = StringToClass(*argv, -1, NULL))
 				   != -1) { 
-				qclass = tmp; 
+				queryClass = tmp; 
 			} else {
 				bzero(domain, (sizeof domain));
 				sprintf(domain,"%s",*argv);
@@ -493,7 +504,7 @@ main(argc, argv)
 		if (_res.pfcode & 0x80000)
 			printf("; pfcode: %08x, options: %08x\n",
 			       _res.pfcode, _res.options);
-	  
+
 /*
  * Current env. (after this parse) is to become the
  * new "working environmnet. Used in conj. with sticky.
@@ -518,7 +529,7 @@ main(argc, argv)
 			    ((fp = open(SAVEENV,
 					O_WRONLY|O_CREAT|O_TRUNC,
 					S_IREAD|S_IWRITE)) > 0)) {
-				write(fp, &_res, (sizeof _res));
+				write(fp, (char *)&_res, (sizeof _res));
 				close(fp);
 			}
 			envsave = 0;
@@ -531,7 +542,7 @@ main(argc, argv)
 
 /*
  * Find address of server to query. If not dot-notation, then
- * try to resolve domain-name (if so, save and turn off print 
+ * try to resolve domain-name (if so, save and turn off print
  * options, this domain-query is not the one we want. Restore
  * user options when done.
  * Things get a bit wierd since we need to use resolver to be
@@ -622,7 +633,13 @@ main(argc, argv)
 			continue;
 		}
 
-		bytes_out = n = res_mkquery(QUERY, domain, qclass, qtype,
+		if (*domain && !qtypeSet) {
+			queryType = T_A;
+			qtypeSet++;
+		}
+		
+		bytes_out = n = res_mkquery(QUERY, domain,
+					    queryClass, queryType,
 					    NULL, 0, NULL,
 					    packet, sizeof(packet));
 		if (n < 0) {
@@ -665,7 +682,7 @@ main(argc, argv)
 			printf(";; MSG SIZE  sent: %d  rcvd: %d\n",
 			       bytes_out, bytes_in);
 		}
-	  
+
 		fflush(stdout);
 /*
  *   Argh ... not particularly elegant. Should put in *real* ping code.
@@ -741,8 +758,8 @@ SetOption(string)
     if (i != 1) {
 	fprintf(stderr, ";*** Invalid option: %s\n",  option);
 	return(ERROR);
-    } 
-   
+    }
+
     if (strncmp(option, "aa", 2) == 0) {	/* aaonly */
 	    _res.options |= RES_AAONLY;
 	} else if (strncmp(option, "noaa", 4) == 0) {
@@ -825,47 +842,47 @@ SetOption(string)
 	      _res.pfcode &= ~RES_PRF_ANS;
 	} else if (strncmp(option, "qu", 2) == 0) {  /* question section */
 	      _res.pfcode |= RES_PRF_QUES;
-	} else if (strncmp(option, "noqu", 4) == 0) {  
+	} else if (strncmp(option, "noqu", 4) == 0) {
 	      _res.pfcode &= ~RES_PRF_QUES;
 	} else if (strncmp(option, "au", 2) == 0) {  /* authority section */
 	      _res.pfcode |= RES_PRF_AUTH;
-	} else if (strncmp(option, "noau", 4) == 0) {  
+	} else if (strncmp(option, "noau", 4) == 0) {
 	      _res.pfcode &= ~RES_PRF_AUTH;
 	} else if (strncmp(option, "ad", 2) == 0) {  /* addition section */
 	      _res.pfcode |= RES_PRF_ADD;
-	} else if (strncmp(option, "noad", 4) == 0) {  
+	} else if (strncmp(option, "noad", 4) == 0) {
 	      _res.pfcode &= ~RES_PRF_ADD;
 	} else if (strncmp(option, "tt", 2) == 0) {  /* TTL & ID */
 	      _res.pfcode |= RES_PRF_TTLID;
-	} else if (strncmp(option, "nott", 4) == 0) {  
+	} else if (strncmp(option, "nott", 4) == 0) {
 	      _res.pfcode &= ~RES_PRF_TTLID;
 	} else if (strncmp(option, "he", 2) == 0) {  /* head flags stats */
 	      _res.pfcode |= RES_PRF_HEAD2;
-	} else if (strncmp(option, "nohe", 4) == 0) {  
+	} else if (strncmp(option, "nohe", 4) == 0) {
 	      _res.pfcode &= ~RES_PRF_HEAD2;
 	} else if (strncmp(option, "H", 1) == 0) {  /* header all */
 	      _res.pfcode |= RES_PRF_HEADX;
-	} else if (strncmp(option, "noH", 3) == 0) {  
+	} else if (strncmp(option, "noH", 3) == 0) {
 	      _res.pfcode &= ~(RES_PRF_HEADX);
 	} else if (strncmp(option, "qr", 2) == 0) {  /* query */
 	      _res.pfcode |= RES_PRF_QUERY;
-	} else if (strncmp(option, "noqr", 4) == 0) {  
+	} else if (strncmp(option, "noqr", 4) == 0) {
 	      _res.pfcode &= ~RES_PRF_QUERY;
 	} else if (strncmp(option, "rep", 3) == 0) {  /* reply */
 	      _res.pfcode |= RES_PRF_REPLY;
-	} else if (strncmp(option, "norep", 5) == 0) {  
+	} else if (strncmp(option, "norep", 5) == 0) {
 	      _res.pfcode &= ~RES_PRF_REPLY;
 	} else if (strncmp(option, "cm", 2) == 0) {  /* command line */
 	      _res.pfcode |= RES_PRF_CMD;
-	} else if (strncmp(option, "nocm", 4) == 0) {  
+	} else if (strncmp(option, "nocm", 4) == 0) {
 	      _res.pfcode &= ~RES_PRF_CMD;
 	} else if (strncmp(option, "cl", 2) == 0) {  /* class mnemonic */
 	      _res.pfcode |= RES_PRF_CLASS;
-	} else if (strncmp(option, "nocl", 4) == 0) {  
+	} else if (strncmp(option, "nocl", 4) == 0) {
 	      _res.pfcode &= ~RES_PRF_CLASS;
 	} else if (strncmp(option, "st", 2) == 0) {  /* stats*/
 	      _res.pfcode |= RES_PRF_STATS;
-	} else if (strncmp(option, "nost", 4) == 0) {  
+	} else if (strncmp(option, "nost", 4) == 0) {
 	      _res.pfcode &= ~RES_PRF_STATS;
 	} else {
 	    fprintf(stderr, "; *** Invalid option: %s\n",  option);
@@ -1011,7 +1028,7 @@ printZone(zone, sin)
 	__putshort(msglen, (u_char *)&len);
 
         if (write(sockFD, (char *)&len, INT16SZ) != INT16SZ ||
-            write(sockFD, (char *) &buf, msglen) != msglen) {
+            write(sockFD, (char *)&buf, msglen) != msglen) {
 		int e = errno;
 		perror(";; write");
 		(void) close(sockFD);
@@ -1046,7 +1063,7 @@ printZone(zone, sin)
 	     * The server sent too much data to fit the existing buffer --
 	     * allocate a new one.
 	     */
-	    if (len > answerLen) {
+	    if (len > (u_int)answerLen) {
 		if (answerLen != 0) {
 		    free(answer);
 		}
