@@ -279,7 +279,7 @@ va_dcl
 		fmt = va_arg(ap, char *);
 		if (strncmp("PASS ", fmt, 5) == 0)
 			printf("PASS XXXX");
-		else 
+		else
 			vfprintf(stdout, fmt, ap);
 		va_end(ap);
 		printf("\n");
@@ -605,7 +605,7 @@ sendrequest(cmd, local, remote, printnames)
 		if (c < 0)
 			warn("local: %s", local);
 		if (d < 0) {
-			if (errno != EPIPE) 
+			if (errno != EPIPE)
 				warn("netout");
 			bytes = -1;
 		}
@@ -629,7 +629,7 @@ sendrequest(cmd, local, remote, printnames)
 	/*		if (c == '\r') {			  	*/
 	/*		(void)	putc('\0', dout);  // this violates rfc */
 	/*			bytes++;				*/
-	/*		}                          			*/	
+	/*		}                          			*/
 		}
 		if (hash) {
 			if (bytes < hashbytes)
@@ -1000,7 +1000,9 @@ initconn()
 	char *p, *a;
 	int result, len, tmpno = 0;
 	int on = 1;
+	int count;
 	u_long a1,a2,a3,a4,p1,p2;
+	static u_short last_port = FTP_DATA_BOTTOM;
 
 	if (passivemode) {
 		data = socket(AF_INET, SOCK_STREAM, 0);
@@ -1051,9 +1053,6 @@ initconn()
 	}
 
 noport:
-	data_addr = myctladdr;
-	if (sendport)
-		data_addr.sin_port = 0;	/* let system pick one */ 
 	if (data != -1)
 		(void) close(data);
 	data = socket(AF_INET, SOCK_STREAM, 0);
@@ -1063,15 +1062,53 @@ noport:
 			sendport = 1;
 		return (1);
 	}
-	if (!sendport)
-		if (setsockopt(data, SOL_SOCKET, SO_REUSEADDR, (char *)&on, sizeof (on)) < 0) {
+	data_addr = myctladdr;
+	if (sendport) {
+		if (restricted_data_ports) {
+			for (count = 0;
+			     count < FTP_DATA_TOP-FTP_DATA_BOTTOM; count++) {
+				last_port++;
+				if (last_port < FTP_DATA_BOTTOM ||
+				    last_port > FTP_DATA_TOP)
+					last_port = FTP_DATA_BOTTOM;
+
+				data_addr.sin_port = htons(last_port);
+				if (bind(data, (struct sockaddr *)&data_addr,
+					 sizeof(data_addr)) < 0) {
+					if (errno == EADDRINUSE)
+						continue;
+					else {
+						warn("bind");
+						goto bad;
+					}
+				}
+				break;
+			}
+			if (count >= FTP_DATA_TOP-FTP_DATA_BOTTOM) {
+				perror("ftp: all data ports in use");
+				goto bad;
+			}
+		} else {
+			data_addr.sin_port = 0;		 /* use any port */
+			if (bind(data, (struct sockaddr *)&data_addr,
+				 sizeof(data_addr)) < 0) {
+				warn("bind");
+				goto bad;
+			}
+		}
+	} else {
+		if (setsockopt(data, SOL_SOCKET, SO_REUSEADDR,
+			       (char *)&on, sizeof (on)) < 0) {
 			warn("setsockopt (reuse address)");
 			goto bad;
 		}
-	if (bind(data, (struct sockaddr *)&data_addr, sizeof (data_addr)) < 0) {
-		warn("bind");
-		goto bad;
-	}
+		if (bind(data, (struct sockaddr *)&data_addr,
+			 sizeof (data_addr)) < 0) {
+			warn("bind");
+			goto bad;
+		}
+	}	
+
 	if (options & SO_DEBUG &&
 	    setsockopt(data, SOL_SOCKET, SO_DEBUG, (char *)&on, sizeof (on)) < 0)
 		warn("setsockopt (ignored)");
@@ -1152,8 +1189,12 @@ ptransfer(direction, bytes, t0, t1)
 		s = td.tv_sec + (td.tv_usec / 1000000.);
 #define	nz(x)	((x) == 0 ? 1 : (x))
 		bs = bytes / nz(s);
-		printf("%ld bytes %s in %.2g seconds (%.2g Kbytes/s)\n",
-		    bytes, direction, s, bs / 1024.);
+		if(bs > ( 1024 * 1024 ))
+			printf("%ld bytes %s in %.2f seconds (%.2f Meg/s)\n",
+				bytes, direction, s, bs / (1024. * 1024.));
+		else
+			printf("%ld bytes %s in %.2f seconds (%.2f Kbytes/s)\n",
+				bytes, direction, s, bs / 1024.);
 	}
 }
 
@@ -1499,7 +1540,7 @@ abort_remote(din)
 	(void) fflush(cout);
 	FD_ZERO(&mask);
 	FD_SET(fileno(cin), &mask);
-	if (din) { 
+	if (din) {
 		FD_SET(fileno(din), &mask);
 	}
 	if ((nfnd = empty(&mask, 10)) <= 0) {
