@@ -3,7 +3,7 @@
  * -
  * Copyright (c) 1986
  *    The Regents of the University of California.  All rights reserved.
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -19,7 +19,7 @@
  * 4. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
- * 
+ *
  * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -33,14 +33,14 @@
  * SUCH DAMAGE.
  * -
  * Portions Copyright (c) 1993 by Digital Equipment Corporation.
- * 
+ *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
  * copyright notice and this permission notice appear in all copies, and that
  * the name of Digital Equipment Corporation not be used in advertising or
  * publicity pertaining to distribution of the document or software without
  * specific, written prior permission.
- * 
+ *
  * THE SOFTWARE IS PROVIDED "AS IS" AND DIGITAL EQUIPMENT CORP. DISCLAIMS ALL
  * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
  * OF MERCHANTABILITY AND FITNESS.   IN NO EVENT SHALL DIGITAL EQUIPMENT
@@ -61,13 +61,13 @@ char copyright[] =
 #endif /* not lint */
 
 /*
- * Actually, this program is from Rutgers University, however it is 
+ * Actually, this program is from Rutgers University, however it is
  * based on nslookup and other pieces of named tools, so it needs
  * that copyright notice.
  */
 
 #ifndef lint
-static char rcsid[] = "$Id: host.c,v 1.1.1.1 1994/09/22 21:34:21 pst Exp $";
+static char rcsid[] = "$FreeBSD: stable/2.1/usr.bin/host/host.c 50952 1999-09-05 11:54:50Z peter $";
 #endif /* not lint */
 
 #include <sys/types.h>
@@ -125,6 +125,7 @@ char *pr_class(), *pr_rr(), *pr_type();
 extern char *hostalias();
 
 main(c, v)
+	int c;
 	char **v;
 {
 	unsigned addr;
@@ -189,12 +190,12 @@ main(c, v)
 			gettype = T_ANY;
 			v++;
 			c--;
-		}		
+		}
         }
 	if (c > 2) {
 		s = v[2];
 		server_specified++;
-		
+
 		if (!inet_aton(s, (struct in_addr *)&addr)) {
 		  hp = gethostbyname(s);
 		  if (hp == NULL) {
@@ -216,8 +217,8 @@ main(c, v)
 		_res.nscount = 1;
 		_res.retry = 2;
 	      }
-	if (strcmp (v[1], ".") == 0 ||
-	    !inet_aton(v[1], (struct in_addr *)&addr))
+	if (c > 1 && (strcmp (v[1], ".") == 0 ||
+	    !inet_aton(v[1], (struct in_addr *)&addr)))
 		addr = -1;
 	hp = NULL;
 	h_errno = TRY_AGAIN;
@@ -242,6 +243,7 @@ main(c, v)
 		  printf("Too many cnames.  Possible loop.\n");
 		  exit(1);
 		}
+		strcat(cname, ".");
 		oldcname = cname;
 		hp = NULL;
 		h_errno = TRY_AGAIN;
@@ -360,10 +362,10 @@ printanswer(hp)
 	printf("\n\n");
 }
 
-hperror(errno) 
-int errno;
+hperror(errnum) 
+int errnum;
 {
-switch(errno) {
+switch(errnum) {
 	case HOST_NOT_FOUND:
 		fprintf(stderr,"Host not found.\n");
 		break;
@@ -459,7 +461,7 @@ gethostinfo(name)
 	char *name;
 {
 	register char *cp, **domain;
-	int n;
+	u_int n;
 	int hp;
 	int nDomain;
 	int asis = 0;
@@ -480,7 +482,7 @@ gethostinfo(name)
 	if (n == 0 && (cp = hostalias(name))) {
 	        if (verbose)
 		    printf("Aliased to \"%s\"\n", cp);
-		_res.options |= RES_DEFNAMES;	  
+		_res.options |= RES_DEFNAMES;
 		return (getdomaininfo(cp, (char *)NULL));
 	}
 	if (n >= _res.ndots) {
@@ -539,6 +541,7 @@ getdomaininfo(name, domain)
 
 getinfo(name, domain, type)
 	char *name, *domain;
+	int type;
 {
 
 	HEADER *hp;
@@ -592,7 +595,7 @@ printinfo(answer, eom, filter, isls)
 	nscount = ntohs(hp->nscount);
 	arcount = ntohs(hp->arcount);
 	if (_res.options & RES_DEBUG || (verbose && isls == 0))
-		printf("rcode = %d (%s), ancount=%d\n", 
+		printf("rcode = %d (%s), ancount=%d\n",
 		       hp->rcode, DecodeError(hp->rcode), ancount);
 	if (hp->rcode != NOERROR || (ancount+nscount+arcount) == 0) {
 		switch (hp->rcode) {
@@ -746,9 +749,9 @@ pr_rr(cp, msg, file, filter)
 		}
 		break;
 	case T_CNAME:
-		if (dn_expand(msg, msg + 512, cp, cnamebuf, 
+		if (dn_expand(msg, msg + 512, cp, cnamebuf,
 			      sizeof(cnamebuf)) >= 0)
-			cname = cnamebuf;				
+			cname = cnamebuf;
 	case T_MB:
 #ifdef OLDRR
 	case T_MD:
@@ -765,15 +768,20 @@ pr_rr(cp, msg, file, filter)
 
 	case T_HINFO:
 	case T_ISDN:
-		if (n = *cp++) {
-			if (doprint)
-			  fprintf(file,"%c%.*s", punc, n, cp);
-			cp += n;
-		}
-		if (n = *cp++) {
-			if (doprint)
-			  fprintf(file,"%c%.*s", punc, n, cp);
-			cp += n;
+		{
+			u_char *cp2 = cp + dlen;
+			if (n = *cp++) {
+				if (doprint)
+				  fprintf(file,"%c%.*s", punc, n, cp);
+				cp += n;
+			}
+			if ((cp < cp2) && (n = *cp++)) {
+				if (doprint)
+				  fprintf(file,"%c%.*s", punc, n, cp);
+				cp += n;
+			} else if (type == T_HINFO)
+				if (doprint)
+				  fprintf(file,"\n; *** Warning *** OS-type missing");
 		}
 		break;
 
@@ -805,10 +813,13 @@ pr_rr(cp, msg, file, filter)
 	case T_AFSDB:
 	case T_RT:
 		if (doprint)
-		  if (verbose)
-		    fprintf(file,"\t%ld ",_getshort(cp));
+		  if (type == T_MX && !verbose)
+		    fprintf(file," (pri=%d) by ", _getshort(cp));
 		  else
-		    fprintf(file," ");
+		    if (verbose)
+		      fprintf(file,"\t%d ", _getshort(cp));
+		    else
+		      fprintf(file," ");
 		cp += sizeof(u_short);
 		cp = (u_char *)pr_cdname(cp, msg, name, sizeof(name));
 		if (doprint)
@@ -836,13 +847,13 @@ pr_rr(cp, msg, file, filter)
 		{
 		  int n,j;
 		  u_char * end = cp + dlen;
-		 
+
 		  if (doprint)
 		    (void) fputs(" \"", file);
 		  while (cp < end) {
 		    if (n = *cp++) {
 		      for (j = n; j > 0 && cp < end ; j --)
-			if (*cp == '\n') {
+			if ((*cp == '\n') || (*cp == '"')) {
 			  if (doprint){
 			    (void) putc('\\', file);
 			    (void) putc(*cp++, file);
@@ -947,7 +958,7 @@ pr_type(type)
 	case T_MG:		/* mail group member */
 		return("MG");
 	case T_MX:		/* mail routing info */
-		return(verbose? "MX" : "mail is handled by");
+		return(verbose? "MX" : "mail is handled");
 	case T_TXT:		/* TXT - descriptive info */
 		return(verbose? "TXT" : "descriptive text");
 	case T_AFSDB:		/* AFS/DCE info */
@@ -1061,7 +1072,7 @@ char *resultcodes[] = {
  *
  *  Results:
  *	SUCCESS		the listing was successful.
- *	ERROR		the server could not be contacted because 
+ *	ERROR		the server could not be contacted because
  *			a socket could not be obtained or an error
  *			occured while receiving, or the output file
  *			could not be opened.
@@ -1105,8 +1116,8 @@ ListHosts(namePtr, queryType)
 	int			thisns;
 	struct hostent		*hp;
 	enum {
-	    NO_ERRORS, 
-	    ERR_READING_LEN, 
+	    NO_ERRORS,
+	    ERR_READING_LEN,
 	    ERR_READING_MSG,
 	    ERR_PRINTING
 	} error = NO_ERRORS;
@@ -1139,13 +1150,13 @@ ListHosts(namePtr, queryType)
 	}
 
 	msglen = res_send(buf.qb2, msglen, answer.qb2, sizeof answer);
-	
+
 	if (msglen < 0) {
 		printf("Unable to get to nameserver -- try again later\n");
 		return (ERROR);
 	}
 	if (_res.options & RES_DEBUG || verbose)
-		printf("rcode = %d (%s), ancount=%d\n", 
+		printf("rcode = %d (%s), ancount=%d\n",
 		       answer.qb1.rcode, DecodeError(answer.qb1.rcode),
 		       ntohs(answer.qb1.ancount));
 
@@ -1200,7 +1211,7 @@ ListHosts(namePtr, queryType)
 	  dlen = _getshort(cp);
 	  cp += INT16SZ;
 	  if (type == T_NS) {
-	    if (dn_expand(answer.qb2, answer.qb2 + msglen, cp, 
+	    if (dn_expand(answer.qb2, answer.qb2 + msglen, cp,
 			  name, sizeof(name)) >= 0) {
 	      if (numns < NUMNS && strcasecmp((char *)domain, namePtr) == 0) {
 		for (i = 0; i < numns; i++)
@@ -1305,7 +1316,7 @@ again:
 	    perror("Connection failed, trying next server");
 	  (void) close(sockFD);
 	  sockFD = -1;
-	}	
+	}
 	if (thisns >= numnsaddr) {
 	  printf("No server for that domain responded\n");
 	  if (!verbose)
@@ -1314,7 +1325,7 @@ again:
 	}
 
 	/*
-	 * Send length & message for zone transfer 
+	 * Send length & message for zone transfer
 	 */
 
 	__putshort(msglen, (u_char *)&len);
@@ -1344,7 +1355,7 @@ again:
 	    if (numRead <= 0) {
 		error = ERR_READING_LEN;
 		break;
-	    }	
+	    }
 
 	    if ((len = _getshort((u_char*)&buf)) == 0) {
 		break;	/* nothing left to read */
@@ -1371,7 +1382,7 @@ again:
 		  (i == SERVFAIL || i == NOTIMP || i == REFUSED)) {
 		if (_res.options & RES_DEBUG || verbose)
 		  printf("Server failed, trying next server: %s\n",
-			 i != NOERROR ? 
+			 i != NOERROR ?
 			 DecodeError(i) : "Premature end of data");
 		(void) close(sockFD);
 		sockFD = -1;
@@ -1418,7 +1429,7 @@ again:
 		return(ERROR);
 
 	    case ERR_PRINTING:
-		fprintf(stderr,"*** Error during listing of %s: %s\n", 
+		fprintf(stderr,"*** Error during listing of %s: %s\n",
 				namePtr, DecodeError(result));
 		return(result);
 
@@ -1426,9 +1437,9 @@ again:
 		headerPtr = (HEADER *) &buf;
 		fprintf(stderr,"ListHosts: error receiving zone transfer:\n");
 		fprintf(stderr,
-	       "  result: %s, answers = %d, authority = %d, additional = %d\n", 
-		    	resultcodes[headerPtr->rcode], 
-		    	ntohs(headerPtr->ancount), ntohs(headerPtr->nscount), 
+	       "  result: %s, answers = %d, authority = %d, additional = %d\n",
+		    	resultcodes[headerPtr->rcode],
+		    	ntohs(headerPtr->ancount), ntohs(headerPtr->nscount),
 			ntohs(headerPtr->arcount));
 		return(ERROR);
 	    default:
@@ -1447,12 +1458,14 @@ DecodeError(result)
 	    case NXDOMAIN:	return("Non-existent domain"); break;
 	    case NOTIMP:	return("Not implemented"); break;
 	    case REFUSED:	return("Query refused"); break;
+#ifdef NOCHANGE
 	    case NOCHANGE:	return("No change"); break;
+#endif
 	    case NO_INFO: 	return("No information"); break;
 	    case ERROR: 	return("Unspecified error"); break;
 	    case TIME_OUT: 	return("Timed out"); break;
 	    case NONAUTH: 	return("Non-authoritative answer"); break;
 	    default: 		break;
 	}
-	return("BAD ERROR VALUE"); 
+	return("BAD ERROR VALUE");
 }
