@@ -94,7 +94,7 @@ flush_remote ()
 	ioctl (FD, TIOCSETP, &buf);	/*   wflushtty */
 #endif
 }
-	
+
 /*
  * FTP - remote ==> local
  *  get a file from the remote host
@@ -103,7 +103,7 @@ getfl(c)
 	char c;
 {
 	char buf[256], *cp, *expand();
-	
+
 	putchar(c);
 	/*
 	 * get the UNIX receiving file's name
@@ -115,7 +115,7 @@ getfl(c)
 		printf("\r\n%s: cannot creat\r\n", copyname);
 		return;
 	}
-	
+
 	/*
 	 * collect parameters
 	 */
@@ -137,7 +137,7 @@ cu_take(cc)
 
 	if (prompt("[take] ", copyname))
 		return;
-	if ((argc = args(copyname, argv)) < 1 || argc > 2) {
+	if ((argc = args(copyname, argv, sizeof(argv)/sizeof(argv[0]))) < 1 || argc > 2) {
 		printf("usage: <take> from [to]\r\n");
 		return;
 	}
@@ -182,14 +182,14 @@ xfer(buf, fd, eofchars)
 	quit = 0;
 	kill(pid, SIGIOT);
 	read(repdes[0], (char *)&ccc, 1);  /* Wait until read process stops */
-	
+
 	/*
 	 * finish command
 	 */
 	r = '\r';
 	pwrite(FD, &r, 1);
 	do
-		read(FD, &c, 1); 
+		read(FD, &c, 1);
 	while ((c&0177) != '\n');
 
 	usedefchars ();
@@ -269,14 +269,14 @@ transfer(buf, fd, eofchars)
 	quit = 0;
 	kill(pid, SIGIOT);
 	read(repdes[0], (char *)&ccc, 1);  /* Wait until read process stops */
-	
+
 	/*
 	 * finish command
 	 */
 	r = '\r';
 	pwrite(FD, &r, 1);
 	do
-		read(FD, &c, 1); 
+		read(FD, &c, 1);
 	while ((c&0177) != '\n');
 	usedefchars ();
 	(void) setjmp(intbuf);
@@ -506,7 +506,7 @@ cu_put(cc)
 
 	if (prompt("[put] ", copyname))
 		return;
-	if ((argc = args(copyname, argv)) < 1 || argc > 2) {
+	if ((argc = args(copyname, argv, sizeof(argv)/sizeof(argv[0]))) < 1 || argc > 2) {
 		printf("usage: <put> from [to]\r\n");
 		return;
 	}
@@ -811,14 +811,15 @@ execute(s)
 	execl(value(SHELL), cp, "-c", s, 0);
 }
 
-args(buf, a)
+args(buf, a, nargs)
 	char *buf, *a[];
+	int nargs;
 {
 	register char *p = buf, *start;
 	register char **parg = a;
 	register int n = 0;
 
-	do {
+	while (*p && n < nargs) {
 		while (*p && (*p == ' ' || *p == '\t'))
 			p++;
 		start = p;
@@ -830,7 +831,7 @@ args(buf, a)
 			parg++, n++;
 		if (*p)
 			*p++ = '\0';
-	} while (*p);
+	}
 
 	return(n);
 }
