@@ -102,7 +102,7 @@ vax_ioconf()
 				continue;
 			}
 			if (dp->d_slave != UNKNOWN) {
-				printf("can't specify slave number for %s%d\n", 
+				printf("can't specify slave number for %s%d\n",
 				    dp->d_name, dp->d_unit);
 				continue;
 			}
@@ -694,10 +694,11 @@ isa_biotab(fp, table)
 			continue;
 		fprintf(fp, "{ -1, &%3sdriver, %8s,",
 			mp->d_name, mp->d_port);
-		fprintf(fp, "%6s, %2d, C 0x%05X, %5d, %8s,  %2d, 0x%04X, %2d, 0, 0, 0, 1, %2d, 0 },\n",
+		fprintf(fp, "%6s, %2d, C 0x%05X, %5d, %8s, %3d, 0x%04X, %5d,    0,       0,       0, %6d, %8d,   0 },\n",
 			sirq(mp->d_irq), mp->d_drq, mp->d_maddr,
 			mp->d_msize, shandler(mp), dp->d_unit,
-			dp->d_flags, dp->d_drive, dp->d_conflicts);
+			dp->d_flags, dp->d_drive, !dp->d_disabled,
+			dp->d_conflicts);
 	}
 	fprintf(fp, "0\n};\n");
 }
@@ -729,10 +730,10 @@ isa_devtab(fp, table, dev_idp)
 			fprintf(fp, " %8s,", dp->d_port);
 		else
 			fprintf(fp, "   0x%04x,", dp->d_portn);
-		fprintf(fp, "%6s, %2d, C 0x%05X, %5d, %8s,  %2d, 0x%04X, 0, 0, 0, 0, 1, %2d, 0 },\n",
+		fprintf(fp, "%6s, %2d, C 0x%05X, %5d, %8s, %3d, 0x%04X,     0,    0,       0,       0, %6d, %8d,   0 },\n",
 			sirq(dp->d_irq), dp->d_drq, dp->d_maddr,
 			dp->d_msize, shandler(dp), dp->d_unit,
-			dp->d_flags, dp->d_conflicts);
+			dp->d_flags, !dp->d_disabled, dp->d_conflicts);
 	}
 	fprintf(fp, "0\n};\n");
 }
@@ -794,7 +795,7 @@ add_unique(struct node *node, char *id)
 static  int
 is_old_scsi_device(char *name)
 {
-	static char *tab[] = {"cd", "ch", "sd", "st", "uk"};
+	static char *tab[] = {"cd", "ch", "sd", "st", "od", "uk"};
 	int i;
 	for (i = 0; i < sizeof(tab) / sizeof(tab[0]); i++)
 		if (eq(tab[i], name))
@@ -818,10 +819,10 @@ scbus_devtab(fp, dev_idp)
 
 	fprintf(fp, "#include \"scsi/scsiconf.h\"\n");
 	fprintf(fp, "\nstruct scsi_ctlr_config scsi_cinit[] = {\n");
-	fprintf(fp, "/* unit  driver  driver unit */\n");
+	fprintf(fp, "/* scbus, driver, driver unit, ctlr bus*/\n");
 
 	/* XXX: Why do we always get an entry such as:
-	 * { '?', "ncr", '?' },
+	 * { '?', "ncr", '?', '?' },
 	 */
 
 	for (dp = dtab; dp; dp = dp->d_next) {
@@ -832,9 +833,10 @@ scbus_devtab(fp, dev_idp)
 		}
 		fprintf(fp, "{ %s, ", id(dp->d_unit));
 		fprintf(fp, "\"%s\", ", mp->d_name);
-		fprintf(fp, "%s },\n", id(mp->d_unit));
+		fprintf(fp, "%s, ", id(mp->d_unit));
+		fprintf(fp, "%s },\n", id(dp->d_slave));
 	}
-	fprintf(fp, "{ 0, 0, 0 }\n};\n");
+	fprintf(fp, "{ 0, 0, 0, 0 }\n};\n");
 
 	fprintf(fp, "\nstruct scsi_device_config scsi_dinit[] = {\n");
 	fprintf(fp, "/* name    unit  cunit   target   LUN  flags */\n");
@@ -865,7 +867,7 @@ scbus_devtab(fp, dev_idp)
 			 "%s%s which is not fixed at a single adapter.\n",
 			 mp->d_name, wnum(mp->d_unit));
 		}
-		
+
 		fprintf(fp, "{ ");
 		fprintf(fp, "\"%s\", ", dp->d_name);
 		id_put(fp, dp->d_unit, ", ");
