@@ -39,8 +39,8 @@ static char copyright[] =
 
 #ifndef lint
 /* from: @(#)inetd.c	8.4 (Berkeley) 4/13/94"; */
-static char inetd_c_rcsid[] = 
-	"$Id$";
+static char inetd_c_rcsid[] =
+  "$FreeBSD: stable/2.1/usr.sbin/inetd/inetd.c 50952 1999-09-05 11:54:50Z peter $";
 #endif /* not lint */
 
 /*
@@ -58,7 +58,7 @@ static char inetd_c_rcsid[] =
  * to receive further messages on, or ``take over the socket'',
  * processing all arriving datagrams and, eventually, timing
  * out.	 The first type of server is said to be ``multi-threaded'';
- * the second type of server ``single-threaded''. 
+ * the second type of server ``single-threaded''.
  *
  * Inetd uses a configuration file which is read at startup
  * and, possibly, at some later time in response to a hangup signal.
@@ -296,7 +296,16 @@ main(argc, argv, envp)
 	if (argc > 0)
 		CONFIG = argv[0];
 	if (debug == 0) {
+		FILE *fp;
 		daemon(0, 0);
+		pid = getpid();
+		fp = fopen(_PATH_INETDPID, "w");
+		if (fp) {
+			fprintf(fp, "%ld\n", (long)pid);
+			fclose(fp);
+		} else {
+			syslog(LOG_WARNING, _PATH_INETDPID ": %m");
+		}
 	}
 	memset(&sv, 0, sizeof(sv));
 	sv.sv_mask = SIGBLOCK;
@@ -331,9 +340,10 @@ main(argc, argv, envp)
 	    readable = allsock;
 	    if ((n = select(maxsock + 1, &readable, (fd_set *)0,
 		(fd_set *)0, (struct timeval *)0)) <= 0) {
-		    if (n < 0 && errno != EINTR)
+		    if (n < 0 && errno != EINTR) {
 			syslog(LOG_WARNING, "select: %m");
-		    sleep(1);
+			sleep(1);
+		    }
 		    continue;
 	    }
 	    for (sep = servtab; n && sep; sep = sep->se_next)
@@ -372,11 +382,14 @@ main(argc, argv, envp)
 			     */
 			    if (sep->se_bi &&
 				sep->se_bi->bi_fn == (void (*)()) tcpmux) {
-				    sep = tcpmux(ctrl);
-				    if (sep == NULL) {
+				    struct servtab *tsep;
+
+				    tsep = tcpmux(ctrl);
+				    if (tsep == NULL) {
 					    close(ctrl);
 					    continue;
 				    }
+				    sep = tsep;
 			    }
 		    } else
 			    ctrl = sep->se_fd;
@@ -460,7 +473,7 @@ main(argc, argv, envp)
 				if (pwd->pw_uid) {
 					if (setgid(pwd->pw_gid) < 0) {
 						syslog(LOG_ERR,
-						  "%s: can't set gid %d: %m", 
+						  "%s: can't set gid %d: %m",
 						  sep->se_service, pwd->pw_gid);
 						_exit(1);
 					}
@@ -468,7 +481,7 @@ main(argc, argv, envp)
 							pwd->pw_gid);
 					if (setuid(pwd->pw_uid) < 0) {
 						syslog(LOG_ERR,
-						  "%s: can't set uid %d: %m", 
+						  "%s: can't set uid %d: %m",
 						  sep->se_service, pwd->pw_uid);
 						_exit(1);
 					}
@@ -500,7 +513,7 @@ reapchild(signo)
 		if (pid <= 0)
 			break;
 		if (debug)
-			fprintf(stderr, "%d reaped, status %#x\n", 
+			fprintf(stderr, "%d reaped, status %#x\n",
 				pid, status);
 		for (sep = servtab; sep; sep = sep->se_next)
 			if (sep->se_wait == pid) {
@@ -550,10 +563,10 @@ config(signo)
 			/*
 			 * sep->se_wait may be holding the pid of a daemon
 			 * that we're waiting for.  If so, don't overwrite
-			 * it unless the config file explicitly says don't 
+			 * it unless the config file explicitly says don't
 			 * wait.
 			 */
-			if (cp->se_bi == 0 && 
+			if (cp->se_bi == 0 &&
 			    (sep->se_wait == 1 || cp->se_wait == 0))
 				sep->se_wait = cp->se_wait;
 #define SWAP(a, b) { char *c = a; a = b; b = c; }
@@ -685,7 +698,7 @@ setup(sep)
 
 	if ((sep->se_fd = socket(AF_INET, sep->se_socktype, 0)) < 0) {
 		if (debug)
-			fprintf(stderr, "socket failed on %s/%s: %s\n", 
+			fprintf(stderr, "socket failed on %s/%s: %s\n",
 				sep->se_service, sep->se_proto,
 				strerror(errno));
 		syslog(LOG_ERR, "%s/%s: socket: %m",
@@ -719,13 +732,13 @@ setsockopt(fd, SOL_SOCKET, opt, (char *)&on, sizeof (on))
         if (sep->se_rpc) {
                 int i, len = sizeof(struct sockaddr);
 
-                if (getsockname(sep->se_fd, 
+                if (getsockname(sep->se_fd,
 				(struct sockaddr*)&sep->se_ctrladdr, &len) < 0){
                         syslog(LOG_ERR, "%s/%s: getsockname: %m",
                                sep->se_service, sep->se_proto);
                         (void) close(sep->se_fd);
                         sep->se_fd = -1;
-                        return;       
+                        return;
                 }
                 if (debug)
                         print_service("REG ", sep);
@@ -736,7 +749,7 @@ setsockopt(fd, SOL_SOCKET, opt, (char *)&on, sizeof (on))
                                  ? IPPROTO_UDP : IPPROTO_TCP,
                                  ntohs(sep->se_ctrladdr.sin_port));
                 }
-                     
+
         }
 	if (sep->se_socktype == SOCK_STREAM)
 		listen(sep->se_fd, 10);
@@ -870,7 +883,8 @@ more:
 		sep->se_socktype = -1;
 	sep->se_proto = newstr(sskip(&cp));
         if (strncmp(sep->se_proto, "rpc/", 4) == 0) {
-                sep->se_proto += 4;
+                memmove(sep->se_proto, sep->se_proto + 4,
+                    strlen(sep->se_proto) + 1 - 4);
                 sep->se_rpc = 1;
                 sep->se_rpc_prog = sep->se_rpc_lowvers =
 			sep->se_rpc_lowvers = 0;
@@ -889,8 +903,8 @@ more:
                                         sep->se_rpc_lowvers;
                                 break;
                         default:
-                                syslog(LOG_ERR, 
-					"bad RPC version specifier; %s\n", 
+                                syslog(LOG_ERR,
+					"bad RPC version specifier; %s\n",
 					sep->se_service);
                                 freeconfig(sep);
                                 goto more;
@@ -911,13 +925,13 @@ more:
 		sep->se_wait = 0;
 
 		if (strcmp(sep->se_proto, "tcp")) {
-			syslog(LOG_ERR, 
+			syslog(LOG_ERR,
 				"%s: bad protocol for tcpmux service %s",
 				CONFIG, sep->se_service);
 			goto more;
 		}
 		if (sep->se_socktype != SOCK_STREAM) {
-			syslog(LOG_ERR, 
+			syslog(LOG_ERR,
 				"%s: bad socket type for tcpmux service %s",
 				CONFIG, sep->se_service);
 			goto more;
@@ -1055,9 +1069,9 @@ setproctitle(a, s)
 	cp = Argv[0];
 	size = sizeof(sin);
 	if (getpeername(s, (struct sockaddr *)&sin, &size) == 0)
-		(void) sprintf(buf, "-%s [%s]", a, inet_ntoa(sin.sin_addr)); 
+		(void) sprintf(buf, "-%s [%s]", a, inet_ntoa(sin.sin_addr));
 	else
-		(void) sprintf(buf, "-%s", a); 
+		(void) sprintf(buf, "-%s", a);
 	strncpy(cp, buf, LastArg - cp);
 	cp += strlen(cp);
 	while (cp < LastArg)
@@ -1098,7 +1112,7 @@ int check_loop(sin, sep)
 		if (sin->sin_port == se2->se_ctrladdr.sin_port) {
 			syslog(LOG_WARNING,
 			       "%s/%s:%s/%s loop request REFUSED from %s",
-			       sep->se_service, sep->se_proto, 
+			       sep->se_service, sep->se_proto,
 			       se2->se_service, se2->se_proto,
 			       inet_ntoa(sin->sin_addr));
 			return 1;
@@ -1118,7 +1132,7 @@ echo_dg(s, sep)			/* Echo service -- echo data back */
 	struct sockaddr_in sin;
 
 	size = sizeof(sin);
-	if ((i = recvfrom(s, buffer, sizeof(buffer), 0, 
+	if ((i = recvfrom(s, buffer, sizeof(buffer), 0,
 			  (struct sockaddr *)&sin, &size)) < 0)
 		return;
 
@@ -1226,7 +1240,7 @@ chargen_dg(s, sep)		/* Character generator */
 	}
 
 	size = sizeof(sin);
-	if (recvfrom(s, text, sizeof(text), 0, 
+	if (recvfrom(s, text, sizeof(text), 0,
 		     (struct sockaddr *)&sin, &size) < 0)
 		return;
 
@@ -1243,7 +1257,7 @@ chargen_dg(s, sep)		/* Character generator */
 		rs = ring;
 	text[LINESIZ] = '\r';
 	text[LINESIZ + 1] = '\n';
-	(void) sendto(s, text, sizeof(text), 0, 
+	(void) sendto(s, text, sizeof(text), 0,
 		      (struct sockaddr *)&sin, sizeof(sin));
 }
 
@@ -1293,7 +1307,7 @@ machtime_dg(s, sep)
 	int size;
 
 	size = sizeof(sin);
-	if (recvfrom(s, (char *)&result, sizeof(result), 0, 
+	if (recvfrom(s, (char *)&result, sizeof(result), 0,
 		     (struct sockaddr *)&sin, &size) < 0)
 		return;
 
@@ -1301,7 +1315,7 @@ machtime_dg(s, sep)
 		return;
 
 	result = machtime();
-	(void) sendto(s, (char *) &result, sizeof(result), 0, 
+	(void) sendto(s, (char *) &result, sizeof(result), 0,
 		      (struct sockaddr *)&sin, sizeof(sin));
 }
 
@@ -1334,7 +1348,7 @@ daytime_dg(s, sep)		/* Return human-readable time of day */
 	clock = time((time_t *) 0);
 
 	size = sizeof(sin);
-	if (recvfrom(s, buffer, sizeof(buffer), 0, 
+	if (recvfrom(s, buffer, sizeof(buffer), 0,
 		     (struct sockaddr *)&sin, &size) < 0)
 		return;
 
@@ -1342,7 +1356,7 @@ daytime_dg(s, sep)		/* Return human-readable time of day */
 		return;
 
 	(void) sprintf(buffer, "%.24s\r\n", ctime(&clock));
-	(void) sendto(s, buffer, strlen(buffer), 0, 
+	(void) sendto(s, buffer, strlen(buffer), 0,
 		      (struct sockaddr *)&sin, sizeof(sin));
 }
 
@@ -1359,10 +1373,10 @@ print_service(action, sep)
 		fprintf(stderr,
 	    		"%s: %s proto=%s, wait=%d, user=%s builtin=%x server=%s\n",
 	    		action, sep->se_service, sep->se_proto,
-	    		sep->se_wait, sep->se_user, (int)sep->se_bi, 
+	    		sep->se_wait, sep->se_user, (int)sep->se_bi,
 			sep->se_server);
 	else
-		fprintf(stderr, 
+		fprintf(stderr,
 			"%s: %s proto=%s, wait=%d, user=%s builtin=%x server=%s\n",
 			action, sep->se_service, sep->se_proto,
 			sep->se_wait, sep->se_user, (int)sep->se_bi,
