@@ -102,7 +102,7 @@ static int	 tof;		/* true if at top of form */
 static char	class[32];		/* classification field */
 static char	fromhost[32];		/* user's host machine */
 				/* indentation size in static characters */
-static char	indent[10] = "-i0"; 
+static char	indent[10] = "-i0";
 static char	jobname[100];		/* job or file name */
 static char	length[10] = "-l";	/* page length in lines */
 static char	logname[32];		/* user's login name */
@@ -271,6 +271,8 @@ again:
 			if (TR != NULL)		/* output trailer */
 				(void) write(ofd, TR, strlen(TR));
 		}
+		(void) close(ofd);
+		(void) wait(NULL);
 		(void) unlink(tempfile);
 		exit(0);
 	}
@@ -967,7 +969,7 @@ scan_out(scfd, scsp, dlm)
 		while (*--strp == BACKGND && strp >= outbuf)
 			;
 		strp++;
-		*strp++ = '\n';	
+		*strp++ = '\n';
 		(void) write(scfd, outbuf, strp-outbuf);
 	}
 }
@@ -1072,6 +1074,7 @@ dofork(action)
 	int action;
 {
 	register int i, pid;
+	struct passwd *pwd;
 
 	for (i = 0; i < 20; i++) {
 		if ((pid = fork()) < 0) {
@@ -1081,8 +1084,15 @@ dofork(action)
 		/*
 		 * Child should run as daemon instead of root
 		 */
-		if (pid == 0)
+		if (pid == 0) {
+			if ((pwd = getpwuid(DU)) == NULL) {
+				syslog(LOG_ERR, "Can't lookup default uid in password file");
+				break;
+			}
+			initgroups(pwd->pw_name, pwd->pw_gid);
+			setgid(pwd->pw_gid);
 			setuid(DU);
+		}
 		return(pid);
 	}
 	syslog(LOG_ERR, "can't fork");
@@ -1298,6 +1308,8 @@ struct bauds {
 	9600,	B9600,
 	19200,	EXTA,
 	38400,	EXTB,
+	57600,	B57600,
+	115200,	B115200,
 	0,	0
 };
 
