@@ -88,7 +88,7 @@ static int	 nact;		/* number of jobs to act on */
 static int	 ncopies = 1;	/* # of copies to make */
 static char	*person;	/* user name */
 static int	 qflag;		/* q job, but don't exec daemon */
-static int	 rflag;		/* remove files upon completion */	
+static int	 rflag;		/* remove files upon completion */
 static int	 sflag;		/* symbolic link flag */
 static int	 tfd;		/* control file descriptor */
 static char	*tfname;	/* tmp copy of cf before linking */
@@ -109,8 +109,9 @@ static char	*lmktemp __P((char *, int, int));
 static void	 mktemps __P((void));
 static int	 nfile __P((char *));
 static int	 test __P((char *));
+static int	 checkwriteperm __P((char*, char *));
 
-void
+int
 main(argc, argv)
 	int argc;
 	char *argv[];
@@ -260,7 +261,7 @@ main(argc, argv)
 		fatal2("only %d copies are allowed", MC);
 	/*
 	 * Get the identity of the person doing the lpr using the same
-	 * algorithm as lprm. 
+	 * algorithm as lprm.
 	 */
 	userid = getuid();
 	if (userid != DU || person == 0) {
@@ -287,7 +288,7 @@ main(argc, argv)
 	/*
 	 * Check to make sure queuing is enabled if userid is not root.
 	 */
-	(void) sprintf(buf, "%s/%s", SD, LO);
+	(void) snprintf(buf, sizeof(buf), "%s/%s", SD, LO);
 	if (userid && stat(buf, &stb) == 0 && (stb.st_mode & 010))
 		fatal2("Printer queue is disabled");
 	/*
@@ -330,7 +331,8 @@ main(argc, argv)
 			continue;	/* file unreasonable */
 
 		if (sflag && (cp = linked(arg)) != NULL) {
-			(void) sprintf(buf, "%d %d", statb.st_dev, statb.st_ino);
+			(void) snprintf(buf, sizeof(buf), "%d %d", statb.st_dev,
+				statb.st_ino);
 			card('S', buf);
 			if (format == 'p')
 				card('T', title ? title : arg);
@@ -348,12 +350,12 @@ main(argc, argv)
 			printf("%s: %s: not linked, copying instead\n", name, arg);
 		if ((i = open(arg, O_RDONLY)) < 0) {
 			printf("%s: cannot open %s\n", name, arg);
-			continue;
+		} else {
+			copy(i, arg);
+			(void) close(i);
+			if (f && unlink(arg) < 0)
+				printf("%s: %s: not removed\n", name, arg);
 		}
-		copy(i, arg);
-		(void) close(i);
-		if (f && unlink(arg) < 0)
-			printf("%s: %s: not removed\n", name, arg);
 	}
 
 	if (nact) {
@@ -425,7 +427,7 @@ copy(f, n)
 		}
 	}
 	(void) close(fd);
-	if (nc==0 && nr==0) 
+	if (nc==0 && nr==0)
 		printf("%s: %s: empty input file\n", name, f ? n : "stdin");
 	else
 		nact++;
@@ -440,10 +442,10 @@ linked(file)
 	register char *file;
 {
 	register char *cp;
-	static char buf[BUFSIZ];
+	static char buf[MAXPATHLEN];
 
 	if (*file != '/') {
-		if (getwd(buf) == NULL)
+		if (getcwd(buf, sizeof(buf)) == NULL)
 			return(NULL);
 		while (file[0] == '.') {
 			switch (file[1]) {
@@ -460,8 +462,8 @@ linked(file)
 			}
 			break;
 		}
-		strcat(buf, "/");
-		strcat(buf, file);
+		strncat(buf, "/", sizeof(buf) - strlen(buf) - 1);
+		strncat(buf, file, sizeof(buf) - strlen(buf) - 1);
 		file = buf;
 	}
 	return(symlink(file, dfname) ? NULL : file);
@@ -480,7 +482,7 @@ card(c, p2)
 	register int len = 2;
 
 	*p1++ = c;
-	while ((c = *p2++) != '\0') {
+	while ((c = *p2++) != '\0' && len < sizeof(buf)) {
 		*p1++ = (c == '\n') ? ' ' : c;
 		len++;
 	}
@@ -561,6 +563,7 @@ test(file)
 	char *file;
 {
 	struct exec execb;
+	char	*path;
 	register int fd;
 	register char *cp;
 
@@ -592,14 +595,16 @@ test(file)
 	(void) close(fd);
 	if (rflag) {
 		if ((cp = rindex(file, '/')) == NULL) {
-			if (access(".", 2) == 0)
+			if (checkwriteperm(file,".") == 0)
 				return(1);
 		} else {
 			if (cp == file) {
-				fd = access("/", 2);
+				fd = checkwriteperm(file,"/");
 			} else {
+				path = alloca(strlen(file) + 1);
+				strcpy(path,file);
 				*cp = '\0';
-				fd = access(file, 2);
+				fd = checkwriteperm(path,file);
 				*cp = '/';
 			}
 			if (fd == 0)
@@ -615,6 +620,23 @@ error1:
 	return(-1);
 }
 
+static int
+checkwriteperm(file, directory)
+	char *file, *directory;
+{
+	struct	stat	stats;
+	if (access(directory, W_OK) == 0) {
+		stat(directory, &stats);
+		if (stats.st_mode & S_ISVTX) {
+			stat(file, &stats);
+			if(stats.st_uid == userid) {
+				return(0);
+			}
+		} else return(0);
+	}
+	return(-1);
+}
+
 /*
  * itoa - integer to string conversion
  */
@@ -670,7 +692,7 @@ mktemps()
 	char buf[BUFSIZ];
 	char *lmktemp();
 
-	(void) sprintf(buf, "%s/.seq", SD);
+	(void) snprintf(buf, sizeof(buf), "%s/.seq", SD);
 	if ((fd = open(buf, O_RDWR|O_CREAT, 0661)) < 0) {
 		printf("%s: cannot create %s\n", name, buf);
 		exit(1);
@@ -694,7 +716,7 @@ mktemps()
 	inchar = strlen(SD) + 3;
 	n = (n + 1) % 1000;
 	(void) lseek(fd, (off_t)0, 0);
-	sprintf(buf, "%03d\n", n);
+	snprintf(buf, sizeof(buf), "%03d\n", n);
 	(void) write(fd, buf, strlen(buf));
 	(void) close(fd);	/* unlocks as well */
 }
@@ -711,7 +733,7 @@ lmktemp(id, num, len)
 
 	if ((s = malloc(len)) == NULL)
 		fatal2("out of memory");
-	(void) sprintf(s, "%s/%sA%03d%s", SD, id, num, host);
+	(void) snprintf(s, len, "%s/%sA%03d%s", SD, id, num, host);
 	return(s);
 }
 
