@@ -1,6 +1,6 @@
 /**************************************************************************
 **
-**  $Id: ncrcontrol.c,v 1.6 1995/03/14 21:12:26 se Exp $
+** $FreeBSD: stable/2.1/usr.sbin/ncrcontrol/ncrcontrol.c 50952 1999-09-05 11:54:50Z peter $
 **
 **  Utility for NCR 53C810 device driver.
 **
@@ -357,7 +357,7 @@ do_info(void)
 		set_lun_mask(tip);
 		if (!lun_mask) lun_mask=1;
 		fl=1;
-		
+
 		for (l=0; l<MAX_LUN; l++) {
 			if (!((lun_mask>>l)&1)) continue;
 
@@ -423,7 +423,7 @@ do_info(void)
 			printf (" --- no target.\n");
 			continue;
 		};
-		
+
 		if (verbose<1) continue;
 
 		for (i=0; i<8; i++) {
@@ -467,7 +467,7 @@ do_info(void)
 				if (d&0x80) printf (" AEN");
 				if (d&0x40) printf (" TERMINATE-I/O");
 				break;
-	
+
 			case 7:
 				if (d&0xfb) printf ("capabilities:");
 				if (d&0x80) printf (" relative");
@@ -504,7 +504,7 @@ do_profile(void)
 	int tra,line,t;
 
 	open_kvm(O_RDONLY);
-	
+
 	set_target_mask();
 
 	if (interval<1) interval=1;
@@ -537,9 +537,9 @@ do_profile(void)
 			printf (" t/s kb/s");
 		};
 
-		printf (" length   exp une fly brk"); 
+		printf (" length   exp une fly brk");
 
-		if (verbose>=1) printf ("  total  pre post  disc"); 
+		if (verbose>=1) printf ("  total  pre post  disc");
 
 		printf ("\n");
 
@@ -560,7 +560,7 @@ do_profile(void)
 			diff.ms_post	= new.ms_post - old.ms_post;
 			diff.num_disc	= new.num_disc - old.num_disc;
 			diff.num_break	= new.num_break - old.num_break;
-		
+
 			tra = diff.num_trans;
 			if (!tra) tra=1;
 
@@ -595,7 +595,7 @@ do_profile(void)
 				printf ("%7.1f",
 					(diff.ms_disc+diff.ms_data+diff.ms_setup+diff.ms_post)
 					* 1.0  / tra);
-	
+
 					printf ("%5.1f%5.1f%6.1f",
 					1.0 * diff.ms_setup / tra,
 					1.0 * diff.ms_post  / tra,
@@ -677,6 +677,23 @@ void do_set (char * arg)
 	u_long addr;
 	int i;
 
+	if (!strcmp(arg, "?")) { printf (
+"async:         disable synchronous transfers.\n"
+"sync=value:    set the maximal synchronous transfer rate (MHz).\n"
+"fast:          set FAST SCSI-2.\n"
+"\n"
+"wide=value:    set the bus width (0=8bit 1=16bit).\n"
+"\n"
+"tags=value:    use this number of tags.\n"
+"orderedtag:    use ordered tags only.\n"
+"simpletag:     use simple tags only.\n"
+"orderedwrite:  use simple tags for read, else ordered tags.\n"
+"\n"
+"debug=value:   set debug mode.\n"
+"\n");
+		return;
+	};
+
 	open_kvm(O_RDWR);
 	addr = ncr_base + offsetof (struct ncb, user);
 
@@ -703,22 +720,6 @@ void do_set (char * arg)
 	user.data   = 0;
 	user.cmd    = 0;
 
-	if (!strcmp(arg, "?")) { printf (
-"async:         disable synchronous transfers.\n"
-"sync=value:    set the maximal synchronous transfer rate (MHz).\n"
-"fast:          set FAST SCSI-2.\n"
-"\n"
-"wide=value:    set the bus width (0=8bit 1=16bit).\n"
-"\n"
-"tags=value:    use this number of tags.\n"
-"orderedtag:    use ordered tags only.\n"
-"simpletag:     use simple tags only.\n"
-"orderedwrite:  use simple tags for read, else ordered tags.\n"
-"\n"
-"debug=value:   set debug mode.\n"
-"\n");
-		return;
-	};
 
 	if (!strcmp(arg, "async")) {
 		user.data = 255;
@@ -1061,7 +1062,7 @@ static void dump_reg(struct ncr_reg * rp)
 	printf ("        DSP  %08x %-20s  CMD %08x DSPS %08x %s\n",
 		l(0x2c),sn(l(0x2c)),l(0x24),l(0x30), sn(l(0x30)));
 	printf ("        TEMP %08x %-20s  DSA %08x\n",
-		l(0x1c),sn(l(0x1c)),l(0x10)); 
+		l(0x1c),sn(l(0x1c)),l(0x10));
 	printf ("\n");
 	printf ("        Busstatus: ");
 	if ((reg[0x0b]>>7)&1) printf (" Req");
@@ -1320,7 +1321,7 @@ static void dump_ncr (void)
 			fprintf (stderr, "%s: bad kvm read.\n", prog);
 			exit (1);
 		};
-		
+
 		printf ("\n");
 		dump_reg (&reg);
 	};
@@ -1357,7 +1358,7 @@ static void dump_ncr (void)
 			fprintf (stderr, "%s: bad kvm read.\n", prog);
 			exit (1);
 		};
-		
+
 		printf ("    startpos: %x\n", startpos);
 		printf ("        slot: %d\n", (startpos-
 			(ncr.p_script+offsetof(struct script, tryloop)))/20);
@@ -1504,11 +1505,13 @@ void main(argc, argv)
 		};
 		vmunix = optarg;
 		break;
+#ifdef OPT_F
 	case 'f':
 		fprintf (stderr,
 			"%s: -f: option not yet implemented.\n",
 			prog);
 		exit (1);
+#endif
 
         case 'u':
 		i = strtoul (optarg, &charp, 0);
@@ -1598,8 +1601,9 @@ void main(argc, argv)
 		fprintf (stderr,
 	"Usage:\n"
 	"\n"
-	"%s [-M$] [-N$] {-f$} {-t#} {-l#} [-hivw?] [-d$] [-s$] [-k] [[-p] <time>]\n"
+	"%s [-M$] [-N$] [-u] {-t#} {-l#} [-hivw?] [-d$] [-s$] [-k] [[-p] <time>]\n"
 	"\n"
+	"-u <#>             select controller number\n"
 	"-t <#>             select target number\n"
 	"-l <#>             select lun number\n"
 	"-i                 get info\n"
