@@ -1,4 +1,4 @@
-/* $Id: add.h,v 1.4 1994/12/06 00:51:31 jkh Exp $ */
+/* $FreeBSD: stable/2.1/usr.sbin/pkg_install/add/add.h 50952 1999-09-05 11:54:50Z peter $ */
 
 /*
  * FreeBSD install - a package for the installation and maintainance
@@ -34,9 +34,8 @@ extern char	*Owner;
 extern char	*Group;
 extern char	*Directory;
 extern char	*PkgName;
-extern char	*PlayPen;
+extern char	FirstPen[];
 extern add_mode_t AddMode;
-extern char	*Home;
 
 int		make_hierarchy(char *);
 void		extract_plist(char *, Package *);
