@@ -1,4 +1,4 @@
-/* $Id: create.h,v 1.7 1995/04/09 15:04:57 jkh Exp $ */
+/* $FreeBSD: stable/2.1/usr.sbin/pkg_install/create/create.h 50952 1999-09-05 11:54:50Z peter $ */
 
 /*
  * FreeBSD install - a package for the installation and maintainance
@@ -31,7 +31,7 @@ extern char	*Install;
 extern char	*DeInstall;
 extern char	*Contents;
 extern char	*Require;
-extern char	*PlayPen;
+extern char	PlayPen[];
 extern char	*ExcludeFrom;
 extern char	*Mtree;
 extern char	*Pkgdeps;
