@@ -15,7 +15,7 @@
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  *
- * $Id:$
+ * $FreeBSD: stable/2.1/usr.sbin/ppp/auth.h 50952 1999-09-05 11:54:50Z peter $
  *
  *	TODO:
  */
@@ -24,7 +24,7 @@
 #define	_AUTH_H_
 
 typedef enum { VALID, INVALID, NOT_FOUND } LOCAL_AUTH_VALID;
-LOCAL_AUTH_VALID	LocalAuthValidate( char *, char *, char *); 
+LOCAL_AUTH_VALID	LocalAuthValidate( char *, char *, char *);
 
 struct authinfo {
   void (*ChallengeFunc)();
@@ -39,4 +39,6 @@ extern void SendPapChallenge __P((int));
 extern void SendChapChallenge __P((int));
 extern void StopAuthTimer __P((struct authinfo *));
 extern void StartAuthChallenge __P((struct authinfo *));
+extern LOCAL_AUTH_VALID LocalAuthInit __P((void));
+extern int AuthValidate __P((char *, char *, char *));
 #endif
