@@ -15,7 +15,7 @@
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  *
- * $Id:$
+ * $FreeBSD: stable/2.1/usr.sbin/ppp/chap.h 50952 1999-09-05 11:54:50Z peter $
  *
  *	TODO:
  */
@@ -23,7 +23,7 @@
 #ifndef _CHAP_H_
 #define	_CHAP_H_
 #include "global.h"
-#include "md5.h"
+#include <md5.h>
 
 #define	CHAP_CHALLENGE	1
 #define	CHAP_RESPONSE	2
