@@ -16,17 +16,21 @@
  * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
- * 
- * $Id: command.c,v 1.3 1995/02/27 10:57:45 amurai Exp $
- * 
+ *
+ * $FreeBSD: stable/2.1/usr.sbin/ppp/command.c 50952 1999-09-05 11:54:50Z peter $
+ *
  */
+#include <sys/types.h>
 #include <ctype.h>
 #include <termios.h>
+#include <sys/wait.h>
+#include <time.h>
 #include "fsm.h"
 #include "phase.h"
 #include "lcp.h"
 #include "ipcp.h"
 #include "modem.h"
+#include "filter.h"
 #include "command.h"
 #include "hdlc.h"
 #include "vars.h"
@@ -36,6 +40,7 @@
 #include <arpa/inet.h>
 #include <net/route.h>
 #include "os.h"
+#include <paths.h>
 
 extern int  MakeArgs();
 extern void Cleanup(), TtyTermMode(), PacketMode();
@@ -46,6 +51,7 @@ extern int  LoadCommand(), SaveCommand();
 extern int  ChangeParity(char *);
 extern int  SelectSystem();
 extern int  ShowRoute();
+extern void TtyOldMode(), TtyCommandMode();
 extern struct pppvars pppVars;
 
 struct in_addr ifnetmask;
@@ -53,6 +59,7 @@ struct in_addr ifnetmask;
 static int ShowCommand(), TerminalCommand(), QuitCommand();
 static int CloseCommand(), DialCommand(), DownCommand();
 static int SetCommand(), AddCommand(), DeleteCommand();
+static int ShellCommand();
 
 static int
 HelpCommand(list, argc, argv, plist)
@@ -93,11 +100,11 @@ IsInteractive()
   char *mes = NULL;
 
   if (mode & MODE_AUTO)
-    mes = "Working as auto mode.";
+    mes = "Working in auto mode.";
   else if (mode & MODE_DIRECT)
-    mes = "Working as direct mode.";
+    mes = "Working in direct mode.";
   else if (mode & MODE_DEDICATED)
-    mes = "Workring as dedicated mode.";
+    mes = "Working in dedicated mode.";
   if (mes) {
     printf("%s\n", mes);
     return(0);
@@ -111,6 +118,8 @@ struct cmdtab *cmdlist;
 int argc;
 char **argv;
 {
+  int tries;
+
   if (LcpFsm.state > ST_CLOSED) {
     printf("LCP state is [%s]\n", StateNames[LcpFsm.state]);
     return(1);
@@ -123,25 +132,95 @@ char **argv;
       return(1);
     }
   }
-  modem = OpenModem(mode);
-  if (modem < 0) {
-    printf("failed to open modem.\n");
-    modem = 0;
-    return(1);
-  }
-  if (DialModem()) {
-    sleep(1);
-    ModemTimeout();
-    PacketMode();
-  }
+  tries = 0;
+  do {
+    printf("Dial attempt %u\n", ++tries);
+    modem = OpenModem(mode);
+    if (modem < 0) {
+      printf("failed to open modem.\n");
+      modem = 0;
+      break;
+    }
+    if (DialModem()) {
+      sleep(1);
+      ModemTimeout();
+      PacketMode();
+      break;
+    }
+  } while (VarDialTries == 0 || tries < VarDialTries);
   return(1);
 }
 
+static int
+ShellCommand(cmdlist, argc, argv)
+struct cmdtab *cmdlist;
+int argc;
+char **argv;
+{
+  const char *shell;
+  pid_t shpid;
+  
+  if((shell = getenv("SHELL")) == 0) {
+    shell = _PATH_BSHELL;
+  }
+
+#ifndef HAVE_SHELL_CMD_WITH_ANY_MODE
+  if( mode != MODE_INTER) {
+     fprintf(stdout,
+             "Can start an shell only in interactive mode\n");
+     return(1);
+  }
+#else
+  if(argc == 0 && !(mode & MODE_INTER)) {
+      fprintf(stderr,
+             "Can start an interactive shell only in interactive mode\n");
+      return(1);
+  }
+#endif /* HAVE_SHELL_CMD_WITH_ANY_MODE */
+
+  if((shpid = fork()) == 0) {
+     int i;
+     for(i = 3; i < getdtablesize(); i++)
+	(void)close(i);
+
+     /*
+      * We are running setuid, we should change to
+      * real user for avoiding security problems.
+      */
+     if (setgid(getgid()) < 0) {
+	perror("setgid");
+	exit(1);
+     }
+     if (setuid(getuid()) < 0) {
+	perror("setuid");
+	exit(1);
+     }
+     TtyOldMode();
+     if(argc > 0)
+       execvp(argv[0], argv);
+     else
+       execl(shell, shell, NULL);
+      
+     fprintf(stdout, "exec() of %s failed\n", argc > 0? argv[0]: shell);
+     exit(255);
+  }
+  if( shpid == (pid_t)-1 ) {
+    fprintf(stdout, "Fork failed\n");
+  } else {
+    int status;
+    (void)waitpid(shpid, &status, 0);
+  }
+  
+  TtyCommandMode(1);
+      
+  return(0);
+}
+
 static char StrOption[] = "option ..";
 static char StrRemote[] = "[remote]";
 char StrNull[] = "";
 
-struct cmdtab Commands[] = {
+struct cmdtab const Commands[] = {
   { "accept",  NULL,    AcceptCommand,	LOCAL_AUTH,
   	"accept option request",	StrOption},
   { "add",     NULL,	AddCommand,	LOCAL_AUTH,
@@ -161,13 +240,15 @@ struct cmdtab Commands[] = {
   { "enable",  NULL,    EnableCommand,	LOCAL_AUTH,
   	"Enable option",		StrOption},
   { "passwd",  NULL,	LocalAuthCommand,LOCAL_NO_AUTH,
-  	"Password for manupilation", StrOption},
+  	"Password for manipulation", StrOption},
   { "load",    NULL,    LoadCommand,	LOCAL_AUTH,
   	"Load settings",		StrRemote},
   { "save",    NULL,    SaveCommand,	LOCAL_AUTH,
   	"Save settings", StrNull},
   { "set",     "setup", SetCommand,	LOCAL_AUTH,
   	"Set parameters",  "var value"},
+  { "shell",   "!",     ShellCommand,   LOCAL_AUTH,
+	"Run a subshell",  "[sh command]"},
   { "show",    NULL,    ShowCommand,	LOCAL_AUTH,
   	"Show status and statictics", "var"},
   { "term",    NULL,    TerminalCommand,LOCAL_AUTH,
@@ -243,9 +324,10 @@ static int ShowAuthKey()
 
 static int ShowVersion()
 {
-  extern char *VarVersion[];
+  extern char VarVersion[];
+  extern char VarLocalVersion[];
 
-  printf("%s\n", VarVersion);
+  printf("%s - %s \n", VarVersion, VarLocalVersion);
   return(1);
 }
 
@@ -255,9 +337,29 @@ static int ShowLogList()
   return(1);
 }
 
+static int ShowRedial()
+{
+  printf(" Redial Timer: ");
+
+  if (VarRedialTimeout >= 0) {
+    printf(" %d seconds, ", VarRedialTimeout);
+  }
+  else {
+    printf(" Random 0 - %d seconds, ", REDIAL_PERIOD);
+  }
+
+  if (VarDialTries)
+      printf("%d dial tries", VarDialTries);
+
+  printf("\n");
+
+  return(1);
+}
+
+
 extern int ShowIfilter(), ShowOfilter(), ShowDfilter(), ShowAfilter();
 
-struct cmdtab ShowCommands[] = {
+struct cmdtab const ShowCommands[] = {
   { "afilter",  NULL,     ShowAfilter,		LOCAL_AUTH,
 	"Show keep Alive filters", StrOption},
   { "auth",     NULL,     ShowAuthKey,		LOCAL_AUTH,
@@ -294,6 +396,8 @@ struct cmdtab ShowCommands[] = {
 	"Show routing table", StrNull},
   { "timeout",  NULL,	  ShowTimeout,		LOCAL_AUTH,
 	"Show Idle timeout value", StrNull},
+  { "redial",   NULL,	  ShowRedial,		LOCAL_AUTH,
+	"Show Redial timeout value", StrNull},
   { "version",  NULL,	  ShowVersion,		LOCAL_NO_AUTH | LOCAL_AUTH,
 	"Show version string", StrNull},
   { "help",     "?",      HelpCommand,		LOCAL_NO_AUTH | LOCAL_AUTH,
@@ -490,6 +594,49 @@ char **argv;
   return(1);
 }
 
+static int SetRedialTimeout(list, argc, argv)
+struct cmdtab *list;
+int argc;
+char **argv;
+{
+  int timeout;
+  int tries;
+
+  if (argc == 1 || argc == 2 ) {
+    if (strcasecmp(argv[0], "random") == 0) {
+      VarRedialTimeout = -1;
+      printf("Using random redial timeout.\n");
+      srandom(time(0));
+    }
+    else {
+      timeout = atoi(argv[0]);
+
+      if (timeout >= 0) {
+	VarRedialTimeout = timeout;
+      }
+      else {
+	printf("invalid redial timeout\n");
+	printf("Usage: %s %s\n", list->name, list->syntax);
+      }
+    }
+    if (argc == 2) {
+      tries = atoi(argv[1]);
+
+      if (tries >= 0) {
+	  VarDialTries = tries;
+      }
+      else {
+	printf("invalid retry value\n");
+	printf("Usage: %s %s\n", list->name, list->syntax);
+      }
+    }
+  }
+  else {
+    printf("Usage: %s %s\n", list->name, list->syntax);
+  }
+  return(1);
+}
+
 static int SetModemParity(list, argc, argv)
 struct cmdtab *list;
 int argc;
@@ -618,7 +765,6 @@ struct cmdtab *list;
 int argc;
 char **argv;
 {
-  int width;
 
   DefMyAddress.ipaddr.s_addr = DefHisAddress.ipaddr.s_addr = 0L;
   if (argc > 0) {
@@ -629,6 +775,12 @@ char **argv;
 	&DefHisAddress.ipaddr, &DefHisAddress.mask, &DefHisAddress.width);
       if (--argc > 0) {
         ifnetmask = GetIpAddr(*argv);
+    	if (--argc > 0) {
+      		ParseAddr(argc, argv++,
+		        &DefTriggerAddress.ipaddr,
+			&DefTriggerAddress.mask, 
+			&DefTriggerAddress.width);
+	}
       }
     }
   }
@@ -673,25 +825,31 @@ int param;
     switch (param) {
     case VAR_AUTHKEY:
       strncpy(VarAuthKey, *argv, sizeof(VarAuthKey)-1);
+      VarAuthKey[sizeof(VarAuthKey)-1] = '\0';
       break;
     case VAR_AUTHNAME:
       strncpy(VarAuthName, *argv, sizeof(VarAuthName)-1);
+      VarAuthName[sizeof(VarAuthName)-1] = '\0';
       break;
     case VAR_DIAL:
       strncpy(VarDialScript, *argv, sizeof(VarDialScript)-1);
+      VarDialScript[sizeof(VarDialScript)-1] = '\0';
       break;
     case VAR_LOGIN:
       strncpy(VarLoginScript, *argv, sizeof(VarDialScript)-1);
+      VarLoginScript[sizeof(VarLoginScript)-1] = '\0';
       break;
     case VAR_DEVICE:
       strncpy(VarDevice, *argv, sizeof(VarDevice)-1);
+      VarDevice[sizeof(VarDevice)-1] = '\0';
       break;
     case VAR_ACCMAP:
-      sscanf(*argv, "%x", &map);
+      sscanf(*argv, "%lx", &map);
       VarAccmap = map;
       break;
     case VAR_PHONE:
       strncpy(VarPhone, *argv, sizeof(VarPhone)-1);
+      VarPhone[sizeof(VarPhone)-1] = '\0';
       break;
     }
   }
@@ -719,7 +877,7 @@ static char StrValue[] = "value";
 
 extern int SetIfilter(), SetOfilter(), SetDfilter(), SetAfilter();
 
-struct cmdtab SetCommands[] = {
+struct cmdtab const SetCommands[] = {
   { "accmap",   NULL,	  SetVariable,		LOCAL_AUTH,
 	"Set accmap value", "hex-value", (void *)VAR_ACCMAP},
   { "afilter",  NULL,     SetAfilter, 		LOCAL_AUTH,
@@ -758,6 +916,8 @@ struct cmdtab SetCommands[] = {
 	"Set modem speed", "speed"},
   { "timeout",  NULL,     SetIdleTimeout,	LOCAL_AUTH,
 	"Set Idle timeout", StrValue},
+  { "redial",   NULL,     SetRedialTimeout,	LOCAL_AUTH,
+	"Set Redial timeout", "value|random [dial_attempts]"},
   { "help",     "?",      HelpCommand,		LOCAL_AUTH | LOCAL_NO_AUTH,
 	"Display this message", StrNull, (void *)SetCommands},
   { NULL,       NULL,     NULL },
