@@ -17,14 +17,15 @@
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  *
- * $Id: main.c,v 1.3 1995/02/27 10:57:50 amurai Exp $
- * 
+ * $FreeBSD: stable/2.1/usr.sbin/ppp/main.c 50952 1999-09-05 11:54:50Z peter $
+ *
  *	TODO:
  *		o Add commands for traffic summary, version display, etc.
  *		o Add signal handler for misc controls.
  */
 #include "fsm.h"
 #include <fcntl.h>
+#include <paths.h>
 #include <sys/time.h>
 #include <termios.h>
 #include <signal.h>
@@ -36,14 +37,17 @@
 #include "modem.h"
 #include "os.h"
 #include "hdlc.h"
+#include "ccp.h"
 #include "lcp.h"
 #include "ipcp.h"
 #include "vars.h"
 #include "auth.h"
 #include "filter.h"
+#include "systems.h"
+#include "ip.h"
 
 #define LAUTH_M1 "Warning: No password entry for this host in ppp.secret\n"
-#define LAUTH_M2 "Warning: All manipulation is allowed by anyone in a world\n"
+#define LAUTH_M2 "Warning: All manipulation is allowed by anyone in the world\n"
 
 #ifndef O_NONBLOCK
 #ifdef O_NDELAY
@@ -59,12 +63,14 @@ extern void DecodeCommand(), Prompt();
 extern int IsInteractive();
 extern struct in_addr ifnetmask;
 static void DoLoop(void);
+static void TerminalStop();
 
 static struct termios oldtio;		/* Original tty mode */
 static struct termios comtio;		/* Command level tty mode */
 static int TermMode;
-static int server, update;
+static int server;
 struct sockaddr_in ifsin;
+char pid_filename[128];
 
 static void
 TtyInit()
@@ -91,8 +97,9 @@ TtyInit()
 /*
  *  Set tty into command mode. We allow canonical input and echo processing.
  */
-static void
-TtyCommandMode()
+void
+TtyCommandMode(prompt)
+int prompt;
 {
   struct termios newtio;
   int stat;
@@ -100,7 +107,7 @@ TtyCommandMode()
   if (!(mode & MODE_INTER))
     return;
   tcgetattr(0, &newtio);
-  newtio.c_lflag |= (ECHO|ICANON);
+  newtio.c_lflag |= (ECHO|ISIG|ICANON);
   newtio.c_iflag = oldtio.c_iflag;
   newtio.c_oflag |= OPOST;
   tcsetattr(0, TCSADRAIN, &newtio);
@@ -108,7 +115,7 @@ TtyCommandMode()
   stat |= O_NONBLOCK;
   fcntl(0, F_SETFL, stat);
   TermMode = 0;
-  Prompt(0);
+  if(prompt) Prompt(0);
 }
 
 /*
@@ -127,33 +134,34 @@ TtyTermMode()
 }
 
 void
-Cleanup(excode)
-int excode;
+TtyOldMode()
 {
   int stat;
 
-  OsLinkdown();
-#ifdef notdef
   stat = fcntl(0, F_GETFL, 0);
   stat &= ~O_NONBLOCK;
   fcntl(0, F_SETFL, stat);
   tcsetattr(0, TCSANOW, &oldtio);
-#endif
+}
+
+void
+Cleanup(excode)
+int excode;
+{
+
+  OsLinkdown();
   OsCloseLink(1);
   sleep(1);
-  if (mode & MODE_AUTO)
+  if (mode & MODE_AUTO) {
     DeleteIfRoutes(1);
+    unlink(pid_filename);
+  }
   OsInterfaceDown(1);
   LogPrintf(LOG_PHASE, "PPP Terminated.\n");
   LogClose();
   if (server > 0)
     close(server);
-#ifndef notdef
-  stat = fcntl(0, F_GETFL, 0);
-  stat &= ~O_NONBLOCK;
-  fcntl(0, F_SETFL, stat);
-  tcsetattr(0, TCSANOW, &oldtio);
-#endif
+  TtyOldMode();
 
   exit(excode);
 }
@@ -162,7 +170,6 @@ static void
 Hangup()
 {
   LogPrintf(LOG_PHASE, "SIGHUP\n");
-  signal(SIGHUP, Hangup);
   Cleanup(EX_HANGUP);
 }
 
@@ -174,10 +181,30 @@ CloseSession()
   Cleanup(EX_TERM);
 }
 
+
+static void
+TerminalCont()
+{
+  (void)signal(SIGCONT, SIG_DFL);
+  (void)signal(SIGTSTP, TerminalStop);
+  TtyCommandMode(getpgrp() == tcgetpgrp(0));
+}
+
+static void
+TerminalStop(signo)
+int signo;
+{
+  (void)signal(SIGCONT, TerminalCont);
+  TtyOldMode();
+  signal(SIGTSTP, SIG_DFL);
+  kill(getpid(), signo);
+}
+
+
 void
 Usage()
 {
-  fprintf(stderr, "Usage: ppp [-auto | -direct -dedicated] [system]\n");
+  fprintf(stderr, "Usage: ppp [-auto | -direct | -dedicated] [system]\n");
   exit(EX_START);
 }
 
@@ -226,7 +253,6 @@ int argc;
 char **argv;
 {
   int tunno;
-  int on = 1;
 
   argc--; argv++;
 
@@ -280,6 +306,7 @@ char **argv;
   signal(SIGHUP, Hangup);
   signal(SIGTERM, CloseSession);
   signal(SIGINT, CloseSession);
+  signal(SIGQUIT, CloseSession);
 #ifdef SIGSEGV
   signal(SIGSEGV, Hangup);
 #endif
@@ -289,6 +316,18 @@ char **argv;
 #ifdef SIGALRM
   signal(SIGALRM, SIG_IGN);
 #endif
+  if(mode & MODE_INTER)
+    {
+#ifdef SIGTSTP
+      signal(SIGTSTP, TerminalStop);
+#endif
+#ifdef SIGTTIN
+      signal(SIGTTIN, TerminalStop);
+#endif
+#ifdef SIGTTOU
+      signal(SIGTTOU, SIG_IGN);
+#endif
+    }
 
   if (dstsystem) {
     if (SelectSystem(dstsystem, CONFFILE) < 0) {
@@ -332,8 +371,22 @@ char **argv;
 
     DupLog();
     if (!(mode & MODE_DIRECT)) {
+      int fd;
+      char pid[32];
+
       if (fork())
         exit(0);
+
+      snprintf(pid_filename, sizeof (pid_filename), "%s/PPP.%s",
+		  _PATH_VARRUN, dstsystem);
+      unlink(pid_filename);
+      snprintf(pid, sizeof(pid), "%d\n", (int)getpid());
+
+      if ((fd = open(pid_filename, O_RDWR|O_CREAT, 0666)) != -1)
+      {
+	  write(fd, pid, strlen(pid));
+	  close(fd);
+      }
     }
     LogPrintf(LOG_PHASE, "Listening at %d.\n", port);
 #ifdef DOTTYINIT
@@ -348,7 +401,7 @@ char **argv;
   } else {
     server = -1;
     TtyInit();
-    TtyCommandMode();
+    TtyCommandMode(1);
   }
   LogPrintf(LOG_PHASE, "PPP Started.\n");
 
@@ -383,7 +436,7 @@ PacketMode()
   else
     LcpOpen(VarOpenMode);
   if ((mode & (MODE_INTER|MODE_AUTO)) == MODE_INTER) {
-    TtyCommandMode();
+    TtyCommandMode(1);
     fprintf(stderr, "Packet mode.\r\n");
   }
 }
@@ -391,7 +444,7 @@ PacketMode()
 static void
 ShowHelp()
 {
-  fprintf(stderr, "Following commands are available\r\n");
+  fprintf(stderr, "The following commands are available:\r\n");
   fprintf(stderr, " ~p\tEnter to Packet mode\r\n");
   fprintf(stderr, " ~.\tTerminate program\r\n");
 }
@@ -475,7 +528,7 @@ ReadTty()
 #endif
       case '.':
 	TermMode = 1;
-	TtyCommandMode();
+	TtyCommandMode(1);
 	break;
       default:
 	if (write(modem, &ch, n) < 0)
@@ -515,7 +568,8 @@ int n;
     fp = *hp;
     if (DEV_IS_SYNC)
       fp++;
-    if (ptr = strstr((char *)cp, fp))
+    ptr = strstr((char *)cp, fp);
+    if (ptr)
       break;
   }
   return((u_char *)ptr);
@@ -533,12 +587,20 @@ RedialTimeout()
 static void
 StartRedialTimer()
 {
-  LogPrintf(LOG_PHASE, "Enter pause for redialing.\n");
   StopTimer(&RedialTimer);
-  RedialTimer.state = TIMER_STOPPED;
-  RedialTimer.load = REDIAL_PERIOD * SECTICKS;
-  RedialTimer.func = RedialTimeout;
-  StartTimer(&RedialTimer);
+
+  if (VarRedialTimeout) {
+    LogPrintf(LOG_PHASE, "Enter pause for redialing.\n");
+    RedialTimer.state = TIMER_STOPPED;
+
+    if (VarRedialTimeout > 0)
+	RedialTimer.load = VarRedialTimeout * SECTICKS;
+    else
+	RedialTimer.load = (random() % REDIAL_PERIOD) * SECTICKS;
+
+    RedialTimer.func = RedialTimeout;
+    StartTimer(&RedialTimer);
+  }
 }
 
 
@@ -553,10 +615,15 @@ DoLoop()
   u_char *cp;
   u_char rbuff[MAX_MRU];
   int dial_up;
+  int tries;
+  int qlen;
+  pid_t pgroup;
+
+  pgroup = getpgrp();
 
   if (mode & MODE_DIRECT) {
     modem = OpenModem(mode);
-    fprintf(stderr, "Packet mode enabled\n");
+    LogPrintf(LOG_PHASE, "Packet mode enabled\n");
     PacketMode();
   } else if (mode & MODE_DEDICATED) {
     if (!modem)
@@ -565,16 +632,12 @@ DoLoop()
 
   fflush(stdout);
 
-#ifdef SIGALRM
   timeout.tv_sec = 0;
-#else
   timeout.tv_usec = 0;
-#endif
 
   dial_up = FALSE;			/* XXXX */
+  tries = 0;
   for (;;) {
-    if ( modem ) 
-	IpStartOutput();
     FD_ZERO(&rfds); FD_ZERO(&wfds); FD_ZERO(&efds);
 
    /*
@@ -585,27 +648,42 @@ DoLoop()
 #ifdef DEBUG
       logprintf("going to dial: modem = %d\n", modem);
 #endif
-       modem = OpenModem(mode);
-       if (modem < 0) {
-         modem = 0;	       /* Set intial value for next OpenModem */
-         StartRedialTimer();
-       } else {
-         if (DialModem()) {
-           sleep(1);	       /* little pause to allow peer starts */
-           ModemTimeout();
-           PacketMode();
-           dial_up = FALSE;
-         } else {
-           CloseModem();
-           /* Dial failed. Keep quite during redial wait period. */
-           StartRedialTimer();
-         }
-       }
+      modem = OpenModem(mode);
+      if (modem < 0) {
+	modem = 0;	       /* Set intial value for next OpenModem */
+	StartRedialTimer();
+      } else {
+	tries++;
+	LogPrintf(LOG_CHAT, "Dial attempt %u\n", tries);
+	if (DialModem()) {
+	  sleep(1);	       /* little pause to allow peer starts */
+	  ModemTimeout();
+	  PacketMode();
+	  dial_up = FALSE;
+	  tries = 0;
+	} else {
+	  CloseModem();
+	  /* Dial failed. Keep quite during redial wait period. */
+	  StartRedialTimer();
+
+	  if (VarDialTries && tries >= VarDialTries) {
+	      dial_up = FALSE;
+	      tries = 0;
+	  }
+	}
+      }
     }
+    qlen = ModemQlen();
+
+    if (qlen == 0) {
+      IpStartOutput();
+      qlen = ModemQlen();
+    }
+
     if (modem) {
       FD_SET(modem, &rfds);
       FD_SET(modem, &efds);
-      if (ModemQlen() > 0) {
+      if (qlen > 0) {
 	FD_SET(modem, &wfds);
       }
     }
@@ -623,13 +701,15 @@ DoLoop()
     TimerService();
 #endif
 
-    FD_SET(tun_in, &rfds);
+    /* If there are aren't many packets queued, look for some more. */
+    if (qlen < 20)
+      FD_SET(tun_in, &rfds);
+
     if (netfd > -1) {
       FD_SET(netfd, &rfds);
       FD_SET(netfd, &efds);
     }
 
-
 #ifndef SIGALRM
     /*
      *  Normally, select() will not block because modem is writable.
@@ -638,12 +718,14 @@ DoLoop()
     tp = (RedialTimer.state == TIMER_RUNNING)? &timeout : NULL;
     i = select(tun_in+10, &rfds, &wfds, &efds, tp);
 #else
-    /* 
+    /*
      * When SIGALRM timer is running, a select function will be
-     * return -1 and EINTR after a Time Service signal hundler 
-     * is done. 
+     * return -1 and EINTR after a Time Service signal hundler
+     * is done.  If the redial timer is not running and we are
+     * trying to dial, poll with a 0 value timer.
      */
-    i = select(tun_in+10, &rfds, &wfds, &efds, NULL);
+    tp = (dial_up && RedialTimer.state != TIMER_RUNNING) ? &timeout : NULL;
+    i = select(tun_in+10, &rfds, &wfds, &efds, tp);
 #endif
     if ( i == 0 ) {
         continue;
@@ -651,11 +733,11 @@ DoLoop()
 
     if ( i < 0 ) {
        if ( errno == EINTR ) {
-          continue;            /* Got SIGALRM, Do check a queue for dailing */
+          continue;            /* Got SIGALRM, Do check a queue for dialing */
        }
        perror("select");
        break;
-    } 
+    }
 
     if ((netfd > 0 && FD_ISSET(netfd, &efds)) || FD_ISSET(modem, &efds)) {
       logprintf("Exception detected.\n");
@@ -693,7 +775,8 @@ DoLoop()
       Prompt(0);
     }
 
-    if ((mode & MODE_INTER) && FD_ISSET(netfd, &rfds)) {
+    if ((mode & MODE_INTER) && FD_ISSET(netfd, &rfds) &&
+	((mode & MODE_AUTO) || pgroup == tcgetpgrp(0))) {
       /* something to read from tty */
       ReadTty();
     }
@@ -742,12 +825,6 @@ DoLoop()
     }
 
     if (FD_ISSET(tun_in, &rfds)) {	/* something to read from tun */
-      /*
-       *  If there are many packets queued, wait until they are drained.
-       */
-      if (ModemQlen() > 5)
-	continue;
-      
       n = read(tun_in, rbuff, sizeof(rbuff));
       if (n < 0) {
 	perror("read from tun");
