@@ -18,22 +18,21 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Id: main.c,v 1.2 1994/09/25 02:32:07 wollman Exp $";
+static char rcsid[] = "$FreeBSD: stable/2.1/usr.sbin/pppd/main.c 50952 1999-09-05 11:54:50Z peter $";
 #endif
 
 #define SETSID
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <signal.h>
-#include <unistd.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <syslog.h>
 #include <netdb.h>
 #include <utmp.h>
 #include <pwd.h>
-#include <sys/wait.h>
 
 /*
  * If REQ_SYSOPTIONS is defined to 1, pppd will not run unless
@@ -57,6 +56,7 @@ static char rcsid[] = "$Id: main.c,v 1.2 1994/09/25 02:32:07 wollman Exp $";
 #include <sys/stat.h>
 #include <sys/socket.h>
 #include <sys/time.h>
+#include <sys/wait.h>
 #include <net/if.h>
 
 #include "callout.h"
@@ -97,13 +97,15 @@ char hostname[MAXNAMELEN];	/* Our hostname */
 char our_name[MAXNAMELEN];
 char remote_name[MAXNAMELEN];
 static char pidfilename[MAXPATHLEN];
+char peername[MAXNAMELEN];
+time_t t1,t2;
 
 static pid_t	pid;		/* Our pid */
 static pid_t	pgrpid;		/* Process Group ID */
 uid_t uid;			/* Our real user-id */
 gid_t gid;
 
-char devname[MAXPATHLEN] = "/dev/tty";	/* Device name */
+char devnam[MAXPATHLEN] = "/dev/tty";	/* Device name */
 int default_device = TRUE;	/* use default device (stdin/out) */
 
 int fd = -1;			/* Device file descriptor */
@@ -121,8 +123,8 @@ static int initfdflags = -1;	/* Initial file descriptor flags */
 
 static int restore_term;	/* 1 => we've munged the terminal */
 
-u_char outpacket_buf[MTU+DLLHEADERLEN]; /* buffer for outgoing packet */
-static u_char inpacket_buf[MTU+DLLHEADERLEN]; /* buffer for incoming packet */
+u_char outpacket_buf[MTU+PPP_HDRLEN]; /* buffer for outgoing packet */
+static u_char inpacket_buf[MTU+PPP_HDRLEN]; /* buffer for incoming packet */
 
 int hungup;			/* terminal has been hung up */
 static int n_children;		/* # child processes still running */
@@ -138,6 +140,8 @@ char *disconnector = NULL;	/* "disconnect" command */
 int inspeed = 0;		/* Input/Output speed requested */
 int baud_rate;			/* bits/sec currently used */
 u_long netmask = 0;		/* netmask to use on ppp interface */
+u_long dns1 = 0;		/* primary DNS */
+u_long dns2 = 0;		/* secondary DNS */
 int crtscts = 0;		/* use h/w flow control */
 int nodetach = 0;		/* don't fork */
 int modem = 0;			/* use modem control lines */
@@ -151,25 +155,24 @@ int lockflag = 0;		/* lock the serial device */
 
 
 /* prototypes */
-static void hup __ARGS((int));
-static void intr __ARGS((int));
-static void term __ARGS((int));
-static void alrm __ARGS((int));
-static void io __ARGS((int));
-static void chld __ARGS((int));
-static void incdebug __ARGS((int));
-static void nodebug __ARGS((int));
-void establish_ppp __ARGS((void));
+static void hup __P((int));
+static void term __P((int));
+static void alrm __P((int));
+static void io __P((int));
+static void chld __P((int));
+static void incdebug __P((int));
+static void nodebug __P((int));
+void establish_ppp __P((void));
 
-void reap_kids __ARGS((void));
-void cleanup __ARGS((int, caddr_t));
-void die __ARGS((int));
-void novm __ARGS((char *));
+void reap_kids __P((void));
+void cleanup __P((int, caddr_t));
+void die __P((int));
+void novm __P((char *));
 
-void log_packet __ARGS((u_char *, int, char *));
-void format_packet __ARGS((u_char *, int,
+void log_packet __P((u_char *, int, char *));
+void format_packet __P((u_char *, int,
 			   void (*) (void *, char *, ...), void *));
-void pr_log __ARGS((void *, char *, ...));
+void pr_log __P((void *, char *, ...));
 
 /*
  * PPP Data Link Layer "protocol" table.
@@ -184,9 +187,9 @@ static struct protent {
     char *name;
 } prottbl[] = {
     { LCP, lcp_init, lcp_input, lcp_protrej, lcp_printpkt, "LCP" },
-    { IPCP, ipcp_init, ipcp_input, ipcp_protrej, ipcp_printpkt, "IPCP" },
-    { UPAP, upap_init, upap_input, upap_protrej, upap_printpkt, "PAP" },
-    { CHAP, ChapInit, ChapInput, ChapProtocolReject, ChapPrintPkt, "CHAP" },
+    { PPP_IPCP, ipcp_init, ipcp_input, ipcp_protrej, ipcp_printpkt, "PPP_IPCP" },
+    { PPP_PAP, upap_init, upap_input, upap_protrej, upap_printpkt, "PAP" },
+    { PPP_CHAP, ChapInit, ChapInput, ChapProtocolReject, ChapPrintPkt, "CHAP" },
 };
 
 #define N_PROTO		(sizeof(prottbl) / sizeof(prottbl[0]))
@@ -202,10 +205,12 @@ main(argc, argv)
     char *p;
     struct passwd *pw;
 
+    time(&t1);
+    strcpy(peername,"[startup]");
     p = ttyname(0);
     if (p)
-	strcpy(devname, p);
-  
+	strcpy(devnam, p);
+
     if (gethostname(hostname, MAXNAMELEN) < 0 ) {
 	perror("couldn't get hostname");
 	die(1);
@@ -228,7 +233,7 @@ main(argc, argv)
      */
     for (i = 0; i < N_PROTO; i++)
 	(*prottbl[i].init)(0);
-  
+
     progname = *argv;
 
     if (!options_from_file(_PATH_SYSOPTIONS, REQ_SYSOPTIONS, 0) ||
@@ -314,7 +319,7 @@ main(argc, argv)
 #endif
 
     if (lockflag && !default_device)
-	if (lock(devname) < 0)
+	if (lock(devnam) < 0)
 	    die(1);
 
     /* Get an internet socket for doing socket ioctl's on. */
@@ -322,7 +327,7 @@ main(argc, argv)
 	syslog(LOG_ERR, "socket : %m");
 	die(1);
     }
-  
+
     /*
      * Compute mask of all interesting signals and install signal handlers
      * for each.  Only one signal handler may be active at a time.  Therefore,
@@ -341,7 +346,7 @@ main(argc, argv)
 #define SIGNAL(s, handler)	{ \
 	sa.sa_handler = handler; \
 	if (sigaction(s, &sa, NULL) < 0) { \
-	    syslog(LOG_ERR, "sigaction(%d): %m", s); \
+	    syslog(LOG_ERR, "Couldn't establish signal handler (%d): %m", s); \
 	    die(1); \
 	} \
     }
@@ -349,7 +354,7 @@ main(argc, argv)
     sa.sa_mask = mask;
     sa.sa_flags = 0;
     SIGNAL(SIGHUP, hup);		/* Hangup */
-    SIGNAL(SIGINT, intr);		/* Interrupt */
+    SIGNAL(SIGINT, term);		/* Interrupt */
     SIGNAL(SIGTERM, term);		/* Terminate */
     SIGNAL(SIGALRM, alrm);		/* Timeout */
     SIGNAL(SIGIO, io);			/* Input available */
@@ -374,8 +379,8 @@ main(argc, argv)
     /*
      * Open the serial device and set it up to be the ppp interface.
      */
-    if ((fd = open(devname, O_RDWR /*| O_NDELAY*/)) < 0) {
-	syslog(LOG_ERR, "open(%s): %m", devname);
+    if ((fd = open(devnam, O_RDWR /*| O_NDELAY*/)) < 0) {
+	syslog(LOG_ERR, "open(%s): %m", devnam);
 	die(1);
     }
     hungup = 0;
@@ -411,7 +416,7 @@ main(argc, argv)
 	syslog(LOG_INFO, "Connected...");
 	sleep(1);		/* give it time to set up its terminal */
     }
-  
+
     /* set line speed, flow control, etc.; clear CLOCAL if modem option */
     set_up_tty(fd, 0);
 
@@ -473,12 +478,12 @@ main(argc, argv)
 	syslog(LOG_ERR, "fcntl(F_SETFL, FNDELAY | FASYNC): %m");
 	die(1);
     }
-  
+
     /*
      * Block all signals, start opening the connection, and  wait for
      * incoming signals (reply, timeout, etc.).
      */
-    syslog(LOG_NOTICE, "Connect: %s <--> %s", ifname, devname);
+    syslog(LOG_NOTICE, "Connect: %s <--> %s", ifname, devnam);
     sigprocmask(SIG_BLOCK, &mask, NULL); /* Block signals now */
     lcp_lowerup(0);		/* XXX Well, sort of... */
     lcp_open(0);		/* Start protocol */
@@ -732,7 +737,7 @@ int fd, on;
 /*
  * quit - Clean up state and exit.
  */
-void 
+void
 quit()
 {
     die(0);
@@ -746,7 +751,9 @@ die(status)
     int status;
 {
     cleanup(0, NULL);
-    syslog(LOG_INFO, "Exit.");
+    time(&t2);
+    syslog(LOG_INFO, "Exit User %s Peer %s %d seconds",
+	username, peername, t2-t1);
     exit(status);
 }
 
@@ -784,7 +791,7 @@ cleanup(status, arg)
 	fd = -1;
     }
 
-    if (pidfilename[0] != 0 && unlink(pidfilename) < 0) 
+    if (pidfilename[0] != 0 && unlink(pidfilename) < 0)
 	syslog(LOG_WARNING, "unable to unlink pid file: %m");
     pidfilename[0] = 0;
 
@@ -810,10 +817,10 @@ timeout(func, arg, time)
 {
     struct itimerval itv;
     struct callout *newp, **oldpp;
-  
+
     MAINDEBUG((LOG_DEBUG, "Timeout %x:%x in %d seconds.",
 	       (int) func, (int) arg, time));
-  
+
     /*
      * Allocate timeout.
      */
@@ -823,7 +830,7 @@ timeout(func, arg, time)
     }
     newp->c_arg = arg;
     newp->c_func = func;
-  
+
     /*
      * Find correct place to link it in and decrement its time by the
      * amount of time used by preceding timeouts.
@@ -837,7 +844,7 @@ timeout(func, arg, time)
     if (*oldpp)
 	(*oldpp)->c_time -= time;
     *oldpp = newp;
-  
+
     /*
      * If this is now the first callout then we have to set a new
      * itimer.
@@ -871,9 +878,9 @@ untimeout(func, arg)
     struct itimerval itv;
     struct callout **copp, *freep;
     int reschedule = 0;
-  
+
     MAINDEBUG((LOG_DEBUG, "Untimeout %x:%x.", (int) func, (int) arg));
-  
+
     /*
      * If the first callout is unscheduled then we have to set a new
      * itimer.
@@ -882,7 +889,7 @@ untimeout(func, arg)
 	callout->c_func == func &&
 	callout->c_arg == arg)
 	reschedule = 1;
-  
+
     /*
      * Find first matching timeout.  Add its time to the next timeouts
      * time.
@@ -897,7 +904,7 @@ untimeout(func, arg)
 	    (void) free((char *) freep);
 	    break;
 	}
-  
+
     if (reschedule) {
 	itv.it_interval.tv_sec = itv.it_interval.tv_usec =
 	    itv.it_value.tv_usec = 0;
@@ -925,7 +932,7 @@ adjtimeout()
 {
     struct timeval tv;
     int timediff;
-  
+
     if (callout == NULL)
 	return;
     /*
@@ -941,7 +948,7 @@ adjtimeout()
     if (timediff < 0 ||
 	timediff > callout->c_time + 1)
 	return;
-  
+
     callout->c_time -= timediff;	/* OK, Adjust time */
 }
 
@@ -981,22 +988,6 @@ term(sig)
 }
 
 
-/*
- * intr - Catch SIGINT signal (DEL/^C).
- *
- * Indicates that we should initiate a graceful disconnect and exit.
- */
-static void
-intr(sig)
-    int sig;
-{
-    syslog(LOG_INFO, "Interrupt received: terminating link");
-    persist = 0;		/* don't try to restart */
-    adjtimeout();		/* Adjust timeouts */
-    lcp_close(0);		/* Close connection */
-}
-
-
 /*
  * alrm - Catch SIGALRM signal.
  *
@@ -1052,7 +1043,7 @@ alrm(sig)
 	list = list->c_next;
 	(void) free((char *) freep);
     }
-  
+
 }
 
 
@@ -1097,21 +1088,21 @@ io(sig)
 
 	if (len == 0) {
 	    MAINDEBUG((LOG_DEBUG, "End of file on fd!"));
-	    lcp_lowerdown(0);
+	    hup(0);
 	    return;
 	}
 
 	if (debug /*&& (debugflags & DBG_INPACKET)*/)
 	    log_packet(p, len, "rcvd ");
 
-	if (len < DLLHEADERLEN) {
+	if (len < PPP_HDRLEN) {
 	    MAINDEBUG((LOG_INFO, "io(): Received short packet."));
 	    return;
 	}
 
 	p += 2;				/* Skip address and control */
 	GETSHORT(protocol, p);
-	len -= DLLHEADERLEN;
+	len -= PPP_HDRLEN;
 
 	/*
 	 * Toss all non-LCP packets unless LCP is OPEN.
@@ -1134,7 +1125,7 @@ io(sig)
 	if (i == sizeof (prottbl) / sizeof (struct protent)) {
 	    syslog(LOG_WARNING, "input: Unknown protocol (%x) received!",
 		   protocol);
-	    lcp_sprotrej(0, p - DLLHEADERLEN, len + DLLHEADERLEN);
+	    lcp_sprotrej(0, p - PPP_HDRLEN, len + PPP_HDRLEN);
 	}
     }
 }
@@ -1330,17 +1321,17 @@ void
 format_packet(p, len, printer, arg)
     u_char *p;
     int len;
-    void (*printer) __ARGS((void *, char *, ...));
+    void (*printer) __P((void *, char *, ...));
     void *arg;
 {
     int i, n;
     u_short proto;
     u_char x;
 
-    if (len >= DLLHEADERLEN && p[0] == ALLSTATIONS && p[1] == UI) {
+    if (len >= PPP_HDRLEN && p[0] == ALLSTATIONS && p[1] == UI) {
 	p += 2;
 	GETSHORT(proto, p);
-	len -= DLLHEADERLEN;
+	len -= PPP_HDRLEN;
 	for (i = 0; i < N_PROTO; ++i)
 	    if (proto == prottbl[i].protocol)
 		break;
@@ -1419,7 +1410,7 @@ void
 print_string(p, len, printer, arg)
     char *p;
     int len;
-    void (*printer) __ARGS((void *, char *, ...));
+    void (*printer) __P((void *, char *, ...));
     void *arg;
 {
     int c;
