@@ -138,6 +138,14 @@ struct nlist nl[] = {
 	{ "_rc_tty" },
 #define NRC (SNPTY+6)
 	{ "_nrc_tty" },
+#define CY  (SNPTY+7)
+	{ "_cy_tty" },
+#define NCY (SNPTY+8)
+	{ "_ncy_tty" },
+#define SI  (SNPTY+9)
+	{ "_si_tty" },
+#define NSI (SNPTY+10)
+	{ "_si_Nports" },
 #endif
 	{ "" }
 };
@@ -178,7 +186,7 @@ int	nfs_print __P((struct vnode *));
 void	swapmode __P((void));
 void	ttymode __P((void));
 void	ttyprt __P((struct tty *, int));
-void	ttytype __P((struct tty *, char *, int, int));
+void	ttytype __P((struct tty *, char *, int, int, int));
 void	ufs_header __P((void));
 int	ufs_print __P((struct vnode *));
 void	vnode_header __P((void));
@@ -365,7 +373,7 @@ vnode_print(avnode, vp)
 	struct vnode *avnode;
 	struct vnode *vp;
 {
-	char *type, flags[16]; 
+	char *type, flags[16];
 	char *fp = flags;
 	register int flag;
 
@@ -391,7 +399,7 @@ vnode_print(avnode, vp)
 		type = "fif"; break;
 	case VBAD:
 		type = "bad"; break;
-	default: 
+	default:
 		type = "unk"; break;
 	}
 	/*
@@ -420,13 +428,13 @@ vnode_print(avnode, vp)
 }
 
 void
-ufs_header() 
+ufs_header()
 {
 	(void)printf(" FILEID IFLAG RDEV|SZ");
 }
 
 int
-ufs_print(vp) 
+ufs_print(vp)
 	struct vnode *vp;
 {
 	register int flag;
@@ -465,7 +473,7 @@ ufs_print(vp)
 	type = ip->i_mode & S_IFMT;
 	if (S_ISCHR(ip->i_mode) || S_ISBLK(ip->i_mode))
 		if (usenumflag || ((name = devname(ip->i_rdev, type)) == NULL))
-			(void)printf("   %2d,%-2d", 
+			(void)printf("   %2d,%-2d",
 			    major(ip->i_rdev), minor(ip->i_rdev));
 		else
 			(void)printf(" %7s", name);
@@ -475,13 +483,13 @@ ufs_print(vp)
 }
 
 void
-nfs_header() 
+nfs_header()
 {
 	(void)printf(" FILEID NFLAG RDEV|SZ");
 }
 
 int
-nfs_print(vp) 
+nfs_print(vp)
 	struct vnode *vp;
 {
 	struct nfsnode nfsnode, *np = &nfsnode;
@@ -515,7 +523,7 @@ nfs_print(vp)
 	type = VT.va_mode & S_IFMT;
 	if (S_ISCHR(VT.va_mode) || S_ISBLK(VT.va_mode))
 		if (usenumflag || ((name = devname(VT.va_rdev, type)) == NULL))
-			(void)printf("   %2d,%-2d", 
+			(void)printf("   %2d,%-2d",
 			    major(VT.va_rdev), minor(VT.va_rdev));
 		else
 			(void)printf(" %7s", name);
@@ -523,7 +531,7 @@ nfs_print(vp)
 		(void)printf(" %7qd", np->n_size);
 	return (0);
 }
-	
+
 /*
  * Given a pointer to a mount structure in kernel space,
  * read it in and return a usable pointer to it.
@@ -740,7 +748,7 @@ kinfo_vnodes(avnodes)
 	*avnodes = num;
 	return ((struct e_vnode *)vbuf);
 }
-	
+
 char hdr[]="  LINE RAW CAN OUT  HWT LWT     COL STATE  SESS  PGID DISC\n";
 int ttyspace = 128;
 
@@ -760,59 +768,64 @@ ttymode()
 	}
 #endif
 #ifdef vax
-	if (nl[SNQD].n_type != 0) 
+	if (nl[SNQD].n_type != 0)
 		qdss();
 	if (nl[SNDZ].n_type != 0)
-		ttytype(tty, "dz", SDZ, SNDZ);
+		ttytype(tty, "dz", SDZ, SNDZ, 0);
 	if (nl[SNDH].n_type != 0)
-		ttytype(tty, "dh", SDH, SNDH);
+		ttytype(tty, "dh", SDH, SNDH, 0);
 	if (nl[SNDMF].n_type != 0)
-		ttytype(tty, "dmf", SDMF, SNDMF);
+		ttytype(tty, "dmf", SDMF, SNDMF, 0);
 	if (nl[SNDHU].n_type != 0)
-		ttytype(tty, "dhu", SDHU, SNDHU);
+		ttytype(tty, "dhu", SDHU, SNDHU, 0);
 	if (nl[SNDMZ].n_type != 0)
-		ttytype(tty, "dmz", SDMZ, SNDMZ);
+		ttytype(tty, "dmz", SDMZ, SNDMZ, 0);
 #endif
 #ifdef tahoe
 	if (nl[SNVX].n_type != 0)
-		ttytype(tty, "vx", SVX, SNVX);
+		ttytype(tty, "vx", SVX, SNVX, 0);
 	if (nl[SNMP].n_type != 0)
-		ttytype(tty, "mp", SMP, SNMP);
+		ttytype(tty, "mp", SMP, SNMP, 0);
 #endif
 #ifdef hp300
 	if (nl[SNITE].n_type != 0)
-		ttytype(tty, "ite", SITE, SNITE);
+		ttytype(tty, "ite", SITE, SNITE, 0);
 	if (nl[SNDCA].n_type != 0)
-		ttytype(tty, "dca", SDCA, SNDCA);
+		ttytype(tty, "dca", SDCA, SNDCA, 0);
 	if (nl[SNDCM].n_type != 0)
-		ttytype(tty, "dcm", SDCM, SNDCM);
+		ttytype(tty, "dcm", SDCM, SNDCM, 0);
 	if (nl[SNDCL].n_type != 0)
-		ttytype(tty, "dcl", SDCL, SNDCL);
+		ttytype(tty, "dcl", SDCL, SNDCL, 0);
 #endif
 #ifdef mips
 	if (nl[SNDC].n_type != 0)
-		ttytype(tty, "dc", SDC, SNDC);
+		ttytype(tty, "dc", SDC, SNDC, 0);
 #endif
 #ifdef __FreeBSD__
 	if (nl[NSCCONS].n_type != 0)
-		ttytype(tty, "vty", SCCONS, NSCCONS);
+		ttytype(tty, "vty", SCCONS, NSCCONS, 0);
 	if (nl[NSIO].n_type != 0)
-		ttytype(tty, "sio", SIO, NSIO);
+		ttytype(tty, "sio", SIO, NSIO, 0);
 	if (nl[NRC].n_type != 0)
-		ttytype(tty, "rc", RC, NRC);
+		ttytype(tty, "rc", RC, NRC, 0);
+	if (nl[NCY].n_type != 0)
+		ttytype(tty, "cy", CY, NCY, 0);
+	if (nl[NSI].n_type != 0)
+		ttytype(tty, "si", SI, NSI, 1);
 #endif
 	if (nl[SNPTY].n_type != 0)
-		ttytype(tty, "pty", SPTY, SNPTY);
+		ttytype(tty, "pty", SPTY, SNPTY, 0);
 }
 
 void
-ttytype(tty, name, type, number)
+ttytype(tty, name, type, number, indir)
 	register struct tty *tty;
 	char *name;
-	int type, number;
+	int type, number, indir;
 {
 	register struct tty *tp;
 	int ntty;
+	struct tty **ttyaddr;
 
 	if (tty == NULL)
 		return;
@@ -823,7 +836,12 @@ ttytype(tty, name, type, number)
 		if ((tty = realloc(tty, ttyspace * sizeof(*tty))) == 0)
 			err(1, NULL);
 	}
-	KGET1(type, tty, ntty * sizeof(struct tty), "tty structs");
+	if (indir) {
+		KGET(type, ttyaddr);
+		KGET2(ttyaddr, tty, ntty * sizeof(struct tty), "tty structs");
+	} else {
+		KGET1(type, tty, ntty * sizeof(struct tty), "tty structs");
+	}
 	(void)printf(hdr);
 	for (tp = tty; tp < &tty[ntty]; tp++)
 		ttyprt(tp, tp - tty);
@@ -833,15 +851,37 @@ struct {
 	int flag;
 	char val;
 } ttystates[] = {
+#ifdef TS_WOPEN
 	{ TS_WOPEN,	'W'},
+#endif
 	{ TS_ISOPEN,	'O'},
 	{ TS_CARR_ON,	'C'},
+#ifdef TS_CONNECTED
+	{ TS_CONNECTED,	'c'},
+#endif
 	{ TS_TIMEOUT,	'T'},
 	{ TS_FLUSH,	'F'},
 	{ TS_BUSY,	'B'},
+#ifdef TS_ASLEEP
 	{ TS_ASLEEP,	'A'},
+#endif
+#ifdef TS_SO_OLOWAT
+	{ TS_SO_OLOWAT,	'A'},
+#endif
+#ifdef TS_SO_OCOMPLETE
+	{ TS_SO_OCOMPLETE, 'a'},
+#endif
 	{ TS_XCLUDE,	'X'},
 	{ TS_TTSTOP,	'S'},
+#ifdef TS_CAR_OFLOW
+	{ TS_CAR_OFLOW,	'm'},
+#endif
+#ifdef TS_CTS_OFLOW
+	{ TS_CTS_OFLOW,	'o'},
+#endif
+#ifdef TS_DSR_OFLOW
+	{ TS_DSR_OFLOW,	'd'},
+#endif
 	{ TS_TBLOCK,	'K'},
 	{ TS_ASYNC,	'Y'},
 	{ TS_BKSL,	'D'},
@@ -849,9 +889,14 @@ struct {
 	{ TS_LNCH,	'L'},
 	{ TS_TYPEN,	'P'},
 	{ TS_CNTTB,	'N'},
-#ifdef __FreeBSD__
-	{ TS_SNOOP,     's'},
+#ifdef TS_CAN_BYPASS_L_RINT
 	{ TS_CAN_BYPASS_L_RINT, 'l'},
+#endif
+#ifdef TS_SNOOP
+	{ TS_SNOOP,     's'},
+#endif
+#ifdef TS_ZOMBIE
+	{ TS_ZOMBIE,	'Z'},
 #endif
 	{ 0,	       '\0'},
 };
@@ -867,11 +912,11 @@ ttyprt(tp, line)
 
 	if (usenumflag || tp->t_dev == 0 ||
 	   (name = devname(tp->t_dev, S_IFCHR)) == NULL)
-		(void)printf("%7d ", line); 
+		(void)printf("%7d ", line);
 	else
 		(void)printf("%7s ", name);
 	(void)printf("%2d %3d ", tp->t_rawq.c_cc, tp->t_canq.c_cc);
-	(void)printf("%3d %4d %3d %7d ", tp->t_outq.c_cc, 
+	(void)printf("%3d %4d %3d %7d ", tp->t_outq.c_cc,
 		tp->t_hiwat, tp->t_lowat, tp->t_column);
 	for (i = j = 0; ttystates[i].flag; i++)
 		if (tp->t_state&ttystates[i].flag)
@@ -931,7 +976,7 @@ filemode()
 	addr = *((struct file **)buf);
 	fp = (struct file *)(buf + sizeof(struct file *));
 	nfile = (len - sizeof(struct file *)) / sizeof(struct file);
-	
+
 	(void)printf("%d/%d open files\n", nfile, maxfile);
 	(void)printf("   LOC   TYPE    FLG     CNT  MSG    DATA    OFFSET\n");
 	for (; (char *)fp < buf + len; addr = fp->f_filef, fp++) {
@@ -1076,7 +1121,7 @@ swapmode()
 		 * Don't report statistics for partitions which have not
 		 * yet been activated via swapon(8).
 		 */
-		if (!(sw[i].sw_flags & SW_FREED)) 
+		if (!(sw[i].sw_flags & SW_FREED))
 			continue;
 
 		if (!totalflag)
@@ -1084,7 +1129,7 @@ swapmode()
 			    devname(sw[i].sw_dev, S_IFBLK),
 			    hlen, sw[i].sw_nblks / div);
 
-		/* The first dmmax is never allocated to avoid trashing of 
+		/* The first dmmax is never allocated to avoid trashing of
 		 * disklabels
 		 */
 		xsize = sw[i].sw_nblks - dmmax;
@@ -1094,14 +1139,14 @@ swapmode()
 		avail += xsize;
 		if (totalflag)
 			continue;
-		(void)printf("%8d %8d %5.0f%%    %s\n", 
+		(void)printf("%8d %8d %5.0f%%    %s\n",
 		    used / div, xfree / div,
 		    (double)used / (double)xsize * 100.0,
 		    (sw[i].sw_flags & SW_SEQUENTIAL) ?
 			     "Sequential" : "Interleaved");
 	}
 
-	/* 
+	/*
 	 * If only one partition has been set up via swapon(8), we don't
 	 * need to bother with totals.
 	 */
