@@ -1,20 +1,19 @@
-#!/usr/local/bin/perl
+#!/usr/bin/perl
 'di ';
 'ds 00 \\"';
-'ig00 ';
+'ig 00 ';
 #
 #       THIS PROGRAM IS ITS OWN MANUAL PAGE.  INSTALL IN man & bin.
 #
 
 # hardcoded constants, should work fine for BSD-based systems
-$AF_INET = 2;
-$SOCK_STREAM = 1;
+require 'sys/socket.ph';
 $sockaddr = 'S n a4 x8';
 
 # system requirements:
 # 	must have 'nslookup' and 'hostname' programs.
 
-# $Header: /home/muir/bin/RCS/expn,v 3.6 1994/02/23 22:26:19 muir Exp muir $
+# $FreeBSD: stable/2.1/usr.sbin/sendmail/contrib/expn.pl 50952 1999-09-05 11:54:50Z peter $
 
 # TODO:
 #	less magic should apply to command-line addresses
@@ -70,7 +69,7 @@ $sockaddr = 'S n a4 x8';
 #	ask each server in turn a whole bunch of questions, addresses to
 #	be expanded are queued up.
 #
-#	This means that all account w.r.t. an address must be stored in
+#	This means that all accounting w.r.t. an address must be stored in
 #	various arrays.  Generally these arrays are indexed by the
 #	string "$addr *** $server" where $addr is the address to be
 #	expanded "foo" or maybe "foo@bar" and $server is the hostname
@@ -102,13 +101,18 @@ $sockaddr = 'S n a4 x8';
 $have_nslookup = 1;	# we have the nslookup program
 $port = 'smtp';
 $av0 = $0;
-$0 = "$av0 - running hostname";
 $ENV{'PATH'} .= ":/usr/etc" unless $ENV{'PATH'} =~ m,/usr/etc,;
-chop($hostname = `hostname`);
+$ENV{'PATH'} .= ":/usr/ucb" unless $ENV{'PATH'} =~ m,/usr/ucb,;
 select(STDERR);
 
-$usage = "Usage: $av0 [-1avwd] user[@host] [user2[host2] ...]";
+$0 = "$av0 - running hostname";
+chop($name = `hostname || uname -n`);
+
+$0 = "$av0 - lookup host FQDN and IP addr";
+($hostname,$aliases,$type,$len,$thisaddr) = gethostbyname($name);
+
 $0 = "$av0 - parsing args";
+$usage = "Usage: $av0 [-1avwd] user[\@host] [user2[host2] ...]";
 for $a (@ARGV) {
 	die $usage if $a eq "-";
 	while ($a =~ s/^(-.*)([1avwd])/$1/) {
@@ -143,8 +147,7 @@ $0 = "$av0 - building local socket";
 ($name,$aliases,$proto) = getprotobyname('tcp');
 ($name,$aliases,$port) = getservbyname($port,'tcp')
 	unless $port =~ /^\d+/;
-($name,$aliases,$type,$len,$thisaddr) = gethostbyname($hostname);
-$this = pack($sockaddr, $AF_INET, 0, $thisaddr);
+$this = pack($sockaddr, &AF_INET, 0, $thisaddr);
 
 HOST:
 while (@hosts) {
@@ -174,8 +177,8 @@ while (@hosts) {
 				
 	# get a connection, or look for an mx
 	$0 = "$av0 - socket to $server";
-	$that = pack($sockaddr, $AF_INET, $port, $thataddr);
-	socket(S, $AF_INET, $SOCK_STREAM, $proto)
+	$that = pack($sockaddr, &AF_INET, $port, $thataddr);
+	socket(S, &AF_INET, &SOCK_STREAM, $proto)
 		|| die "socket: $!";
 	$0 = "$av0 - bind to $server";
 	bind(S, $this) 
@@ -200,8 +203,8 @@ while (@hosts) {
 		print if $watch;
 		if (/^(\d+)([- ])/) {
 			if ($1 != 220) {
-				$0 = "$av0 - bad numeric responce from $server";
-				&alarm("giving up after bet responce from $server",'');
+				$0 = "$av0 - bad numeric response from $server";
+				&alarm("giving up after bad response from $server",'');
 				&read_response($2,$watch);
 				alarm(0);
 				print STDERR "$server: NOT 220 greeting: $_"
@@ -213,7 +216,7 @@ while (@hosts) {
 			}
 			last if ($2 eq " ");
 		} else {
-			$0 = "$av0 - bad responce from $server";
+			$0 = "$av0 - bad response from $server";
 			print STDERR "$server: NOT 220 greeting: $_"
 				if ($debug || $vw);
 			unless (&mxlookup(0,$server,"$server: did not respond with SMTP codes",*users)) {
@@ -261,7 +264,9 @@ while (@hosts) {
 			@toExpn = ();
 		}
 
-		($ecode,@expansion) = &expn_vrfy($u,$server);
+#		($ecode,@expansion) = &expn_vrfy($u,$server);
+		(@foo) = &expn_vrfy($u,$server);
+		($ecode,@expansion) = @foo;
 		if ($ecode) {
 			&giveup('',$ecode,$u);
 			last USER;
@@ -329,7 +334,7 @@ while (@hosts) {
 			# 550 is a known code...  Should the be
 			# included in -a output?  Might be a bug
 			# here.  Does it matter?  Can assume that
-			# there won't be UNKNOWN USER responces 
+			# there won't be UNKNOWN USER responses 
 			# mixed with valid users?
 			if ($s =~ /^(550)([- ])/) {
 				if ($valid) {
@@ -414,6 +419,8 @@ sub giveup
 {
 	local($redirect_okay,$reason,$user) = @_;
 	local($us,@so,$nh,@remaining_users);
+	local($pk,$file,$line);
+	($pk, $file, $line) = caller;
 
 	$0 = "$av0 - giving up on $server: $reason";
 	#
@@ -427,7 +434,7 @@ sub giveup
 		$giveup{$server} = $reason;
 		print STDERR "$reason\n";
 	}
-	print "Giveup!!! redirect okay = $redirect_okay; $reason\n" if $debug;
+	print "Giveup at $file:$line!!! redirect okay = $redirect_okay; $reason\n" if $debug;
 	#
 	# Wait!
 	# Before giving up, see if there is a chance that
@@ -486,7 +493,7 @@ sub try_fallback
 	if (defined $fellback{$us}) {
 		#
 		# Undo a previous fallback so that we can try again
-		# Nest fallbacks are avoided because they could
+		# Nested fallbacks are avoided because they could
 		# lead to infinite loops
 		#
 		$fallhost = $fellback{$us};
@@ -579,7 +586,7 @@ sub do_validAddr
 #
 # Ranking of inputs: best: user@host.domain, okay: user
 #
-# Return value: $error_string, @responces_from_server
+# Return value: $error_string, @responses_from_server
 sub expn_vrfy
 {
 	local($u,$server) = @_;
@@ -594,7 +601,7 @@ sub expn_vrfy
 	TRY:
 	for $c (@c) {
 		for $try_u (@try_u) {
-			&alarm("$c'ing $try_u on $server",'',$u);
+			&alarm("${c}'ing $try_u on $server",'',$u);
 			&ps("$c $try_u");
 			alarm(0);
 			$s = <S>;
@@ -608,7 +615,7 @@ sub expn_vrfy
 				$code = 250;
 				@ret = ("",$s);
 				push(@ret,&read_response($2,$debug));
-				return @ret;
+				return (@ret);
 			} 
 			if ($1 == 551 || $1 == 251) {
 				$code = $1;
@@ -648,7 +655,7 @@ sub parse
 }
 
 # returns ($new_smtp_server,$new_address,$new_name)
-# given a responce from a SMTP server ($newaddr), the 
+# given a response from a SMTP server ($newaddr), the 
 # current host ($server), the old "name" and a flag that
 # indicates if it is being called during the initial 
 # command line parsing ($parsing_args)
@@ -672,13 +679,13 @@ sub parse2
 	#
 	if ($newaddr =~ /^\<(.*)\>$/) {
 		print "<A:$1>\n" if $debug;
-		$newaddr = &trim($1);
+		($newaddr) = &trim($1);
 		print "na = $newaddr\n" if $debug;
 	}
 	if ($newaddr =~ /^([^\<\>]*)\<([^\<\>]*)\>([^\<\>]*)$/) {
 		# address has a < > pair in it.
 		print "N:$1 <A:$2> N:$3\n" if $debug;
-		$newaddr = &trim($2);
+		($newaddr) = &trim($2);
 		unshift(@names, &trim($3,$1));
 		print "na = $newaddr\n" if $debug;
 	}
@@ -754,7 +761,7 @@ sub trim
 }
 # using the host part of an address, and the server name, add the
 # servers' domain to the address if it doesn't already have a 
-# domain.  Since this sometimes failes, save a back reference so
+# domain.  Since this sometimes fails, save a back reference so
 # it can be unrolled.
 sub domainify
 {
@@ -1047,11 +1054,6 @@ sub mxlookup
 
 	# provide fallbacks in case $nserver doesn't work out
 	if (defined $fallback{$cpref}) {
-#		for $u (@users) {
-#			print "mx_secondary{$u *** $nserver} = ".$fallback{$cpref}."\n"
-#				if $debug;
-#			$mx_secondary{"$u *** $nserver"} = $fallback{$cpref};
-#		}
 		$mx_secondary{$server} = $fallback{$cpref};
 	}
 
@@ -1191,15 +1193,15 @@ sub alarm
 	alarm(3600);
 	$SIG{ALRM} = 'handle_alarm';
 }
-# this involves one GREAT hack.
-# the "next HOST" has to unwind the stack!
+# this involves one great big ugly hack.
+# the "next HOST" unwinds the stack!
 sub handle_alarm
 {
 	&giveup($alarm_redirect,"Timed out during $alarm_action",$alarm_user);
 	next HOST;
 }
 
-# read the rest of the current smtp daemon's responce (and toss it away)
+# read the rest of the current smtp daemon's response (and toss it away)
 sub read_response
 {
 	local($done,$watch) = @_;
@@ -1227,11 +1229,11 @@ $flag_1;
 %already_mx_fellback;
 &handle_alarm;
 ################### BEGIN PERL/TROFF TRANSITION 
-.00;	
+.00 ;	
 
-'di		\\ " finish diversion--previous line must be blank
-.nr nl 0-1	\\ " fake up transition to first page again
-.nr % 0		\\ " start at page 1
+'di
+.nr nl 0-1
+.nr % 0
 .\\"'; __END__ 
 .\" ############## END PERL/TROFF TRANSITION
 .TH EXPN 1 "March 11, 1993"
@@ -1359,7 +1361,7 @@ and Jon Broome has dropped off the face of the earth!
 .SH AVAILABILITY
 The latest version of 
 .B expn
-is available through anonymous ftp to
-.IR idiom.berkeley.ca.us .
+is available through anonymous ftp at
+.IR ftp://ftp.idiom.com/pub/muir-programs/expn .
 .SH AUTHOR
-.I David Muir Sharnoff\ \ \ \ <muir@idiom.berkeley.ca.us>
+.I David Muir Sharnoff\ \ \ \ <muir@idiom.com>
