@@ -18,9 +18,9 @@
  * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
  * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
-#ifndef lint
+#if !defined(lint) && !defined(__GNUC__)
 static char rcsid[] =
-    "@(#)$Header: search.c,v 1.8 93/11/18 13:11:51 vern Exp $ (LBL)";
+    "@(#)$FreeBSD: stable/2.1/usr.sbin/tcpdump/tcpslice/search.c 50952 1999-09-05 11:54:50Z peter $ (LBL)";
 #endif
 
 /*
